/*
 *  This software may only be used by you under license from AT&T Corp.
 *  ("AT&T").  A copy of AT&T's Source Code Agreement is available at
 *  AT&T's Internet website having the URL:
 *  <http://www.research.att.com/sw/tools/graphviz/license/source.html>
 *  If you received this software without first entering into a license
 *  with AT&T, you have an infringing copy of this software and cannot use
 *  it without violating AT&T's intellectual property rights.
 */

package att.grappa;
import java.io.*;
import java.util.*;

/**
 * A class providing <I>sprintf</I> support.
 *
 * @version 1.2, 01 Oct 2001; Copyright 1996 - 2001 by AT&T Corp.
 * @author  <a href="mailto:john@research.att.com">John Mocenigo</a> and Rich Drechsler, <a href="http://www.research.att.com">Research @ AT&T Labs</a>
 */
public class GrappaSupportPrintf

    implements GrappaConstants

{

    ///////////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////////
    //
    // GrappaSupportPrintf
    //
    ///////////////////////////////////////////////////////////////////////////

    /**
     * The familiar C-language sprintf rendered in Java and extended for
     * some Grappa types.
     *
     * @param args the first element of this array is a string giving the
     *             format of the returned string, the remaining elements
     *             are object to be formatted according to the format.
     * @return a string giving a formatted representation of the arguments.
     */
    public final static String
    sprintf(Object args[]) {

	PrintfParser	cvt;
	StringBuffer	prtbuf;
	char		format[];
	int		flen;
	int		argn;
	int		n;
	char		ch;
	boolean         flag;

	if(!(args[0] instanceof String)) {
	    throw new RuntimeException("initial argument must be format String");
	}

	argn = 0;
	format = ((String)args[argn++]).toCharArray();

	flen = format.length;
	prtbuf = new StringBuffer(2 * flen);
	cvt = new PrintfParser();

	for (n = 0; n < flen; ) {
	    if ((ch = format[n++]) == '%') {
		if ((n = cvt.parse(format, n)) < flen) {
		    switch (ch = format[n++]) {
		    case 'b':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof GrappaBox)
			    flag = ((GrappaBox)args[argn]).isDimensioned();
			else
			    flag = true;
			if (args[argn] instanceof java.awt.geom.Rectangle2D)
			    cvt.buildBox(prtbuf, ((java.awt.geom.Rectangle2D)args[argn++]), false, flag);
			else throw new RuntimeException("argument " + argn + " should be a Rectangle2D");
			break;

		    case 'B':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof GrappaBox)
			    flag = ((GrappaBox)args[argn]).isDimensioned();
			else
			    flag = true;
			if (args[argn] instanceof java.awt.geom.Rectangle2D)
			    cvt.buildBox(prtbuf, ((java.awt.geom.Rectangle2D)args[argn++]), true, flag);
			else throw new RuntimeException("argument " + argn + " should be a Rectangle2D");
			break;
		    case 'c':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildChar(prtbuf, ((Character)args[argn++]).charValue());
			else throw new RuntimeException("argument " + argn + " should be a Character");
			break;

		    case 'd':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Number)
			    cvt.buildInteger(prtbuf, ((Number)args[argn++]).intValue());
			else throw new RuntimeException("argument " + argn + " should be a Number");
			break;

		    case 'o':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildOctal(prtbuf, ((Character)args[argn++]).charValue());
			else if (args[argn] instanceof Number)
			    cvt.buildOctal(prtbuf, ((Number)args[argn++]).intValue());
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'p':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof java.awt.geom.Point2D)
			    cvt.buildPoint(prtbuf, ((java.awt.geom.Point2D)args[argn++]), false);
			else if (args[argn] instanceof java.awt.geom.Dimension2D)
			    cvt.buildSize(prtbuf, ((java.awt.geom.Dimension2D)args[argn++]), false);
			else throw new RuntimeException("argument " + argn + " should be a Point2D");
			break;

		    case 'P':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof java.awt.geom.Point2D)
			    cvt.buildPoint(prtbuf, ((java.awt.geom.Point2D)args[argn++]), true);
			else if (args[argn] instanceof java.awt.geom.Dimension2D)
			    cvt.buildSize(prtbuf, ((java.awt.geom.Dimension2D)args[argn++]), true);
			else throw new RuntimeException("argument " + argn + " should be a Point2D");
			break;

		    case 'x':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildHex(prtbuf, ((Character)args[argn++]).charValue(), false);
			else if (args[argn] instanceof Number)
			    cvt.buildHex(prtbuf, ((Number)args[argn++]).intValue(), false);
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'X':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildHex(prtbuf, ((Character)args[argn++]).charValue(), true);
			else if (args[argn] instanceof Number)
			    cvt.buildHex(prtbuf, ((Number)args[argn++]).intValue(), true);
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'e':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildExp(prtbuf, ((Character)args[argn++]).charValue(), false);
			else if (args[argn] instanceof Number)
			    cvt.buildExp(prtbuf, ((Number)args[argn++]).doubleValue(), false);
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'E':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildExp(prtbuf, ((Character)args[argn++]).charValue(), true);
			else if (args[argn] instanceof Number)
			    cvt.buildExp(prtbuf, ((Number)args[argn++]).doubleValue(), true);
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'f':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildFloat(prtbuf, ((Character)args[argn++]).charValue());
			else if (args[argn] instanceof Number)
			    cvt.buildFloat(prtbuf, ((Number)args[argn++]).doubleValue());
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'g':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildFlex(prtbuf, ((Character)args[argn++]).charValue(), false);
			else if (args[argn] instanceof Number)
			    cvt.buildFlex(prtbuf, ((Number)args[argn++]).doubleValue(), false);
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 'G':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			if (args[argn] instanceof Character)
			    cvt.buildFlex(prtbuf, ((Character)args[argn++]).charValue(), true);
			else if (args[argn] instanceof Number)
			    cvt.buildFlex(prtbuf, ((Number)args[argn++]).doubleValue(), true);
			else throw new RuntimeException("argument " + argn + " should be a Character or Number");
			break;

		    case 's':
			if (args.length <= argn) throw new RuntimeException("too few arguments for format");
			cvt.buildString(prtbuf, args[argn++].toString());
			break;

		    case '%':
			prtbuf.append('%');
			break;

		    default:
			// different compilers handle this different ways,
			// some just do the equivalent of prtbuf.append(ch),
			// but we will just ignore the unrecognized format
			break;
		    }
		} else prtbuf.append(ch);
	    } else if (ch == '\\') {
		if (n < flen) {
		    switch (ch = format[n++]) {
		    case 'b':
			prtbuf.append('\b');
			break;
		    case 'f':
			prtbuf.append('\f');
			break;
		    case 'n':
			prtbuf.append('\n');
			break;
		    case 'r':
			prtbuf.append('\r');
			break;
		    case 't':
			prtbuf.append('\t');
			break;
		    case 'u':
			if ((n+3) < flen) {
			    if (
				GrappaSupport.isdigit(format[n])
				&&
				GrappaSupport.isdigit(format[n+1])
				&&
				GrappaSupport.isdigit(format[n+2])
				&&
				GrappaSupport.isdigit(format[n+3])
				) {
				int uni = (int)format[n+3]+16*(int)format[n+2]+256*(int)format[n+1]+4096*(int)format[n];
				prtbuf.append((char)uni);
				n += 4;
			    } else prtbuf.append('u');
			} else prtbuf.append('u');
			break;
		    case '"':
			prtbuf.append('\"');
			break;
		    case '\'':
			prtbuf.append('\'');
			break;
		    case '\\':
			prtbuf.append('\\');
			break;
		    case '0':
		    case '1':
		    case '2':
		    case '3':
		    case '4':
		    case '5':
		    case '6':
		    case '7':
		    case '8':
		    case '9':
			// need to fix this, assumes 3 digit octals
			if ((n+1) < flen) {
			    if (
				GrappaSupport.isdigit(format[n])
				&&
				GrappaSupport.isdigit(format[n+1])
				) {
				int oct = (int)format[n+1]+8*(int)format[n]+64*(int)ch;
				prtbuf.append((char)oct);
				n += 2;
			    } else prtbuf.append(ch);
			} else prtbuf.append(ch);
			break;
		    }
		} else prtbuf.append(ch);
	    } else prtbuf.append(ch);
	}

	return(prtbuf.toString());
    }

    ///////////////////////////////////////////////////////////////////////////
}

