package edu.wpi.cs.dsrg.xmldb.xat.automata.utils;

/**
 * records the run time of the run core method of an XATStreamOperator
 * and unit cost/prob \not= emptyset
 * Creation date: (4/23/2003 1:42:51 PM)
 * @author: Hong Su
 */
import java.io.Serializable;


public class ClassExecTimeKeeper extends TimeKeeper implements Serializable, Cloneable {
	// the class whose certain methods the timekeeper records	
	protected Object evaluatedClass;
	protected String methodDesc = null;
	
	protected double unitCost = 0;
		
	protected int inputCount = 0;
	protected int outputCount = 0;
	protected int notEmptyCount = 0;
	boolean notEmpty = false;
	
	//the token id last time counted
	protected int lastTokenCount = 0;
	//the overall number of tokens 
	protected int curTokenCount = 0;
	//the token count within a period
	protected int deltaTokenCount = 0;
	
	//the start token id last time counted
	protected int lastStartTokenCount = 0;
	//the overall number of start tokens
	protected int curStartTokenCount = 0;
	//the start token within a period
	protected int deltaStartTokenCount = 0;
	
	//especially for NavNest or ExtractNest
	protected int itemCountWithinEntrySum =0;
	
	
	/**
	 * ClassExecTimeKeeper constructor comment.
	 */
	public ClassExecTimeKeeper(Object evaluatedClass) {
		super();
		this.evaluatedClass = evaluatedClass;
	}
	/**
	 * ClassExecTimeKeeper constructor comment.
	 */
	public ClassExecTimeKeeper(Object evaluatedClass, String methodDesc) {
		super();
		this.evaluatedClass = evaluatedClass;
		this.methodDesc = methodDesc;
	}
	
	public void setItemCountWithinEntrySum(int itemCountWithinEntrySum){
		this.itemCountWithinEntrySum = itemCountWithinEntrySum;
	}
	
	public int getItemCountWithinEntrySum(){
		return itemCountWithinEntrySum;
	}
	
	public void setNotEmpty(){
		notEmpty = true;
	}
	
	public void resetNotEmpty(){
		notEmpty = false;
	}
	  
	public boolean getNotEmpty(){
		return notEmpty;
	}
	public void resetNotEmptyCount(){
		notEmptyCount = 0;
	}
	
	public void increInputCount(int n){
		inputCount = inputCount+ n;
	}
	
	public void increOutputCount(int n){
		outputCount = outputCount + n;
	}
	
	public void increNotEmptyCount(){
		notEmptyCount++;
	}
	
	public void increItemCountWithinEntrySum(int n){
		itemCountWithinEntrySum += n;
	}
	
	public void increDeltaStartTokenCount(int n){
		deltaStartTokenCount += n;
	}
	
	public int getInputCount(){
		return inputCount;
	}
	
	public int getOutputCount(){
		return outputCount;
	}
	
	private void resetInputCount(){
		inputCount = 0;
	}
	
		
	public void resetOutputCount(){
		outputCount = 0;
	}
		
	public void setUnitCost(double newUnitCost){
		unitCost = newUnitCost;
	}
	
	
	public long getTime(){
		return lastStamp;
	}
	
	public int getNotEmptyCount(){
		return notEmptyCount;
	}
	
	/**
	 * Insert the method's description here.
	 * Creation date: (5/9/2003 2:06:18 PM)
	 * @return java.lang.Object
	 */
	public Object getEvaluatedClass() {
		return evaluatedClass;
	}
	
	/**
	 * Insert the method's description here.
	 * Creation date: (5/10/2003 1:17:54 PM)
	 * @return java.lang.String
	 */
	public String getMethodDesc() {
		return methodDesc;
	}
	
	/**
	 * reset the curTokenCount
	 */
	public void resetDeltaTokenCount() {
		deltaTokenCount = 0;
	}
	
	public void resetDeltaStartTokenCount(){
		deltaStartTokenCount = 0;
	}
	
	public void resetTime(){
		lastStamp = 0;
	}
	
	public int getCurTokenCount(){
		return curTokenCount;
	}
	
	public int getDeltaTokenCount(){
		return deltaTokenCount;
	}
	
	public int getDeltaStartTokenCount(){
		return deltaStartTokenCount;
	}
	
	public int getCurStartTokenCount(){
		return curStartTokenCount;
	}
	
	public void pauseTokenCount(int tokenId) {
		int delta = tokenId - lastTokenCount;
		deltaTokenCount += delta;
		curTokenCount += delta;
		
	}
	
	public void pauseStartTokenCount(int startTokenId){
		int delta = startTokenId - lastStartTokenCount;
		deltaStartTokenCount += delta;
	    curStartTokenCount += delta;	
	}
	
	public void tokenCount(int tokenId) {
		this.lastTokenCount = tokenId;	
	}
	
	public void startTokenCount(int startTokenId){
		this.lastStartTokenCount = startTokenId;
	}
	
	public void reset(){
		resetNotEmptyCount();
		resetInputCount();
		resetOutputCount();
		resetDeltaTokenCount();
		resetDeltaStartTokenCount();
		resetTime();
		itemCountWithinEntrySum = 0;
	}


	public String toString() {
		return evaluatedClass.toString()+methodDesc;
	}
	
	public Object clone(){
		ClassExecTimeKeeper o = new ClassExecTimeKeeper(this.evaluatedClass, this.methodDesc);
		o.lastStamp = lastStamp;
		o.lastStartRealTime = lastStartRealTime;
		return o;
	}
}
