package edu.wpi.cs.dsrg.xmldb.xat.automata.utils;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.logicaloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.Select;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xmlstreamexpression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.DagTraversal;

import java.util.*;
/**
 * Provide utilities for visiting a semantics-focused or a stream logical plan
 * Creation date: (1/21/2003 4:07:43 PM)
 * @author: Hong Su
 */

public class FindOpVisitor {

	//expedite finding the intermediate operators between a source operator and a destination operator
	//keyed on an operator, and the object returned is another hashmap,
	//the returned hashmap is keyed on the destination operator
	//in this inner hashmap, it returns the list of intermediate operators between the ops at the first and the second level
	static HashMap intermediateOpMgr = new HashMap();
	
	static List nodesVisited = new ArrayList();
	

	//hashed on a child node, return its position among its siblings 
	static HashMap childPosMgr = new HashMap();
	
	static HashMap opToAncestorNodeNavs = new HashMap();
	static HashMap opToDescTokenNavs = new HashMap();
	
	static HashMap tokenNavToRelatedTokenNavs = new HashMap();

	/**
	 * find whether there exists an extraction parent requiring a storing method of COMPLETE_TREE
	 * Creation date: (1/21/2003 4:26:29 PM)
	 * @return boolean
	 * @param navOp edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStream
	 */
	static public boolean existsExtractParentFor(
		edu
			.wpi
			.cs
			.dsrg
			.xmldb
			.xat
			.common
			.operator
			.streamoperator
			.xmloperator
			.NavigateStream navOp,
		int storingMethod) {

		/*	boolean exists = false;
			
			DagNode navNode = (DagNode)(navOp.getNode());
		
			DagNode[] parents = navNode.getParents();
		
			for (int i=0; i<navNode.getParentsSize(); i++){
				XATStreamOperator parentOp = (XATStreamOperator)(parents[i].getOperator());
				if (parentOp instanceof Extract){
					int[] methods = ((Extract)parentOp).getStoringMethods();
					if (methods[storingMethod] == 1){
						exists = true;
						break;
						}
				}
			}
			
			return exists;*/
		return false;
	}
	/**
	 * find ancestor operators
	 * also the operators are ordered by their closeness to the node
	 * Creation date: (1/30/2003 11:00:12 PM)
	 * @return java.util.Vector
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
	 */
	static public List findAncestorOps(DagNode node) {

		List result = new ArrayList();

		for (int i = 0; i < node.getParentsSize(); i++) {
			XATOperator parentOp = node.getParent(i).getOperator();
			if (!result.contains(parentOp))
				result.add(parentOp);

			List ancestors = findAncestorOps((DagNode) node.getParent(i));
			for (Iterator it = ancestors.iterator(); it.hasNext();) {
				XATOperator ancestorOp = (XATOperator) it.next();
				if (!result.contains(ancestorOp))
					result.add(ancestorOp);
			}

		}

		return result;
	}
	
	/*
	 * @param node DagNode
	 * @return List find all ancestor nodeNavs of node
	 */
	static public List findAncestorNodeNavs(DagNode node){
		List result = new ArrayList();

		for (int i = 0; i < node.getParentsSize(); i++) {
			XATOperator parentOp = node.getParent(i).getOperator();
			if (parentOp instanceof NavUnnestStream
					&& parentOp instanceof NavCollectionStream
					&& !result.contains(parentOp))
				result.add(parentOp);

			List ancestors = findAncestorNodeNavs((DagNode) node.getParent(i));
			for (Iterator it = ancestors.iterator(); it.hasNext();) {
				XATOperator ancestorOp = (XATOperator) it.next();
				if (!result.contains(ancestorOp))
					result.add(ancestorOp);
			}

		}

		return result;
		
	}
	
	static public List quickGetAncestorNodeNavs(DagNode node){
		XATStreamOperator op = (XATStreamOperator)node.getOperator();
		List result = (List)opToAncestorNodeNavs.get(op);
		if (result == null){
			result = getAncestorNodeNavs(node);
			opToAncestorNodeNavs.put(op, result);
		}
		return result;
	}
	
	static public List getAncestorNodeNavs(DagNode node){
		List result = new ArrayList();

		for (int i = 0; i < node.getParentsSize(); i++) {
			XATOperator parentOp = node.getParent(i).getOperator();
			if (parentOp instanceof NavUnnestStream
					&& parentOp instanceof NavCollectionStream
					&& !result.contains(parentOp))
				result.add(parentOp);

			List ancestors = findAncestorNodeNavs((DagNode) node.getParent(i));
			for (Iterator it = ancestors.iterator(); it.hasNext();) {
				XATOperator ancestorOp = (XATOperator) it.next();
				if (!result.contains(ancestorOp))
					result.add(ancestorOp);
			}

		}

		return result;
		
	}

	/**
		 * find nearest ancestor extract operators
		 * the returned results must be on the depth from node
		 * Creation date: (1/30/2003 11:00:12 PM)
		 * @return java.util.Vector
		 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
		 */
	static public List findNearestAncestorExtracts(DagNode node) {

		List result = new ArrayList();

		List curChildSet = new ArrayList();
		curChildSet.add(node);

		while (true) {
			List newChildSet = new ArrayList();

			for (Iterator it = curChildSet.iterator(); it.hasNext();) {
				DagNode curChild = (DagNode) it.next();
				for (Iterator it2 = curChild.getParents(); it2.hasNext();) {
					DagNode curParent = (DagNode) it2.next();

					XATOperator op = curParent.getOperator();
					newChildSet.add(curParent);
					if (op instanceof ExtractStream)
						result.add(op);
				}

				if (result.size() > 0)
					return result;

				//this level does not have extract yet, continue
				if (newChildSet.size() > 0)
					curChildSet = newChildSet;
				else
					return result;
			}

		}
	}

	static public XATStreamOperator findExtractDesc(DagNode node) {
		if (node.getOperator() instanceof ExtractStream)
			return (XATStreamOperator) node.getOperator();

		List descs = FindOpVisitor.getDescNodes(node);
		for (Iterator it = descs.iterator(); it.hasNext();) {
			DagNode desc = (DagNode) it.next();
			if (desc.getOperator() instanceof ExtractStream)
				return (XATStreamOperator) desc.getOperator();
		}

		return null;

	}

	/**
			 * find nearest ancestor structural join operators
			 * the returned results must be on the depth from node
			 * Creation date: (1/30/2003 11:00:12 PM)
			 * @return java.util.Vector
			 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
			 */
	static public InTimeSJStream findNearestAncestorSJ(DagNode node) {
		if (node.getOperator() instanceof InTimeSJStream)
			return (InTimeSJStream) node.getOperator();

		List curChildSet = new ArrayList();
		curChildSet.add(node);

		while (true) {
			List newChildSet = new ArrayList();

			for (Iterator it = curChildSet.iterator(); it.hasNext();) {
				DagNode curChild = (DagNode) it.next();
				for (Iterator it2 = curChild.getParents(); it2.hasNext();) {
					DagNode curParent = (DagNode) it2.next();

					XATOperator op = curParent.getOperator();
					newChildSet.add(curParent);
					if (op instanceof InTimeSJStream)
						return (InTimeSJStream) op;
				}

				//this level does not have extract yet, continue
				if (newChildSet.size() > 0)
					curChildSet = newChildSet;
				else
					return null;
			}

		}
	}

	/**
	 * this method only supports node that has unique parent
	 * Creation date: (4/30/2003 9:28:56 PM)
	 * @return int
	 * @param childNode edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
	 * @param parentNode edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
	 */
	static public int findChildOpPosInParentOp(
		DagNode childNode,
		DagNode parentNode) {

		Integer childPos = (Integer) childPosMgr.get(childNode);

		if (childPos != null)
			return childPos.intValue();

		XATNode[] childNodes = parentNode.getChildren();

		int length = childNodes.length;

		if (length == 1) {
			childPosMgr.put(childNode, new Integer(0));
			//childNode must be at pos 0
			return 0;
		}

		int intChildPos = -1;

		for (int j = 0; j < length; j++) {
			if (childNodes[j] == childNode) {
				intChildPos = j;
				break;
			}
		}

		childPosMgr.put(childNode, new Integer(intChildPos));

		return intChildPos;
	}
	/**
	 * find the operator which generates this column
	 * note! might check whether the source is also in an existential format
	 * Creation date: (2/20/2003 7:21:19 PM)
	 * @return java.util.ArrayList
	 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator
	 * @param colName java.lang.String
	 */
	static public DagNode findColGeneratingSource(
		XATOperator op,
		String colName) {

		//select must have only one single child
		XATNode[] childrenNode = op.getNode().getChildren();

		boolean found = false;

		for (int i = 0; i < childrenNode.length; i++) {

			XATNode childNode = childrenNode[i];

			XATOperator childOp = (XATOperator) childNode.getOperator();

			if (childOp instanceof StreamNavigate
				|| childOp instanceof PureNavigateAutoStream)
				continue;

			Object[] outSchema = childOp.getNode().getSchema().toArray();

			if (childOp instanceof ExtractStream
				|| childOp instanceof Extract
				//	|| childOp instanceof NavigateStream
				//	|| childOp instanceof StreamNavigate
				|| childOp instanceof Navigate)
				if ((outSchema.length == 1
					&& outSchema[0].toString().equals(colName))
					|| (childOp instanceof Navigate
						&& ((Navigate) childOp).getOutputColumnName().equals(
							colName)))
					//this operator has only one output column and equals to the colname
					//this operator must be the source
					//this is for extract which does not have inschema
					return (DagNode) childOp.getNode();

			//not an extract
			DagNode source =
				findColGeneratingSource(
					(XATOperator) childNode.getOperator(),
					colName);
			if (!(source == null)) {
				return source;
			}

		} /* for (Iterator j = childOp.getInputQueues(); j.hasNext();) {
																						
																						     Vector inSchema = ((XATQueue) j.next()).getSchema().getNames();
																						
																						     if (outSchema.contains(colName)) {
																						
																						         if ((!(inSchema.contains(colName))) {
																						             //the col is generated in this op
																						
																						             return childOp;
																						
																						         } else {
																						             //though the col is not directly generated by this column, it processes this col and this processing could be shortcircuited as well
																						
																						             XATStreamOperator source =
																						                 findColGeneratingSource((XATStreamOperator) childNode.getOperator(), colName);
																						
																						             if (!(source == null)) {
																						                 return source;
																						             }
																						         }
																						     }
																						 }*/

		return null;
	} /**
										 * find the operators on the immediate child level that generate/pass the specified colName in op 
										 * Creation date: (2/20/2003 7:21:19 PM)
										 * @return java.util.ArrayList
										 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator
										 * @param colName java.lang.String
										 */
	static public XATStreamOperator findColsGeneratePropagateChildLevelOps(
		XATStreamOperator op,
		String colName) {
		//select must have only one single child
		XATNode[] childrenNode = op.getNode().getChildren();
		for (int i = 0; i < childrenNode.length; i++) {

			XATNode childNode = childrenNode[i];
			Set outSchema = childNode.getSchema();
			if (outSchema.contains(colName)) {

				return (XATStreamOperator) childNode.getOperator();
			}

		}

		return null;
	} /**
										 * find all the operators that generate/pass the specified colName in op
										 * Creation date: (2/20/2003 7:21:19 PM)
										 * @return java.util.ArrayList
										 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator
										 * @param colName java.lang.String
										 */
	static public List findColsGeneratePropagateOps(
		XATStreamOperator op,
		String colName) {
		ArrayList result = new ArrayList();
		//select must have only one single child
		XATNode[] childrenNode = op.getNode().getChildren();
		boolean found = false;
		for (int i = 0; i < childrenNode.length; i++) {

			XATNode childNode = childrenNode[i];
			Set outSchema = childNode.getSchema();
			Set inSchema = childNode.getSchema();
			if (outSchema.contains(colName)) {

				if (!(inSchema.contains(colName))) {
					//the col is generated in this column
					result.add(childNode.getOperator());
					found = true;
				} else {
					//though the col is not directly generated by this column, it processes this col and this processing could be shortcircuited as well
					result.add(childNode.getOperator());
					result.addAll(
						findColsGeneratePropagateOps(
							(XATStreamOperator) childNode.getOperator(),
							colName));
				}
			}
			if (found) {
				break;
			}
		}

		return result;
	} /**
	
										 * find ancestor operators
										 * Creation date: (1/30/2003 11:00:12 PM)
										 * @return java.util.Vector
										 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
										 */
	static public List findInterOpsNeitherPureNavNorDataDriven(
		DagNode srcNode,
		DagNode targetNode) {

		ArrayList result = new ArrayList();
		for (Iterator i = srcNode.getParents(); i.hasNext();) {

			DagNode parentNode = (DagNode) i.next();
			if (parentNode == targetNode) {
				result = new ArrayList();
				return result;
			} else {
				List tempResult =
					findInterOpsNeitherPureNavNorDataDriven(
						parentNode,
						targetNode);
				if (tempResult.size()>0) { //target is found
					result = new ArrayList();
					XATStreamOperator parentOp =
						(XATStreamOperator) parentNode.getOperator();
					if (!(parentOp instanceof PureNavigateAutoStream)
						&& parentOp.getProperty("DATA_DRIVEN_INVOKING").equals(
							"0")) // && !(parentOp instanceof ExtractByUnnestStream))
						result.add(parentOp);
					result.addAll(tempResult);
				}
			}
		}
		return result;
	} /**
											 * find intermediate operators that are not pure nav neither data driven
											 * Creation date: (1/30/2003 11:00:12 PM)
											 * @return java.util.Vector
											 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
											 */
	static public List findInterOps(DagNode srcNode, DagNode targetNode) {

		ArrayList result = new ArrayList();
		for (Iterator i = srcNode.getParents(); i.hasNext();) {

			DagNode parentNode = (DagNode) i.next();
			if (parentNode == targetNode) {
				return result;
			} else {
				List tempResult =
					findInterOpsNeitherPureNavNorDataDriven(
						parentNode,
						targetNode);
				if (tempResult.size() > 0) { //target is found
					XATStreamOperator parentOp =
						(XATStreamOperator) parentNode.getOperator();
					result.add(parentOp);
					result.addAll(tempResult);
				}
			}
		}
		return result;
	}

	/* find all intermediate operator between srcNode and targetNode
		 * Creation date: (1/30/2003 11:00:12 PM)
		 * @return java.util.Vector
		 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
		 */
	static public List findAllInterOps(DagNode srcNode, DagNode targetNode) {

		ArrayList result = null;
		for (Iterator i = srcNode.getParents(); i.hasNext();) {

			DagNode parentNode = (DagNode) i.next();
			if (parentNode == targetNode) {
				if (result==null)
				 result = new ArrayList();
				 
				//return result;
			} else {
				List tempResult = findAllInterOps(parentNode, targetNode);
				if (tempResult != null) { //target is found
					XATOperator parentOp =
						(XATOperator) parentNode.getOperator();
					if (result ==null)
					 result = new ArrayList();
					result.add(parentOp);
					result.addAll(tempResult);
				}
			}
		}
		return result;
	}

	/**
				 * find the needed storing method of a logic extractOp
				 * Creation date: (2/17/2003 11:02:49 AM)
				 * @return int[]
				 * @param nav edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStream
				 */
	static public int[] findNeedFormats(Extract extractOp) {
		//represent COMPLETE_TREE,TPED_VALUE,NODE_ID,DOM, ATTRS
		int[] results = { 0, 0, 0, 0, 0 };
		try {

			String outputColName = extractOp.getOutputColumnName();
			DagNode extractNode = (DagNode) (extractOp.getNode());
			List ancestors = findAncestorOps(extractNode);

			for (int i = 0; i < ancestors.size(); i++) {

				XATOperator ancestor = (XATOperator) ancestors.get(i);
				if ((ancestor instanceof StreamSelect)
					|| (ancestor instanceof SelectStream)) {
					//this ensures it works for both bottom-up or top-down     
					boolean typedValGoOn = false;
					boolean attrGoOn = false;
					NavigationStep[] dest = extractOp.getDestination();
					if (!(dest[dest.length - 1] instanceof AttStep)) {
						if (results[ExtractStream.TYPED_VALUE] == 0)
							typedValGoOn = true;
					} else if (results[ExtractStream.ATTRS] == 0)
						attrGoOn = true;
					if (typedValGoOn || attrGoOn) {

						if (ancestor
							.getNode()
							.getChild(0)
							.getSchema()
							.contains(outputColName)) {
							//if the output column is needed in a select's input
							//see whether output column is applied by the expression
							ExpressionStr exprStr = null;
							if (ancestor instanceof StreamSelect) {
								Expression expr =
									((StreamSelect) ancestor).getExpression();
								ExpressionStrImp exprFac =
									new ExpressionStrImp();
								exprStr = exprFac.createExpressionStr(expr);
							} else
								exprStr =
									((SelectStream) ancestor).getExpression();
							Vector terminalColumnExprs =
								exprStr.getTerminalColumnExprs();
							Vector terminals = new Vector();
							for (int j = 0;
								j < terminalColumnExprs.size();
								j++)
								terminals.add(
									(
										(ColumnName)
											(
												(TerminalExpressionStr) terminalColumnExprs
										.get(j))
										.getValue())
										.getColumnName());
							if (terminals.contains(outputColName)) {
								//make sure this outputColName is for attribute value
								if (typedValGoOn)
									results[ExtractStream.TYPED_VALUE] = 1;
								else //must be attrGoOn
									results[ExtractStream.ATTRS] = 1;
							} // break;
						} //end of if (ancestor.getNode().getChild(0).getSchema().contains(outputColName))
					} //end of if (results[ExtractStream.TYPED_VALUE] == 0) 
				} //end of if ((ancestor instanceof StreamSelect) || (ancestor instanceof SelectStream))

				if ((ancestor instanceof Tagger)
					|| (ancestor instanceof TaggerStream)) {
					if (results[ExtractStream.COMPLETE_TREE] == 0) {
						String[] patternCols =
							((Tagger) ancestor).getPattern().getColumnNames();
						for (int j = 0; j < patternCols.length; j++) {
							if (patternCols[j].equals(outputColName)) {
								//if the output column is needed in a tagger's pattern list
								results[ExtractStream.COMPLETE_TREE] = 1;

							}
						}
					}
				}

				if ((ancestor instanceof NavCollection)
					|| (ancestor instanceof NavUnnest)
					|| ((ancestor instanceof NavigateStream)
						&& (!(ancestor instanceof PureNavigateAutoStream))))
					//ancestor is a regular navigate stream rather than a streamnavigate
					if (results[ExtractStream.DOM] == 0) {
						// if (ancestor.getNode().getChild(0).getSchema().contains(outputColName))
						String entryCol = null;
						if (ancestor instanceof NavCollection
							|| ancestor instanceof NavUnnest)
							entryCol =
								((Navigate) ancestor)
									.getEntryPoint()[0]
									.getStepValue();
						else if (
							ancestor instanceof NavigateStream
								&& (!(ancestor
									instanceof PureNavigateAutoStream)))
							entryCol =
								((NavigateStream) ancestor)
									.getEntryPoint()[0]
									.getStepValue();
						if (entryCol.equals(outputColName))
							//if the output column is needed in a regular navigate
							results[ExtractStream.DOM] = 1;
					} //end of if if (results[ExtractStream.DOM] == 0)
				if (ancestor instanceof StructuralJoinStream
					|| ancestor instanceof StructuralJoin
					|| ancestor instanceof NaturalJoinStream) {
					String joinedOn = "";

					if (ancestor instanceof StructuralJoinStream)
						joinedOn =
							((StructuralJoinStream) ancestor)
								.getBranchNavOp()
								.getBinding();

					//						we now check whether its node id is needed, i.e., whether the count of its occurrence is needed
					//FOR    $o IN document("auction.xml")/open_auction_list/auction,
					//	   $a IN $o/initial
					//RETURN <result>$o/reserve</result>
					//the count of $a is needed when all the other representations are not needed
					if (extractOp instanceof ExtractUnnest
						|| extractOp instanceof ExtractByUnnestStream) {
						if (extractOp
							.getEntryPoint()[0]
							.getStepValue()
							.equals(joinedOn)
							&& !ancestor.getNode().getSchema().contains(
								extractOp.getOutputColumnName())
							|| ancestor instanceof NaturalJoinStream 
						//	|| ancestor.getNode().getSchema().contains(extractOp.getOutputColumnName())
							)
							results[ExtractStream.NODE_ID] = 1;

					}

				}

			} //end of for

			if (results[ExtractStream.COMPLETE_TREE] == 0
				&& results[ExtractStream.DOM] == 0
				&& results[ExtractStream.NODE_ID] == 0
				&& results[ExtractStream.TYPED_VALUE] == 0) {
				//check whether node id is needed
				boolean nodeid = true;
				List nodeAncestors = findAncestorOps(extractNode);
				Vector nodes =
					TreeTraversal.treeTopDownTraverse(
						((XATOperator) nodeAncestors
							.get(nodeAncestors.size() - 1))
							.getNode());
				for (Iterator it = nodes.iterator(); it.hasNext();) {
					DagNode otherNode = (DagNode) it.next();
					XATOperator otherOp = otherNode.getOperator();
					if (otherOp instanceof Navigate) {
						String entry =
							((Navigate) otherOp)
								.getEntryPoint()[0]
								.getStepValue();
						if (entry
							.equals(
								((XATStreamOperator) extractOp)
									.getOutputColumnName())) {
							nodeid = false;
							//can be removed
							break;
						}
					}

				} //end of for

				if (nodeid)
					results[ExtractStream.NODE_ID] = 1;
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		//for getting rid of complete tree
		if (results[ExtractStream.COMPLETE_TREE] == 1) {
			results[ExtractStream.COMPLETE_TREE] = 0;
			results[ExtractStream.DOM] = 1;
		}

		return results;
	} /**
										 * return the node that consumes the "colName" in "node" in an existential filtering manner
										 * the node's op must be a filtering op, i.e., select or join
										 * Creation date: (4/10/2003 10:01:14 PM)
										 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
										 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
										 * @param colName java.lang.String
										 */
	public static DagNode findNodeExistentialConsuming(
		DagNode node,
		String colName) {
		DagNode traverseNode = (DagNode) node.getParent();
		DagNode lastNode = node;
		while (traverseNode != null) {

			XATOperator traverseOp = traverseNode.getOperator();
			if (traverseOp instanceof SelectStream
				|| traverseOp instanceof StructuralJoinStream) {

				Set inSchema = lastNode.getSchema();
				Set outSchema = traverseNode.getSchema();
				if (!outSchema.contains(colName))
					return traverseNode;
			}

			lastNode = traverseNode;
			traverseNode = (DagNode) traverseNode.getParent();
		}

		return null;
	} /**
										 * given a source operator and a destination, find all the intermediate operators between they two.
										 * Creation date: (2/3/2003 9:12:28 PM)
										 * @return java.util.Vector
										 * @param source edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator
										 * @param dest edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator
										 */
	static public List findOpsOnPathForPartition(
		XATOperator sourceOp,
		XATOperator destOp,
		boolean store) {

		HashMap sourceOpRelated = (HashMap) intermediateOpMgr.get(sourceOp);
		if (sourceOpRelated != null) {
			ArrayList ops = (ArrayList) sourceOpRelated.get(destOp);
			if (ops != null)
				return ops;
		}

		DagNode sourceNode = (DagNode) sourceOp.getNode();
		DagNode destNode = (DagNode) destOp.getNode();
		List result =
			findInterOpsNeitherPureNavNorDataDriven(sourceNode, destNode);
		/*for (Iterator i = sourceNode.getParents(); i.hasNext();) {
		
		  DagNode currentNode = (DagNode) i.next();
		
		  Vector ancestors = findAncestorOps(currentNode);
		
		  if (ancestors.contains(destNode.getOperator())) {
		      //if the parent is on the path to dest, add it in
		      XATStreamOperator currentOp = (XATStreamOperator) currentNode.getOperator();
		
		      if (!(currentOp instanceof PureNavigateAutoStream))
		          result.add(currentOp);
		
		      //also add on the nodes from the parent to the dest
		      //however avoid duplicates
		      ArrayList v = findOpsOnPathForPartition(currentOp, destOp, false);
		
		      for (int j = 0; j < v.size(); j++) {
		          if (!(result.contains(v.get(j)))) {
		              result.add(v.get(j));
		          }
		      }
		
		  }
		}*/

		if (store) //store the results
			if (sourceOpRelated != null) {
				sourceOpRelated.put(destOp, result);
			} else {
				sourceOpRelated = new HashMap();
				sourceOpRelated.put(destOp, result);
				intermediateOpMgr.put(sourceOp, sourceOpRelated);
			}

		return result;
	} /**
										 * find whether column by colName consumed by node is a collection type
										 * Creation date: (4/10/2003 9:14:12 PM)
										 * @return boolean
										 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
										 * @param colName java.lang.String
										 */
	static public boolean isCollectionType(DagNode node, String colName) {

		XATOperator colSrcOp =
			findColGeneratingSource(
				(XATStreamOperator) node.getOperator(),
				colName)
				.getOperator();
		if (colSrcOp instanceof ExtractColl
			|| colSrcOp instanceof ExtractByCollectionStream)
			return true;
		return false;
	} /**
											 * find whether the given NavNest/ExtractCol filters or not filters
											 * if not filters, we say the NavNest/ExtractCol is an access operator
											 * @return boolean
											 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
											 * @param colName java.lang.String
											 */
	static public boolean isNotAccess(DagNode node) throws RuntimeException {
		XATStreamOperator op = (XATStreamOperator) node.getOperator();
		if (!(op instanceof ExtractColl))
			throw new RuntimeException("Should not pass a non ExtractColl operator to isAccess");
		else //revisit: a non-access type operator must be operated by an selection operator since
			//only those navigation appearing in a where clause will filter if the desired pattern does not appear
			return XATPhyTreeTraversal.selOpsReliedOn(node);
	} /*
											 * @param planNode DagNode must be an navigate operator
											 * find its corresponding extract operator, if any
											 */
	public static ExtractStream findAssociatedExtract(DagNode planNode) {
		//find the associated extract operator
		XATStreamOperator associatedExtract = null;
		for (Iterator it = planNode.getParents(); it.hasNext();) {
			XATStreamOperator parentOp =
				(XATStreamOperator) ((XATNode) it.next()).getOperator();
			if (parentOp instanceof ExtractStream) {
				//there should be at most one
				associatedExtract = parentOp;
				break;
			}
		}

		return (ExtractStream) associatedExtract;
	} /*
										 * whether there exists a simple selection on it
										 * revisit: hacking, we assume the selection must be a simple one
										 */
	public static SelectStream simpleSelDefinedOn(ExtractStream extract) {
		DagNode extractNode = (DagNode) ((XATStreamOperator) extract).getNode();
		SelectStream associatedSel = null;
		for (Iterator it = extractNode.getParents(); it.hasNext();) {
			XATStreamOperator parentOp =
				(XATStreamOperator) ((XATNode) it.next()).getOperator();
			if (parentOp instanceof SelectStream) {
				//there should be at most one
				associatedSel = (SelectStream) parentOp;
				break;
			}
		}

		return associatedSel;
	} 
	
	//find all desc nodes of root node
	
	
	public static List getDescNodes(DagNode rootNode) {
		List result = new ArrayList();
		
		if (rootNode.getChildrenSize() == 0) { //	result.add(rootNode);
			return result;
		} else {
			DagNode curNode = rootNode;
			for (int i = 0; i < curNode.getChildrenSize(); i++) {
				DagNode childNode = (DagNode) curNode.getChild(i);
			//	if (!nodesVisited.contains(childNode)){
				result.add(childNode);
				result.addAll(getDescNodes(childNode));
			//	nodesVisited.add(childNode);
			//	}
			}

		}

		return result;
	}
	
	public static List quickGetDescTokenNavs(DagNode node){
		XATStreamOperator op = (XATStreamOperator)node.getOperator();
		List result = (List)opToAncestorNodeNavs.get(op);
		if (result == null){
			result = getDescTokenNavs(node);
			opToDescTokenNavs.put(op, result);
		}
		return result;
	}
	
	public static List quickGetRelatedTokenNavs(DagNode node){
		List relatedTokenNavs = (List)tokenNavToRelatedTokenNavs.get(node.getOperator());
		if (relatedTokenNavs == null){
			relatedTokenNavs = new ArrayList();
			relatedTokenNavs.addAll(getDescTokenNavs(node));
			relatedTokenNavs.addAll(getAncesTokenNavs(node));
			tokenNavToRelatedTokenNavs.put(node, relatedTokenNavs);
		}
		return relatedTokenNavs;
	}
	
	/*
	*@param rootNode DagNode
	*find all desc tokenNav or 
	*/
	public static List getDescTokenNavs(DagNode node) {
		List result = new ArrayList();
		
		if (!(node.getOperator() instanceof PureNavigateAutoStream)) { //	result.add(rootNode);
			return result;
		} else {
			result.add(node.getOperator());
			result.addAll(getDescTokenNavs((DagNode)node.getChild(0)));
		}

		return result;
	}
	
	public static List getAncesTokenNavs(DagNode node){
		List result = new ArrayList();
		
		for (int i = 0; i < node.getParentsSize(); i++) {
			DagNode parentNode = (DagNode) node.getParent(i);
			XATStreamOperator parentOp = (XATStreamOperator)parentNode.getOperator();
			if (parentOp instanceof PureNavigateAutoStream){
			result.add(parentNode.getOperator());
			result.addAll(getAncesTokenNavs(parentNode));
			}
		}
		
		return result;
	}
	
	

	public static boolean hasNatureJoinAncestor(DagNode node) {
		List ancestors = findAncestorOps(node);

		for (Iterator it = ancestors.iterator(); it.hasNext();) {
			XATOperator ancestor = (XATOperator) it.next();
			if (ancestor instanceof NaturalJoinStream)
				return true;
		}

		return false;
	}
	
	public static boolean hasSJAncestor(DagNode node) {
			List ancestors = findAncestorOps(node);

			for (Iterator it = ancestors.iterator(); it.hasNext();) {
				XATOperator ancestor = (XATOperator) it.next();
				if (ancestor instanceof StructuralJoinStream)
					return true;
			}

			return false;
		}

	static public int numOfNavs(XATDag dag) {
		int num = 0;
		List ops = DagTraversal.dagTopDownTraverse(dag);
		for (Iterator it = ops.iterator(); it.hasNext();) {
			XATOperator op = ((DagNode) it.next()).getOperator();
			if (op instanceof Navigate)
				num++;

		}
		return num;

	}

	static public int numOfSels(XATDag dag) {
		int num = 0;
		List ops = DagTraversal.dagTopDownTraverse(dag);
		for (Iterator it = ops.iterator(); it.hasNext();) {
			XATOperator op = ((DagNode) it.next()).getOperator();
			if (op instanceof Select)
				num++;

		}
		return num;

	}

	/*
	 * find the leave nodes between sjNode (which is a structuraljoin) and other sj nodes
	 */
	static public List findInterBetweenSJs(XATNode sjNode) {
		List leaveNodes = new ArrayList();
		for (int i = 0; i < sjNode.getChildrenSize(); i++) {
			XATNode childNode = sjNode.getChild(i);
			if (childNode.getOperator() instanceof Extract
				|| childNode.getOperator() instanceof ExtractStream
				|| childNode.getOperator() instanceof InTimeSJStream
				|| childNode.getOperator() instanceof StructuralJoin)
				leaveNodes.add(childNode);
			else
				leaveNodes.addAll(findInterBetweenSJs(childNode));

		}

		return leaveNodes;

	}
	
	/**
	 * get the output column in a sorted order and set up the output positions
	 * Creation date: (4/4/2003 2:43:43 PM)
	 * @return java.util.Vector
	 * @param schemaNameSet java.util.Set
	 */
	static public Vector getSchemaNameList(XATNode node) {
		//store the output column in a sorted order, based on their order in the overall input columns
		Vector nameListResult = new Vector();

		Set originalSchema = node.getSchema();
		XATOperator op = node.getOperator();

		if (op instanceof ExtractStream) {
			//there is only one item in originalSchema
			for (Iterator it = originalSchema.iterator(); it.hasNext();)
				nameListResult.add(it.next());
			return nameListResult;

		}

		List schema = SetConverter.convertToOrderList(originalSchema);

		//int outputColPos = 0;

		XATNode[] children = node.getChildren();

		//store all the input columns that appear in the output columns

		for (int j = 0; j < children.length; j++) {

			//List outputPosList = new ArrayList();

			//String[] childColNames =
			//    (String[]) children[j].getSchema().toArray(new String[0]);

			String[] childColNames =
				(String[]) SetConverter.convertToOrderList(
					children[j].getSchema()).toArray(
					new String[0]);

			for (int k = 0; k < childColNames.length; k++) {

				Object inputCol = childColNames[k];

				for (Iterator i1 = schema.iterator(); i1.hasNext();) {

					Object outputCol = i1.next();

					if (outputCol.equals(inputCol)) {
						nameListResult.add(outputCol);
						schema.remove(outputCol);
						//outputPosList.add(new Integer(k));
						break;
					}

				} //end of for i1

			} //end of for k

			//int currentOutputPosListLength = outputPosList.size();

			//convert list to int[]
			/*int[] outputPosArray = new int[outputPosList.size()];
			
			for (int count = 0; count < outputPosList.size(); count++) {
				outputPosArray[count] =
					((Integer) outputPosList.get(count)).intValue();
			}*/

		} //end of for j

		for (Iterator i2 = schema.iterator(); i2.hasNext();) {

			//only add for the first child, otherwise duplicate would be added
			nameListResult.add(i2.next());
		}

		return nameListResult;
	}
	
	public static List findEntryOps(XATStreamOperator op){
		List entryOps = new ArrayList();
		DagNode node = (DagNode)op.getNode();
		XATNode[] children = node.getChildren();
		int childrenNum = children.length;
		for (int i=0; i<childrenNum; i++){
			DagNode child = (DagNode)node.getChild(i);
			XATStreamOperator childOp = (XATStreamOperator)child.getOperator();
			if (childOp instanceof ExtractStream || childOp instanceof StructuralJoinStream)
				entryOps.add(childOp);
			else
				entryOps.addAll(findEntryOps(childOp));
				
		}
		
		return entryOps;
		
	}
	
	
}
