package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.AggregateStatisticsObject;
/**
 * This is the average tuple processing time calculator.  This takes the
 * number of tuples dequeued total and divides it by the time took to run total. This
 * gives us an estimate how long an operator spends processing 1 tuple.  
 */
public class AverageTupleProcessingTime extends AggregateStatisticsObject {
/**
 * AverageTupleProcessingTime constructor comment.
 */
public AverageTupleProcessingTime() {
	super();
}
/**
 * AverageTupleProcessingTime constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public AverageTupleProcessingTime(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
}
/**
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * 
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {
	Object oldCategory = stats.getCurrentCategory();
	stats.setCurrentCategory(this.getCategory());
	double time = Double.parseDouble(stats.getValue("TIME_TOOK_TO_RUN_TOTAL", this.getAssociatedQueryObject())); 
	double dequeued = Double.parseDouble(stats.getValue("NUMBER_OF_TUPLES_DEQUEUED_TOTAL", this.getAssociatedQueryObject()));
	if(dequeued == 0){
		this.setPropertyValue("0");		
	} else{
		double average_time = time / dequeued;	
		this.setPropertyValue("" + average_time);
	}	
	stats.setCurrentCategory(oldCategory);	
}
}
