package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.AggregateStatisticsObject;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsReference;
/**
 * This calculates the priority of an operator based on the formula:
 * (1-selectivity) / average_tuple_processing_time.  This was published in the 
 * Chain paper, although I am sure it can be found elsewhere.  
 */
public class GreedyPriorityCalculator extends AggregateStatisticsObject {
/**
 * GreedyPriority constructor comment.
 */
public GreedyPriorityCalculator() {
	super();	
}
/**
 * GreedyPriority constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public GreedyPriorityCalculator(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
}
/**
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * 
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {
	Object oldCategory = stats.getCurrentCategory();
	stats.setCurrentCategory(this.getCategory());
	try{		
		double average_cost = Double.parseDouble(stats.getValue("AVERAGE_TUPLE_PROCESSING_TIME", this.getAssociatedQueryObject(), StatisticsReference.AVERAGE));		
		double selectivity = Double.parseDouble(stats.getValue("SELECTIVITY", this.getAssociatedQueryObject(), StatisticsReference.AVERAGE));
//		System.out.println("##### Average Cost: " + average_cost + " #####");
//		System.out.println("##### Selectivity: " + selectivity + " #####");
		if(average_cost == 0){
			//keep the existing greedy priority
			//this.setPropertyValue("" + new Double(0));
		} else{
			this.setPropertyValue("" + ( (1 - selectivity) / average_cost));
		}
	} catch(NumberFormatException e){
		System.out.println(e.toString());
		System.out.println("Try to parse this into a Double: " + stats.getValue("AVERAGE_TUPLE_PROCESSING_TIME", this.getAssociatedQueryObject()));			
	}
	stats.setCurrentCategory(oldCategory);
}
/**
 * Insert the method's description here.
 * Creation date: (1/14/2003 10:45:41 AM)
 * @param newAssociatedOperator edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator
 */
public void setAssociatedQueryObject(XATQueryObject newAssociatedQueryObject) {
	super.setAssociatedQueryObject(newAssociatedQueryObject);
	//have to set the initial greedy priority
	setPropertyValue(getAssociatedQueryObject().getProperty("INITIAL_GREEDY_PRIORITY"));
	
}
}
