package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsObject;
/**
 * Calculates the number of times that operators were run for an entire query plan.  This
 * allows for a comparison between schedulers as to which scheduler runs operators more often
 * @author: Brad Pielech
 */
public class NumberOfTimesOperatorRunTotal extends StatisticsObject {
/**
 * NumberOfTimesOperatorRunTotal constructor comment.
 */
public NumberOfTimesOperatorRunTotal() {
	super();
}
/**
 * NumberOfTimesOperatorRunTotal constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public NumberOfTimesOperatorRunTotal(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
}
/**
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * 
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {

	Object oldCategory = stats.getCurrentCategory();
	stats.setCurrentCategory(this.getCategory());	
	XATTree tree = (XATTree)this.getAssociatedQueryObject();
	XATNode[] nodes = tree.convertToArray();
	int timesRun = 0;
	for (int i = 0; i < nodes.length; i++){
		XATOperator operator = nodes[i].getOperator();
		timesRun = timesRun + Integer.parseInt(stats.getValue("NUMBER_OF_TIMES_RUN", operator));		
	} 
	this.setPropertyValue("" + timesRun);
	stats.setCurrentCategory(oldCategory);	
}
}
