package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
/**
 * This will look at all of the output queues of the associated query object 
 * (which has to be cast to a StreamOperator b/c XATOperators do not have queues)
 * and sum the TUPLE_COUNT properties 
 */
public class NumberOfTuplesInInputQueues extends TuplesInQueueCalculator {
/**
 * NumberOfTuplesInOutoutQueues constructor comment.
 */
public NumberOfTuplesInInputQueues() {
	super();
}
/**
 * NumberOfTuplesInOutoutQueues constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public NumberOfTuplesInInputQueues(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
}
/**
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * 
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {

	XATStreamOperator op = (XATStreamOperator)this.getAssociatedQueryObject();
	Iterator queues = op.getInputQueues();
	int count = 0;
	/** iterate through queues */
	for (;queues.hasNext();) {
		XATQueue queue = (XATQueue)queues.next();
		if(!queue.isRegistered(op)) queue.register(op);
		count = count + queue.getQueueSize(op);		
	}
	this.setPropertyValue("" + count);		
	
}
}
