package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

//import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATSingleQueueStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.AggregateStatisticsObject;
/**
 * Calculates the selectivity of an operator by dividing the total possible output tuples 
 * by the total tuples outputted.
 * If the operator is a single queue stream operator, the selectivity will be calculated by
 * using the total number of tuples dequeued (not possible outputs).  This is due to a timing
 * issue with the stream controller and execution where the number of possible outputs is
 * incorrect.  Using the total dequeued resolves this issue
 * 
 */
public class SelectivityCalculator extends AggregateStatisticsObject {
/**
 * SelectivityCalculator constructor comment.
 */
public SelectivityCalculator() {
	super();	
}
/**
 * SelectivityCalculator constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public SelectivityCalculator(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);	
}
/**
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * 
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {
	Object oldCategory = stats.getCurrentCategory();
	stats.setCurrentCategory(this.getCategory());
	double outputted = 0;
	outputted = Double.parseDouble(stats.getValue("NUMBER_OF_TUPLES_OUTPUTTED", this.getAssociatedQueryObject())); 
	double possible = 0;
	possible = Double.parseDouble(stats.getValue("NUMBER_OF_TUPLES_DEQUEUED", this.getAssociatedQueryObject()));

	if(possible == 0){
		this.setPropertyValue("" + new Double(0.0));		
	} else{
		this.setPropertyValue("" + (outputted / possible));
	}	

	stats.setCurrentCategory(oldCategory);
}
}
