package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

//import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.AggregateStatisticsObject;
/**
 * This calculates the Total Number of Tuples in all input queues in the query plan.  This
 * is used to determine how a scheduling algorithm is affecting queue sizes . 
 * @author: Brad Pielech
 */
public class TotalTuplesInAllQueues extends AggregateStatisticsObject{
/**
 * AverageTuplesInQueues constructor comment.
 */
public TotalTuplesInAllQueues() {
	super();
}
/**
 * AverageTuplesInQueues constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public TotalTuplesInAllQueues(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
}
/**
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * 
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {

	/** 
	 * traverse tree and find all operators with queues and sum those tuple counts
	 * 
	 */
	XATTree tree = (XATTree)this.getAssociatedQueryObject();
	XATNode[] nodes = tree.convertToArray();
	double tupleCount = 0;
	for (int i = 0; i < nodes.length; i++){
		XATStreamOperator operator = (XATStreamOperator)nodes[i].getOperator();
		stats.updateValue("NUMBER_OF_TUPLES_IN_INPUT_QUEUES", operator);
		double thisCount = Double.parseDouble(stats.getValue("NUMBER_OF_TUPLES_IN_INPUT_QUEUES", operator));
		tupleCount += thisCount;
	} 	
	this.setPropertyValue("" + tupleCount);	
}
}
