package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.AggregateStatisticsObject;
/**
 * Calculate the output rate of the tree.  This is really just a normalized throughput, that
 * is, it takes the number of tuples outputted total (throughput) and divides it by
 * the time that the query has been running.  
 * @author: Brad Pielech
 */
public class TreeOutputRateCalculator extends AggregateStatisticsObject {
	private long startTime;
/**
 * OutputRateCalculator constructor comment.
 */
public TreeOutputRateCalculator() {
	super();
	startTime = System.currentTimeMillis();
}
/**
 * OutputRateCalculator constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public TreeOutputRateCalculator(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
	startTime = System.currentTimeMillis();
}
/**
 * This is semantically the same as <tt>calculateNewValue()</tt>, but this
 * provides a parameter which assists in the calculation.
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {
    Object oldCategory = stats.getCurrentCategory();
    stats.setCurrentCategory(this.getCategory());
       
    long timeRun = (System.currentTimeMillis() - startTime) / 1000;
    double throughput = Double.parseDouble(stats.getValue("THROUGHPUT", getAssociatedQueryObject()));
    double rate = throughput / timeRun;

    this.setPropertyValue("" + rate);
 
    stats.setCurrentCategory(oldCategory);
}
}
