package edu.wpi.cs.dsrg.xmldb.xat.common.calculators;

import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.AggregateStatisticsObject;
/**
 * Calculates the throughput of a tree. This is equal to the sum of the number of tuples the
 * roots have outputted
 * @author: Brad Pielech
 */
public class TreeThroughputCalculator extends AggregateStatisticsObject {
/**
 * OutputRateCalculator constructor comment.
 */
public TreeThroughputCalculator() {
	super();
}
/**
 * OutputRateCalculator constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public TreeThroughputCalculator(String name, edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject queryObject) {
	super(name, queryObject);
}
/**
 * This is semantically the same as <tt>calculateNewValue()</tt>, but this
 * provides a parameter which assists in the calculation.
 * Calculates the new Value of the given property of this operator.  This
 * should be overwritten by subclasses who can tailor this method to calculating 1
 * specific property. <p>
 * This method should update at least the local propertyValue and possibly the
 * value with the operator The entire StatisticsGatherer is available for assistance.   <p>
 *
 * The procedure for calculating new values should be as follows:
 *
 * <ol>
 * <li> Consult the statsTable for any propertyValues that are needed.
 * <li> Perform the required calculations using the propertyValues from step 1
 * <li> Update the local propertyValue.
 * <li> Update the lastTimeUpdated value by using <tt>lastTimeUpdated = System.currentTimeMillis();</tt>
 * </ol> 
 */
public void calculateNewValue() {
    Object oldCategory = stats.getCurrentCategory();
    stats.setCurrentCategory(this.getCategory());
    /** 
     * set the tree's output rate to be that of its root
     * if we have a DAG, then we will just average the average rates 
    */
    if (this.getAssociatedQueryObject() instanceof XATDag) {
        XATDag dag = ((XATDag) this.getAssociatedQueryObject());
        int throughput = 0;
        for (Iterator i = dag.getRoots(); i.hasNext();) {
            DagNode dagnode = (DagNode) i.next();
            XATOperator operator = dagnode.getOperator();
            throughput += Double.parseDouble(stats.getValue("NUMBER_OF_TUPLES_OUTPUTTED_TOTAL", operator));
        }
        //get the old throughput value and figure out how many we outputted since then 
   
        this.setPropertyValue("" + throughput);
    } else {
        //nope, dont have a dag	
        XATOperator root =
            ((XATTree) this.getAssociatedQueryObject()).getRoot().getOperator();

		double throughput = Double.parseDouble(stats.getValue("NUMBER_OF_TUPLES_OUTPUTTED_TOTAL", root));
        this.setPropertyValue("" + throughput);
    }
    stats.setCurrentCategory(oldCategory);
}
}
