package edu.wpi.cs.dsrg.xmldb.xat.common.dag;

/**
 * Insert the type's description here.
 * Creation date: (2003-1-9 22:13:21)
 * @author: JinHui Jian
 */
import java.util.Iterator;
import java.util.Set;
import java.io.Serializable;

import edu.wpi.cs.dsrg.utils.DeepCopyMaker;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNodeImp;

public class DagNodeImp extends XATNodeImp implements DagNode, Serializable {

	/**
	 * DagNodeImp constructor comment.
	 */
	public DagNodeImp() {
		super();
	}
	/**
	 * adds a new parent.  The parent is added to the right of the previous parents
	 * @param parent edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.DagNode
	 */
	public void addParent(XATNode newParent) {
		if (!parent.contains(newParent)) {
			parent.add(newParent);
		}
	}

	/**
	 * Removes the specified parent
	 * @param parent edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.DagNode
	 */
	public void deleteParent(XATNode removeParent) {
		parent.remove(removeParent);
	}

	public void deleteAllParents() {
		parent.clear();
	}

	/**
	 * Deep clone this XATNode.
	 * @return Object The cloned result.
	 * @since 1.0
	 */
	public Object clone() {
		DagNode result = new DagNodeImp();
		try {
			result.setOperator(
				(XATOperator) DeepCopyMaker.makeDeepCopy(operator));
			result.setSchema((Set) DeepCopyMaker.makeDeepCopy(schema));
		} catch (Exception e) {
			System.out.println("Deep clone failed!");
			e.printStackTrace(System.err);
		}
		result.getOperator().setNode(result);
		return result;
	}

	/** 
	 * Get all the children nodes of this node.
	 * @return XATNode[] The array of all the children.
	 * @since 1.0
	 */
	public XATNode[] getChildren() {
		/* hideous, hideous kludge... java is too pedantic */
		return (XATNode[]) children.toArray(new DagNode[] {
		});
	}
	/**
	 * Add one child to this node. This should be called only by the XATTree.addNode().
	 * Remember the children are order unsensitive.
	 * @return int The number of children.
	 * @param new_child The new child node added.
	 * @since 1.0
	 */
	public int addChild(XATNode new_child) {
		if ((new_child != null) && !children.contains(new_child))
			children.add((DagNode) new_child);
		return children.size();
	}
	/**
	 * Add one child to this node at specified position. 
	 * This method should be called only by the XATTree.addNode().
	 * The children are order unsensitive.
	 * @return The number of children.
	 * @param new_child The new child node.
	 * @param position The position of new added node.
	 * @since 1.0
	 */
	public int addChild(XATNode new_child, int position) {
		children.add(position, (DagNode) new_child);
		return children.size();
	}

	/**
	 * Returns the parent at the specified index
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.DagNode
	 * @param id int
	 */
	public DagNode getParent(int id) {
		return (DagNode) parent.get(id);
	}
	/**
	 * Returns an Iterator of all of the parents
	 * @return Iterator of parents
	 */
	public Iterator getParents() {
		return parent.iterator();
	}
	/**
	 * Returns how many parents this DagNode has
	 * @return int
	 */
	public int getParentsSize() {
		return parent.size();
	}

	/**
	 * Returns the index of child
	 * @return int
	 */
	public int getIndexOfChild(DagNode dn) {
		for (int i = 0; i < children.size(); i++) {
			if (children.get(i) == dn)
				return i;
		}
		return -1;
	}

	/**
	 * Returns the index of parent
	 * @return int
	 */
	public int getIndexOfParent(DagNode dn){
		return parent.indexOf(dn);
	}

	/**
	 * Set the specified parent 
	 * @param id int
	 * @param newParent the new XATNode
	 */
	public void setParent(int id, XATNode newParent) {
		parent.set(id, newParent);
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2003-2-4 14:23:01)
	 * @return java.util.Set
	 * @param newSchema java.util.Set
	 */
	public Set setSchema(Set newSchema) {
		Set old = this.schema;
		this.schema = newSchema;
		return old;
	}

}
