package edu.wpi.cs.dsrg.xmldb.xat.common.dag;

import java.lang.reflect.Array;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;
import java.io.Serializable;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
/**
 * Insert the type's description here.
 * Creation date: (1/31/03 4:40:07 PM)
 * @author: Brad Pielech
 */
public class XATDagImp extends edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTreeImp implements XATDag, Serializable {
/**
 * Extends XATTree to allow for multiple roots.  Several convience methods are provided
 */
public XATDagImp() {
	super();
	
	}
/**
 * Adds a new root to the Dag.  This will be added to the right of the others
 * @param newRoot edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public void addRoot(DagNode newRoot) {	
	root.add(newRoot);	
}
    /**
     * Add a new parent node for a given DOMPatternNode node. 
     * The parent node is updated to have a link to the child node.
     * The child node is updates to have a link to the parent node.
     * @return void
     * @param parent The parent node it attached to.
     * @param child The child node attached.
     * @since 1.0
     */
    public void addNode(DagNode newNode, DagNode parentNode) {
        if (newNode == null || parentNode == null)
            return;
        if (newNode.getParent() ==null)newNode.setParent( parentNode); 
        else newNode.addParent(parentNode);
        parentNode.addChild(newNode);
    }

/*
 * cannot inherit from XATTree's appendChildLeaves method
 * in a dag, a leaf node may be reached more than one time, make sure no duplicates are stored
 * Creation date: (4/13/2003 3:17:40 PM)
 * @param v java.util.Vector
 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
 */
protected void appendChildLeaves(Vector v, DagNode node) {
	DagNode[] children = (DagNode[])node.getChildren();

	int i;
	int length = Array.getLength(children);
	if (length == 0) {
		if (!(v.contains(node)))
			v.add(node);
		return;
	}
	for (i = 0; i < length; i++) {
		appendChildLeaves(v, (DagNode)children[i]);
	}

}
/**
	* Converts this tree into an array, similar to getLeaves()
	* @return an array of XATNodes 
	*/
public edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode[] convertToArray() {

	Vector resultSet = new Vector();
  try {

	//Do breath first traverse
	Set curSet = new HashSet();//contains nodes on current visiting level
	Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from root
	curSet.add(getRoot());
	do {
			//for each node in current level
		for (Iterator it = curSet.iterator(); it.hasNext();) {
		XATNode curNode = (XATNode) it.next();

		//visit this node
		resultSet.add(curNode);

		if (curNode.getChildrenSize() > 0) {
		  XATNode[] children = curNode.getChildren();

		  //push each child in next level
		  for (int i = 0; i < children.length; i++) {
			XATNode curChild = (XATNode) children[i];

						//flag == true means all parents are visited
			boolean flag = true;
			//DagNode[] prevLevel = curChild.getParents();
			//for (int j = 0; j < prevLevel.length; j++) {
			for(Iterator prevIt = curChild.getParents(); prevIt.hasNext();){
				XATNode curPrev = (XATNode)prevIt.next();
				if (!resultSet.contains(curPrev)) {
				flag = false;
				break;
			  }
			}

			if (flag)
			  nextSet.add(curChild);
		  }//push each child
		} //push next level

	  } //for

	  curSet = nextSet;
	  nextSet = new HashSet();
	} while (!curSet.isEmpty());

  } catch (Exception e) {
	e.printStackTrace();
  }

  return (XATNode[]) resultSet.toArray(new XATNode[]{});
}

/**
 * Creation date: (2002-12-20 15:57:08)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
 */
public static XATDag convertTreeToDag(XATTree xattree) {

	XATDag xatdag = new XATDagImp();
	
	//make logical plan by
	//Top-down visit
	xatdag.setRoot(visit(xattree.getRoot()));
  
  return xatdag;
}
/**
 * Clean a dag, make sure all parent and children connection are correct
 * Alg: first, traverse the dag and make all nodes' parent empty
 * then traverse the dag again and adding suitable parent to every dagnode.
 *
 */
public void cleanup(){
	clearAllParent((DagNode)this.getRoot());
	setAllParent((DagNode)this.getRoot());
}

private void clearAllParent(DagNode root){
	if (root ==null) return;
	
	root.deleteAllParents();
	for (int i=0;i<root.getChildrenSize();i++){
		clearAllParent((DagNode)root.getChild(i));		
	}
	//for FOR op
	if (root.getOperator() instanceof FORImp)
		clearAllParent((DagNode)((FOR)root.getOperator()).getXtreFunctions().getRoot());
	//for GB op
	if (root.getOperator() instanceof GroupByImp)
		clearAllParent((DagNode)((GroupBy) root.getOperator()).getXATTree().getRoot());
}
private void setAllParent(DagNode root){
	if (root ==null) return;
	
	for (int i=0;i<root.getChildrenSize();i++){
		DagNode child = (DagNode)root.getChild(i);
		if (child.getParentsSize()<1) setAllParent(child);
		child.addParent(root);		
	}
	//for FOR op
	if (root.getOperator() instanceof FORImp){
		DagNode child =	(DagNode)((FOR)root.getOperator()).getXtreFunctions().getRoot();
		if (child == null) return;
		if (child.getParentsSize()<1) setAllParent(child);
	}
	//for GB op
	if (root.getOperator() instanceof GroupByImp){
		DagNode child =	(DagNode)((GroupBy) root.getOperator()).getXATTree().getRoot();
		if (child == null) return;
		if (child.getParentsSize()<1) setAllParent(child);
	}	
}
/**
 * Traverse the tree and add the node one by one into DAGNode.
 *
 * Creation date: (2002-12-20 16:30:32)
 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnest
 */
private static DagNode visit(XATNode node) {

	DagNode curNode = new DagNodeImp();

	//pre-order Top-down visit
	XATOperator curOp = (XATOperator)node.getOperator().clone();
	curNode.setOperator(curOp);
	curOp.setNode(curNode);
	
	
	XATNode[] children = node.getChildren();
	for(int i = 0; i < children.length; i++){
		// recursively visit the child node.
		DagNode childNode = visit(children[i]);
		// add the child to the parent.
		curNode.addChild(childNode);
		childNode.setParent(curNode);
	}

	return curNode;
}


/**
 * Override the getLeaves() in the XAT tree.
 * Creation date: (4/3/2003 1:22:08 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode[]
 */
public XATNode[] getLeaves() {
	Iterator roots = this.getRoots();
	Vector children = new Vector();
	while(roots.hasNext()){
		this.appendChildLeaves(children,(DagNode)roots.next());
	}
	return (XATNode[])(children.toArray(new DagNode[]{}));
}
/**
 * Insert the method's description here.
 * Creation date: (1/31/03 4:48:42 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 * @param i int
 */
public DagNode getNthRoot(int i) {
	return (DagNode)root.get(i);
}

public void deleteRoot(DagNode node) {
	root.remove(node);
}

/**
 * Returns true if node is a root node; false otherwise.
 * @return boolean
 */
public boolean isRoot(DagNode node){
	return root.contains(node);
}

/**
 * Insert the method's description here.
 * Creation date: (1/31/03 4:49:34 PM)
 * @param i int
 * @param newRoot edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public void setNthRoot(int i, DagNode newRoot) {
	
	root.set(i, newRoot);
	
	}
}
