package edu.wpi.cs.dsrg.xmldb.xat.common.dag;

import java.io.Serializable;

/**
 * The cursor for a queue.  Each customer of a queue has a cursor that points to the
 * next tuple that customer would dequeue 
 * @author: Brad Pielech
 */
public class XATQueueCursor implements Comparable, Serializable{
	protected int cursor;
/**
 * XATQueueCursor constructor comment.
 */
public XATQueueCursor() {
	super();
	cursor = 0;
}
/**
 * Insert the method's description here.
 * Creation date: (5/16/2003 11:17:47 PM)
 * @param cursor int
 */
public XATQueueCursor(int cursor) {
	super();
	this.cursor = cursor;
	
}
	/**
	 * Compares this object with the specified object for order.  Returns a
	 * negative integer, zero, or a positive integer as this object is less
	 * than, equal to, or greater than the specified object.<p>
	 *
	 * The implementor must ensure <tt>sgn(x.compareTo(y)) ==
	 * -sgn(y.compareTo(x))</tt> for all <tt>x</tt> and <tt>y</tt>.  (This
	 * implies that <tt>x.compareTo(y)</tt> must throw an exception iff
	 * <tt>y.compareTo(x)</tt> throws an exception.)<p>
	 *
	 * The implementor must also ensure that the relation is transitive:
	 * <tt>(x.compareTo(y)&gt;0 &amp;&amp; y.compareTo(z)&gt;0)</tt> implies
	 * <tt>x.compareTo(z)&gt;0</tt>.<p>
	 *
	 * Finally, the implementer must ensure that <tt>x.compareTo(y)==0</tt>
	 * implies that <tt>sgn(x.compareTo(z)) == sgn(y.compareTo(z))</tt>, for
	 * all <tt>z</tt>.<p>
	 *
	 * It is strongly recommended, but <i>not</i> strictly required that
	 * <tt>(x.compareTo(y)==0) == (x.equals(y))</tt>.  Generally speaking, any
	 * class that implements the <tt>Comparable</tt> interface and violates
	 * this condition should clearly indicate this fact.  The recommended
	 * language is "Note: this class has a natural ordering that is
	 * inconsistent with equals."
	 * 
	 * @param   o the Object to be compared.
	 * @return  a negative integer, zero, or a positive integer as this object
	 *		is less than, equal to, or greater than the specified object.
	 * 
	 * @throws ClassCastException if the specified object's type prevents it
	 *         from being compared to this Object.
	 */
public int compareTo(java.lang.Object o) {
	try {
		XATQueueCursor x1 = (XATQueueCursor)o;
		int otherCursor = x1.getCursor();
		if(cursor == otherCursor){
			return 0;
		}
		if(cursor < otherCursor){
			return -1;
		}
		if(cursor > otherCursor){
			return 1;
		}
	} catch (ClassCastException e1) {
		throw e1;
	}
	

	
	return 0;
}
/**
 * decrements the cursor by howMuch
 * 
 * @param howMuch int
 * @exception NumberFormatException if howMuch < 0
 */
public void decrementCursor(int howMuch) throws NumberFormatException{
	if(howMuch >= 0){
		cursor = cursor - howMuch;
	} else {
		throw new NumberFormatException("Error trying to increment cursor by a negative number");
	}
	
	}
/**
 * get the cursor's position
 * 
 * @return int
 */
public int getCursor() {
	return cursor;
}
/**
 * increments the cursor by howMuch
 * 
 * @param howMuch int
 * @exception NumberFormatException if howMuch < 0
 */
public void incrementCursor(int howMuch) throws NumberFormatException{
	if(howMuch >= 0){
		cursor = cursor + howMuch;
	} else {
		throw new NumberFormatException("Error trying to increment cursor by a negative number");
	}
	
	}
/**
 * set the cursor to a new value
 * 
 * @param newCursor int
 */
public void setCursor(int newCursor) {
	cursor = newCursor;
}
}
