package edu.wpi.cs.dsrg.xmldb.xat.common.dag;

/**
 * Implements XATQueue.
 * All methods are synchronized for multi-thread accessing.
 * 
 * Creation date: (4/27/2003 1:23:37 AM)
 * @author: Jinhui Jian
 */
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataSourceImp;

public class XATQueueImp extends XATDataSourceImp implements XATQueue {
	public static int CLEANTHRESHOLD = 0;
	private int cachedCursor = 0;

	//the cachedCustomer records the most recent customer.
	//if the same customer access a queue again,
	//the cachedCursor is used avoiding checking the customerMap.
	//This is solely for performance purpose.
	private Object cachedCustomer = null;

	//customer list that maps a cutomer to its cursor
	//Object => XATQueueCursor
	//The map contains all customers except for the cached one
	private Map customerMap = null;
	private boolean isOldBoxInput = false;

	//the minimal cursor of all customers except for the cached one.
	//This variable is solely for performance purpose,
	//it avoids iteration over all customers to find the minimal cursor.
	private int minCursor = Integer.MAX_VALUE;
	private String queueName = new String();
	private int queueNumber = -1;

	/** the owners of this queue */
	private List queueOwner = new ArrayList();

	//the schema object
	private XATQueueSchema schema = null;
	private int totalDequeued = 0;
	private int totalEnqueued = 0;
	
	//the list containing all tuples in a queue
	private List tuples = new ArrayList();
	
	//how many consumer has consumed it
	private int count=0;
	private int customerNum = -1;
	
	
/**
 * XATQueueImp constructor comment.
 */
public XATQueueImp() {
	super();
}
/**
 * XATQueueImp constructor comment.
 */
public XATQueueImp(Object owner) {
	super();
	this.queueOwner.add(owner);
}

public XATQueueImp(Object owner, String queueName) {
	super();
	this.queueName = queueName;
	this.queueOwner.add(owner);
}
/**
 * Adds a new owner for this queue. The newOwner object will now be able to enqueue tuples
 * @param newOwner java.lang.Object
 */
public synchronized void addNewOwner(java.lang.Object newOwner) {
	
	if(!queueOwner.contains(newOwner)){
		queueOwner.add(newOwner);
	}
	
	}
/**
 * Clean up tuples that have been dequeued by all customers.
 *
 * Creation date: (4/27/2003 3:10:56 AM)
 */
public void cleanTuples() {

	//if no more customers, clear the queue
	if(this.cachedCustomer == null){
		this.tuples.clear();
	}

	//otherwise, clean up tuples when necessary.
	//the number "5" is a tradeoff between speed and storage:
	//larger number requires less cleanup,
	//smaller number reducees the storage overhead for old tuples
	if (this.cachedCursor > CLEANTHRESHOLD && this.minCursor > CLEANTHRESHOLD) {

		int min = Math.min(this.minCursor, this.cachedCursor);

		//update the cached customer
		this.cachedCursor -= min;

		//update the rest customers
		if (this.customerMap != null) {
			Set keySet = this.customerMap.keySet();
			for (Iterator it = keySet.iterator(); it.hasNext();) {
				Object curCustomer = it.next();
				XATQueueCursor oldCursor = (XATQueueCursor) this.customerMap.get(curCustomer);
				oldCursor.setCursor(oldCursor.getCursor() - min);
			}

			//also update the minimal cursor
			this.minCursor -= min;
		}

		//clean up tuples
		for (int i = 0; i < min; i++)
			this.tuples.remove(0);

	}

}
/**
 * Compute the minimal cursor in the customer map.
 * 
 * Creation date: (4/27/2003 3:21:38 AM)
 */
private void computeMinCursor() {

	this.minCursor = Integer.MAX_VALUE;

	Collection cursors = this.customerMap.values();
	for(Iterator it = cursors.iterator(); it.hasNext();){
		XATQueueCursor cursor = (XATQueueCursor)it.next();
		if(cursor.getCursor() < this.minCursor)
			this.minCursor = cursor.getCursor();	
	}

}
/**
 * Dequeues a number of tuples.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
 * @param number int
 * @param customer java.lang.Object
 */
public synchronized XATTuple[] dequeue(int number, Object customer) {

	//get cursor
	//int cursor = this.getCursor(customer);

	//check validity
	//if(cursor + number > this.tuples.size())
	//	throw new XATQueueException("Not enough tuples to dequeue!");
		
	//if(number > this.tuples.size())
	//	number = this.tuples.size();
		
	//get tuples
	XATTuple[] result = new XATTuple[number];
	
	for(int i = 0; i < number; i++){
		result[i] = (XATTuple)this.tuples.remove(0);	
	}
	
	
	//for(int i = 0; i < number; i++){
	//	this.tuples.remove(i);	
	//}

	//increase cursor
	//this.increaseCursor(number);
	
	this.totalDequeued += result.length;
	
	return result;
}


protected int getCustomerNum(){
	if (customerNum==-1){
		XATStreamOperator owner = (XATStreamOperator)queueOwner.get(0);
		DagNode ownerNode = (DagNode)owner.getNode();
		customerNum = ownerNode.getParentsSize();
	}
		
	return customerNum;
	
}

public void resetCustomerNum(){
	customerNum = -1;
}

public void cleanAll(){
	tuples.clear();
	count = 0;
}

public List dequeueAll() {

	//get cursor
	//int cursor = this.getCursor(customer);

	//check validity
	//if(cursor + number > this.tuples.size())
	//	throw new XATQueueException("Not enough tuples to dequeue!");
		
	//if(number > this.tuples.size())
	//	number = this.tuples.size();
		
	//get tuples
	
	
	
	//for(int i = 0; i < number; i++){
	//	this.tuples.remove(i);	
	//}

	//increase cursor
	//this.increaseCursor(number);
	count++;
	if (count == getCustomerNum()){
		List result = tuples;
		//all customers have used the content, clean it up
		tuples = new ArrayList();
		count =0;
		return result;
	}else
		return tuples;
	
	
	
	//this.totalDequeued += tuples.size();
}
/**
 * Dequeues a tuple.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple
 * @param customer java.lang.Object
 */
public synchronized XATTuple dequeue(Object customer) {

	if(this.tuples.size()==0)
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//get tuple
	XATTuple result = (XATTuple)this.tuples.remove(0);

	this.totalDequeued += 1;
	
	return result;
}


/**
 * Insert the method's description here.
 * Creation date: (5/19/2003 3:16:02 PM)
 * @return java.util.List
 * @param howMany int
 * @param caller java.lang.Object
 */
public List dequeueN(int number, Object customer) {

	//get cursor
	int cursor = this.getCursor(customer);

	//check validity
	if(cursor + number > this.tuples.size())
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//get tuples
	List result = new ArrayList(number);
	//for(int i = 0; i < number; i++)
	//	result.add(this.tuples.get(cursor + i));
	result.addAll(tuples.subList(cursor, cursor+ number));

	//increase cursor
	this.increaseCursor(number);
	
	this.totalDequeued += result.size();
	return result;
}
/**
 * Enqueues a list of tuples.
 * 
 * Note: the provide is reserved for future usage.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param tuples java.util.List
 * @param provider java.lang.Object
 */
public synchronized void enqueue(java.util.List tuples, Object provider) {
	this.tuples.addAll(tuples);
	
	//this.totalEnqueued += tuples.size();
}

public void enqueue(List tuples){
	this.tuples.addAll(tuples);
}

public void setQueueContent(List newTuples){
	tuples.addAll(newTuples);
}
/**
 * Enqueues a tuple.
 * 
 * Note: the provide is reserved for future usage.
 *
 * @param tuple edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple
 * @param provider java.lang.Object
 */
public synchronized void enqueue(XATTuple tuple, Object provider) {
	this.tuples.add(tuple);
	
	this.totalEnqueued += 1;
}
/**
 * Enqueues an array of tuples.
 * 
 * Note: the provide is reserved for future usage.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param tuples edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
 * @param provideer java.lang.Object
 */
public synchronized void enqueue(XATTuple[] tuples, Object provideer) {
	for(int i = 0; i < tuples.length; i++)
		this.tuples.add(tuples[i]);	
		
	this.totalEnqueued += tuples.length;
}
/**
 * Return the cursor of the customer.
 * There is a side effect: the customer is cached.
 *
 * This method needs to be protected such that any subclass can extend enqueue or dequeue
 * Creation date: (4/27/2003 1:40:07 AM)
 * @return int
 * @param customer java.lang.Object
 */
public int getCursor(Object customer) {

	//if cached, return directly
	if(this.cachedCustomer == customer)
		return this.cachedCursor;

	//if not cached, check the customer map
	if(this.customerMap == null)
		throw new XATQueueException("Customer not registered!");

	XATQueueCursor cursor = (XATQueueCursor)this.customerMap.get(customer);
	if(cursor == null)
		throw new XATQueueException("customer "+ customer + " not registered!");
	int result = cursor.getCursor();
	
	//before return, first save the old cache in customer map
	this.customerMap.put(this.cachedCustomer, new XATQueueCursor(this.cachedCursor));
	this.customerMap.remove(customer);
	
	//also recompute the minCursor because the 
	//old minCursor may be now moved out to be cached
	if(this.cachedCursor < this.minCursor){
		this.minCursor = this.cachedCursor;
	}
	else{
		this.computeMinCursor();
	}
	
	//cache the customer
	this.cachedCustomer = customer;
	this.cachedCursor = result;
	
	return result;
}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject("XATQueueImp",this.getClass().getName(),true);
	}

	/**
	 * @return
	 */
	public String getQueueName() {
		return queueName;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#getQueueNumber()
	 */
	public int getQueueNumber() {
		
		return this.queueNumber;
	}
	/**
	 * @return
	 */
	public List getQueueOwner() {
		return queueOwner;
	}
	/**
	* Returns the number of tuples that are in the queue.
	* @param caller Object 
	* @return long the number of tuples on the queue
	*/
public synchronized int getQueueSize() {
	
	return this.tuples.size();
}
/**
 * Return the number of tuples in the queue for the given customer.
 * 
 * Creation date: (4/27/2003 1:23:37 AM)
 * @return int
 * @param customer java.lang.Object
 */
public synchronized int getQueueSize(Object customer) {
	int cursor = this.getCursor(customer);
	
	return this.tuples.size() - cursor;
}
/**
 * Return the schema of the queue.
 * 
 * Creation date: (4/27/2003 1:23:37 AM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema
 */
public synchronized XATQueueSchema getSchema() {
	return this.schema;
}
/**
 * Increase the cursor of the cached customer.
 * Also clear all tuples that have been dequeued by all customers.
 *
 * Creation date: (4/27/2003 1:53:53 AM)
 * @param number int
 */
public void increaseCursor(int number) {

	this.cachedCursor += number;

	this.cleanTuples();
	//reset the cursor
	if (cachedCursor == this.tuples.size()) {
		cachedCursor = 0;
	}
	
}

public boolean isOldBoxInput() {
	return isOldBoxInput;
}

/**
 * Checks to see if the caller is the owner of the queue
 * @return boolean
 * @param caller java.lang.Object
 */
public boolean isOwner(java.lang.Object caller) {
	
	//This if statement is needed for GUI, see tim if you need it changed.
	if(caller instanceof XATStreamOperator)
		if (queueOwner.get(0) instanceof XATStreamOperator) 
			if(((XATStreamOperator) queueOwner.get(0)).equals((XATStreamOperator)caller))
				return true;

	return queueOwner.contains(caller);
}

public synchronized boolean isRegistered(Object customer){
	
	if(this.customerMap == null)
		return false;
		
	if(this.cachedCustomer == null){
		return false;
	}

	//first check if it has been registered
	if(this.cachedCustomer == customer)
		return true;
		
	if(this.customerMap.containsKey(customer)) 
		return true;
	
	//if it isnt in the cache or isnt in the map, its not registered.
	
	return false;
}
/**
 * Peek at the first tuple in the queue.  This is analagous to dequeue, but
 * this will not increment or cleanup cursors, i.e. the queue is unchanged
 * @param int howMany howMany tuples to peek at
 * @param caller the caller / customer that wishes to peek
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple
 */
public edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[] peek(int number, java.lang.Object customer) {

	//check validity
	if(number > this.tuples.size())
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//get tuples
	XATTuple[] result = new XATTuple[number];
	for(int i = 0; i < number; i++)
		result[i] = (XATTuple)this.tuples.get(i);

		
	return result;
}
/**
 * Retrieve a tuple but also keeps the tuple in the queue.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple
 * @param customer java.lang.Object
 */
public synchronized XATTuple peek(Object customer) {

	if(this.tuples.size() == 0)
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//get tuple
	XATTuple result = (XATTuple)this.tuples.get(0);

	//no need to increase cursor
	
	return result;
}

/**
 * Peek at the last tuple in the queue. This is for XATSubQueueImp.
 * XATSubQueueImp needs to merge a "subtuple" into the last tuple (which must also be a "subtuple") to form a "full tuple".
 * This is analagous to dequeue, but
 * this will not increment or cleanup cursors, i.e. the queue is unchanged
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple the last tuple in the queue
 */
protected edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple peekLast() {
	return (XATTuple)tuples.get(tuples.size()-1);
}
/**
 * Retrieve a number of tuples but also keeps the tuples in the queue.
 * 
 * Creation date: (4/27/2003 1:23:37 AM)
 * @return List
 * @param number int
 * @param customer java.lang.Object
 */
public synchronized List peekN(int number, Object customer) {
	//get cursor
	int cursor = this.getCursor(customer);

	//check validity
	if(cursor + number > this.tuples.size())
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//get tuples
	List list = new ArrayList();
	for(int i = 0; i < number; i++)
		list.add(this.tuples.get(cursor + i));

	//no need to increase cursor
	
	return list;
}
/**
 * Returns the contents of the queue by calling toString() on
 * each tuple.
 * caller is the consumer of the queue you want to print out
 * caller can be null if this is the top output queue
 */
public String printContent(Object caller) {
	StringBuffer rs = new StringBuffer();

	rs.append(tuples.size());
	rs.append("tuples: \n");

	if (caller != null) {
		for (int i = getCursor(caller); i < this.tuples.size(); i++) {
			XATTuple tuple = (XATTuple) tuples.get(i);
			rs.append(tuple.toString());
			rs.append("\n");
		}
	}
	else {
		for (int i = 0; i < tuples.size(); i++) {
			XATTuple tuple = (XATTuple) tuples.get(i);
			rs.append(tuple.toString());
			rs.append("\n");
		}
	}
	return rs.toString();
}
/**
 * Register a customer so that the customer can dequeue tuples from a queue.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param customer java.lang.Object
 */
public synchronized void register(Object customer) {
	if(this.cachedCustomer == null){
		this.cachedCustomer = customer;
		this.cachedCursor = 0;
	}
	else{
		//first check if it has been registered
		if(this.cachedCustomer == customer)
			return;
		if(this.customerMap != null && this.customerMap.containsKey(customer))
			return;

		//add to customer map
		if(this.customerMap == null)
			this.customerMap = new HashMap();
		this.customerMap.put(customer, new XATQueueCursor(0));

		//the new customer must have the minimal cursor
		this.minCursor = 0;
	}
			
}

/**
 * This will forcefully remove all tuples from the queue, even if the queue may still have some customers.  
 * Note, this method obviously can create havoc if it is used indiscriminately, but
 * it is a huge speedup for the XATApplication. 
 * Creation date: (4/13/2003 1:38:36 AM)
 */
public synchronized void removeAllTuples() {

	this.tuples.clear();

	//reset all cursors
	//update the cached customer
	this.cachedCursor = 0;

	//update the rest customers
	if (this.customerMap != null) {
		Set keySet = this.customerMap.keySet();
		for (Iterator it = keySet.iterator(); it.hasNext();) {
			Object curCustomer = it.next();
			XATQueueCursor oldCursor = (XATQueueCursor) this.customerMap.get(curCustomer);
			oldCursor.setCursor(0);
		}

		//also update the minimal cursor
		this.minCursor = Integer.MAX_VALUE;
	}

}
/**
 * Remove the owner from the queue.  This will prevent the owner from enqueuing tuples in the future
 * @param owner java.lang.Object
 */
public void removeOwner(java.lang.Object owner) {
	
	queueOwner.remove(owner);
	
	}
	/**
	* called by the constructor.  this will set the default properties
	* that the ALL query objects will support. Each subclass should override
	* this method and add their own unique properties, but should also call
	* super.setDefaultProperties() so that the common properties may be set.
	*/   
	protected void setDefaultProperties(){
		super.setDefaultProperties();
		// this property should be set in each operator's run() method
		// when certain number of tuples are to be dequeued from this queue.
		addProperty("DEQUEUED_TUPLE_COUNT", "0");
		
	}
	/**
	* called by the constructor.  this will set the default Statistics
	* that the ALL query objects will support. Each subclass should override
	* this method and add their own unique properties, but should also call
	* super.setDefaultStatistics() so that the common statistics may be set.
	*/   
	protected void setDefaultStatistics(){
		/**
		* this statistic will just reflect queue.getQueueSize(), but
		* this will allow the statistics gatherer to store how many tuples
		* are in the queue at certain intervals.  This will allow for the
		* SG to infer how many tuples were dequeued from the queue the
		* previous time
		*/
		super.setDefaultStatistics();
		addStatistic("TUPLE_COUNT");
		
		
	}
	
	public void setOldBoxInput(boolean flag) {
		isOldBoxInput = flag;
	}
/**
* sets the given property to the new value.  This method is responsible
* for validating the given property.  An exception is thrown if
* the value is not correct.
* Validation should be done in the following manner: 
* First the method should check if the object supports the given property by calling
* <tt>isValidPropertyName(String)<tt>.  If it is not a valid property name, then an exception should
* be thrown. <p>
*
* Once it is known that the object supports the property, the value must be validated.  Each implementor of
* XATStatistics has a set of properties that are common to all of its types and some that are specific to types.  
* For example, all XATOperators (which implement XATStatistics) have certain properties while an individual XATOperator
* has specific properties. The indivual types should only validate their own specific types.  If the caller is trying
* to set a common type, it should be validated by the super class by calling <tt>super.setProperty(name, value)<tt>.  This
* will reduce duplicate code. <p>
*
* When it is determined the name / value pair is correct, the pair is updated using <tt>setNewPropertyValue()<tt> <p>
* <tt>XATStatisticsImp<tt> provides default validation by checking to see if the property's value is
* >=0.  propertyValues that are Strings will just be set.  If the property requires more strict constraints than this, 
* it is up to the subclass that created the property to validate it (as described above).
* @param name the name of the property to set
* @param value the new value of the property
* @throws Exception if the given property cannot be found or
* if the value is not valid for the given property.
*/
public void setProperty(String name, String value) throws Exception{
	super.setProperty(name, value);
}

	/**
	 * @param string
	 */
	public void setQueueName(String string) {
		queueName = string;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#setQueueNumber(int)
	 */
	public void setQueueNumber(int num) {
		this.queueNumber = num;

	}
/**
 * Set schema for a queue.
 * 
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param schema edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema
 */
public synchronized void setSchema(XATQueueSchema schema) {
	this.schema = schema;	
}
	
	public int size(){
		return totalEnqueued - totalDequeued;
	}
/**
 * Skips the next tuple.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param customer java.lang.Object
 */
public synchronized void skip(Object customer) {
	//get cursor
	int cursor = this.getCursor(customer);

	//check validity
	if(cursor >= this.tuples.size())
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//increase cursor
	this.increaseCursor(1);
	
}
/**
 * Skip a number of tuples.
 *
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param number int
 * @param customer java.lang.Object
 */
public synchronized void skipN(int number, Object customer) {
	//get cursor
	int cursor = this.getCursor(customer);

	//check validity
	if(cursor + number > this.tuples.size())
		throw new XATQueueException("Not enough tuples to dequeue!");
		
	//increase cursor
	this.increaseCursor(number);
	
}
	
	public String toString(){
		String ret = "Queue: ";
		ret += this.getQueueName() + " ";
		ret += this.getQueueNumber() + " ";
		ret += "Owner: " + this.getQueueOwner().toString(); 
		
		return ret;
	}
/**
 * Unregister a customer.
 * The customer can not dequeue tuple after unregistration.
 * 
 * Creation date: (4/27/2003 1:23:37 AM)
 * @param customer java.lang.Object
 */
public synchronized void unregister(Object customer) {

	//if the cached one is to be moved
	if (this.cachedCustomer == customer) {

		//no other customers, easy
		if (this.customerMap == null) {
			this.cachedCursor = 0;
			this.cachedCustomer = null;

			//also flush queues
			this.tuples.clear();
		}
		//otherwise, move some lucky guy from the customer map to the cache
		else {
			Set keySet = this.customerMap.keySet();
			Iterator it = keySet.iterator();
			this.cachedCustomer = it.next();
			this.cachedCursor =
				((XATQueueCursor) this.customerMap.get(this.cachedCustomer)).getCursor();
			this.customerMap.remove(this.cachedCustomer);

			//if no other customers, delete the map
			if (this.customerMap.isEmpty()) {
				this.customerMap = null;
				this.minCursor = Integer.MAX_VALUE;

				//also flush queues
				for(int i = 0; i < this.cachedCursor; i++)
					this.tuples.remove(i);
				this.cachedCursor = 0;
			}
			//otherwise it may need to recompute the minimal cursor 
			//because the old one may have been removed
			else {
				if (this.cachedCursor == this.minCursor) {
					this.computeMinCursor();
				}
			}

		}
	}
	//otherwise remove some one in the customer map
	else {
		//first check whether customer has registerd
		if (this.customerMap == null)
			throw new XATQueueException("The customer to be deleted does not exist!");

		XATQueueCursor cursor = (XATQueueCursor) this.customerMap.get(customer);
		if (cursor == null)
			throw new XATQueueException("The customer to be deleted does not exist!");

		//remove customer from map
		this.customerMap.remove(customer);

		//if no other customers, delete the map
		if (this.customerMap.isEmpty()) {
			this.customerMap = null;
			this.minCursor = Integer.MAX_VALUE;
		}
		//otherwise it may need to recompute the minimal cursor 
		//because the old one may have been removed
		else {
			if (cursor.getCursor() == this.minCursor) {
				this.computeMinCursor();
			}
		}

	}

	//
	this.cleanTuples();
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#addTuple(java.lang.Object)
 */
public void addTuple(Object newTuple) {
	// TODO Auto-generated method stub
	
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#addTuples(java.util.List)
 */
public void addTuples(List newTuples) {
	// TODO Auto-generated method stub
	
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#cleanTuples()
 */

/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#getContents()
 */
public List getContents() {
	return tuples;
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#getCustCursor(java.lang.Object)
 */
public int getCustCursor(Object customer) {
	// TODO Auto-generated method stub
	return 0;
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#getQueueCustomer()
 */
public List getQueueCustomer() {
	// TODO Auto-generated method stub
	return null;
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#getQueueLayout()
 */
public String getQueueLayout() {
	// TODO Auto-generated method stub
	return null;
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#getTuples()
 */
public List getTuples() {
	// TODO Auto-generated method stub
	return null;
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#increaseCursor(int)
 */

/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#setCursor(java.lang.Object, int)
 */
public void setCursor(Object customer, int newCursor) {
	// TODO Auto-generated method stub
	
}
/* (non-Javadoc)
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue#setTuples(java.util.List)
 */
public void setTuples(List newTuples) {
	// TODO Auto-generated method stub
	
}

}
