package edu.wpi.cs.dsrg.xmldb.xat.common.dag;

/**
 * XATTuple is the basic data unit of a XATQueue.
 *
 * Each tuple contains an order-set of cells,
 * each cell is an XATDataValue.
 * The order is maintained by the queue that contains the tuple.
 * 
 * Creation date: (4/26/2003 5:51:41 PM)
 * @author: Jinhui Jian
 */
import java.util.List;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
 
public interface XATTuple {
/**
 * Add one cell.
 *
 * Creation date: (4/29/2003 2:30:40 AM)
 * @param cell edu.wpi.cs.dsrg.xmldb.raindrop.common.XATData
 */
void addCell(XATDataValue cell);
void combineTimeStamps(XATTuple tuple1, XATTuple tuple2);
/**
 * Returns all cells in the tuple by an array.
 * 
 * Creation date: (4/26/2003 6:22:34 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue[]
 */
XATDataValue[] getArrayData();
/**
 * Returns the cell in the given position.
 * 
 * Creation date: (4/26/2003 6:22:00 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue
 * @param pos int
 */
XATDataValue getCell(int pos);
/**
 * Returns the nubmer of cells in the tuple.
 * 
 * Creation date: (4/26/2003 6:26:37 PM)
 * @return int
 */
int getLength();
/**
 * Returns true if the tuple is empty
 * Creation date: (5/16/2003 11:42:27 PM)
 * @return boolean
 */
boolean isEmpty();
/**
 * Depricated method. Do not use.
 * @return
 */
boolean isNew();
/**
 * Depricated method. Do not use.
 * 
 * @param isNew
 */
void setToNew(boolean isNew);
/**
 * Sets a cell at the given position.
 * The tuple must be large enough to include the given position,
 *
 * Creation date: (5/17/2003 2:00:13 PM)
 * @param pos int
 * @param data edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue
 */
void setCell(int pos, XATDataValue data);
/**
 * Set all cells with the given XATDataValue array.
 * 
 * Creation date: (4/26/2003 6:24:27 PM)
 * @param datas edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
 */
void setValues(XATDataValue[] datas);

void setUniTimeStamp(long unitime);
/**
 * Set all cells with the given XATDataValue array.
 * 
 * Creation date: (4/26/2003 6:24:27 PM)
 * @param datas edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
 */
void setValues(List datas);
/**
 * Find out if this tuple is the last tuple propogating through the system.
 * (this method is for experimentation only)
 * @return
 */
boolean getIfLastTuple();
/**
 * Set that this tuple should be the last tuple processed in the query processor.
 * @param ifLast
 */
void setIfLastTuple(boolean ifLast);


/**
 * punctuation-related attributes
 * @return
 */
boolean isPunctuation();

void setAsPunctuation(boolean isPunct);

int getPunctType(int pos);

int[] getPunctTypeArray();

void setPunctType(int pos, int type);

void setPunctTypeArray(int[] pTypeArray);

/* The methods below this are for the storage manager's purposes only */

/**
 * Gets the punctuation ID assigned to this tuple.
 * 
 * Creation Date: (Jun 7, 2004 1:29:37 PM)
 * @author Natasha Bogdanova
 * @return long
 */
long getPunctID();
/**
 * Gets the max time stamp.
 * 
 * Creation Date: (Jun 7, 2004 1:33:13 PM)
 * @author Natasha Bogdanova
 * @return long
 */
long[] getTimeStamp();
long getNthTimeStamp(int n);
long getMaxTimeStamp();
long getMinTimeStamp();
void setNthTimeStamp(long time, int n);
void setMaxTimeStamp(long max);
void setMinTimeStamp(long min);

/**
 * Sets the punctuation ID.
 * 
 * Creation Date: (Jun 7, 2004 1:35:16 PM)
 * @author Natasha Bogdanova
 * @param pid long
 */
void setPunctID(long pid);
/**
 * Sets the time stamp.
 * 
 * Creation Date: (Jun 7, 2004 1:38:42 PM)
 * @author Natasha Bogdanova
 * @param ts long
 */
void setTimeStamp(long[] ts);
/**
 * Returns the index of this tuple. 
 * Note: the tuple will have a valid index value only after
 * it has retreived from the disk.
 * 
 * Creation Date: (Jun 7, 2004 1:39:53 PM)
 * @author Natasha Bogdanova
 * @return int
 */
int getIndex();
/**
 * Sets the index of this tuple. Used by the storage manager
 * when it retreives tuples from the disk.
 * 
 * Creation Date: (Jun 7, 2004 1:41:06 PM)
 * @author Natasha Bogdanova
 * @param index int
 */
void setIndex(int index);

}
