package edu.wpi.cs.dsrg.xmldb.xat.common.dag;

/**
 * Implements XATTuple.
 * 
 * Creation date: (5/16/2003 11:51:55 PM)
 * @author: JinHui Jian
 */
import java.util.List;
import java.io.Serializable;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValueImp;

public class XATTupleImp implements XATTuple, Serializable {

	/* This was edited to remove the "final" keyword, which
	 * was causing problems for the storage manager.
	 */
	public static int SINGLE_VALUE = 1;
	public static int RANGE_VALUE = 2;
	public static int SET_VALUE = 3;
	public static int ALL_VALUE = 4;
	public static int NULL_VALUE = 5;

	/* It is solely for performance purpose that we use the oneCell/listCell
	   design. The motivation is that a lot of tuples have only one cell,
	   using a java collection to encapsulate them is expensive. In this case,
	   we use the oneCell to store the only cell. If there are more than one cell,
	   we set oneCell to null and use listCell.
	*/
	protected XATDataValue oneCell = null;
	protected XATDataValue[] arrayCell = null;
	protected long[] timeStamp = null;
	protected long maxTimeStamp;
	protected long minTimeStamp;

	protected int[] punctTypeArray;
	protected boolean isPunctuation = false;
	//protected boolean valid = true;

	protected boolean ifLastTuple = false;
	protected boolean isNewTuple = false;

	/* The following attributes are for the storage manager.*/

	/** The ID of the punctuation marking this tuple. */
	private long punctID = 0;

	/**Stores the index of the tuple for the purpose of the storage manager*/
	private int index = -1;

	/* End storage manager attributes. */

	/**
	 * XATTupleImp2 constructor comment.
	 */
	public XATTupleImp() {
		super();
	}
	/**
	 * XATTupleImp2 constructor comment.
	 */
	public XATTupleImp(XATDataValue[] array) {
		
		if (array.length == 1)
			oneCell = array[0];
		else
			arrayCell = array;
	}
	/**
	 * XATTupleImp2 constructor comment.
	 */
	public XATTupleImp(int length) {
		super();
		//construct an array only if length > 1
		if (length > 1)
			this.arrayCell = new XATDataValue[length];
		else if (length == 1)
			this.oneCell = new XATDataValueImp();
	}
	/**
	 * XATTupleImp2 constructor comment.
	 */
	public XATTupleImp(List list) {
		super();

		if (list == null || list.isEmpty())
			return;

		if (list.size() == 1) {
			this.oneCell = (XATDataValue) list.get(0);
		} else {
			this.arrayCell = (XATDataValue[]) list.toArray(new XATDataValue[] {
			});
		}
	}
	/**
	 * Add one cell.
	 *
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @param cell edu.wpi.cs.dsrg.xmldb.raindrop.common.XATData
	 */
	public void addCell(XATDataValue cell) {

		if (this.oneCell == null) {
			//this is the first cell
			if (this.arrayCell == null) {
				this.oneCell = cell;
			}
			//otherwise add to listCell 
			else {
				XATDataValue[] temp =
					new XATDataValue[this.arrayCell.length + 1];
				for (int i = 0; i < this.arrayCell.length; i++)
					temp[i] = this.arrayCell[i];
				temp[this.arrayCell.length] = cell;
				this.arrayCell = temp;
			}
		}
		//otherwise it must be the second cell
		else {
			XATDataValue[] temp = new XATDataValue[2];
			temp[0] = this.oneCell;
			temp[1] = cell;

			this.arrayCell = temp;
			this.oneCell = null;
		}
	}

	public void combineTimeStamps(XATTuple tuple1, XATTuple tuple2) {
		long[] time1 = tuple1.getTimeStamp();
		long[] time2 = tuple2.getTimeStamp();

		this.timeStamp = new long[time1.length + time2.length];
		int i, j;
		for (i = 0; i < time1.length; i++) {
			timeStamp[i] = time1[i];
		}
		for (j = 0; j < time2.length; j++) {
			timeStamp[i + j] = time2[j];
		}
		long m1 = tuple1.getMinTimeStamp();
		long m2 = tuple2.getMinTimeStamp();
		this.minTimeStamp = (m1 < m2) ? m1 : m2;
		m1 = tuple1.getMaxTimeStamp();
		m2 = tuple2.getMaxTimeStamp();
		this.maxTimeStamp = (m1 > m2) ? m1 : m2;

		if (tuple1.isNew() && tuple2.isNew()) {
			this.isNewTuple = true;
		} else {
			this.isNewTuple = false;
		}
	}
	/**
	 * fill the tuple content into the holder from the first cell
	 * Creation date: (5/14/2003 5:59:49 PM)
	 * @param holder edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue[]
	 */
	public void fillDataInListTo(XATDataValue[] holder) {
		int size = getLength();

		for (int i = 0; i < size; i++) {
			holder[i] = this.getCell(i);
		}

	}
	/**
	 * given a holder, fill it from position begin 
	 * the filling content is part of the "data field" specified by list
	 * Creation date: (2/9/2003 11:47:11 AM)
	 * @param list java.util.Vector
	 */
	public void fillDataInListTo(
		XATDataValue[] holder,
		int[] list,
		int begin) {

		int length = list.length;
		for (int i = 0; i < length; i++) {
			holder[begin + i] = getCell(list[i]);
		}

	}
	/**
	 * Returns all cells in the tuple by an array.
	 * 
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue[]
	 */
	public XATDataValue[] getArrayData() {
		//either have no cell or more than one cell
		if (this.oneCell == null) {
			//no cell
			if (this.arrayCell == null) {
				return new XATDataValue[] {
				};
			}
			//more than one cell
			else {
				return this.arrayCell;
			}
		}
		//otherwise there must be only one cell
		else {
			return new XATDataValue[] { this.oneCell };
		}
	}
	/**
	 * Returns the cell in the given position.
	 * 
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue
	 * @param pos int
	 */
	public XATDataValue getCell(int pos) {
		//either no cell or more than one cell
		if (oneCell == null) {
			//no cell
			return arrayCell[pos];
		}
		//must be one cell
		else 
			return oneCell;

		

	}
	/**
	 * Returns the nubmer of cells in the tuple.
	 * 
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @return int
	 */
	public int getLength() {
		//either no cell or more than one cell
		if (this.oneCell == null) {
			//no cell
			if (this.arrayCell == null) {
				return 0;
			}
			//more than one  cell
			else {
				return this.arrayCell.length;
			}
		}
		//must be one cell
		else {
			return 1;
		}

	}
	/**
	 * Insert the method's description here.
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStamp
	 */
	//	public XATTimeStamp getTimeStamp() {
	//		return this.timeStamp;
	//	}
	/**
	 * Insert the method's description here.
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @return boolean
	 */
	public boolean isEmpty() {
		return this.oneCell == null && this.arrayCell == null;
	}

	public boolean isNew() {
		return this.isNewTuple;
	}

	public void setToNew(boolean isNew) {
		this.isNewTuple = isNew;
	}

	/**
	 * Sets a cell at the given position.
	 * The tuple must be large enough to include the given position,
	 *
	 * Creation date: (5/17/2003 1:55:57 PM)
	 * @param pos int
	 * @param data edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue
	 */
	public void setCell(int pos, XATDataValue data) {
		if (this.oneCell != null) {
			if (pos != 0)
				throw new RuntimeException("must initialize a tuple to certain size first!");
			this.oneCell = data;
		} else {
			this.arrayCell[pos] = data;
		}
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @param ts edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStamp
	 */
	//	public void setTimeStamp(XATTimeStamp ts) {
	//		this.timeStamp = ts;
	//	}
	/**
	 * Set all cells with the given XATDataValue array.
	 * 
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @param datas edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
	 */
	public void setValues(XATDataValue[] datas) {
		if (datas == null || datas.length == 0) {
			this.oneCell = null;
			this.arrayCell = null;
		} else {
			if (datas.length == 1) {
				this.oneCell = datas[0];
				this.arrayCell = null;
			} else {
				this.oneCell = null;
				this.arrayCell = datas;
			}
		}

	}

	public void setUniTimeStamp(long unitime) {
		if (oneCell != null) {
			timeStamp = new long[1];
			timeStamp[0] = unitime;
		} else {
			int size = this.arrayCell.length;
			this.timeStamp = new long[size];
			for (int i = 0; i < size; i++) {
				timeStamp[i] = unitime;
			}
		}
		this.minTimeStamp = unitime;
		this.maxTimeStamp = unitime;
	}

	/**
	 * Set all cells with the given XATDataValue array.
	 * 
	 * Creation date: (5/16/2003 11:51:55 PM)
	 * @param datas edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
	 */
	public void setValues(List list) {
		if (list == null || list.isEmpty()) {
			this.oneCell = null;
			this.arrayCell = null;
			return;
		}

		if (list.size() == 1) {
			this.oneCell = (XATDataValue) list.get(0);
			this.arrayCell = null;
		} else {
			this.oneCell = null;
			this.arrayCell = (XATDataValue[]) list.toArray(new XATDataValue[] {
			});
		}

	}
	/**
	 * Insert the method's description here.
	 * Creation date: (5/17/2003 1:54:59 AM)
	 * @return java.lang.String
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < this.getLength(); i++) {
			sb.append(this.getCell(i).toString());
			if (i < this.getLength() - 1)
				sb.append(", ");
		}
		return sb.toString();
	}

	public boolean getIfLastTuple() {
		return ifLastTuple;
	}

	public void setIfLastTuple(boolean ifLast) {
		ifLastTuple = ifLast;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple#isPunctuation()
	 */
	// punctuation-related methods

	public boolean isPunctuation() {
		return isPunctuation;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple#setAsPunctuation(boolean)
	 */
	public void setAsPunctuation(boolean isPunct) {
		isPunctuation = isPunct;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple#getPunctType(int)
	 */
	public int getPunctType(int pos) {
		return punctTypeArray[pos];
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple#getPunctTypeArray()
	 */
	public int[] getPunctTypeArray() {
		return punctTypeArray;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple#setPunctType(int, int)
	 */
	public void setPunctType(int pos, int type) {
		punctTypeArray[pos] = type;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple#setPunctTypeArray(int[])
	 */
	public void setPunctTypeArray(int[] pTypeArray) {
		punctTypeArray = pTypeArray;
	}

	/* The methods below this are for the storage manager's purposes only */
	/**
	 * Gets the punctuation ID assigned to this tuple.
	 * 
	 * Creation Date: (Jun 7, 2004 1:29:37 PM)
	 * @author Natasha Bogdanova
	 * @return long
	 */
	public long getPunctID() {
		return this.punctID;
	}

	/**
	 * Gets the time stamp.
	 * 
	 * Creation Date: (Jun 7, 2004 1:33:13 PM)
	 * @author Natasha Bogdanova
	 * @return long
	 */
	public long[] getTimeStamp() {
		return timeStamp;
		//		long maxTimeStamp = 0;
		//		try {
		//			maxTimeStamp = this.timeStamp.getMaxTimeStamp();
		//		} catch (XATTimeStampException e) {
		//			System.out.println(e.toString());
		//		}
		//		return maxTimeStamp;
	}

	/**
	 * Sets the punctuation ID.
	 * 
	 * Creation Date: (Jun 7, 2004 1:35:16 PM)
	 * @author Natasha Bogdanova
	 * @param pid long
	 */
	public void setPunctID(long pid) {
		this.punctID = pid;
	}

	/**
	 * Sets the time stamp.
	 * 
	 * Creation Date: (Jun 7, 2004 1:38:42 PM)
	 * @author Natasha Bogdanova
	 * @param ts long
	 */
	public void setTimeStamp(long[] ts) {
		//this.timeStamp.setMaxTimeStamp(ts);
		this.timeStamp = ts;
	}

	/**
	 * Returns the index of this tuple. 
	 * Note: the tuple will have a valid index value only after
	 * it has retreived from the disk.
	 * 
	 * Creation Date: (Jun 7, 2004 1:39:53 PM)
	 * @author Natasha Bogdanova
	 * @return int
	 */
	public int getIndex() {
		return this.index;
	}

	/**
	 * Sets the index of this tuple. Used by the storage manager
	 * when it retreives tuples from the disk.
	 * 
	 * Creation Date: (Jun 7, 2004 1:41:06 PM)
	 * @author Natasha Bogdanova
	 * @param index int
	 */
	public void setIndex(int index) {
		this.index = index;
	}
	public long getMaxTimeStamp() {
		return maxTimeStamp;
	}
	public long getMinTimeStamp() {
		return minTimeStamp;
	}
	public void setMaxTimeStamp(long max) {
		minTimeStamp = max;
	}
	public void setMinTimeStamp(long min) {
		minTimeStamp = min;
	}
	public long getNthTimeStamp(int n) {
		if (n < timeStamp.length) {
			return timeStamp[n];
		}
		return -1;
	}
	public void setNthTimeStamp(long time, int n) {
		if (n < timeStamp.length) {
			timeStamp[n] = time;
		}
	}
}
