package edu.wpi.cs.dsrg.xmldb.xat.common.dompattern;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.Vector;
import java.lang.String;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

/**
 * This Class implement the DOMPatternNode interface.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public abstract class DOMPatternNodeImp implements DOMPatternNode {

    /**
     * The parent of this DOMPatternNode.
     * @since 1.0
     */
    protected DOMPatternNode parent;
    /**
     * The children of this DOMPatternNode.
     * @since 1.0
     */
    protected Vector children;
    /**
     * The tag value of this DOMPatternNode.
     * @since 1.0
     */
    protected Object tagValue;
    /**
     * The tag name of this DOMPatternNode.
     * @since 1.0
     */
    protected String tagName;
    /**
     * Indicate if this DOMpatternNode is canceled out or not.
     * @since 1.0
     */
    protected boolean canceledOut;

    public DOMPatternNodeImp() {
        children = new Vector();
        canceledOut = false;
        tagName = null;
        tagValue = null;
    }
    public DOMPatternNodeImp(NavigationStep[] value) {
        children = new Vector();
        tagName = null;
        tagValue = value;
        canceledOut = false;
    }
    public DOMPatternNodeImp(String name) {
        children = new Vector();
        tagName = name;
        tagValue = null;
        canceledOut = false;
    }
    public DOMPatternNodeImp(String name, String value) {
        children = new Vector();
        tagName = name;
        tagValue = value;
        canceledOut = false;
    }
    public DOMPatternNodeImp(String name, NavigationStep[] value) {
        children = new Vector();
        tagName = name;
        tagValue = value;
        canceledOut = false;
    }
    /**
     * Add one child to this node. 
     * @return void
     * @param new_child The new child node adding to.
     * @since 1.0
     */
    public void addChild(DOMPatternNode new_node) {
        children.addElement(new_node);
    }
    /**
     * Add a child to the node at a certain position
     * @return void
     * @param new_child The new child node adding to.
     * @param position The position for this new node.
     * @since 1.0
     */
    public void addChild(DOMPatternNode new_node, int position) {
        children.insertElementAt(new_node, position);
    }
    /**
     * Delete a child from this node.
     * @return int The number of children.
     * @param child The to-be-deleted child.
     * @since 1.0
     */
    public int deleteChild(DOMPatternNode node) {
        int position = children.indexOf(node);
        children.removeElement(node);
        return position;
    }
    /**
     * Get a specific child node of this node by their position.
     * @return DOMPatternNode The child node specified.
     * @param position The position of the child node.
     * @since 1.0
     */
    public DOMPatternNode getChild(int position) {
        return (DOMPatternNode) children.elementAt(position);
    }
    /** 
     * Get all the children nodes of this node.
     * @return DOMPatternNode[] The array containing all children nodes.
     * @since 1.0
     */
    public DOMPatternNode[] getChildren() {
        return (DOMPatternNode[]) children.toArray(new DOMPatternNode[] {
        });
    }
    /** 
     * Get the parent node of this node.
     * @param DOMPatternNode The parent node.
     * @since 1.0
     */
    public DOMPatternNode getParent() {
        return parent;
    }
    /**
     * Get the tag name of this DOMPatternNode.
     * @return String The tag name of this DOMPatternNode.
     * @since 1.0
     */
    public java.lang.String getTagName() {
        return tagName;
    }
    /**
     * Get the tag value of this DOMPatternNode.
     * @return Object The tag value of this DOMPatternNode.
     * @since 1.0
     */

    public Object getTagValue() {
        return tagValue;
    }
    /**
     * Check if the DOMPattern node has been canceled out.
     * @return boolean True: if the DOMPattern node has been canceled out. False: otherwise.
     * @since 1.0
     */
    public boolean isCanceledOut() {
        return canceledOut;
    }
    /**
     * Set the canceledOut property.
     * @return void
     * @param newCanceledOut The boolean indicate if the DOMPattern Node is canceled or not.
     * @since 1.0
     */
    public void setCanceledOut(boolean newCanceledOut) {
        canceledOut = newCanceledOut;
    }
    /**
     * Set a specific child to be a new child node.
     * @return void
     * @param position The position of the child node.
     * @param new_child The new child node.
     * @since 1.0
     */
    public void setChild(int position, DOMPatternNode new_node) {
        children.setElementAt(new_node, position);
    }
    /**
     * Set the parent node of this node.
     * @return void
     * @param new_parent The new parent of this node.
     * @since 1.0
     */
    public void setParent(DOMPatternNode parentNode) {
        parent = parentNode;
    }
    /**
     * Set the tag name of this DOMPatternNode.
     * @return void
     * @param name The name for tag.
     * @since 1.0
     */
    public void setTagName(java.lang.String newTagName) {
        tagName = newTagName;
    }
    /**
     * Set the tag value of DOMPatternNode.
     * @return void
     * @param param The tag value of this DOMPatternNode.
     * @since 1.0
     */
    public void setTagValue(NavigationStep[] newTagValue) {
        tagValue = newTagValue;
    }
    /**
     * Construct a NavigationStep using given value string. Then set the tag value
     * of DOMPatternNode as new value.
     * @return void
     * @param value The string of new value.
     * @since 1.0
     */
    public void setTagValue(String newTagValue) {
        tagValue = newTagValue;
    }
    /**
     * Convert the tagValue of the node as a String.
     * @return String The result string.
     * @since 1.0
     */
    public String toString() {

        String printValue = new String();
        if (tagValue != null) {

            try {
                NavigationStep[] printArray = (NavigationStep[]) tagValue;

                for (int i = 0; i < printArray.length; i++) {
                    printValue = printValue + printArray[i].getStepValue() + " ";

                }
            } catch (ClassCastException e) {
                printValue = (String) tagValue;
            }
        }
        String returnString = new String("");
        String className = new String(this.getClass().toString());
        int lastIndex = className.lastIndexOf(".") + 1;
        className = className.substring(lastIndex);

        returnString = className + "[" + tagName + ":" + printValue + "]";

        return returnString;

    }
}