/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package edu.wpi.cs.dsrg.xmldb.xat.common.expression;


public interface Constants {

    /**
     * These constants represent the
     * different <code>AxisSpecifier</code>
     * in <i>XPath</i>. 
     */
    public final static String[] AXISSPECIFIERS = {"ancestor", 
					    "ancestor-or-self",
					    "attribute",
					    "child",
					    "descendant",
					    "descendant-or-self",
					    "following",
					    "following-sibling",
					    "namespace",
					    "parent",
					    "preceding",
					    "preceding-sibling",
					    "self"};
    public final static int ANCESTOR_AXIS = 0;
    public final static int ANCESTOR_SELF_AXIS = 1;
    public final static int ATTRIBUTE_AXIS = 2;
    public final static int CHILD_AXIS = 3;
    public final static int DESCENDANT_AXIS = 4;
    public final static int DESCENDANT_SELF_AXIS = 5;
    public final static int FOLLOWING_AXIS = 6;
    public final static int FOLLOWING_SIBLING_AXIS = 7;
    public final static int NAMESPACE_AXIS = 8;
    public final static int PARENT_AXIS =  9;
    public final static int PRECEDING_AXIS = 10;
    public final static int PRECEDING_SIBLING_AXIS = 11;
    public final static int SELF_AXIS = 12;

    /**
     * These constants represent the
     * quantifiers used in <i>Xpath</i> 
     * expressions.
     */
    public final static String[] QUANTIFIERS = {"SOME",
					 "EVERY"};

    public final static int SOME = 0;
    public final static int EVERY = 1;

    /**
     * These constants represent the
     * different node types in <i>XPath</i>.
     */
    public final static String[] NODETYPES = {"node",
				       "root",
				       "element",
				       "text",
				       "attribute",
				       "namespace",
				       "comment",
				       "processing-instruction"};

    public final static int NODE_TYPE = 0;
    public final static int ROOT_NODE = 1;
    public final static int ELEMENT_NODE = 2;
    public final static int TEXT_NODE = 3;
    public final static int ATTRIBUTE_NODE = 4;
    public final static int NAMESPACE_NODE = 5;
    public final static int COMMENT_NODE = 6;
    public final static int PROCESSING_NODE = 7;
    
    /**
     * These constants represent the 
     * comparison operators used in 
     * <i>Quilt</i>.
     */
    public final static String[] COMPOPS = {"<",
				     ">",
				     ">=",
				     "<=",
				     "=",
				     "!="};

    /**
     * The entry in this array gives the
     * key of the reverse comparison 
     * operator in the array <code>COMPOPS</code>.
     */
    public final static int[] COMPOPS_REVERSE = {1,
					  0,
					  3,
					  2,
					  4,
					  5};
    
    public final static int LT  = 0;
    public final static int GT  = 1;
    public final static int GEQ = 2;
    public final static int LEQ = 3;
    public final static int EQ  = 4;
    public final static int NEQ = 5;
    
    /**
     * These constants represent the 
     * binary operators on node-lists
     * used in <i>Quilt</i>.
     */
    public final static String[] NODELISTOPS = {"|",
					 "INTERSECT",
					 "AFTER",
					 "BEFORE"};

    public final static int KEEP_UNION = 0;
    public final static int KEEP_INTER = 1;
    public final static int KEEP_AFTER = 2;
    public final static int KEEP_BEFORE = 3;    

    /**
     * These constants refer to the
     * different types of basic 
     * <i>Quilt</i> values used in our
     * evaluation semantics.
     */
    public final static String[] VALUETYPES = {"any",
					"node-list",
					"node",
					"string",
					"num",
					"bool"};

    public final static int TYPE_ANY      = 0;
    public final static int TYPE_NODELIST = 1;
    public final static int TYPE_NODE     = 2;
    public final static int TYPE_STRING   = 3;
    public final static int TYPE_NUM      = 4;
    public final static int TYPE_BOOL     = 5;


    public final static int SAME = 0;
    public final static int BEFORE = -1;
    public final static int AFTER = 1;
    public final static int NOT_COMPARABLE = Integer.MIN_VALUE;
} 








