package edu.wpi.cs.dsrg.xmldb.xat.common.operator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import java.lang.reflect.Array;

/**
 * Implementation for the Interface for XATMultiSourceOperator.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0      
 */

public abstract class XATMultiSourceOperatorImp
    extends XATOperatorImp
    implements XATMultiSourceOperator {

    /**
     * Assigns a node to this opertor.
     * @param init is an XAT node
     * @since   1.0          
     */
    public XATMultiSourceOperatorImp(XATNode init) {
        super(init);
    }
    /**
    * Get a specific source of a certain number.
    * @return XATDataSource of that position.
    * @since   1.0         
    */
    public XATDataSource getDataSourceAt(int position) {
        try {
            return this.getNode().getChild(position).getOperator().getOutputDataSource();
        } catch (XATNodeDoesNotExistException e) {
            e.printStackTrace();
            return null;
        }
    }
    /**
    * Return an array of XATDataSource objects got from its child nodes.
    * You can get those XATDataSource objects from children nodes.
    * @return Vector of XATDataSource.
    * @since   1.0         
    */
    public XATDataSource[] getDataSources() {

        int iNumChildren = Array.getLength(this.getNode().getChildren());
        /** Get all the children nodes of this node. */
        XATNode[] xnodChildren = new XATNode[iNumChildren];
        xnodChildren = this.getNode().getChildren();

        Vector vXtables = new Vector();
        int iNumNodes = iNumChildren;
        for (int i = 0; i < iNumNodes; i++) {
            vXtables.addElement(xnodChildren[i].getOperator().getOutputDataSource());
        }
        return (XATDataSource[]) vXtables.toArray(new XATDataSource[] {
        });
    }
    /**
    * Get a specific source of a certain number.
    * @return XATTable of that position.
    * @since   1.0         
    * @deprecated use XATMultiSourceOperator.getDataSourceAt(int position)
    */
    public XATTable getSourceAt(int position) {
        try {
            return (XATTable) this
                .getNode()
                .getChild(position)
                .getOperator()
                .getOutputDataSource();
        } catch (XATNodeDoesNotExistException e) {
            e.printStackTrace();
            return null;
        }
    }
    /**
    * Return a vector of XATTable objects got from its child nodes.
    * You can get those XATTable objects from children nodes.
    * @return Vector of XATTable.
    * @since   1.0         
    * @deprecated use XATMultiSourceOperator.getDataSources(int position)
    */
    public XATTable[] getSources() {

        int iNumChildren = Array.getLength(this.getNode().getChildren());
        /** Get all the children nodes of this node. */
        XATNode[] xnodChildren = new XATNode[iNumChildren];
        xnodChildren = this.getNode().getChildren();

        Vector vXtables = new Vector();
        int iNumNodes = iNumChildren;
        for (int i = 0; i < iNumNodes; i++) {
            vXtables.addElement(xnodChildren[i].getOperator().getOutputDataSource());
        }
        return (XATTable[]) vXtables.toArray(new XATTable[] {
        });
    }
    /**
    * sets the given property to the new value.  This method is responsible
    * for validating the given property.  An exception is thrown if
    * the value is not correct.
    * Validation should be done in the following manner: 
    * First the method should check if the object supports the given property by calling
    * <tt>isValidPropertyName(String)<tt>.  If it is not a valid property name, then an exception should
    * be thrown. <p>
    *
    * Once it is known that the object supports the property, the value must be validated.  Each implementor of
    * XATStatistics has a set of properties that are common to all of its types and some that are specific to types.  
    * For example, all XATOperators (which implement XATStatistics) have certain properties while an individual XATOperator
    * has specific properties. The indivual types should only validate their own specific types.  If the caller is trying
    * to set a common type, it should be validated by the super class by calling <tt>super.setProperty(name, value)<tt>.  This
    * will reduce duplicate code. <p>
    *
    * When it is determined the name / value pair is correct, the pair is updated using <tt>setNewPropertyValue()<tt> <p>
    * <tt>XATStatisticsImp<tt> provides default validation by checking to see if the property's value is
    * >=0.  propertyValues that are Strings will just be set.  If the property requires more strict constraints than this, 
    * it is up to the subclass that created the property to validate it (as described above).
    * @param name the name of the property to set
    * @param value the new value of the property
    * @throws Exception if the given property cannot be found or    
    * if the value is not valid for the given property.
    * @since   1.0         
    */
    public void setProperty(java.lang.String name, java.lang.String value)
        throws java.lang.Exception {
    }
}
