package edu.wpi.cs.dsrg.xmldb.xat.common.operator.motionoperator;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATSingleQueueStreamOperatorImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValueImp;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.basics.HashState;

/**
 * Motion point query operator. 
 * @author: Rimma Kaftanchikova
 */
public abstract class MotionPointQueryOperatorImp extends XATSingleQueueStreamOperatorImp 
		implements MotionPointQueryOperator {

   private java.lang.String streamName;



	/** join algorithm. */
	public static final int HASH = 0;
	public static final int NLOOP = 1;

	protected HashState synop;
	
	protected long window;
	protected long rightWindow;
		
	
	protected int hashSize; /** The number of hash buckets. */
	protected long bucketSize;
	
	/** number of output tuples. */
	protected long numOutTuples = 0;
	protected long maxNumOutTuples = 1000;
	
	protected long[] timeOutTuples = new long[10];
	protected long[] timeOutPuncts = new long[10];
	protected long[] timeTuplesInState = new long[10];

	protected int countLeftPurge = 0;
	protected int countRightPurge = 0;

	/** thresholds for deciding when to purge */
	protected int countLeftPurgeThreshold = 10;
	protected int countRightPurgeThreshold = 10;
	
	protected int countLeftPropagate = 0;
	protected int countRightPropagate = 0;
	
	protected int countLeftPropagateThreshold = 10;
	protected int countRightPropagateThreshold = 10;
	
	protected boolean eagerWindowPurge;
	
	public MotionPointQueryOperatorImp(XATNode node) 
	{
		super(node);
	}

	public java.lang.String getStreamName() 
	{
		return streamName;
	}

	public int getHashSize() {
		return hashSize;
	}
	public long getBucketSize() {
		return bucketSize;
	}
	public long getNumTuplesInState() {
		//return rightSynop.getNumTuples();
		return (synop.getNumTuples());
	}
	public long getNumOutTuples() {
		return numOutTuples;
	}

	public long getMaxNumOutTuples() {
		return maxNumOutTuples;
	}
	public int getCountLeftPurge() {
		return countLeftPurge;	
	}
	public int getCountRightPurge() {
		return countRightPurge;
	}
	public int getCountLeftPurgeThreshold() {
		return countLeftPurgeThreshold;
	}
	public int getCountRightPurgeThreshold() {
		return countRightPurgeThreshold;
	}
	public int getCountLeftPropagate() {
		return countLeftPropagate;
	}
	public int getCountRightPropagate() {
		return countRightPropagate;
	}
	public int getCountLeftPropagateThreshold() {
		return countLeftPropagateThreshold;
	}
	public int getCountRightPropagateThreshold() {
		return countRightPropagateThreshold;
	}

	public void setRightWindow(long rightWin) {
		rightWindow = rightWin;
	}
	public void setHashSize(int hs) {
		hashSize = hs;
	}
	public void setBucketSize(long bs) {
		bucketSize = bs;
	}
	public void setMaxNumOutTuples(long mnot) {
		maxNumOutTuples = mnot;
	}
	public void setCountLeftPurge(int clp) {
		countLeftPurge = clp;
	}
	public void setCountRightPurge(int crp) {
		countRightPurge = crp;
	}
	public void setCountLeftPurgeThreshold(int i) {
		countLeftPurgeThreshold = i;
	}
	public void setCountRightPurgeThreshold(int i) {
		countRightPurgeThreshold = i;
	}
	public void setCountLeftPropagate(int clp) {
		countLeftPropagate = clp;
	}
	public void setCountRightPropagate(int crp) {
		countRightPropagate = crp;
	}
	public void setCountLeftPropagateThreshold(int leftThreshold) {
		countLeftPropagateThreshold = leftThreshold;
	}
	public void setCountRightPropagateThreshold(int rightThreshold) {
		countRightPropagateThreshold = rightThreshold;
	}
	public void setEagerWindowPurge(boolean eagerWinPurge) {
		eagerWindowPurge = eagerWinPurge;
	}
	public void increNumOutTuples(long delta) {
		numOutTuples += delta;
	}
	public void decreNumOutTuples(long delta) {
		numOutTuples -= delta;
	}
	public void increCountLeftPurge() {
		countLeftPurge ++;
	}
	public void increCountRightPurge() {
		countRightPurge ++;
	}
	public void increCountLeftPropagate() {
		countLeftPropagate ++;
	}
	public void increCountRightPropagate() {
		countRightPropagate ++;
	}
	public boolean ifStop() {
		return (numOutTuples >= maxNumOutTuples);
	}
	public boolean isStreamEmpty() {
		boolean empty = false;	
		if (this.getInputQueue().getQueueSize(this) == 0) 
		{		
			empty = true;							
		}
		return(empty);
	}
	
	public XATTuple joinTuples(XATTuple lTuple, XATTuple rTuple) {
		int lLen = lTuple.getLength();
		int rLen = rTuple.getLength();
		XATTuple xatTuple = new XATTupleImp();
	    
		for (int i=0; i<lLen; i++) {
		 xatTuple.addCell(new XATDataValueImp(lTuple.getCell(i)));
		}
		for (int i = 0; i < rLen; i++) {        
		xatTuple.addCell(new XATDataValueImp(rTuple.getCell(i)));
		}
    
		return(xatTuple);
	}

	public Object hash(Object data) {
		return new Integer(((Integer) data).intValue() % hashSize);
	}
	
	
	public boolean run() 
	{

		/**
		 * first figure out how many tuples to dequeue, the selectivity, and 
		 * result delay  
		 */
		if (this.getInputQueue().getSchema() == null)
		{
		   return false;
		}
		
		
		if(this.getOutputQueue().getSchema() == null)
		{
			init();
		}
		
		int howMany = Integer.parseInt(this.getProperty("NUMBER_OF_TUPLES_TO_DEQUEUE"));

		double selectivity =
			Double.parseDouble(this.getProperty("REAL_SELECTIVITY"));
		double delay = Double.parseDouble(this.getProperty("RESULT_DELAY"));
		
		/** get the tuples */

		XATTuple[] tuples = this.getInputQueue().dequeue(howMany, this);
		
		
		this.hash(tuples[0]);

   
		return true;
	}
	

 	
}
