package edu.wpi.cs.dsrg.xmldb.xat.common.operator.operatortester;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.io.*;
import java.util.Vector;

public class ConfigFileParser {
	String operator_class;
	int num_input;
	String join_state_class;
	int[] join_pos;
	double[] stream_selectivity;
	double[] stream_rate;
	int material_threshold;
	long window_size;
	int max_num_tuple_read;
	long[] time_interval;
	String[] input_data_file;
	
	Document doc;

	public ConfigFileParser(String configFileName) throws ParserConfigurationException, SAXException, IOException {
		// Step 1: create a DocumentBuilderFactory and configure it
		DocumentBuilderFactory dbf =
			DocumentBuilderFactory.newInstance();

		// Step 2: create a DocumentBuilder that satisfies the constraints
		// specified by the DocumentBuilderFactory
		DocumentBuilder db = dbf.newDocumentBuilder();

		// Step 3: parse the input file
		doc = db.parse(new File(configFileName));
		this.getData();
	}	
	
	/**
	 * Reads the dat items from the xml input config file
	 */
	private void getData() {
		Vector v_join_pos = new Vector();
		Vector v_stream_selectivity = new Vector();
		Vector v_stream_rate = new Vector();
		Vector v_input_data_file = new Vector();	
		Vector v_time_interval = new Vector();	
		
		//NodeList variableList = doc.getElementsByTagName("variable");
		NodeList rootNodes = doc.getChildNodes();
		NodeList variableList = rootNodes.item(0).getChildNodes();
		for (int i = 0; i < variableList.getLength(); i++){
			Node next = variableList.item(i);
			if(next.getNodeType() == Node.ELEMENT_NODE){
				NamedNodeMap attrs = next.getAttributes();
				String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
				if (name.equals("operator")) {
					operator_class = ((Node) attrs.getNamedItem("value")).getNodeValue();
				} else if (name.equals("num_input")) {
					num_input = Integer.parseInt(((Node) attrs.getNamedItem("value")).getNodeValue());
				} else if (name.equals("window_size")) {
					window_size = Long.parseLong(((Node) attrs.getNamedItem("value")).getNodeValue());
				} else if (name.equals("join_pos")) {
					v_join_pos.add(new Integer(((Node) attrs.getNamedItem("value")).getNodeValue()));
				} else if (name.equals("state")) {//can be implemented by constructor
					join_state_class = ((Node) attrs.getNamedItem("value")).getNodeValue();
				} else if (name.equals("stream_selectivity")) {
					v_stream_selectivity.add(new Double(((Node) attrs.getNamedItem("value")).getNodeValue()));
				} else if (name.equals("stream_rate")) {
					v_stream_rate.add(new Double(((Node) attrs.getNamedItem("value")).getNodeValue()));
				} else if (name.equals("material_threshold")) {
					material_threshold = Integer.parseInt(((Node) attrs.getNamedItem("value")).getNodeValue());
				} else if (name.equals("max_num_tuple_read")) {
					max_num_tuple_read = Integer.parseInt(((Node) attrs.getNamedItem("value")).getNodeValue());
				} else if (name.equals("time_interval")) {
					v_time_interval.add(new Long(((Node) attrs.getNamedItem("value")).getNodeValue()));
				} else if (name.equals("input_data_file")) {
					v_input_data_file.add(((Node) attrs.getNamedItem("value")).getNodeValue());
				}
			}
		}
		
		join_pos = new int[num_input];
		stream_selectivity = new double[num_input];
		stream_rate = new double[num_input];
		input_data_file = new String[num_input];
		time_interval = new long[num_input];
		for (int i=0; i<num_input; i++) {
			join_pos[i] = ((Integer) v_join_pos.get(i)).intValue();
			stream_selectivity[i] = ((Double) v_stream_selectivity.get(i)).doubleValue();
			stream_rate[i] = ((Double) v_stream_rate.get(i)).doubleValue();
			input_data_file[i] = (String) v_input_data_file.get(i);
			time_interval[i] = ((Long) v_time_interval.get(i)).longValue();
		}
	}

	public String getOperatorClass() {
		return operator_class;
	}
	public int getNumInput() {
		return num_input;
	}
	public String getJoinStateClass() {
		return join_state_class;
	}
	public int[] getJoinPos() {
		return join_pos;
	}
	public double[] getStreamSelectivity() {
		return stream_selectivity;
	}
	public double[] getStreamRate() {
		return stream_rate;
	}
	public int getMaterialThreshold() {
		return material_threshold;
	}
	public long getWindowSize() {
		return window_size;
	}
	public int getMaxNumTupleRead() {
		return max_num_tuple_read;
	}
	public String[] getInputDataFile() {
		return input_data_file;
	}
	public long[] getTimeInterval() {
		return time_interval;
	}
}
