package edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.ColumnNameList;

/**
 * An interface for the For opertor.
 * The FOR is for XML Algebra. Op1[FOR(CB)[Op2[Correlated Operator[A],B]]] -->
 * Op1[Groupby(CB, Op2[])[Operator[Cartesian[A,B]]]] 
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0    
 */
public interface FOR extends SpecialOperator, ColumnNameList {
    /**
    * Get binding Columns
    * @return An array of Column Names (Strings).
    * @since   1.0        
    */
    String[] getColumnNames();
    
   /**
    * Get inner data source.
    * @return inner data source.
    * @since   1.0        
    */
    XATTable getInnerSource();

   /**
    * Get outer data source. It should be same as the getPrimarySource.
    * @return outer data source.
    * @since   1.0        
    */
    XATTable getOuterSource();
 
   /**
    * get the tree for the FOR node.
    * @return the XAT tree.
    * @since   1.0       
    */
    
    XATTree getXtreFunctions();
   /**
    * set the tree for the FOR node.
    * @param xat is the For node tree.
    * @since   1.0       
    */
    void setXtreFunctions(XATTree xat);

}