package edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function;
/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;

/**
 * This function uses two static buffers to determine midpoint values,
 * therefore, if this code is used within recursive calls, it will
 * be necessary to call this function with "reset" as the parameter
 * in order to zero out the static strings.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0    
 */
public class FunLEXICOGRAPHICALORDER extends FunctionImp {

    private static float key_counter = 1;
    private static char[] letterArr =
        {
            'a',
            'b',
            'c',
            'd',
            'e',
            'f',
            'g',
            'h',
            'i',
            'j',
            'k',
            'l',
            'm',
            'n',
            'o',
            'p',
            'q',
            'r',
            's',
            't',
            'u',
            'v',
            'w',
            'x',
            'y',
            'z' };
    private static String lowBuff = "aa";
    private static String highBuff = "zz";

    /**
     * Generates Lexicographical keys.
     * @since   1.0          
     */
    public void execute() {

        String retString = new String("");
        XATTable xatOutput;
        Vector newColItems = new Vector();
        Object[] params = this.getParams();

        try {
            // get the input XAT table.
            try {
                xatOutput = (XATTable) this.getPrimaryDataSource().deepCloneDataSource();
            } catch (XATNodeDoesNotExistException e) {
                xatOutput = new XATTableImp();
            }

            // creat a new XAT table, which only has one cell.
            xatOutput.setName("LEXICOGRAPHICALORDER");
            this.setOutput(xatOutput);

            if (this.getParams().length == 1) {
                //childCount should be "count of a parameter" but instead is really
                // # of tuples
                int childCount = xatOutput.getNumberOfTuples();

                if (this.getParams().toString().toLowerCase().equals("reset")) {
                    lowBuff = "bb";
                    highBuff = "zz";
                    return;
                }

                if (childCount == 0) {
                    xatOutput.appendColumn(this.getOutputColumnName());
                    lowBuff = "a";
                    return;
                }

                double strLength = 1;
                double twensix = 26.0;
                String tempString;

                //first time through, set up appropriate lengthed lowBuff
                while (java.lang.Math.pow(twensix, strLength) < childCount)
                    strLength++;

                if (lowBuff.length() < strLength) {
                    tempString = new String();
                    for (int j = 0; j < strLength; j++) {
                        tempString += 'a';
                    }
                } else {
                    tempString = new String(lowBuff);
                }
                lowBuff = tempString;
                //new lowBuff set

                //get allowable distance between each child
                double distance = java.lang.Math.pow(twensix, strLength) / childCount;
                for (int p = 0; p < childCount; p++) {
                    retString = incrementBuff(lowBuff, distance);
                    newColItems.add(retString);
                }
                //add column to output
                xatOutput.appendColumn(this.getOutputColumnName(), newColItems);
                lowBuff = "a";
            } else {
                //if two arguements, should be something like "aaa", "bbbb"
                // assumes first arguement is "less" than second arguement        
                //two arguments, used for inserts, find the key inbetween the two
                String low = new String((String) params[0]);
                String high = new String((String) params[1]);
                retString = findMedianOrder(low, high);
                newColItems.add(retString);
                xatOutput.appendColumn(this.getOutputColumnName(), newColItems);
                lowBuff = "a";
            }
        } catch (Exception e) {
            System.out.println("thrown in eval");
            e.printStackTrace();
        }
    }

    /**
    * Finds the median order between two key values.
    * @param lowEnd is the low key value
    * @param highEnd is the high key value
    * @return java.lang.String    
    * @since   1.0         
    */
    public String findMedianOrder(String lowEnd, String highEnd) {
        String newBuff = new String();
        int q = Math.max(lowEnd.length(), highEnd.length());
        char[] newBuffArr = new char[q];
        double[] encodeLow = new double[lowEnd.length()];
        double[] encodeHigh = new double[highEnd.length()];
        int lastInd = 0;

        double countLow = 0;
        for (int i = 0; i < lowEnd.length(); i++) {
            int newInd = 0;
            for (int j = 0; j < letterArr.length; j++) {
                if (letterArr[j] == lowEnd.charAt(i)) {
                    newInd = j;
                    lastInd = j;
                    break;
                }
            }
            //numeral form of the letter
            // a in the leftmost (1st) position = 0 * 26^0
            // a in the 2nd position = 0 * 26^-1
            encodeLow[i] = newInd * java.lang.Math.pow(26, countLow--);
        }

        double countHigh = 0;
        for (int i = 0; i < highEnd.length(); i++) {
            int newInd = 0;
            for (int j = 0; j < letterArr.length; j++) {
                if (letterArr[j] == highEnd.charAt(i)) {
                    newInd = j;
                    break;
                }
            }
            //numeral form of the letter
            // a in the leftmost (1st) position = 0 * 26^0
            // a in the 2nd position = 0 * 26^-1
            encodeHigh[i] = newInd * java.lang.Math.pow(26, countHigh--);
        }

        if (encodeHigh.length == encodeLow.length) {
            for (int k = 0; k < encodeLow.length; k++) {
                Double val1 = new Double(java.lang.Math.pow(26, k));
                Double locLow = new Double(encodeLow[k] * val1.intValue());
                Double locHigh = new Double(encodeHigh[k] * val1.intValue());
                int newInd =
                    (int) Math.floor(
                        locLow.intValue() + (locHigh.intValue() - locLow.intValue()) / 2);
                //    System.out.println(newInd + " " +  k );
                newBuffArr[k] = letterArr[newInd];
            }

            //if there is no difference, say between "aa" and "ab"
            // append a middle value to the lowEnd buffer, "aam"

            System.out.println(newBuffArr.toString() + " " + lowEnd.toString());

            newBuff = new String(newBuffArr);
            if (newBuff.toString().equals(lowEnd.toString())) {
                newBuff = new String(lowEnd + 'm');
            }

        } else
            if (encodeHigh.length > encodeLow.length) {
                //if the lengths are different, must use "b" * length difference as low bits
                String temp = new String(lowEnd);
                int cInd = 0;
                for (int k = encodeLow.length; k < encodeHigh.length; k++) {
                    Double val1 = new Double(java.lang.Math.pow(26, k));
                    Double locLow = new Double(0.0);
                    Double locHigh = new Double(encodeHigh[k] * val1.intValue());
                    int newInd =
                        (int) Math.floor(
                            locLow.intValue() + (locHigh.intValue() - locLow.intValue()) / 2);
                    temp += letterArr[newInd];
                }
                newBuff = new String(temp);
            } else {
                //if the lengths are different, must use "y" * length difference as high bits
                String temp = new String(lowEnd);
                int cInd = 0;
                for (int k = encodeHigh.length; k < encodeLow.length; k++) {
                    Double val1 = new Double(java.lang.Math.pow(26, k));
                    Double locLow = new Double(encodeLow[k] * val1.intValue());
                    Double locHigh = new Double(25.0);
                    int newInd =
                        (int) Math.floor(
                            locLow.intValue() + (locHigh.intValue() - locLow.intValue()) / 2);
                    temp += letterArr[newInd];
                }
                newBuff = new String(temp);
            }

        return newBuff;
    }

    /**
     * Incerments the buffer. 
     * @param inBuff java.lang.String
     * @param distance double
     * @return java.lang.String     
     * @since   1.0              
     */
    public String incrementBuff(String inBuff, double distance) {
        String newBuff = new String();
        char[] newBuffCharArr = new char[inBuff.length()];
        double[] encodeCharInt = new double[inBuff.length()];
        boolean addCharToEnd = false;

        if (distance == 0)
            distance += 1;

        //encode the string as an integer array with base 26
        double count = 0;
        for (int i = 0; i < inBuff.length(); i++) {
            int newInd = 0;
            for (int j = 0; j < letterArr.length; j++) {
                if (letterArr[j] == inBuff.charAt(i)) {
                    newInd = j;
                    break;
                }
            }
            //numeral form of the letter
            // a in the leftmost (1th) position = 0 * 26^0
            // a in the 10th position = 0 * 26^-1
            encodeCharInt[i] = newInd * java.lang.Math.pow(26, count--);
        }

        //increment final decrement
        count += 1;
        //find distance in terms of power of 26
        distance *= java.lang.Math.pow(26, count);
        //add it to the final bit
        encodeCharInt[encodeCharInt.length - 1] += distance;

        char[] nextBuffArr = new char[inBuff.length()];

        //check to see if any array value "overflowed"
        for (int j = encodeCharInt.length - 1; j >= 0; j--) {
            //increment count in order to find next highest value
            count += 1;
            double highVal = java.lang.Math.pow(26, count);
            if (encodeCharInt[j] >= highVal) {
                if (j != 0) {
                    double overFlowVal = encodeCharInt[j] - highVal;
                    encodeCharInt[j] = overFlowVal;
                    encodeCharInt[j - 1] += java.lang.Math.pow(26, count);
                } else {
                    encodeCharInt[j] = highVal - 1;
                    addCharToEnd = true;
                }
            }
        }

        //encode new letter array
        int pow = 0;
        for (int k = 0; k < encodeCharInt.length; k++) {
            Double val1 = new Double(java.lang.Math.pow(26, pow++));
            Double loc = new Double(encodeCharInt[k] * val1.intValue());
            nextBuffArr[k] = letterArr[loc.intValue()];
        }

        //if, for some reason, we come across 'zzzz...', append "bb"
        // this will still allow for space for insertion between 'zzzzz...' and 'zzzzz...bb'
        if (addCharToEnd) {
            char[] nextBuffTmp = new char[nextBuffArr.length + 2];
            for (int l = 0; l < nextBuffArr.length; l++)
                nextBuffTmp[l] = 'z';
            nextBuffTmp[nextBuffTmp.length - 2] = 'b';
            nextBuffTmp[nextBuffTmp.length - 1] = 'b';
            nextBuffArr = nextBuffTmp;
        }

        newBuff = new String(nextBuffArr);

        //update the low buffer for the next value
        lowBuff = newBuff;

        return newBuff;
    }
}