package edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;
import java.lang.reflect.Array;

import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;

/**
 * Implementation for the OuterUnion interface.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0   
 */
public class OuterUnionImp
    extends XATMultiSourceOperatorImp
    implements OuterUnion {

    /**
     * Assigns a node to this operator .
     * @param XATNode edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
     * @since   1.0   
     */
    public OuterUnionImp(XATNode xnodInit) {
        super(xnodInit);
    }
    /** 
     * Used to implement a visitor pattern. 
     * @param visitor for the visitor pattern
     * @since   1.0       
     */
    public void accept(Visitor visitor) {

    }
    /**
     * Gets a specific source of certain position.
     * @return XATTable of that position.
     * @since   1.0            
     */
    public XATTable getSourceAt(int position) {
        try {
            return xnodOwner.getChild(position).getOperator().getOutput();
        } catch (XATNodeDoesNotExistException e) {
            e.printStackTrace();
            return null;
        }
    }
    /**
     * Return an arry of XATTable objects got from child nodes.
     * @return an arry of XATTable.
     * @since   1.0                 
     */
    public XATTable[] getSources() {
        int iNumChildren = Array.getLength(xnodOwner.getChildren());
        /** Get all the children nodes of this node. */
        XATNode[] xnodChildren = new XATNode[iNumChildren];
        xnodChildren = xnodOwner.getChildren();

        Vector vXtables = new Vector();
        int iNumNodes = iNumChildren;
        for (int i = 0; i < iNumNodes; i++) {
            vXtables.addElement(xnodChildren[i].getOperator().getOutput());
        }
        return (XATTable[]) vXtables.toArray(new XATTable[] {
        });
    }
    /**
    * Returns a String that represents the value of this object.
    * @return a string representation of the receiver
    * @since   1.0          
    */
    public String toString() {
        return "OuterUnion";
    }
}
