package edu.wpi.cs.dsrg.xmldb.xat.common.operator.state;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import java.io.Serializable;
//import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
//import edu.wpi.cs.dsrg.xmldb.xat.common.relationalstreamexpression.ExpressionStr;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Feb 23, 2004
 * 
 * The operator state class represents internal state for StreamOperators
 * during execution. This state holds XATTuples for query processing. This state
 * must be serializable so if the operator is migrated, it's entire state can be sent.
 * 
 */
public abstract class OperatorStateImp implements OperatorState, Serializable {
	protected boolean debugFlag = false;
	protected boolean isMatched = false; //used by moving-state-migration strategy

	protected XATStreamOperator operator;
	protected int stateID; // used by moving state migration strategy to match states
	protected int totalNumTuples;

	public OperatorStateImp() {
		totalNumTuples = 0;
	}
	
	public OperatorStateImp(OperatorState oldstate, XATStreamOperator newop) {
		this((OperatorStateImp)oldstate, newop);
	}
	
	public OperatorStateImp(OperatorStateImp oldstate, XATStreamOperator newop) {
		//OperatorStateImp newstate = new OperatorStateImp();
		this.stateID = oldstate.stateID;
		this.totalNumTuples = 0;
		this.operator = newop;
	}
	
	public OperatorStateImp(XATStreamOperator op) {
		operator = op;
		totalNumTuples = 0;
	}
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#decreStateSize()
	 */
	public void decreStateSize() {
		totalNumTuples --;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#decreStateSize(int)
	 */
	public void decreStateSize(int decrement) {
		totalNumTuples -= decrement;
	}
	public boolean getDebugFlag() {
		return this.debugFlag;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#getOperator()
	 */
	public XATStreamOperator getOperator() {
		return operator;
	}
	public int getStateID () {
		return stateID;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#getStateSize()
	 */
	public int getStateSize() {
		return totalNumTuples;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#increStateSize()
	 */
	public void increStateSize() {
		totalNumTuples ++;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#increStateSize(int)
	 */
	public void increStateSize(int increment) {
		totalNumTuples += increment;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (5/25/2003 2:47:32 AM)
	 * @return boolean
	 */
	public boolean isMatched() {
		return isMatched;
	}
	
	public void setDebugFlag (boolean flag) {
		this.debugFlag = flag;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/25/2003 11:27:47 AM)
	 * @param newTuple streamgenerator.XATStreamTuple
	 */
	//public abstract int purgeByTuple(XATTuple newTuple, Object customer);
	/**
	 * Insert the method's description here.
	 * Creation date: (4/4/2003 3:09:55 PM)
	 * @return boolean
	 * @param debugFlag boolean
	 */
	//public boolean setDebug(boolean debugFlag) {
	//	this.debugFlag = debugFlag;
	//	return debugFlag;
	//}
	/**
	 * Insert the method's description here.
	 * Creation date: (5/25/2003 4:22:20 AM)
	 * @param matched boolean
	 */
	public void setMatched(boolean matched) {
	    isMatched = matched;
	}
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState#setOperator(edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator)
	 */
	public void setOperator(XATStreamOperator op) {
		operator = op;
	}
	
	public void setStateID (int id) {
		stateID = id;
	}
	
//	public void setStateContent(OperatorState os) {
//		
//	}
	
//	public void probe(XATTuple tuple) {
//		// sub-classes should overwrite this method.
//	}
//	public void probe(XATTuple tuple, ExpressionStr expression) {
//		//	sub-classes should overwrite this method.
//	}
//	public void probe(XATTuple[] tuples) {
//		//	sub-classes should overwrite this method.	
//	}
//	public void probe(XATTuple[] tuples, ExpressionStr expression) {
//		//	sub-classes should overwrite this method.
//	}
}
