package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataSource;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import java.util.Iterator;
/**
 * Insert the type's description here.
 * Creation date: (1/6/2003 3:07:43 PM)
 * @author: Brad Pielech
 */
public abstract class XATMultiQueueStreamOperatorImp extends XATStreamOperatorImp implements XATMultiQueueStreamOperator {
/**
 * XATMultiSourceStreamOperatorImp constructor comment.
 * @param init edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
 */
public XATMultiQueueStreamOperatorImp(XATNode init) {
	super(init);
} 

/**
 * copy constructor
 * @param oldop
 * @param newnode
 */
public XATMultiQueueStreamOperatorImp(XATMultiQueueStreamOperatorImp oldop, XATNode newnode) {
	super(oldop, newnode);
}

public XATMultiQueueStreamOperatorImp(XATMultiQueueStreamOperator oldop, XATNode newnode) {
	super((XATMultiQueueStreamOperatorImp)oldop, newnode);
}

public void cleanTemp(){
	getOutputQueue().removeAllTuples();
}

/**
 * Adds another input queue to this operator.  This will be added to the
 * right of the others
 * @param queue edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 */
public synchronized void addInputQueue(edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue queue) {
	inputQueue.add(queue);
	queue.register(this);
	
	queue.setQueueNumber(inputQueue.indexOf(queue));
		
}
/**
 * Adds another output queue to this operator.  This will be added to the
 * right of the others
 * @param queue edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 */
public synchronized void addOutputQueue(edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue queue) {
	outputQueue.add(queue);	
}
 /**
    * Get a specific source of a certain number.
    * @return XATDataSource of that position.
    * 
    */
public synchronized XATDataSource getDataSourceAt(int position) {
	return getNthInputQueue(position);
}
 /**
  * Return an array of XATDataSource objects that represent this operator's input queues
  */
public synchronized edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataSource[] getDataSources() {
	return (XATDataSource[])inputQueue.toArray();
}
/**
 * get the left most input queue
 * this has the effect of calling super.getInputQueue()
 * Creation date: (1/6/2003 4:27:25 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 */
public synchronized XATQueue getLeftInputQueue() {
	return (XATQueue)inputQueue.get(0);
}
/**
 * get the left most output queue
 * this has the effect of calling super.getOutputQueue()
 * Creation date: (1/10/2003 2:12:56 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 */
public synchronized XATQueue getLeftOutputQueue() {
	return (XATQueue)outputQueue.get(0);
}
/**
 * Get the Nth input queue.  This is useful when there are more than 2 input
 * queues
 * Creation date: (1/6/2003 4:27:25 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 * @throws ArrayIndexOutOfBoundsException if the Nth queue does not exist
 */
public synchronized XATQueue getNthInputQueue(int n) throws java.lang.ArrayIndexOutOfBoundsException {
	return (XATQueue)inputQueue.get(n);	
}
/**
 * Get the Nth output queue.  This is useful when there are more than 2 output
 * queues
 * Creation date: (1/10/2003 2:12:56 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 * @throws ArrayIndexOutOfBoundsException if the Nth queue does not exist
 */
public synchronized XATQueue getNthOutputQueue(int n) throws ArrayIndexOutOfBoundsException {
	return (XATQueue)outputQueue.get(n);
}
/**
 * get the right most input queue.  This gets the right-most
 * input queue.  If an operator has more than 2 inputs, this will get the
 * farthest right input. This has the same effect as calling getNthInput(2); 
 * Creation date: (1/6/2003 4:27:25 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 */
public synchronized XATQueue getRightInputQueue() {
	return (XATQueue)inputQueue.get(inputQueue.size() - 1);
}
/**
 * get the right most output queue.  This gets the right-most
 * output queue.  If an operator has more than 2 outputs, this will get the
 * farthest right output.  This has the same effect as calling getNthOutput(2); 
 * Creation date: (1/10/2003 2:12:56 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 */
public synchronized XATQueue getRightOutputQueue() {
	return (XATQueue)outputQueue.get(outputQueue.size() - 1);
}
 /**
    * Get a specific source of a certain number.
    * @return XATTable of that position.
    * @deprecated use XATMultiSourceOperator.getDataSourceAt(int position)
    */
public synchronized edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable getSourceAt(int position) {
	return null;
}
 /**
    * Return a vector of XATTable objects got from its child nodes.
    * You can get those XATTable objects from children nodes.
    * @return Vector of XATTable.
    * @deprecated use XATMultiSourceOperator.getDataSources()
    */
public synchronized edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable[] getSources() {
	return null;
}
/**
 * Insert the method's description here.
 * Creation date: (4/4/2003 10:29:16 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple
 * @param inputs edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple[]
 */
public synchronized XATTuple projectToDesiredOutput(XATTuple[] inputs) {
	

    XATDataValue[] resultData = new XATDataValue[getNode().getSchema().size()];

    int[][] outputPositions = getOutputPositions();

    int fillPos = 0;
    
    for (int i=0; i<outputPositions.length; i++){
	   
	    ((XATTupleImp)inputs[i]).fillDataInListTo(resultData, outputPositions[i], fillPos);

	    //free the space
	    inputs[i] = null;

	    fillPos = fillPos + outputPositions[i].length;
    }
    
    XATTuple newTuple = new XATTupleImp(resultData);

    return newTuple;
	
	
}
/**
 * Runs the actual operator.
 *
 * Unlike in Rainbow, the Stream operators have the implementation tied to the operator
 * because the StreamOperators represent physical operators instead of Logic.
 * Creation date: (2/3/03 1:49:06 PM)
 */
public abstract boolean run() throws java.lang.Exception;
/**
 * set the left most input queue
 * this has the effect of calling super.setInputQueue()
 * Creation date: (1/6/2003 4:21:19 PM)\
 * @param newQueue the new left-most input queue
 */
public synchronized void setLeftInputQueue(XATQueue newQueue) {
	/*throws XATQueueException {
	try {
		newQueue.register(this);
		if(inputQueue.size() == 0){
			inputQueue.add(newQueue);	
	    } else{
			//inputQueue.set(0,newQueue);
			inputQueue.add(0, newQueue);
	    }
		
	} catch (XATQueueException e) {
		throw e;
	}*/
	if (inputQueue.size() == 0) {
		inputQueue.add(newQueue);
		newQueue.setQueueNumber(inputQueue.indexOf(newQueue));
		newQueue.register(this);
	} else {
		setNthInputQueue(newQueue, 0);
	}
}
/**
 * set the left most output queue
 * The operator must be the owner of the queue, otherwise an exception will be thrown.
 * this has the effect of calling super.setOutputQueue()
 * Creation date: (1/20/03 4:43:17 PM)
 * @param newQueue the new left-most input queue
 */
public synchronized void setLeftOutputQueue(XATQueue newQueue) {
	/*
	throws XATQueueException {
    if (newQueue.isOwner(this)) {
        if (outputQueue.size() == 0) {
            outputQueue.add(newQueue);
        } else {
            outputQueue.set(0, newQueue);
        }
    } else {
        throw new XATQueueException("Cannot set this to be the output queue because the operator does not own the queue");
    }*/
	setNthOuputQueue(newQueue,0);	

}
/**
 * Set the Nth input queue.  This is useful when there are more than 2 input
 * queues
 * This first register's the Operator with the queue
 * Creation date: (1/20/03 4:43:17 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 * @param newQueue the new Nth input queue
 * @param int N the inputQueue to set
 */
public synchronized void setNthInputQueue(XATQueue newQueue, int n) {
	try {
		newQueue.register(this);
		int initialSize = inputQueue.size();
		if(initialSize > n){
			inputQueue.set(n,newQueue);
			newQueue.setQueueNumber(inputQueue.indexOf(newQueue));
		} else {
			//pad all the queues before it
			for (int i=0; i < n - initialSize; i++){
				inputQueue.add(null);
			}
			inputQueue.add(newQueue);
			newQueue.setQueueNumber(inputQueue.indexOf(newQueue));
			//inputQueue.set(n, newQueue);
		}	
					
	} catch (XATQueueException e) {
		throw e;
	}	
}
/**
 * Set the Nth output queue.  This is useful when there are more than 2 output
 * queues
 * The operator must be the owner of the queue, otherwise an exception will be thrown. 
 * Creation date: (1/20/03 4:43:17 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
 * @param newQueue the new Nth output queue
 * @param int N the outputQueue to set
 */
public synchronized void setNthOuputQueue(XATQueue newQueue, int n) {
    if(outputQueue.size() > n){
			outputQueue.set(n,newQueue);
		} else {
			outputQueue.add(newQueue);	
		}
	
}
/**
 * set the right most input queue
 * This first register's the Operator with the queue
 * Creation date: (1/20/03 4:43:17 PM)
 * @param newQueue the new right-most input queue
 */
public synchronized void setRightInputQueue(XATQueue newQueue) {
    if (inputQueue.size() == 0) {
        inputQueue.add(newQueue);
        //this.addInputQueue(newQueue);
		newQueue.setQueueNumber(inputQueue.indexOf(newQueue));
        newQueue.register(this);
    } else
        setNthInputQueue(newQueue, inputQueue.size());
}
/**
 * set the right most output queue
 * this has the effect of calling super.setOutputQueue()
 * The operator must be the owner of the queue, otherwise an exception will be thrown. 
 * Creation date: (1/20/03 4:43:17 PM)
 * @param newQueue the new right-most input queue
 */
public synchronized void setRightOutputQueue(XATQueue newQueue) {
	setNthOuputQueue(newQueue,outputQueue.size());	

}


	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasState()
	 */
	public boolean hasState() {
		return true;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#getStateSize()
	 */
	public abstract int getStateSize();
		
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#init()
	 */
	public void init() {
		Iterator i = this.getInputQueues();
		String schemaName = new String();
		XATQueueSchema s;
		XATQueueSchema outSchema = new XATQueueSchema();

		while (i.hasNext()) {

			s = ((XATQueue) i.next()).getSchema();
			schemaName += "#" + s.getName();
			
			Iterator j = s.getColNames().iterator();
			while(j.hasNext()){
				outSchema.addColumn((String) j.next());
			}
		}

		outSchema.setName(schemaName);
		this.getOutputQueue().setSchema(outSchema);
		
		computeOutputPositions();
	}
}
