package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState;
//import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;

//import java.util.List;
//import edu.wpi.cs.dsrg.xmldb.xat.common.operator.predicate.Predicate;
/**
 * Insert the type's description here.
 * Creation date: (1/6/2003 2:59:17 PM)
 * @author: Brad Pielech
 */
public interface XATStreamOperator
	extends edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator {
	/**
	 * the outputpositions records for each input queue, which columns are going to be output
	 * outputPositions[i] specifies for input queue i, which columns are going to be output
	 * OutputPosisions[i] stores an array, each integer in this array specifies the position of a column that is going to be output
	 * e.g., outputPositions[2] = [3, 5]
	 * it means for input queue 2, its 3 and 5th columns are in the output queue
	 * Creation date: (4/4/2003 5:17:53 PM)
	 * @return int[][]
	 */
	public int[][] getOutputPositions();
	/**
	 * get the output positions for the ith input queue
	 * Creation date: (4/4/2003 5:53:05 PM)
	 * @return int[]
	 * @param i int
	 */
	public int[] getOutputPositions(int i);
	
	public void resetOutputPositions();

	public final static java.lang.String FULL_TUPLE = "FULL_TUPLE";
	public final static java.lang.String SUB_TUPLE = "SUB_TUPLE";
	public void addInputQueue(XATQueue newInputQueue) throws XATQueueException;

	//public List getPredicates();
	//public void addPredicate(Predicate p);
	//public void removePredicate(Predicate p);

	public void clearAllTuples();
	/* 
	 * process all accumulated tuples in input queues that have their maxtimestamp 
	 * smaller than the given maxtimestamp 
	 */
	public boolean cleanAccumulatedTuples(long maxtimestamp);
	/*
	 * process all accumulated tuples in this operators' input queues.
	 */
	public boolean cleanAllAccumulatedTuples();
	
	/*
	 * compare the states of this operaotr and the passed-in operator,
	 * the current operator is served as the old operator in old box, while the passed-in operator 
	 * is served as the new operator in new box.
	 * 
	 * when a pair of matching states is found, set the matched flag of matching states, copy the contents of the 
	 * old matching state to the matching new state by calling the copyStateContent() method in the corresponding state class.
	 * return:
	 * TRUE -- is this operator and the given operator have matched states.
	 * FALSE -- otherwise.
	 */
	public boolean matchStates(XATStreamOperator op);
	/*
	 * check if this operator has any unmatched states.
	 * return: TRUE if this operator has unmatched states, FALSE otherwise.
	 */
	public boolean hasUnmatchedState();
	/*
	 * recompute the unmatched states inside this operator.
	 * Usually the unmatched states are computed based on the matched states inside one of the 
	 * children operator of this operator.
	 * 
	 * return: 
	 * TRUE -- if the states (inside child operator) that the recomputation of the unmatched states are based on are 
	 *      all matched, and the recomputation is successful.
	 * FALSE -- if any of the states (inside child operator) that are used to recompute the unmatched states
	 *          is unmatched, or the recomputation is unsucessful.  
	 */
	public boolean recomputeUnmatchedState();
	
	/*
	 * Assign new ID to each state inside this operator, starting from the given startID.
	 * 
	 * return the next available ID. For example, if startID is 2, and there are 2 states inside this operator,
	 * then these two states are assigned new IDs as 2 and 3, and 4 is then returned as the next available stateID.
	 */
	public int setStateID(int startID);
	boolean equals(XATStreamOperator op);
	
	/**
	 * The input queue of an operator is a pointer to the output queue
	 * of the <emph>left</emph> child.  When an operator only has 1 child, this
	 * points to the queue of its only child, but it is important to note this has the
	 * implied semantic that it points to the left one.
	 * @return XATQueue the queue that serves as input to this operator
	 */
	public edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue getInputQueue();

	public XATQueue getInputQueue(int queueNum);

	/**
	 * Returns the input queue with the given queue name;
	 * @param queueName
	 * @return
	 */
	public XATQueue getInputQueue(String queueName);

	/**
	 * Returns the number of inputqueues this stream operator has
	 * @return int
	 */
	public int getInputQueueCount();
	/**
	 * Returns an iterator of all of the output queues of this operator
	 * This is mostly a convience method where, if you dont know how many
	 * output queues an operator has, but you want access to all of them. 
	 * @return java.util.Iterator
	 */
	public java.util.Iterator getInputQueues();

	/**
	 * Returns the Nth state of the Operator
	 * @param n
	 */
	public OperatorState getNthState(int n);
	public int getObjectID();

	public int getOperatorID();
	/**
	 * Each operator has only 1 queue in which it places its ouput.
	 * @return XATQueue the queue that stores the output of this operator
	 */
	public edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue getOutputQueue();

	/**
	 * Returns the number of outputqueues this stream operator has
	 * @return int
	 */
	public int getOutputQueueCount();
	/**
	 * Returns an iterator of all of the output queues of this operator
	 * This is mostly a convience method where, if you dont know how many
	 * output queues an operator has, but you want access to all of them. 
	 * @return java.util.Iterator
	 */
	public java.util.Iterator getOutputQueues();
	/**
	 * Gets an array of state objects from the operator to set across the network.
	 * This is like a Serialization of states that will be deserialized by setState()
	 * 
	 * @return
	 */
	public OperatorState[] getState();

	public abstract int getStateSize();

	/**
	 * Returns true if the operator has a state, false otherwise
	 * @return
	 */
	public boolean hasState();
	/**
	 * Initializes the operator. This class is abstract to force each implementation to 
	 * initialize their operator. It is assumed that when the init() method is called, 
	 * the input queues are already created and the schema is set. This is a good method
	 * to set the schema for the output queue of the operator or instantiate any state
	 * memory.
	 * 
	 */
	public abstract void init();
	/**
	 * Returns whether the XATStreamOperator is active in
	 * the query plan. When a query plan is distributed, it is
	 * possible that only a partial number of operators in the
	 * query plan are active. 
	 * 
	 */
	public boolean isActive();

	public boolean isBoxOperator();

	public boolean isOldBoxLeaf();
	/**
	 * Returns true if the operator is currently running.
	 * 
	 */
	public boolean isRunning();

	//public XATStreamOperatorImp makeNewCopy(XATNode newnode);

	public void replaceInputQueue(XATQueue existingQueue, XATQueue newQueue);

	/**
	 * Runs the actual operator.
	 *
	 * Unlike in Rainbow, the Stream operators have the implementation tied to the operator
	 * because the StreamOperators represent physical operators instead of Logic.
	 * Creation date: (1/6/2003 3:39:36 PM)
	 */
	public abstract boolean run() throws Exception;
	/**
	 * Sets whether or not the operator is active
	 * 
	 * @param active boolean
	 * 
	 */
	public void setActive(boolean active);

	/**
	 * Insert the method's description here.
	 * Creation date: (6/26/2003 4:58:44 AM)
	 */
	public void setAsNewBoxRoot();

	/**
	 * Insert the method's description here.
	 * Creation date: (4/6/2003 7:46:53 AM)
	 */
	public void setAsOldBoxLeaf();

	/**
	 * Insert the method's description here.
	 * Creation date: (4/6/2003 7:46:04 AM)
	 */
	public void setAsOldBoxRoot();

	/**
	 * Insert the method's description here.
	 * Creation date: (4/6/2003 8:29:45 AM)
	 */
	public void setAsOldOperator();

	public void setBoxOperator(boolean isboxop);

	/** 
	 * set the debug flag
	 * @param debug
	 */
	public void setDebug(boolean debug);

	/**
	 * Sets the input queue of this operator. When the input queue is set,
	 * the operator is first registered with the queue. If the operator may not register with the queue,
	 * then the XATQueueException is thrown
	 * @param newInputQueue edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
	 * @throws XATQueueException if the operator cannot register with the queue 
	 *	(it is the queue owner or has already registered)
	 */
	public void setInputQueue(
		edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue newInputQueue)
		throws XATQueueException;

	/**
	 * Sets the nth state of the operator to the given state.
	 * @param state
	 * @param n
	 */
	public void setNthState(OperatorState state, int n);

	/**
	 * This method sets the property "NUMBER_OF_POSSIBLE_OUTPUT_TUPLES", which is the 
	 * possible output tuples from current operator from each run.
	 * For a single stream operator, this is simply the number of dequeued tuple.
	 * For a multiple input stream operator, such as join, special method need to be applied 
	 * in order to compute this property.
	 * This property will be used by stats gathere to compute the reduction factor of this operator.
	 */
	public void setNumberOfPossibleOutputs();

	public void setOperatorID(int id);
	/**
	 * sets the output queue of this operator.  The operator must be the
	 * owner of the queue, otherwise an exception will be thrown.
	 * @param newOutputQueue edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
	 * @throws XATQueueException if this operator is not the owner of the queue
	 */
	public void setOutputQueue(
		edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue newOutputQueue)
		throws XATQueueException;

	/**
	 * Sets whether or not the operator isrunning
	 * 
	 * @param active boolean
	 * 
	 */
	public void setRunning(boolean running);

	/**
	 * The sets the state of the operator based on the
	 * array of operator state objects sent. This is the deserialize part
	 * of the getState method, which happens after the states are sent across the network.
	 * 
	 * @param state
	 */
	public void setState(OperatorState[] state);
	public String toShortString();

	public abstract String toToolTipString();
	/**
	 * set the outputpositions for the i th output
	 * Creation date: (4/4/2003 5:17:32 PM)
	 * @param i int
	 * @param ithOutputPositions int[]
	 */
	public void setOutputPositions(int i, int[] ithOutputPositions);

	/**
	 * set the whole output positions (for every output)
	 * Creation date: (4/4/2003 5:17:53 PM)
	 * @param newOutputPositions int[][]
	 */
	public void setOutputPositions(int[][] newOutputPositions);
	
	

}