package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.lang.reflect.Constructor;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperatorImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataSource;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
/**
 * Insert the type's description here.
 * Creation date: (1/6/2003 g3:02:21 PM)
 * @author: Brad Pielech
 */
public abstract class XATStreamOperatorImp
	extends XATOperatorImp
	implements XATStreamOperator {

	protected boolean active;
	protected boolean debugFlag = false;
	protected ArrayList inputQueue;
	int operatorID = 0; // useful during query plan optimization
	int[][] outputPositions;
	protected ArrayList outputQueue;
	protected boolean running;
	protected OperatorState[] states = null;
	protected boolean isBoxOperator = false;

	//set to true and checked during processing only if isOldOperator == true
	protected boolean isNewBoxRoot = false;

	//set to true and checked during processing only if isOldOperator == true
	protected boolean isOldBoxLeaf = false;

	/* The migrationController should connect the new and old plan together, set migration flags, islastoperator,
	 * and get the current system time and set each old operator's migrationStartTime. 
	 * for new operators and operators that are not involved in migration, the migrationStartTime is useless, and 
	 * by default set to be -1.
	 *
	 */
	//private int migrationStartTime = -1;

	/* If isLastOperator is true, 
	 * that means the migration has started and haven't ended, so special processing need to be done at the last operator
	 * of the old query plan in order to prevent duplicate results.
	 *
	 * When isLastOperator is true, the isOldOperator must be true, and the migrationStartTime must be a positive integer.
	 */
	protected boolean isOldBoxRoot = false;

	/* isOldOperator = true means that migration has started and the operator belongs to "old" query plan. 
	 * the value is false by default.
	 * If isOldOperator = true, this operator should be removed when both states and both inputs does not contain
	 * tuples with isNew set to be true.
	 * when this happens, the operator will stop processing any new tuple and disconnect itself from the
	 * upper node.
	 *
	 * If isOldOperator == false, that means this operator either belongs to the new query plan, or does not involve
	 * in migration at all. In that case, the operator should be acting as usual.
	 */
	protected boolean isOldOperator = false;

	/**
	 * XATStreamOperatorImp constructor comment.
	 * Creates the StreamOperator with 1 input queue.  The output queue
	 * is set later
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 */
	public XATStreamOperatorImp(XATNode node) {
		super(node);
		inputQueue = new ArrayList();
		outputQueue = new ArrayList();
		setOutputQueue(new XATQueueImp(this));
	/*	if (System.getProperty("DATA_MODEL").endsWith("XATMemoryQueueImp")) {
			setOutputQueue(new XATQueueImp(this));
		} else if (
			System.getProperty("DATA_MODEL").endsWith(
				"XATPersistentQueueImp")) {
			setOutputQueue(new XATPersistentQueueImp(this));
		} else {
			System.out.println("!!! Error: Unknow type of DATA_MODEL !!!");
			System.exit(-1);
		}*/
		active = false;
		running = false;
		states = null;
	}

	/**
	 * copy constructure
	 * copy everything except queues and tuples
	 * used by the cloning stage at the biginning of query optimization.
	 * @param oldop
	 * @param newnode
	 */
	public XATStreamOperatorImp(XATStreamOperatorImp oldop, XATNode newnode) {
		this(newnode);

		//XATStreamOperatorImp newop = new XATStreamOperatorImp(newnode);
		this.active = oldop.active;
		this.columnsFiltered = oldop.columnsFiltered;
		this.debugFlag = oldop.debugFlag;
		this.operatorID = oldop.operatorID;
		this.outputColumnName = oldop.outputColumnName;
		this.outputPositions = oldop.outputPositions;
		this.statistics = (java.util.Properties) oldop.statistics.clone();

		//this.states = new OperatorState[oldop.states.length];		
		Constructor construct;
		Class[] c = new Class[2];
		c[1] = this.getClass();
		try {
			for (int i = 0; i < states.length; i++) {
				c[0] = oldop.states[i].getClass();
				construct = oldop.states[i].getClass().getConstructor(c);
				this.states[i] =
					(OperatorState) construct.newInstance(
						new Object[] { oldop.states[i], this });
				//this.state[i] = oldop.state[i].makeNewCopy();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		this.stats = oldop.stats;
		this.statsPresent = oldop.statsPresent;
	}

	public XATStreamOperatorImp(XATStreamOperator oldop, XATNode newnode) {
		this((XATStreamOperatorImp) oldop, newnode);
	}

	public synchronized void addInputQueue(XATQueue newInputQueue)
		throws XATQueueException {
		try {
			newInputQueue.register(this);
			inputQueue.add(newInputQueue);
			newInputQueue.setQueueNumber(inputQueue.indexOf(newInputQueue));

		} catch (XATQueueException e) {
			throw e;
		}
	}

	/**
	 * Clear tuples in input and output queues.
	 */
	public synchronized void clearAllTuples() {

		for (int i = 0; i < inputQueue.size(); i++) {
			XATQueue queue = (XATQueue) inputQueue.get(i);
			queue.removeAllTuples();
		}

		for (int i = 0; i < outputQueue.size(); i++) {
			XATQueue queue = (XATQueue) outputQueue.get(i);
			queue.removeAllTuples();
		}
	}

	public void computeOutputPositions() {
		if (outputPositions == null) {
			DagNode node = (DagNode) getNode();

			//Set schema = getNode().getSchema();
			Vector outColNames = getOutputQueue().getSchema().getColNames();

			int outputColPos = 0;
			Iterator inputs = getInputQueues();

			//XATNode[] children = node.getChildren();

			//store all the input columns
			int j = 0;
			while (inputs.hasNext()) {
				//for (int j = 0; j < children.length; j++) {

				Vector outputPosVec = new Vector();

				//Vector childColNames = ((XATStreamOperator)children[j].getOperator()).getOutputQueue().getSchema().getColNames();
				Vector inputColNames =
					((XATQueue) inputs.next()).getSchema().getColNames();
				for (Iterator i1 = outColNames.iterator(); i1.hasNext();) {

					Object outputCol = i1.next();
					for (int k = 0; k < inputColNames.size(); k++) {
						Object inputCol = inputColNames.get(k);
						if (outputCol.equals(inputCol)) {
							outputPosVec.add(new Integer(k));
							break;
						}
					}
				}
				//convert vector to int[]
				int[] outputPosArray = new int[outputPosVec.size()];
				for (int count = 0; count < outputPosVec.size(); count++) {
					outputPosArray[count] =
						((Integer) outputPosVec.get(count)).intValue();
				}
				((XATStreamOperatorImp) node.getOperator()).setOutputPositions(
					j,
					outputPosArray);
				j++;
			} //end of while
		} //end of if
		//return outputPositions;
	}

	public boolean equals(XATStreamOperator op) {
		if (this
			.getProperty("OBJECT_ID")
			.equals(op.getProperty("OBJECT_ID"))) {
			return true;
		} else
			return false;
	}

	/**
	 * The input queue of an operator is a pointer to the output queue
	 * of the <emph>left</emph> child.  When an operator only has 1 child, this
	 * points to the queue of its only child, but it is important to note this has the
	 * implied semantic that it points to the left one.
	 * @return XATQueue the queue that serves as input to this operator
	 */
	public synchronized edu
		.wpi
		.cs
		.dsrg
		.xmldb
		.xat
		.common
		.dag
		.XATQueue getInputQueue() {
		return (XATQueue) inputQueue.get(0);
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#getInputQueue(int)
	 */
	public XATQueue getInputQueue(int queueNum) {
		for (Iterator i = this.getInputQueues(); i.hasNext();) {

			XATQueue next = (XATQueue) i.next();

			if (next.getQueueNumber() == queueNum)
				return next;

		}

		return null;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#getInputQueue(java.lang.String)
	 */
	public XATQueue getInputQueue(String queueName) {
		for (Iterator i = this.getInputQueues(); i.hasNext();) {

			XATQueue next = (XATQueue) i.next();

			if (next.getQueueName().equals(queueName))
				return next;

		}

		return null;
	}
	/**
	 * Returns the number of inputqueues this stream operator has
	 * @return int
	 */
	public synchronized int getInputQueueCount() {
		return inputQueue.size();
	}
	/**
	 * Returns an iterator of all of the input queues of this operator
	 * This is mostly a convience method where, if you dont know how many
	 * input queues an operator has, but you want access to all of them. 
	 * @return java.util.Iterator
	 */
	public synchronized java.util.Iterator getInputQueues() {
		return inputQueue.iterator();
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject(
			"Op " + this.getProperty("OBJECT_ID"),
			Integer.parseInt(this.getProperty("OBJECT_ID")),
			this.getClass().getName(),
			this.isActive());
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#getNthState(int)
	 */
	public OperatorState getNthState(int n) {
		return states[n];
	}

	public int getObjectID() {
		return Integer.parseInt(this.getProperty("OBJECT_ID"));
	}

	public int getOperatorID() {
		return operatorID;
	}
	/**
	* reimplemented the XATOperator's getOutputDataSource.
	* Note that this casts the Queue to an XATDataSource which can be
	* downcast to a queue 
	*/
	public synchronized XATDataSource getOutputDataSource() {
		return (XATQueue) outputQueue.get(0);
	}
	/**
	    * Get all output data sources for this XATOperator
	    * @return Iterator of data sources.      
	    */
	public synchronized java.util.Iterator getOutputDataSources() {
		return getOutputQueues();
	}
	
	public void resetOutputPositions(){
		outputPositions = null;
	}
	
	/**
	 * the outputpositions records for each input queue, which columns are going to be output
	 * outputPositions[i] specifies for input queue i, which columns are going to be output
	 * OutputPosisions[i] stores an array, each integer in this array specifies the position of a column that is going to be output
	 * e.g., outputPositions[2] = [3, 5]
	 * it means for input queue 2, its 3 and 5th columns are in the output queue
	 * Creation date: (4/4/2003 5:17:53 PM)
	 * @return int[][]
	 */
	public int[][] getOutputPositions() {

		if (outputPositions == null) {
			DagNode node = (DagNode) getNode();

			Vector schema = getOutputQueue().getSchema().getColNames();

			Vector schemaBackup = (Vector) schema.clone();

			int outputColPos = 0;
            XATNode[] children = getNode().getChildren();
            int childrenNum = children.length;
			
			for (int j = 0; j < childrenNum; j++) {

				List outputPosList = new ArrayList();
				
				Vector childColNames = ((XATQueueImp)((XATStreamOperator)children[j].getOperator()).getOutputQueue()).getSchema().getColNames();

				for (Iterator i1 = schemaBackup.iterator(); i1.hasNext();) {

					Object outputCol = i1.next();
					
					int size = childColNames.size();

					for (int k = 0; k < size; k++) {
						Object inputCol = childColNames.get(k);
						if (outputCol.equals(inputCol)) {
							outputPosList.add(new Integer(k));
							//an output schema of a join may occur in both side, if the schema is found in one input schema
							//do not find it in the other
							i1.remove();
							break;
						}
					}
				}

				//convert vector to int[]
				int[] outputPosArray = new int[outputPosList.size()];

				int size = outputPosList.size();
				for (int count = 0; count < size; count++) {
					outputPosArray[count] =
						((Integer) outputPosList.get(count)).intValue();
				}

				((XATStreamOperatorImp) node.getOperator()).setOutputPositions(
					j,
					outputPosArray);

			} //end of for j
		} //end of if
		
		return outputPositions;
	}
	/**
	 * get the output positions for the ith input queue
	 * Creation date: (4/4/2003 5:53:05 PM)
	 * @return int[]
	 * @param i int
	 */
	public synchronized int[] getOutputPositions(int i) {
		return getOutputPositions()[i];
	}
	/**
	 * Each operator has only 1 queue in which it places its ouput.
	 * @return XATQueue the queue that stores the output of this operator
	 */
	public synchronized XATQueue getOutputQueue() {
		return (XATQueue) outputQueue.get(0);
	}
	/**
	 * Returns the number of outputqueues this stream operator has
	 * @return int
	 */
	public synchronized int getOutputQueueCount() {
		return outputQueue.size();
	}
	/**
	 * Returns an iterator of all of the output queues of this operator
	 * This is mostly a convience method where, if you dont know how many
	 * output queues an operator has, but you want access to all of them. 
	 * @return java.util.Iterator
	 */
	public synchronized java.util.Iterator getOutputQueues() {
		return outputQueue.iterator();
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#getState()
	 */
	public OperatorState[] getState() {
		return states;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasState()
	 */
	public boolean hasState() {
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#init()
	 */

	public abstract void init();
	/**
	 * Returns whether the XATStreamOperator is active in
	 * the query plan. When a query plan is distributed, it is
	 * possible that only a partial number of operators in the
	 * query plan are active. 
	 * 
	 */
	public boolean isActive() {

		return this.active;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#isRunning()
	 */
	public boolean isRunning() {
		return running;
	}

	public boolean isBoxOperator() {
		return isBoxOperator;
	}

	public boolean isOldBoxLeaf() {
		return isOldBoxLeaf;
	}

	//	public abstract XATStreamOperatorImp makeNewCopy();

	public void replaceInputQueue(XATQueue existingQueue, XATQueue newQueue) {
		newQueue.register(this);
		existingQueue.unregister(this);
		int i = inputQueue.indexOf(existingQueue);
		inputQueue.set(i, newQueue);
	}

	/**
	 * Runs the actual operator.
	 *
	 * Unlike in Rainbow, the Stream operators have the implementation tied to the operator
	 * because the StreamOperators represent physical operators instead of Logic.
	 * Creation date: (1/22/03 4:18:24 PM)
	 */
	public abstract boolean run() throws java.lang.Exception;
	/**
	 * Sets whether or not the operator is active
	 * 
	 * @param active boolean
	 * 
	 */
	public void setActive(boolean active) {

		this.active = active;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (6/26/2003 4:58:44 AM)
	 */
	public void setAsNewBoxRoot() {
		isNewBoxRoot = true;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (4/6/2003 7:46:53 AM)
	 */
	public void setAsOldBoxLeaf() {
		isOldBoxLeaf = true;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (4/6/2003 7:46:04 AM)
	 */
	public void setAsOldBoxRoot() {
		isOldBoxRoot = true;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (4/6/2003 8:29:45 AM)
	 */
	public void setAsOldOperator() {
		isOldOperator = true;
	}

	/**
	 * set current operator to be an operator belong to a migration box.
	 * this flag is set to be true by the run-time optimizer if this operator belongs to a box.
	 * and the flag is reset to be false after the migration is done.
	 * 
	 * @param isboxop
	 */
	public void setBoxOperator(boolean isboxop) {
		this.isBoxOperator = isboxop;
	}
	/**
	 * set the debug flag of this operator.
	 */
	public void setDebug(boolean debug) {
		debugFlag = debug;
	}
	/**
	 * This will set the default properties that all
	 * XATStreamOperators will have.
	 * NUMBER_OF_TUPLES_TO_DEQUEUE is how many tuples an operator should dequeue
	 * The granularities are relate to the input and ouput granularities
	 */
	protected void setDefaultProperties() {
		super.setDefaultProperties();
		addProperty("NUMBER_OF_TUPLES_TO_DEQUEUE", "0");
		addProperty("OUTPUT_TUPLE_GRANULARITY", FULL_TUPLE);
		addProperty("INPUT_TUPLE_GRANULARITY", FULL_TUPLE);
		//whether it is invoked by child 
		addProperty("DATA_DRIVEN_INVOKING", "1");
		//whether it is invoked by group_by_end info by descendent
		addProperty("PARTITION_DELIMITATION_INVOKING", "0");
		//whether it is directly schedulable
		addProperty("DIRECTLY_SCHEDULABLE", "0");
		addProperty("INITIAL_GREEDY_PRIORITY", "1");
		//whether we turn on the quick response feature
		addProperty("QUICK_RESPONSE", "0");
		addProperty("NUMBER_OF_POSSIBLE_OUTPUTS", "0");
		addProperty("NUMBER_OF_TUPLES_OUTPUTTED", "0");
		// The property "PERFORMANCE_FACTOR" indicates the performance of current operator. 
		// A simple formula used in our system now is 
		// PERFORMANCE_FACOTR = reduction_factor (later we can add in operator cost).
		// This property is used by optimizer and rewrite rule to make optimization decisions, 
		// push down operator with higher PERFORMANCE_FACTOR.
		addProperty("PERFORMANCE_FACTOR", "0");
		addProperty("REDUCTION_FACTOR", "0");

		/** 
		  * this is a unique ID assigned to each operator
		  * this will help to keep track of operators from
		  * one experiment to the next.  The idea is that
		  * if a query plan is generated the same way from 
		  * experiment to experiment, then its ID will be the same always.   
			*/
		addProperty("OBJECT_ID", "-1");
	}
	/**
	 * This will set the default properties that all
	 * XATStreamOperators will have.
	 * NUMBER_OF_TUPLES_TO_DEQUEUE: how many tuples the operator should dequeue 
	 * 		from its input queues in <emph> total <emph>
	 * NUMBER_OF_TUPLES_OUTPUTTED: how many tuples the operator placed into its output buffer
	 * 		during its last timeslice.
	 * NUMBER_OF_TIMES_RUN: how many times has this operator's run method been called
	 * LAST_TIME_RUN: when was the last time the operator was run
	 * TIME_TOOK_TO_RUN: how long did the operator take to run last time
	 * NUMBER_OF_TUPLES_IN_INPUT_QUEUE: how many tuples are in the operator's input queue's (combined)
	 * SELECTIVITY: the ratio of tuples that were dequeued to the number of tuples outputted
	 */
	protected void setDefaultStatistics() {
		super.setDefaultStatistics();
		addStatistic("NUMBER_OF_TUPLES_DEQUEUED_TOTAL");
		addStatistic("NUMBER_OF_TUPLES_DEQUEUED");
		addStatistic("NUMBER_OF_POSSIBLE_OUTPUTS");
		addStatistic("NUMBER_OF_TUPLES_OUTPUTTED");
		addStatistic("NUMBER_OF_TUPLES_OUTPUTTED_TOTAL");
		addStatistic("NUMBER_OF_TIMES_RUN");
		addStatistic("LAST_TIME_RUN");
		addStatistic("TIME_TOOK_TO_RUN");
		addStatistic("TIME_TOOK_TO_RUN_TOTAL");
		addStatistic("NUMBER_OF_TUPLES_IN_INPUT_QUEUES");
		addStatistic("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES");
		addStatistic("SUM_OF_TUPLES_DEQUEUED_TOTAL");
		addStatistic("SELECTIVITY");
		addStatistic("REDUCTION_FACTOR");
		addStatistic("AVERAGE_TUPLE_PROCESSING_TIME");
		addStatistic("GREEDY_PRIORITY");
		addStatistic("AVERAGE_OUTPUT_RATE");
		addStatistic("TUPLES_IN_LEFT_STATE");
		addStatistic("TUPLES_IN_RIGHT_STATE");
		//addStatistic("OLDEST_TUPLE_IN_QUEUE");

	}
	/**
	 * Sets the input queue of this operator. When the input queue is set,
	 * the operator is first registered with the queue. If the operator may not register with the queue,
	 * then the XATQueueException is thrown
	 * @param newInputQueue edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
	 * @throws XATQueueException if the operator cannot register with the queue 
	 *	(it is the queue owner or has already registered)
	 */
	public synchronized void setInputQueue(XATQueue newInputQueue)
		throws XATQueueException {
		try {
			newInputQueue.register(this);
			newInputQueue.setQueueNumber(0);
			if (inputQueue.size() == 0) {
				inputQueue.add(newInputQueue);
			} else {
				inputQueue.set(0, newInputQueue);
			}

		} catch (XATQueueException e) {
			throw e;
		}
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setNthState(edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState, int)
	 */
	public void setNthState(OperatorState state, int n) {
		this.states[n] = state;
	}

	/**
	 * This method sets the property "NUMBER_OF_POSSIBLE_OUTPUTS", which is the 
	 * possible output tuples from current operator from each run.
	 * For a single stream operator, this is simply the number of dequeued tuple.
	 * For a multiple input stream operator, such as join, special method need to be applied 
	 * in order to compute this property.
	 * This property will be used by stats gatherer to compute the reduction factor of this operator.
	 */
	public void setNumberOfPossibleOutputs() {
		try {
			this.setProperty(
				"NUMBER_OF_POSSIBLE_OUTPUTS",
				this.getProperty("NUMBER_OF_TUPLES_TO_DEQUEUE"));
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	public void setOperatorID(int id) {
		operatorID = id;
	}
	/**
	* Set the output table
	* 
	*/
	public void setOutput(XATDataSource output) {
		throw new UnsupportedOperationException("setOutput is not supported by XATStreamOperator, please use setOutputQueue");
	}
	/**
	 * set the outputpositions for the i th output
	 * Creation date: (4/4/2003 5:17:32 PM)
	 * @param i int
	 * @param ithOutputPositions int[]
	 */
	public synchronized void setOutputPositions(
		int i,
		int[] ithOutputPositions) {
		if (outputPositions == null) {
			outputPositions = new int[this.inputQueue.size()][];
		}
		outputPositions[i] = ithOutputPositions;
	}
	/**
	 * set the whole output positions (for every output)
	 * Creation date: (4/4/2003 5:17:53 PM)
	 * @param newOutputPositions int[][]
	 */
	public void setOutputPositions(int[][] newOutputPositions) {
		outputPositions = newOutputPositions;
	}
	/**
	 * sets the output queue of this operator.  The operator must be the
	 * owner of the queue, otherwise an exception will be thrown.
	 * @param newOutputQueue edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue
	 * @throws XATQueueException if this operator is not the owner of the queue
	 */
	public void setOutputQueue(XATQueue newOutputQueue) {
		if (newOutputQueue.isOwner(this)) {
			if (outputQueue.size() == 0) {
				outputQueue.add(newOutputQueue);
			} else {
				//first remove this operator as the owner of the existing queue
				 ((XATQueue) outputQueue.get(0)).removeOwner(this);
				outputQueue.set(0, newOutputQueue);
			}
		} else {
			throw new XATQueueException("Cannot set this to be the output queue because the operator does not own the queue");
		}

	}

	/**
	* <pre>
	* sets the given property to the new value.  This method is responsible
	* for validating the given property.  An exception is thrown if
	* the value is not correct.
	* Validation should be done in the following manner: 
	* First the method should check if the object supports the given property by calling
	* isValidPropertyName(String).  If it is not a valid property name, then an exception should
	* be thrown.
	*
	* Once it is known that the object supports the property, the value must be validated.  Each implementor of
	* XATStatistics has a set of properties that are common to all of its types and some that are specific to types.  
	* For example, all XATOperators (which implement XATStatistics) have certain properties while an individual XATOperator
	* has specific properties. The indivual types should only validate their own specific types.  If the caller is trying
	* to set a common type, it should be validated by the super class by calling super.setProperty(name, value).  This
	* will reduce duplicate code.  
	* </pre>
	* @param name the name of the property to set
	* @param value the new value of the property
	* @throws Exception if the given property cannot be found or
	* if the value is not valid for the given property.
	*/
	public void setProperty(String name, String value)
		throws java.lang.Exception {

		//first see if the XATOperator supports the property name	
		if (isValidPropertyName(name)) {
			if (name.equals("OUTPUT_TUPLE_GRANULARITY")
				|| name.equals("INPUT_TUPLE_GRANULARITY")) {
				try {
					if ((value == XATStreamOperator.FULL_TUPLE)
						|| (value.equals(XATStreamOperator.SUB_TUPLE))) {
						setNewPropertyValue(name, value);
					} else {
						throw new Exception(
							"The property named "
								+ name
								+ " requires a valid string ");
					}
				} catch (Exception e) {
					throw new Exception(
						"The property named "
							+ name
							+ " requires a valid string ");
				}
			} else {
				super.setProperty(name, value);
			}
		} else
			throw new Exception(
				"The XATOperator does not support the property named " + name);
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setRunning(boolean)
	 */
	public void setRunning(boolean running) {
		this.running = running;

	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setState(edu.wpi.cs.dsrg.xmldb.xat.common.operator.state.OperatorState[])
	 */
	public void setState(OperatorState[] state) {
		for (int i = 0; i < this.states.length; i++) {
			this.states[i] = state[i];
		}
	}
	/**
	 * Make a new copy of the current operator. This is a deep copy, not bit by bit copy as the default clone in java.
	 * make exact copy of every fields except queues. 
	 * @return
	 */
	//	public abstract XATStreamOperator makeNewCopy(XATNode newnode); 
	//	{

	//		XATStreamOperatorImp newop = new XATStreamOperatorImp(newnode);
	//		newop.active = this.active;
	//		newop.columnsFiltered = this.columnsFiltered;
	//		newop.debugFlag = this.debugFlag;
	//		newop.operatorID = this.operatorID;
	//		newop.outputColumnName = this.outputColumnName;
	//		newop.outputPositions = this.outputPositions;
	//		
	//		newop.state = new OperatorState[this.state.length];
	//		for(int i=0; i<state.length; i++) {
	//			newop.state[i] = this.state[i].makeNewCopy(newop);
	//		}
	//		
	//		newop.stats = this.stats;
	//		newop.statsPresent = this.statsPresent;
	//	
	//		return newop;
	//	}

	public String toShortString() {
		return "Op " + this.getObjectID();
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (1/21/03 10:44:48 AM)
	 * @return java.lang.String
	 */
	public String toString() {

		String rs = new String();

		String sName = this.getClass().getName();
		//find the last dot.
		int index = 0;
		for (index = sName.length() - 1; index >= 0; index--)
			if (sName.charAt(index) == '.')
				break;
		rs = sName.substring(index + 1) + " ";
		String id = this.getProperty("OBJECT_ID");
		if (id == "-1") {
			rs = rs + hashCode();
		} else {
			rs = rs + id;
		}

		if (this.isActive())
			rs += " ACTIVE ";
		else
			rs += " INACTIVE ";
		return rs;

	}

	/*
	 * check if this operator has any unmatched states.
	 * return: TRUE if this operator has unmatched states, FALSE otherwise.
	 */
	//public boolean hasUnmatchedState() {
	//	if (states == null) {
	//		return false;
	//	}
	//	
	//	for(int i=0; i<states.length; i++) {
	//		if(!states[i].isMatched())
	//			return true;
	//	}
	//	return false;
	//}

	/*
	 * Assign new ID to each state inside this operator, starting from the given startID.
	 * 
	 * return the next available ID. For example, if startID is 2, and there are 2 states inside this operator,
	 * then these two states are assigned new IDs as 2 and 3, and 4 is then returned as the next available stateID.
	 */
	//public int setStateID(int startID) {
	//	if (states == null) {
	//		return startID;
	//	}
	//	
	//	for(int i=0; i<states.length; i++) {
	//		states[i].setStateID(startID);
	//		startID++;
	//	}
	//	return startID;
	//}
}
