package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.InTimeSJStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.NaturalJoinStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.*;

import java.io.Serializable;
import java.util.*;

/**
 * for common methods for multiple scheduling modes
 * 
 * @author: Hong Su
 */
public class AllInvokingModeImp implements DataDrivenInvoking,
		DemandDrivenInvoking, Serializable {

	private XATStreamOperator physicalOp;

	//child operators of physicalOp
	private XATStreamOperator[] childOps;

	public void resetChildOps() {
		childOps = null;
	}

	public AllInvokingModeImp(XATStreamOperator op) {
		physicalOp = op;
	}

	/*
	 * for data-driven operators that can have at most one parent
	 * @param inputTuple List input to this operator
	 * @param inputPos int from which input source (in case this operator has multiple children)
	 */
	public boolean lightweightDataDrivenInvokingRun(List inputTuples,
			int inputPos) {
		List outputTuples;

		outputTuples = ((XATStreamOpRunCore) physicalOp).run(inputTuples,
				inputPos);

		if (outputTuples.size() == 0)
			return false;

		DagNode physicalOpNode = (DagNode) physicalOp.getNode();

		DagNode parentNode = (DagNode) physicalOpNode.getParent();
		XATStreamOperator parentOp = (XATStreamOperator) parentNode
				.getOperator();

		if (parentOp.getProperty("DATA_DRIVEN_INVOKING").equals("1")) 
			//parent operator agains invokes its parent operator
			return ((DataDrivenInvoking) parentOp).dataDrivenInvokingRun(
					outputTuples, 0);
		 else 
			//current operator enqueus its output 
			((XATQueueImp) (physicalOp.getOutputQueue())).enqueue(outputTuples);

		return true;
	}

	/*
	 * @param inputTuples List 
	 * @param inputPos which input source the inputTuples are generated from (inputPos is always 0 for a single input
	 * source operator) 
	 * @return boolean true if any output is generated
	 * inputTuples are passed to the operator A implementing this interface if
	 * inputTuples are not null, this means A has a child operator which
	 * directly passes its output to A if inputTuples are null, this means A is
	 * the source of data driven run chain, thus A has to fetch from its input
	 * queue the outputs of A are passed as parameters to invoke its parent
	 * operator B, therefore B does not need to fetch from the queue if B is not
	 * the end of this data driven run chain, it agains directly passes its
	 * output to its parent operator otherwise it puts the output in its output
	 * queue
	 */
	public boolean dataDrivenInvokingRun(List inputTuples, int inputPos) {
		List outputTuples;

		if (physicalOp instanceof AutomataEnabled)
			outputTuples = ((AutomataEnabled) physicalOp).runGroupBy();
				else
			outputTuples = ((XATStreamOpRunCore) physicalOp).run(inputTuples,
					inputPos);
		
		/** change by Drew 06/05/06
		 *  reason: outputTuples is never null, even when it's empty
		 */
		//Print output tuple
//		if (outputTuples!=null)
		if(outputTuples.size() > 0)
		/** end change **/
		{
			/*System.out.println("output tuple invoked by automata");
			System.out.println("size  " + outputTuples.size());*/
			for (int j = 0; j< outputTuples.size(); j++)
			
			//System.out.println( "j " + j + " " + outputTuples.get(j).toString());
			if (outputTuples.size() == 0)
				if (!(physicalOp instanceof NaturalJoinStream))
					//no inputs to parent operator, no need to prepare running
					// parent operator
					//if the inputs are the first inputs to naturaljoinstream
					//it does not have the other side to join with yet
					//but it does not indicate nothing yields when the other side
					// comes in
					return false;
				else
					return true;
		}
/*		else
			{
			System.out.println("no outputTuples! ");
			return false;
			} //*/
		

		

		DagNode physicalOpNode = (DagNode) physicalOp.getNode();

		boolean parentHasOutputs = true;

		for (Iterator i = physicalOpNode.getParents(); i.hasNext();) {
			DagNode parentNode = (DagNode) i.next();
			XATStreamOperator parentOp = (XATStreamOperator) parentNode
					.getOperator();
			if (parentHasOutputs) {

				if (parentOp.getProperty("DATA_DRIVEN_INVOKING").equals("1")) {
					//parent operator agains invokes its parent operator
					int childInputPos = FindOpVisitor.findChildOpPosInParentOp(
							physicalOpNode, parentNode);

					parentHasOutputs = ((DataDrivenInvoking) parentOp)
							.dataDrivenInvokingRun(outputTuples, childInputPos);

				} else 
					//current operator enqueue the outputs
					((XATQueueImp) (physicalOp.getOutputQueue()))
							.enqueue(outputTuples);
			}

		} 

		return parentHasOutputs;
	}

	/*
	 * for SQO
	 */
	public boolean selfPartitionDelimiInvokingRunForExtract(
			List partitionStates, SuspensionEdge suspendComp) {
		
		SelfPartitionDelimiInvoking selfDelimiOp = (SelfPartitionDelimiInvoking) physicalOp;

		if (!(selfDelimiOp.getPartitionChecked())) {
			selfDelimiOp.setPartitionChecked(true);
			int partitionStateNum = partitionStates.size();
			//no matter whether the state is suspended or not,
			//the paritionchecked flag should reset for the operator
			//curOp is extractbycollection
			if (lightweightDataDrivenInvokingRun(null, 0)) 
				//extractbycollection yields sth
				return true;			

			//this pattern fails
			//suspend the navOp which provides the bindings to the variable
			// that is partitioned on
			//suspend all the navs from this partition

			for (int i = 0; i < partitionStateNum - 1; i++)
				((ExtendedState) partitionStates.get(i)).suspendAllTransits(
						suspendComp, false);
			//clean temp for the last one
			//it covers all the temps for the previous ones
			//boolean indicates clean temporary results
			((ExtendedState) partitionStates.get(partitionStateNum - 1))
					.suspendAllTransits(suspendComp, true);

			return false;

		}

		return true;

	}

	/*
	 * @param partitionStates List 
	 * @param suspendComp SuspensionEdge 
	 * @return boolean output on a partition from this operator has finished send this
	 * output to its parent, run until hit the ancestor in time SJ which closes
	 * on this partition if there is output from this SJ return true else return
	 * false not for intimesj not for extractcollection
	 */
	public boolean selfPartitionDelimiInvokingRunForSJ(List partitionStates,
			SuspensionEdge suspendComp) {

		SelfPartitionDelimiInvoking selfPhysicalOp = (SelfPartitionDelimiInvoking) physicalOp;

		if (!selfPhysicalOp.getPartitionChecked()) {
			selfPhysicalOp.setPartitionChecked(true);

			//no matter whether the state is suspended or not,
			//the paritionchecked flag should reset for the operator

			//find the node whose parent is not data driven
			//if extract by unnest, it must have output its output
			if (physicalOp.getOutputQueue().getContents().size() > 0)
				return true;

			//this pattern fails
			//suspend the navOp which provides the bindings to the variable
			// that is partitioned on
			//suspend all the navs from this partition
			int partitionStateNum = partitionStates.size();

			for (int i = 0; i < partitionStateNum - 1; i++)
				((ExtendedState) partitionStates.get(i)).suspendAllTransits(
						suspendComp, false);
			//clean temp for the last one
			//it covers all the temps for the previous ones
			//boolean indicates clean temporary results
			((ExtendedState) partitionStates.get(partitionStateNum - 1))
					.suspendAllTransits(suspendComp, true);

			return false;

		}

		return true;

	}

	/**
	 * @return List[] a list of inputs generated by this operator's child
	 *         operators operator implementing this interface needs to call its
	 *         child operator(s) in order to generate any outputs demand driven
	 *         can be issued when a sj is invoked in a partition manner 
	 */
	public List[] demandDrivenInvokingRun() {

		XATStreamOperator[] childOps = getChildOps();
		int childrenNum = childOps.length;
		List[] outputs = new List[childrenNum];

		//whether one input source does not have inputs
		boolean onefailed = false;

		for (int i = 0; i < childrenNum; i++) {
			XATStreamOperator childOp = childOps[i];

			if (onefailed) {
				if (!((SelfPartitionDelimiInvoking) childOp)
						.getPartitionChecked())
					//only need to clean up the outputs
					((XATStreamOpRunCore) childOp).cleanTemp();
				//else
				((XATQueueImp) childOp.getOutputQueue()).dequeueAll();

				continue;
			}

			if (childOp instanceof ExtractStream) {
				//a single input source operator
				if (!((SelfPartitionDelimiInvoking) childOp)
						.getPartitionChecked()) {

					outputs[i] = ((ExtractStream) childOp).runGroupBy();
				} else
					outputs[i] = ((XATQueueImp) childOp.getOutputQueue())
							.dequeueAll();

				if (outputs[i].size() == 0) {
					//no inputs to parent operator, no need to prepare running
					// parent operator
					//an empty input
					//revisit for outer join
					onefailed = true;
				}

			} //end of if

			else { //not extract

				if (childOp instanceof XATSingleQueueStreamOperator) {
					if (!((SelfPartitionDelimiInvoking) childOp)
							.getPartitionChecked()) {

						List[] inputs = ((DemandDrivenInvoking) childOp)
								.demandDrivenInvokingRun();
						if (inputs != null)
							//might be a select over sj
							outputs[i] = ((XATStreamOpRunCore) childOp).run(
									inputs[0], 0);
						else
							//might be a select over an extract
							outputs[i] = ((XATQueueImp) (childOp
									.getOutputQueue())).dequeueAll();
					} else
						outputs[i] = ((XATQueueImp) (childOp.getOutputQueue()))
								.dequeueAll();
					if (outputs[i].size() == 0)
						onefailed = true;
				} else {
					//for the sj one, the outputs must have been in the output
					// queue

					outputs[i] = ((XATQueueImp) (childOp.getOutputQueue()))
							.dequeueAll();
					if (outputs[i].size() == 0)
						onefailed = true;
				}
			}

		} //end of for
		
		if (onefailed)
			//no outputs for the operator
			return null;

		return outputs;

	}

	//only for singletonSJ
	public List demandDrivenInvokingRun(int[] usefulInputs, int tupleLength) {
		XATStreamOperator[] childOps = getChildOps();
		int childrenNum = childOps.length;
		List outputs = new ArrayList();

		XATTuple outputTuple = new XATTupleImp(tupleLength);
		List tempOutput = new ArrayList();

		//whether one input source does not have inputs
		boolean onefailed = false;

		int usefulInputCursor = 0;
		int outputTupleCursor = 0;

		for (int i = 0; i < childrenNum; i++) {
			XATStreamOperator childOp = childOps[i];

			/*if (onefailed) {
				if (!((SelfPartitionDelimiInvoking) childOp)
						.getPartitionChecked())
					//only need to clean up the outputs
					((XATStreamOpRunCore) childOp).cleanTemp();
				//else
				((XATQueueImp) childOp.getOutputQueue()).dequeueAll();
				continue;

			}*/

			if (childOp instanceof ExtractStream) {
				//a single input source operator
				if (!((SelfPartitionDelimiInvoking) childOp)
						.getPartitionChecked())
					tempOutput = ((ExtractStream) childOp).runGroupBy();
				else
					tempOutput.addAll(((XATQueueImp) childOp.getOutputQueue())
							.dequeueAll());

			} //end of if

			else { //not extract

				if (childOp instanceof XATSingleQueueStreamOperator) {
					//must be selection
					if (!((SelfPartitionDelimiInvoking) childOp)
							.getPartitionChecked()) {
						List[] inputs = ((DemandDrivenInvoking) childOp)
								.demandDrivenInvokingRun();
						if (inputs != null) {

							tempOutput = ((XATStreamOpRunCore) childOp).run(
									inputs[0], 0);
						} else
							tempOutput = ((XATQueueImp) (childOp
									.getOutputQueue())).dequeueAll();

					} else
						onefailed = true;
				} else {
					//for the sj one, the outputs must have been in the output
					// queue

					tempOutput = ((XATQueueImp) (childOp.getOutputQueue()))
							.dequeueAll();

				}
			}

			if (tempOutput.size() == 0) {
				//no inputs to parent operator, no need to prepare running
				// parent operator
				//an empty input
				//revisit for outer join
				onefailed = true;
			} else {
				if (usefulInputCursor < usefulInputs.length
						&& i == usefulInputs[usefulInputCursor]) {
					XATDataValue[] curTuple = ((XATTupleImp) tempOutput.get(0))
							.getArrayData();

					for (int j = 0; j < curTuple.length; j++) {

						outputTuple.setCell(outputTupleCursor, curTuple[j]);
						outputTupleCursor++;
					}
					usefulInputCursor++;
				}
			}
			
			if (onefailed) {
				
				/*if (!((SelfPartitionDelimiInvoking) childOp)
						.getPartitionChecked())
					//only need to clean up the outputs
					((XATStreamOpRunCore) childOp).cleanTemp();
				//else
				((XATQueueImp) childOp.getOutputQueue()).dequeueAll();*/
				
				if (physicalOp instanceof InTimeSJStream){
					List entryOps = ((InTimeSJStream)physicalOp).getEntryOps();
					int entryOpNum = entryOps.size();
					for (int j=0; j<entryOpNum; j++){
						XATStreamOperator op = (XATStreamOperator)entryOps.get(j);
						((XATQueueImp) childOp.getOutputQueue()).dequeueAll();
					}
					
				}
				break;
			}

		} //end of for

		if (onefailed)
			//return an empty list
			return outputs;

		outputs.add(outputTuple);

		return outputs;

	}

	/*
	 * we user branchEvalOrder instead of reajusting the children's order
	 * because we have encoded what particular cells need to be projected in the
	 * out and we do not want to recompute that when the chilren's order is
	 * changed
	 */
	public List[] demandDrivenInvokingRunWithOrder() {
		List branchEvalOrder = ((DemandDrivenInvoking) physicalOp)
				.getBranchEvalOrder();

		XATStreamOperator[] childOps = getChildOps();
		int childrenNum = childOps.length;
		List[] outputs = new List[childrenNum];

		//whether one input source does not have inputs
		boolean onefailed = false;

		for (int i = 0; i < childrenNum; i++) {
			XATStreamOperator childOp = (XATStreamOperator) branchEvalOrder
					.get(i);

			

			if (childOp instanceof ExtractStream) {
				outputs[i] = ((XATQueueImp) childOp.getOutputQueue())
						.dequeueAll();

			} else { //not extract, intermediate op between extract and sj,
					 // e.g., select

				if (childOp instanceof XATSingleQueueStreamOperator) {
					if (!((SelfPartitionDelimiInvoking) childOp)
							.getPartitionChecked()) {

						List[] inputs = ((DemandDrivenInvoking) childOp)
								.demandDrivenInvokingRunWithOrder();
						if (inputs != null)
							outputs[i] = ((XATStreamOpRunCore) childOp).run(
									inputs[0], 0);
						else
							outputs[i] = ((XATQueueImp) (childOp
									.getOutputQueue())).dequeueAll();
					} else
						outputs[i] = ((XATQueueImp) (childOp.getOutputQueue()))
								.dequeueAll();
					if (outputs[i].size() == 0)
						onefailed = true;
					else
						/** **** commented for statistics collection** */

						((XATStreamOpRunCore) childOp).getTimeKeeper()
								.increNotEmptyCount();
					if (childOp instanceof ExtractByCollectionStream)
						((XATStreamOpRunCore) childOp.getNode().getChild(0)
								.getOperator()).getTimeKeeper()
								.increNotEmptyCount();
					/** ****end of commenting out for statistics collection *** */

				} else {
					//for the sj one, the outputs must have been in the output
					// queue
					// also, sj is an entry operator whose statistics have been
					// collected in step 1

					outputs[i] = ((XATQueueImp) (childOp.getOutputQueue()))
							.dequeueAll();
					if (outputs[i].size() == 0)
						onefailed = true;

				}
			}
			
			if (onefailed) {
				
				/*if (!((SelfPartitionDelimiInvoking) childOp)
						.getPartitionChecked())
					//only need to clean up the outputs
					((XATStreamOpRunCore) childOp).cleanTemp();
				//else
				((XATQueueImp) childOp.getOutputQueue()).dequeueAll();*/
				
				break;
			}

		} //end of for

		if (onefailed)
			//no outputs for the operator
			return null;

		return outputs;
	}

	/*	only for singletonSJ
	 * 
	 */
	public List demandDrivenInvokingRunWithOrder(int[] usefulInputs,
			int tupleLength) {
		
		List branchEvalOrder = ((DemandDrivenInvoking) physicalOp)
				.getBranchEvalOrder();

		XATStreamOperator[] childOps = getChildOps();
		int childrenNum = childOps.length;
		List outputs = new ArrayList();

		XATTuple outputTuple = new XATTupleImp(tupleLength);
		List tempOutput = new ArrayList();

		//whether one input source does not have inputs
		boolean onefailed = false;

		int usefulInputCursor = 0;
		int outputTupleCursor = 0;

		for (int i = 0; i < childrenNum; i++) {
			XATStreamOperator childOp = (XATStreamOperator) branchEvalOrder
					.get(i);
			if (onefailed) {
				/*((XATQueueImp) childOp.getOutputQueue()).dequeueAll();
				continue;*/
				break;

			}

			if (childOp instanceof ExtractStream)
				tempOutput = (((XATQueueImp) childOp.getOutputQueue())
						.dequeueAll());
			else { //not extract

				if (childOp instanceof XATSingleQueueStreamOperator) {
					//must be selection
					//if (!((SelfPartitionDelimiInvoking) childOp)
					//		.getPartitionChecked()) {
					List[] inputs = ((DemandDrivenInvoking) childOp)
							.demandDrivenInvokingRun();
					if (inputs != null) {

						tempOutput = ((XATStreamOpRunCore) childOp).run(
								inputs[0], 0);
						//	} else
						//		tempOutput = ((XATQueueImp) (childOp
						//				.getOutputQueue())).dequeueAll();

					} else
						onefailed = true;
				} else {
					//for the sj one, the outputs must have been in the output
					// queue

					tempOutput = ((XATQueueImp) (childOp.getOutputQueue()))
							.dequeueAll();

				}
			}

			if (tempOutput.size() == 0) {
				//no inputs to parent operator, no need to prepare running
				// parent operator
				//an empty input
				//revisit for outer join
				onefailed = true;
			} else {

				/** **** commented for statistics collection** */
				((XATStreamOpRunCore) childOp).getTimeKeeper()
						.increNotEmptyCount();
				if (childOp instanceof ExtractByCollectionStream)
					((XATStreamOpRunCore) childOp.getNode().getChild(0)
							.getOperator()).getTimeKeeper()
							.increNotEmptyCount();
				/** ****end of commenting out for statistics collection *** */
				if (usefulInputCursor < usefulInputs.length
						&& i == usefulInputs[usefulInputCursor]) {
					XATDataValue[] curTuple = ((XATTupleImp) tempOutput.get(0))
							.getArrayData();
// Mingzhu' change for singleton join
					for (int j = 0; j < curTuple.length; j++) {

						outputTuple.setCell(outputTupleCursor, curTuple[j]);
						outputTupleCursor++;
					}//*/
					/*for (int j = 0; j < tupleLength; j++) {

						outputTuple.setCell(outputTupleCursor, curTuple[j]);
						outputTupleCursor++;
					}*/
//					outputTupleCursor = 0;
					usefulInputCursor++;
				}
			}

		} //end of for
		//	demandDrivenTimeKeeper.pause();

		if (onefailed)
			//return an empty list
			return outputs;

		outputs.add(outputTuple);

		return outputs;

	}

	private XATStreamOperator[] getChildOps() {
		if (childOps != null)
			return childOps;
		else {
			XATNode[] childrenNodes = physicalOp.getNode().getChildren();
			int childrenNum = childrenNodes.length;

			childOps = new XATStreamOperator[childrenNum];
			for (int i = 0; i < childrenNum; i++)
				childOps[i] = (XATStreamOperator) childrenNodes[i]
						.getOperator();

			return childOps;
		}

	}

	/**
	 * only directly called when this operator is the beginning (i.e., topmost
	 * upstream) of a data driven pipelining chain
	 */
	public void run() {
		((DataDrivenInvoking) this).dataDrivenInvokingRun(null, 0);
	}

	public boolean getPartitionChecked() {
		return false;
	}

	public void setPartitionChecked(boolean partitionChecked) {

	}

	public List getBranchEvalOrder() {
		return null;
	}

	public void setBranchEvalOrder(List branchEvalOrder) {

	}

}