package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode;

/**
 * For operators that need special information collected in automata, such as
 * category 1: push_stack, pop_stack, slip_stack (for extract operator to compose the tuples from tokens)
 * category 2: groupby_end (for any operators that have collection semantics, e.g., extractNest, collSelect)
 * for operators that fall in category 2, we additionally support another event "nothing",
 * it allows the operator to run when it is expecting a groupby_end event to happen,
 * but in the interim between the occurrence of expected events, it does not really rely on any information collection in automata
 * 
 */

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.InTimeSJStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;

import java.io.Serializable;
import java.util.*;

public class AutomataEnabledImp implements AutomataEnabled, Serializable {
	protected XATStreamOperator physicalOp;
	
	
	//especially for the automata state whose activate triggers an Extract operator in a regular-invoke mode
	//it is not the automata state that triggers the wrapping up of collections, i.e., it does not invoke in a groupby-end mode
	//we say the latter sends the metat-data info while the former send the data
	private ExtendedState opStateForData;
	
	private boolean partitionInvoked = false;
	private boolean partitionInvokedSet = false;
	
	public AutomataEnabledImp(XATStreamOperator physicalOp) {
		super();
		this.physicalOp = physicalOp;

	}
	
	public ExtendedState getOpState() {
		return opStateForData;
	}
	
	public int getTriggeringRunTimeStateId() {
		return 0;
	}
	
	/**
	 * whether the physicalOp is expecting a groupby_end event, i.e., can be partition-invoked
	 * 
	 */
	public boolean isPartitionInvoked() {
		if (partitionInvokedSet)
			return partitionInvoked;

		try {
			if (physicalOp instanceof ExtractStream){
			
			List ancestors = FindOpVisitor.findAncestorOps((DagNode)physicalOp.getNode());
			for (Iterator it = ancestors.iterator(); it.hasNext();){
				XATOperator op = (XATOperator)it.next();
				if (op instanceof InTimeSJStream){
				  partitionInvokedSet = true;
				  partitionInvoked = false;
				  return false;	
				}
			}
			}

			if (physicalOp instanceof XATSingleQueueStreamOperator) {
				//if input tuple is subtuple and output is tuple or
				//if extractByCollection and output is tuple
				if (physicalOp
					.getProperty("OUTPUT_TUPLE_GRANULARITY")
					.equals(XATStreamOperator.FULL_TUPLE)
					&& (physicalOp
						.getNode()
						.getChild(0)
						.getOperator()
						.getProperty("OUTPUT_TUPLE_GRANULARITY")
						.equals(XATStreamOperator.SUB_TUPLE)
						|| physicalOp instanceof ExtractByCollectionStream
						))
					partitionInvoked = true;
					
			
				partitionInvokedSet = true;
				return true;

			} else if (physicalOp instanceof XATMultiQueueStreamOperator) {

			}

		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

		partitionInvoked = false;
		partitionInvokedSet = true;

		return false;
	}
	
	
	public void associateWithAuto(ExtendedAutomata auto) {
		
	}

	
	public void cleanTemp() {
		
	}

	
	public List runGroupBy() {
		return null;
	}

	
	public void runPopStack(int tokenId, int startTokenId) {
		
	}
	
	public void runPushStack(int tokenId, int startTokenId) {
		
	}
}
