package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;

/**
 * This scheduler is for Hong's testing
 * it implements a pure bottom-up pipelining execution strategy
 * the only freedom is how to schedule the XMLScan operators spanning over multiple sources
 *
 */

public class BottomUpPipelineScheduler {

	//the plan that is to be executed
	private XATDag planDag;

	/**
	 * BottomUpPipelineScheduler constructor comment.
	 */
	public BottomUpPipelineScheduler() {
			super();
		}
	
	public BottomUpPipelineScheduler(XATDag planDag) {
		super();
		this.planDag = planDag;
	}
	
	public void setPlanDag(XATDag planDag){
		this.planDag = planDag;
	}

	public XATDag getPlanDag() {
		return planDag;
	}

	
	/*
	 * schedule the give dagplan
	 * need to invoke the statistics registration
	 * to put the statistic information into statisticscollector
	 * @int i the number of the destination element node of the bottommost TokenNav operator
	 * once the $i^{th}$ element is finished processing
	 * start register 
	 */
	public void scheduling(int desiredBottomDestNum){
		XATNode[] leaves = planDag.getLeaves();
		XATStreamOperator[] leafOps = new XATStreamOperator[leaves.length];

		//set up the token consuming property
		for (int i = 0; i < leaves.length; i++) {
			XATStreamOperator leafOp =
				(XATStreamOperator) leaves[i].getOperator();
			if (!(leafOp instanceof XMLScanStream)) {
				System.out.println(
					"exception: leaf op " + leafOp + " not an XMLScanStream");
				System.exit(-1);
			}

			try {
				leafOp.setProperty("NUMBER_OF_TUPLES_TO_DEQUEUE", "500");
			} catch (Exception e) {
				e.printStackTrace();
			}
			leafOps[i] = leafOp;
		}

		int finishCount = 0;
		int srcNum = leaves.length;
		boolean[] finishFlags = new boolean[srcNum];
		for (int i = 0; i < srcNum; i++) {
			finishFlags[i] = false;
		}

		int round = 0;

		while (true) {

			// System.out.println("");
			if (finishCount == srcNum)
				//all done
				break;

			round++;

			//always run when exception thrown, should be queue empty exception
			for (int i = 0; i < leaves.length; i++) {
				try {
					// System.out.println("start run");
					if (!finishFlags[i])
						((XMLScanStream)leafOps[i]).run(desiredBottomDestNum);
					if (((XMLScanStream) leafOps[i]).isFinished()) {
						//if control returned because of no more tokens to consume
						finishFlags[i] = true;
						//this is for consecutive running, reutilizing the opeator
						 ((XMLScanStream) leafOps[i]).setFinished(false);
						finishCount++;
						continue;
					}

				} catch (Exception e) {
					e.printStackTrace();
					System.exit(-1);
				}
			} //end of for
		} //end of while

	}
	/**
	 * schedule the given dagplan
	 * Creation date: (4/13/2003 2:13:38 PM)
	 */
	public void scheduling() {

		XATNode[] leaves = planDag.getLeaves();
		XATStreamOperator[] leafOps = new XATStreamOperator[leaves.length];

		//set up the token consuming property
		for (int i = 0; i < leaves.length; i++) {
			XATStreamOperator leafOp =
				(XATStreamOperator) leaves[i].getOperator();
			if (!(leafOp instanceof XMLScanStream)) {
				System.out.println(
					"exception: leaf op " + leafOp + " not an XMLScanStream");
				System.exit(-1);
			}

			try {
				leafOp.setProperty("NUMBER_OF_TUPLES_TO_DEQUEUE", "500");
			} catch (Exception e) {
				e.printStackTrace();
			}
			leafOps[i] = leafOp;
		}

		int finishCount = 0;
		int srcNum = leaves.length;
		boolean[] finishFlags = new boolean[srcNum];
		for (int i = 0; i < srcNum; i++) {
			finishFlags[i] = false;
		}

		int round = 0;

		while (true) {

			if (finishCount == srcNum)
				//all done
				break;

			round++;

			//always run when exception thrown, should be queue empty exception
			for (int i = 0; i < leaves.length; i++) {
				try {
					if (!finishFlags[i])
						leafOps[i].run();
					if (((XMLScanStream) leafOps[i]).isFinished()) {
						//if control returned because of no more tokens to consume
						finishFlags[i] = true;
						//this is for consecutive running, reutilizing the opeator
						 ((XMLScanStream) leafOps[i]).setFinished(false);
						finishCount++;
						continue;
					}

				} catch (Exception e) {
					e.printStackTrace();
					System.exit(-1);
				}
			} //end of for
		} //end of while
	}
}
