package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode;

/**
 * Mode means how invocation of this operator may invoke other operators, i.e., from the run method
 * of this operator, run methods of what other operators are called
 
 * DataDrivenMode: if an operator implements this interface, once it generates some outputs
 * its parent operator is invoked to consume the outputs
 * i.e., the control flow is from child operator to parent operator
 *
 */
import java.util.*;

public interface DataDrivenInvoking {
	/**
	 * output tuples of an operator A implementing this interface are passed as parameters  
	 * to the parent operator B, therefore B does not need to fetch from the queue
	 * this is a pipeling manner in the sense that all intermediate data is directly passed
	 * to be consumed rather than stored (neither in second storage nor memory)
	  
	 * Creation date: (2/22/2003 2:24:23 PM)
	 */
	boolean dataDrivenInvokingRun(List inputTuples, int inputPos);
	boolean lightweightDataDrivenInvokingRun(List inputTuples, int inputPos);
	
}
