package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator;

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.schedulemode
	.AllInvokingModeImp;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.schedulemode
	.DataDrivenInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore;
/**
 * perform cartesian product 
 */
public class CartesianStream
	extends XATMultiQueueStreamOperatorImp
	implements XATStreamOpRunCore, DataDrivenInvoking {
	protected XATSynopsisQueueImp[] synopsis;
	protected AllInvokingModeImp allInvokingModes;

	int tupleLength = 0;
	int[] usefulSrcs = null;

	protected ClassExecTimeKeeper timeKeeper;
	
	public CartesianStream(DagNode node) {
		super(node);
		allInvokingModes = new AllInvokingModeImp(this);
	}
	
	
	public void resetUsefulSrcs(){
		usefulSrcs = null;
	}
	
	public AllInvokingModeImp getAllInvokingModes(){
		return allInvokingModes;
	}
	
	public void cleanup() {
		int size = synopsis.length;

		for (int i = 0; i < size; i++)
			//clear all synopsises
			synopsis[i].clear();

	}
	
	public void initializeSynopsis(int inputNum) {
		synopsis = new XATSynopsisQueueImp[inputNum];
		for (int i = 0; i < inputNum; i++) {
			synopsis[i] = new XATSynopsisQueueImp(this);
			synopsis[i].register(this);
		}

	}
	/**
	 * join the tuples
	 * @param tuples List[] inputs
	 * @param List outputs
	 */
	public List join(List[] tuples) {

		int[][] temp = getOutputPositions();
		List outputTuples = new ArrayList();

		int srcNum = tuples.length;

		int tupleLength = getTupleLength();

		int[] cursors = new int[srcNum];

		int[] inputsInResults = getUsefulInputSrc();
		int inputsInResultsLength = inputsInResults.length;

		while (true) {

			XATDataValue[] newData = new XATDataValue[tupleLength];
			int startPos = 0;

			for (int j = 0; j < inputsInResultsLength; j++) {
				int inputSrc = inputsInResults[j];

				//there is something in source j that needs to be filled
				(
					(XATTupleImp) tuples[inputSrc].get(
						cursors[inputSrc])).fillDataInListTo(
					newData,
					temp[inputSrc],
					startPos);

				startPos = startPos + temp[inputSrc].length;

			}

			outputTuples.add(new XATTupleImp(newData));

			//adjust the cursor 
			for (int k = srcNum - 1; k >= 0; k--) {
				cursors[k]++;
				if (cursors[k] == tuples[k].size()) {
					if (k == 0)
						//all inputs has been consumed
						return outputTuples;
					else
						cursors[k] = 0;
				} else
					//cursor has been set up correctly at this point
					break;

			}

		} //end of while true

		
	}

	protected int getTupleLength() {
		if (tupleLength != 0)
			return tupleLength;
		else {
			tupleLength = getOutputQueue().getSchema().getLength();
			return tupleLength;
		}
	}

	/*
	 * the sources whose inputs need to be filled
	 */
	protected int[] getUsefulInputSrc() {
		if (usefulSrcs != null)
			return usefulSrcs;
		else {
			int count = 0;
			//set up
			int[][] temp = getOutputPositions();
			for (int i = 0; i < temp.length; i++)
				if (temp[i].length != 0)
					count++;

			usefulSrcs = new int[count];
			int curPos = 0;
			for (int j = 0; j < temp.length; j++)
				if (temp[j].length != 0) {
					usefulSrcs[curPos] = j;
					curPos++;
				}

			return usefulSrcs;

		}
	}

	/*
	 * @param lTuple XATTuple left input
	 * @param rTuple XATTuple right input
	 * @return XATTuple an output tuple
	 */
	public XATTuple join(XATTuple lTuple, XATTuple rTuple) {

		XATDataValue[] resultData =
			new XATDataValue[getOutputQueue().getSchema().getLength()];

		int[][] temp = getOutputPositions();

		((XATTupleImp) lTuple).fillDataInListTo(resultData, temp[0], 0);

		((XATTupleImp) rTuple).fillDataInListTo(
			resultData,
			temp[1],
			temp[0].length);

		return new XATTupleImp(resultData);
	}
	
	/*
	 * binary join
	 * @param lTuples List left inputs 
	 * @param rTuples List right inputs
	 * @return List outputs
	 */
	public List join(List lTuples, List rTuples) {

		List result = new ArrayList();
		int kk = 0;

		XATDataValue[] resultData =
			new XATDataValue[getOutputQueue().getSchema().getLength()];

		int[][] temp = getOutputPositions();

		for (int i = 0; i < lTuples.size(); i++)
			for (int j = 0; j < rTuples.size(); j++) {
				resultData =
					new XATDataValue[getOutputQueue().getSchema().getLength()];

				((XATTupleImp) lTuples.get(i)).fillDataInListTo(
					resultData,
					temp[0],
					0);

				((XATTupleImp) rTuples.get(j)).fillDataInListTo(
					resultData,
					temp[1],
					temp[0].length);

				result.add(new XATTupleImp(resultData));

			}

		System.out.println(result);
		return result;
	}
	
	public List join(List lTuples, List rTuples, int[] rightPos) {

		List result = new ArrayList();

		//the position where the right tuple should fill from in the result tuple
		XATTupleImp lFirstTuple = (XATTupleImp) lTuples.get(0);

		int resultSize = 0;

		int rTupleFillPos = lFirstTuple.getLength();

		if (rightPos != null)
			resultSize = rightPos.length + rTupleFillPos;

		XATDataValue[] resultData = new XATDataValue[resultSize];

		//if (rTupleFillPos == 1 && lFirstTuple.getCell(0) == null)
		//these rTuples may not actually have any thing, they just indicate the existence
		//no need to put this null in the result data
		//	rTupleFillPos = 0;

		int lTupleSize = lTuples.size();
		int rTupleSize = rTuples.size();

		for (int i = 0; i < lTupleSize; i++) {
			XATTupleImp lTuple = (XATTupleImp) lTuples.get(i);

			for (int j = 0; j < rTupleSize; j++) {

				/////////////////
				// change part...
				/////////////////
				resultData = new XATDataValue[resultSize];

				lTuple.fillDataInListTo(resultData);
				((XATTupleImp) rTuples.get(j)).fillDataInListTo(
					resultData,
					rightPos,
					rTupleFillPos);
				result.add(new XATTupleImp(resultData));
			}
		}

		return result;
	}

	/*
	 * @param lTuplesSize the number of tuples in results, the final result holder 
	 */
	public int join(
		int lTupleSize,
		int rTupleFillPos,
		List rTuples,
		int[] rightPos,
		List results) {

		//if (rTupleFillPos == 1 && lFirstTuple.getCell(0) == null)
		//these rTuples may not actually have any thing, they just indicate the existence
		//no need to put this null in the result data
		//	rTupleFillPos = 0;

		int rTupleSize = rTuples.size();

		int resultPos = 0;

		for (int i = 0; i < lTupleSize; i++) {
			XATTupleImp lTuple = (XATTupleImp) results.get(i);
			resultPos = i;

			for (int j = 0; j < rTupleSize; j++) {

				if (j == 0) {
					//does not need to copy lTuple
					//put rTuples in the same tuple as lTuple
					XATDataValue[] lTupleArrayData = lTuple.getArrayData();

					((XATTupleImp) rTuples.get(j)).fillDataInListTo(
						lTupleArrayData,
						rightPos,
						rTupleFillPos);

					lTuple.setValues(lTupleArrayData);

				} else {
					//make a copy of lTuple
					resultPos = resultPos + lTupleSize;

					XATTupleImp newTuple = (XATTupleImp) results.get(resultPos);

					XATDataValue[] newResultData = newTuple.getArrayData();

					lTuple.fillDataInListTo(newResultData);

					((XATTupleImp) rTuples.get(j)).fillDataInListTo(
						newResultData,
						rightPos,
						rTupleFillPos);

					newTuple.setValues(newResultData);

				}
			}
		}

		//output size is the index + 1
		return resultPos + 1;

	}

	/*
	 * used when the first list is joined with the second list
	 */
	public int join(
		List lTuples,
		List rTuples,
		int[] leftPos,
		int[] rightPos,
		List results) {

		int lTupleSize = lTuples.size();
		int rTupleSize = rTuples.size();

		int resultPos = 0;

		for (int i = 0; i < lTupleSize; i++) {
			XATTupleImp lTuple = (XATTupleImp) lTuples.get(i);

			for (int j = 0; j < rTupleSize; j++) {				

				XATTuple newTuple = (XATTuple) results.get(resultPos);
				XATDataValue[] toFill = newTuple.getArrayData();

				//	if (leftPos != null)
				//there exists something in lTuples that are in the final results
				lTuple.fillDataInListTo(toFill, leftPos, 0);

				//if (rightPos != null)
				((XATTupleImp) rTuples.get(j)).fillDataInListTo(
					toFill,
					rightPos,
					leftPos.length);

				newTuple.setValues(toFill);

				resultPos++;

			}
		}

		return resultPos;

	}

	/*
	 * @param lTuples List tuples from left side
	 * @param rTuples List tuples from right side
	 * @param leftPos int[] positions of cells from left side tuples that should appear in final resultant tuples
	 * @param rightPos int[] positions of cells from right side tuples that should appear in final resultant tuples
	 */
	public List join(
		List lTuples,
		List rTuples,
		int[] leftPos,
		int[] rightPos) {

		List result = new ArrayList();

		int resultSize = 0;

		if (leftPos != null)
			resultSize = leftPos.length;

		if (rightPos != null)
			resultSize = resultSize + rightPos.length;

		XATDataValue[] resultData = new XATDataValue[resultSize];

		int lTupleSize = lTuples.size();
		int rTupleSize = rTuples.size();

		for (int i = 0; i < lTupleSize; i++) {
			XATTupleImp lTuple = (XATTupleImp) lTuples.get(i);

			for (int j = 0; j < rTupleSize; j++) {
				resultData = new XATDataValue[resultSize];

				if (leftPos != null)
					//there exists something in lTuples that are in the final results
					lTuple.fillDataInListTo(resultData, leftPos, 0);

				if (rightPos != null)
					((XATTupleImp) rTuples.get(j)).fillDataInListTo(
						resultData,
						rightPos,
						leftPos.length);

				result.add(new XATTupleImp(resultData));
			}
		}

		return result;
	}

	public boolean run() {
		allInvokingModes.run();
		return true;

	}

	/**
	 * Returns a String that represents the value of this object.
	 * @return a string representation of the receiver
	 */
	public String toString() {
		return "Stream Cartesian Product";
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperatorImp#init()
	 */
	public void init() {
		// TODO Auto-generated method stub

	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator#setLeftInputQueue(edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue)
	 */
	public void setLeftInputQueue(XATQueue newQueue) {
		// TODO Auto-generated method stub

	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator#setLeftOutputQueue(edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue)
	 */
	public void setLeftOutputQueue(XATQueue newQueue) {
		// TODO Auto-generated method stub

	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#toToolTipString()
	 */
	public String toToolTipString() {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATMultiQStreamOpRunCore#run(java.util.List, int)
	 */
	public List run(List inputTuples, int inputPos) {
		List result = new ArrayList();

		for (int i = 0; i < inputTuples.size(); i++) {

			XATTuple tIncoming = (XATTuple) inputTuples.get(i);

			synopsis[inputPos].enqueue(tIncoming, this);

			int otherPartyPos;

			if (inputPos == 0)
				otherPartyPos = 1;
			else
				otherPartyPos = 0;

			synopsis[otherPartyPos].resetCursor(this);

			for (int j = 0; j < synopsis[1].getQueueSize(); j++) {
				try {
					XATTuple tExistSynop =
						(XATTuple) synopsis[otherPartyPos].dequeue(this);

					if (inputPos < otherPartyPos)
						result.add(join(tIncoming, tExistSynop));
					else
						result.add(join(tIncoming, tExistSynop));

					//oQueue.enqueue(newTuple, this);

				} catch (XATQueueException e1) {
					break;
				}
			}
		}

		return result;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.DataDrivenInvoking#dataDrivenInvokingRun(java.util.List, int)
	 */
	public boolean dataDrivenInvokingRun(List inputTuples, int inputPos) {
		return allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATMultiQStreamOpRunCore#runWithBlockingInputs(java.util.List[], boolean[])
	 */
	public List runWithBlockingInputs(List[] inputTuples, int pos) {
		// TODO Auto-generated method stub
		return null;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperatorImp#getStateSize()
	 */
	public int getStateSize() {
		// TODO Auto-generated method stub
		return 0;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAccumulatedTuples(long)
	 */
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAllAccumulatedTuples()
	 */
	public boolean cleanAllAccumulatedTuples() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#matchStates(edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator)
	 */
	public boolean matchStates(XATStreamOperator op) {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasUnmatchedState()
	 */
	public boolean hasUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#recomputeUnmatchedState()
	 */
	public boolean recomputeUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setStateID(int)
	 */
	public int setStateID(int startID) {
		// TODO Auto-generated method stub
		return 0;
	}
	
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.DataDrivenInvoking#lightweightDataDrivenInvokingRun(java.util.List, int)
	 */
	public boolean lightweightDataDrivenInvokingRun(List inputTuples, int inputPos) {
		return allInvokingModes.lightweightDataDrivenInvokingRun(inputTuples, inputPos);
	}
	
	
	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}
	
	
	

}
