package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATSubTuple;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.Finer_StreamStoreMgr;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.XATKeyDataValueForRecursion;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;

/**
 * sturctural join that is synchronized with automata events
 * 
 * @author: Hong Su
 */

public class InTimeSJStreamImp extends CartesianStream implements
		InTimeSJStream, DemandDrivenInvoking {

	public PureNavigateAutoStream branchNavOp;

	public boolean holderIncludesBegin;

	public XATSubTuple subTupleHolder = new XATSubTuple();

	public boolean holderPassed = false;

	protected AllInvokingModeImp allInvokingModes;

	public boolean shortCircuit;

	boolean needSetPartitionCheck = false;

	//key is the column
	//object is the source that generates the column, must be an extract
	protected Hashtable shortCircuitMgr;

	protected List shortCircuitedOps = null;

	protected AutomataEnabledImp automataCommon;

	protected ClassExecTimeKeeper timeKeeper;

	protected ExtendedState opState;

	//revisit
	//paritionchecked may not be set correctly, but this is not cared anyway
	protected boolean partitionChecked = false;

	//the list of entry operators
	protected List entryOps = new ArrayList();

	List branchEvalOrder = new ArrayList();;

	/**
	 * InTimeSJImp constructor comment.
	 * 
	 * @param node
	 *            edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
	 */
	public InTimeSJStreamImp(DagNode node) {
		super(node);
		allInvokingModes = new AllInvokingModeImp(this);
		timeKeeper = new ClassExecTimeKeeper(this);

	}

	public AllInvokingModeImp getAllInvokingModes() {
		return allInvokingModes;
	}

	public void setBranchEvalOrder(List branchEvalOrder) {
		this.branchEvalOrder = branchEvalOrder;
	}
	
	public void removeEntry(XATStreamOperator op){
		entryOps.remove(op);
	}
	
	public void addEntry(XATStreamOperator op){
		entryOps.add(op);
	}

	/**
	 * Insert the method's description here. Creation date: (2/2/2003 5:42:24
	 * PM)
	 * 
	 * @param auto
	 *            edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.ExtendedAutomata
	 */
	public void associateWithAuto(ExtendedAutomata auto) {

		//need to add itself to the groupbyops field of an appropriate state
		PureNavigateAutoStream navOp = getBranchNavOp();

		List states = auto.getFrStateHash(navOp);

		for (Iterator it = states.iterator(); it.hasNext();) {

			ExtendedState state = (ExtendedState) it.next();
			//possibly this node has been visit twice if it has two child
			if (!(state.getGroupbyOps().contains(this))) {
				state.addGroupbyOp(this);
				opState = state;
				//	auto.addIntoStateHash(this, state);
			}
		}

		//		set up default branch evaluation order, from left to right
		int childrenSize = xnodOwner.getChildrenSize();
		for (int i = 0; i < childrenSize; i++)
			branchEvalOrder.add(xnodOwner.getChild(i).getOperator());

		//set up entry operator
		entryOps = FindOpVisitor.findEntryOps(this);
	}

	/**
	 * output tuples of an operator A implementing this interface are passed as
	 * parameters to the parent operator B, therefore B does not need to fetch
	 * from the queue this is a pipeling manner in the sense that all
	 * intermediate data is directly passed to be consumed rather than stored
	 * (neither in second storage nor memory)
	 * 
	 * Creation date: (4/8/2003 1:20:09 PM)
	 */
	public boolean dataDrivenInvokingRun(List inputTuples, int inputPos) {
		return allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);
	}

	/**
	 * Insert the method's description here. Creation date: (2/2/2003 10:06:28
	 * PM)
	 * 
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedAutomata
	 */
	public ExtendedAutomata getAutomata() {
		return null;
	}

	/**
	 * Insert the method's description here. Creation date: (2/2/2003 7:02:16
	 * PM)
	 * 
	 * @return edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAuto
	 */
	public PureNavigateAutoStream getBranchNavOp() {
		return branchNavOp;
	}

	/**
	 * Insert the method's description here. Creation date: (4/30/2003 4:40:46
	 * PM)
	 * 
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState
	 */
	public ExtendedState getOpState() {
		return opState;
	}

	/**
	 * given the column name, find related ops that can be shortcircuited
	 * colName must be one in an existential format Creation date: (2/20/2003
	 * 6:58:03 PM)
	 * 
	 * @return java.util.List
	 */
	public XATStreamOperator getShortCircuitedOp(String colName) {

		XATStreamOperator op = (XATStreamOperator) shortCircuitMgr.get(colName);

		if (op == null) {
			//FindOpVisitor visitor = new FindOpVisitor();
			XATStreamOperator foundOp = (XATStreamOperator) FindOpVisitor
					.findColGeneratingSource(this, colName);
			shortCircuitMgr.put(colName, foundOp);
			return foundOp;
		} else
			return op;
	}

	/**
	 * Insert the method's description here. Creation date: (2/2/2003 10:06:28
	 * PM)
	 * 
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.StreamStoreMgr
	 */
	public Finer_StreamStoreMgr getStoreMgr() {
		return null;
	}

	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}

	/**
	 * Insert the method's description here. Creation date: (2/2/2003 10:06:28
	 * PM)
	 */
	public int getTriggeringRunTimeStateId() {
		return 0;
	}

	public boolean isPartitionInvoked() {
		return automataCommon.isPartitionInvoked();
	}

	/**
	 * revisit: how it is enqueued to its output queue Creation date: (4/8/2003
	 * 1:39:56 PM)
	 * 
	 * @return java.util.List
	 */
	public List nestWithNest(List inputTuples, int inputPos) {
		List result = new ArrayList();

		int otherPartyPos;
		if (inputPos == 0)
			otherPartyPos = 1;
		else
			otherPartyPos = 0;

		XATQueue otherQueue = getNthInputQueue(otherPartyPos);

		/*
		 * if ((inputTuples.size()> 0) && (synopsis[otherPartyPos].size() > 0)) {
		 * //both not null, can output as subtuple
		 * 
		 * holderPassed = true; }
		 */

		// if (holderPassed) {
		XATSubTuple t = new XATSubTuple(getNthInputQueue(inputPos).getSchema()
				.getLength());
		//XATTuple copyRight = new XATTupleImp(rQueue.getSchema().getLength());

		//union all nested subtuples together into one
		for (int i = 0; i < inputTuples.size(); i++) {
			t.union(new XATSubTuple((XATTuple) inputTuples.get(i)));
		}

		synopsis[inputPos].enqueue(t, this);

		synopsis[otherPartyPos].resetCursor(this);

		for (int j = 0; j < synopsis[otherPartyPos].getQueueSize(); j++) {

			if (j > 1) {
				System.out
						.println("impossible, at most 1 tuple in synopsis for nested tuples");
				System.exit(-1);
			}

			XATTuple tSynop = (XATTuple) synopsis[otherPartyPos].dequeue(this);

			XATTuple newTuple;

			if (inputPos < otherPartyPos)
				newTuple = join(t, tSynop);
			else
				newTuple = join(tSynop, t);

			newTuple = join(t, tSynop);

			//oQueue.enqueue(newTuple, this);
			result.add(newTuple);
			holderPassed = true;

		}

		/*
		 * for (int j = 0; j < iRightNum; j++) {
		 * copyRight.union(rQueue.dequeue(this)); }
		 */

		//XATTuple[] tempInputTuples = new XATTuple[2];
		//tempInputTuples[0] = copyIncomingNest;
		//tempInputTuples[1] = copyRight;
		//XATTuple joinedTuple = projectToDesiredOutput(tempInputTuples);
		XATTuple joinedTuple = join(t, synopsis[otherPartyPos].dequeue(this));

		if (joinedTuple == null)
			//no result yet, return an empty result
			return result;

		holderPassed = true;

		if (getProperty("OUTPUT_TUPLE_GRANULARITY").equals(SUB_TUPLE)) {
			if (holderIncludesBegin) {
				//the holder itself holds the beginning of the tuple
				//enqueue it like a regular tuple
				//oQueue.enqueue(joinedTuple, this);
				result.add(joinedTuple);
				//dataDrivenChainRun();
				holderIncludesBegin = false;
			} else {
				//need to enqueue it using special method
				// ((XATSubTupleQueue) oQueue).enqueueSubTuple(joinedTuple,
				// this);
				result.add(joinedTuple);
				//dataDrivenChainRun();
			}
		} else {
			//output in tuple granularity
			System.out
					.println("make no sense: subtuple input queue, tuple output queue in InTimeSJ");
			if (subTupleHolder.getArrayData().length == 0) {
				//not set up yet
				subTupleHolder = new XATSubTuple(joinedTuple);
			} else {
				subTupleHolder.union(new XATSubTuple(joinedTuple));
			}
		}

		return result;

	}

	/**
	 * Insert the method's description here. Creation date: (4/8/2003 1:39:56
	 * PM)
	 * 
	 * @return java.util.List
	 */
	public List nestWithUnnest(List inputTuples, int inputPos) {
		List result = new ArrayList();

		int otherPartyPos;
		if (inputPos == 0)
			otherPartyPos = 1;
		else
			otherPartyPos = 0;

		XATQueue otherQueue = getNthInputQueue(otherPartyPos);

		/*
		 * if ((inputTuples.size()> 0) && (synopsis[otherPartyPos].size() > 0)) {
		 * //both not null, can output as subtuple
		 * 
		 * holderPassed = true; }
		 */

		// if (holderPassed) {
		XATSubTuple t = new XATSubTuple(new XATTupleImp(getNthInputQueue(
				inputPos).getSchema().getLength()));
		//XATTuple copyRight = new XATTupleImp(rQueue.getSchema().getLength());

		//union all nested subtuples together into one
		for (int i = 0; i < inputTuples.size(); i++) {
			t.union(new XATSubTuple((XATTuple) inputTuples.get(i)));
		}

		synopsis[inputPos].enqueue(t, this);

		synopsis[otherPartyPos].resetCursor(this);

		for (int j = 0; j < synopsis[otherPartyPos].getQueueSize(); j++) {

			XATTuple tSynop = (XATTuple) synopsis[otherPartyPos].dequeue(this);

			XATTuple newTuple;

			if (inputPos < otherPartyPos)
				newTuple = join(t, tSynop);
			else
				newTuple = join(tSynop, t);

			newTuple = join(t, tSynop);

			//oQueue.enqueue(newTuple, this);
			result.add(newTuple);
			holderPassed = true;

		}

		if (result.size() == 0)
			//no result yet, return an empty result
			return result;

		holderPassed = true;

		if (getProperty("OUTPUT_TUPLE_GRANULARITY").equals(SUB_TUPLE)) {
			if (holderIncludesBegin) {
				//the holder itself holds the beginning of the tuple
				//enqueue it like a regular tuple
				//oQueue.enqueue(joinedTuple, this);
				//result.add(joinedTuple);
				//dataDrivenChainRun();
				holderIncludesBegin = false;
			} else {
				//need to enqueue it using special method
				// ((XATSubTupleQueue) oQueue).enqueueSubTuple(joinedTuple,
				// this);
				//result.add(joinedTuple);
				//dataDrivenChainRun();
			}
		} else {
			//output in tuple granularity
			System.out
					.println("make no sense: subtuple input queue, tuple output queue in InTimeSJ");
			/*
			 * if (subTupleHolder.getData().size() == 0) { //not set up yet
			 * subTupleHolder = joinedTuple; } else {
			 * subTupleHolder.union(joinedTuple); }
			 */
		}

		return result;
	}

	/**
	 * Insert the method's description here. Creation date: (2/20/2003 10:06:57
	 * PM)
	 */
	public void resumeRegularComp() {

		if (!(shortCircuitedOps == null)) {

			if (!(shortCircuitedOps.size() == 0)) {
				//there are ops short circuited
				for (int i = 0; i < shortCircuitedOps.size(); i++) {

					ExtractStream extractOp = (ExtractStream) shortCircuitedOps
							.get(i);

					ExtendedAutomata auto = extractOp.getAutomata();

					List states = auto
							.getFrStateHash((XATStreamOperator) extractOp);

					for (Iterator it = states.iterator(); it.hasNext();) {
						ExtendedState state = (ExtendedState) it.next();
						state.resumeExtractOp();
					}

				}

				shortCircuitedOps.clear();
			}
		}
	}

	/**
	 * Runs the actual operator.
	 * 
	 * revisit: do not copy to a workspace, instead access data instead of
	 * dequeue Creation date: (2/2/2003 10:06:28 PM)
	 */
	public boolean run() {
		allInvokingModes.run();
		return true;

	}

	/**
	 * Insert the method's description here. Creation date: (2/2/2003 7:02:16
	 * PM)
	 * 
	 * @param newBranchNavOp
	 *            edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAuto
	 */
	public void setBranchNavOp(PureNavigateAutoStream newBranchNavOp) {
		//if (newBranchNavOp instanceof PureNavigateAutoStreamWithRecur)
		branchNavOp = newBranchNavOp;
		//else 
		//	branchNavOp = newBranchNavOp;
	}
	
	public void setBranchNavOp(PureNavigateAutoStreamWithRecur newBranchNavOp) {
		branchNavOp =(PureNavigateAutoStream) newBranchNavOp;
	}

	/**
	 * Insert the method's description here. Creation date: (2/19/2003 10:12:36
	 * PM)
	 * 
	 * @param newShortCircuit
	 *            boolean
	 */
	public void setShortCircuit(boolean newShortCircuit) {
		shortCircuit = newShortCircuit;
		//initialize the shortCicruite manager as well
		shortCircuitMgr = new Hashtable();
	}

	/**
	 * Insert the method's description here. Creation date: (2/20/2003 7:34:03
	 * PM)
	 * 
	 * @param relatedOps
	 *            java.util.ArrayList
	 */
	public void shortCircuitOp(XATStreamOperator op) {

		if (!(op instanceof ExtractStream)) {
			System.out
					.println("Impossible: an short circuit source operator is not an extract operator");
		} else {
			ExtractStream extractOp = (ExtractStream) op;

			ExtendedAutomata auto = extractOp.getAutomata();

			List states = auto.getFrStateHash(op);

			for (Iterator it = states.iterator(); it.hasNext();) {
				ExtendedState state = (ExtendedState) it.next();

				state.suspendExtractOp();
			}

			if (shortCircuitedOps == null) {
				shortCircuitedOps = new ArrayList();
				shortCircuitedOps.add(op);
			} else {
				shortCircuitedOps.add(op);
			}

		}

	}

	/**
	 * Insert the method's description here. Creation date: (2/14/2003 9:45:43
	 * AM)
	 * 
	 * @return java.lang.String
	 */
	public String toString() {
		String rs = new String();

		if (this instanceof SingletonInTimeSJStreamImp)
			rs = "SingletonSJ";
		else
			rs = "InTimeSJ";

		rs = rs + " on " + getBranchNavOp().getOutputColumnName();

		rs = rs + " with schema: ";

		for (int i = 0; i < outputQueue.size(); i++) {
			XATQueue queue = (XATQueue) outputQueue.get(i);

			XATQueueSchema schema = queue.getSchema();

			if (schema != null) {
				Vector v = schema.getColNames();

				for (int j = 0; j < v.size(); j++) {
					rs = rs + (String) v.get(j);
				}
			}
		}

		/*
		 * if (getProperty("DATA_DRIVEN_INVOKING").equals("1")) rs = rs + " data
		 * driven"; else rs = rs + " not data driven";
		 */

		//	if (getProperty("OUTPUT_TUPLE_GRANULARITY").equals(SUB_TUPLE))
		//		rs = rs + " *subtuple output queue";
		return rs;

	}

	/**
	 * unnest pipelining in, join with existing subtuples Creation date:
	 * (4/8/2003 1:46:33 PM) revisit: what if the other party is not data-driven
	 * 
	 * @return java.util.List
	 */
	public List unnestWithNest(List inputTuples, int inputPos) {
		List result = new ArrayList();

		int otherPartyPos;
		if (inputPos == 0)
			otherPartyPos = 1;
		else
			otherPartyPos = 0;

		XATQueue otherQueue = getNthInputQueue(otherPartyPos);

		//int existNum = otherQueue.getQueueSize(this);
		//int iRightNum = rQueue.getQueueSize(this);

		/*
		 * if ((inputTuples.size() > 0) && (existNum > 0)) { //both not null,
		 * can output as subtuple
		 * 
		 * holderPassed = true; }
		 */

		// if (holderPassed) {
		for (int i = 0; i < inputTuples.size(); i++) {

			XATTuple t = (XATTuple) inputTuples.get(i);

			synopsis[inputPos].enqueue(t, this);

			synopsis[otherPartyPos].resetCursor(this);

			for (int j = 0; j < synopsis[otherPartyPos].getQueueSize(); j++) {

				if (j > 1) {
					System.out
							.println("impossible, at most 1 tuple in synopsis for nested tuples");
					System.exit(-1);
				}

				XATTuple tSynop = (XATTuple) synopsis[otherPartyPos]
						.dequeue(this);

				XATTuple newTuple;

				if (inputPos < otherPartyPos)
					newTuple = join(t, tSynop);
				else
					newTuple = join(tSynop, t);

				//oQueue.enqueue(newTuple, this);
				result.add(newTuple);
				holderPassed = true;

			}
		}

		return result;
	}

	/**
	 * unnest pipelining in, join with existing tuples Creation date: (4/8/2003
	 * 1:39:23 PM)
	 * 
	 * @return java.util.List
	 */
	public List unnestWithUnnest(List inputTuples, int inputPos, int inputNum) {
		List result = new ArrayList();

		/*
		 * boolean synopisAllNonNull = true;
		 * 
		 * for (int i = 0; i < inputNum; i++) if (i != inputPos) {
		 * synopsis[i].resetCursor(this); if (synopsis[i].getQueueSize() == 0)
		 * synopisAllNonNull = false; }
		 */

		// if (!synopisAllNonNull) {
		//nothing to join with the other party right now
		//enqueue the new incoming tuples in one batch instead of one by one
		synopsis[inputPos].enqueue(inputTuples, this);
		return result;
		// }

		//there exists something to join
		//  synopsis[inputPos].enqueue(inputTuples, this);

		/*
		 * List[] joinTuples = new List[inputNum];
		 * 
		 * for (int i = 0; i < inputNum; i++) if (i != inputPos) joinTuples[i] =
		 * synopsis[i].dequeueN(synopsis[i].getQueueSize(), this); else
		 * joinTuples[i] = inputTuples;
		 * 
		 * result = join(joinTuples);
		 */

		/*
		 * if (inputPos < otherPartyPos) result = join(inputTuples,
		 * synopsis[otherPartyPos].dequeueN(otherPartySynopSize, this)); else
		 * result = join(synopsis[otherPartyPos].dequeueN(otherPartySynopSize,
		 * this), inputTuples);
		 */

		//holderPassed = true;
		//return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.AutomataEnabled#cleanTemp()
	 */
	public void cleanTemp() {
		//clean those intermeditate results that haven't been sent to
		// outputqueue yet
		//result.clear();
		((XATQueueImp) outputQueue.get(0)).dequeueAll();
		//commented out for schema-blindless case
		//partitionChecked = false;

	}

	public boolean selfPartitionDelimiInvokingRun(List partitionStates,
			SuspensionEdge suspendComp) {
		return allInvokingModes.selfPartitionDelimiInvokingRunForSJ(
				partitionStates, suspendComp);
	}
	
	public boolean getPartitionChecked() {
		return partitionChecked;
	}

	public void setPartitionChecked(boolean partitionChecked) {

		this.partitionChecked = partitionChecked;
	}

	public List[] demandDrivenInvokingRun() {
		return allInvokingModes.demandDrivenInvokingRun();
	}

	public List[] demandDrivenInvokingRunWithOrder() {
		return allInvokingModes.demandDrivenInvokingRunWithOrder();

	}

	public List runGroupBy() {
		return runGroupByWithOrder();
	}

	/*
	 * three steps: step1: check entry ops step2: evaluate each branch in turn
	 * step3: perform local join
	 */
	public List runGroupByWithOrder() {
		timeKeeper.start();
		timeKeeper.increInputCount(1);

		List result = new ArrayList();	

		boolean onefailed = false;

		int size = entryOps.size();

		//branchNavOp generates sth that triggers the invocation of the
		// structural join
		branchNavOp.getTimeKeeper().setNotEmpty();

		List entryHasOutput = new ArrayList();
		//step1:
		for (int i = 0; i < size; i++) {
			XATStreamOperator entryOp = (XATStreamOperator) entryOps.get(i);

			if (entryOp instanceof ExtractStream) {
				List output = ((ExtractStream) entryOp).runGroupBy();
				if (output.size() > 0) {
					((XATQueueImp) entryOp.getOutputQueue()).enqueue(output);
					entryHasOutput.add(entryOp);
				} else
					onefailed = true;
			} else {
				//entry op must be a structural join
				PureNavigateAutoStream tokenNav = ((InTimeSJStream) entryOp)
						.getBranchNavOp();
				ClassExecTimeKeeper tokenNavKeeper = tokenNav.getTimeKeeper();
				if (tokenNavKeeper.getNotEmpty()) {
					//e.g., Sjoin $a
					//        / \
					//               Sjoin $b
					//                  ...
					//               TokenNav $a,p->$b
					//           ...
					//         TokenNav$s, p'->$a
					//if TokenNav$a,p->$b generates sth when Sjoin$a is invoked
					tokenNavKeeper.increNotEmptyCount();
					tokenNavKeeper.resetNotEmpty();
				}

				if (((XATQueueImp) entryOp.getOutputQueue()).getQueueSize() > 0) {
					/*((InTimeSJStream) entryOp).getTimeKeeper()
							.increNotEmptyCount();*/
					entryHasOutput.add(entryOp);
				} else
					onefailed = true;
			}
		}

		boolean needCleanUp = true;
		if (!onefailed) {
			//all entry operators have output
			//then continue to work on the branches
			//step 2:
			List inputs[] = demandDrivenInvokingRunWithOrder();
			//step 3:
			if (inputs != null){
  			result = join(inputs);
				needCleanUp = false;
			}
		}
		
		if (needCleanUp){
//			clean up all 
			int nonEmptySize = entryHasOutput.size();
			for (int i=0; i<nonEmptySize; i++){
				XATStreamOperator entryOp = (XATStreamOperator)entryHasOutput.get(i);
				XATQueueImp outputQueue = (XATQueueImp)entryOp.getOutputQueue();
				outputQueue.cleanAll();
			}
		}
				
		timeKeeper.increOutputCount(result.size());
		timeKeeper.pause();
		//System.out.println("old join time"+  timeKeeper.getTime());
		return result;

	}

	public List run(List inputTuples, int inputPos) {
		
		return runGroupByWithOrder();
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.AutomataEnabled#runPushStack()
	 */
	public void runPushStack(int tokenId, int startTokenId) {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.AutomataEnabled#runPopStack()
	 */
	public void runPopStack(int tokenId, int startTokenId) {

	}

	public boolean needSetPartitionCheck() {
		return needSetPartitionCheck;
	}

	public void setNeedSetPartitionCheck() {
		needSetPartitionCheck = true;
	}

	public List getBranchEvalOrder() {
		return branchEvalOrder;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.InTimeSJStream#getEntryOps()
	 */
	public List getEntryOps() {
		return entryOps;
	}

}