package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.DataDrivenInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.common.xmlstreamexpression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore;

import java.util.*;

/**
 * Insert the type's description here. Creation date: (1/20/2003 12:17:57 PM)
 * 
 * @author: Luping Ding
 */

public abstract class JoinStreamImp extends CartesianStream implements
		XATStreamOpRunCore, DataDrivenInvoking {  
	protected ClassExecTimeKeeper timeKeeper;
	
	protected ExpressionStr expression;

	//we assume the join expression is a primitive one
	//joinColPos[0] is the position of the join col in the first input src
	//joinColPos[1] is the position of the join col in the second input src
	public int[] joinColPos;
	/**
	 * Insert the method's description here. Creation date: (1/21/2003 1:01:06
	 * PM)
	 */
	public JoinStreamImp(DagNode node, ExpressionStr exp) {
		super(node);
		expression = exp;
	}

	/**
	 * Insert the method's description here. Creation date: (1/21/2003 1:01:06
	 * PM)
	 */
	public ExpressionStr getExpression() {
		return expression;
	}

	/**
	 * Insert the method's description here. Creation date: (4/6/2003 2:50:58
	 * PM)
	 * 
	 * @return int[]
	 */
	public int[] getJoinColPos() {
		if (joinColPos == null) {
			//not set yet
			for (Iterator i = expression.getTerminalColumnNames().iterator(); i
					.hasNext();) {
				String colName = (String) i.next();
				try {
					int lJoinPos = getLeftInputQueue().getSchema().getPos(
							colName);
					joinColPos[0] = lJoinPos;
				} catch (NoSuchElementException e1) {
					//not in left, must in right
					try {
						int rJoinPos = getRightInputQueue().getSchema().getPos(
								colName);
						joinColPos[1] = rJoinPos;
					} catch (NoSuchElementException e2) {
						System.out.println("terminal expression not valid");
						System.exit(-1);
					}
				}
			}
		}
		return joinColPos;
	}

	public XATTuple join(XATTuple lTuple, XATTuple rTuple, int[][] colPos) {
		XATDataValue[] newData = new XATDataValueImp[colPos[0].length
				+ colPos[1].length];
		((XATTupleImp) lTuple).fillDataInListTo(newData, colPos[0], 0);
		((XATTupleImp) rTuple).fillDataInListTo(newData, colPos[1],
				colPos[0].length);

		return new XATTupleImp(newData);
	}

	public void setExpression(ExpressionStr exp) {
		expression = exp;
	}

	/**
	 * Insert the method's description here. Creation date: (2/14/2003 9:33:59
	 * AM)
	 * 
	 * @return java.lang.String
	 */
	public String toString() {
		String rs = new String();

		if (this instanceof InTimeSJStream) {
			rs = "InTimeSJ";
		} else {
			if (this instanceof JoinStreamImp) {
				rs = "JoinStreamImp";
			}
		}

		rs = rs + expression;

		rs = rs + " with schema: ";

		for (int i = 0; i < outputQueue.size(); i++) {
			XATQueue queue = (XATQueue) outputQueue.get(i);
			Vector v = queue.getSchema().getColNames();

			for (int j = 0; j < v.size(); j++) {
				rs = rs + (String) v.get(j);
			}
		}

		return rs;

	}

	public void cleanTemp() {
		getOutputQueue().removeAllTuples();
	}
	
	public ClassExecTimeKeeper getTimeKeeper(){
		return timeKeeper;
	}

	
}