package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator;

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;
/**
 * Insert the type's description here.
 * Creation date: (4/4/2003 9:53:41 PM)
 * @author: Hong Su
 */

 
public class PostponedSJStreamImp extends CartesianStream 
			implements PostponedSJStream{
	public AllInvokingModeImp allInvokingModes;

	private int leftTargetPos;
	private int rightTargetPos;

	private List leftBuffer = new ArrayList();
	private List rightBuffer = new ArrayList();
	
/**
 * CartesioanExecStream constructor comment.
 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public PostponedSJStreamImp(edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode node) {
	super(node);
	allInvokingModes = new AllInvokingModeImp(this);
}
/**
 * output tuples of an operator A implementing this interface are passed as parameters  
 * to the parent operator B, therefore B does not need to fetch from the queue
 * this is a pipeling manner in the sense that all intermediate data is directly passed
 * to be consumed rather than stored (neither in second storage nor memory)
  
 * Creation date: (4/8/2003 1:23:12 PM)
 */
public boolean dataDrivenInvokingRun(java.util.List inputTuples, int inputPos) {
   return	allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);
	}
/**
 * Insert the method's description here.
 * Creation date: (6/3/2003 11:40:13 PM)
 * @return java.util.List
 */
public List generateTuples() {

    List result = new ArrayList();

    //init
    if (leftBuffer.isEmpty() || rightBuffer.isEmpty())
        return result;

    int curLeftId = -1;
    int curRightId = -1;

    XATTuple curLeftTuple = null;
    XATTuple curRightTuple = null;

    //clean buffer until at least one buffer is empty
    boolean runLeft = true;
    while (!leftBuffer.isEmpty() && !rightBuffer.isEmpty()) {

        if (runLeft) {
            //init right hand side
            if (curRightId == -1) {
                //curRightTuple = (XATTuple) rightBuffer.get(0);
                //this.rightBuffer.remove(0);
                curRightTuple = (XATTuple)rightBuffer.remove(0);
                curRightId =
                    curRightTuple.getCell(rightTargetPos).convertToDouble().intValue();
            }

            //curLeftTuple = (XATTuple) this.leftBuffer.get(0);
			//this.leftBuffer.remove(0);
			curLeftTuple = (XATTuple)leftBuffer.remove(0);
			
            curLeftId =
                curLeftTuple.getCell(leftTargetPos).convertToDouble().intValue();

            if (curLeftId < curRightId){
                continue;
            }
            if (curLeftId == curRightId) {
                //generate new tuple from leftTuple and rightTuple
                XATDataValue[] resultData =
                    new XATDataValue[getOutputQueue().getSchema().getLength()];

                int[][] temp = getOutputPositions();

                ((XATTupleImp) curLeftTuple).fillDataInListTo(resultData, temp[0], 0);

                ((XATTupleImp) curRightTuple).fillDataInListTo(
                    resultData,
                    temp[1],
                    temp[0].length);

                result.add(new XATTupleImp(resultData));
            }
            else{
	            runLeft = false;
            }
        }
        //run right hand side
        else {
            //init right hand side
            if (curLeftId == -1) {
                //curLeftTuple = (XATTuple) this.leftBuffer.get(0);
				//this.leftBuffer.remove(0);
				curLeftTuple = (XATTuple)leftBuffer.remove(0);
                curLeftId =
                    curLeftTuple.getCell(this.leftTargetPos).convertToDouble().intValue();
            }

           // curRightTuple = (XATTuple) this.rightBuffer.get(0);
		   //	this.rightBuffer.remove(0);
		    curRightTuple = (XATTuple)rightBuffer.remove(0);
            curRightId =
                curRightTuple.getCell(this.rightTargetPos).convertToDouble().intValue();

            if (curRightId < curLeftId){
                continue;
            }
            if (curRightId == curLeftId) {
                //generate new tuple from rightTuple and leftTuple
                XATDataValue[] resultData =
                    new XATDataValue[getOutputQueue().getSchema().getLength()];

                int[][] temp = getOutputPositions();

                ((XATTupleImp) curLeftTuple).fillDataInListTo(resultData, temp[0], 0);

                ((XATTupleImp) curRightTuple).fillDataInListTo(
                    resultData,
                    temp[1],
                    temp[0].length);

                result.add(new XATTupleImp(resultData));
            }
            else{
	            runLeft = true;
            }
        }
    }
    return result;
}
/**
 * Insert the method's description here.
 * Creation date: (4/4/2003 9:57:50 PM)
 */
public boolean run() {
    allInvokingModes.run();
    return true;

}
/**
 * Pipelining version
 * hack: only handle a 2-way join
 * Creation date: (4/8/2003 11:25:52 AM)
 * @return java.util.List
 * @param multiSrcInputTuples java.util.List[] each List in the array is input tuples from a source
 * @exception java.lang.Exception The exception description.
 */
public java.util.List run(java.util.List inputTuples, int inputPos){

    int size = inputTuples.size();
	    
    for (int num = 0; num < size; num++) {

        //tuple from left
        if (inputPos == 0) {
	        leftBuffer.add(inputTuples.get(num));
        }
        //tuple from right
        else {
	        rightBuffer.add(inputTuples.get(num));
        }
    }

    return generateTuples();

}
/**
 * Insert the method's description here.
 * Creation date: (6/4/2003 12:53:05 AM)
 * @param leftPos int
 */
public void setLeftJoinPosition(int leftPos) {
	this.leftTargetPos = leftPos;	
}
/**
 * Insert the method's description here.
 * Creation date: (6/4/2003 12:53:19 AM)
 * @return int
 * @param rightPos int
 */
public void setRightJoinPosition(int rightPos) {
	this.rightTargetPos = rightPos;
}
/**
 * Insert the method's description here.
 * Creation date: (6/4/2003 12:32:31 AM)
 * @return java.lang.String
 */
public String toString() {
	StringBuffer sb = new StringBuffer();
	sb.append("PostpondSJStreamImp \n");
	sb.append("    join on column:");
	sb.append(this.leftTargetPos);
	sb.append(" at the left hand side,\n");
	sb.append("    and on column:");
	sb.append(this.leftTargetPos);
	sb.append(" at the right hand side,\n");
	return sb.toString();
}
}
