package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xmlstreamexpression.ExpressionStr;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.schedulemode
	.AllInvokingModeImp;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.schedulemode
	.DataDrivenInvoking;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.schedulemode
	.DemandDrivenInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;

import java.util.*;

/**
 * This is a general select implementation, regardless of the column's type (i.e., flat or collection)
 * and the predicate's type (i.e., simple or join)
 * Creation date: (1/28/2003 1:54:33 PM)
 * @author: Luping Ding
 */
public class SelectStreamImp
	extends XATSingleQueueStreamOperatorImp  
	implements
		SelectStream,
		DataDrivenInvoking,
		DemandDrivenInvoking {

	protected ExpressionStr expression;
	public int dequeueStrategy;
	public static final int DEQUEUE_AVAILABLE = 1;
	public static final int DEQUEUE_ONE = 2;
	protected ClassExecTimeKeeper timeKeeper;

	public AllInvokingModeImp allInvokingModes;

	List branchEvalOrder;
	
	boolean needSetPartitionCheck = false;
	/**
	 * StreamSelectImp constructor comment.
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 */
	public SelectStreamImp(DagNode node) {
		super(node);
		allInvokingModes = new AllInvokingModeImp(this);
		timeKeeper = new ClassExecTimeKeeper(this);
		branchEvalOrder = new ArrayList();
		branchEvalOrder.add(new Integer(0));
	}
	/**
	 * SelectStreamImp constructor comment.
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 */
	public SelectStreamImp(XATNode node, ExpressionStr expression) {
		super(node);
		this.expression = expression;
		//temperal solution: operator controls the dequeue strategy
		dequeueStrategy = DEQUEUE_AVAILABLE;
		allInvokingModes = new AllInvokingModeImp(this);
		timeKeeper = new ClassExecTimeKeeper(this);
	}

	public AllInvokingModeImp getAllInvokingModes(){
		return allInvokingModes;
	}
	
	public ExpressionStr getExpression() {
		return expression;
	}

	/**
	 * Runs the actual operator.
	 *
	 * Unlike in Rainbow, the Stream operators have the implementation tied to the operator
	 * because the StreamOperators represent physical operators instead of Logic.
	 * Creation date: (1/28/2003 1:54:33 PM)
	 */
	public boolean run() throws Exception {
		allInvokingModes.run();
		return true;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (2/1/2003 8:57:52 PM)
	 * @param newExpression edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.ExpressionStr
	 */
	public void setExpression(ExpressionStr newExpression) {
		expression = newExpression;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/12/2003 4:56:27 PM)
	 * @return java.lang.String
	 */
	public String toString() {
		String rs = new String();

		if (this instanceof FlatSelectStreamImp) {
			rs = "FlatSelect:";
		} else {
			if (this instanceof CollSimpleSelStream) 
				rs = "CollectionSimpleSel:";
			else if (this instanceof SelectStreamImp) 
						rs = "SelectStream:";
		}		

		rs = rs + expression;

		/*	rs = rs + " |schema: ";
		
			for (int i = 0; i < outputQueue.size(); i++) {
				XATQueue queue = (XATQueue) outputQueue.get(i);
				XATQueueSchema schema = queue.getSchema();
		
				if (schema != null) {
					Vector v = schema.getColNames();
		
					for (int j = 0; j < v.size(); j++) {
						rs = rs + (String) v.get(j);
					}
				}
			}*/

		if (getProperty("OUTPUT_TUPLE_GRANULARITY").equals(SUB_TUPLE))
			rs = rs + " *subtuple output";

		return rs;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperatorImp#init()
	 */
	public void init() {

	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#toToolTipString()
	 */

	public String toToolTipString() {

		return null;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATSingleQStreamOpRunCore#run(java.util.List)
	 */
	public List run(List inputTuples, int inputPos) {
		//record how much time have been spent on this method
		//	timeKeeper.start();
		int inputNum = inputTuples.size();
		timeKeeper.increInputCount(inputNum);

		List result = new ArrayList();

		for (int i=0; i<inputNum; i++) {

			XATTuple t = (XATTuple) inputTuples.get(i);
			List l = new ArrayList();
			l.add(t);

			Object eval = getExpression().eval(l, this);
			if (eval instanceof Boolean) {
				if (((Boolean) eval).booleanValue()) {

					result.add(projectToDesiredOutput(t));
					timeKeeper.increOutputCount(1);
				}
			}
		} //end of for

		//timeKeeper.pause();

		return result;
	}
	
	public boolean dataDrivenInvokingRun(List inputTuples, int inputPos) {
		return allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);

	}
	
	public boolean getPartitionChecked() {
		return allInvokingModes.getPartitionChecked();
	}
	
	public void setPartitionChecked(boolean partitionChecked) {
		allInvokingModes.setPartitionChecked(partitionChecked);
	}
	
	public boolean selfPartitionDelimiInvokingRun(
		List partitionStates,
		SuspensionEdge edge) {
        //should not be used in current version
		//since extractnest underneath it will be called selfPartitiondelimiInvoking instead
		System.out.println("should not reach select.selectPartitionDelimiInvokingRun here");
		return allInvokingModes.selfPartitionDelimiInvokingRunForSJ(
			partitionStates,
			edge);
	}
	
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAllAccumulatedTuples()
	 */
	public boolean cleanAllAccumulatedTuples() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#matchStates(edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator)
	 */
	public boolean matchStates(XATStreamOperator op) {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasUnmatchedState()
	 */
	public boolean hasUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#recomputeUnmatchedState()
	 */
	public boolean recomputeUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setStateID(int)
	 */
	public int setStateID(int startID) {
		// TODO Auto-generated method stub
		return 0;
	}

	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}
	
	public List[] demandDrivenInvokingRun() {
		return allInvokingModes.demandDrivenInvokingRun();
	}
	
	
	public boolean lightweightDataDrivenInvokingRun(List inputTuples, int inputPos) {
		return allInvokingModes.lightweightDataDrivenInvokingRun(inputTuples, inputPos);
	}
	
	public boolean needSetPartitionCheck() {
			return needSetPartitionCheck;
	}
	
	public void setNeedSetPartitionCheck() {
		needSetPartitionCheck = true;
		
	}
	
	public List getBranchEvalOrder() {
		return branchEvalOrder;
	}
	
	public List[] demandDrivenInvokingRunWithOrder() {
		return allInvokingModes.demandDrivenInvokingRunWithOrder();
	}
	
	public void setBranchEvalOrder(List branchEvalOrder) {
		this.branchEvalOrder = branchEvalOrder;
		
	}
}
