/*
 * structural join_{$v} operator that no input branch contains ExtractUnnest_{\$v, p} \$v'
 * or NavUnnest_{\$v, p} \$v'
 */
package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator;

import java.util.ArrayList;
import java.util.List;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.SelfPartitionDelimiInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedAutomata;

public class SingletonInTimeSJStreamImp extends InTimeSJStreamImp {

	public SingletonInTimeSJStreamImp(DagNode node) {
		super(node);
	}

	public void associateWithAuto(ExtendedAutomata auto) {
		super.associateWithAuto(auto);
		int tupleLength = getTupleLength();

		XATDataValue[] newData = new XATDataValue[tupleLength];
			
		//set up default branch evaluation order, from left to right
		int childrenSize = getNode().getChildrenSize();
		for (int i=0; i<childrenSize; i++)
			branchEvalOrder.add(new Integer(i));
	}


	public List join(List[] tuples) {
		//each input source contains only one tuple
		int[][] temp = getOutputPositions();

		List outputTuples = new ArrayList();

		int tupleLength = getTupleLength();

		XATDataValue[] newData = new XATDataValue[tupleLength];
		int startPos = 0;
		int[] inputsInResults = getUsefulInputSrc();
		int inputsInResultsLength = inputsInResults.length;

		for (int j = 0; j < inputsInResultsLength; j++) {
			int inputSrc = inputsInResults[j];

			//there is something in source j that needs to be filled
			((XATTupleImp) tuples[inputSrc].get(0)).fillDataInListTo(
				newData,
				temp[inputSrc],
				startPos);
			startPos = startPos + temp[inputSrc].length;

		}

		outputTuples.add(new XATTupleImp(newData));

		return outputTuples;

	}

	public List runGroupBy() {
		return runGroupByWithOrder();
	}
	
	public List runGroupByWithOrder(){
		timeKeeper.start();	
		timeKeeper.increInputCount(1);
		
		List result = new ArrayList();

		boolean onefailed = false;

		int size = entryOps.size();

		//branchNavOp generates sth that triggers the invocation of the
		// structural join
		branchNavOp.getTimeKeeper().setNotEmpty();
		
		List entryHasOutput = new ArrayList();

		//step1:
		for (int i = 0; i < size; i++) {
			XATStreamOperator entryOp = (XATStreamOperator) entryOps.get(i);
			
			//set checked flag
			((SelfPartitionDelimiInvoking) entryOp).setPartitionChecked(true);

			if (entryOp instanceof ExtractStream) {
				List output = ((ExtractStream) entryOp).runGroupBy();
				if (output.size() > 0) {
					entryHasOutput.add(entryOp);
					/*PureNavigateAutoStream navOp = (PureNavigateAutoStream) entryOp
							.getNode().getChild(0).getOperator();
					if (navOp != branchNavOp)
						navOp.getTimeKeeper().increNotEmptyCount();*/
					((XATQueueImp) entryOp.getOutputQueue()).enqueue(output);
				} else
					onefailed = true;
			} else {
				//entry op must be a structural join
				PureNavigateAutoStream tokenNav = ((InTimeSJStream) entryOp)
						.getBranchNavOp();
				ClassExecTimeKeeper tokenNavKeeper = tokenNav.getTimeKeeper();
				if (tokenNavKeeper.getNotEmpty()) {
					//e.g., Sjoin $a
					//        / \
					//               Sjoin $b
					//                  ...
					//               TokenNav $a,p->$b
					//           ...
					//         TokenNav$s, p'->$a
					//if TokenNav$a,p->$b generates sth when Sjoin$a is invoked
					tokenNavKeeper.increNotEmptyCount();
					tokenNavKeeper.resetNotEmpty();
				}

				if (((XATQueueImp) entryOp.getOutputQueue()).getQueueSize() > 0) {
					entryHasOutput.add(entryOp);
				/*	((InTimeSJStream) entryOp).getTimeKeeper()
							.increNotEmptyCount();*/
				} else
					onefailed = true;
			}
		}

		boolean needCleanUp = true;
		if (!onefailed){
			//all entry operators have output
			//then continue to work on the branches
			//steps 2 and 3 combined together:
			result = allInvokingModes.demandDrivenInvokingRunWithOrder(getUsefulInputSrc(),tupleLength);
			if (result.size()>0)
				needCleanUp = false;
		}
		
		if (needCleanUp){
			//clean up all 
			int nonEmptySize = entryHasOutput.size();
			for (int i=0; i<nonEmptySize; i++){
				XATStreamOperator entryOp = (XATStreamOperator)entryHasOutput.get(i);
				((XATQueueImp)entryOp.getOutputQueue()).cleanAll();
			}
		}
	
		timeKeeper.increOutputCount(result.size());
		timeKeeper.pause();
		
		return result;	
		
	}

}
