package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.FindOpVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.sqloperator
	.InTimeSJStream;

import java.util.*;

import org.w3c.dom.Element;
import edu.wpi.cs.dsrg.xmldb.xat.testing.TestExecBaseLocal;
//import org.xml.sax.*;

/**a separate tuple for each item extracted
 * Creation date: (1/15/2003 7:59:32 PM)  
 * @author: Hong Su
 */
public class ExtractByUnnestStream  
	extends NavigateStreamImp
	implements ExtractStream 
	{
	boolean needSetPartitionCheck = false;
	//this op is done in the automata
	protected ExtendedAutomata automata;

	//for common methods to implement extractstream
	protected ExtractStreamImp extractCommon;

	//storage manager for extracted items
	protected Finer_StreamStoreMgr storeMgr;

	//different physical representations for items
	protected int[] storingMethods = { 0, 0, 0, 0, 0 };

    //holding the content of the newestSubTuple
	protected XATKeyDataValue[] tupleContentHolder;

	//reference to the item in the storage manager
	protected Finer_StreamStoreMgrKey typedTupleKey;

	//the state whose activation leads to the beginning of extraction
	protected ExtendedState opState;
	protected ClassExecTimeKeeper timeKeeper;
	protected int numberOfTuplesToEnqueue = 1;
	protected ArrayList tempResult = new ArrayList();
	//for the purpose of using occurrence constraints
	private int itemCountWithinEntry = 0;
	protected boolean partitionChecked = false;

	private int outputCount;

	protected int posDom = 0;
	protected int posType = 0;
	protected int posAttr = 0;
	protected int posNodeId = 0;

	public AllInvokingModeImp allInvokingModes = new AllInvokingModeImp(this);

	/**
	 * NavUnnestAutoImp constructor comment.
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 * @param ep edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavigationStep[]
	 * @param d edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavigationStep[]
	 */
	public ExtractByUnnestStream(
		XATNode node,
		NavigationStep[] ep,
		NavigationStep[] dest) {
		super(node, ep, dest);

		extractCommon = new ExtractStreamImp(this);

		timeKeeper = new ClassExecTimeKeeper(this);

	}
	

	public void setUpCellNum() {
		tupleContentHolder = new XATKeyDataValue[getTupleColNum()];

	}

	/**associate the operator with automata
	 * Creation date: (1/29/2003 9:03:25 PM)
	 * @param auto edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.ExtendedAutomata
	 */
	public void associateWithAuto(ExtendedAutomata auto) {

		this.automata = auto;

		extractCommon.associateWithAuto(auto);

		//commented out for using demandriven from sj
		List ances = FindOpVisitor.findAncestorOps((DagNode) getNode());
		boolean hasSJAnces = false;
		for (Iterator it = ances.iterator(); it.hasNext();) {
			XATOperator anceOp = (XATOperator) it.next();
			if (anceOp instanceof InTimeSJStream) {
				hasSJAnces = true;
				break;
			}

		}

		if (!hasSJAnces) {

			PureNavigateAutoStream navOp =
				(PureNavigateAutoStream) getNode().getChild(0).getOperator();

			List states = auto.getFrStateHash(navOp);

			for (Iterator it = states.iterator(); it.hasNext();) {
				ExtendedState state = (ExtendedState) it.next();

				state.addGroupbyOp(this);
			}
		}

		for (int i = 0; i < ExtractStream.TYPED_VALUE; i++) {
			if (storingMethods[i] == 1)
				posType++;
		}

		for (int i = 0; i < ExtractStream.DOM; i++) {
			if (storingMethods[i] == 1)
				posDom++;
		}

		for (int i = 0; i < ExtractStream.ATTRS; i++) {
			if (storingMethods[i] == 1)
				posAttr++;
		}

		for (int i = 0; i < ExtractStream.NODE_ID; i++) {
			if (storingMethods[i] == 1)
				posNodeId++;
		}

	}

	/**
	 * output tuples of an operator A implementing this interface are passed as parameters  
	 * to the parent operator B, therefore B does not need to fetch from the queue
	 * this is a pipeling manner in the sense that all intermediate data is directly passed
	 * to be consumed rather than stored (neither in second storage nor memory)
	  
	 * Creation date: (4/8/2003 1:24:05 PM)
	 */
	public boolean dataDrivenInvokingRun(
		List inputTuples,
		int inputPos) {
		return allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);
	}
	
	public boolean lightweightDataDrivenInvokingRun(List inputTuples, int inputPos){
		return allInvokingModes.lightweightDataDrivenInvokingRun(inputTuples, inputPos);
	}

	/**
	 * get the automata
	 * Creation date: (1/16/2003 1:22:09 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedAutomata
	 */
	public ExtendedAutomata getAutomata() {
		return automata;
	}

	/**
	 * get the state corresponding to the destination
	 * Creation date: (4/19/2003 11:07:02 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState
	 */
	public ExtendedState getOpState() {
		return opState;
	}
	
	/**
	 * get the storage manager
	 * Creation date: (1/30/2003 10:37:31 AM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.StreamStoreMgr
	 */
	public Finer_StreamStoreMgr getStoreMgr() {
		return storeMgr;
	}
	/**
	 * get the storing methods of the targets
	 * Creation date: (1/22/2003 1:44:23 PM)
	 * @return int
	 */
	public int[] getStoringMethods() {
		return storingMethods;
	}
	/**
	 * get the time keeper
	 * Creation date: (4/23/2003 2:11:11 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.IndividualOpTimeKeeper
	 */
	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}
	
	/**
	 * get the number of columns in a tuple
	 * Creation date: (2/19/2003 10:10:44 AM)
	 * @return int
	 */
	public int getTupleColNum() {
		return extractCommon.getTupleColNum();
	}
	/**
	 * are targets attributes?
	 * Creation date: (3/19/2003 6:17:58 PM)
	 * @return boolean
	 */
	public boolean isAttrs() {
		return extractCommon.isAttrs();
	}
	/**
	 * store as a complete tree (string)
	 * Creation date: (1/18/2003 4:50:14 PM)
	 * @return boolean
	 */
	public boolean isCompleteTree() {
		return extractCommon.isCompleteTree();
	}
	/**
	 * store as a dom node
	 * Creation date: (2/15/2003 9:18:36 PM)
	 * @return boolean
	 */
	public boolean isDom() {
		return extractCommon.isDom();		
	}
	/**
	 * store node id
	 * Creation date: (1/18/2003 4:50:14 PM)
	 * @return boolean
	 */
	public boolean isNodeId() {
		return extractCommon.isNodeId();
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (4/11/2003 2:42:11 PM)
	 * @return boolean
	 */
	public boolean isPartitionInvoked() {
		return false;
	}
	/**
	 * store a typed value (string)
	 * Creation date: (1/18/2003 4:50:14 PM)
	 * @return boolean
	 */
	public boolean isTypedValue() {
		return extractCommon.isTypedValue();
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (1/22/2003 2:27:32 PM)
	 */
	public boolean run() {
		allInvokingModes.run();
		return true;

	}
	
	/**
	 * Insert the method's description here.
	 * Creation date: (4/19/2003 11:07:02 PM)
	 * @param newExtractOpState edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState
	 */
	public void setOpState(ExtendedState newOpState) {
		opState = newOpState;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (1/30/2003 10:37:31 AM)
	 * @param newStoreMgr edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.StreamStoreMgr
	 */
	public void setStoreMgr(Finer_StreamStoreMgr newStoreMgr) {
		storeMgr = newStoreMgr;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (1/31/2003 11:43:23 AM)
	 * @param storingMethod int
	 */
	public void setStoringMethod(int storingMethod) {
		extractCommon.setStoringMethod(storingMethod);

	}
	
	/**
	 * Insert the method's description here.
	 * Creation date: (2/19/2003 10:10:44 AM)
	 * @return int
	 */
	public int tupleColNum() {
		return extractCommon.tupleColNum();
	}

	public List runWithBlockingInputs(List[] inputTuples, int inputPos) {
		return null;
	}

	
	public boolean selfPartitionDelimiInvokingRun(
		List partitionStates,
		SuspensionEdge suspendComp) {
		return 
			allInvokingModes.selfPartitionDelimiInvokingRunForExtract(
				partitionStates,
				suspendComp);

	}

	public void cleanTemp() {
		itemCountWithinEntry = 0;

		tempResult.clear();

		// ((XATQueueImp) getOutputQueue()).dequeueAll();

	}

	public int getItemCountWithinEntry() {
		return itemCountWithinEntry;
	}

	
	public boolean getPartitionChecked() {
		return partitionChecked;
	}

	public void setPartitionChecked(boolean partitionChecked) {
		this.partitionChecked = partitionChecked;
	}

	
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		return false;
	}

	
	public boolean cleanAllAccumulatedTuples() {
		return false;
	}

	public boolean matchStates(XATStreamOperator op) {
		return false;
	}

	public boolean hasUnmatchedState() {
		return false;
	}

	public boolean recomputeUnmatchedState() {
		return false;
	}

	public int setStateID(int startID) {
		return 0;
	}

	public int getOutputCount() {
		return outputCount;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore#resetOutputCount()
	 */
	public void resetOutputCount() {
		outputCount = 0;

	}

	public List runGroupBy() {
		itemCountWithinEntry = 0;
		List returnResult = (List) (tempResult.clone());
		
		//this is added in case runGrpupby() is called multiple times within the same entry node
		//this can happen when SQO is applied,
		//before the end element of the context node is encountered
		//it will be called once
		//it will be called again when the end element of the context node is encountered
		//returnResult.addAll(((XATQueueImp)getOutputQueue()).dequeueAll());
		if (!TestExecBaseLocal.queryIsRecur)
			tempResult.clear();

		return returnResult;
	}

	public void runPushStack(int tokenId, int startTokenId) {		
		//this is for estimating the cost of storing an element
		timeKeeper.start();
		
		itemCountWithinEntry++;

		if (isTypedValue()) {
			//next text descendant encountered would be the first text descendant of the subtuple element
			typedTupleKey =
				new Finer_StreamStoreMgrKey(
					(StringBuffer) storeMgr.getCurrentUnitToPutTypedVal(),
					storeMgr.getTypedValBegin());
			tupleContentHolder[posType] = new XATKeyDataValue(typedTupleKey);

		} else if (isDom()) {
			//add in the dom object handle
			Element currentNode = (Element) storeMgr.getCurrentUnitToPutDom();

			tupleContentHolder[posDom] = new XATKeyDataValue(currentNode);

		}

		if (isNodeId()) {
			//create the node id (synthetic, not intrinsic in data), which is equivalent to run time state id
			tupleContentHolder[posNodeId] =
				new XATKeyDataValue(
					new Integer(tokenId));

		}

		XATTuple newTuple = new XATTupleImp(tupleContentHolder);

		tempResult.add(newTuple);
		timeKeeper.pause();

	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream#runPopStack()
	 */
	public void runPopStack(int tokenId, int startTokenId) {
//Mingzhu's change to run experiment, should change back after running experiment.
		typedTupleKey.setEnd(storeMgr.getTypedValEnd());

	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore#run(java.util.List, int)
	 */
	public List run(List inputTuples, int inputPos) {
		return runGroupBy();
	}


	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.SelfPartitionDelimiInvoking#needSetPartitionCheck()
	 */
	public boolean needSetPartitionCheck() {
		return needSetPartitionCheck;
	}
	
	public void setNeedSetPartitionCheck(){
		needSetPartitionCheck = true;
	}


	public List[] demandDrivenInvokingRunWithOrder() {
		return null;
	}
	public NavigationStep[] getDest(){
		return dest;
	}
	
	public void runPushStackWithRecur(int tokenId, int startTokenId)
	{
		
	}
	public void runPopStackWithRecur(int tokenId, int startTokenId){
	
	}



	
}
