package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator;

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavCollection;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.XATKeyDataValue;

import org.w3c.dom.*;
/**
 * stream version of navcollection
 * Creation date: (5/8/2003 5:15:26 PM)
 * @author: Hong Su
 */

public class NavCollectionStream
	extends NavigateStreamImp
	implements DataDrivenInvoking, NavCollection {

	
	/**
	 * NavCollectionExecStream constructor comment.
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 * @param ep edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep[]
	 * @param d edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep[]
	 */
	public NavCollectionStream(
		XATNode node,
		NavigationStep[] ep,
		NavigationStep[] d) {
		super(node, ep, d);
		timeKeeper = new ClassExecTimeKeeper(this);
	}

	/**
	 * @param inputTuples List
	 * @param inputPos int
	 * @return boolean
	 * output tuples of an operator A implementing this interface are passed as parameters  
	 * to the parent operator B, therefore B does not need to fetch from the queue
	 * this is a pipeling manner in the sense that all intermediate data is directly passed
	 * to be consumed rather than stored (neither in second storage nor memory)
	 */
	public boolean dataDrivenInvokingRun(
		List inputTuples,
		int inputPos) {
		return allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);
	}

	/**
	 * Creation date: (5/8/2003 5:15:26 PM)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.IndividualOpTimeKeeper
	 */
	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}

	/**
	 * Creation date: (5/8/2003 5:17:58 PM)
	 */
	public boolean run() {
		allInvokingModes.run();
		return true;  
	}

	/**
	 * given input, generate output
	 * Creation date: (5/8/2003 5:15:26 PM)
	 * @return List output tuples of the operator
	 * @param inputTuples List input tuples to the operator
	 */
	public List run(List inputTuples, int inputPos){
		int size = inputTuples.size();
		timeKeeper.increInputCount(size);
		timeKeeper.start();

		List result = new ArrayList();

		int[] outputPos = getOutputPositions()[0];

		int iEntryCol = getEntryColPos();		

		for (int i = 0; i<size; i++) {

			XATTupleImp t = (XATTupleImp) inputTuples.get(i);
			XATKeyDataValue value = (XATKeyDataValue) t.getCell(iEntryCol);			
			
			List collection =
				follow((Node)value.getValue(), dest);
			
			int collectionSize = collection.size();
		
			if (collectionSize > 0) {
				if (getCreateNewTuple()==1){
				//there exists the destination elements, this is a navigation with filtering semantics
				XATKeyDataValue[] newData = new XATKeyDataValue[outputPos.length + 1];
				//copy the input
				t.fillDataInListTo(newData, outputPos, 0);
				//add the newly generated destination
				newData[outputPos.length] = new XATKeyDataValue(collection);

				result.add(new XATTupleImp(newData));
				}else{
					//directly add input tuple 
					result.add(t);
				}
				
				timeKeeper.increOutputCount(1);
			}
			
			timeKeeper.increItemCountWithinEntrySum(collectionSize); 
		}
		
		

		timeKeeper.pause();

		return result;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAccumulatedTuples(long)
	 */
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAllAccumulatedTuples()
	 */
	public boolean cleanAllAccumulatedTuples() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#matchStates(edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator)
	 */
	public boolean matchStates(XATStreamOperator op) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasUnmatchedState()
	 */
	public boolean hasUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#recomputeUnmatchedState()
	 */
	public boolean recomputeUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setStateID(int)
	 */
	public int setStateID(int startID) {
		// TODO Auto-generated method stub
		return 0;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.DemandDrivenInvoking#demandDrivenInvokingRunWithOrder()
	 */
	public List[] demandDrivenInvokingRunWithOrder() {
    	return allInvokingModes.demandDrivenInvokingRunWithOrder();
	}

		
}
