package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator;

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnest;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.XATKeyDataValue;

import org.w3c.dom.*;

/**
 * stream version of navunnest Creation date: (5/8/2003 5:09:38 PM)
 * 
 * @author: Hong Su
 */

public class NavUnnestStream extends NavigateStreamImp implements
		DataDrivenInvoking, NavUnnest {

	/**
	 * NavUnnestExecStream constructor comment.
	 * 
	 * @param node
	 *            edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 * @param ep
	 *            edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep[]
	 * @param d
	 *            edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep[]
	 */
	public NavUnnestStream(XATNode node, NavigationStep[] ep, NavigationStep[] d) {
		super(node, ep, d);

		timeKeeper = new ClassExecTimeKeeper(this);
	}

	/**
	 * output tuples of an operator A implementing this interface are passed as
	 * parameters to the parent operator B, therefore B does not need to fetch
	 * from the queue this is a pipeling manner in the sense that all
	 * intermediate data is directly passed to be consumed rather than stored
	 * (neither in second storage nor memory)
	 * 
	 * Creation date: (5/8/2003 5:09:38 PM)
	 */
	public boolean dataDrivenInvokingRun(java.util.List inputTuples,
			int inputPos) {
		return allInvokingModes.dataDrivenInvokingRun(inputTuples, inputPos);
	}

	/**
	 * Insert the method's description here. Creation date: (5/8/2003 5:09:38
	 * PM)
	 * 
	 * @return edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.IndividualOpTimeKeeper
	 */
	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}

	/**
	 * Insert the method's description here. Creation date: (5/8/2003 5:12:06
	 * PM)
	 */
	public boolean run() {
		allInvokingModes.run();
		return true;
	}

	/**
	 * given input, generate output Creation date: (5/8/2003 5:09:38 PM)
	 * 
	 * @return java.util.List output tuples of the operator
	 * @param inputTuples
	 *            java.util.List input tuples to the operator
	 */
	public List run(List inputTuples, int inputPos) {
		outputCount = 0;
		int size = inputTuples.size();
		timeKeeper.increInputCount(size);
		timeKeeper.start();

		List result = new ArrayList();

		int[] outputPos = getOutputPositions()[0];

		int iEntryCol = getEntryColPos();

		for (int i = 0; i < size; i++) {

			XATTupleImp t = (XATTupleImp) inputTuples.get(i);

			XATDataValueImp value = (XATDataValueImp) t.getCell(iEntryCol);

			List collection = follow((Node) value.getValue(), dest);

			int collectionSize = collection.size();

			//if collectionSize == 0, an empty result is returned, this is a
			// navUnnest with filtering semantics
			if (collectionSize == 1) {
				if (getCreateNewTuple() == 1) {
					XATKeyDataValue[] newData = new XATKeyDataValue[outputPos.length + 1];
					//copy the input
					t.fillDataInListTo(newData, outputPos, 0);
					//add the newly generated destination
					newData[outputPos.length] = new XATKeyDataValue(collection
							.get(0));

					result.add(new XATTupleImp(newData));
				} else {
					//directly add the input tuple to the result set
					result.add(t);
				}

			} else {
				for (int j = 0; j < collectionSize; j++) {
					XATKeyDataValue[] newData = new XATKeyDataValue[outputPos.length + 1];
					((XATTupleImp) t).fillDataInListTo(newData, outputPos, 0);
					//add the newly generated destination
					newData[outputPos.length] = new XATKeyDataValue(collection
							.get(j));

					result.add(new XATTupleImp(newData));

				}

			}

			outputCount = outputCount + collectionSize;
		}

		timeKeeper.pause();
		timeKeeper.increOutputCount(outputCount);

		return result;

	}

	

	

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAccumulatedTuples(long)
	 */
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAllAccumulatedTuples()
	 */
	public boolean cleanAllAccumulatedTuples() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#matchStates(edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator)
	 */
	public boolean matchStates(XATStreamOperator op) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasUnmatchedState()
	 */
	public boolean hasUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#recomputeUnmatchedState()
	 */
	public boolean recomputeUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setStateID(int)
	 */
	public int setStateID(int startID) {
		// TODO Auto-generated method stub
		return 0;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore#getOutputCount()
	 */
	public int getOutputCount() {
		return outputCount;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore#resetOutputCount()
	 */
	public void resetOutputCount() {
		outputCount = 0;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.DemandDrivenInvoking#demandDrivenInvokingRunWithOrder()
	 */
	public List[] demandDrivenInvokingRunWithOrder() {
		return allInvokingModes.demandDrivenInvokingRunWithOrder();
	}

}