package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;

/**
 * pure navigation, separated from extraction, encoded in automata Creation
 * date: (1/22/2003 3:36:20 PM)
 * 
 * @author: Hong Su
 */

public class PureNavigateAutoStream extends NavigateStreamImp implements
		AutomataEnabled
{

	protected ClassExecTimeKeeper timeKeeper;

	protected ExtendedAutomata automata;

	//its destination is associated with a state
	protected ExtendedState opState;

	protected ArrayList couplingRels = new ArrayList();

	public PureNavigateAutoStream(XATNode node) {
		super(node);
		timeKeeper = new ClassExecTimeKeeper(this);
	}

	public ExtendedAutomata getAutomata() {
		return automata;
	}

	public PureNavigateAutoStream(
			edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode node,
			NavigationStep[] ep, NavigationStep[] d) {
		super(node, ep, d);
		timeKeeper = new ClassExecTimeKeeper(this);
	}

	/**
	 * associate the operator with a state in a given automata
	 * 
	 * @param auto
	 *            ExtendedAutomata the automata that encodes the path expression
	 */
	public void associateWithAuto(ExtendedAutomata auto) {
		automata = auto;

		List statesEncodingPath = new ArrayList();

		XATNode node = getNode();

		try {
			//get parent info: the state associated with the child operator
			//child operator must be a pureNavigateAuto as well
			XATOperator childOp = node.getChild(0).getOperator();

			List childStas = auto.getFrStateHash((XATStreamOperator) childOp);

			for (Iterator it = childStas.iterator(); it.hasNext();) {
				ExtendedState childSta = (ExtendedState) it.next();
				//construct states for the NavigateSteps
				NavigationStep[] dest = getDestination();

				NavigationStep[] autoDest = null;

				if (dest[dest.length - 1] instanceof AttStep) {
					//if the target is an attr, should not add this attr into
					// the automata
					autoDest = new NavigationStep[dest.length - 1];
					for (int i = 0; i < dest.length - 1; i++) {
						autoDest[i] = dest[i];
					}
				} else {
					autoDest = dest;
				}

				ExtendedState sta = null;

				if (autoDest.length > 0) {
					sta = (ExtendedState) auto.addPath(childSta, autoDest,
							statesEncodingPath);
				} else
					sta = (ExtendedState) auto
							.getFrStateHash((XATStreamOperator) childOp);

				//map the tokenNav with statesEncodingPath
				auto.registerTokenNavToStates(this, statesEncodingPath);

				sta.addDestNavOp(this);
				childSta.addEntryNavOp(this);

				auto.addIntoStateHash(this, sta);

				opState = sta;

				//if this tokennav is the bottommost tokenav
				//also mark this special state
				if (getNode().getChild(0).getOperator() instanceof XMLScanStream)
					automata.setBottomDestState(sta);

			}

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/**
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState
	 *         the state associated with the destination element
	 */
	public ExtendedState getOpState() {
		return opState;
	}

	public boolean isPartitionInvoked() {
		return false;
	}

	/**
	 * Runs the actual operator, required by XATStreamOperator
	 */
	public boolean run() throws Exception {
		System.out.println("pure navigate auto stream should not be called.");
		return true;
	}

	/*
	 * give a navigation step, find how far away is it from the dest point
	 */
	private int posFrDest(NavigationStep step) {

		int stepLength = dest.length;
		for (int i = 0; i < stepLength; i++) {
			NavigationStep curStep = dest[stepLength - 1 - i];
			if (curStep.equals(step))
				return i;
		}

		return -1;
	}

	public String toString() {
		String rs = super.toString();

		return rs;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.AutomataEnabled#runPushStack()
	 */
	public void runPushStack() {
		// TODO Auto-generated method stub

	}

	public void runPopStack(int tokenId, int startTokenId) {
		//for counting how many tokens are there in the destination element
		//this is for estimating extraction cost
		timeKeeper.pauseTokenCount(tokenId);
		timeKeeper.pauseStartTokenCount(startTokenId);
	}

	public void runPushStack(int tokenId, int startTokenId) {
		//for counting how many tokens are there in the destination element
		//this is for estimating extraction cost
		timeKeeper.tokenCount(tokenId);
		timeKeeper.startTokenCount(startTokenId);
		
	}
	
	public List runGroupBy() {
		// TODO Auto-generated method stub
		return null;
	}	
	
	public ClassExecTimeKeeper getTimeKeeper() {
		return timeKeeper;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.DemandDrivenInvoking#demandDrivenInvokingRunWithOrder()
	 */
	public List[] demandDrivenInvokingRunWithOrder() {
		// TODO Auto-generated method stub
		return null;
	}


	public void init() {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#toToolTipString()
	 */
	public String toToolTipString() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * will never be called
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATSingleQStreamOpRunCore#run(java.util.List)
	 */
	public List run(List inputTuples, int inputPos) {
		// TODO Auto-generated method stub
		return null;
	}	

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.AutomataEnabled#cleanTemp()
	 */
	public void cleanTemp() {
		System.out.println("should not reach here");

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAccumulatedTuples(long)
	 */
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#cleanAllAccumulatedTuples()
	 */
	public boolean cleanAllAccumulatedTuples() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#matchStates(edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator)
	 */
	public boolean matchStates(XATStreamOperator op) {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#hasUnmatchedState()
	 */
	public boolean hasUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#recomputeUnmatchedState()
	 */
	public boolean recomputeUnmatchedState() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#setStateID(int)
	 */
	public int setStateID(int startID) {
		// TODO Auto-generated method stub
		return 0;
	}
	
    
	
	

}