package edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator;

import java.io.Serializable;
import java.util.Iterator;
import java.util.List;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.SelfPartitionDelimiInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState;

/**
 * theorectically, a suspension edge is in the format of (occurrenceNum, operatorToBeChecked, partitionState)
 * the semantics is that a suspension edge is coupled with an automata state
 * this automata state corresponds to an extract operator
 * if the extract operator has generated outputs occurrenceNum within its entry
 * the operatorToBeChecked is checked, if it does not have output
 * the partition state is suspended 
 * the operatorToBeChecked can be null, which means the parition state is directly suspended
 */
public class SuspensionEdge implements Serializable {
	int occurrenceNum = 0;
	
	//the operator whose output is to be checked
	//it can be null resulted from an exclusive type rule
	SelfPartitionDelimiInvoking checkedOp;
	
	//the state corresponds to the reaching the element
	// on which the extracted operator is partitioned on, i.e., the node of the entry node of the XPath expression 
	//extract's corresponding navigation uses
	//i.e., the ancestor state
	//ExtendedState partitionState;
	//all ancestor entry states that benefit from the ending marks
	List partitionStates;
	
	
	public SuspensionEdge(int occurrenceNum, SelfPartitionDelimiInvoking checkedOp, List partitionStates){
		this.occurrenceNum = occurrenceNum;
		this.checkedOp = checkedOp;
		this.partitionStates = partitionStates;
	}
	
	public int getOccurrenceNum(){
		return occurrenceNum;
	}
	
	public SelfPartitionDelimiInvoking getCheckedOp(){
		return checkedOp;
	}
	
	public List getPartitionStates(){
		return partitionStates;
	}	
	
	public String toString(){
		String output = "";
		
		output = output + occurrenceNum + " ->";
		
		if (checkedOp==null)
		 output = output + " ,";
		 else output = output + checkedOp+" ,";
		 
		output = output + " -";
		
		for (Iterator it = partitionStates.iterator();it.hasNext();){
			output = output + ((ExtendedState)it.next()).getId() +"|";
		}
		
		return output; 
	}


}
