/*
 * Created on 2004-3-24
 *
 * CS4432 Project 2: Implementing Stream-Oriented Query Operators
 * 
 * @author Luping Ding
 * 
 * This class will implement the window-based set-difference operator.
 * The method you need to implement is run() method. And you may feel free
 * to add any other private methods or private variables that you need.
 * 
 * I have given you the skeleton code of run() method. Please feel free
 * to change it if you have a more efficient way to implement it.
 */
package edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStampException;
//import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.XATMultiQueueWindowStreamOperatorImp;
//import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
//import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;

import java.util.Iterator;
//import java.util.LinkedList;
import java.util.ListIterator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;

//import java.io.*;

public class WStreamDifferenceOperatorRecomputeImp
	extends XATMultiQueueWindowStreamOperatorImp 
	implements WStreamDifferenceOperator {

	//int leftCount = 0; //count of tuples being dequeued from the left input queue
	//int rightCount = 0; //count of tuples being dequeued from the right input queue
	int flag = -1; //0: dequeue from the left input queue
	               //1: dequeue from the right input queue
	
	/* Keep track of left and right queue windows and our output cache */
	
	/* For the queue windows and the output cache, we use a custom TreeMap wrapper that 
	 * allows for rapid searches using CRC32 hashes.
	 * 
	 * See TuplesList.java for more details.
	 */
	private TuplesList leftQueueWindow = null;
	private TuplesList rightQueueWindow = null;
	private TuplesList outputQueueCache = null;
	
	private long start = 0;
	
	
	/** Constructor
	 * @param init
	 */
	public WStreamDifferenceOperatorRecomputeImp(XATNode init) {
		super(init);
	}
	
	/********* YOU MUST IMPLEMENT THIS METHOD ********/
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#run()
	 * 
	 * @author Brandon Kilgore
	 */
	public boolean run() throws Exception {
		Iterator i = this.getInputQueues();

		//check schema of the input queues
		while (i.hasNext()) {
			XATQueue q = (XATQueue) i.next();
			if (q.getSchema() == null){
				return false;
			}
		}

		//initialize the schema of the output queue
		if (this.getOutputQueue().getSchema() == null) {
			init();
		}

		/* if our windows dont exist yet, create TupleLists telling them to ignore the 'timestamp' field */
		if (rightQueueWindow == null)
			rightQueueWindow = new TuplesList(this.getOutputQueue().getSchema(), "timestamp");
		if (leftQueueWindow == null)
			leftQueueWindow = new TuplesList(this.getOutputQueue().getSchema(), "timestamp");
		
		XATTuple newObj = null;

		/* Load the current time_window and increment lengths */
		long increment = super.getIncrement();
		long time_window = super.getWindowSize();
		
		while (true) {
			
			try {
				// if the left input queue is empty, try to dequeue the right input queue
				if (this.getLeftInputQueue().getQueueSize(this) == 0) {
					newObj = this.getRightInputQueue().dequeue(this);
					flag = 1;
					//rightCount++;
				} 
				//if the right input queue is empty, try to dequeue the left input queue
				else if (this.getRightInputQueue().getQueueSize(this) == 0) {
					newObj = this.getLeftInputQueue().dequeue(this);
					flag = 0;
					//leftCount++;
				} 
				/* If both queues are non-empty, then compare the timestamp 
				   of the head tuple of two input queues
				   and dequeue the tuple with the earlier timestamp.
				   */
				else {
					long leftTimeStamp = 0;
					long rightTimeStamp = 0;
			
					leftTimeStamp =
						this.getLeftInputQueue().peek(this).getMinTimeStamp();
					rightTimeStamp = 
						this.getRightInputQueue().peek(this).getMinTimeStamp();
							
					if(leftTimeStamp < rightTimeStamp) {
						newObj = this.getLeftInputQueue().dequeue(this);
						flag = 0;
						//leftCount++;
					} else {
						newObj = this.getRightInputQueue().dequeue(this);
						flag = 1;
						//rightCount++;
					}					
				}
				
				// record the timestamp of start
				//if (leftCount + rightCount == 1)
					start = newObj.getMinTimeStamp();
				
				//record the first tuple received from the left and the right input stream
				//if (flag == 0 && leftCount == 1) {
				//	OutputStream outStream = new FileOutputStream("first_tuple_from_left");
				//	outStream.write((this.getLeftInputQueue().getQueueName()+"\n").getBytes());
				//	outStream.write((newObj.toString()+"\n").getBytes());
				//	outStream.write(newObj.getTimeStamp().toString().getBytes());
				//} else if (flag == 1 && rightCount == 1) {
				//	OutputStream outStream = new FileOutputStream("first_tuple_from_right");
				//	outStream.write((this.getRightInputQueue().getQueueName()+"\n").getBytes());
				//	outStream.write((newObj.toString()+"\n").getBytes());
				//	outStream.write(newObj.getTimeStamp().toString().getBytes());
				//}

				//process, your code goes here...
				/* newObj is a dequeued tuple, and
				 * flag is what queue it came from
				 */
				
				/* If a whole time_window has passed since start, 
				 * update the output queue
				 */
				if (newObj.getMinTimeStamp() >= start + this.getWindowSize())
				{
					/* At this point we know that both input queues are synced
					 * up to the end of the time_window because either both
					 * queues have tuples, in which case we are taking the smaller
					 * of the two timestamps, or only one has tuples and so we have
					 * nothing on the other queue to sync up
					 */
										
					/*  
					 * We have a full window, update the output queue 
					 */


					if (outputQueueCache == null)
					{
						/* We dont have an ouput queue yet, 
						 * construct the first one
						 * To do this we simply iterate over the entire left window
						 * and each item we dont find in the right window we add to
						 * the output cache.
						 */
						outputQueueCache = new TuplesList(this.getOutputQueue().getSchema(), "timestamp");
						ListIterator leftIt = leftQueueWindow.listIterator();
						while (leftIt.hasNext())
						{
							XATTuple l = (XATTuple) leftIt.next();
							if (!rightQueueWindow.hasTuple(l))
							{
								/* The 'true' means that we want this tuple to be unique
								 * (the output is a set, not a bag)
								 */
								outputQueueCache.addToEnd(l, true);
							}
						}
											
					}
					else
					{
						/* We have an existing output queue, update it
						 * 
						 * This means that our current queue windows consts of the old time_window
						 * plus a new increment, so we can use both the increment we're about to remove
						 * and the new increment to update the output queue. 
						 *
						 * To keep the output queue in sync, we need to do 4 updates:
						 *  1. Iterate over what we're deleting from <left> and if it doesn't exist anywhere else in <left>, delete it from the output
						 *  2. Iterate over what we just added to <right> and delete it from the output queue
						 *  3. Iterate over what we're deleting from <right> and if its nowhere else in right and we find it in the new window of <left>, add it to the queue
						 *  4. Iterate over what we just added to <left> and if we can't find it in the new window of <right>, add it to the queue
						 * 
						 * note: this is faster than recreating the queue each time, as long as time_window is greater than 4*increment because we only have to iterate
						 * over four increments instead of the whole time_window (and searching is incredibly fast compared to iterations) 
						 */
						
						/* 1. Iterate over what we're deleting from <left> and if it doesn't exist anywhere else in <left>, delete it from the output
						 */
						ListIterator oldLeft = leftQueueWindow.listIterator();
						while (oldLeft.hasNext())
						{
							XATTuple t = (XATTuple) oldLeft.next();

							/* We only want to iterate up to what is no longer part 
							 * of our window, which is up to the start of our current window
							 */
							if (t.getMinTimeStamp() < start)
							{
								/* We search through everything after start (what is actually our new window) 
								 * to see if we find it anywhere else (meaning that it still needs to be in the 
								 * output)
								 */
								if (!leftQueueWindow.hasTupleGTETimestamp(t, start))
								{
									outputQueueCache.remove(t);
								}
							}
							else
								break;
						}
						
						/* 2. Iterate over what we just added to <right> and delete it from the output queue
						 */
						ListIterator newRight = rightQueueWindow.listIteratorTimestamp(start + time_window - increment);
						while (newRight.hasNext())
						{
							XATTuple t = (XATTuple) newRight.next();

							/* Remove the tuple matching these new tuples from the cache (remove 
							 * automatically searches first to see if anything really needs to be deleted)
							 */
							outputQueueCache.remove(t);
						}
						
						/* 3. Iterate over what we're deleting from <right> and if its nowhere else in right and we find it in the new window of <left>, add it to the queue
						 */
						ListIterator oldRight = rightQueueWindow.listIterator();
						while (oldRight.hasNext())
						{
							XATTuple t = (XATTuple) oldRight.next();

							/* We only want to iterate up to what is no longer part 
							 * of our window, which is up to the start of our current window
							 */
							if (t.getMinTimeStamp() < start)
							{
								/* First check to make sure that this tuple is nowhere else in our right queue window
								 */
								if (!rightQueueWindow.hasTupleGTETimestamp(t, start))
								{
									/* This means we no longer need to filter by this tuple, so if it
									 * is anywhere in our new left queue window, add it to the output
									 */
									if (leftQueueWindow.hasTupleGTETimestamp(t, start))
									{
										/* The 'true' means that we want this tuple to be unique
										 * (the output is a set, not a bag)
										 */
										outputQueueCache.addToEnd(t, true);
									}
								}
							}
							else
								break;
						}
						
						/* 4. Iterate over what we just added to <left> and if we can't find it in the new window of <right>, add it to the queue
						 */
						ListIterator newLeft = leftQueueWindow.listIteratorTimestamp(start + time_window - increment);
						while (newLeft.hasNext())
						{
							XATTuple t = (XATTuple) newLeft.next();

							/* Only compare against the new right queue window, which is everything after start */
							if (!rightQueueWindow.hasTupleGTETimestamp(t, start))
							{
								/* Again, the 'true' means that we want this tuple to be unique
								 * (the output is a set, not a bag)
								 */
								outputQueueCache.addToEnd(t, true);
							}
						}
						
						
						/* Now that we're finally done with the old increment we no longer need, delete
						 * it from the queue windows (delete everything from each window up to the new start of our window)
						 */
						leftQueueWindow.deleteStartToTimestamp(start);
						rightQueueWindow.deleteStartToTimestamp(start);
					}
					
					/* Change start to reflect the new beginning of the queue windows */ 
					start += increment;
					
					/* Cycle through the queue cache, enqueueing each tuple 
					 * in our output queue
					 */
					ListIterator cacheIt = outputQueueCache.listIterator();
					while (cacheIt.hasNext())
					{
						XATTuple t = (XATTuple) cacheIt.next();
						this.getOutputQueue().enqueue(t, this);
					}				
				}
				
				/* Store the newly dequeued tuple in the appropriate queue window */
				if (flag == 0)
					leftQueueWindow.addToEnd(newObj);
				else if (flag == 1)
					rightQueueWindow.addToEnd(newObj);
				
			} catch(XATQueueException e1) {
				return true;
			} catch(XATTimeStampException e2) {
				e2.printStackTrace();
			}
		}
	}
	
	

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#init()
	 */
	public void init() {
		this.getOutputQueue().setSchema(this.getLeftInputQueue().getSchema());		
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator#setLeftInputQueue(edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue)
	 */
	public synchronized void setLeftInputQueue(XATQueue newQueue)
		throws XATQueueException {
		super.setLeftInputQueue(newQueue);
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator#setLeftOutputQueue(edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue)
	 */
	public synchronized void setLeftOutputQueue(XATQueue newQueue)
		throws XATQueueException {
		super.setLeftOutputQueue(newQueue);
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator#toToolTipString()
	 */
	public String toToolTipString() {
		return "Set-Difference";
	}
	
	public int getStateSize() {
		return 0;
	}
	
	/**
	 * The method cleanAccumulatedTuples() is similar to the method run(),
	 * except the way of figuring out how many tuples to dequeue from input queues.
	 * Inside method run() the numbers are computed based on the property
	 * "NUMBER_OF_TUPLES_TO_DEQUEUE" set by the system scheduler,
	 * while in cleanAccumulatedTuples, all tuples in input queues that have their max timesamp
	 * smaller than the given maxtimestamp should be dequened.
	 */
	public boolean cleanAccumulatedTuples(long maxtimestamp) {
		return true;
	}


	/*
	 * process all accumulated tuples in this operators' input queues.
	 */
	public boolean cleanAllAccumulatedTuples() {
		return true;
	}
	

	/*
	 * check if this operator has any unmatched states.
	 * return: TRUE if this operator has unmatched states, FALSE otherwise.
	 */
	public boolean hasUnmatchedState() {
		// TODO
		return false;
	}


	public boolean matchStates(XATStreamOperator op) {
		// TODO
		return false;
	}
	

	public boolean recomputeUnmatchedState() {
		// TODO
		return true;
	}
	

	public int setStateID(int nextid) {
		// TODO
		return nextid;
	}


}
