package edu.wpi.cs.dsrg.xmldb.xat.common.relationalstreamexpression;

import java.util.List;
//import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.expression.Expression;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.Visitable;

/**
 * Expression can be
 * used in the Join and Select operators as a condition.
 * It can also be used in the LET clause to do a variable change.
 * We take the design from the xacute.quilt.Operator and their
 * subclasses.
 * <p>
 * You don't really need a visitor pattern to implement the execution
 * of it. Also, it's not using the XATTree.
 * So, just implement the eval() function.
*/
public interface ExpressionStr extends Visitable, Expression, Cloneable
{
	public static final int LT = 0;
	public static final int GT = 1;
	public static final int LEQ = 2;
	public static final int GEQ = 3;
	public static final int EQ = 4;
	public static final int NEQ = 5;
	
	public final static int STRING = 6;
	public final static int DOUBLE = 7;
	public final static int LONG = 8;
	public final static int INTEGER = 9;
	public final static int BOOL = 10;
	
/**
 * Creation date: (2/12/2003 11:28:39 AM)
 * @return java.lang.Object
 */
//Object clone();
    /**
    * It will evaluate itself and return the value as an object.
    * Remember, it could be like: Integer, Float, Double, String, Boolean.
    */
    public abstract Object eval(List tuples, XATStreamOperator op);   
/**
 * all the terminal expressions, i.e., the variables, involved in the expressions
 * Creation date: (2/17/2003 4:25:13 PM)
 * @return java.util.Vector
 */
//java.util.Vector getTerminalColumnExprs();
/**
 * the same as getTerminalColumnExprs except that this returns a list of items of type ColumnName
 * while the column names are wrapped in expressions and returned in getTerminalColumnExprs
 * @return java.util.Vector
 */
//java.util.Vector getTerminalColumnNames();
/**
 * get all untrue subexpressions, suppose this expression is a conjuctions of several subexpressions
 * Creation date: (1/26/2003 3:25:22 PM)
 * @return java.util.List
 */
//ExpressionStr getUntrueExpression();
/**
 * get all column names that involve in the untrue subexpressions
 * Creation date: (2/19/2003 5:17:04 PM)
 * @return java.lang.String[]
 */
//Vector getUntrueTerminalColumnNames();
/**
 * all expressions that return a boolean evaluation value should setuntrueexpression
 * BinAnd, BinCOMP, BinOR, UniNot
 * expressions that do not return a boolean evaluation value may not setuntrueexpression
 * Creation date: (2/13/2003 2:50:38 PM)
 * @param newUntrueExpression edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.ExpressionStr
 */
//void setUntrueExpression(ExpressionStr newUntrueExpression);

	/**
	 * reset column positions in expression according to the given position mapping.
	 * this method is usually called by optimizer when switching two operators and as a result we
	 * have to reset the column IDs of involving operators' expressions.
	 * 
	 * This method is usually called right before the method "getRelatedSubExpression" in this class.
	 * @param pos
	 * @return
	 */
	boolean resetColumnPosition(int[][] positionmap);
    
	/**
	* get the sub-expressions that contains ONLY the given
	* queue position and column positions.
	* the returned expression could be as large as this expression itself, or as small as null.
	* 
	* This method is usually called by the optimizer right after the method "resetColumnPosition" in this class.
	*/
	ExpressionStr getRelatedSubExpression(int queuepos, int[] columnpos);
	
	boolean containColumnPositions(int queuePos, int[] columnPos);
}
