package edu.wpi.cs.dsrg.xmldb.xat.common.xatnode;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;

/**
 * This class is the Row structure of the XAT.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0 
 */

public interface RowAccessor {
    /**
     * Get a cell by column and tuple positions.
     * @return XATValue The value in the cell specified by column_positon and tuple_positon.
     * @param column_position The column position of the cell extracting.
     * @param tuple_position The tuple position of the cell extracting.
     * @exception XATTableOutOfRangeException
     * @exception XATTableEmptyException
     * @since 1.0
     */
    XATValue getValue(int column_position, int tuple_position)
        throws XATTableOutOfRangeException, XATTableEmptyException;
    /**
     * Update a cell with a specific object.
     * @return XATValue The original value in the cell.
     * @param column_position The column position of the cell extracting.
     * @param tuple_position The tuple position of the cell extracting.
     * @param new_value The new value of the cell.
     * @since 1.0     
     */
    XATValue updateValue(
        int column_position,
        int tuple_position,
        XATValue new_value)
        throws XATTableOutOfRangeException, XATTableEmptyException;
}