package edu.wpi.cs.dsrg.xmldb.xat.component.NEXMarkdatagenerator;

/*  
   NEXMark Generator -- Niagara Extension to XMark Data Generator

   Acknowledgements:
   The NEXMark Generator was developed using the xmlgen generator 
   from the XMark Benchmark project as a basis. The NEXMark
   generator generates streams of auction elements (bids, items
   for auctions, persons) as opposed to the auction files
   generated by xmlgen.  xmlgen was developed by Florian Waas.
   See http://www.xml-benchmark.org for information.

   Copyright (c) Dept. of  Computer Science & Engineering,
   OGI School of Science & Engineering, OHSU. All Rights Reserved.

   Permission to use, copy, modify, and distribute this software and
   its documentation is hereby granted, provided that both the
   copyright notice and this permission notice appear in all copies
   of the software, derivative works or modified versions, and any
   portions thereof, and that both notices appear in supporting
   documentation.

   THE AUTHORS AND THE DEPT. OF COMPUTER SCIENCE & ENGINEERING 
   AT OHSU ALLOW USE OF THIS SOFTWARE IN ITS "AS IS" CONDITION, 
   AND THEY DISCLAIM ANY LIABILITY OF ANY KIND FOR ANY DAMAGES 
   WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.

   This software was developed with support from NSF ITR award
   IIS0086002 and from DARPA through NAVY/SPAWAR 
   Contract No. N66001-99-1-8098.

*/

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.Random;

class XMLAuctionStreamGenerator {
    
    
    String tab = " | "; // attribute delimiter
    String punctBegin = "##"; //beginning of punctuations

    public static int ITEMS_PER_PERSON = 10;
    public static int BIDS_PER_ITEM = 10;
    public static final String yesno[] = { "yes", "no"};
    public static final String auction_type[] = {"Regular", "Featured"};
    
    // will generate bids, items and persons in a ratio of 10 bids/item 5 items/person
    private Random  rnd = new Random(103984);
    private SimpleCalendar cal = new SimpleCalendar(rnd);

    private Persons persons = new Persons(); // for managing person ids
    private OpenAuctions openAuctions; // for managing open auctions
    private PersonGen p = new PersonGen();  // used for generating values for person    

    private MyBuffer personBuf;
	private MyBuffer auctionBuf;
	private MyBuffer bidBuf;
    private BufferedWriter personWriter;
    private BufferedWriter auctionWriter;
    private BufferedWriter bidWriter;
    private int numGenCalls;
    
    private static int MAXINCREMENT_MILLISEC = 1000;
    private static int WARP = 10;
    private static int DELAY = 24000;    

    public boolean LIMIT_ATTRIBUTES = false;

    public XMLAuctionStreamGenerator(int genCalls){
        numGenCalls = genCalls;
        openAuctions = new OpenAuctions(cal);
    }

    public void generateStream(BufferedWriter personWriter,
	                           BufferedWriter auctionWriter,
	                           BufferedWriter bidWriter) throws IOException {
        personBuf = new MyBuffer();
        auctionBuf = new MyBuffer();
        bidBuf = new MyBuffer();
        this.personWriter = personWriter;
		this.auctionWriter = auctionWriter;
		this.bidWriter = bidWriter;
        if(LIMIT_ATTRIBUTES)
            System.out.println("WARNING: LIMITING ATTRIBUTES");
	
        // first do startup - generate some people and open auctions that
        // can be bid on

        // put 10 persons in a document
        for(int i = 0;i<5; i++) {
            initMyBuf();
            for(int j = 0; j<10; j++) {
                generatePerson(personBuf, 1);
            }
            writeMyBuf(personBuf, personWriter);
        }

        for(int i = 0; i<5; i++) {
            initMyBuf();
            for(int j = 0; j<10; j++) {
                generateOpenAuction(auctionBuf, 1);
            }
            writeMyBuf(auctionBuf, auctionWriter);
        }

        // now go into a loop generating bids and persons and so on
        // want on average 10 items/person and 10 bids/item
        int count = 0;
        while(count < numGenCalls) {
            initMyBuf();

            // generating a person approximately 10th time will
            // give is 10 items/person since we generate on average
            // one bid per loop
            if(rnd.nextInt(10) == 0) {
                generatePerson(personBuf, 1);
            } 
	    
            // want on average 1 item and 10 bids
            int numItems = rnd.nextInt(3); // should average 1
            generateOpenAuction(auctionBuf, numItems);
	    
            int numBids = rnd.nextInt(21); // should average 10
            generateBid(bidBuf, numBids);
            
            OpenAuction[] auctions = openAuctions.getOpenAuctions();
            int offset = openAuctions.getOffset();
			for (int i=0; i<auctions.length; i++) {
				if (auctions[i] == null) {
					break;
				}
				OpenAuction auction = auctions[i];
				if (!auction.isClosed()) {
					if (auction.checkClosed(cal)) {
						int closeAuctionId = offset+i;
						bidBuf.append(punctBegin + closeAuctionId + tab + "*" + tab + "*" + tab + "*" + "\n");
					}
				}
			}
			
            writeMyBuf(personBuf, personWriter);
			writeMyBuf(auctionBuf, auctionWriter);
			writeMyBuf(bidBuf, bidWriter);
            count++;
        }
        personWriter.close();
        auctionWriter.close();
        bidWriter.close();
    }

    private void initMyBuf() throws IOException {
        personBuf.clear();
		auctionBuf.clear();
		bidBuf.clear();
    }
    
    private void writeMyBuf(MyBuffer myBuf, BufferedWriter writer) throws IOException {
        writer.write(myBuf.array(), 0, myBuf.length());
    } 
    
    private void generateBid(MyBuffer myb, int numBids) throws IOException { 
    	long ts=0, temp;
    	boolean start=true;
    	
        cal.incrementTime();
	
        for (int i=0; i<numBids;i++) {
        	// auction id
            int itemId = openAuctions.getExistingId(); 
            myb.append(itemId);
            myb.append(tab);

			// time
            myb.append(cal.getTimeInSecs());//here, datetime is in second;
			myb.append(tab);
            // Alternate time stamps
            //myb.append(System.currentTimeMillis() - rnd.nextInt(MAXINCREMENT_MILLISEC));     
            //ts = System.currentTimeMillis() * WARP + DELAY;
            //myb.append(ts);
            // person reference    	
            myb.append(persons.getExistingId()); 
			myb.append(tab);
	    
	    	//bid price
            myb.append(openAuctions.increasePrice(itemId));
            myb.append(".00");
            myb.append("\n");
        }
    }
    
    // uugh, a bad thing here is that a person can be selling items that are in
    // different regions, ugly, but to keep it consistent requires maintaining
    // too much data and also I don't think this will affect results
    private void generateOpenAuction(MyBuffer myb, int numItems) throws IOException {
        cal.incrementTime();

        // open auction contains:
        // initial, reserve?, bidder*, current, privacy?, itemref, seller, annotation, 
        // quantity, type, interval    
	
        for (int i=0; i<numItems; i++) {
            // at this point we are not generating items, we are generating
            // only open auctions, id for open_auction is same as id of item
            // up for auction
	    
        	//auction id
            int auctionId = openAuctions.getNewId();
            myb.append(auctionId);
            myb.append(tab);

			// reserve
            myb.append((int)Math.round((openAuctions.getCurrPrice(auctionId))*(1.2+(rnd.nextDouble()+1))));
			myb.append(tab);
			
			// privacy
            myb.append(yesno[rnd.nextInt(2)]);
			myb.append(tab);
			
			// item reference	    
            myb.append(auctionId);
			myb.append(tab);
	    
            // seller
            myb.append(persons.getExistingId());
			myb.append(tab);
	    
            // skip annotation - too hard to generate - need to just get this done KT

            // KT - add category id XMark items can be in 1-10 categories
            // we allow an item to be in one category
            // category
            int catid = rnd.nextInt(303);
            myb.append(catid);
			myb.append(tab);

			// quantity
            int quantity = 1+rnd.nextInt(10);
            myb.append(quantity);
			myb.append(tab);
		
			// type
            myb.append(auction_type[rnd.nextInt(2)]);
            if(quantity>1 && rnd.nextBoolean())
	            myb.append(", Dutch"); // 
			myb.append(tab);
		
			// start time
            myb.append(cal.getTimeInSecs());
			myb.append(tab);

			// end time
            myb.append(openAuctions.getEndTime(auctionId));
			myb.append("\n");
        }
    }
    
    // append region AFRICA, ASIA, AUSTRALIA, EUROPE, NAMERICA, SAMERICA
    //Item contains:
    // location, quantity, name, payment, description, shipping, incategory+, mailbox)>
    // weird, item doesn't contain a reference to the seller, open_auction contains
    // a reference to the item and a reference to the seller
    
  
    private void generatePerson(MyBuffer myb, int numPersons) throws IOException {
        cal.incrementTime();

        for (int i=0; i<numPersons; i++) {
            p.generateValues(openAuctions); // person object is reusable now
	    	
	    	// person id
            myb.append(persons.getNewId()); 
			myb.append(tab);
	    
			// person name
            myb.append(p.m_stName);
			myb.append(tab);

			// email address
            myb.append(p.m_stEmail);
			myb.append(tab);

			// phone
            myb.append(p.m_stPhone);
			myb.append(tab);

			// street
            myb.append(p.m_address.m_stStreet);
			myb.append(tab);
		    
		    // city
            myb.append(p.m_address.m_stCity);
			myb.append(tab);
		    
			// country
            myb.append(p.m_address.m_stCountry);
			myb.append(tab);

			// province		    
            myb.append(p.m_address.m_stProvince);
			myb.append(tab);
		    
			// zip code
            myb.append(p.m_address.m_stZipcode);
			myb.append(tab);

			// credit card
            myb.append(p.m_stCreditcard);
			myb.append("\n");
        }
    }
}

