package edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata;

/**
 * This class represents a NFA, with stack-based execution.
 *  
 */
import java.io.Serializable;
import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;

public class Automata implements Serializable{

	//NFA variables
	protected List[] activeStack = new List[100];

	int activeStackPos = 1;

	protected List curSet = new ArrayList();

	protected List states = new ArrayList();

	//counting those tags which do not activate any states
	protected int errorStateCount = 0;

	protected State rootState;
	
	XATDag plan;
	
	public void setPlan(XATDag dag){
		plan = dag;
	}
	
	public XATDag getPlan(){
		return plan;
	}

	

	/**
	 * @param State
	 *            the start state of the automata
	 */
	public State getRootState() {
		return rootState;
	}

	/**
	 * set the start state for the automata
	 * 
	 * @param rootState
	 */
	public void setRootState(State rootState) {
		this.rootState = rootState;
	}

	/**
	 * This method drives the autamata by backtracking to the previous state.
	 * Since this is a NFA, the previous state may be a collection of states.
	 *  @boolean return whether a dest of bottommost tokennav has been encountered
	 */
	public boolean backtrack() {

		activeStackPos--;

		curSet = activeStack[activeStackPos - 1];
		
		return false;

	}

	
	

	/**
	 * get all the states in the automata
	 * 
	 * @return
	 */
	public List getStates() {
		return states;
	}

	/**
	 * remove a state (and its descendant state) from the automata note, we
	 * disallow a state shared by different precdecessors
	 * 
	 * @param state
	 *            State the state that is to be removed
	 */
	public void removeState(State state) {
		states.remove(state);
		/*
		 * List curStates = new ArrayList(); curStates.add(state); while
		 * (curStates.size() > 0) { List newCurStates = new ArrayList(); for
		 * (Iterator it = curStates.iterator(); it.hasNext();) { State curState =
		 * (State) it.next(); states.remove(curState); if (curState.transits !=
		 * null) for (Iterator it2 = curState.transits.keySet().iterator();
		 * it2.hasNext(); ) { String transitName = (String) it2.next();
		 * newCurStates.add(curState.getTransit(transitName)); } //end of it2
		 * 
		 * if (curState.toLambdaState != null)
		 * newCurStates.add(curState.getTransit(State.Lambda_Transit));
		 * 
		 * if (curState.toStarState != null)
		 * newCurStates.add(curState.getTransit(State.Star_Transit));
		 *  } //end of for it curStates = newCurStates;
		 *  } //end of while
		 */
	}

	/**
	 * This method drives the automata by transiting the curSet over the
	 * tagName.
	 * 
	 * @param tagName
	 *            String
	 */
	public void move(String tagName) {
		//get the new active set
		List target = new ArrayList();
		int curSetSize = curSet.size();
		for (int i = 0; i < curSetSize; i++) {
			State state = (State) curSet.get(i);
			state.updateTargetSet(tagName, target);
		}
		
		//if no state is transitioned to, an empty list is pushed onto the stack
		activeStack[activeStackPos] = target;
		activeStackPos++;
		
		curSet = target;
	}

	/**
	 * This method prints the automata in a nice way. Creation date: (2002-12-21
	 * 23:33:06)
	 */
	public void pprint() {

		System.out.println("---------automata------------");
		System.out.println(this);
		System.out.println("-----------------------------");
	}

	/**
	 * This method resets the automata, i.e., making the state 0 the active
	 * state.
	 */
	public void reset() {
		if (curSet==null)
			curSet = new ArrayList();
		else curSet.clear();
		activeStackPos = 1;

		//make s0 to be curState
		curSet.add(states.get(0));
		activeStack[activeStackPos] = curSet;

	}

	/**
	 * 
	 * @return String
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (Iterator it = states.iterator(); it.hasNext();) {
			State state = (State) it.next();
			if (state != null)
				sb.append(state.toString() + "\n");
		}
		return sb.toString();
	}
}