package edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.*;
import edu.wpi.cs.dsrg.xmldb.xat.testing.TestExecBaseLocal;

import java.io.Serializable;
import java.util.*;

/**
 * be cautious: super() is different from StaticAutomata() may have problems
 * This class adds more functionality to a traditional NFA, such as help
 * information for deciding whether a node should be materialized and trace the
 * parent operator in case of recursive elements
 * 
 * 
 * @author: Hong Su
 *  
 */

public class ExtendedAutomata extends Automata implements Serializable {

	private int navOpCount = 0;

	private int[] counts = new int[3];

	//give an operator, find its associated state
	private HashMap stateHash = new HashMap();

	private Finer_StreamStoreMgr storeMgr;

	private ClassExecTimeKeeper triggerRunContextKeeper = new ClassExecTimeKeeper(
			this, "triggerRunContext");

	//a stack to store token id
	private int[] tokenStack = new int[100];

	private int tokenPosition = 0;

	//the state which will be activated by a bottom destiantion node
	private ExtendedState bottomDestState;

	//map a tokennav to a list of states that encode its path
	private Map tokenNavMaptoStates = new HashMap();
	public long timeSum=0;;

	/**
	 * Automata constructor comment.
	 */
	public ExtendedAutomata(Finer_StreamStoreMgr storeMgr) {
		this.storeMgr = storeMgr;
	}

	public ClassExecTimeKeeper getTriggerRunContextKeeper() {
		return triggerRunContextKeeper;
	}

	public void registerTokenNavToStates(PureNavigateAutoStream tokenNav,
			List statesEncodingPath) {
		tokenNavMaptoStates.put(tokenNav, statesEncodingPath);

	}

	public Map getTokenNavMaptoStates() {
		return tokenNavMaptoStates;
	}

	/**
	 * remove the state from the inner hashtable of the entry for op refer to
	 * addIntoStateHash
	 * 
	 * @param op
	 *            XATStreamOperator
	 * @param state
	 *            ExtendedState
	 */
	public void resetStateHash(XATStreamOperator op, ExtendedState state) {
		HashMap innerHash = (HashMap) stateHash.get(op);

		for (Iterator it = innerHash.keySet().iterator(); it.hasNext();) {
			List states = (List) innerHash.get(it.next());
			if (states.contains(state))
				states.remove(state);
		}
	}

	/**
	 * a nav/extract op is associated with the automata states which corresponds
	 * to its destination given an op, we have a hashmap this inner hashmap is
	 * hashed on the name of the last navigation step
	 * 
	 * @param op
	 *            XATStreamOperator
	 * @param state
	 *            State
	 */
	public void addIntoStateHash(XATStreamOperator op, ExtendedState state) {
		HashMap innerHash = (HashMap) stateHash.get(op);
		if (innerHash == null)
			//not set up yet
			innerHash = new HashMap();

		//which transition leads to this state
		ExtendedState parentState = state.backward(1);

		if (parentState == null) {
			//			state must be the starting state
			//inner hash, given the last navigation step name, may map to
			// multiple auto states
			List states = new ArrayList();
			states.add(state);
			innerHash.put("", states);
			stateHash.put(op, innerHash);
			return;
		}

		//though a state may have multiple parent state,
		//all its parent state must all go to it via the same transition tag
		// name
		String transitName = "";

		if (parentState.transits != null)
			for (Iterator it = parentState.transits.keySet().iterator(); it
					.hasNext();) {
				transitName = (String) it.next();
				if (parentState.transits.get(transitName) == state)
					break;
			}
		//the last navigation step cannot be // or lambda
		if (transitName.equals("")) {
			//check whether it is a star
			if (parentState.toStarState != null)
				transitName = State.Star_Transit;
		}

		List states = (List) innerHash.get(transitName);
		if (states == null)
			states = new ArrayList();
		states.add(state);
		innerHash.put(transitName, states);
		stateHash.put(op, innerHash);
	}

	/**
	 * add a state into the automata
	 * 
	 * @param State
	 */
	public void addIntoStates(State state) {
		states.add(state);

	}

	/**
	 * This method adds states and transitions based on the Navigation Steps. It
	 * reuse states and transitions for common prefixes.
	 * 
	 * @param curParent
	 *            state
	 * @param path
	 *            NavigationStep[]
	 * @param tokenNavStates
	 *            List the states encoding the path
	 *  
	 */
	public State addPath(State curParent, NavigationStep[] path,
			List tokenNavStates) {

		//from where to add transition
		//ExtendedState curParent = (ExtendedState) states.elementAt(childId);

		//for each stemp
		for (int i = 0; i < path.length; i++) {
			//NodeStep, ie, the child axis
			if (path[i] instanceof NodeStep) {

				String stepName = path[i].getStepValue();

				ExtendedState temp = (ExtendedState) curParent
						.getTransit(stepName);

				if (temp == null) {
					temp = new ExtendedState(this);
					states.add(temp);
					curParent.addTransit(stepName, temp);
					tokenNavStates.add(temp);
				}
				curParent = temp;
			} //NodeStep

			//DescendantStep, i.e., the descendant axis
			else if (path[i] instanceof DescendantStep) {

				//add lambda transit
				ExtendedState temp = (ExtendedState) curParent
						.getTransit(State.Lambda_Transit);

				if (temp == null) {
					temp = new ExtendedState(this);
					states.add(temp);
					tokenNavStates.add(temp);
					curParent.addTransit(State.Lambda_Transit, temp);
				} else
					tokenNavStates.add(temp);

				curParent = temp;

				//add star self transit
				temp = (ExtendedState) curParent
						.getTransit(State.Star_Self_Transit);
				if (temp == null) {
					curParent.addTransit(State.Star_Self_Transit, curParent);
				}
			} //DescendantStep

			//TextStep, VarStep, AttStep...?
			else {
				//do nothing
			}
		}
		//  return curParent.getId();
		return curParent;
	}

	/**
	 * This method adds states and transitions based on the Navigation Steps. It
	 * reuse states and transitions for common prefixes.
	 * 
	 * @param curParent
	 *            state
	 * @param path
	 *            NavigationStep[]
	 *  
	 */
	public State addPath(State curParent, NavigationStep[] path) {

		//from where to add transition
		//ExtendedState curParent = (ExtendedState) states.elementAt(childId);

		//for each stemp
		for (int i = 0; i < path.length; i++) {
			//NodeStep, ie, the child axis
			if (path[i] instanceof NodeStep) {

				String stepName = path[i].getStepValue();

				ExtendedState temp = (ExtendedState) curParent
						.getTransit(stepName);

				if (temp == null) {
					temp = new ExtendedState(this);
					states.add(temp);
					curParent.addTransit(stepName, temp);
				}
				curParent = temp;
			} //NodeStep

			//DescendantStep, i.e., the descendant axis
			else if (path[i] instanceof DescendantStep) {

				//add lambda transit
				ExtendedState temp = (ExtendedState) curParent
						.getTransit(State.Lambda_Transit);

				if (temp == null) {
					temp = new ExtendedState(this);
					states.add(temp);
					curParent.addTransit(State.Lambda_Transit, temp);
				}

				curParent = temp;

				//add star self transit
				temp = (ExtendedState) curParent
						.getTransit(State.Star_Self_Transit);
				if (temp == null) {
					curParent.addTransit(State.Star_Self_Transit, curParent);
				}
			} //DescendantStep

			//TextStep, VarStep, AttStep...?
			else {
				//do nothing
			}
		}
		//  return curParent.getId();
		return curParent;
	}

	/**
	 * This method drives the autamata by backtracking to the previous state.
	 * Since this is a NFA, the previous state may be a collection of states.
	 *  
	 */
	public boolean backtrack() {
		boolean bottommostDestBackTracked = false;

		//maintain the token id stack
		storeMgr.descreaseStackPosition();

		for (Iterator it = curSet.iterator(); it.hasNext();) {
			ExtendedState state = (ExtendedState) it.next();

			//if the state corresponds to dest node of the bottommost tokenNav
			//increment the counter
			if (state == bottomDestState)
				bottommostDestBackTracked = true;

			counts[ExtractStream.DOM] = counts[ExtractStream.DOM]
					- state.getDomCount();
			counts[ExtractStream.TYPED_VALUE] = counts[ExtractStream.TYPED_VALUE]
					- state.getTypedValCount();

			/** ***commented out for statistics*** */
			/*
			 * List destNavOps = state.getDestNavOps(); if (destNavOps.size() >
			 * 0) { PureNavigateAutoStream destNav = (PureNavigateAutoStream)
			 * destNavOps .get(0); destNav.getTimeKeeper().pauseTokenCount(
			 * tokenStack[tokenPosition]); }
			 */

			/** ***end of commenting out for statistics*** */
		}

		super.backtrack();

		return bottommostDestBackTracked;

	}

	/**
	 * get the number of parent nodes this element the automata currently
	 * processes is in parent nodes need to be stored in a complete tree manner
	 * 
	 * @return int
	 */
	public int getCompleteTreeCount() {
		return counts[ExtractStream.COMPLETE_TREE];
	}

	/**
	 * reset the complete tree count to n
	 * 
	 * @param n
	 *            int
	 */
	public void resetCompleteTreeCount(int n) {
		counts[ExtractStream.COMPLETE_TREE] = n;
	}

	/**
	 * get the number of parent nodes this element the automata currently
	 * processes is in parent nodes need to be stored in a dom manner
	 * 
	 * @return int
	 */
	public int getDomCount() {
		return counts[ExtractStream.DOM];
	}

	/**
	 * reset the dom count to n
	 * 
	 * @param n
	 *            int
	 */
	public void resetDomCount(int n) {
		counts[ExtractStream.DOM] = n;
	}

	public void minusDomCount() {
		counts[ExtractStream.DOM]--;
	}

	/**
	 * get the number of parent nodes this element the automata currently
	 * processes is in parent nodes need to be stored in a typed val manner
	 * 
	 * @return int
	 */
	public int getTypedValCount() {
		return counts[ExtractStream.TYPED_VALUE];
	}

	/**
	 * reset the typed val count to n
	 * 
	 * @param n
	 *            int
	 */
	public void resetTypedValCount(int n) {
		counts[ExtractStream.TYPED_VALUE] = n;
	}

	public void minusTypedValCount() {
		counts[ExtractStream.TYPED_VALUE]--;
	}

	/**
	 * get the destination node's states for op
	 * 
	 * @return List
	 * @param op
	 *            XATStreamOperator
	 */
	public List getFrStateHash(XATStreamOperator op) {
		List destStates = new ArrayList();
		HashMap innerHash = (HashMap) stateHash.get(op);
		//by default, there is only one entry in innerHash
		if (innerHash.keySet().size() > 1)
			System.out.println("inappropariate, tagname not specified");

		String transitName = "";
		for (Iterator it = innerHash.keySet().iterator(); it.hasNext();) {
			transitName = (String) it.next();
			destStates.addAll((List) innerHash.get(transitName));
		}

		return destStates;
	}

	/**
	 * an op may have multiple destination nodes find the state of the
	 * destination node with tagName
	 * 
	 * @return List
	 * @param op
	 *            XATStreamOperator
	 * @param tagName
	 *            String
	 */
	public List getFrStateHash(XATStreamOperator op, String tagName) {
		HashMap innerHash = (HashMap) stateHash.get(op);
		return (List) innerHash.get(tagName);
	}

	/**
	 * @return StreamStoreMgr
	 */
	public Finer_StreamStoreMgr getStoreMgr() {
		return storeMgr;
	}

	/**
	 * This method drives the automata by transiting the curSet over the
	 * tagName.
	 * 
	 * @param tagName
	 *            String
	 */
	public void move(String tagName) {

		//get the new active set
		super.move(tagName);

		storeMgr.increaseStackPosition();

		for (int i = 0; i < curSet.size(); i++) {

			ExtendedState state = (ExtendedState) curSet.get(i);

			/** **commented out for schema-blindless*** */
			for (int j = 0; j < state.opsResetPartitionChecked.size(); j++)
				((SelfPartitionDelimiInvoking) state.opsResetPartitionChecked
						.get(j)).setPartitionChecked(false);

			if (state.suspend)
				state.resumeAllTransits();
			/** ***commented out for schema-blindless*** */

			/** ******************************************** */
	//		System.out.println("state's Domcount " + state.getDomCount());
			
			counts[ExtractStream.DOM] = counts[ExtractStream.DOM]
					+ state.getDomCount();
			
	//		System.out.println("ExtractStream.DOM " + ExtractStream.DOM);
			
	//		System.out.println("state's TypedValCount " + state.getTypedValCount());
			counts[ExtractStream.TYPED_VALUE] = counts[ExtractStream.TYPED_VALUE]
					+ state.getTypedValCount();
			
		//	System.out.println("ExtractStream.TYPED_VALUE " + ExtractStream.TYPED_VALUE);

			}

	}

	/**
	 * Keeps a stack of token ids.
	 * 
	 * @param id
	 *            int
	 */
	public void setCurTokenId(int id) {
		tokenPosition++;
		tokenStack[tokenPosition] = id;
	}

	/**
	 * @param newStoreMgr
	 *            StreamStoreMgr
	 */
	public void setStoreMgr(Finer_StreamStoreMgr newStoreMgr) {
		storeMgr = newStoreMgr;
	}

	/**
	 * @return String
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (Iterator it = states.iterator(); it.hasNext();) {
			ExtendedState state = (ExtendedState) it.next();
			if (state != null)
				if (state == bottomDestState)
					sb.append(state.toString() + "(b)" + "\n");
				else
					sb.append(state.toString() + "\n");

		}
		return sb.toString();
	}

	/**
	 * when a state is reached, trigger its associated operators to run
	 * 
	 * @param stackAction
	 *            int push/pop/slip
	 * @param unitContent
	 *            String the tag name of the token that triggers the operation
	 * @param attrs
	 *            Attributes the attributes of the token
	 */
	public void triggerOpsToRunPush(int tokenId, int startTokenId) {

		for (int i = 0; i < curSet.size(); i++) {

			ExtendedState state = (ExtendedState) curSet.get(i);

			//in case some operators are take out from the groupbyOps
			//in SQO because they've already been performed
			//state.groupbyOps = state.suspendGroupbyOps;

			//state.suspendGroupbyOps =
			// (ArrayList)state.suspendGroupbyOps.clone();
			List destTokenNavs = state.getDestNavOps();
			for (int j=0; j<destTokenNavs.size(); j++){
				PureNavigateAutoStream destTokenNav = (PureNavigateAutoStream)destTokenNavs.get(j);
				destTokenNav.runPushStack(tokenId, startTokenId);
			}
			
			List extractOps = state.getExtractOps();

			if (extractOps != null)

				for (int j = 0; j < extractOps.size(); j++) {
					ExtractStream extractOp = (ExtractStream) extractOps.get(j);
					extractOp.runPushStack(tokenId, startTokenId);
				}

		}
	}
	/**
	 * when a state is reached, trigger its associated operators to run
	 * 
	 * @param stackAction
	 *            int push/pop/slip
	 * @param unitContent
	 *            String the tag name of the token that triggers the operation
	 * @param attrs
	 *            Attributes the attributes of the token
	 */
	public void triggerOpsToRunPushWithRecur(int tokenId, int startTokenId) {

		boolean queryHasRecur = true;
		
		for (int i = 0; i < curSet.size(); i++) {
			
			
			ExtendedState state = (ExtendedState) curSet.get(i);

			//in case some operators are take out from the groupbyOps
			//in SQO because they've already been performed
			//state.groupbyOps = state.suspendGroupbyOps;

			//state.suspendGroupbyOps =
			// (ArrayList)state.suspendGroupbyOps.clone();
			List destTokenNavs = state.getDestNavOps();
			for (int j=0; j<destTokenNavs.size(); j++){
				if (TestExecBaseLocal.queryIsRecur) 
				{
				PureNavigateAutoStreamWithRecur destTokenNav = (PureNavigateAutoStreamWithRecur)destTokenNavs.get(j);	
				
				destTokenNav.runPushStack(tokenId, startTokenId);
				}
				else
				{
					PureNavigateAutoStream destTokenNav = (PureNavigateAutoStream)destTokenNavs.get(j);
					 destTokenNav.runPushStack(tokenId, startTokenId);
				}	
				
				
				 
			}
			
			List extractOps = state.getExtractOps();

			if (extractOps != null)

				for (int j = 0; j < extractOps.size(); j++) {
					ExtractStream extractOp = (ExtractStream) extractOps.get(j);
					//Judge whether the query is recursive
					if (TestExecBaseLocal.queryIsRecur) {
					if (extractOp instanceof ExtractByUnnestWithRecur)
						{
							((ExtractByUnnestWithRecur)extractOp).runPushStackWithRecur(tokenId, startTokenId);
						}
					if (extractOp instanceof ExtractByCollectionStream)
						{
					/*	ExtractByCollectionWithRecur extractCollOp = new ExtractByCollectionWithRecur((ExtractByCollectionStream)extractOp);
						extractCollOp.runPushStack(tokenId, startTokenId);*/
						((ExtractByCollectionWithRecur)extractOp).runPushStackWithRecur(tokenId, startTokenId);
						}
					}
					else
					{
						
						extractOp.runPushStack(tokenId, startTokenId);	
					}
						
					//extractOp.runPushStackWithRecur(tokenId, startTokenId);
				}

		}
	}
	public void triggerSuspensionPop() {
		//triggerRunContextKeeper.start();
		for (int i = 0; i < curSet.size(); i++) {
			ExtendedState state = (ExtendedState) curSet.get(i);

			List suspendComps = state.getSuspensionEdgesAtPop();

			//coupleOp should be activated when its invoking nav encounters the
			// beginning tag
			//the activation of the state indicates that coupledOp can be
			// wrapped up
			for (int j = 0; j < suspendComps.size(); j++) {
				//wrap each coupledOp
				SuspensionEdge suspendComp = (SuspensionEdge) suspendComps
						.get(j);

				int occurrenceNum = suspendComp.getOccurrenceNum();

				SelfPartitionDelimiInvoking checkOp = suspendComp
						.getCheckedOp();

				//initiating operator is also checkOp
				//which must be an extract
				//check whether the occurrence number has been reached
				if (((ExtractStream) checkOp).getItemCountWithinEntry() == occurrenceNum) {
					checkOp.selfPartitionDelimiInvokingRun(suspendComp
							.getPartitionStates(), suspendComp);
					//	checkOp.selfPartitionDelimiInvokingRun(
					//		suspendComp.getPartitionState());
				} //end of if (
				//	} //end of for iterator it2
			}

			//triggerRunContextKeeper.pause();
		}
	}

	public void triggerSuspensionPush() {
		//triggerRunContextKeeper.start();

		for (int i = 0; i < curSet.size(); i++) {
			ExtendedState state = (ExtendedState) curSet.get(i);

			List suspendComps = state.getSuspensionEdgesAtPush();

			//coupleOp should be activated when its invoking nav encounters
			// the beginning tag
			//the activation of the state indicates that coupledOp can be
			// wrapped up
			//triggerRunContextKeeper.start();
			for (int j = 0; j < suspendComps.size(); j++) {
				//wrap each coupledOp

				SuspensionEdge suspendComp = (SuspensionEdge) suspendComps
						.get(j);

				SelfPartitionDelimiInvoking checkOp = suspendComp
						.getCheckedOp();

				List partitionStates = suspendComp.getPartitionStates();

				if (checkOp == null) {
					//triggered by an exclusive type
					//definitely there would not be any out on this
					// partition
					//suspend this partition
					int partitionStateNum = partitionStates.size();
					for (int k = 0; k < partitionStateNum - 1; k++)
						((ExtendedState) partitionStates.get(k))
								.suspendAllTransits(suspendComp, false);
					//clean temp for the last one
					((ExtendedState) partitionStates.get(partitionStateNum - 1))
							.suspendAllTransits(suspendComp, true);
					break;

				} else if (!checkOp.selfPartitionDelimiInvokingRun(
						partitionStates, suspendComp))
					//if checkop fail, the whole thing fails, no need to check
					// others
					break;

			}

		}
		//triggerRunContextKeeper.pause();
	}

	/**
	 * when a state is reached, trigger its associated operators to run
	 * 
	 * @param stackAction
	 *            int push/pop/slip
	 * @param unitContent
	 *            String the tag name of the token that triggers the operation
	 * @param attrs
	 *            Attributes the attributes of the token
	 */
	public void triggerOpsToRunPop(int tokenId, int startTokenId) {

		

		for (int i = 0; i < curSet.size(); i++) {

			ExtendedState state = (ExtendedState) curSet.get(i);
			
			List destTokenNavs = state.getDestNavOps();
			for (int j=0; j<destTokenNavs.size(); j++){
				PureNavigateAutoStream tokenNav = (PureNavigateAutoStream)destTokenNavs.get(j);
				tokenNav.runPopStack(tokenId, startTokenId);
			}

			List popExtractOps = state.getPopExtractOps();

			for (int j = 0; j < popExtractOps.size(); j++) {
				ExtractStream extractOp = (ExtractStream) popExtractOps.get(j);
				//if (!((SelfPartitionDelimiInvoking)extractOp).getPartitionChecked())
				//not performed within this partition yet
				extractOp.runPopStack(tokenId, startTokenId);
			}

			//moreover, wrap up all ops that need group-by-end info

			List groupbyOps = state.getGroupbyOps();
			
		//	triggerRunContextKeeper.start();
			if (groupbyOps != null) {
				
				for (int k = 0; k < groupbyOps.size(); k++) {

					DataDrivenInvoking op = (DataDrivenInvoking) groupbyOps
							.get(k);
					

					//if (!((SelfPartitionDelimiInvoking)op).getPartitionChecked())
					
					
					op.dataDrivenInvokingRun(null, 0);
					// op.dataDrivenInvokingRun(null, 0);

				}

				//triggerRunContextKeeper.pause();
			
				
				

				//		partitionRunKeeper.pause();
			} //end of if (groupbyOps.size() > 0)

		} //end of for (Iterator it = curSet.iterator(); it.hasNext();)
	}
	
	/**
	 * when a state is reached, trigger its associated operators to run
	 * 
	 * @param stackAction
	 *            int push/pop/slip
	 * @param unitContent
	 *            String the tag name of the token that triggers the operation
	 * @param attrs
	 *            Attributes the attributes of the token
	 */
	public void triggerOpsToRunPopWithRecur(int tokenId, int startTokenId) {

			

		for (int i = 0; i < curSet.size(); i++) {

			ExtendedState state = (ExtendedState) curSet.get(i);
			
			List destTokenNavs = state.getDestNavOps();
			for (int j=0; j<destTokenNavs.size(); j++){
				/*if (queryHasRecur )
				{
				PureNavigateAutoStreamWithRecur tokenNav = (PureNavigateAutoStreamWithRecur)destTokenNavs.get(j);	
				tokenNav.runPopStack(tokenId, startTokenId);
				}*/
				// Either query is recursive or not, just go this step
				PureNavigateAutoStream tokenNav = (PureNavigateAutoStream)destTokenNavs.get(j);	
				tokenNav.runPopStack(tokenId, startTokenId);
			}
// Comments hong's code out to run experiment
			List popExtractOps;
			if (TestExecBaseLocal.queryIsRecur)
				
				popExtractOps = state.getExtractOps();
			else
				popExtractOps = state.getPopExtractOps();
			

			for (int j = 0; j < popExtractOps.size(); j++) {
				ExtractStream extractOp = (ExtractStream) popExtractOps.get(j);
				//if (!((SelfPartitionDelimiInvoking)extractOp).getPartitionChecked())
				//not performed within this partition yet
				if (TestExecBaseLocal.queryIsRecur) {
					
				
				if (extractOp instanceof ExtractByUnnestWithRecur)
					((ExtractByUnnestWithRecur)extractOp).runPopStackWithRecur(tokenId, startTokenId);
				if (extractOp instanceof ExtractByCollectionWithRecur)
					((ExtractByCollectionWithRecur)extractOp).runPopStackWithRecur(tokenId, startTokenId);
				}
				else
				extractOp.runPopStack(tokenId, startTokenId);
			}

			//moreover, wrap up all ops that need group-by-end info

			List groupbyOps = state.getGroupbyOps();
			triggerRunContextKeeper.start();

			if (groupbyOps != null) {

				for (int k = 0; k < groupbyOps.size(); k++) {

					DataDrivenInvoking op = (DataDrivenInvoking) groupbyOps
							.get(k);
					
					
					//if (!((SelfPartitionDelimiInvoking)op).getPartitionChecked())
					
					op.dataDrivenInvokingRun(null, 0);

				}

				triggerRunContextKeeper.pause();
				timeSum +=triggerRunContextKeeper.getTime();
				//		partitionRunKeeper.pause();
			} //end of if (groupbyOps.size() > 0)

		} //end of for (Iterator it = curSet.iterator(); it.hasNext();)
	}

	/**
	 * get the count of the specified storing method
	 * 
	 * @param storingMethod
	 * @return
	 */
	public int getCount(int storingMethod) {
		return counts[storingMethod];

	}

	public ExtendedState getBottomDestState() {
		return bottomDestState;
	}

	public void setBottomDestState(ExtendedState newBottomDestState) {
		bottomDestState = newBottomDestState;
	}

}