package edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.FindOpVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.SelfPartitionDelimiInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStreamImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.SuspensionEdge;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.common.XATStreamOpRunCore;

public class ExtendedState extends State implements Serializable {
	//the automata the state is in
	private ExtendedAutomata automata;

	// all the operators associated with the state
	private ArrayList extractOps = new ArrayList();

	//the extract ops that are needed to be invoked when end token is
	// encountered
	private ArrayList popExtractOps = new ArrayList();

	//the number of complete representations the extract operators whose
	// destinations
	//are on this state or the ancestor of this state
	private int completeCount = 0;

	private int completeCountCopy = 0;

	//the number of typed val representations the extractops are to store
	private int typedValCount = 0;

	private int typedValCountCopy = 0;

	//the number of dom representations the extractops are to store
	private int domCount = 0;

	protected int domCountCopy = 0;

	private ArrayList suspendExtractOps = new ArrayList();

	ArrayList groupbyOps = new ArrayList();

	ArrayList suspendGroupbyOps = new ArrayList();

	//a list of operators in groupbyops that need to be cleaned temp results
	// when the state is suspended
	//private List groupbyOpsNeededClean = new ArrayList();
	//a hashmap, given a suspension edge, find which ops need to be cleaned
	private HashMap groupbyOpsNeededClean = new HashMap();

	//the nav ops whose destination is mapped to this state
	private List destNavOps = new ArrayList();

	//the nav ops whose entry is mapped to this state
	private List entryNavOps = new ArrayList();

	//the suspension edges that should be check when a push stack action
	// happens
	private List suspensionEdgesAtPush = new ArrayList();

	//	the suspension edges that should be check when a push stack action
	// happens,
	//arise from the occurrence one
	private List suspensionEdgesAtPop = new ArrayList();

	//from the end of the path to which step is the navigation suspended
	//e.g., destination a/b/c/d, if suspendedNavOpStepCount = 2, means from
	// navigationStep[2], i.e., c, is suspended
	private int suspendedNavOpStepCount = -1;

	private ExtractStream suspendedExtractOp;

	private boolean extractOpJustSuspended = false;

	//the state where the current state is transitted from by one single step
	//we can trace back from this state (each state can have only one
	// parentstate)
	private ExtendedState parentState;

	private boolean resumeNavOp = false;

	//navOpCount stores the count of the navOp whose path would go thru this
	// state
	private HashMap navOpCount = new HashMap();

	//to suspend a navigate, manage an arraylist, each element in the arraylist
	// corresponds to a shortcircuitable navigation step
	//the element itself is an object[3], object[0] = tag name, object[1] =
	// state, object[2] = childstate where the transit should be got rid of
	//i.e., from object[1] via object[0], we can reach object[2]
	private List suspendableStepMgr;

	private HashMap suspendTransits;
	protected State suspendToLambdaState;
	protected State suspendToStarState;
	protected State suspendToSelfStarState;

	//operators whose partition checked status need to be set back to false
	//at the beginning of each partition
	//this list not only includes operators that are directly associated with
	//this state, must those operators that are associated with the descendant
	//states of this state
	List opsResetPartitionChecked = new ArrayList();

	boolean suspend = false;

	//whether assocaited with a dom/string extract operator
	private List storingMethods = new ArrayList();
	
	public ExtendedState(ExtendedAutomata automata) {
		super();
		this.automata = automata;
	}

	public ExtendedAutomata getAutomata() {
		return automata;
	}

	public ExtendedState getParentState() {
		return parentState;
	}
	
	public void setParentState(ExtendedState parentState){
		this.parentState = parentState;
	}

	public HashMap getGroupbyOpsNeededClean() {
		return groupbyOpsNeededClean;
	}

	/**
	 * 
	 * @param tagName
	 * @return
	 */
	public int getNavOpCountOn(String tagName) {
		Integer count = (Integer) navOpCount.get(tagName);

		return count.intValue();
	}

	/**
	 * add an extract op, a state may be associated with multiple extract ops
	 * 
	 * @param op
	 *            edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator
	 */
	public void addExtractOp(ExtractStream op) {
		extractOps.add(op);
		//make a clone into suspenedExtractOp for quick suspension and resuming
		suspendExtractOps.add(op);
		if (op.isCompleteTree()) {
			completeCount++;
			completeCountCopy = completeCount;
			Set successors = this.getSuccessorStates();
			for (Iterator it = successors.iterator(); it.hasNext();)
				((ExtendedState) it.next()).completeCount++;

			storingMethods.add(new Integer(ExtractStream.COMPLETE_TREE));
		}
		if (op.isDom()) {
			domCount++;
			domCountCopy = domCount;
			Set successors = this.getSuccessorStates();
			for (Iterator it = successors.iterator(); it.hasNext();)
				((ExtendedState) it.next()).completeCount++;

			storingMethods.add(new Integer(ExtractStream.DOM));
		}

		if (op.isTypedValue()) {
			typedValCount++;
			typedValCountCopy = typedValCount;
			Set successors = this.getSuccessorStates();
			for (Iterator it = successors.iterator(); it.hasNext();)
				((ExtendedState) it.next()).completeCount++;

			storingMethods.add(new Integer(ExtractStream.TYPED_VALUE));
		}
	}
	
	public void removeExtractOp(ExtractStream op){
		extractOps.remove(op);
		suspendExtractOps.remove(op);
		if (op.isCompleteTree()) {
			completeCount--;
			completeCountCopy = completeCount;
			Set successors = this.getSuccessorStates();
			for (Iterator it = successors.iterator(); it.hasNext();)
				((ExtendedState) it.next()).completeCount--;

			for (Iterator it = storingMethods.iterator(); it.hasNext();){
				Integer method = (Integer)it.next();
				if (method.intValue() == ExtractStream.COMPLETE_TREE)
					it.remove();
			}
		}
		if (op.isDom()) {
			domCount--;
			domCountCopy = domCount;
			Set successors = this.getSuccessorStates();
			for (Iterator it = successors.iterator(); it.hasNext();)
				((ExtendedState) it.next()).completeCount++;

			for (Iterator it = storingMethods.iterator(); it.hasNext();){
				Integer method = (Integer)it.next();
				if (method.intValue() == ExtractStream.DOM)
					it.remove();
			}
		}

		if (op.isTypedValue()) {
			typedValCount--;
			typedValCountCopy = typedValCount;
			Set successors = this.getSuccessorStates();
			for (Iterator it = successors.iterator(); it.hasNext();)
				((ExtendedState) it.next()).completeCount++;

			for (Iterator it = storingMethods.iterator(); it.hasNext();){
				Integer method = (Integer)it.next();
				if (method.intValue() == ExtractStream.TYPED_VALUE)
					it.remove();
			}
		}
		
	}

	public void addPopExtractOp(ExtractStream op) {
		popExtractOps.add(op);
	}

	public List getPopExtractOps() {
		return popExtractOps;
	}

	public void setDestNavOps(List destNavOps) {
		destNavOps = this.destNavOps;
	}

	
	public void addGroupbyOp(XATStreamOperator op) {

		List ancestors = FindOpVisitor.findAncestorOps((DagNode) op.getNode());

		int k = -1;

		for (int i = 0; i < groupbyOps.size(); i++) {
			if (ancestors.contains(groupbyOps.get(i))) {
				//insert before the first ancestor it finds
				k = i;
				break;
			}
		}

		if (k != -1) {
			//groupbyOps.insertElementAt(op, k);
			groupbyOps.add(k, op);
			suspendGroupbyOps.add(k, op);
		} else {
			groupbyOps.add(op);
			suspendGroupbyOps.add(op);
		}
	}
	
	public void removeGroupbyOp(XATStreamOperator op){
		groupbyOps.remove(op);
		suspendGroupbyOps.remove(op);
	}

	/*
	 * this state initiates an suspension checking
	 */
	public void addSuspensionEdgeAtPush(SuspensionEdge edge) {
		suspensionEdgesAtPush.add(edge);
		//	this.setUpGroupByOpsNeededClean();
	}

	public List getSuspensionEdgesAtPush() {
		return suspensionEdgesAtPush;
	}

	public void addSuspensionEdgeAtPop(SuspensionEdge edge) {
		suspensionEdgesAtPop.add(edge);
		//	this.setUpGroupByOpsNeededClean();
	}

	public List getSuspensionEdgesAtPop() {
		return suspensionEdgesAtPop;
	}

	public void setEntryNavOps(List entryNavOps) {
		this.entryNavOps = entryNavOps;
	}

	/**
	 * from current state, add a transition step by tagname between it and the
	 * "state" Creation date: (4/29/2003 11:16:11 AM)
	 * 
	 * @param tagName
	 *            java.lang.String
	 * @param state
	 *            edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.State
	 */
	public void addTransit(String tagName, State state) {
		super.addTransit(tagName, state);

		if (!resumeNavOp) {
			//navOpCount stores the count of the navOp whose path would go thru
			// this state
			Integer relyingNavOpCount = (Integer) navOpCount.get(tagName);

			if (relyingNavOpCount == null)
				navOpCount.put(tagName, new Integer(1));
			else
				navOpCount.put(tagName, new Integer(relyingNavOpCount
						.intValue() + 1));
		}

		((ExtendedState) state).parentState = this;
		
		

	}

	/**
	 * from the current state, delete the transition step by tagName Creation
	 * date: (4/29/2003 10:59:37 AM)
	 * 
	 * @param tagName
	 *            java.lang.String
	 */
	public void deleteTransit(String tagName) {

		if (tagName.equals(State.Lambda_Transit)) {

			toLambdaState = null;

		} else if (tagName.equals(State.Star_Transit)) {
			toStarState = null;

		} else if (tagName.equals(State.Star_Self_Transit)) {
			toSelfStarState = null;

		} else {

			if (transits == null)
				transits = new HashMap();

			if (transits.get(tagName) == null) {
				System.out.println("Exception: no such tagname!");
				System.exit(-1);
			}
			transits.remove(tagName);
		}

	}
	
	/**
	 * get all associated extract ops. Creation date: (2/2/2003 6:16:11 PM)
	 * 
	 * @return java.util.List
	 */
	public List getExtractOps() {
		return extractOps;
	}

	/**
	 * get all associated group by ops Creation date: (2/2/2003 6:16:17 PM)
	 * 
	 * @return List
	 */
	public List getGroupbyOps() {
		return groupbyOps;
	}

	/**
	 * get the nav ops Creation date: (2/2/2003 6:51:17 PM)
	 * 
	 * @return List
	 */
	public List getDestNavOps() {
		return destNavOps;
	}

	public List getEntryNavOps() {
		return entryNavOps;
	}

	public ExtractStream getSuspendedExtractOp() {
		return suspendedExtractOp;
	}

	public boolean isExtractOpJustSuspended() {
		return extractOpJustSuspended;
	}

	/**
	 * resume extract op, i.e., start to extract again Creation date: (2/20/2003
	 * 8:01:39 PM)
	 */
	public void resumeExtractOp() {

		//resume the navigation operator

		//navOp = suspendedNavOp;

		/*
		 * for (Iterator i = ((DagNode)navOp.getNode()).getParents();
		 * i.hasNext(); ){
		 * 
		 * XATStreamOperator op =
		 * (XATStreamOperator)((DagNode)i.next()).getOperator();
		 * 
		 * if (op instanceof ExtractStream){
		 * 
		 * //resume the corresponding extraction of the suspended navigation
		 * operator extractOps.add(op); } }
		 */

		extractOps.add(suspendedExtractOp);

		//suspendedNavOp = null;

		suspendedExtractOp = null;

	}

	/**
	 * We donot support resuming of multiple navops Creation date: (2/20/2003
	 * 8:01:39 PM)
	 */
	public void resumeNavOp() {

		/*
		 * NavigationStep[] destination = navOp.getDestination();
		 * 
		 * int stepPos = destination.length - 1;
		 * 
		 * ExtendedState pState = this; ExtendedState cState = this;
		 * 
		 * while (stepPos>= suspendedNavOpStepCount) { String currentStep =
		 * destination[stepPos].getStepValue();
		 * 
		 * cState = pState; pState = pState.parentState;
		 * 
		 * if (((Integer) pState.navOpCount.get(currentStep)).intValue() == 1) {
		 * 
		 * pState.setResumeNavOp(true);
		 * 
		 * pState.addTransit(currentStep, cState);
		 * 
		 * pState.setResumeNavOp(false); }
		 * 
		 * stepPos--; }
		 */

		for (Iterator i = suspendableStepMgr.iterator(); i.hasNext();) {
			Object[] pair = (Object[]) i.next();
			ExtendedState state = (ExtendedState) pair[1];
			state.setResumeNavOp(true);
			state.addTransit((String) pair[0], (ExtendedState) pair[2]);
			state.setResumeNavOp(false);
		}

		//reset
		//suspendedNavOpStepCount = -1;

	}

	/**
	 * Creation date: (2/21/2003 8:29:43 PM)
	 * 
	 * @param newNavOpJustSuspended
	 *            boolean
	 */
	public void setExtractOpJustSuspended(boolean newExtractOpJustSuspended) {
		extractOpJustSuspended = newExtractOpJustSuspended;
	}

	/**
	 * @param newNavOp
	 *            PureNavigateAutoStream
	 */
	public void addDestNavOp(PureNavigateAutoStream newNavOp) {
		destNavOps.add(newNavOp);
	}

	public void addEntryNavOp(PureNavigateAutoStream newNavOp) {
		entryNavOps.add(newNavOp);
	}

	/**
	 * Creation date: (4/30/2003 4:05:19 PM)
	 * 
	 * @param newResumeNavOp
	 *            boolean
	 */
	public void setResumeNavOp(boolean newResumeNavOp) {
		resumeNavOp = newResumeNavOp;
	}

	/**
	 * stop extraction Creation date: (2/20/2003 8:01:39 PM)
	 */
	public void suspendExtractOp() {

		//suspendedNavOp = navOp;

		/*
		 * for (Iterator i = ((DagNode)navOp.getNode()).getParents();
		 * i.hasNext(); ){
		 * 
		 * XATStreamOperator op =
		 * (XATStreamOperator)((DagNode)i.next()).getOperator();
		 * 
		 * if (op instanceof ExtractStream){
		 * 
		 * //suspend the corresponding extraction of the suspended navigation
		 * operator extractOps.remove(op); } }
		 */

		//revisit
		//there should be only one extractOp in the extractOps
		suspendedExtractOp = (ExtractStream) extractOps.remove(0);

		//suspend the navigation operator
		//navOp = null;

		extractOpJustSuspended = true;

	}

	/**
	 * stop all the navigation step of navOp Creation date: (2/20/2003 8:01:39
	 * PM)
	 */
	public void suspendNavOp(PureNavigateAutoStream navOp) {

		//there should be only one extractOp in the extractOps

		//suspendedExtractOp = (ExtractStream)extractOps.remove(0);

		//navOpJustSuspended = true;

		if (suspendableStepMgr == null) {
			//not set yet
			suspendableStepMgr = new ArrayList();

			NavigationStep[] destination = navOp.getDestination();

			int stepPos = destination.length - 1;

			ExtendedState pState = this;
			ExtendedState cState = this;

			while (stepPos >= 0) {
				String currentStep = destination[stepPos].getStepValue();

				cState = pState;

				pState = pState.parentState;

				if (((Integer) pState.navOpCount.get(currentStep)).intValue() == 1) {
					//this state is only used for the navigate op, can be
					// suspended its transit
					pState.deleteTransit(currentStep);

					//save it in suspendableStepMgr for late reuse
					Object[] triple = new Object[3];
					triple[0] = currentStep;
					triple[1] = pState;
					triple[2] = cState;
					suspendableStepMgr.add(triple);
				} else {
					//otherwise, cannot go backwards anymore

					suspendedNavOpStepCount = stepPos + 1;
					return;
				}

				stepPos--;

			}

			//all suspended
			suspendedNavOpStepCount = 0;
		} //if (suspendableStepMgr == null)
		else {
			//already set
			for (Iterator i = suspendableStepMgr.iterator(); i.hasNext();) {
				Object[] pair = (Object[]) i.next();
				//remove transit in pair[0] from state in pair[1]
				((ExtendedState) pair[1]).deleteTransit((String) pair[0]);
			}

		}

	}

	/*
	 * suspend all transits from THIS STATE
	 */
	public void suspendAllTransits(SuspensionEdge suspensionEdge,
			boolean cleanTemp) {

		suspend = true;

		//backup
		suspendTransits = transits;
		transits = null;

		//clean intermediate result
		if (cleanTemp) {

			List groups = (List) groupbyOpsNeededClean.get(suspensionEdge);
			//if (groups != null)
				for (int i = 0; i < groups.size(); i++) {
					XATStreamOpRunCore op = (XATStreamOpRunCore) groups.get(i);
					if (!((SelfPartitionDelimiInvoking)op).getPartitionChecked())
					op.cleanTemp();
					//else
						((XATQueueImp)op.getOutputQueue()).dequeueAll();
					
					//only certain op needs to be set partition checked
					//if they are not in the opResetpartitionList
					//set them true here, but never back to false may have problem
					if (((SelfPartitionDelimiInvoking)op).needSetPartitionCheck())
					((SelfPartitionDelimiInvoking)op).setPartitionChecked(true);
					
					//set up the paritionchecked flag for these operators
					//in case they are checked and mistakenly thought they do not 
					//yeild outputs
					//then the same state that has already been suspended is suspended again
					
				}
		}
		
		//if (completeCount > 0)
		//		automata.resetCompleteTreeCount(automata.getCompleteTreeCount()-1);

		if (domCount > 0){
			automata.resetDomCount(automata.getDomCount() - 1);
			domCount--;
			
			
		}

		if (typedValCount > 0){
			automata.resetTypedValCount(automata.getTypedValCount() - 1);
			typedValCount--;
			if (automata.getTypedValCount() == 0)
				//if the following element is not be to stored
				//then clean the whole thing by restoring the count in storage
				// manager
				//i.e.,
				// a
				// |
				// b
				//if b is not to be stored, clean the count to 0, therefore
				//the a is discarded
				automata.getStoreMgr().resetTypedValCount();
		}

		//backup
		//suspendExtractOps = (ArrayList) extractOps.clone();
		//suspendGroupbyOps = (ArrayList) groupbyOps.clone();

		//reduce the count
		//revisit: hardcode here, should calculate the number of related
		// extract

		//minus the complete tree count this state contributes
		/*
		 * automata.minusCompleteTreeCount(completeCount);
		 * automata.minusDomCount(domCount);
		 * automata.minusTypedValCount(typedValCount);
		 */

		

		extractOps=null;
		groupbyOps=null;

	}

	/*
	 * resume the transits from the backup
	 */
	public void resumeAllTransits() {
		suspend = false;

		transits = suspendTransits;
		extractOps = suspendExtractOps;
		groupbyOps = suspendGroupbyOps;

		//resume their partition checked flat
		/*
		 * for (int i=0; i <extractOps.size(); i++)
		 * ((SelfPartitionDelimiInvoking)extractOps.get(i)).setPartitionChecked(false);
		 * 
		 * for (int i=0; i <groupbyOps.size();i++)
		 * ((SelfPartitionDelimiInvoking)extractOps.get(i)).setPartitionChecked(false);
		 */

		//resume the counts
		/*
		 * automata.plusCompleteTreeCount(completeCount);
		 * automata.plusDomCount(domCount);
		 * automata.plusTypedValCount(typedValCount);
		 */

		/*
		 * if (completeCount != completeCountCopy) automata
		 * .resetCompleteTreeCount(automata.getCompleteTreeCount() + 1);
		 */

		if (this.typedValCount != typedValCountCopy){
			//automata.resetTypedValCount(automata.getTypedValCount() + 1);
			typedValCount++;
		}

		if (this.domCount != domCountCopy){
			//automata.resetDomCount(automata.getDomCount() + 1);
			domCount++;
		}

	}

	/*
	 * find the extended state that is n positions before this state @param n
	 * int @return ExtendedState
	 */
	public ExtendedState backward(int n) {

		ExtendedState curState = this;

		for (int i = 0; i < n; i++)
			curState = curState.parentState;

		return curState;

	}

	/**
	 * @return java.lang.String
	 */
	public String toString() {
		String general = super.toString();

		if (destNavOps.size() > 0) {
			general = general + "dest navOps: ";

			for (int i = 0; i < destNavOps.size(); i++)
				general = general
						+ ((PureNavigateAutoStream) destNavOps.get(i))
								.toString();
		}

		if (entryNavOps.size() > 0) {
			general = general + "entry navOps: ";

			for (int i = 0; i < destNavOps.size(); i++)
				general = general
						+ ((PureNavigateAutoStream) entryNavOps.get(i))
								.toString();
		}

		if (extractOps!=null && extractOps.size() > 0) {
			general = general + "extractOps: ";

			for (int i = 0; i < extractOps.size(); i++)
				general = general
						+ ((NavigateStreamImp) extractOps.get(i)).toString();
		}

		if (groupbyOps!=null && groupbyOps.size() > 0) {
			general = general + "groupbyOps: ";

			for (int i = 0; i < groupbyOps.size(); i++)
				general = general + groupbyOps.get(i).toString();
		}

		if (suspensionEdgesAtPush.size() > 0) {
			general = general + "\n";
			general = general + "suspension edge at push: ";
			for (int i = 0; i < suspensionEdgesAtPush.size(); i++)
				general = general + suspensionEdgesAtPush.get(i).toString();

		}

		if (suspensionEdgesAtPop.size() > 0) {
			general = general + "\n";
			general = general + "suspension edge at pop: ";
			for (int i = 0; i < suspensionEdgesAtPop.size(); i++)
				general = general + suspensionEdgesAtPop.get(i).toString();

		}

		return general;
	}

	/*
	 * suspend this state as well as any other navigation steps (1)on a nav op
	 * which goes thru this state and (2) not on a nav op which does not go thru
	 * this state
	 */
	/*
	 * public void suspend() {
	 * 
	 * ExtendedState beginSuspendState = findBeginningOfSuspendable();
	 * 
	 * 
	 * if (suspendableStepMgr == null) { //not set yet suspendableStepMgr = new
	 * ArrayList();
	 * 
	 * NavigationStep[] destination = navOp.getDestination();
	 * 
	 * int stepPos = destination.length - 1;
	 * 
	 * ExtendedState pState = this; ExtendedState cState = this;
	 * 
	 * while (stepPos >= 0) { String currentStep =
	 * destination[stepPos].getStepValue();
	 * 
	 * cState = pState;
	 * 
	 * pState = pState.parentState;
	 * 
	 * if (((Integer) pState.navOpCount.get(currentStep)).intValue() == 1) {
	 * //this state is only used for the navigate op, can be suspended its
	 * transit pState.deleteTransit(currentStep);
	 * 
	 * //save it in suspendableStepMgr for late reuse Object[] triple = new
	 * Object[3]; triple[0] = currentStep; triple[1] = pState; triple[2] =
	 * cState; suspendableStepMgr.add(triple); } else { //otherwise, cannot go
	 * backwards anymore
	 * 
	 * suspendedNavOpStepCount = stepPos + 1; return; }
	 * 
	 * stepPos--; }
	 * 
	 * //all suspended suspendedNavOpStepCount = 0; } //if (suspendableStepMgr ==
	 * null) else { //already set for (Iterator i =
	 * suspendableStepMgr.iterator(); i.hasNext();) { Object[] pair = (Object[])
	 * i.next(); //remove transit in pair[0] from state in pair[1]
	 * ((ExtendedState) pair[1]).deleteTransit((String) pair[0]); } } }
	 */

	/*
	 * find the beginning of suspendable navigation steps all the steps between
	 * the found one and this step are (1)on a nav op which goes thru this state
	 * and (2) not on a nav op which does not go thru this state
	 */
	private ExtendedState findBeginningOfSuspendable() {
		ExtendedState curState = this;

		while (true) {
			if (curState.parentState != null)
				curState = curState.parentState;
			else
				//must hit the root node
				return curState;

			if (curState.getDestNavOps().size() > 0)
				//this state is the destination of other nav
				return curState;
		}

	}

	/*
	 * merge THIS state into the master state
	 */
	public void mergeState(ExtendedState masterState) {

		if (transits != null)
			for (Iterator it = getTransits().keySet().iterator(); it.hasNext();) {
				String oldStateTransitName = (String) it.next();
				ExtendedState transitToInMaster = (ExtendedState) masterState
						.getSingleTransit(oldStateTransitName);
				ExtendedState transitTo = (ExtendedState) getSingleTransit(oldStateTransitName);
				if (transitToInMaster == null) {
					//the transit to state in this does not exist in
					// transitToInMaster yet

					masterState.addTransit(oldStateTransitName, transitTo);
				} else {
					//the transit to state already exist in trasit table of
					// master state
					//merge them too
					transitTo.mergeState(transitToInMaster);
				}
			}

		if (hasLambdaTransit())
			masterState.addTransit(ExtendedState.Lambda_Transit,
					getTransit(ExtendedState.Lambda_Transit));

		if (hasSelfStarTransit())
			masterState
					.addTransit(ExtendedState.Star_Self_Transit, masterState);

		if (hasStarTransit())
			masterState.addTransit(ExtendedState.Star_Transit,
					getTransit(ExtendedState.Star_Transit));

		//also associated with operators

		masterState.getDestNavOps().addAll(destNavOps);

		for (Iterator it = destNavOps.iterator(); it.hasNext();) {
			PureNavigateAutoStream destNavOp = (PureNavigateAutoStream) it
					.next();
			//automata.resetStateHash(destNavOp, this);
			automata.addIntoStateHash(destNavOp, masterState);
		}

		masterState.getEntryNavOps().addAll(entryNavOps);

		masterState.getExtractOps().addAll(extractOps);
		//masterState.suspendExtractOps.addAll(suspendExtractOps);

		//update the counts
		masterState.completeCount = masterState.completeCount + completeCount;
		masterState.completeCount = completeCount;
		masterState.domCount = masterState.domCount + domCount;
		masterState.domCountCopy = masterState.domCount;
		masterState.typedValCount = masterState.typedValCount + typedValCount;
		masterState.typedValCountCopy = masterState.typedValCount;
		for (Iterator it = entryNavOps.iterator(); it.hasNext();) {
			PureNavigateAutoStream extractOp = (PureNavigateAutoStream) it
					.next();
			//automata.resetStateHash(extractOp, this);
			automata.addIntoStateHash(extractOp, masterState);
		}

		masterState.getGroupbyOps().addAll(groupbyOps);
		masterState.suspendGroupbyOps.addAll(groupbyOps);
		//	masterState.groupbyOpsNeededClean.addAll(groupbyOpsNeededClean);

	}

	/*
	 * set up the groupbyOpsNeededClean for partition state of the suspension
	 * edge
	 */
	/*
	 * public void setupGroupbyOpsNeededClean() {
	 * 
	 * List edges = new ArrayList(); edges.addAll(suspensionEdgesAtPop);
	 * edges.addAll(suspensionEdgesAtPush);
	 * 
	 * for (Iterator it = edges.iterator(); it.hasNext();) { SuspensionEdge edge =
	 * (SuspensionEdge) it.next(); XATStreamOperator checkedOp =
	 * (XATStreamOperator) edge.getCheckedOp();
	 * 
	 * List checkedOpDesc = new ArrayList(); List checkedOpBetween = new
	 * ArrayList();
	 * 
	 * //the furtherest ancestor entry ExtendedState partitionState =
	 * (ExtendedState) edge.getPartitionStates().get(
	 * edge.getPartitionStates().size() - 1);
	 * 
	 * if (checkedOp != null) { checkedOpDesc =
	 * FindOpVisitor.getDescNodes((DagNode) checkedOp.getNode());
	 * XATStreamOperator checkedOpSJ = (XATStreamOperator)
	 * FindOpVisitor.findNearestAncestorSJ( (DagNode) checkedOp.getNode());
	 * checkedOpBetween = FindOpVisitor.findInterOps( (DagNode)
	 * checkedOp.getNode(), (DagNode) checkedOpSJ.getNode()); } else {
	 * ExtendedState entryState = getEntryState(); List navs =
	 * entryState.getEntryNavOps(); for (Iterator it2 = navs.iterator();
	 * it2.hasNext();) { //e.g., extract(,/sameAddr) ExtractStream extract =
	 * FindOpVisitor.findAssociatedExtract( (DagNode) ((XATStreamOperator)
	 * it2.next()) .getNode()); //e.g, select(sameAddr!="") List groupbyOps =
	 * partitionState.getGroupbyOps(); DagNode partitionOpNode = (DagNode)
	 * ((XATStreamOperator) groupbyOps .get(groupbyOps.size() - 1)) .getNode();
	 * List extractAnces = FindOpVisitor.findInterOps( (DagNode)
	 * ((XATStreamOperator) extract).getNode(), partitionOpNode);
	 * checkedOpBetween.add(extract); checkedOpBetween.addAll(extractAnces);
	 * //checkedOpBetween.add(partitionOpNode.getOperator()); } }
	 * 
	 * List cleanOps = new ArrayList(); XATStreamOperator highestGroupby =
	 * (XATStreamOperator) partitionState.getGroupbyOps().get(
	 * partitionState.getGroupbyOps().size() - 1); //operators between checkop
	 * and highestGroupby List descNodes = DagTraversal.dagTopDownTraverse(
	 * (DagNode) highestGroupby.getNode()); //do not include the highest group
	 * by itself descNodes.remove(highestGroupby.getNode());
	 * 
	 * for (Iterator it2 = descNodes.iterator(); it2.hasNext();) { DagNode
	 * descNode = (DagNode) it2.next(); if (!checkedOpDesc.contains(descNode) &&
	 * !checkedOpBetween.contains(descNode.getOperator()) && //no duplicates
	 * !partitionState .groupbyOpsNeededClean .contains( descNode.getOperator()) &&
	 * !(descNode.getOperator() instanceof PureNavigateAutoStream) //&&
	 * descNode.getOperator() instanceof ExtractStream && descNode.getOperator() !=
	 * checkedOp) cleanOps.add(descNode.getOperator()); }
	 * 
	 * partitionState.groupbyOpsNeededClean.addAll(cleanOps); } }
	 */

	/*
	 * from THIS state, find the state whose operators whose destinate on them
	 */
	public ExtendedState getEntryState() {
		ExtendedState curState = parentState;
		while (curState.getEntryNavOps().size() == 0)
			curState = curState.parentState;

		return curState;

	}

	/*
	 * private void setUpGroupByOpsNeededClean() {
	 * 
	 * for (Iterator it = suspensionEdgesAtPop.iterator(); it.hasNext();) {
	 * SuspensionEdge edge = (SuspensionEdge) it.next();
	 * 
	 * XATStreamOperator checkedOp = (XATStreamOperator) edge.getCheckedOp(); if
	 * (checkedOp == null) return; //the operators from checkOp to the op
	 * corresponding to the partition state //has been cleaned when
	 * selfPartitionDelimiInvokingRun is called //no need to clean again List
	 * ancestors = FindOpVisitor.findInterOps((DagNode) checkedOp.getNode(),
	 * FindOpVisitor.); ExtendedState partitionState = edge.getPartitionState();
	 * for (Iterator it2 = partitionState.groupbyOps.iterator(); it2.hasNext(); ) {
	 * XATStreamOperator groupbyOp = (XATStreamOperator) it2.next(); if
	 * (groupbyOp == checkedOp || ancestors.contains(groupbyOp))
	 * partitionState.groupbyOpsNeededClean.remove(groupbyOp);
	 * 
	 * if (groupbyOp instanceof ExtractByUnnestStream) { ExtendedState destOn =
	 * ((ExtractByUnnestStream) groupbyOp).getOpState(); if (destOn ==
	 * partitionState) partitionState.groupbyOpsNeededClean.remove(groupbyOp); } }
	 * //end of inner for it2
	 * 
	 * for (int i = 0; i < partitionState.groupbyOps.size(); i++) {
	 * XATStreamOperator op = (XATStreamOperator)
	 * partitionState.groupbyOps.get(i); if (op instanceof
	 * ExtractByCollectionStream) { //its parent does not need to be clean up
	 * List opancestors = FindOpVisitor.findAncestorOps((DagNode) op.getNode());
	 * for (int j = i + 1; j < partitionState.groupbyOps.size(); j++) {
	 * XATStreamOperator afterOp = (XATStreamOperator)
	 * partitionState.groupbyOps.get( j); if (opancestors.contains(afterOp))
	 * partitionState.groupbyOpsNeededClean.remove( afterOp); } } } //end of for
	 * int i }
	 * 
	 * for (Iterator it = suspensionEdgesAtPush.iterator(); it.hasNext();) {
	 * SuspensionEdge edge = (SuspensionEdge) it.next(); XATStreamOperator
	 * checkedOp = (XATStreamOperator) edge.getCheckedOp(); //the operators from
	 * checkOp to the op corresponding to the partition state //has been cleaned
	 * when selfPartitionDelimiInvokingRun is called //no need to clean again if
	 * (checkedOp == null) return; List ancestors =
	 * FindOpVisitor.findAncestorOps((DagNode) checkedOp.getNode());
	 * ExtendedState partitionState = edge.getPartitionState(); for (Iterator
	 * it2 = partitionState.groupbyOps.iterator(); it2.hasNext(); ) {
	 * XATStreamOperator groupbyOp = (XATStreamOperator) it2.next(); if
	 * (groupbyOp == checkedOp || ancestors.contains(groupbyOp))
	 * partitionState.groupbyOpsNeededClean.remove(groupbyOp);
	 * 
	 * if (groupbyOp instanceof ExtractByUnnestStream) { ExtendedState destOn =
	 * ((ExtractByUnnestStream) groupbyOp).getOpState(); if (destOn ==
	 * partitionState) partitionState.groupbyOpsNeededClean.remove(groupbyOp); } }
	 * 
	 * for (int i = 0; i < partitionState.groupbyOps.size(); i++) {
	 * XATStreamOperator op = (XATStreamOperator)
	 * partitionState.groupbyOps.get(i); if (op instanceof
	 * ExtractByCollectionStream) { //its parent does not need to be clean up
	 * List opancestors = FindOpVisitor.findAncestorOps((DagNode) op.getNode());
	 * for (int j = i + 1; j < partitionState.groupbyOps.size(); j++) {
	 * XATStreamOperator afterOp = (XATStreamOperator)
	 * partitionState.groupbyOps.get( j); if (opancestors.contains(afterOp))
	 * partitionState.groupbyOpsNeededClean.remove( afterOp); } } } //end of for
	 * int i } //end of for }
	 */

	/*
	 * get all predecessors of THIS state
	 */
	public Set getPredecessors() {
		Set predecessors = new HashSet();
		for (Iterator it = automata.getStates().iterator(); it.hasNext();) {
			ExtendedState state = (ExtendedState) it.next();
			if (state.getSuccessorStates().contains(this))
				predecessors.add(state);
		}

		return predecessors;
	}

	public int getCompleteCount() {
		return completeCount;
	}

	public void setCompleteCount(int n) {
		completeCount = n;
	}

	public int getTypedValCount() {
		return typedValCount;
	}

	public void setTypedValCount(int n) {
		typedValCount = n;
	}

	public int getDomCount() {
		return domCount;
	}

	public void setDomCount(int n) {
		domCount = n;
	}

	public void addOpResetPartitionChecked(XATStreamOperator op) {
		opsResetPartitionChecked.add(op);
	}

	
}