package edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata;

/**
 * Insert the type's description here.
 * Creation date: (2002-11-27 21:39:05)
 * @author: sk
 */

import java.io.Serializable;
import java.util.*;


public class State implements Serializable {

	//increase by one aftre each state generated to guarantee unique id
	public static int globalId = 0;
	int id;

	//tag name==>state
	protected HashMap transits = null;

	public static final String Lambda_Transit = "Lambda";
	public static final String Star_Transit = "*";
	public static final String Star_Self_Transit = "//";

	protected State toLambdaState = null;
	protected State toStarState = null;
	protected State toSelfStarState = null;
	
	//for statistics gathering purpose
	//count that transition ends at this state
	private int activateCount = 0;
	private int startCount = 0;
	
	public int getActivateCount(){
		return activateCount;
	}
	
	public void reset(){
		activateCount = 0;
		startCount = 0;
	}
	
	public int getStartCount(){
		return startCount;
	}

	/**
	 * State constructor comment.
	 */
	public State() {
		this.id = globalId;
		globalId++;
	}

	public Map getTransits() {
		return transits;
	}

	
	public void removeSingleState(String tagName){
		if (tagName.equals(State.Lambda_Transit)) {
			toLambdaState = null;

		} else if (tagName.equals(State.Star_Transit)) {
			toStarState = null;

		} else if (tagName.equals(State.Star_Self_Transit)) {
			toSelfStarState = null;

		} else transits.remove(tagName);
		
		
	}

	/** This method adds an out going transit
	 *@param tagName String the tag name
	 *@param state State the state which the tagname enables to THIS state to transit to
	 *
	*/

	public void addTransit(String tagName, State state) {

		if (tagName.equals(State.Lambda_Transit)) {
			toLambdaState = state;

		} else if (tagName.equals(State.Star_Transit)) {
			toStarState = state;

		} else if (tagName.equals(State.Star_Self_Transit)) {
			toSelfStarState = state;

		} else {

			if (transits == null)
				transits = new HashMap();

			if (transits.get(tagName) != null) {
				System.out.println("Panic: duplicate transit!");
				System.exit(-1);
			}
			transits.put(tagName, state);
		}
	} //addTransit

	public int getId() {
		return id;
	}
	
	/**
	 * given a tag name, find the next state to transit to  
	 * Creation date: (2003-2-14 15:11:06)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.State
	 * @param tagName java.lang.String
	 */
	public State getSingleTransit(String tagName) {

		if (tagName.equals(State.Lambda_Transit))
			return toLambdaState;

		if (tagName.equals(State.Star_Transit))
			return toStarState;

		if (tagName.equals(State.Star_Self_Transit))
			return toSelfStarState;

		return getTransit(tagName);

	}

	/*
	 * get all succesor states of this state (not including THIS state)
	 * @return Set
	 */
	public Set getSuccessorStates() {
		Set successors = new HashSet();

		if (transits != null)
			for (Iterator it = transits.keySet().iterator(); it.hasNext();) {
				State nextState = (State)transits.get(it.next());
				if (nextState!=this){
				successors.add(nextState);
				successors.addAll(nextState.getSuccessorStates());
				}
			}

		return successors;

	}
	
	

	/**
	 * find non-self/lamda transition when a tag name is given
	 * Creation date: (2002-12-21 19:41:27)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.State
	 * @param tagName java.lang.String
	 */
	State getTransit(String tagName) {
		if (transits == null)
			return null;
		return (State) transits.get(tagName);
	}

	/**
	 * does it have lambda transit
	 * Creation date: (2003-2-14 15:06:51)
	 * @return boolean
	 */
	public boolean hasLambdaTransit() {
		return toLambdaState != null;
	}
	/**
	 * does it have self start transit
	 * Creation date: (2003-2-14 15:08:00)
	 * @return boolean
	 */
	public boolean hasSelfStarTransit() {
		return toSelfStarState != null;
	}

	/**
	 * does it have star transit
	 * Creation date: (2003-2-14 15:07:35)
	 * @return boolean
	 */
	
	public State returnLambdaTransit() {
		return toLambdaState;
	}
	
	
	public State returntoStarState() {
		return toStarState;
	}
	
	
	public State returntoSelfStarState() {
		return toSelfStarState;
	}
	
	
	
	public boolean hasStarTransit() {
		return toStarState != null;
	}

	/**
	 * given a tag name, find the new active state set
	 * Creation date: (2003-2-14 14:51:21)
	 * @param tagName java.lang.String
	 * @param targetSet java.util.Set
	 */
	public void updateTargetSet(String tagName, List targetSet) {
        //a transition is from this state
		startCount ++;
		
		//add direct transit
		State target = getTransit(tagName);
		
		if (target != null){
			targetSet.add(target);
			target.activateCount++;
		}			

		//add star transit
		if (toStarState != null){
			targetSet.add(toStarState);
			toStarState.activateCount++;
		}

		//add star self transit
		if (toSelfStarState != null){
			targetSet.add(toSelfStarState);
			toSelfStarState.activateCount++;
		}

		//iteratively call lambda transits
		if (toLambdaState != null) {
			toLambdaState.updateTargetSet(tagName, targetSet);
			for (int i=0; i<targetSet.size(); i++){
				((State)targetSet.get(i)).activateCount++;
			}
		}
		
		
	}

	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("s" + id + "  ");
		if (transits != null) {
			for (Iterator it = transits.keySet().iterator(); it.hasNext();) {
				String key = (String) it.next();
				sb.append(
					" ( " + key + " ==> " + getTransit(key).getId() + " ) ");
			}
		}

		if (toLambdaState != null)
			sb.append(" ( lambda ==> " + toLambdaState.getId() + " )");

		if (toStarState != null)
			sb.append(" ( star ==> " + toStarState.getId() + " )");

		if (toSelfStarState != null)
			sb.append(" ( self_star ==> " + toSelfStarState.getId() + " )");

		return sb.toString();

	} //toString
}
