package edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor;

/**
 * This class will generate AutoMegaOperators to replace
 * parts in an XATtree. To do so, we need:
 *
 * 1. find the boundary nodes, i.e., determine the parts to be replaced
 * 2. generate an AutoMegaOperator in the place of each boundary node
 * 3. connect the output queue of the AutoMegaOperator to the parents
 *
 * Creation date: (2002-12-1 3:28:59)
 * @author: sk
 */

import java.util.Iterator;
import java.util.Set;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.TreeTraversal;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.logicaloperator.*;

public class AutoDistiller {

	//maximal pushdown
	public static final int MAPPING_MAX = 0;

	//all navigation
	public static final int MAPPING_ALLBINDING = 1;

	//minimal pushdown, only push down source
	public static final int MAPPING_SOURCE = 3;

	//navigation with selection
	public static final int MAPPING_NAVWITHSELONLY = 4;

	//navunnest + selection
	public static final int MAPPING_NAVUNNESTWITHSELONLY = 5;

	//specified number of navigation
	public static final int MAPPING_NAVONLY = 6;

	//all navigation and sepecified number of selections (partial in)
	public static final int MAPPING_SELPARTIALIN = 7;

	protected int mappingStrategy = MAPPING_MAX;

	//	the number of nav to be pushed into the automata	
	private int inAutomataNum;
	private int inAutomataCount = 0;
	private XATTree tree;
	private boolean record = true;
	
	AutoLogicalPlanGenerator generator = new AutoLogicalPlanGenerator();
	/**
	 * AutoDistiller constructor comment.
	 */
	public AutoDistiller() {
		super();
	}
	/**
	 * AutoDistiller constructor comment.
	 */
	public AutoDistiller(int mappingStrategy) {
		super();
		this.mappingStrategy = mappingStrategy;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2002-12-20 17:23:17)
	 * @return boolean
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 */
	protected boolean autoDoable(XATNode node) {

		//if (node.getChildren().length > 1)
		//	return false;

		XATOperator op = node.getOperator();

		switch (this.mappingStrategy) {
			//maximum
			case MAPPING_MAX :
				if (op instanceof NavCollection)
					return true;
				if (op instanceof NavUnnest)
					return true;
				if (op instanceof Select)
					return true;
				if (op instanceof Source)
					return true;
				if (op instanceof NatureJoin)
					//nature join must orignate from concatenating two navs,
					//similar to structural join
					return true;
				break;

				//source
			case MAPPING_SOURCE :
				if (op instanceof Source)
					return true;
				break;

				//main binding
		/*	case MAPPING_MAINBINDING :
				if (op instanceof Source)
					return true;

				if (op instanceof NavUnnest) {

					XATNode childNode = null;
					try {
						childNode = node.getChild(0);
					} catch (Exception e) {
						e.printStackTrace();
					}

					XATOperator childOp = childNode.getOperator();
					if (childOp instanceof Source)
						return true;
				}
				
				
				break;*/

				//all navs
			case MAPPING_ALLBINDING :
				if (op instanceof NavCollection)
					return true;
				if (op instanceof NavUnnest)
					return true;
				if (op instanceof Source)
					return true;
				if (op instanceof NatureJoin)
					return true;
				break;

			case MAPPING_NAVWITHSELONLY :

				if (op instanceof Source)
					return true;

				if (op instanceof NavCollection || op instanceof NavUnnest) {
					//find this NavCollection has a select defined on or not

					Set navsReliedBySels =
						TreeTraversal.getNavReliedBySels(tree);

					if (navsReliedBySels.contains(node)) {
						//int navsReliedByNum = TreeTraversal.getNavsReliedBy((Navigate) op).size();
						//int independentNavsWithSelNum =
						//    TreeTraversal.getNavWithSelsButNotReliedBy((Navigate) op).size();
						//if (inAutomataCount + navsReliedByNum + independentNavsWithSelNum
						//    < inAutomataNum) {
						Set navsReliedBySelsOnNode =
							TreeTraversal.getNavReliedBySels(node);

						if (inAutomataCount + navsReliedBySelsOnNode.size()
							< inAutomataNum) {
							//there is selection defined on
							if (record)
								inAutomataCount++;
							return true;
						} else
							return false;
					} else
						return false;
				}

				if (op instanceof Select) {
					//only if its corresponding nav is autodoable
					XATNode srcNode = TreeTraversal.getSrcNode(node);
					record = false;
					if (autoDoable(srcNode)) {
						record = true;
						return true;
					}
					record = true;

				}

				break;

			case MAPPING_NAVONLY :

				if (op instanceof Source)
					return true;

				if (op instanceof NavCollection || op instanceof NavUnnest) {

					Set navs = TreeTraversal.getNavs(node);

					if (inAutomataCount + navs.size() < inAutomataNum) {
						//there is selection defined on
						if (record)
							inAutomataCount++;
						return true;
					} else
						return false;

				}

				if (op instanceof Select) {
					//only if its corresponding nav is autodoable
					XATNode srcNode = TreeTraversal.getSrcNode(node);
					record = false;
					if (autoDoable(srcNode)) {
						record = true;
						return true;
					}
					record = true;
				}

				break;

			case MAPPING_NAVUNNESTWITHSELONLY :
				//we rely on no navcollection underneathr navnest that navnest is dependent on, they should be merged

				if (op instanceof Source)
					return true;

				if (op instanceof NavUnnest)
					//find this NavUnnest has a select defined on or not
					if (TreeTraversal.selOpsDefinedOn(node)
						&& (inAutomataCount < inAutomataNum)) {
						//there is selection defined on
						inAutomataCount++;
						return true;
					} else
						return false;

				if (op instanceof Select) {
					XATNode srcNode = TreeTraversal.getSrcNode(node);
					if (autoDoable(srcNode))
						return true;
				}

				break;

			case MAPPING_SELPARTIALIN :
				if (op instanceof Source)
					return true;

				if (op instanceof NavCollection || op instanceof NavUnnest)
					return true;

				if (op instanceof Select)
					//only if its corresponding nav is autodoable
					if (TreeTraversal.getSels(op.getNode()).size()
						< inAutomataNum)
						return true;
					else
						return false;

				break;

		}

		return false;
	}
	/**
	 * This method transforms an XAT tree by replacing automata-doable
	 * subtrees with AutoMegaOperators.
	 * 
	 * Creation date: (2002-12-20 17:21:08)
	 */
	public void distill(XATTree xatTree) {
		tree = xatTree;
		//seperate automata part from the rest of the plan
		visit(tree.getRoot(), false);
	}
	/**
	 * This method generate a new AutoMegaOperator
	 * and replace with it the boundary node
	 * 
	 * Creation date: (2002-12-20 17:23:33)
	 * @param boundary edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 */
	protected void makeAutoMegaOp(XATNode boundary) {

		DagNode autoNode = new DagNodeImp();
		AutoMegaOperator autoOp = new AutoMegaImp(autoNode);

		for (Iterator it = boundary.getParents(); it.hasNext();) {
			XATNode parentNode = (XATNode) it.next();

			autoNode.addParent(parentNode);

			parentNode.deleteChild(boundary);
			parentNode.addChild(autoNode);

		}

		//make physical plan for the auto part
		generator.boundaryOperator = boundary.getOperator();
		generator.megaSchema.addAll(boundary.getSchema());
		
		DagNode root = generator.createInAutomataPlan(boundary);
		generator.findStructuralJoin();		
		generator.computeSchema();
		generator.computeSchemaBottomUp();

		autoOp.setBoundary(root);
		
	}
	/**
	 * This method visit a node to see whether it is 
	 * automata doable.
	 * 
	 * Creation date: (2002-12-20 17:24:01)
	 * @return boolean
	 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 * @param parentDoable boolean
	 */
	protected boolean visit(XATNode node, boolean parentDoable) {
		try {

			if (!autoDoable(node)) {
				for (int i = 0; i < node.getChildren().length; i++) {
					visit(node.getChild(i), false);
				}
				return false;
			}

			//node is autoDoable

			if (parentDoable) {
				//node is a leaf, i.e., source
				if (node.getChildren().length == 0)
					return true;
				else {
					boolean alltrue = true;
					for (int i = 0; i < node.getChildrenSize(); i++)
						if (!(visit(node.getChild(i), true)))
							//if any child is not doable, then this node is not doable either						
							alltrue = false;
							
					if (!alltrue)
						return false;
				}
			} else {
				//!parentDoable and node is a leaf				
				if (node.getChildren().length == 0)
					makeAutoMegaOp(node);
				else {
					boolean alltrue = true;
					//!parentDoable and node is not a leaf
					for (int i = 0; i < node.getChildrenSize(); i++)
						if (!(visit(node.getChild(i), true)))
							alltrue = false;
					if (alltrue)
						//every child is doable		
						makeAutoMegaOp(node);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return true;
	}

	public void setInAutomataNum(int newInAutomataNum) {
		inAutomataNum = newInAutomataNum;
	}
}
