package edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor;

/**
 * This class convert a xat subtree into a dag.
 * The subtree will be implemented in automata,
 * it is wrapped by a mega-operator.
 * 
 *   
 * Schema is set in three places:
 * 1) in each visit method (maximization)
 * 2) in findStruralJoin (maximization)
 * 3) in computeSchema (minimization)
 * 
 * Creation date: (2002-12-20 15:44:55)
 * @author: sk
 */

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.xmloperator
	.navigation
	.NavigationStep;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.xmloperator
	.navigation
	.NodeStepImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.logicaloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.DagTraversal;

public class AutoLogicalPlanGenerator extends InheritableVisitor {

	//open binding name ==> DagNode
	protected Hashtable openHash = new Hashtable();

	//clase binding name ==> DagNode
	protected Hashtable closeHash = new Hashtable();

	//boundary operator
	protected XATOperator boundaryOperator = null;

	protected DagNode physicalPlanRoot = null;

	protected DagNode physicalPlanLeaf = null;

	protected Set megaSchema = new HashSet();

	List nodesVisited = new ArrayList();

	/**
	 * PhysicalPlanGenerator constructor comment.
	 */
	public AutoLogicalPlanGenerator() {
		super();
	}
	/**
	 * This method computes the schema for a raindrop plan.
	 * It does this by minimize the schema.
	 * 
	 * Creation date: (2003-1-23 21:18:31)
	 */
	protected void computeSchema() {

		Set curSet = new HashSet(); //nodes in current level
		Set nextSet = new HashSet(); //nodes in the next level

		Set curRequired = new HashSet(); //required bindings for this level
		Set nextRequired = new HashSet(); //required bindings for next level
		Set lastRequired = new HashSet();

		//do a breadth first traverse from topdown
		//can not use the Utility.dagTopdownTraverse because
		//we need to seperate nodes in each level
		curSet.add(this.physicalPlanRoot);
		curRequired.addAll(megaSchema);

		do {
			for (Iterator it = curSet.iterator(); it.hasNext();) {
				DagNode curNode = (DagNode) it.next();
				//push all children to the nextSet
				if (curNode != this.physicalPlanLeaf) {
					XATNode[] children = curNode.getChildren();
					for (int i = 0; i < children.length; i++)
						nextSet.add(children[i]);
				}

				//visit this node
				//Strucural Join
				if (curNode.getOperator() instanceof StructuralJoin) {
					//set next required
					XATNode[] children = curNode.getChildren();
					for (int i = 0; i < children.length; i++)
						nextRequired.addAll(children[i].getSchema());

					//set schema for curNode
					for (Iterator i = curNode.getSchema().iterator();
						i.hasNext();
						) {
						Object item = i.next();
						if (!curRequired.contains(item)
							|| (lastRequired.size() > 0
								&& !lastRequired.contains(item)))
							//remove schema that are not needed by parent, lastRequired may be empty when intialized)))
							i.remove();
					}

				}

				//StreamSelect
				else if (curNode.getOperator() instanceof StreamSelect) {
					//set next required
					nextRequired.addAll(curNode.getSchema());

					XATNode[] children = curNode.getChildren();
					Set overallChildrenSchema = new HashSet();
					for (int i = 0; i < children.length; i++)
						overallChildrenSchema.addAll(children[i].getSchema());

					//set schema for curNode
					for (Iterator i = curNode.getSchema().iterator();
						i.hasNext();
						) {
						Object item = i.next();
						//if (!curRequired.contains(i.next()))
						if (!curRequired.contains(item)
							|| !overallChildrenSchema.contains(item)
							|| !lastRequired.contains(
								item)) //remove schema that are not needed by parent)
							i.remove();

					}

				} //StreamSelect

				//end of visit
			}

			//go to next level
			curSet = nextSet;
			nextSet = new HashSet();
			lastRequired = curRequired;
			curRequired = nextRequired;
			nextRequired = new HashSet();
		} while (!curSet.isEmpty());

	}

	/*another pass of schema clean up bottom up to ensure the minimization of the schema
	 * structuralJoin is mapped from a navigation
	 * the output schema of a navigation includes both the entry and the dest
	 * structuraljoin's output actually does not have the entry
	 * we do a bottom up to remove the entry column
	 */
	protected void computeSchemaBottomUp() {

		Set curSet = new HashSet(); //nodes in current level
		Set parentSet = new HashSet(); //nodes in the next level

		curSet.add(this.physicalPlanLeaf);

		do {
			for (Iterator it = curSet.iterator(); it.hasNext();) {
				DagNode curNode = (DagNode) it.next();
				//push all parents to the parentSet
				if (curNode != this.physicalPlanRoot) {
					for (Iterator it2 = curNode.getParents(); it2.hasNext();)
						parentSet.add((DagNode) it2.next());
				}

				//visit this node
				//Strucural Join
				if (curNode.getOperator() instanceof StructuralJoin
					|| curNode.getOperator() instanceof StreamSelect) {
					//set next required
					XATNode[] children = curNode.getChildren();
					Set childGenerated = new HashSet();
					for (int i = 0; i < children.length; i++)
						childGenerated.addAll(children[i].getSchema());

					//set schema for curNode
					for (Iterator i = curNode.getSchema().iterator();
						i.hasNext();
						) {
						Object item = i.next();
						if (childGenerated.size() > 0
							&& !childGenerated.contains(item))
							//remove schema that are not generated by children
							i.remove();
					}

				}

				//end of visit
			}

			//go to next level
			curSet = parentSet;
			parentSet = new HashSet();

		} while (!curSet.isEmpty());

	}

	public Object dispatch() {
		return null;
	}
	public Object dispatch(Object obj) {
		return null;
	}
	public Object dispatch(Object obj1, Object obj2) {
		return null;
	}
	/**
	 * given the boundary node, change the subplan rooted at this boundary node to a new subplan which all operators are executed in automata
	 * Creation date: (2002-12-20 15:57:08)
	 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	 */
	public DagNode createInAutomataPlan(XATNode boundary) {

		//	boundaryOperator = boundary.getOperator();
		//	megaSchema.addAll(boundary.getSchema());

		try {
			//find leaf node
			XATNode rootNode = boundary;
			XATNode leafNode = rootNode;

			for (int i = 0; i < boundary.getChildrenSize(); i++)
				createInAutomataPlan(boundary.getChild(i));
				
			if (!nodesVisited.contains(boundary)){	
				visit(boundary.getOperator());
				nodesVisited.add(boundary);
			}

			/*List path = new ArrayList();
				
				
				while (leafNode.getChildren().length > 0) {
					path.add(leafNode);
					leafNode = leafNode.getChild(0);
				}
				//add in the leave node
				path.add(leafNode);
			
				//post order visit, bottom-up
				boolean justVisited = false;
				for (int i = path.size(); i > 0; i--) {
					DagNode curNode = (DagNode) path.get(i - 1);
					if (!nodesVisited.contains(curNode)) {
						visit(curNode.getOperator());
						nodesVisited.add(curNode);
					}
					leafNode = leafNode.getParent();
			
				}*/

		} catch (Exception e) {
			e.printStackTrace();
		}

		/*findStructuralJoin();
		computeSchema();
		computeSchemaBottomUp();*/

		return this.physicalPlanRoot;
	}
	/**
	 * This method finds extract nodes that have multiple children.
	 * If found,
	 * it will transform to a StructuralJoin node
	 * This method also computes the schema for the StructuralJoin op.
	 * 
	 * Creation date: (2003-1-22 21:43:22)
	 */
	protected void findStructuralJoin() {

		//do a breadth first traverse from bottom up
		Vector traversal =
			DagTraversal.dagBottomUpTraverse(
				physicalPlanLeaf,
				physicalPlanRoot);

		for (Iterator it = traversal.iterator(); it.hasNext();) {
			DagNode curNode = (DagNode) it.next();

			//visit this node
			if (curNode.getOperator() instanceof Extract) {
				if (curNode.getChildrenSize() == 1
					&& curNode.getChild(0).getOperator() instanceof Extract) {
					//only one child, but of type extract
					//curNode is redundant
					DagNode curChild = (DagNode) curNode.getChild(0);
					curChild.deleteParent(curNode);
					curNode.deleteChild(curChild);

					if (curNode.getParentsSize() > 0) {
						((DagNode) curNode.getChild(0)).addParent(
							curNode.getParent());
						curNode.getParent().addChild(curNode.getChild(0));
						curNode.getParent().deleteChild(curNode);
					}

					//this snippet is added to 1.6
					if (physicalPlanRoot == curNode) {
						//curNode has been deleted
						//must update physicalplan root
						physicalPlanRoot = curChild;
					}
					//end of snippet added to 1.6
					/*copy the extract node, it will be a branch of the structuraljoin node	
					DagNode newExtractNode = new DagNodeImp();
					newExtractNode.setOperator(curNode.getOperator());
					//associate this node with its navigate node
						DagNode navNodeForNewExtract = (DagNode)openHash.get(((Navigate)newExtractNode.getOperator()).getBinding());
						newExtractNode.addChild(navNodeForNewExtract);
						navNodeForNewExtract.addParent(newExtractNode);
					
					//create a structural join for it      	
					StructuralJoin joinOp = new StructuralJoinUnnestImp(curNode);
					//add the other branch derived from curNode's op
					curNode.addChild(newExtractNode);
					newExtractNode.addParent(curNode); */
				}

				if (curNode.getChildrenSize() > 1) {

					//find one, transform
					StructuralJoin joinOp = null;
					Extract extractOp = (Extract) curNode.getOperator();

					XATNode[] children = curNode.getChildren();
					int i = 0;
					for (i = 0; i < children.length; i++) {
						DagNode curChild = (DagNode) children[i];
						//if it connects to a StreamNavigate,
						//make an extra node for join
						if (curChild.getOperator() instanceof StreamNavigate) {
							DagNode extractNode = new DagNodeImp();
							extractNode.setOperator(extractOp);
							extractOp.setNode(extractNode);
							if (extractOp instanceof ExtractUnnest)
								joinOp = new StructuralJoinUnnestImp(curNode);
							else //extractop instanceof ExtractColl
								joinOp = new StructuralJoinCollImp(curNode);
							//delete the connections
							curNode.deleteChild(curChild);
							curChild.deleteParent(curNode);
							//make new connections
							curNode.addChild(extractNode);
							extractNode.addParent(curNode);
							extractNode.addChild(curChild);
							curChild.addParent(extractNode);
							//set schema
							extractNode.setSchema(curNode.getSchema());
							break;
						}

					} //for

					//if it does not connects to a StreamNavigate
					//just change it to a join node
					if (i == children.length) {
						if (extractOp instanceof ExtractUnnest)
							joinOp = new StructuralJoinUnnestImp(curNode);
						else
							joinOp = new StructuralJoinCollImp(curNode);
						//the old Op is gone
					}

					//set shadow node
					joinOp.setShadowNode(extractOp.getShadowNode());

					//compute schema
					Set schema = new HashSet();
					//the children may have changed, get the new one
					children = curNode.getChildren();
					for (i = 0; i < children.length; i++) {
						schema.addAll(children[i].getSchema());
					}
					curNode.setSchema(schema);
				} //transform
			} //visit

		}

	}
	/**
	* Insert the method's description here.
	* Creation date: (2002-12-20 22:18:51)
	* @param root edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
	*/
	public void printTree() {
		try {
			//do a breadth first traverse from top down
			Vector traversal =
				DagTraversal.dagTopDownTraverse(
					physicalPlanRoot,
					physicalPlanLeaf);
			for (Iterator it = traversal.iterator(); it.hasNext();) {
				DagNode curNode = (DagNode) it.next();
				//visit curNode
				System.out.println("( " + curNode.getOperator());

				for (int i = 0; i < curNode.getChildrenSize(); i++)
					System.out.println(
						"    -->" + curNode.getChild(i).getOperator());
				System.out.println(")");
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	public void visit(Source sourceOp) {

		DagNode dagNode = new DagNodeImp();
		StreamSource srcOp = new StreamSourceImp(dagNode);
		srcOp.setName(sourceOp.getName());

		Set schemaSource = new HashSet();
		schemaSource.add(sourceOp.getOutputColumnName());
		dagNode.setSchema(schemaSource);

		this.physicalPlanLeaf = dagNode;
		openHash.put(sourceOp.getOutputColumnName(), dagNode);

		if (sourceOp == boundaryOperator) {

			NavigationStep[] eps = new NavigationStep[1];
			eps[0] = new NodeStepImp(sourceOp.getOutputColumnName());
			//  eps[0] = new TextStepImp("fake");

			NavigationStep[] dests = new NavigationStep[1];

			dests[0] = new NodeStepImp("*");

			NavUnnest op = new NavUnnestImp(sourceOp.getNode(), eps, dests);

			XATNode xatNode = op.getNode();

			op.setOutputColumnName(sourceOp.getOutputColumnName());
			// String binding = op.getOutputColumnName();
			String binding = sourceOp.getOutputColumnName();
			// System.out.println("_________________"+op.getOutputColumnName()+"________");
			//String binding = "source";

			//create open node
			DagNode openNode = new DagNodeImp();
			StreamNavigate openOp = new StreamNavigateImp(openNode, op);
			openHash.put(binding, openNode);

			//create close node
			DagNode closeNode = new DagNodeImp();
			ExtractUnnest closeOp = new ExtractUnnestImp(closeNode, op);
			//closeHash.put(binding, closeNode);

			//set shadow node
			closeOp.setShadowNode(openNode);
			closeOp.setOutputColumnName(sourceOp.getOutputColumnName());

			//set schema
			Set schema = new HashSet();
			schema.add(binding);
			closeNode.setSchema(schema);

			schema = new HashSet();
			openNode.setSchema(schema);

			//connect to child
			//DagNode childNode =
			//    (DagNode) openHash.get(op.getEntryPoint()[0].getStepValue());
			//childNode always exists for Navigate
			openNode.addChild(dagNode);
			dagNode.addParent(openNode);

			//connect to parent
			DagNode parentNode =
				(DagNode) closeHash.get(op.getEntryPoint()[0].getStepValue());
			if (parentNode != null) {
				parentNode.addChild(closeNode);
				closeNode.addParent(parentNode);
			}

			//if in schema, connect the Navigate with the StructuralJoin
			// if (this.schema.contains(binding)) {
			closeNode.addChild(openNode);
			openNode.addParent(closeNode);
			// }
			//if it is not in the schema, 
			//we will have a navigate or select 
			//to connect openNode and closeNode,
			//i.e., the graph will always be connected.

			//???????
			//what about the schema???
			///???????

			this.physicalPlanRoot = closeNode;
			while (physicalPlanRoot.getParentsSize() > 0)
				physicalPlanRoot = physicalPlanRoot.getParent(0);
		}

		//mapping source only

	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2002-12-20 16:30:45)
	 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.Select
	 */
	public void visit(Select op) {

		//make node
		DagNode dagNode = new DagNodeImp();
		StreamSelect streamOp = new StreamSelectImp(dagNode, op);

		//the entry set contains all bindings

		//Set entrySet = Utility.getExpressionEntrySet(op.getExpression());
		ColumnName[] columnNames = op.getExpression().getRelatedColumnNames();
		Set entrySet = new HashSet();
		for (int i = 0; i < columnNames.length; i++)
			entrySet.add(columnNames[i].toString());

		//	String[] entrys = (String[]) entrySet.toArray(new String[0]);
		String[] entrys = new String[entrySet.size()];
		int entryPos = 0;

		for (Iterator it = entrySet.iterator(); it.hasNext();) {
			//entrys[entryPos] = ((ColumnName) it.next()).getColumnName();
			entrys[entryPos] = (String) it.next();
			entryPos++;
		}

		//set schema
		Set sc = new HashSet();
		sc.addAll(entrySet);
		//add the columns needed by parents
		XATNode xatNode = op.getNode();
		sc.addAll(xatNode.getSchema());
		dagNode.setSchema(sc);

		//make sure every entry has output
		for (int i = 0; i < entrys.length; i++) {
			DagNode openNode = (DagNode) openHash.get(entrys[i]);
			DagNode closeNode = (DagNode) closeHash.get(entrys[i]);

			boolean hasOutput = false;
			//DagNode[] parents = openNode.getParents();
			//for(int j = 0; j < parents.length; j++){
			for (Iterator it = openNode.getParents(); it.hasNext();) {
				DagNode curParent = (DagNode) it.next();
				if (curParent == closeNode) {
					hasOutput = true;
					break;
				}
			}
			if (!hasOutput) {
				openNode.addParent(closeNode);
				closeNode.addChild(openNode);
			}
		}

		//find common parent
		DagNode parentNode = null;
		if (entrys.length == 1) {
			parentNode = (DagNode) closeHash.get(entrys[0]);
		} else {
			//first, create a path from the first entry to the root
			DagNode curNode = (DagNode) closeHash.get(entrys[0]);
			DagNode rootNode = curNode;

			ArrayList path = new ArrayList();
			path.add(rootNode);

			while (rootNode.getParentsSize() > 0) {
				rootNode = rootNode.getParent(0);
				path.add(rootNode);
			}

			//then, find the common parent by climbing up the path
			int pos = 0;
			for (int i = 1; i < entrys.length; i++) {

				DagNode tempNode = (DagNode) closeHash.get(entrys[i]);
				while (pos < path.size()) {

					//climb up the tempNode to find the cross node
					DagNode curPosNode = (DagNode) path.get(pos);
					boolean isFound = true;
					while (tempNode != curPosNode) {

						if (tempNode.getParentsSize() > 0) {
							tempNode = tempNode.getParent(0);
						} else {
							isFound = false;
							break; // go to next pos
						}
					}

					if (isFound)
						break;
					//go to next entry
					else
						pos++;
				} //while

			} //for

			if (pos < path.size()) {
				//connect to the commonparent
				parentNode = (DagNode) path.get(pos);
			} else { //pos overbound
				System.out.println("Panic: no common parent!!!");
				System.exit(-1);
			}
		} //entrys.length > 1

		//take care the parent of the parentNode
		if (parentNode.getParentsSize() > 0) {
			DagNode oldParent = parentNode.getParent(0);
			parentNode.deleteParent(oldParent);
			oldParent.deleteChild(parentNode);

			//insert the select between parentNode and oldParent
			oldParent.addChild(dagNode);
			dagNode.addParent(oldParent);
		}
		parentNode.addParent(dagNode);
		dagNode.addChild(parentNode);

		if (op == boundaryOperator) {
			this.physicalPlanRoot = dagNode;
			while (physicalPlanRoot.getParentsSize() > 0)
				physicalPlanRoot = physicalPlanRoot.getParent(0);
		}

	}
	/**
	 * This method maps a Navigate operator to a pair of physical operators.
	 * i.e, a navigate and a structural join.
	 * The Navigate is called openOp, 
	 * its semantic is to "goto" a specific position of the document,
	 * i.e, the open tag.
	 * The Structural Join is called closeOp,
	 * its semantic is to "collect" data from the node,
	 * it is executed when the close tag arrives.
	 *
	 * Creation date: (2002-12-20 16:30:32)
	 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnest
	 */
	public void visit(NavCollection op) {

		XATNode xatNode = op.getNode();
		String binding = op.getOutputColumnName();

		//create open node
		DagNode openNode = new DagNodeImp();
		StreamNavigate openOp = new StreamNavigateImp(openNode, op);
		openHash.put(binding, openNode);

		//create close node
		DagNode closeNode = new DagNodeImp();
		Extract closeOp = null;
		if (op instanceof OuterNavCollectionImp)
			closeOp = new OuterExtractColl(closeNode, op);
		else
			closeOp = new ExtractCollImp(closeNode, op);
		closeHash.put(binding, closeNode);

		//set shadow node
		closeOp.setShadowNode(openNode);

		//set schema
		Set closeSchema = new HashSet();
		closeSchema.add(binding);
		closeNode.setSchema(closeSchema);

		//the open node schema is created only to make
		//sure every node has non-null schema
		Set openSchema = new HashSet();
		openNode.setSchema(openSchema);

		//connect to child
		DagNode childNode =
			(DagNode) openHash.get(op.getEntryPoint()[0].getStepValue());
		//childNode (must be a streamnavigate) always exists for Navigate, 
		childNode.addParent(openNode);
		openNode.addChild(childNode);

		//connect to parent
		DagNode parentNode =
			(DagNode) closeHash.get(op.getEntryPoint()[0].getStepValue());
		if (parentNode != null) {
			parentNode.addChild(closeNode);
			closeNode.addParent(parentNode);
		}

		closeNode.addChild(openNode);
		openNode.addParent(closeNode);

		if (op == boundaryOperator) {
			this.physicalPlanRoot = closeNode;
			while (physicalPlanRoot.getParentsSize() > 0)
				physicalPlanRoot = physicalPlanRoot.getParent(0);
		}
	}

	/**
	 * This method maps a Navigate operator to a pair of physical operators.
	 * i.e, a navigate and a structural join.
	 * The Navigate is called openOp, 
	 * its semantic is to "goto" a specific position of the document,
	 * i.e, the open tag.
	 * The Structural Join is called closeOp,
	 * its semantic is to "collect" data from the node,
	 * it is executed when the close tag arrives.
	 *
	 * Creation date: (2002-12-20 16:30:32)
	 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnest
	 */
	public void visit(NavUnnest op) {

		XATNode xatNode = op.getNode();
		String binding = op.getOutputColumnName();

		//create open node
		DagNode openNode = new DagNodeImp();
		StreamNavigate openOp = new StreamNavigateImp(openNode, op);
		openHash.put(binding, openNode);

		//create close node
		DagNode closeNode = new DagNodeImp();

		ExtractUnnest closeOp = null;
		if (op instanceof OuterNavUnnestImp)
			closeOp = new OuterExtractUnnest(closeNode, op);
		else
			closeOp = new ExtractUnnestImp(closeNode, op);
		closeHash.put(binding, closeNode);

		//set shadow node
		closeOp.setShadowNode(openNode);

		//set schema
		Set closeSchema = new HashSet();
		closeSchema.add(binding);
		closeNode.setSchema(closeSchema);

		//the open node schema is created only to make
		//sure every node has non-null schema
		Set openSchema = new HashSet();
		openNode.setSchema(openSchema);

		//connect to child
		DagNode childNode =
			(DagNode) openHash.get(op.getEntryPoint()[0].getStepValue());
		//childNode always exists for Navigate
		childNode.addParent(openNode);
		openNode.addChild(childNode);

		//connect to parent
		DagNode parentNode =
			(DagNode) closeHash.get(op.getEntryPoint()[0].getStepValue());
		if (parentNode != null) {
			parentNode.addChild(closeNode);
			closeNode.addParent(parentNode);
		}

		//if in schema, connect the Navigate with the StructuralJoin
		//
		//It will have problem if some operator inside the mega operator
		//needs this binding!!!
		//
		//
		if (this.megaSchema.contains(binding)) {
			closeNode.addChild(openNode);
			openNode.addParent(closeNode);
		} else {
			//if the node has a parent node, 
			//i.e., parent node is derived from NavUnnest($s, /p1->$a)
			//and this node is derived from NavUnnest($a, /p2->$b)
			//the output of $b is needed since it reflects the dimensionality of b's within a
			if (closeNode.getParentsSize() > 0) {
				closeNode.addChild(openNode);
				openNode.addParent(closeNode);
			}

		}

		//if it is not in the schema, 
		//we will have a navigate or select 
		//to connect openNode and closeNode,
		//i.e., the graph will always be connected.

		if (op == boundaryOperator) {
			this.physicalPlanRoot = closeNode;
			while (physicalPlanRoot.getParentsSize() > 0)
				physicalPlanRoot = physicalPlanRoot.getParent(0);
		}
	}

}
