package edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor;

/**
 * This class maps a rainbow plan (XAT) to a raindrop plan (dag)
 * 
 * Control sequences:
 * -> this(RaindropLogicalPlanGenerator)
 *  	-> AutoDistiller, push operators into megaoperator
 *       ->AutoLogicalPlanGenerator, map to auto operators
 *  	-> visit(), conver xattree to dag
 *
 *
 * -> Finer_RaindropPhysicalPlanGenerator
 *    -> map to all physical operators in one shot
 *
 * All schemas are set in visit(XATNode) by copying the schema
 * in the rainbow plan.
 * 
 * Creation date: (2002-12-20 15:44:55)
 * @author: sk
 */

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Merge;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Source;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.LeftOuterJoin;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavCollection;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnest;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.Navigate;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.xmloperator
	.navigation
	.NavigationStep;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.xmloperator
	.navigation
	.VarStepImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.logicaloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.DagTraversal;
import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.TreeTraversal;

public class RaindropLogicalPlanGenerator extends InheritableVisitor {
	static final int NORELATION = -100;
	static final int EQUI = 0;

	//open binding name ==> DagNode
	protected Hashtable openHash = new Hashtable();

	//clase binding name ==> DagNode
	protected Hashtable closeHash = new Hashtable();

	//schema for the mega operator
	protected Set schema = new HashSet();

	//boundary operator
	protected XATOperator boundaryOperator = null;

	protected XATDag xatdag = null;

	//protected DagNode curParent = null;
	//protected DagNode curNode = null;

	protected int mappingStrategy = AutoDistiller.MAPPING_MAX;

	//from a map to a node
	protected Map opToNode = new HashMap();

	/**
	 * PhysicalPlanGenerator constructor comment.
	 */
	public RaindropLogicalPlanGenerator() {
		super();
	}
	/**
	 * PhysicalPlanGenerator constructor comment.
	 */
	public RaindropLogicalPlanGenerator(int mappingStrategy) {
		super();
		this.mappingStrategy = mappingStrategy;
	}
	public Object dispatch() {
		return null;
	}
	public Object dispatch(Object obj) {
		return null;
	}
	public Object dispatch(Object obj1, Object obj2) {
		return null;
	}
	/**
	 * @param xattree XATTree a rainbow plan
	 * @return XATDag
	 * generate a raindrop logical plan out of the rainbow plan
	 */
	public XATDag generatePlan(XATTree xattree) {
		//linearizeNaturalJoin((XATDag) xattree);
		//System.out.println("rainbow plan:\n" + xattree);

		//distill
		AutoDistiller distiller = new AutoDistiller(this.mappingStrategy);
		distiller.distill(xattree);

		//make logical plan by
		//Top-down visit
		visit(xattree.getRoot(), null);


		return xatdag;
	}

	/*
	 * for a 
	 *              NaturalJoin
	 *                /       \
	 *     NavUnnest($a/b->$b) NavUnnest($a/c->$c)
	 *                \        /
	 *              NavUnnest($o/a->$a)
	 * 
	 * linearize it to
	 *                   
	 *              NavUnnest($a/c->$c)   
	 *                   |
	 *              NavUnnest($a/b->$b)
	 *                   |
	 *               NavUnnest
	 * and then turn it into
	 *               NavUnnest($c, $b ->null)
	 *                   |
	 *               NavNest($a/c->$c)
	 *                   |
	 *               NavNest($a/b->$b)
	 *                   |
	 *               NavUnnest
	 */
/*	public void linearizeNaturalJoin(XATDag xatdag) {

		Vector nodes = DagTraversal.dagBottomUpTraverse(xatdag);
		DagNode multiNavUnnestNode = new DagNodeImp();
		MultiNavUnnest multiNavUnnest = null;

		for (Iterator it = nodes.iterator(); it.hasNext();) {
			DagNode node = (DagNode) it.next();
			XATOperator op = node.getOperator();
			if (op instanceof NatureJoin) {
				//node is now the natural join
				List desc = FindOpVisitor.getDescNodes(node);
				for (Iterator it2 = desc.iterator(); it2.hasNext();) {
					DagNode descNode = (DagNode) it2.next();
					if (descNode.getParentsSize() > 1) {
						//a desc node has multiple parent
						//must be a common ancestor
						DagNode firstParent = (DagNode) descNode.getParent(0);
						//intermediate does not include firstparent, so we include firstParent 
						List intermediate =
							FindOpVisitor.findAllInterOps(firstParent, node);
						intermediate.add(0, firstParent);

						if (firstParent.getOperator() instanceof NavUnnest) {
							NavUnnest navUnnest =
								(NavUnnest) firstParent.getOperator();
							//change it to NavNest
							NavCollection navColl =
								new NavCollectionImp(
									firstParent,
									navUnnest.getEntryPoint(),
									navUnnest.getDestination());
							navColl.setOutputColumnName(
								navUnnest.getOutputColumnName());

							//add the output column name to the entries of the top MultiNavUnnest operator
							multiNavUnnest =
								new MultiNavUnnest(multiNavUnnestNode);
							multiNavUnnest.addEntry(
								navUnnest.getOutputColumnName());
						}

						DagNode currentNjChild =
							(DagNode) ((XATOperator) intermediate
								.get(intermediate.size() - 1))
								.getNode();

						for (int i = 1; i < descNode.getParentsSize(); i++) {
							//put all nodes on the first branch 
							DagNode ithParent = (DagNode) descNode.getParent(i);

							if (ithParent.getOperator() instanceof NavUnnest) {
								NavUnnest navUnnest =
									(NavUnnest) ithParent.getOperator();
								//change it to NavNest
								NavCollection navColl =
									new NavCollectionImp(
										ithParent,
										navUnnest.getEntryPoint(),
										navUnnest.getDestination());

								navColl.setOutputColumnName(
									navUnnest.getOutputColumnName());

								//add the output column name to the entries of the top MultiNavUnnest operator
								multiNavUnnest.addEntry(
									navUnnest.getOutputColumnName());
							}

							ithParent.addChild(currentNjChild);
							currentNjChild.deleteParent(node);
							currentNjChild.addParent(ithParent);
							ithParent.deleteChild(descNode);
							descNode.deleteParent(ithParent);
							node.deleteChild(currentNjChild);

							intermediate =
								FindOpVisitor.findAllInterOps(ithParent, node);
							intermediate.add(0, ithParent);
							currentNjChild =
								(DagNode) ((XATOperator) intermediate
									.get(intermediate.size() - 1))
									.getNode();
						} //end of for
						//remove naturejoin and add in multiNavUnnest
						currentNjChild.addParent(multiNavUnnestNode);
						multiNavUnnestNode.addChild(currentNjChild);
						multiNavUnnestNode.addParent(node.getParent());
						node.getParent().addChild(multiNavUnnestNode);
						node.getParent().deleteChild(node);
						break;
					} //end of if (descNode.getParentsSize()>1)

				} //end of for (iterator it2)
			} //end of if
		}

	}*/

	/**
	 * By Hong, allows more mapping strategies compared with generatePlan
	 * generates raindrop plan from a rainbow plan
	 *@param xattree XATTree the rainbow plan
	 *@param inAutoNum the number of operators that we target to have in the automata in the generated raindrop logical plan
	 */
	public XATDag generatePlan(XATTree xattree, int inAutoNum) {
		//distill
		AutoDistiller distiller = new AutoDistiller(mappingStrategy);

		if (mappingStrategy == AutoDistiller.MAPPING_NAVWITHSELONLY
			|| mappingStrategy == AutoDistiller.MAPPING_NAVUNNESTWITHSELONLY
			|| mappingStrategy == AutoDistiller.MAPPING_NAVONLY
			|| mappingStrategy == AutoDistiller.MAPPING_SELPARTIALIN)
			//need to specify inAutoNum in distiller
			distiller.setInAutomataNum(inAutoNum);

		distiller.distill(xattree);

		//make logical plan by
		//Top-down visit
		visit(xattree.getRoot(), null);

		//printTree();

		return this.xatdag;

	}

	/**
	* print the raindrop plan
	*/
	public void printTree() {

		//do a breadth first traverse from top down
		try {
			//do a breadth first traverse from top down
			Vector traversal = DagTraversal.dagTopDownTraverse(this.xatdag);
			for (Iterator it = traversal.iterator(); it.hasNext();) {
				DagNode curNode = (DagNode) it.next();
				//visit curNode
				System.out.println("( " + curNode.getOperator());

				for (int i = 0; i < curNode.getChildrenSize(); i++)
					System.out.println(
						"    -->" + curNode.getChild(i).getOperator());
				System.out.println(")");
				System.out.println();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
	/**
	 * This method maps a Navigate operator to a pair of physical operators.
	 * i.e, a navigate and a structural join.
	 * The Navigate is called openOp, 
	 * its semantic is to "goto" a specific position of the document,
	 * i.e, the open tag.
	 * The Structural Join is called closeOp,
	 * its semantic is to "collect" data from the node,
	 * it is executed when the close tag arrives.
	 *
	 * The schema in each dagnode is generated by copying
	 * the schema in each xatnode, which is assumed to have been 
	 * properly set up by rainbow codes.
	 * 
	 * Creation date: (2002-12-20 16:30:32)
	 * @param op edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnest
	 */
	public void visit(XATNode node, DagNode curParent) {

		XATOperator curOp = node.getOperator();
		DagNode curNode = (DagNode) opToNode.get(curOp);
		if (curNode == null) {
			curNode = new DagNodeImp();
			opToNode.put(curOp, curNode);		
		}

		//connect the node
		if (curParent == null) {
			curParent = curNode;
			xatdag = new XATDagImp();
			xatdag.setRoot(curNode);
		} else {
			curParent.addChild(curNode);
			curNode.addParent(curParent);
		}

		//pre-order Top-down visit

		curNode.setOperator(curOp);
		curOp.setNode(curNode);
		//curParent = curNode;

		//set schema, 
		if (node.getSchema() == null) {
			//it must be a AutoMegaOperator
			//because the AutoDistiller is called first,
			//the mega operator should have been properly set up
			if (!(curOp instanceof AutoMegaOperator)) {
				System.out.println("schema is not set in " + curOp);
				System.exit(-1);
			}
		//	Set schema = ((AutoMegaOperator) curOp).getBoundary().getSchema();
		//	curNode.setSchema(schema);
		} else 
			curNode.setSchema(new HashSet(node.getSchema()));
		

		XATNode[] children = node.getChildren();
		for (int i = 0; i < children.length; i++) 
			visit(children[i], curNode);
	}

	/*
	 * this extends the original autologicalplangenerator
	 * the original one supposes for one source, there is only a linear pattern on this source
	 * while this is not necessarily true for the below case
	 * for $a in /a, $b in $a/b
	 * it would NOT be translated into
	 *   NavUnnest($a, /b->$b)
	 *     |
	 *   NavUnnest(source, /a->$a)
	 *     |
	 *   source
	 * but rather
	 *                   merge
	 *        |                            |
	 *        |                          NavUnnest($a, /b->$b)
	 *        |                            |
	 *     NavUnnest(source, /a->$a)     NavUnnest(source, /a->$b)
	 *        |                            |
	 *     source
	 * however we want the former format which lead to less complex raindrop plan
	 * so we linearize the latter format to the former format
	 */
	private void linearize(XATTree xattree) {
		Vector nodes = TreeTraversal.treeTopDownTraverse(xattree);
		for (Iterator it = nodes.iterator(); it.hasNext();) {
			XATNode node = (XATNode) it.next();
			XATOperator nodeOp = node.getOperator();
			if (nodeOp instanceof Merge || nodeOp instanceof LeftOuterJoin) {
				//its branches can be linearized

				//create a Vector on navigation step and navigation type
				XATTree subtree = new XATTreeImp();
				subtree.setRoot(node);

				Vector leaves = subtree.getAllLeafNodes(node);

				Vector nodeSetGuard =
					getNodesOnPath((XATNode) leaves.get(0), node);

				XATNode finalRoot = null;
				for (int i = 1; i < leaves.size(); i++) {
					Vector nodeSetCompared =
						getNodesOnPath((XATNode) leaves.get(i), node);
					finalRoot = merge(nodeSetGuard, nodeSetCompared);
				}

				//keep only the first branch of merge
				//also remove merge or LOJ
				XATNode mergeParent = node.getParent();
				mergeParent.deleteChild(node);
				mergeParent.addChild(finalRoot);
				finalRoot.setParent(mergeParent);
			} //end of if (node.getOperator() instanceof Merge) 
		}
	}

	/*
	 * @return Vector the nodes from descNode to anceNode, does not include either end
	 * @param descNode XATNode the descendant node
	 * @param anceNode XATNode the ancestor node
	 */
	private Vector getNodesOnPath(XATNode descNode, XATNode anceNode) {
		Vector nodes = new Vector();
		XATNode curNode = descNode.getParent();

		while (curNode != anceNode) {
			nodes.add(curNode);
			curNode = curNode.getParent();
		}

		return nodes;
	}

	/* merge them into a linear set
	 * @return XATNode the root of the merged vector
	 * @param Vector nodeSet1, a set of nav operator nodes
	 * @param Vector nodeSet2, a set of nav operator nodes
	 */
	private XATNode merge(Vector nodeSet1, Vector nodeSet2) {
		//keep track
		Vector nodeSet1Copy = (Vector) nodeSet1.clone();

		int preNodePos = -1;
		int nodeSet1Size = nodeSet1.size();
		int nodeSet2Size = nodeSet2.size();
		for (int i = 0; i < nodeSet2Size; i++) {
			XATNode nodeInSet2 = (XATNode) nodeSet2.get(i);
			XATOperator nodeInSet2Op = nodeInSet2.getOperator();

			if (!(nodeInSet2Op instanceof Navigate)) {
				//insert after where its child is inserted into
				nodeSet1Copy.insertElementAt(nodeInSet2, preNodePos + 1);
				continue;
			}

			if (preNodePos >= nodeSet1Size - 1) {
				//the previous node has been added to the last of nodeSet1Copy
				//so add this to the last as well
				nodeSet1Copy.add(nodeInSet2);
				continue;
			}

			//NavigationStep[] dest2 = ((Navigate) nodeInSet2Op).getDestination();
			NavigationStep[] dest2 = expand((Navigate) nodeInSet2Op);

			for (int j = preNodePos + 1; j < nodeSet1Size; j++) {
				NavigationStep[] entry2 =
					((Navigate) nodeInSet2Op).getEntryPoint();

				XATNode nodeInSet1 = (XATNode) nodeSet1.get(j);
				XATOperator nodeInSet1Op = nodeInSet1.getOperator();

				if (!(nodeInSet1Op instanceof Navigate))
					continue;

				NavigationStep[] entry1 =
					((Navigate) nodeInSet1Op).getEntryPoint();
				//NavigationStep[] dest1 =
				//	((Navigate) nodeInSet1Op).getDestination();
				NavigationStep[] dest1 = expand((Navigate) nodeInSet1Op);
				if (!equi(entry1, entry2)) {
					//no order dependency between these two
					//insert nodeInSet2 before nodeInSet1
					int nodeInSet1Pos = nodeSet1Copy.indexOf(nodeInSet1);
					nodeSet1Copy.insertElementAt(nodeInSet2, nodeInSet1Pos);
					preNodePos = j;
					break;
				} else { //two nodes have same entry
					//check who should be first
					int containResult = containing(dest1, dest2);
					if (containResult > 0) {
						//dest 1 is longer than dest 2
						//dest 1 should be after dest 2

						int nodeInSet1Pos = nodeSet1Copy.indexOf(nodeInSet1);
						nodeSet1Copy.insertElementAt(nodeInSet2, nodeInSet1Pos);
						preNodePos = j;

						//dest 1 should be rewritten with dest 2  
						NavigationStep[] newEntry = new NavigationStep[1];
						newEntry[0] =
							new VarStepImp(
								((Navigate) nodeInSet2Op)
									.getOutputColumnName());
						//((Navigate) nodeInSet2Op).getBinding());
						//						
						 ((Navigate) nodeInSet1Op).setEntryPoint(newEntry);
						((Navigate) nodeInSet1Op).setDestination(
							truncate(dest1, containResult));

						break;
					} else if (containResult == EQUI) {
						if ((nodeInSet1Op instanceof NavCollection
							&& nodeInSet2Op instanceof NavCollection)
							|| (nodeInSet1Op instanceof NavUnnest
								&& nodeInSet1Op instanceof NavUnnest)) {
							//exactly the same
							//do not insert nodeInSet2Op, i.e., the two nodes are merged into the same one
							preNodePos = j;
							break;
						} else {
							//they must be of different types
							//continue to find whether there are an exactly same one
							preNodePos = j;
							XATNode exactSameNode =
								this.findExactSame(nodeInSet2, nodeInSet1);
							if (exactSameNode == null) {
								//no dependency
								int nodeInSet1Pos =
									nodeSet1Copy.indexOf(nodeInSet1);
								nodeSet1Copy.insertElementAt(
									nodeInSet2,
									nodeInSet1Pos);
								break;
							} else { //find exact same node, merge to the same
								//the child of nodeInSet2 will start comparing with the parent of the exactsamenode
								preNodePos = nodeSet1.indexOf(exactSameNode);
								break;
							}
						}
					} else if (
						containResult < 0 && containResult != NORELATION) {
						//dest2 is longer than dest1	
						//rewrite dest2 with dest1
						NavigationStep[] newEntry = new NavigationStep[1];
						newEntry[0] =
							new VarStepImp(
								((Navigate) nodeInSet1Op)
									.getOutputColumnName());
						//((Navigate) nodeInSet1Op).getBinding());
						 ((Navigate) nodeInSet2Op).setEntryPoint(newEntry);
						((Navigate) nodeInSet2Op).setDestination(
							truncate(dest2, -containResult));
						if (j == nodeSet1Size) {
							//last one in set 1
							//nodeInSet2 can insert after it
							nodeSet1Copy.add(nodeInSet2);
							preNodePos = j;
							break;
						}
						//cannot break, continue to find the op in set1 whose dest is longer than dest2
						//so that nodeInSet2 can be inserted into nodeSet1Copy												
					} else if (containResult == NORELATION) {
						int nodeInSet1Pos = nodeSet1Copy.indexOf(nodeInSet1);
						nodeSet1Copy.insertElementAt(nodeInSet2, nodeInSet1Pos);
						//do not set preNodePos, current node does not have dependency on nodeInSet1
						//but next node may have
						//preNodePos = j;
					}
				} //end of if (!entry1.equals(entry2))

			}
		}

		//finally make it a linear plan
		int size = nodeSet1Copy.size();
		for (int i = 1; i < size; i++) {
			XATNode curNode = (XATNode) nodeSet1Copy.get(i);
			XATNode preNode = (XATNode) nodeSet1Copy.get(i - 1);
			//hardcode, each can has only one child
			curNode.deleteChild(curNode.getChild(0));
			curNode.addChild(preNode);
			preNode.setParent(curNode);
		}
		//return the last one in the vector, i.e., the top one
		return (XATNode) nodeSet1Copy.get(nodeSet1Copy.size() - 1);
	}

	//whether steps 1 longer than steps 2 and contains it
	private int containing(NavigationStep[] steps1, NavigationStep[] steps2) {
		int length1 = steps1.length;
		int length2 = steps2.length;

		if (length1 > length2) {
			for (int i = 0; i < length2; i++) {
				if (!steps1[i].equals(steps2[i]))
					return NORELATION;
			}
			//positive value,
			return length2;
		}

		if (length1 == length2) {
			for (int i = 0; i < length2; i++) {
				if (!steps1[i].equals(steps2[i]))
					return NORELATION;
			}
			return EQUI;
		}

		if (length2 > length1) {
			for (int i = 0; i < length1; i++) {
				if (!steps1[i].equals(steps2[i]))
					return NORELATION;
			}
			//1 is contained in 2, return negative value
			return -length1;
		}
		//will never reach
		return NORELATION;
	}

	/* @return boolean, whether steps1 is equivalent to steps2
	 * @param steps1 NavigationStep[]
	 * @param steps2 NavigationStep[]
	 */
	private boolean equi(NavigationStep[] steps1, NavigationStep[] steps2) {
		if (steps1.length != steps2.length)
			return false;
		else
			for (int i = 0; i < steps1.length; i++)
				if (!steps1[i].equals(steps2[i]))
					return false;

		return true;
	}

	/* @return NavigationStep[] 
	 * @param nav Navigate
	 */
	private NavigationStep[] expand(Navigate nav) {

		XATNode navNode = nav.getNode();
		NavigationStep[] dest = nav.getDestination();

		Vector result = new Vector();

		String entry = nav.getEntryPoint()[0].toString();
		XATNode curNode = navNode;
		XATOperator curOp = curNode.getOperator();

		while (!(curNode.getChild(0).getOperator() instanceof Source)) {
			curNode = curNode.getChild(0);
			curOp = curNode.getOperator();

			if (!(curOp instanceof Navigate)) {
				curNode = curNode.getChild(0);
				curOp = curNode.getOperator();
				continue;
			}

			//must be navigate operator
			if (((Navigate) curNode.getOperator())
				.getOutputColumnName()
				.equals(entry)) {

				NavigationStep[] curStep = expand((Navigate) curOp);
				for (int i = 0; i < curStep.length; i++)
					result.add(curStep[i]);
				break;
			}

		}

		//convert vector to array		  
		NavigationStep[] finalResult =
			new NavigationStep[result.size() + dest.length];
		for (int j = 0; j < result.size(); j++)
			finalResult[j] = (NavigationStep) result.get(j);

		for (int j = 0; j < dest.length; j++)
			finalResult[j + result.size()] = dest[j];

		return finalResult;
	}

	/* @return NavigationStep[] return navigation steps truncated from the int position of the input steps
	 * @param steps NavigationStep[] 
	 * @param pos int
	 */
	private NavigationStep[] truncate(NavigationStep[] steps, int pos) {
		NavigationStep[] result = new NavigationStep[steps.length - pos];
		for (int i = pos; i < steps.length; i++)
			result[i - pos] = steps[i];

		return result;
	}

	//find exact same node of navNode1 which is ancestor of navNode2
	XATNode findExactSame(XATNode navNode1, XATNode navNode2) {
		Navigate navOp = (Navigate) navNode1.getOperator();
		NavigationStep[] dest = navOp.getDestination();

		XATNode curNode = navNode2;

		while (curNode != null) {

			//find the immediate navigate ancestor
			while (curNode.getParent() != null) {
				curNode = curNode.getParent();
				if (curNode.getOperator() instanceof Navigate)
					break;
			}

			if (curNode.getParent() != null) {
				//must be a navigate parent
				if (containing(((Navigate) curNode.getOperator())
					.getDestination(),
					dest)
					== EQUI) {
					if ((curNode.getOperator() instanceof NavUnnest
						&& navNode1.getOperator() instanceof NavUnnest)
						|| (curNode.getOperator() instanceof NavCollection
							&& navNode1.getOperator() instanceof NavCollection))
						//same type && same destination, our target
						return curNode;
				} else //else of if (containing(expand((Navigate) curNode), dest) == EQUI)
					//not same destination, stop finding
					return null;
			} //end if (curNode != null)

		} //end of while (curNode != null) 

		return null;
	}

}
