/*
 * move the specified nodeNav in, including the nodeNavs that are dependent on
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNodeImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.SelectStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.StructuralJoinStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractByCollectionStreamImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractByUnnestStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavUnnestStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.Navigate;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.BranchStatistics;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.CopiedDag;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.CostComp;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.RuntimeOptUtil;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.SimulateMoveResult;

/**
 * @author suhong
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class SpecifiedNavPushinRewriteRule extends GeneralRaindropRewriteRules {
	SimulateMoveResult result;

	CopiedDag tempPlan;

	List addedSjs = new ArrayList();

	//these sj nodes appear in both plans before and after
	List affectedSjs = new ArrayList();

	List addedTokenNavs = new ArrayList();

	AffectedPart affectedPart;

	NavigateStream targetNodeNav;

	PureNavigateAutoStream derivedTokenNav;

	Map newSjToBranchStatistics = new HashMap();

	Map newEntryOpToInputSj = new HashMap();
	
	Map newBranchToStatistics = new HashMap();

	public void setAffectedPart(AffectedPart affectedPart) {
		this.affectedPart = affectedPart;
	}

	public AffectedPart getAffectedPart() {
		return affectedPart;
	}

	public List getAffectedSjs() {
		return affectedSjs;
	}

	public List getAddedTokenNavs() {
		return addedTokenNavs;
	}

	public SpecifiedNavPushinRewriteRule(SimulateMoveResult result,
			NavigateStream targetNodeNav) {
		this.result = result;
		this.tempPlan = result.getTempPlan();
		this.targetNodeNav = targetNodeNav;
	}

	public Object visit(NavigateStream parent, NavigateStream child) {
		//first decide whether parent is dependent on child
		DagNode parentNode = tempPlan.getNode(parent);
		DagNode childNode = tempPlan.getNode(child);
				
		//if not, swap them
		String parentEntry = parent.getEntryPoint()[0].getStepValue();
		String destination = child.getOutputColumnName();
		if (!parentEntry.equals(destination)){
			//no data dependancy
			swap(parentNode, childNode);
			DagNode grandChildNode = (DagNode)parentNode.getChild(0);
			XATStreamOperator grandChildOp = (XATStreamOperator)grandChildNode.getOperator();
			return visit(parent, grandChildOp);
		}
		
		
		//child is dependent by parent
		//push child first
		//and then push parent
		

		//navigatestream can have only one child
		DagNode grandChildNode = (DagNode) childNode.getChild(0);
		XATStreamOperator grandChildOp = (XATStreamOperator) grandChildNode
				.getOperator();

		visit(child, grandChildOp);

		DagNode curChildNode = (DagNode) parentNode.getChild(0);
		XATStreamOperator curChildOp = (XATStreamOperator) curChildNode
				.getOperator();

		visit(parent, curChildOp);

		return new AffectedPart(new ArrayList(), addedSjs, new ArrayList(),
				addedTokenNavs, affectedSjs, new HashMap(), newSjToBranchStatistics,
				newEntryOpToInputSj, newBranchToStatistics);
	}

	//return the strucutral join node that tokenNav's entry operators lies on
	public Object visit(NavigateStream parent, StructuralJoinStream child) {

		NavigationStep[] entries = parent.getEntryPoint();
		DagNode navNode = tempPlan.getNewNodeFromOldNode((DagNode) parent
				.getNode());

		//find which branch of child the sourceNavNode lies on
		DagNode sjNode = tempPlan.getNewNodeFromOldNode((DagNode) child
				.getNode());
		if (sjNode == null)
			//sjNode is newly added
			sjNode = (DagNode) child.getNode();

		int i = findPushPos(parent, child);

		if (i == -1) {
			return null;
		}

		//mark the changed part
		if (parent == targetNodeNav) {
			AffectedSj affectedSj = new AffectedSj(child,
					(XATStreamOperator) sjNode.getChild(i).getOperator(),
					false, false, false);
			affectedSjs.add(affectedSj);
		}

		//put navNode above the i th child of sjNode
		sjNode.getChild(i).setParent(navNode);
		navNode.deleteChild(sjNode);
		navNode.addChild(sjNode.getChild(i));
		sjNode.deleteParent(navNode);
		for (Iterator it = navNode.getParents(); it.hasNext();) {
			DagNode navNodeParent = (DagNode) it.next();
			sjNode.addParent(navNodeParent);
			navNodeParent.deleteChild(navNode);
			navNodeParent.addChild(sjNode);
		}
		navNode.deleteAllParents();
		navNode.addParent(sjNode);
		sjNode.addChild(navNode);
		sjNode.deleteChild(sjNode.getChild(i));

		return visit(parent, navNode.getChild(0).getOperator());

	}

	public Object visit(NavigateStream parent, ExtractStream child) {
		boolean entryOpRemoved = false;

		DagNode parentNode = tempPlan.getNewNodeFromOldNode((DagNode) parent
				.getNode());
		DagNode childNode = tempPlan.getNewNodeFromOldNode((DagNode) child
				.getNode());
		if (childNode == null)
			//newly added
			childNode = (DagNode) child.getNode();

		//split parent to a pure nav + extract + structuraljoin (if there is
		// none)
		NavigationStep[] entries = parent.getEntryPoint();

		if (entries[0].getStepValue().equals(child.getOutputColumnName())) {
			//add a new nav
			DagNode extractChild = (DagNode) childNode.getChild(0);
			PureNavigateAutoStream navForParent = (PureNavigateAutoStream) RuntimeOptUtil.nodeNavToTokenNav
					.get(parent);
			if (navForParent == null) {
				navForParent = new PureNavigateAutoStream(parentNode);
				navForParent.setEntryPoint(entries);
				navForParent.setDestination(parent.getDestination());
				navForParent.setOutputColumnName(parent.getOutputColumnName());
				RuntimeOptUtil.nodeNavToTokenNav.put(parent, navForParent);
			} else
				parentNode.setOperator(navForParent);

			extractChild.addParent(parentNode);
			parentNode.addChild(extractChild);
            //cut the relationship of parentNode and childNode
			parentNode.deleteChild(childNode);
			childNode.deleteParent(parentNode);

			addedTokenNavs.add(navForParent);
			//register new node
			tempPlan.registerOpToNode(navForParent,  parentNode);

			//add a new extract
			ExtractStream extractForParent = (ExtractStream) RuntimeOptUtil.tokenNavToExtract
					.get(navForParent);
			DagNode newExtractNode = new DagNodeImp();
			if (extractForParent == null) {				
				if (parent instanceof NavUnnestStream) {
					extractForParent = new ExtractByUnnestStream(newExtractNode,
							parent.getEntryPoint(), parent.getDestination());
					extractForParent.setOutputColumnName(parent
							.getOutputColumnName());
				} else {
					extractForParent = new ExtractByCollectionStreamImp(
							newExtractNode, parent.getEntryPoint(), parent
									.getDestination());
					extractForParent.setOutputColumnName(parent
							.getOutputColumnName());
				}

				//register
				RuntimeOptUtil.tokenNavToExtract.put(navForParent,
						extractForParent);
			}
			//cannot be skiped, extractForParent may come from the initial plan
			//have to reset its node to the current node
			extractForParent.setNode(newExtractNode);
			newExtractNode.setOperator(extractForParent);
			         
			//insert extract node between parentnode and parentnode's original parents
			for (Iterator it = parentNode.getParents(); it.hasNext();){
				DagNode grandParent = (DagNode)it.next();
				grandParent.deleteChild(parentNode);
				newExtractNode.addParent(grandParent);
				grandParent.addChild(newExtractNode);
				it.remove();
			}
			parentNode.addParent(newExtractNode);
			newExtractNode.addChild(parentNode);
			
			//register new node
			tempPlan.registerOpToNode(extractForParent, newExtractNode);

			boolean createNewSj = true;
			PureNavigateAutoStream childTokenNav = (PureNavigateAutoStream) child
					.getNode().getChild(0).getOperator();
			StructuralJoinStream sj = (StructuralJoinStream) RuntimeOptUtil.getTokenNavToFanInSj()
					.get(childTokenNav);
			DagNode sjNode = tempPlan.getNode(sj);

			if (sjNode != null) 
				createNewSj = false;
			else {
				sjNode = new DagNodeImp();
				sj.setNode(sjNode);
				sjNode.setOperator(sj);
			}

			if (!createNewSj) {
				//place ancestors of parentNode into right branch
				//if they rely on parent, they stay the same
				//otherwise, they must rely on child, and must be placed above
				DagNode curNodeOnExtract = childNode;

				//all the entries that relay on the extract
				List entriesRelatedToExtract = new ArrayList();
				entriesRelatedToExtract.add(child.getOutputColumnName());
				DagNode curNodeOnNav = newExtractNode;

				while (curNodeOnNav.getParentsSize() == 1
						&& curNodeOnNav.getParent().getOperator() != sj) {

					DagNode navNodeAnce = (DagNode) curNodeOnNav.getParent();
					XATStreamOperator navNodeAnceOp = (XATStreamOperator) navNodeAnce
							.getOperator();

					String entry = "";
					if (navNodeAnceOp instanceof NavigateStream)
						entry = ((NavigateStream) navNodeAnceOp)
								.getEntryPoint()[0].getStepValue();
					else if (navNodeAnceOp instanceof SelectStream)
						entry = ((SelectStream) navNodeAnceOp).getExpression()
								.getRelatedColumnNames()[0].getColumnName();

					if (entriesRelatedToExtract.contains(entry)) {
						//rely on the extract
						entriesRelatedToExtract.add(entry);
						navNodeAnce.deleteChild(curNodeOnNav);
						DagNode navNodeAnceParent = (DagNode) navNodeAnce
								.getParent(0);
						navNodeAnceParent.addChild(curNodeOnNav);
						curNodeOnNav.addParent(navNodeAnceParent);

						//add navNodeAnce on curNodeOnExtract
						curNodeOnExtract.addParent(navNodeAnce);
						navNodeAnce.addChild(curNodeOnExtract);
						curNodeOnExtract = navNodeAnce;
					}

					curNodeOnNav = (DagNode) curNodeOnNav.getParent();
				}

				//set up parent-child relatonship between sj and
				// curNodeOnExtrac
				//before setting up relationship, first make sure we do need this extract
				
				if (curNodeOnExtract == childNode){
					//there is no further navigation depending on this extract
					//we further check whether it needs to be returned
					String outputCol = child.getOutputColumnName();
					List outputCols = sj.getOutputQueue().getSchema().getColNames();
					if (!outputCols.contains(outputCol)){
						//not returned
						//futher check whether it is needed by other branch rather than the one by parent
						if (curNodeOnExtract.getParentsSize()== 0 || (curNodeOnExtract.getParentsSize()==1 && curNodeOnExtract.getParent() instanceof StructuralJoinStream))
							entryOpRemoved = true;		
						
					}
				}
				
				if (!entryOpRemoved){
				//	if (curNodeOnExtract.getParentsSize()==0){
				//		curNodeOnExtract.addParent(sjNode);
				//	    sjNode.addChild(curNodeOnExtract);
				//	}
					    
				}else
				if (entryOpRemoved){
					//remove this extract
					//grandChildNode must be a tokenNav
					DagNode grandChildNode = (DagNode)childNode.getChild(0);
					grandChildNode.deleteParent(childNode);
				}
				
				//register changes
				AffectedSj affectedSj = new AffectedSj(sj, child, false, entryOpRemoved, false);
				affectedSjs.add(affectedSj);

			}
			if (createNewSj) {
				//put extract's original parents all above sjNode
				for (Iterator it = newExtractNode.getParents(); it.hasNext();) {
					DagNode extractNodeParent = (DagNode) it.next();
					if (extractNodeParent != parentNode) {
						sjNode.addParent(extractNodeParent);
						extractNodeParent.deleteChild(newExtractNode);
						extractNodeParent.addChild(sjNode);
					} else {
						//put nav's parent above sjNode
						for (Iterator it2 = newExtractNode.getParents(); it2
								.hasNext();) {
							DagNode navParent = (DagNode) it2.next();
							sjNode.addParent(navParent);
							navParent.deleteChild(newExtractNode);
							navParent.addChild(sjNode);
						}
					}
				}

				newExtractNode.deleteAllParents();

				sjNode.addChild(newExtractNode);
				sjNode.addChild(childNode);
				childNode.addParent(sjNode);
				newExtractNode.setParent(sjNode);
				childNode.setParent(sjNode);
				
				//register new node 
				tempPlan.registerOpToNode(sj, sjNode);
				addedSjs.add(sj); 
			}
            
			//recompute the branch statistics of the old extract (whose parent operators have been changed)
			//and the new extract
			if (!entryOpRemoved){
				BranchStatistics branchStaForChild = CostComp.computeBranchStatistics(childNode);
			    newBranchToStatistics.put(child, branchStaForChild);
			}
			BranchStatistics branchStaForNewExtract = CostComp.computeBranchStatistics(newExtractNode);
			newBranchToStatistics.put(extractForParent, branchStaForNewExtract);
			
		}

		return new AffectedPart(new ArrayList(), addedSjs, new ArrayList(),
				addedTokenNavs, affectedSjs, new HashMap(), newSjToBranchStatistics,
				newEntryOpToInputSj, newBranchToStatistics);

	}

	public Object visit(Navigate parent, SelectStream child) {

		//		push to be underneath select since it will be down to a sj anyway
		XATNode parentNode = tempPlan.getNewNodeFromOldNode((DagNode) parent
				.getNode());
		XATNode childNode = tempPlan.getNewNodeFromOldNode((DagNode) child
				.getNode());

		parentNode.setOperator(child);
		childNode.setOperator(parent);

		return visit(parent, parentNode.getChild(0).getOperator());

	}

}