/*
 * Created on Jan 29, 2004
 *
 * construct a query graph ouft of a subplan (i.e., the automata plan)
 * outof the raindrop physical plan
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo;

import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.FindOpVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.XATStreamOperator;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.OuterExtractByCollectionStream;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.ExtractByCollectionStream;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.ExtractByUnnestStream;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.ExtractStream;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.PureNavigateAutoStream;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.XMLScanStream;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.xmloperator
	.navigation
	.NavigationStep;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;

/**
 * @author hong su
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class QueryGraphInitiator {

	//each vector stores the element's name within one source which is concerned by the query 
	private Set[] concernedSchemas = new Set[20];
	//the hashtable maps a source's name to the position of its concerned elements
	public Hashtable sourceMapper = new Hashtable();
	int globalSourceId = 0;

	//given an entry's name, find the qgn which is bound to this entry
	private Hashtable entryToQgn = new Hashtable();

	private String schemaSourceName;

	public Set getConcernedSchema(String sourceName) {
		return concernedSchemas[(
			(Integer) sourceMapper.get(sourceName.toLowerCase()))
			.intValue()];
	}

	public void setSchemaSourceName(String schemaFile) {
		schemaSourceName = schemaFile;
	}

	/**
		 * given a subplan which is done in automata, construct a query graph out of it.
		 * Creation date: 
		 * @return QueryGraph
		 * @param plan XATDag
		 */
	public QueryGraph[] construct(XATDag plan) {

		QueryGraph[] result = new QueryGraph[10];

		XATNode[] leaves = (XATNode[]) plan.getLeaves();

		int leavesSize = leaves.length;

		for (int i = 0; i < leavesSize; i++) {

			DagNode leaf = (DagNode) leaves[i];
			//for each leaf, i.e., each source, construct a query graph

			DagNode planNode = leaf;

			QueryGraphNode parentQgn = null;

			concernedSchemas[i] = new HashSet();

			XATStreamOperator leafOp =
				(XATStreamOperator) planNode.getOperator();

			if (leafOp instanceof XMLScanStream) {

				sourceMapper.put(
					schemaSourceName.toLowerCase(),
					new Integer(globalSourceId));

				result[i] = addIntoQueryGraph(null, null,
					//source operator can have only one parent which must be a stream navigate
						planNode.getParent(0), concernedSchemas[globalSourceId]);

				globalSourceId++;

				//complementMissingType needs the bottom entries
				//set it
				result[i].setBottomEntries();
				complementMissingType(result[i]);

			} else
				throw new RuntimeException("leaf node of query plan is not source operator");
		}

		System.out.println("query graph:\n" + result[0]);
		return result;
	}

	/**
	 * given a node in XAT dag, generate corresponding query graph constructs
	 * We use planNode to refer the node in the XATDag
	 * We use graphNode to refer the node in the query graph
	 * Creation date: 
	 * @return QueryGraph
	 * @param qg QueryGraph the query graph into which the new node is added to
	 * @param qgParentNode QueryGraphNode the query graph node of which the new node is inserted as a child
	 * @param planNode DagNode the node in the XAT Dag which will be mapped to query graph node(s)b
	 */
	private QueryGraph addIntoQueryGraph(
		QueryGraph qg,
		QueryGraphNode qgnParentNode,
		DagNode planNode,
		Set concernedSchema) {

		XATStreamOperator currentOp =
			(XATStreamOperator) planNode.getOperator();

		if (currentOp instanceof PureNavigateAutoStream) {
			//if associated with NavCollectionStream, it requires the pattern to appear
			//if associated with NavUnnestStream, it does not require the pattern to appear	
			//if not associated, then it must be a navigate directly following source	
			NavigationStep[] steps = null;

			steps = ((PureNavigateAutoStream) currentOp).getDestination();

			//find the associated extract operator
			ExtractStream associatedExtract =
				FindOpVisitor.findAssociatedExtract(planNode);

			for (int j = 0; j < steps.length; j++) {

				QueryGraphNode qgn;
				String tagName = steps[j].getStepValue();
				concernedSchema.add(tagName);

				//case 1: associated extract operator does not filter
				if (associatedExtract instanceof OuterExtractByCollectionStream) {
					//futher check whether the binding of this purenavigate is the group by
					//variable of the associatedExtract
					//if yes, this step filters
					//e.g., AccessCollection($a, $c) PureNavigate /a->$a
					//then /a filters
					String entryName =
						((OuterExtractByCollectionStream) associatedExtract)
							.getEntryPoint()[0]
							.getStepValue();
					QueryGraphNode qgnForEntry =
						(QueryGraphNode) entryToQgn.get(entryName);

					if (qgnForEntry != null)
						qgnForEntry.setType(QueryGraphNode.FILTER);

					qgn = new QueryGraphNode(qg, QueryGraphNode.NONFILTER, tagName);

					if (j == steps.length - 1) {
						//the last navigation step in the access operator
						//qg.addBottomEntry(qgn);
						//access operator is for those nodes returned  but not appearing in FOR clauses
						//qgn.setReturned(true);

						//this qgn is the destination of the associatedExtract
						qgn.addOpOnDest(currentOp);
					} else if (j == 0)
						qgnParentNode.addOpOnEntry(currentOp);
				} else if (
					associatedExtract instanceof ExtractByCollectionStream
						|| associatedExtract instanceof ExtractByUnnestStream) {
					//case 2: associated extract operator filters
					qgn =
						new QueryGraphNode(
							qg, QueryGraphNode.FILTER,
							steps[j].getStepValue());

					if (j == steps.length - 1) {
						if (((ExtractStream) associatedExtract).isTypedValue())
							//there must be a selection on it since typed value is designed for selection
							qgn.setIsSel(true);
						else
							qgn.setIsNonSelDest(true);

						//need to be returned to non-automata computations
						if (((ExtractStream) associatedExtract).isDom()
							|| ((ExtractStream) associatedExtract)
								.isCompleteTree())
							qgn.setIsReturned(true);

						//this qgn is the destination of the associatedExtract
						qgn.addOpOnDest(currentOp);
					} else if (j == 0 && qgnParentNode!=null)
						qgnParentNode.addOpOnEntry(currentOp);

				} //end of if (currentOp instanceof ExtractStream)
				else {

					//case 3: there is no associated extract operator,
					//case 3.1: when the associated extract operator has been eliminated
					//since it has been indicated by its children's EXTRACT type
					//e.g., for $a in ..., $b in $a/..., $c in $a/... return f($b), f($a) can be expressed as
					//                 structuralJoin($a)
					//        /                 |                 \
					//extractunnest($a, $b)     |             extractunnest($a, $c)
					//         \                |                 /
					// nav($a, ..->$b)  extractunnest(,$a)   nav($a, ..->$c)    
					//          \               |               / 
					//                  nav(, ..->$a)
					//in which the extractunnest(, $a) can be elimiated
					//case 3.2:, LET $a = $o/..., for $o in ..., return <tag>$a/b, $a/c$</tag>
					//                structuralJoin($a)
					//        /                             \
					//access(a, $b)                 access($a, $c)
					//         \                             /
					// nav($a, ..->$b)                   nav($a, ..->$c)    
					//             \            |          / 
					//                  nav($o, ..->$a)
					//                          |
					//                  nav(, ..->$o)
					//this one $a is not required to appear
					//therefore should refer to the children's type
					qgn = new QueryGraphNode(qg, tagName);

					if (j == steps.length - 1) {
						qgn.setIsNonSelDest(true);
						qgn.addOpOnDest(currentOp);
					}

					entryToQgn.put(
						((PureNavigateAutoStream) currentOp).getBinding(),
						qgn);
				}

				if (qgnParentNode != null) {
					qgnParentNode.addChild(qgn);
					qgn.setParent(qgnParentNode);

					if (j == 0) {
						//this qgnParentNode is the entry of the associatedExtract
						qgnParentNode.addOpOnEntry(currentOp);

					}

					qgnParentNode = qgn;

				} else {
					//qg must also be null, no node in the graph yet
					//set the newly generated graph node as the top entry of the new graph
					qg = new QueryGraph(qgn);
					qgnParentNode = qgn;
					//qg.addTopEntry(qgn);
				}
			} //finished processing the local planNode

			//continue to process the parent node of the planNode in Dag
			for (Iterator i = planNode.getParents(); i.hasNext();) {
				DagNode parentPlanNode = (DagNode) i.next();
				addIntoQueryGraph(
					qg,
					qgnParentNode,
					parentPlanNode,
					concernedSchema);
			}

		} //end of if (currentOp instanceof PureNavigateAutoStream)

		return qg;

	}

	/**
		 * given a query graph, revisit it, set the types for the nodes who haven't been done so yet
		 * these nodes are mapped from the PureNavigateAutoStream operators which do not have associated extract operator
		 * Creation date: 
		 * @param qg QueryGraph the query graph 		 
		 */
	private void complementMissingType(QueryGraph qg) {
		Vector curLevel = qg.getBottomEntries();

		while (curLevel.size() > 0) {

			Vector nextLevel = new Vector();

			//the bottom entries must have all been set types
			for (Iterator i = curLevel.iterator(); i.hasNext();) {

				QueryGraphNode curBottom = (QueryGraphNode) i.next();

				QueryGraphNode parentNode =
					(QueryGraphNode) curBottom.getParent();

				if (parentNode != null) {

					if (parentNode.getType() == 0)
						//not set yet
						setMissingType(parentNode);
					nextLevel.add(parentNode);
				}

			} //end of for
			curLevel = nextLevel;
		} //end of while
	}

	/**
			 * given a query graph node, set its types according for its children
			 * Creation date: 
			 * @param qgn QueryGraphNode the query graph node	
			 * 
			 */
	private void setMissingType(QueryGraphNode qgn) {
		if (qgn.getType() != 0 || qgn.getParent() == null)
			//has been set or the root node
			return;

		int childrenNum = qgn.getChildrenSize();

		boolean setToExtract = false;

		for (int i = 0; i < childrenNum; i++)
			if (((QueryGraphNode) qgn.getChild(i)).getType()
				== QueryGraphNode.FILTER) {
				setToExtract = true;
				break;
			}

		if (setToExtract)
			qgn.setType(QueryGraphNode.FILTER);
		else
			qgn.setType(QueryGraphNode.NONFILTER);

	}

}
