/*
 * Created on Jan 27, 2004
 *
 * node in the query graph
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.XATStreamOperator;
import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.xmloperator
	.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState;

public class QueryGraphNode extends GraphNodeImp implements GraphNode {
	private QueryGraph qg;

	//for indicating this subtree rooted at a node with this name will not have instance bindings
	public final static String EMPTY = "EMPTY";

	//a query tree node that will definitely exist will be eliminated
	//always non-emtpy result detection
	public final static String ELIMINATE = "ELIMINATED";

	//for wildcard in query, e.g.,a/*/c
	public final static String WILDCARD = "*";

	//for // in query, e.g., a//b
	public final static String DESC = "//";

	//i.e., type nodes
	private List traceStates = new ArrayList();

	//whether this node will be returned
	private boolean isReturned = false;

	//whether this node is an intermediate entry node (has children)
	private boolean isNonSelDest = false;

	//whether the destination node has a selection defined over it
	private boolean isSel = false;

	private int type = 0;
	//is the node connected to parent in dashed line (nonfilter) or solid line (filter) 
	public final static int NONFILTER = 1;
	public final static int FILTER = 2;

	private String tagName;

	//the nodes that may be, but not necessarily, shortcut by this one
	private List negativeNodes = new ArrayList();
	//the nodes that can be definitely shortcut by this one
	private List sureNegativeNodes = new ArrayList();

	//the operators whose entries are mapped to this node
	private List opsOnEntry = new ArrayList();

	//the operators whose destination are mapped to this node
	private List opsOnDest = new ArrayList();

	//given one tracestate of this node, find how many times its children can appear in this state
	private HashMap traceStateToOccurNum = new HashMap();

	//give one tracestate of this node, find how many childstate of it has been resolved
	private HashMap traceStateToResolvedChildrenNum = new HashMap();

	//give an ancestor type node, find the trace states of this query tree node which are derived from the given type node
	private HashMap ancestorStateToTraceStates = new HashMap();

	public QueryGraphNode(QueryGraph qg) {
		super();
		this.qg = qg;  
	}
	
	public QueryGraph getQg(){
		return qg;
	}

	public QueryGraphNode(QueryGraph qg, String tagName) {
		super();
		this.qg = qg;
		this.tagName = tagName;
	}

	public QueryGraphNode(QueryGraph qg, int type, String tagName) {
		super();
		this.qg = qg;
		this.type = type;
		this.tagName = tagName;
	}

	public void addOpOnEntry(XATStreamOperator op) {
		opsOnEntry.add(op);
	}

	public void addOpOnDest(XATStreamOperator op) {
		opsOnDest.add(op);
	}

	public List getOpsOnEntry() {
		return opsOnEntry;
	}

	public List getOpsOnDest() {
		return opsOnDest;
	}

	public String getTagName() {
		return tagName;
	}

	public void setTagName(String tagName) {
		this.tagName = tagName;
	}

	public boolean isReturned() {
		return isReturned;
	}

	public boolean isNonSelDest() {
		return isNonSelDest;
	}

	public void setIsNonSelDest(boolean isNonSelDest) {
		this.isNonSelDest = isNonSelDest;
	}

	public void setIsReturned(boolean returned) {
		this.isReturned = returned;
	}

	public int getType() {
		return type;
	}

	public void setType(int type) {
		this.type = type;
	}

	public boolean getIsSel() {
		return isSel;
	}

	public void setIsSel(boolean isSel) {
		this.isSel = isSel;
	}

	public List getNegativeNodes() {
		return negativeNodes;
	}

	public List getSureNegativeNodes() {
		return sureNegativeNodes;
	}

	public void addNegativeNode(QueryGraphNode qgn) {
		negativeNodes.add(qgn);
	}

	public void addSureNegativeNode(QueryGraphNode qgn) {
		sureNegativeNodes.add(qgn);
	}
	/* public Object clone(){
	 	    QueryGraphNode newNode = (QueryGraphNode) super.clone();
	 	    return newNode;
	 }*/

	/**
		 * given a subplan which is done in automata, construct a query graph out of it.
		 * Creation date: 
		 * @return SchemaGraphNode: the query graph node who is one of the children of this node and name is tagName
		 * @param tagName String: 
		 */
	public QueryGraphNode findChild(String tagName) {
		GraphNode[] children = getChildren();
		int childrenNum = children.length;

		for (int i = 0; i < childrenNum; i++) {
			QueryGraphNode child = (QueryGraphNode) children[i];
			if (((QueryGraphNode) child).getTagName().equals(tagName))
				return child;
		}

		return null;
	}

	public QueryGraphNode cloneSelfAndChildren() {
		QueryGraphNode newQgn = (QueryGraphNode) this.clone();
		newQgn.children = new Vector();
		for (Iterator it = children.iterator(); it.hasNext();) {
			QueryGraphNode child = (QueryGraphNode) it.next();
			QueryGraphNode newChild = (QueryGraphNode) child.clone();
			newQgn.children.add(newChild);
		}
		return newQgn;
	}

	/**
			* find which nodes pointing to negative edges to THIS node
			* @return ArrayList a list of query graph nodes			* 
			*/
	public ArrayList getReverseNegativeNodes() {
		ArrayList reverseNodes = new ArrayList();
		QueryGraphNode parent = (QueryGraphNode) getParent();
		Vector children = parent.children;
		for (Iterator it = children.iterator(); it.hasNext();) {
			QueryGraphNode child = (QueryGraphNode) it.next();
			if (child.negativeNodes.contains(this))
				reverseNodes.add(this);
		}

		return reverseNodes;
	}

	/**
				* find the path from this node's parent + itself all the way down to non-filtering pattern
				* we however do not record this node's parent
				* @return a tree rooted at a node		* 
				*/
	QueryGraphNode getFilteringTreeRoot() {
		//none filtering pattern
		if (this.getType() == QueryGraphNode.FILTER)
			return null;

		QueryGraphNode curNode = this;
		QueryGraphNode newNode = (QueryGraphNode) curNode.clone();
		newNode.removeAllChildren();

		ArrayList filteringChildren = curNode.getFilteringChildren();
		for (Iterator it = filteringChildren.iterator(); it.hasNext();) {
			QueryGraphNode filteringChild = (QueryGraphNode) it.next();
			QueryGraphNode clonedFilteringNode =
				(QueryGraphNode) filteringChild.clone();
			clonedFilteringNode.addChild(filteringChild.getFilteringTreeRoot());
		}

		return newNode;

	}

	/**
					* get all the children that are of filtering type
					* 
					* @return ArrayList a list of fitlering type children
					*  
					*/

	ArrayList getFilteringChildren() {
		ArrayList filteringChildren = new ArrayList();
		for (Iterator it = children.iterator(); it.hasNext();) {
			QueryGraphNode child = (QueryGraphNode) it.next();
			if (child.getType() == FILTER)
				filteringChildren.add(child);
		}

		return filteringChildren;
	}

	/**
						* whether tree 1 and tree2 are mergeable
						* i.e., whether their trees are contained
						* if yes, then all answers to the tree that is bigger must be answers to the tree that is smaller
						* revisit: not correct, only take care of a tree of depth 2
						* 
						* @return boolean
						*  
						*/

	boolean isMergeable(QueryGraphNode root2) {
		if (root2 == null)
			//root2 does not have filter pattern, must be contained in this node
			return true;

		if (!getTagName().equals(root2.getTagName()))
			return false;

		GraphNode[] root1Children = getChildren();
		GraphNode[] root2Children = root2.getChildren();
		int root1ChildrenNum = root1Children.length;
		int root2ChildrenNum = root2Children.length;

		for (int i = 0; i < root1ChildrenNum; i++) {
			boolean found = false;
			String root1ChildName =
				((QueryGraphNode) root1Children[i]).getTagName();

			ArrayList sameNameRoot2Children = new ArrayList();
			//whether exist a root2'child that has exactly the same name
			for (int j = 0; j < root2ChildrenNum; j++) {
				String root2Child =
					((QueryGraphNode) root2Children[j]).getTagName();
				if (root1ChildName.equals(root2Child)) {
					found = true;
					break;
				}

			}

			if (!found)
				return false;
		} //end of for (int i

		return true;
	}

	/**
							* merge qgn into this node 
							* i.e., keep this node, remove qgn
							* they are must be of the same type
							* @return boolean
							*  
							*/

	void merge(QueryGraphNode qgn) {
		ArrayList curChildrenName = new ArrayList();
		for (int i = 0; i < getChildrenSize(); i++)
			curChildrenName.add(((QueryGraphNode) getChild(i)).getTagName());

		//if the other node is stricter than this node, make this node stricter 
		if (getType() != FILTER && qgn.getType() == FILTER)
			setType(FILTER);

		if (!isReturned() && qgn.isReturned())
			setIsReturned(true);

		//merge opsOnEntry and opsOnDest
		opsOnEntry.addAll(qgn.getOpsOnEntry());
		opsOnDest.addAll(qgn.getOpsOnDest());

		//merge their children
		for (int i = 0; i < getChildrenSize(); i++) {
			QueryGraphNode curChild = (QueryGraphNode) getChild(i);
			for (int j = 1; j < qgn.getChildrenSize(); j++) {
				QueryGraphNode toMergeChild = (QueryGraphNode) qgn.getChild(j);
				if (!curChildrenName.contains(toMergeChild.getTagName())) {
					curChildrenName.add(toMergeChild.getTagName());
					this.addChild(toMergeChild);
					//since it has been merged, delete it from its parent
					toMergeChild.getParent().deleteChild(toMergeChild);
				} else
					curChild.merge(toMergeChild);
			} //end of for int j
		} //end of for int i
	}

	/*
		 * find trace states of this query tree node which are derived from ancestorType
		 * which must be a subset of tracestates
		 */
	public List getTraceStates(TraceState ancestorState) {
		List traceStatesDerived =
			(List) ancestorStateToTraceStates.get(ancestorState);
		if (traceStatesDerived != null)
			return traceStates;

		//not set up yet
		traceStatesDerived = ancestorState.getDescStates(this);
		ancestorStateToTraceStates.put(ancestorState, traceStatesDerived);

		return traceStatesDerived;
	}

	public List getTraceStates() {
		return traceStates;
	}

	public void addTraceState(TraceState traceState) {
		traceStates.add(traceState);
	}

	public void clearTraceStates() {
		traceStates.clear();
	}

	public void deleteTraceState(TraceState traceState) {
		traceStates.remove(traceState);
		TraceState parentTraceState = traceState.getParentTraceState();
		if (parentTraceState != null)
			parentTraceState.removeChildTraceState(traceState);
	}

	/*
	 * given child, which position it is in THIS node
	 */
	public int getChildPosition(QueryGraphNode child) {
		return children.indexOf(child);
	}

	public int getFilterChildPosition(QueryGraphNode child) {
		int filterChildCount = 0;
		int childrenNum = getChildrenSize();
		for (int i = 0; i < childrenNum; i++) {
			QueryGraphNode curChild = (QueryGraphNode) getChild(i);
			if (curChild == child)
				break;
			if (curChild.getType() == QueryGraphNode.FILTER)
				filterChildCount++;
		}

		return filterChildCount;
	}
	/*
	 * a child occurs in state (which is a type of this node) n times
	 */
	public void updateTraceStateOccurNum(int n, TraceState state) {
		Integer sumSoFar = (Integer) traceStateToOccurNum.get(state);
		if (sumSoFar == null) {
			sumSoFar = new Integer(n);
		} else {
			sumSoFar = new Integer(sumSoFar.intValue() + n);
		}

		traceStateToOccurNum.put(state, sumSoFar);
	}

	/*
	 * one child trace state's occurrence within this tracestate is definite
	 * record the number 
	 */
	public void updateTraceStateToResolvedChildrenNum(TraceState state) {
		Integer countSoFar =
			(Integer) traceStateToResolvedChildrenNum.get(state);
		if (countSoFar == null) {
			countSoFar = new Integer(1);
		} else {
			countSoFar = new Integer(countSoFar.intValue() + 1);
		}

		traceStateToResolvedChildrenNum.put(state, countSoFar);
	}

	public int getResolvedChildrenNum(TraceState state) {
		return ((Integer) traceStateToResolvedChildrenNum.get(state))
			.intValue();
	}

	/*
	 * whether the node has parallel computations
	 * (0) is an entry node
	 * (1) has multiple chidren or
	 * (2) has only one child but is to be returned
	 */
	public boolean isFilteringSensitive() {

		boolean entryNode = false;

		if (parents.size() == 0)
			//it is a root
			entryNode = true;
		else
			entryNode = isNonSelDest;

		if (entryNode) {

			if (children.size() > 1) {
				//check whether any child is filtering
				for (Iterator it = children.iterator(); it.hasNext();) {
					QueryGraphNode child = (QueryGraphNode) it.next();
					if (child.type == FILTER)
						return true;
				}
			} else if (children.size() == 1) {
				if (isReturned)
					return true;
			}

		}

		return false;

		/*		if (parents.size() == 0)
						//it is a root
						return true;
			
				if (parents.size() == 0)
					//root
					return true;
				else
				  
					return isNonSelDest;*/

	}

	/* THIS must not be an entry
	 * find the dest qgn,
	 * on the path from the entry qgn of THIS node to the dest qgn
	 */
	public QueryGraphNode getDestQgn() {
		//indicate as a dest or have no children
		if (isNonSelDest || children.size() == 0)
         //  	return null;
          return this;
		

		//THIS node itself is not a destination
		QueryGraphNode onlyChild = (QueryGraphNode) getChild(0);
		//can have only one child
		while (!onlyChild.isNonSelDest && onlyChild.children.size() > 0)
			onlyChild = (QueryGraphNode) onlyChild.getChild(0);

		return onlyChild;
	}

	/**
	 * @return get the entry node of this qgn
	 */
	public QueryGraphNode getEntryQgn() {
		//if (children.size() == 0)
	    //		return null;

		QueryGraphNode parent = (QueryGraphNode) getParent();
		//THIS node is the root
		if (parent == null)
			return this;

		while (!parent.isNonSelDest) {

			if (parent.getParent() != null)
				parent = (QueryGraphNode) parent.getParent();
			else
				//parent might be the root
				return parent;
		}

		return parent;
	}

	/*
	 * find the automata states for this qgn
	 * note the automata state is the one whose transition table includes the symbol of qgn's tagname
	 * in our current implementation, there actually is only one auto state for one qgn
	 */
	public List findStatesForQgn() {
		List states = new ArrayList();

		QueryGraphNode destQgn = getDestQgn();
		List opsOnDest = destQgn.getOpsOnDest();
		for (Iterator it = opsOnDest.iterator(); it.hasNext();) {
			PureNavigateAutoStream nav = (PureNavigateAutoStream) it.next();
			ExtendedState destAutoState = nav.getOpState();
			//trace back to THIS qgn

			QueryGraphNode curQgn = destQgn;
			//a
			//|
			//b
			//0->(a)1->(b)->2
			//b's state is pointed to 2
			//we however need to get the state 0 rather than 1
			ExtendedState curState = destAutoState.backward(1);
			while (curQgn != this) {
				curState = curState.backward(1);
				curQgn = (QueryGraphNode) curQgn.getParent();
			}

			states.add(curState);

		}

		return states;
	}

	public String toString() {
		String output = new String("");
		output = output + tagName + traceStates;

		if (isReturned)
			output = output + " return ";
		if (isNonSelDest)
			output = output + " dest ";
		else if (isSel)
			output = output + " sel ";

		if (type == NONFILTER)
			output = output + "(NONFILTER)";
		else if (type == FILTER)
			output = output + "(FILTER)";

		/*	int entrySize = opsOnEntry.size();
			if (entrySize > 0) {
				output = output + "entry ops: ";
				for (int i = 0; i < entrySize; i++) {
					if (i != entrySize - 1)
						output =
							output + (XATStreamOperator) opsOnEntry.get(i) + "| ";
					else
						output = output + (XATStreamOperator) opsOnEntry.get(i);
				}
			}*/

		int destSize = opsOnDest.size();
		if (destSize > 0) {
			output = output + "dest ops: ";
			for (int i = 0; i < destSize; i++) {
				if (i != destSize - 1)
					output =
						output + (XATStreamOperator) opsOnDest.get(i) + "| ";
				else
					output =
						output + (XATStreamOperator) opsOnDest.get(i) + " ";
			}
		}

		for (Iterator it = negativeNodes.iterator(); it.hasNext();) {
			QueryGraphNode pointTo = (QueryGraphNode) it.next();
			output = output + ": - " + pointTo.getTagName();
		}

		return output;
	}
	
	/*
	 * this node is an entry node
	 */
	public List getAllDestQgns(){
		List allDestQgns = new ArrayList();
		for (int i=0; i<children.size();i++)
			allDestQgns.add(((QueryGraphNode)children.get(i)).getDestQgn());
		
		return allDestQgns;
	}

}
