/*
 * 
 * infer types for those non-deterministic nodes, namely, 
 * "//" and "*" 
 * We use the approach in "optimizing regflar path expressions using graph schemas"
 * on ICDE98 by Fernandez and Suciu
 * both query pattern and schema	 
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import edu
	.wpi
	.cs
	.dsrg
	.xmldb
	.xat
	.common
	.operator
	.streamoperator
	.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedAutomata;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.State;

/**
 * @author suhong
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class QueryNodeTypeInference {
	//given a query graph node, find the co-occurrence of its children
	//can be a set of arrays
	//as long as one of arrays is all full, then the co-occurrence constraint is satisfied
	//each element in the array indicate the schema state which the child is bound to
	static HashMap nodeToCocurArray = new HashMap();

	//given a state, find all the other states it can conflict with
	private static HashMap conflictLookup = new HashMap();

	private static SchemaGraph sg;

	static int maximalConflictNum = 10;

	static public List typeInference(QueryGraph qg, SchemaGraph schemaGraph) {
		sg = schemaGraph;
		QueryGraphNode rootQgn = (QueryGraphNode) qg.getTopEntries().get(0);
		SchemaGraphNode rootSgn = null;

		String rootQgnName = rootQgn.getTagName();
		if (!rootQgnName.equals(QueryGraphNode.DESC)
			&& !rootQgnName.equals(QueryGraphNode.WILDCARD)) {
			rootSgn = sg.getNodeForName(rootQgn.getTagName());
			TraceState rootTraceState = new TraceState(rootSgn, rootQgn, null);
			rootQgn.addTraceState(rootTraceState);
		} else {
			if (rootQgnName.equals(QueryGraphNode.DESC))
				setUpChildStatesForDesc(rootQgn, null, null);
			else //must be wildcard
				setUpChildStatesForWildCard(rootQgn, null, null);
		}

		List matchings = typeInference(rootQgn, qg, sg);

		//reflect the change to the automata
		for (Iterator it = matchings.iterator(); it.hasNext();) {
			Object[] matching = (Object[]) it.next();
			translateBackToPlan(matching);
		}

		/*String output = "type inference: \n";
		for (Iterator it = matchings.iterator(); it.hasNext();) {
			Object[] matching = (Object[]) it.next();
			for (int i = 0; i < matching.length; i++)
				output = output + matching[i] + " ";
			output = output + "\n";
		}
		
		System.out.println(output);*/
		return matchings;
	}

	/*
	 * a list of matching
	 * which has to be reflected into the automata
	 */
	static private List typeInference(
		QueryGraphNode curQgn,
		QueryGraph qg,
		SchemaGraph sg) {
		
		List matchings = new ArrayList();

		//initialize for checking whether all children can co-exist	
		initializeAllMatched(curQgn);

		String curQgnName = curQgn.getTagName();

		List generalIncludedStates = curQgn.getTraceStates();

		List generalFreeStates = new ArrayList();

		for (Iterator it = generalIncludedStates.iterator(); it.hasNext();) {
			TraceState generalIncludedState = (TraceState) it.next();
			if (generalIncludedState.isUnionTraceState()) {
				for (Iterator it2 = generalIncludedState.getUnionTraceState();
					it2.hasNext();
					)
					generalFreeStates.add(it2.next());
			} else
				generalFreeStates.add(generalIncludedState);
		}

		GraphNode[] curQgnChildren = curQgn.getChildren();
		int childrenSize = curQgnChildren.length;

		//begin construct the pruned automaton  
		for (int i = 0; i < childrenSize; i++) {
			QueryGraphNode qgnChild = (QueryGraphNode) curQgnChildren[i];
			String childName = qgnChild.getTagName();

			//if this child is a wildcard
			if (childName.equals(QueryGraphNode.WILDCARD)) {

				for (Iterator it = generalFreeStates.iterator();
					it.hasNext();
					) {
					TraceState parentTraceState = (TraceState) it.next();
					SchemaGraphNode parentSgn = parentTraceState.getSgn();

					setUpChildStatesForWildCard(
						qgnChild,
						parentTraceState,
						parentSgn);
				}

			}

			if (childName.equals(QueryGraphNode.DESC)) {

				for (Iterator it = generalFreeStates.iterator();
					it.hasNext();
					) {
					TraceState parentTraceState = (TraceState) it.next();
					SchemaGraphNode parentSgn = parentTraceState.getSgn();

					setUpChildStatesForDesc(
						qgnChild,
						parentTraceState,
						parentSgn);
				}
			}

			if (!childName.equals(QueryGraphNode.WILDCARD)
				&& !childName.equals(QueryGraphNode.DESC)
				&& !childName.equals(QueryGraphNode.EMPTY)) {
				List deletedCurTraceStates = new ArrayList();
				//if this child has a concrete name, 
				//its type has been inferred, check whether it conflicts with parent's type
				//is it allowed in the parent's type
				for (Iterator it = generalFreeStates.iterator();
					it.hasNext();
					) {
					TraceState parentTraceState = (TraceState) it.next();
					SchemaGraphNode parentSgn = parentTraceState.getSgn();

					setUpChildStatesForConcrete(
						qgnChild,
						parentTraceState,
						parentSgn,
						childName);
				} //end of for

				if (qgnChild.getTraceStates().size() == 0) {
					//no binding for this type
					nodeEmpty(qg, qgnChild, sg);
					break;
				}

			} //end of if

			//the above snippet only ensures for query
			//   a
			//  /\
			//  b c
			//that b and c can occur in a respectively, 
			//but it does not ensure b and c can both occur
			//like given the schema
			//  a
			//  |
			// choice
			// / \
			// b  c
			//the query won't have any answer
			//now we start checking co-occurrence
			if (qgnChild.getType() == QueryGraphNode.FILTER)
				checkCoOccur(curQgn, qgnChild, qg, sg);

			if (i == childrenSize - 1) {
				//the last one, wrap up,

				//each element in the array 
				List validCombins = new ArrayList();
				if (!isCoOccurAllSatisfied(curQgn, validCombins)) {
					//if not children of curQgn cooccur, do not continue
					nodeEmpty(qg, curQgn, sg);
					break;
				}

				//if there exist valid combinations
				for (Iterator it = validCombins.iterator(); it.hasNext();) {
					TraceState[] validCombin = (TraceState[]) it.next();
					for (int k = 0; k < validCombin.length; k++) {
						TraceState stateForChildk = validCombin[k];
						List childStates =
							((QueryGraphNode) curQgn.getChild(k))
								.getTraceStates();

						for (Iterator it2 = childStates.iterator();
							it2.hasNext();
							) {
							TraceState curChildState = (TraceState) it2.next();
							TraceState generalState =
								curChildState.getGeneralState();
					//revisit: comment out for a case, not sure what the statement was originally for		
							
						//	if (generalState != stateForChildk)
						//		continue;

							List curChildStatesChoice =
								curChildState.thruChoiceStates();
							if (curChildStatesChoice.size() > 0) {
								//this node lies under some choice
								//set up its exclusive states
								List exclusiveSgns = new ArrayList();
								//sgns that surely occur under the choice node
								List sureExclusiveSgns = new ArrayList();

								SchemaGraphNode curGeneralNode =
									generalState.getSgn();
								for (Iterator it3 =
									curChildStatesChoice.iterator();
									it3.hasNext();
									) {
									TraceState choiceState =
										(TraceState) it3.next();
									SchemaGraphNode choiceNode =
										choiceState.getSgn();
									GraphNode[] childrenOfChoice =
										choiceNode.getChildren();
									for (int l = 0;
										l < childrenOfChoice.length;
										l++) {
										if (childrenOfChoice[l]
											!= curGeneralNode) {
											//the other branches of the choice rather than the child
											exclusiveSgns.addAll(
												(
													(SchemaGraphNode) childrenOfChoice[l])
													.getConcreteDesc());

											if (
												(
													(SchemaGraphNode) childrenOfChoice[l])
												.getOccurWithParent(choiceNode)
												.getMinOccurs()
												> 0) {
												for (Iterator it4 =
													(
														(SchemaGraphNode) childrenOfChoice[l])
														.getConcreteDesc()
														.iterator();
													it4.hasNext();
													) {
													SchemaGraphNode concreteDesc =
														(SchemaGraphNode) it4
															.next();
													if (choiceNode
														.getMinOccurrence(
															concreteDesc
																.tagName)
														> 0)
														sureExclusiveSgns.add(
															concreteDesc);
												}
											}

										}
									}
									curGeneralNode =
										choiceState.getGeneralState().getSgn();
								}

								//set up exclusiveSgn of curChild
								//1. find its starting state
								//2. find whether no occurence linking nodes on the path from starting
								//state to destination node is multiple
								//e.g., <!element a (b*)> <!element b(c|d)>
								//if a is the entry, c and d are exclusive
								TraceState entryState =
									traceEntryForExclusive(curChildState);
								if (entryState != null) {
									curChildState.setExclusiveSgns(
										exclusiveSgns,
										entryState);
									//within the entry state, the sureexclusiveSgns are sure ending marks of curChildState	
									curChildState.setSureExclusiveSgns(
										sureExclusiveSgns,
										entryState);
								}

							} //end of if (curChildStatesChoice.size()>0) 

						} //end for (Iterator it2 = childStates.iterator();
					} //end of for (int k = 0; k < validCombin.length; k++) 
				} //end of 	for (Iterator it = validCombins.iterator(); it.hasNext();)

			} //end of if i=childrenSize - 1

			//the qgnChild so far has a valid type relative to its parent
			//we go on to see whether it is valid in terms of its children relative to it

			matchings.addAll(typeInference(qgnChild, qg, sg));

		} //end of for checking each child for (int i = 0; i < childrenSize; i++) 

		//		check whether the deletion of curTraceState can lead to more delete of its parent trace state
		QueryGraphNode propagatingNode = curQgn;
		//	(QueryGraphNode) curQgn.getParent();

		while (propagatingNode != null) {

			List propagatingNodeStates = propagatingNode.getTraceStates();
			boolean goon = false;
			if (propagatingNodeStates.size() == 0) {
				//no types can be ever bound to this type, an empty node
				nodeEmpty(qg, propagatingNode, sg);
				return matchings;
			} else {
				//clean any type which does not have an successor
				//this type leads to no where
				ArrayList deletedPropagatingStates = new ArrayList();
				for (Iterator it2 = propagatingNodeStates.iterator();
					it2.hasNext();
					) {
					TraceState propagatingNodeState = (TraceState) it2.next();
					if (propagatingNodeState.nonEssentialDesc()) {
						//the state does not have any desc states with concrete names
						QueryGraphNode propQgn = propagatingNodeState.getQgn();
						if (propQgn == null
							|| (propQgn != null && propQgn.getChildrenSize() > 0))
							//and it is not a leave state, should be removed
							deletedPropagatingStates.add(propagatingNodeState);
						goon = true;
					}
				}

				for (Iterator it3 = deletedPropagatingStates.iterator();
					it3.hasNext();
					)
					propagatingNode.deleteTraceState((TraceState) it3.next());

				//the state of propagating node is not changed
				if (!goon)
					break;
				//propagate it up
				propagatingNode = (QueryGraphNode) propagatingNode.getParent();
				//check any type is no longer allowed?
			} //end of else
		} //end of while

		//postprocessing:
		//1. remove unnecessary 

		generalFreeStates = curQgn.getTraceStates();
		//find for those node whose have multiple children
		if (!curQgnName.equals(QueryGraphNode.EMPTY)) {

			if (curQgnName.equals(QueryGraphNode.WILDCARD)) {
				Object[] matching = new Object[1];
				matching[0] = curQgn;
				matchings.add(matching);

				/*if (generalFreeStates.size() == 1) {
					//make this nondeterministic one to a deterministic one
					Object[] curStatesCopy = generalFreeStates.toArray();
					curQgn.setTagName(
						((TraceState) curStatesCopy[0]).getSgn().getTagName());
				
				}*/

			}

			if (curQgnName.equals(QueryGraphNode.DESC)) {
				Object[] matching = new Object[3];
				matching[0] = curQgn;
				matchings.add(matching);

				for (Iterator it = generalFreeStates.iterator();
					it.hasNext();
					) {
					TraceState curType = (TraceState) it.next();
					//its parent cannot again be a desc, otherwise merged
					//can be either deterministic one or a *
					QueryGraphNode parentQgn =
						(QueryGraphNode) curQgn.getParent();
					List parentStates = parentQgn.getTraceStates();
					for (Iterator it2 = parentStates.iterator();
						it2.hasNext();
						) {
						TraceState parentState = (TraceState) it2.next();
						SchemaGraphNode parentType = parentState.getSgn();
						if (parentType.equals(curType.getSgn())) {
							//parent and this node are of the same type
							//if this type does not allow recursive 
							//then curQgn's type means zero navigation step, this type can be removed
							if (parentType
								.getMinOccurrence(parentType.getTagName())
								== SchemaGraphNode.NONEXIST) {
								if (generalFreeStates.size() == 1) {
									//this // is superfluous
									for (int i = 0; i < childrenSize; i++) {
										QueryGraphNode child =
											(QueryGraphNode) curQgnChildren[i];
										parentQgn.addChild(child);
										child.addParent(parentQgn);
										child.removeParent(curQgn);
									}
									parentQgn.deleteChild(curQgn);
								} else
									//remove this type 
									curQgn.deleteTraceState(curType);
							}
						} //end of if (parentType.getTagName().equals(curQgnName))

						//if there is still one valid type remaining
						//make the nondeterministic // to a deterministic navigation step
						if (generalFreeStates.size() == 1) {
							//make this nondeterministic one to a deterministic one
							Object[] curStatesCopy =
								generalFreeStates.toArray();
							curQgn.setTagName(
								((TraceState) curStatesCopy[0])
									.getSgn()
									.getTagName());
						}

					} //end of for

				} //end of for (Iterator it = curStates.iterator(); it.hasNext();)
			} //end of if (curQgnName.equals(QueryGraphNode.DESC)) 

		} //end of if (!curQgnName.equals(QueryGraphNode.EMPTY))
		else {
			//curQgnName.equals(QueryGraphNode.EMPTY)
			Object[] matching = new Object[1];
			matching[0] = curQgn;
			matchings.add(matching);
		}

		return matchings;
	}

	/*
	 * this node's type is null, it won't have any valid bindings
	 */
	static void nodeEmpty(QueryGraph qg, QueryGraphNode qgn, SchemaGraph sg) {

		if (qgn.getType() == QueryGraphNode.NONFILTER) {
			//QueryGraphNode replaceSelf =
			//	new QueryGraphNode(QueryGraphNode.EMPTY);
			//replaceSelf.setType(QueryGraphNode.NONFILTER);

			//if (nonExistChild.isReturned())
			//	replaceSelf.setReturned(true);

			//qgn.getParent().deleteChild(qgn);
			//parent.addChild(replaceSelf);
			//replaceSelf.setParent(parent);
			qgn.setTagName(QueryGraphNode.EMPTY);

		} else {
			//must be filter type
			//propagate it up
			while (true) {
				QueryGraphNode parent = (QueryGraphNode) qgn.getParent();
				if (parent == null) {
					//already hit the root
					//replace the root with a special empty node, indicating no answer
					qgn.setTagName(QueryGraphNode.EMPTY);
					qgn.setType(QueryGraphNode.FILTER);
					qgn.removeAllChildren();

					//reflect back to plan
					List destOps = qgn.getOpsOnDest();
					//revisit, remove all these ops from plan
					//QueryGraphBackToPlan.addToToBeRemovedOps(destOps);

					qg.addTopEntry(qgn);

					break;
				}
				QueryGraphNode grandParent =
					(QueryGraphNode) parent.getParent();
				if (parent.getType() == QueryGraphNode.NONFILTER) {
					//progate ending
					//QueryGraphNode replaceParent =
					//	new QueryGraphNode(QueryGraphNode.EMPTY);
					//replaceParent.setType(QueryGraphNode.NONFILTER);

					grandParent.deleteChild(parent);
					//grandParent.addChild(replaceParent);
					//replaceParent.setParent(grandParent);							

					//reflect back to plan
					List destOps = parent.getOpsOnDest();
					//revisit
					//QueryGraphBackToPlan.addToToBeRemovedOps(destOps);

					break;
				}
				//must fiter

				qgn = parent;
			}
		} //end of else

	}

	/*
	 * @param childState child state which may have exclusive types
	 * trace its starting state, and there must be no occurrence that has a maximal occurrence > 1
	 */
	static TraceState traceEntryForExclusive(TraceState childState) {
		TraceState parentState = childState.getParentTraceState();
		QueryGraphNode parentQgn = parentState.getQgn();
		SchemaGraphNode parentSgn = parentState.getSgn();

		SchemaGraphNode childSgn = childState.getSgn();
		while ((parentQgn != null && !parentQgn.isNonSelDest())
			|| parentQgn == null) {
			//not an entry node yet
			//a choice state's qgn may not have any qgn associated
			Occurrence childInParent = childSgn.getOccurWithParent(parentSgn);

			int maxOccur = childInParent.getMaxOccurs();
			//maxOccur cannot allow multiple occurrence
			if (maxOccur > 1)
				return null;

			childSgn = parentSgn;
			parentState = parentState.getParentTraceState();
			parentSgn = parentState.getSgn();
			parentQgn = parentState.getQgn();

		}

		return parentState;
	}

	/*
	 * check whether qgnChild can co-exist the other children that are checked so far
	 * @param curQgn QueryGraphNode current qgn
	 * @parem qgnChild QueryGraphNode child qgn of current qgn
	 * @param qg QueryGraph query graph
	 * @param sg SchemaGraph schema graph
	 */
	static boolean checkCoOccur(
		QueryGraphNode curQgn,
		QueryGraphNode qgnChild,
		QueryGraph qg,
		SchemaGraph sg) {

		//note, the position is the among filter children rather than all children
		int qgnChildPosition = curQgn.getFilterChildPosition(qgnChild);

		//an array of arrays
		List allMatched = (ArrayList) nodeToCocurArray.get(curQgn);
		//this is called when a new child's type is fixed
		//so add in this satisfied child's type to the existing one
		List newMatcheds = new ArrayList();

		for (Iterator it = allMatched.iterator(); it.hasNext();) {
			TraceState[] matched = (TraceState[]) it.next();
			List typesOfChild = qgnChild.getTraceStates();

			for (Iterator it2 = typesOfChild.iterator(); it2.hasNext();) {
				TraceState childState = ((TraceState) it2.next());
				//before setting the bit represent this child is ready
				//we find whether this child's type conflict with anyone
				List conflictCausedByNew = conflictWith(matched, childState);

				if (conflictCausedByNew.size() > 0) {
					//new state conflict with exsiting one
					//create new matched for the new state
					TraceState[] newMatched = new TraceState[matched.length];
					for (int i = 0; i < newMatched.length; i++) {
						if (matched[i] != null)
							newMatched[i] = (TraceState) matched[i].clone();
					}
					for (Iterator it3 = conflictCausedByNew.iterator();
						it3.hasNext();
						) {
						Integer position = (Integer) it3.next();
						newMatched[position.intValue()] = null;
					}
					newMatched[qgnChildPosition] = childState;
					newMatcheds.add(newMatched);
				} else {
					//no conflict					
					TraceState boundTypeOnChild =
						(TraceState) matched[qgnChildPosition];

					if (boundTypeOnChild == null) {
						//no type of this child has ever been set yet	
						matched[qgnChildPosition] = childState;
					} else {
						//there is some type of this child set already
						//create new valid combination
						TraceState[] newMatched =
							new TraceState[matched.length];
						for (int i = 0; i < newMatched.length; i++) {
							if (matched[i] != null)
								newMatched[i] = (TraceState) matched[i].clone();
						}
						newMatched[qgnChildPosition] = childState;
						newMatcheds.add(newMatched);

					}

					/*if (boundTypeOnChild == null) {
						TraceState generalState = childState.getGeneralState();
						generalState.addUnionTraceState(childState);
						generalState.setUnionTraceState(true);
						matched[qgnChildPosition] =
							childState.getGeneralState();
					} else {
						//there've already something there, must be a general state
						//if the child trace states shares the same parent trace state
						//they co-exist and represented by the same general state
						//otherwise use another general state to represent the child state
						
						if (boundTypeOnChild.isUnionTraceState()) {
							boundTypeOnChild.addUnionTraceState(childState);
						} else {
							System.out.println("wrong: not a general state");
						}
					}*/

				}
			}

		}

		if (newMatcheds.size() > 0)
			allMatched.addAll(newMatcheds);

		return true;

	}

	private static void initializeAllMatched(QueryGraphNode curQgn) {
		GraphNode[] children = curQgn.getChildren();

		int filterChildCount = 0;
		for (int i = 0; i < children.length; i++) {
			QueryGraphNode child = (QueryGraphNode) children[i];
			if (child.getType() == QueryGraphNode.FILTER)
				filterChildCount++;
		}

		TraceState[] matched = new TraceState[filterChildCount];
		for (int i = 0; i < filterChildCount; i++)
			matched[i] = null;

		List allMatched = new ArrayList();
		allMatched.add(matched);
		nodeToCocurArray.put(curQgn, allMatched);

	}

	/*
	 * given a matched array in which each non-null element indicate a state matched for certain child
	 * return the array in which each integer represents 
	 */
	private static List conflictWith(
		TraceState[] matched,
		TraceState childState) {
		List result = new ArrayList();

		for (int i = 0; i < matched.length; i++) {
			TraceState alreadyMatched = matched[i];
			if (alreadyMatched != null)
				if (conflict(alreadyMatched, childState)) {
					result.add(new Integer(i));
				}
		}
		return result;
	}

	/*
	 * whether state1 and state2 conflict
	 * if they share the same choice ancestor, then they cannot
	 */
	private static boolean conflict(TraceState state1, TraceState state2) {
		List choice1 = state1.thruChoiceStates();
		List choice2 = state2.thruChoiceStates();

		boolean share = false;
		//if there is any intersection between choice1 and choice2
		//they share the same choice ancestor, 
		TraceState choiceState1 = null;
		for (Iterator it1 = choice1.iterator(); it1.hasNext();) {
			choiceState1 = (TraceState) it1.next();
			for (Iterator it2 = choice2.iterator(); it2.hasNext();) {
				TraceState choiceState2 = (TraceState) it2.next();
				if (choiceState1.getSgn() == choiceState2.getSgn()) {
					share = true;
					break;
				}
			}
			if (share)
				break;

		}

		if (share) {
			//whether this choice has any ancestor whose occurrence can be multiple
			TraceState curState = choiceState1;
			TraceState anceState = choiceState1.getParentTraceState();
			boolean found = false;
			while (anceState != null) {
				if (curState
					.getSgn()
					.getOccurWithParent(anceState.getSgn())
					.getMaxOccurs()
					> 1)
					found = true;
				curState = anceState;
				anceState = curState.getParentTraceState();
			}

			if (!found) {

				//check whether belong to different branch of this choice
				TraceState generalState1 = state1.getGeneralState();
				TraceState generalState2 = state2.getGeneralState();
				if (generalState1 == generalState2)
					//same branch
					return false;
				else {
					//for special nodes, they are the same while with different id
					if (!generalState1.getSgn().isConcrete()
						&& !generalState2.getSgn().isConcrete()) {
						if (generalState1.getSgn() == generalState2.getSgn())
							return false;
					}
					return true;
				}
			}
		}

		return false;
	}

	/*
	 * whether exist a combination, that all children can coexist
	 * this is called when all chilren of curQgn have been checked
	 * @param curQgn QueryGraphNode the parent qgn whose children are check
	 * @param validCombins
	 * @return boolean 
	 */
	static private boolean isCoOccurAllSatisfied(
		QueryGraphNode curQgn,
		List validCombins) {
		List matchedSoFar = (List) nodeToCocurArray.get(curQgn);
		boolean existCoOccur = false;

		for (Iterator it = matchedSoFar.iterator(); it.hasNext();) {
			TraceState[] oneMatched = (TraceState[]) it.next();
			boolean qualified = true;
			for (int i = 0; i < oneMatched.length; i++) {
				if (oneMatched[i] == null) {
					//there exist one child that does not have match
					//this match is disqualified
					qualified = false;
					break;
				}
			}
			if (qualified) {
				existCoOccur = true;
				//allow this bound to be kept
				validCombins.add(oneMatched);
			}
		}

		return existCoOccur;
	}

	/*
	 * find the temp bind to the wildcard child of qgn
	 * we say its temp is the bind may find not leading to a final state
	 * wildCardChild is a child of qgn which is a wildcard
	 */
	private static void setUpChildStatesForWildCard(
		QueryGraphNode wildCardChild,
		TraceState parentTraceState,
		SchemaGraphNode sgn) {

		GraphNode[] childrenSgn;

		if (sgn == null) {
			//get all the root element type
			childrenSgn = new GraphNode[sg.getTopEntries().size()];
			int i = 0;
			for (Iterator it = sg.getTopEntries().iterator(); it.hasNext();)
				childrenSgn[i] = (GraphNode) it.next();
			i++;
		} else
			childrenSgn = sgn.getChildren();

		int childrenSgnNum = childrenSgn.length;

		for (int j = 0; j < childrenSgnNum; j++) {
			SchemaGraphNode childSgn = (SchemaGraphNode) childrenSgn[j];

			if (childSgn.isConcrete()) {
				TraceState newState =
					new TraceState(childSgn, wildCardChild, parentTraceState);
				//associate the child with childSgn's type
				wildCardChild.addTraceState(newState);
			}
			//new TraceState(childSgn, wildCardChild, parentTraceState));
			else {
				//create trace state for these intermediate sgn
				//but these states are not associated with any qgn
				//but they are traceable from the other states associated with qgn
				TraceState newState =
					new TraceState(childSgn, parentTraceState);
				setUpChildStatesForWildCard(wildCardChild, newState, childSgn);
			}
		}
	}

	private static void setUpChildStatesForDesc(
		QueryGraphNode descChild,
		TraceState parentTraceState,
		SchemaGraphNode sgn) {
		GraphNode[] childrenSgn = sgn.getChildren();
		int childrenSgnNum = sgn.getChildrenSize();

		//desc allows to leave at the state of qgn which is sgn 
		descChild.addTraceState(
			new TraceState(sgn, descChild, parentTraceState));

		for (int j = 0; j < childrenSgnNum; j++) {
			SchemaGraphNode childSgn = (SchemaGraphNode) childrenSgn[j];

			TraceState newState =
				new TraceState(childSgn, descChild, parentTraceState);
			if (childSgn.isConcrete()) {
				//associate the child with childSgn's type
				//				also, the desc state can stay at itself

				descChild.addTraceState(newState);
				setUpChildStatesForDesc(descChild, newState, childSgn);
			} else {
				//create trace state for these intermediate sgn
				//but these states are not associated with any qgn
				//but they are traceable from the other states associated with qgn
				setUpChildStatesForDesc(descChild, newState, childSgn);
			}
		}
	}

	/*
	 * the returned list is a set of states that turn out to be error state
	 * must go backwards for each such state to remove its ancestor states
	 */
	private static void setUpChildStatesForConcrete(
		QueryGraphNode concreteChild,
		TraceState parentTraceState,
		SchemaGraphNode sgn,
		String childName) {

		GraphNode[] childrenSgn = sgn.getChildren();
		int childrenSgnNum = sgn.getChildrenSize();

		for (int j = 0; j < childrenSgnNum; j++) {
			SchemaGraphNode childSgn = (SchemaGraphNode) childrenSgn[j];

			if (childSgn.isConcrete()) {
				if (childSgn.getTagName().equals(childName)) {
					//associate the child with childSgn's type
					TraceState newState =
						new TraceState(
							childSgn,
							concreteChild,
							parentTraceState);
					concreteChild.addTraceState(newState);
				}

			} else {
				//must be a special schema graph node
				//create trace state for these intermediate sgn
				//but these states are not associated with any qgn
				//but they are traceable from the other states associated with qgn
				//this state is not associated with any query graph node
				TraceState newState =
					new TraceState(childSgn, parentTraceState);

				setUpChildStatesForConcrete(
					concreteChild,
					newState,
					childSgn,
					childName);
			}
		} //end of for	

	}

	static private void translateBackToPlan(Object[] matching) {
		ExtendedAutomata auto = null;
		//mapping a trace state to its old automata state
		List toBeRemovedAutoStates = new ArrayList();

		QueryGraphNode modifiedQgn = (QueryGraphNode) matching[0];
		String originalTransitName = modifiedQgn.getTagName();

		//new states a non-deterministic navigation step is resolved to
		List newStates = modifiedQgn.getTraceStates();
		List autoStatesForParentState = modifiedQgn.findStatesForQgn();

		//print the original auto
		for (Iterator it3 = autoStatesForParentState.iterator();
			it3.hasNext();
			) {
			ExtendedState autoStateForParentState = (ExtendedState) it3.next();

			auto = autoStateForParentState.getAutomata();
			System.out.println(auto);
			break;
		}

		for (Iterator it2 = newStates.iterator(); it2.hasNext();) {
			TraceState newTraceState = (TraceState) it2.next();
			TraceState parentOfNewTraceState =
				newTraceState.getParentTraceState();

			for (Iterator it3 = autoStatesForParentState.iterator();
				it3.hasNext();
				) {
				ExtendedState autoStateForParentState =
					(ExtendedState) it3.next();

				//replace original * with concerete one
				//if the concrete one has existed, we reuse the old state
				//all the states originating from * then have to be merged with
				//the successor states of the already-existing concrete ones

				//whether the concrete one has already existed
				boolean newAutoStateExist = true;

				//if autoStateForParentState has self-transition
				//remove it
				//it will replaced by transitions over new type nodes
				autoStateForParentState.removeSingleState(
					State.Star_Self_Transit);

				ExtendedState nextState =
					(ExtendedState) autoStateForParentState.getSingleTransit(
						originalTransitName);

				auto = autoStateForParentState.getAutomata();

				//remove the old transitions if not needed by others
				int opsGoingThru =
					autoStateForParentState.getNavOpCountOn(
						originalTransitName);

				if (opsGoingThru == 1)
					//not used by others, remove this transition
					toBeRemovedAutoStates.add(nextState);

				String transitName = newTraceState.getSgn().getTagName();

				ExtendedState newAutoState =
					(ExtendedState) autoStateForParentState.getSingleTransit(
						transitName);

				//if newAutoState has self-transit but nextStaet does not
				//cannot use newAutoState
				//vice versa
				if (newAutoState != null
					&& ((newAutoState.hasSelfStarTransit()
						&& !nextState.hasStarTransit())
						|| (!newAutoState.hasSelfStarTransit()
							&& nextState.hasSelfStarTransit())))
					newAutoState = null;

				if (newAutoState == null) {
					//not exist
					newAutoStateExist = false;

					newAutoState = new ExtendedState(auto);

					autoStateForParentState.addTransit(
						transitName,
						newAutoState);
					auto.addIntoStates(newAutoState);

					//also associate this state with appropriate operators
					/*if (modifiedQgn.isNonSelDest() || modifiedQgn.getIsSP()) {
						//modifedQgn is a destination
						List destOps = modifiedQgn.getOpsOnDest();
						for (Iterator it = destOps.iterator(); it.hasNext();)
							newAutoState.addDestNavOp(
								(PureNavigateAutoStream) it.next());
					}*/
				}

				if (nextState == null)
					continue;

				nextState.mergeState(newAutoState);

				//update the transit of newly added state

			} //end of for it3

		} //end of for it2

		//remove the old state before type inference
		for (Iterator it = toBeRemovedAutoStates.iterator(); it.hasNext();) {
			ExtendedState toBeRemovedAutoState = (ExtendedState) it.next();
			ExtendedState parentState = toBeRemovedAutoState.backward(1);
			parentState.removeSingleState(originalTransitName);
			auto.removeState(toBeRemovedAutoState);
			for (Iterator it2 = toBeRemovedAutoState.getDestNavOps().iterator();
				it2.hasNext();
				)
				auto.resetStateHash(
					(XATStreamOperator) it2.next(),
					toBeRemovedAutoState);

			for (Iterator it2 = toBeRemovedAutoState.getExtractOps().iterator();
				it2.hasNext();
				) {
				auto.resetStateHash(
					(XATStreamOperator) it2.next(),
					toBeRemovedAutoState);
			}

		}

		System.out.println(auto);
	}

}
