/*
 * Created on Jan 29, 2004
 *
 * construct a schema graph out of an XML schema
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author hong su
 * 
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SchemaGraphInitiator {

	protected static Document schema = null;

	protected static Set concernedSchema = null;

	/**
	 * given a XML schema file's location, construct a DOM out of it
	 * 
	 * @param schemaFileName
	 *            String: file name of XML schema
	 */
	static private void readSchemaFile(String schemaFileName) {

		//sample xml file name: "file:///xerces-4_1_2/data/personal.xml";
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			schema = builder.parse(schemaFileName);
		} catch (FactoryConfigurationError e) {
			// unable to get a document builder factory
		} catch (ParserConfigurationException e) {
			// parser was unable to be configured
			e.printStackTrace();
		} catch (SAXException e) {
			// parsing error
			e.printStackTrace();
		} catch (IOException e) {
			// i/o error
			e.printStackTrace(); 
		}
	}

	/**
	 * given a XML schema file, construct a schema graph out of it Creation
	 * date:
	 * 
	 * @param schemaFileName
	 *            String: file name of XML schema
	 * @param document
	 *            Document: the DOM representation of the XML schema
	 */

	static public SchemaGraph constructSchemaGraph(
			QueryGraphInitiator qgInitiator, String schemaFileName) {
		readSchemaFile(schemaFileName);

		//find the names of the elements in this schema which are concerned by
		// the query
		concernedSchema = qgInitiator.getConcernedSchema(schemaFileName);

		Element rootEle = schema.getDocumentElement();

		Element parentEle = null;
		SchemaGraphNode parentSgn = null;

		SchemaGraph sg = new SchemaGraph();

		constructNode(sg, rootEle);

		resolveEdges(sg, rootEle);

		SchemaGraphNode root = sg.findRoot();
		//System.out.println("initial: \n" + sg);

//		int a;
		purge(null, root);

		//System.out.println("after purge unrelated elements: \n" + sg);

		List normalizedAlready = new ArrayList();
		normalize(root, normalizedAlready);

		System.out.println("after normalization: \n" + sg);

		return sg;

	}

	/**
	 * given a node in XAT dag, generate corresponding query graph constructs We
	 * use docEle to refer the element node type in the doc We use ele to refer
	 * to the element node in the schema
	 * 
	 * @return QueryGraph
	 * @param qg
	 *            QueryGraph the query graph into which the new node is added to
	 * @param qgParentNode
	 *            QueryGraphNode the query graph node of which the new node is
	 *            inserted as a child
	 * @param planNode
	 *            DagNode the node in the XAT Dag which will be mapped to query
	 *            graph node(s) revisit: simpletype can be considered for
	 *            general schema optimization
	 */

	static protected void constructNode(SchemaGraph sg, Element curEle) {
		String curEleName = curEle.getTagName();

		//this node defines an element type
		if (curEleName.equals("xsd:element")) {
			String docEleName = curEle.getAttribute("name");
			if (docEleName.equals(""))
				//might happen when it is a ref
				//docEleName = curEle.getAttribute("ref");
				return;

			String docEleType = curEle.getAttribute("type");

			SchemaGraphNode sgn = new SchemaGraphNode(docEleName);

			//register the node's type
			//multiple sgns can be of the same docEleType
			sg.addNodesForType(sgn, docEleType);

			//register the node's name
			//one name is only mapped to one node
			sg.setNodeForName(sgn, docEleName);
		} //end of if (curEleName.equals("xsd:element"))

		//recursively process its descendants
		NodeList children = curEle.getChildNodes();

		int childrenNum = children.getLength();

		for (int i = 0; i < childrenNum; i++) {
			if (children.item(i) instanceof Element)
				constructNode(sg, (Element) children.item(i));
		}
	}

	/**
	 * given an DOM node in the XML schema which describes the occurrence
	 * property in the complex type e.g., <xsd:element name="item" minOccurs="0"
	 * maxOccurs="unbounded"> set the occurrence member of the corresponding
	 * schema graph node Creation date:
	 * 
	 * @param ele
	 *            Element: the dom node in the XML schema
	 * @param sgn
	 *            SchemaGraphNode: the corresponding schema graph node
	 *  
	 */
	static protected void setOccursProperty(Element ele, SchemaGraphNode sgn,
			SchemaGraphNode parentSgn) {
		String minOccurs = ele.getAttribute("minOccurs");
		String maxOccurs = ele.getAttribute("maxOccurs");

		int min;
		int max;

		if (minOccurs != "")
			min = new Integer(minOccurs).intValue();
		else
			min = 1;

		if (maxOccurs != "") {
			if (maxOccurs.equals("unbounded"))
				max = Occurrence.UNBOUNDED;
			else
				max = new Integer(maxOccurs).intValue();
		} else
			max = 1;

		sgn.registerParentWithOccur(parentSgn, min, max);

	}

	/**
	 * given a DOM node in the XML schema which describes the type relationship,
	 * associate the corresponding schema graph nodes, i.e., add edges among
	 * sepearte schema graph nodes Creation date:
	 * 
	 * @param sg
	 *            SchemaGraph: the schema graph
	 * @param curEle
	 *            Element: the element in XML schema revisit: choice, e.g., c|d
	 */
	static protected void resolveEdges(SchemaGraph sg, Element curEle) {

		String curEleName = curEle.getTagName();

		//edge relationship is indicated in the "sequence" content in
		// "complexType"
		//in "complextType", there can be either "sequence" or "attribute"
		if (curEleName.equals("xsd:complexType")) {

			//find the parent nodes which are of the this docEleType
			String docEleType = curEle.getAttribute("name");
			SchemaGraphNode[] parentSgns = (SchemaGraphNode[]) (((Vector) sg
					.getNodesForType(docEleType))
					.toArray(new SchemaGraphNode[] {}));
			NodeList children = curEle.getChildNodes();
			int childrenNum = children.getLength();

			for (int i = 0; i < childrenNum; i++) {
				if (children.item(i) instanceof Element)
					setEdge(sg, parentSgns, (Element) children.item(i));
			}
		} else {
			//recursively apply this on current node's children
			NodeList children = curEle.getChildNodes();
			int childrenNum = children.getLength();

			for (int i = 0; i < childrenNum; i++) {
				if (children.item(i) instanceof Element)
					resolveEdges(sg, (Element) children.item(i));
			}
		}
	}

	//the curEle, must be mapped to a control node, such as sequence or choice,
	//the control node is added to the parent
	public static void setEdge(SchemaGraph sg, SchemaGraphNode[] parents,
			Element curEle) {
		SchemaGraphNode childNode = null;
		SchemaGraphNodeAttr newAttr = null;
		boolean furtherMore = false;
		boolean element = false;

		if (curEle.getTagName().equals("xsd:sequence")) {
			childNode = new SchemaGraphNode(SchemaGraphNode.SEQUENCE);
			furtherMore = true;
		} else if (curEle.getTagName().equals("xsd:choice")) {
			childNode = new SchemaGraphNode(SchemaGraphNode.CHOICE);
			furtherMore = true;
		} else if (curEle.getTagName().equals("xsd:element")) {
			//may be refer either to a
			String curEleName = curEle.getAttribute("name");
			if (curEleName.equals(""))
				curEleName = curEle.getAttribute("ref");
			childNode = sg.getNodeForName(curEleName);
			element = true;
			//the child node no longer needs to be navigated further
			//since they should have no child
		} else if (curEle.getTagName().equals("xsd:attribute"))
			//this attribute must be associated with all the names of this type
			newAttr = new SchemaGraphNodeAttr(curEle.getAttribute("name"),
					curEle.getAttribute("type"), curEle.getAttribute("use"),
					curEle.getAttribute("fixed"), curEle
							.getAttribute("default"));

		//set up parent-child relationship between parent and new node
		//or attribute relationship
		int parentNum = parents.length;

		if (childNode != null)
			for (int i = 0; i < parentNum; i++) {
				SchemaGraphNode parent = parents[i];
				parent.addChild(childNode);
				childNode.addParent(parent);
				setOccursProperty(curEle, childNode, parent);
			}
		else if (newAttr != null) {
			for (int i = 0; i < parentNum; i++) {
				SchemaGraphNode parent = parents[i];
				parent.addAttr(newAttr);
			}
		}

		if (furtherMore) {
			NodeList children = curEle.getChildNodes();
			int childrenNum = children.getLength();

			for (int i = 0; i < childrenNum; i++) {
				if (children.item(i) instanceof Element) {
					Element childEle = (Element) children.item(i);
					SchemaGraphNode[] nextParents = new SchemaGraphNode[1];
					nextParents[0] = childNode;
					setEdge(sg, nextParents, childEle);
				}
			}
		}

		if (element)
			//if an element declaration is encountered
			//it might again have parent-child relationship in it
			resolveEdges(sg, curEle);
	}

	/**
	 * given a node in the schema graph, purge it if it is not of any interest
	 * in the query graph note, not in the query graph does not necessarily
	 * ensure the purge new pattern might be introduced in some SQO heuristics
	 * Creation date:
	 * 
	 * @param parentSgn
	 *            SchemaGraphNode: the parent node of the current node under
	 *            checking for purge
	 * @param curSgn
	 *            SchemaGraphNode: the current node under checking for purge
	 */

	static protected void purge(SchemaGraphNode parentSgn,
			SchemaGraphNode curSgn) {
		if (concernedSchema.contains(QueryGraphNode.WILDCARD)
				|| concernedSchema.contains(QueryGraphNode.DESC))
			//never know what specific elements are involved
			//do not purge
			return;

		String curName = curSgn.getTagName();

		if (!curName.equals(SchemaGraphNode.SEQUENCE)
				&& !curName.equals(SchemaGraphNode.CHOICE)
				&& !concernedSchema.contains(curSgn.getTagName())) {

			boolean concerned = false;
			if (parentSgn != null) {

				//also its siblings do not appear in the query, then this ele
				// is not of any interest
				GraphNode[] siblings = parentSgn.getChildren();

				int siblingNum = siblings.length;

				for (int i = 0; i < siblingNum; i++) {

					SchemaGraphNode sibling = (SchemaGraphNode) siblings[i];
					
					//hardcode, for |
					

					if (concernedSchema.contains(sibling.getTagName()) || sibling.getParent().getParent().getTagName().equals(SchemaGraphNode.CHOICE)) {
						concerned = true;
						break;
					} else {
						if (sibling.getTagName().equals(
								SchemaGraphNode.SEQUENCE)) {
							List descs = ((SchemaGraphNode) siblings[i])
									.getConcreteDesc();
							for (Iterator it=descs.iterator(); it.hasNext();){
								SchemaGraphNode desc = (SchemaGraphNode)it.next();
								if (concernedSchema.contains(desc.getTagName())){
									concerned = true;
									//out of inner for
									break;
								}
							}
							if (concerned)
								//out of outer for (int i=0; i<siblingNum; i++)
								break;
						}
					}
				}

				if (!concerned)
					//purge this node
					parentSgn.deleteChild(curSgn);

			} //end of it (parentSgn!=null)
		} //end of if (!concernedSchema.contains(curSgn.getTagName()))

		//else this node remains, continue checking its descendant

		GraphNode[] siblings = curSgn.getChildren();

		int siblingNum = siblings.length;

		for (int i = 0; i < siblingNum; i++)
			purge(curSgn, (SchemaGraphNode) siblings[i]);

	}

	public static void normalize(SchemaGraphNode sgn, List alreadyNormalized) {
		if (alreadyNormalized.contains(sgn))
			return;

		boolean completeRemoveSgn = false;
		//	alreadyNormalized.add(sgn);
		if (sgn.getTagName().equals(SchemaGraphNode.SEQUENCE)) {
			List parents = sgn.getParentList();
			List toBeRemovedParents = new ArrayList();
			for (Iterator it = parents.iterator(); it.hasNext();) {
				SchemaGraphNode parent = (SchemaGraphNode) it.next();
				Occurrence occur = sgn.getOccurWithParent(parent);
				if (occur.getMaxOccurs() == 1 && occur.getMinOccurs() == 1
						&& !parent.getTagName().equals(SchemaGraphNode.CHOICE)) {
					//occur exactly one and this sequence is not a child of a
					// choice
					//  choice
					//   / \
					//  a sequence
					//      / \
					//      b c
					//the above sequence cannot be removed
					parent.deleteChild(sgn);
					sgn.deregisterParentOccur(parent);
					toBeRemovedParents.add(parent);
					GraphNode[] sgnChildren = sgn.getChildren();
					int sgnChildrenNum = sgnChildren.length;
					for (int i = 0; i < sgnChildrenNum; i++) {
						SchemaGraphNode sgnChild = (SchemaGraphNode) sgnChildren[i];
						parent.addChild(sgnChild);
						Occurrence childOccur = sgnChild
								.getOccurWithParent(sgn);
						sgnChild.registerParentWithOccur(parent, childOccur);
						//sgnChild.deregisterParentOccur(sgn);
						//sgnChild.removeParent(sgn);
						sgnChild.addParent(parent);
					}
				}
			} //end of for (Iterator it = parents.iterator();
			for (Iterator it = toBeRemovedParents.iterator(); it.hasNext();) {
				SchemaGraphNode toBeRemovedParent = (SchemaGraphNode) it.next();
				sgn.removeParent(toBeRemovedParent);
			} //if sgn does not appear within any parents anymore remove it
			if (sgn.getParentList().size() == 0) {
				completeRemoveSgn = true;
				GraphNode[] sgnChildren = sgn.getChildren();
				int sgnChildrenNum = sgnChildren.length;
				for (int i = 0; i < sgnChildrenNum; i++) {
					SchemaGraphNode sgnChild = (SchemaGraphNode) sgnChildren[i];
					sgnChild.deregisterParentOccur(sgn);
					sgnChild.removeParent(sgn);
				}

			} //end of if (sgn.getParentList().size() == 0)
		} //recursively apply on children
		GraphNode[] children = sgn.getChildren();
		int childrenNum = children.length;
		for (int i = 0; i < childrenNum; i++)
			normalize((SchemaGraphNode) children[i], alreadyNormalized);
		if (completeRemoveSgn)
			sgn.removeAllChildren();
	}
}