/*
 * Created on Jan 27, 2004g
 *
 * node in the schema graph
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * @author hong su
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class SchemaGraphNode extends GraphNodeImp implements GraphNode {

	public static final int NONEXIST = -1;

	//since a schema graph node can occur in multiple parents
	//e.g., <!ELEMENT a (c, d)> <!ElEMENT b (c?, d)>
	//c occurs both within a and b
	//parentToOccur records in a given parent, the occurrence constraint of the child element
	//e.g., c in a (1, 1) - occur once
	//while c in b (0, 1) - occur optionally
	protected HashMap parentToOccur = new HashMap();
	//map a given edge which connects the parent and the child to a state name
	protected HashMap edgeToState = new HashMap();

	protected Vector attrs = new Vector();

	public static final String SEQUENCE = "SEQUENCE";
	public static final String CHOICE = "CHOICE";

	public SchemaGraphNode() {
		super();
	}

	/*
	 * register THIS node which occurs in its parent node sgn with occurrence properties (minOccurs, maxOccr)
	 * @param sgn SchemaGraphNode the parent node of THIS
	 * @param minOccurs int minimal occurrence of THIS within sgn
	 * @param maxOccurs int maximal occurrence of THIS within sgn 
	 */
	public void registerParentWithOccur(
		SchemaGraphNode sgn,
		int minOccurs,
		int maxOccurs) {
		Occurrence occur = new Occurrence(minOccurs, maxOccurs);
		parentToOccur.put(sgn, occur);
	}

	/*
	 * similar to the above one
	 */
	public void registerParentWithOccur(
		SchemaGraphNode sgn,
		Occurrence occur) {
		parentToOccur.put(sgn, occur);
	}

	/*
	 * the reverse of the above one
	 */
	public void deregisterParentOccur(SchemaGraphNode parent) {
		parentToOccur.remove(parent);
	}

	/*
	 * find the occurrence constraints of THIS within parent sgn
	 * @param sgn the parent node of THIS
	 * @return Occurrence the occurrence constraints
	 */
	public Occurrence getOccurWithParent(SchemaGraphNode sgn) {
		Occurrence occur = (Occurrence) parentToOccur.get(sgn);
		return occur;
	}

	public SchemaGraphNode(String tagName) {
		this.tagName = tagName;
	}

	/*public Object clone() {
		SchemaGraphNode newSchemaGraphNode = new SchemaGraphNode();
		newSchemaGraphNode.minOccurs = this.minOccurs;
		newSchemaGraphNode.maxOccurs = this.maxOccurs;
		newSchemaGraphNode.tagName = this.tagName;
		return newSchemaGraphNode;
	}*/
	/*
	 * a schema graph node with tag name is an element type
	 * an element type may have attributes
	 */
	public void addAttr(SchemaGraphNodeAttr attr) {
		attrs.add(attr);
	}

	public Vector getAttrs() {
		return attrs;
	}

	/* given a name, locate sgns of this type in the descendants
	 * the order in returned ArrayList is consistent with their document order
	 * therefore, if a choice node has two children of the same name
	 * we return the choice node
	* @param childName String
	* @return ArrayList an array of SchemaGraphPath
	*/
	private ArrayList locateChildren(String childName) {
		ArrayList locatedChildren = new ArrayList();

		for (Iterator it = children.iterator(); it.hasNext();) {
			SchemaGraphNode child = (SchemaGraphNode) it.next();

			String curChildName = child.getTagName();
			if (curChildName.equals(childName)) {
				SchemaGraphPath path = new SchemaGraphPath();
				path.addPathNode(child);
				locatedChildren.add(path);
			} else if (curChildName.equals(SchemaGraphNode.SEQUENCE)) {
				//if a sequence node, recursively find its children
				ArrayList childrenPaths = child.locateChildren(childName);
				for (Iterator it2 = childrenPaths.iterator(); it2.hasNext();) {
					//concatenate this child node and the path from the child node to the target node (with concrete name)   
					SchemaGraphPath childPath = (SchemaGraphPath) it2.next();
					SchemaGraphPath path = new SchemaGraphPath();
					path.addPathNode(child);
					path.addPathNodes(childPath);
					locatedChildren.add(path);
				}
			} else { //must be a choice node
				ArrayList childrenPaths = child.locateChildren(childName);
				//choose an arbitrary one
				if (childrenPaths.size() > 0) {
					//exist some descendant of this choice that is of desired type
					SchemaGraphPath path = new SchemaGraphPath();
					path.addPathNode(child);
					locatedChildren.add(path);
				}
			}

		}
		return locatedChildren;
	}

	/*
	 * within the child elements of THIS node
	 * find the names of element types that must occur after the element type with childName 
	 */
	public ArrayList nodeTypesMustAfter(String childName) {
		ArrayList nodesMustAfter = new ArrayList();

		ArrayList locatedChildrenPaths = locateChildren(childName);
		//boundary must be behind the last one
		SchemaGraphPath lastChildPath =
			(SchemaGraphPath) locatedChildrenPaths.get(
				locatedChildrenPaths.size() - 1);

		//do a document order traversal after the boundary node	
		ArrayList nodesAfterBoundary = afterBoundary(lastChildPath);
		ArrayList nodesBeforeBoundary = beforeBoundary(lastChildPath);

		//those nodes whose type can only occurr after the boundary node
		nodesMustAfter = (ArrayList) nodesAfterBoundary.clone();

		for (Iterator it = nodesAfterBoundary.iterator(); it.hasNext();) {
			SchemaGraphNode nodeAfterBoundary = (SchemaGraphNode) it.next();
			String nodeName = nodeAfterBoundary.getTagName();
			for (Iterator it2 = nodesBeforeBoundary.iterator();
				it2.hasNext();
				) {
				SchemaGraphNode nodeBeforeBoundary =
					(SchemaGraphNode) it2.next();
				if (nodeBeforeBoundary.getTagName().equals(nodeName)) {
					nodesMustAfter.remove(nodeAfterBoundary);
					break;
				}
			}
		}

		//we need type names instead of schema graph nodes
		ArrayList nodeNames = new ArrayList();
		for (Iterator it = nodesMustAfter.iterator(); it.hasNext();) {
			nodeNames.add(((SchemaGraphNode) it.next()).getTagName());
		}

		return nodeNames;
	}

	private SchemaGraphNode findNonChoiceOnPathAncesNode(SchemaGraphNode root) {
		SchemaGraphNode curAnces = this;
		boolean existChoice = false;
		do {
			existChoice = false;
			ArrayList ancesNodes = curAnces.ancesNodesTo(root);
			for (Iterator it = ancesNodes.iterator(); it.hasNext();) {
				SchemaGraphNode ancesNode = (SchemaGraphNode) it.next();
				if (ancesNode.getTagName().equals(SchemaGraphNode.SEQUENCE))
					existChoice = true;
			}
			curAnces = (SchemaGraphNode) getParent();
		} while (existChoice);

		return curAnces;

	}

	public ArrayList ancesNodesTo(SchemaGraphNode ances) {
		ArrayList ancesNodes = new ArrayList();
		SchemaGraphNode curAnces = (SchemaGraphNode) getParent();
		while (curAnces != ances) {
			ancesNodes.add(curAnces);
			curAnces = (SchemaGraphNode) curAnces.getParent();
		}
		return ancesNodes;
	}

	/*
	 *  boundaryNode is a descendant of THIS node
	 *  find all the first non-sequence and non-choice ancestors of THIS that occur after THIS
	 * e.g.,     this
	 *         /  |   \
	 *      SEQ  CHO   c
	 *       /|   |    |
	 *      a b   a    ..
	 * suppose first this->SEQ->a is boundary, then afterboundary is {b, a, c}
	 */
	private ArrayList afterBoundary(SchemaGraphPath boundaryPath) {
		ArrayList nodesAfter = new ArrayList();

		//there must be no choice on boundarypath except the last one
		SchemaGraphNode nextToLastNode = boundaryPath.getNextToLastNode();
		int length = boundaryPath.getPathLength();

		SchemaGraphNode boundaryChild = null;

		Iterator it = null;
		if (nextToLastNode == null) {
			//this path contains only one node, 
			//i.e., the boundary node is a direct child of THIS node
			SchemaGraphNode lastNode = boundaryPath.getLastNode();
			boundaryChild = lastNode;
		} else {
			for (int i = length - 1; i > 0; i--) {
				SchemaGraphNode curParent = boundaryPath.getNode(i - 1);
				SchemaGraphNode curSgn = boundaryPath.getNode(i);
				nodesAfter.addAll(
					curParent.findFirstConcreteNodesAfter(curSgn));
				if (i == 1) {
					//now curParent is a direct child of THIS
					boundaryChild = curParent;
				}
			}

		}

		nodesAfter.addAll(findFirstConcreteNodesAfter(boundaryChild));

		return nodesAfter;
	}

	/*
		 *  boundaryNode is a descendant of THIS node
		 *  find all the first non-sequence and non-choice ancestors of THIS that occur after THIS
		 * e.g.,     this
		 *         /  |   \
		 *      SEQ  CHO   c
		 *       /|   |    |
		 *      a b   a    ..
		 * suppse first this->SEQ->a is boundary, then afterboundary is {b, a, c}
		 */
	private ArrayList beforeBoundary(SchemaGraphPath boundaryPath) {
		ArrayList nodesBefore = new ArrayList();

		//there must be no choice on boundarypath except the last one
		SchemaGraphNode nextToLastNode = boundaryPath.getNextToLastNode();
		int length = boundaryPath.getPathLength();

		SchemaGraphNode boundaryChild = null;

		Iterator it = null;
		if (nextToLastNode == null) {
			//this path contains only one node, 
			//i.e., the boundary node is a direct child of THIS node
			SchemaGraphNode lastNode = boundaryPath.getLastNode();
			boundaryChild = lastNode;
		} else {
			for (int i = length - 1; i > 0; i--) {
				SchemaGraphNode curParent = boundaryPath.getNode(i - 1);
				SchemaGraphNode curSgn = boundaryPath.getNode(i);
				nodesBefore.addAll(
					curParent.findFirstConcreteNodesBefore(curSgn));
				if (i == 1) {
					//now curParent is a direct child of THIS
					boundaryChild = curParent;
				}
			}

		}

		nodesBefore.addAll(findFirstConcreteNodesBefore(boundaryChild));

		return nodesBefore;
	}

	/*
	 * find all its first descendant nodes with concrete names after sgn
	 * sgn must be a direct child of THIS node
	 */
	private ArrayList findFirstConcreteNodesAfter(SchemaGraphNode sgn) {
		ArrayList nodesAfter = new ArrayList();

		Iterator it = null;

		for (it = children.iterator(); it.hasNext();) {
			SchemaGraphNode child = (SchemaGraphNode) it.next();
			if (child == sgn)
				break;
		}

		//it now points at the node which is the direct child of THIS node

		for (Iterator it2 = it; it2.hasNext();) {
			//start from the boundary node, look at its siblings
			SchemaGraphNode child = (SchemaGraphNode) it.next();
			nodesAfter.addAll(child.findFirstConcreteNodes());
		}

		return nodesAfter;
	}

	/*
		 * find all its first descendant nodes with concrete names after sgn
		 * sgn must be a direct child of THIS node
		 */
	private ArrayList findFirstConcreteNodesBefore(SchemaGraphNode sgn) {
		ArrayList nodesBefore = new ArrayList();

		int pos = 0;

		for (Iterator it = children.iterator(); it.hasNext();) {
			SchemaGraphNode child = (SchemaGraphNode) it.next();
			pos++;
			if (child == sgn)
				break;
		}

		//it now points at the node which is the direct child of THIS node

		for (int i = pos - 1; i >= 0; i--) {
			//start from the boundary node, look at its siblings
			SchemaGraphNode child = (SchemaGraphNode) children.get(i);
			nodesBefore.addAll(child.findFirstConcreteNodes());
		}

		return nodesBefore;
	}

	/* within the child elements of THIS,
	 * find the maximal occurrence of an subelement type with childName  
	 */
	public int getMaxOccurrence(String childName) {
		int maxOccurs = 0;

		if (tagName == childName)
			//for termination of recursive call
			return 1;

		for (Iterator it = children.iterator(); it.hasNext();) {
			SchemaGraphNode child = (SchemaGraphNode) it.next();

			String tagName = child.getTagName();

			if (!tagName.equals(SchemaGraphNode.CHOICE)
				&& !tagName.equals(SchemaGraphNode.SEQUENCE)) {
				if (tagName.equals(childName)) {
					Occurrence occur = child.getOccurWithParent(this);
					int childMaxOccur = occur.getMaxOccurs();
					if (childMaxOccur == Occurrence.UNBOUNDED)
						return Occurrence.UNBOUNDED;
					else
						maxOccurs = maxOccurs + childMaxOccur;
				} 
			} else {
				if (tagName.equals(SchemaGraphNode.CHOICE)) {
					int grandChildMaxOccur = 0;
					for (Iterator it2 = child.children.iterator();
						it2.hasNext();
						) {
						//check this choice's nodes
						//the maximal occurrence will the largest occurrence of choice's child
						SchemaGraphNode grandChild =
							(SchemaGraphNode) it2.next();
						int inChildMaxOccur =
							grandChild.getMaxOccurrence(childName);
						if (inChildMaxOccur > grandChildMaxOccur)
							grandChildMaxOccur = inChildMaxOccur;
					}
					maxOccurs = maxOccurs + grandChildMaxOccur;
				} else {
					//tagname = sequence
					for (Iterator it2 = child.children.iterator();
						it2.hasNext();
						) {
						//the maximal occurrence will be the sum of the maximal occurrence within choice's child
						SchemaGraphNode grandChild =
							(SchemaGraphNode) it2.next();
						maxOccurs =
							maxOccurs + grandChild.getMaxOccurrence(childName);
					}
				}
			}

		}

		return maxOccurs;

	}

	public int getMinOccurrence(String childName) {
		int minOccurs = 0;

		if (this.tagName.equals(childName))
			return 1;			
			

		for (Iterator it = children.iterator(); it.hasNext();) {
			SchemaGraphNode child = (SchemaGraphNode) it.next();

			String tagName = child.getTagName();

			if (!tagName.equals(SchemaGraphNode.CHOICE)
				&& !tagName.equals(SchemaGraphNode.SEQUENCE)) {
				if (tagName.equals(childName)) {
					Occurrence occur = child.getOccurWithParent(this);
					int childMinOccur = occur.getMinOccurs();
					minOccurs = minOccurs + childMinOccur;
				} 
			} else {
				if (tagName.equals(SchemaGraphNode.CHOICE)) {
					//minOccurs = 0;
				} else {
					//tagname = sequence
					for (Iterator it2 = child.children.iterator();
						it2.hasNext();
						) {
						//the maximal occurrence will be the sum of the maximal occurrence within choice's child
						SchemaGraphNode grandChild =
							(SchemaGraphNode) it2.next();
						minOccurs =
							minOccurs
								+ grandChild.getMinOccurrence(childName)
									* (grandChild
										.getOccurWithParent(child)
										.getMinOccurs());
					}
				}
			}
		}
		return minOccurs;
	}

	/*
	 * from THIS node, find all its first descendant nodes with concrete names
	 */
	private ArrayList findFirstConcreteNodes() {
		ArrayList result = new ArrayList();
		//if itself is a concrete node, return itself
		if (this.isConcrete()) {
			result.add(this);
			return result;
		}

		for (Iterator it = children.iterator(); it.hasNext();) {
			SchemaGraphNode child = (SchemaGraphNode) it.next();
			String childName = child.getTagName();
			if (child.isConcrete())
				result.add(child);
			else
				result.addAll(child.findFirstConcreteNodes());
		}

		return result;
	}

	/*
	 * whether this node has a concrete name
	 */
	public boolean isConcrete() {
		if (!tagName.equals(SchemaGraphNode.CHOICE)
			&& !tagName.equals(SchemaGraphNode.SEQUENCE))
			return true;
		else
			return false;
	}

	public String toString(SchemaGraphNode parentNode) {
		String output = new String("") + tagName;

		//if (tagName.equals(SEQUENCE) || tagName.equals(CHOICE))
		//	return output;

		if (parentNode != null) {

			Occurrence occur = getOccurWithParent(parentNode);
			int min = occur.getMinOccurs();
			int max = occur.getMaxOccurs();

			output = output + "(" + min + ",";
			if (max == Occurrence.UNBOUNDED)
				output = output + "infinite)";
			else
				output = output + max + ")";

		}

		/*for (Iterator it = getParents(); it.hasNext();) {
			SchemaGraphNode parent = (SchemaGraphNode) it.next();
			Occurrence occur = getOccurWithParent(parent);
		
			int min = occur.getMinOccurs();
			int max = occur.getMaxOccurs();
		
			output = output + "(" + min + ",";
			if (max == Occurrence.UNBOUNDED)
				output = output + "infinite)";
			else
				output = output + max + ")";
		}*/
		return output;
	}

	/*
	 * if THIS is a concrete name node, return a singleton containing itself
	 * if THIS is a special node, return the nearest concrete desc
	 */
	public List getConcreteDesc() {
		List result = new ArrayList();
		if (isConcrete())
			result.add(this);
		else {
			for (int i = 0; i < children.size(); i++)
				result.addAll(
					((SchemaGraphNode) children.get(i)).getConcreteDesc());

		}

		return result;

	}

	public String toString() {
		String output = new String("") + tagName;
		return output;
	}

}
