

/*
 * Created on Feb 23, 2004
 *
 * self shortcuttinga
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.rules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.FindOpVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.schedulemode.SelfPartitionDelimiInvoking;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.SelectStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.SuspensionEdge;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.NavigationStep;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedAutomata;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.ExtendedState;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.Occurrence;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.QueryGraph;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.QueryGraphNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.SchemaGraph;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.SchemaGraphNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.TraceState;

//import edu.wpi.cs.dsrg.xmldb.xat.component.sqo.redundant.*;

/**
 * If a destination node is defined a selection predicate over, we find whether
 * it occurs constant time. After it has appeared constant time within the entry
 * node, we can then evaluate the selection predicate over the bindings of the
 * destination node e.g., for $a in /a where $a/b = "..." return $a if we know
 * that b occurs in a only twice, then we can start evaluating $a/b="..." when
 * two bindings of b are found
 */
public class OccurrenceRule extends RuleImp {

	//given a tracestate, find whether its occurrence is greater than zero
	// within the root entry state
	HashMap traceStatesWithinRoots = new HashMap();
	
	//hardcode
	int count = 0;

	public OccurrenceRule(SchemaGraph sg) {
		super("occurrence rule", sg);

	}

	/**
	 * find all supension edges within parent that this rule successfully fires
	 * 
	 * @param parent
	 *            QueryEntryNode parent must be (1) an entry node (2) either has
	 *            multiple child or isReturned is true
	 * @param noPotentialRedun
	 *            boolean
	 * @return List a list of matchings each of which is an Object[5] object[0] =
	 *         the query graph node which must be parent itself or an ancestor
	 *         entry node of parent object[1] = the nav with the selection on
	 *         object[2] = the nav opertor whose destination maps to object[0]
	 *         object[3] = the extract on object[1] object[4] = the occurrence
	 *         number if noPotentialRedun is true, only when starting state of
	 *         the parent must occur, find matching within the subtree derived
	 *         from the starting state if noPotentialRedun is false, do not need
	 *         to consider whether the starting state of the parent must occur
	 */

	public List findAllMatchings(TraceState startingState, QueryGraphNode desc,
			boolean noPotentialRedun) {
	//	if (count>0)
		//	return new ArrayList();

		//first check whether starting states are qualified if noPotentialRedun
		// is true
		//it stores a list of integer, the starting state with this index does
		// not surely appear
		boolean surelyWorking = true;
		QueryGraphNode parent = startingState.getQgn();
		int pos = 0;

		//find which branch desc is on starting state
		for (int i = 0; i < parent.getChildrenSize(); i++) {
			if (((QueryGraphNode) parent.getChild(i)).getDestQgn() == desc) {
				pos = i;
				break;
			}
		}

		List matchings = new ArrayList();

		//first check whether it has a descendant node mapped from a variable
		//on which a local selection is defined on
		List associatedNavs = startingState.getQgn().getOpsOnEntry();
		XATStreamOperator nav = (XATStreamOperator) associatedNavs.get(pos);

		DagNode navNode = (DagNode) nav.getNode();
		ExtractStream extract = FindOpVisitor.findAssociatedExtract(navNode);
		if (extract == null) {
			//won't have a selection defined on it
			return matchings;
		}

		SelectStream selectOp = FindOpVisitor.simpleSelDefinedOn(extract);
		//if no selection defined, does not qualify
		if (selectOp == null) {
			return matchings;
		}

		//the child node of the starting state that is on the path to dest
		QueryGraphNode curChildNode = (QueryGraphNode) parent.getChild(pos);
		List curChildrenStates = startingState.getDescStates(curChildNode);

		List leaveQgns = new ArrayList();

		HashMap parentStateToOccurMap = new HashMap();

		while (curChildrenStates.size() > 0) {
			List newCurChildrenStates = new ArrayList();

			for (Iterator it2 = curChildrenStates.iterator(); it2.hasNext();) {
				TraceState curChildState = (TraceState) it2.next();
				TraceState curParentState = curChildState.getParentTraceState();
				//not end of the path expression which a selection is defined
				// on
				//i.e., not reaching the leaves of the subgraph yet

				SchemaGraphNode curParentSgn = curParentState.getSgn();
				SchemaGraphNode curChildSgn = curChildState.getSgn();

				//curParentSGn might be a special node
				// e.g., bidder bidder
				//         | |
				//       sameAddr choice
				//
				//                      sameAddr
				Occurrence childInParent = curParentState
						.getOccur(curChildState);

				int minOccur = childInParent.getMinOccurs();

				if (curParentSgn.getTagName().equals(SchemaGraphNode.CHOICE))
					minOccur = 0;
				int maxOccur = childInParent.getMaxOccurs();

				if (maxOccur == Occurrence.UNBOUNDED)
					//this variable's occurrence cannot be determined
					//one child of the parent state is not determined
					//no need to check other child of the same parent state
					//more over, no need to check any descendant states of the
					// same starting state
					return matchings;

				if (maxOccur != Occurrence.UNBOUNDED) {

					if (minOccur != maxOccur) {
						if (!noPotentialRedun)
							//this occurrence does not surely work
							//since the element may not occur maxoccur
							surelyWorking = false;
						else
							//cause potential redun
							return matchings;
					}

					//update the nodeToOccurMap
					Integer curParentOccur = (Integer) parentStateToOccurMap
							.get(curParentState);
					int occurForChild = 0;
					if (curParentOccur != null)
						occurForChild = curParentOccur.intValue() * maxOccur;
					else
						//happens when parent is the entry point of the path
						// expression
						occurForChild = maxOccur;

					parentStateToOccurMap.put(curChildState, new Integer(
							occurForChild));

					if (curChildState.getQgn()!=null
							&& curChildState.getQgn().getOpsOnDest().size() == 0)
						//not the leaves yet
						newCurChildrenStates.addAll(curChildState
								.getChildrenTraceStates());
					else {
						//already the leaves
						if (!leaveQgns.contains(curChildState.getQgn()))
							leaveQgns.add(curChildState.getQgn());
					}

				}

			} //end of for (Iterator it = newCurChildrenStates.iterator();
			  // it.hasNext();) checking each child type } //end of if: finished
			  // checking all childrenstates of this level

			curChildrenStates.clear();
			curChildrenStates = newCurChildrenStates;
		} //end of while

		//wrap up
		
		count++;

		//determinable, a matching has been found
		Object[] matching = new Object[6];
		//the starting state
		//we find whether it can be further propagated up
		matching[0] = startingState.propagatableStartingStates();
		//the nav with a selection on
		matching[1] = nav;
		//the nav opertor whose destination maps to matching[0]
		/*
		 * List opsOnDest = ((TraceState) ((List) matching[0]) .get(((List)
		 * matching[0]).size() - 1)) .getQgn() .getOpsOnDest(); if
		 * (opsOnDest.size() > 0) matching[2] = ((TraceState) ((List)
		 * matching[0]) .get(((List) matching[0]).size() - 1)) .getQgn()
		 * .getOpsOnDest() .get(0);
		 */
		//matching[2] = startingState;
		//the extract on the nav
		matching[2] = extract;
		//caculate the occurrence
		/*
		 * for (int j = 0; j < leaveQgns.size(); j++) { int occurSum = 0;
		 * QueryGraphNode leaveQgn = (QueryGraphNode) leaveQgns.get(j); for
		 * (Iterator it = leaveQgn.getTraceStates().iterator(); it.hasNext(); ) {
		 * TraceState destNState = (TraceState) it.next(); occurSum = occurSum +
		 * ((Integer) parentStateToOccurMap.get(destNState)) .intValue(); }
		 */

		List destStates = startingState.getDescStates(desc);
		int occurSum = 0;
		for (Iterator it3 = destStates.iterator(); it3.hasNext();) {
			TraceState destState = (TraceState) it3.next();
			occurSum = occurSum
					+ ((Integer) parentStateToOccurMap.get(destState))
							.intValue();
		}

		matching[3] = new Integer(occurSum);

		matching[4] = startingState.getDescStates(desc);

		if (surelyWorking)
			//min = max always
			matching[5] = new Boolean(true);
		else
			matching[5] = new Boolean(false);

		matchings.add(matching);
		//			}

		return matchings;
	}

	/*
	 * given a suspension edge, reflect it into the plan
	 */
	public void translateBackToPlan(Object[] matching) {
		//for operators that have coupling relationships
		//TraceState startingState = (TraceState) matching[0];
		List propagatableEntryStates = (List) matching[0];
		PureNavigateAutoStream selNav = (PureNavigateAutoStream) matching[1];
		/*
		 * PureNavigateAutoStream ancestorEntryNav = (PureNavigateAutoStream)
		 * matching[2];
		 */

		ExtractStream invokedExtract = (ExtractStream) matching[2];

		ExtendedAutomata auto = invokedExtract.getAutomata();

		//find the state for the destination of the nav
		List destStates = auto.getFrStateHash(selNav);

		//find the state for the entry of the nav,
		//i.e., the state for the dest of this nav's child (upstream) nav
		//PureNavigateAutoStream childNav =
		//	(PureNavigateAutoStream) nav.getNode().getChild(0).getOperator();
		List propagatableAutoStates = new ArrayList();
		
			
		for (Iterator it = propagatableEntryStates.iterator(); it.hasNext();) {
			TraceState propagatableEntryState = (TraceState) it.next();
			PureNavigateAutoStream propagatableEntryNav = (PureNavigateAutoStream) propagatableEntryState
					.getQgn().getOpsOnDest().get(0);
			List autoStates = auto.getFrStateHash(propagatableEntryNav,
					propagatableEntryState.getSgn().getTagName());
			propagatableAutoStates.addAll(autoStates);
			}

		SuspensionEdge edge = new SuspensionEdge(((Integer) matching[3])
				.intValue(), (SelfPartitionDelimiInvoking) invokedExtract,
				propagatableAutoStates);
		
		((ExtendedState)propagatableAutoStates.get(propagatableAutoStates.size()-1)).addOpResetPartitionChecked((XATStreamOperator)invokedExtract);
		//((ExtendedState)propagatableAutoStates.get(lastEntryStatePos-1)).addOpResetPartitionChecked((XATStreamOperator)invokedExtract);
		((SelfPartitionDelimiInvoking)invokedExtract).setNeedSetPartitionCheck();

		TraceState startingState = (TraceState) propagatableEntryStates.get(0);
		QueryGraph qg = startingState.getQgn().getQg();

		//for (Iterator it2 = destStates.iterator(); it2.hasNext();) {
		ExtendedState destState = (ExtendedState) destStates.get(0);
		destState.addSuspensionEdgeAtPop(edge);

		//if (destState.getTransits() != null || destState.hasLambdaTransit()
		//	|| destState.hasSelfStarTransit() || destState.hasStarTransit())
			propagatableAutoStates.add(0, destState);

		qg.getAutoStateToTraceStates().put(destState, matching[4]);
		//}

		String output = "from occurrence rule:\n";

		for (int i = 0; i < matching.length; i++)
			output = output + matching[i] + "&&";
		output = output + "\n";

		System.out.println(output);

		System.out.println(auto);

	}

	//for operators that have coupling relationships
	/*
	 * List propagatableEntryStates = (List) matching[0]; PureNavigateAutoStream
	 * selNav = (PureNavigateAutoStream) matching[1];
	 * 
	 * 
	 * ExtractStream invokedExtract = (ExtractStream) matching[2];
	 * 
	 * ExtendedAutomata auto = invokedExtract.getAutomata();
	 * 
	 * //find the state for the destination of the nav //List destStates =
	 * auto.getFrStateHash(selNav,
	 * ((TraceState)matching[2]).getSgn().getTagName()); boolean first = true;
	 * ExtendedState autoState = null;
	 * 
	 * TraceState prevState = null; for (Iterator it =
	 * ((List)matching[4]).iterator();it.hasNext();){
	 * 
	 * 
	 * if (first){
	 * 
	 * //find the initial automata state TraceState firstState =
	 * (TraceState)it.next(); prevState = firstState; QueryGraphNode stateQgn =
	 * firstState.getQgn(); XATStreamOperator nav =
	 * (XATStreamOperator)stateQgn.getOpsOnDest().get(0); autoState =
	 * (ExtendedState)auto.getFrStateHash(nav, stateQgn.getTagName()).get(0);
	 * continue; }
	 * 
	 * TraceState otherState = (TraceState)it.next(); QueryGraphNode stateQgn =
	 * otherState.getQgn(); if (stateQgn==null) //other state can be a special
	 * state such as "choice" or "seq' continue;
	 * 
	 * if (otherState.getConcreteParentState()==prevState){ //there are no
	 * intermediate query tree node ExtendedState newAutoState = null; boolean
	 * shareSameNextState = false; newAutoState =
	 * (ExtendedState)autoState.getTransit(stateQgn.getTagName()); for (Iterator
	 * it2 = autoState.getTransits().entrySet().iterator(); it2.hasNext();){
	 * String transitName = (String)it2.next(); ExtendedState nextState =
	 * (ExtendedState)autoState.getTransit(transitName); if (nextState ==
	 * newAutoState){ shareSameNextState =true; } }
	 * 
	 * if (shareSameNextState){ //need to split newAutoState = new
	 * ExtendedState(); autoState.addTransit(stateQgn.getTagName(),
	 * newAutoState); }else autoState = newAutoState; } else{ Vector path =
	 * prevState.pathDownToDesc(otherState); for (Iterator it3 =
	 * path.iterator();it3.hasNext();){ TraceState curState =
	 * (TraceState)it3.next(); if (curState.getQgn()!=null) autoState =
	 * (ExtendedState)autoState.getTransit(curState.getQgn().getTagName());
	 *  } }
	 *  }
	 * 
	 * //find the state for the entry of the nav, //i.e., the state for the dest
	 * of this nav's child (upstream) nav List propagatableAutoStates = new
	 * ArrayList(); for (Iterator it = propagatableEntryStates.iterator();
	 * it.hasNext();) { TraceState propagatableEntryState = (TraceState)
	 * it.next(); PureNavigateAutoStream propagatableEntryNav =
	 * (PureNavigateAutoStream) propagatableEntryState .getQgn() .getOpsOnDest()
	 * .get( 0); List autoStates = auto.getFrStateHash( propagatableEntryNav,
	 * propagatableEntryState.getSgn().getTagName());
	 * propagatableAutoStates.addAll(autoStates); }
	 * 
	 * SuspensionEdge edge = new SuspensionEdge( ((Integer)
	 * matching[3]).intValue(), (SelfPartitionDelimiInvoking) invokedExtract,
	 * propagatableAutoStates);
	 * 
	 * //for (Iterator it2 = descStates.iterator(); it2.hasNext();) { //
	 * ExtendedState destState = (ExtendedState) it2.next(); //
	 * destState.addSuspensionEdgeAtPop(edge);
	 * autoState.addSuspensionEdgeAtPop(edge); //}
	 * 
	 * String output = "from occurrence rule:\n";
	 * 
	 * for (int i = 0; i < matching.length; i++) output = output + matching[i] +
	 * "&&"; output = output + "\n";
	 * 
	 * System.out.println(output);
	 * 
	 * System.out.println(auto);
	 *  }
	 */

}