/*
 * Created on Feb 23, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.rules;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.GraphNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.QueryGraph;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.QueryGraphNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.SchemaGraph;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.SchemaGraphNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.sqo.TraceState;

/**
 * @author suhong
 * 
 * apply rules on the given query graph based on the given schema graph
 */
public class RuleApplier {

	private SchemaGraph sg;

	private QueryGraph qg;

	boolean applyOccur = false;

	boolean applyExclusive = false;

	boolean applyUncertain = false;

	//the nodes in the list have been applied
	//states that have been applied
	List statesAlreadyApplied = new ArrayList();

	//dependency list, an ordered list of rules (including data rules and
	// control rules)
	List ruleList = new ArrayList();

	//the position on bindinsgMainter, pointing to a binding
	int pos = 0;

	public RuleApplier(SchemaGraph sg, QueryGraph qg, boolean applyOccur,
			boolean applyExclusive, boolean applyUncertain) {
		this.sg = sg;
		this.qg = qg;
		this.applyOccur = applyOccur;
		this.applyExclusive = applyExclusive;
		this.applyUncertain = applyUncertain;

		if (applyOccur)
			ruleList.add(new OccurrenceRule(sg));

		if (applyExclusive)
			ruleList.add(new ExclusivePatternIntro(sg));

		if (applyUncertain)
			ruleList.add(new UncertainBoundaryPattern(sg));

	}

	/*
	 * apply rules on qgn according to dg @param qgn QueryGraphNode this node
	 * matches the parent node in all rule tempaltes @param applyUncertain
	 * boolean whether apply uncertainboundarypattern note, the
	 * uncertainBoundaryPattern rule is only needed to apply on the root once it
	 * traverses the whole query graph during this one application for other
	 * rules, the range of its application is limited to between two entry nodes
	 * so that they need to be applied multiple times
	 *  
	 */
	public List applyOnParent(QueryGraphNode qgn, boolean noPotentialRedun) {

		List matchings = new ArrayList();

		List parentStates = qgn.getTraceStates();

		//conceptually, we do not have duplate tracestates of the same type
		// assocaited with a query tree node
		//we therefore apply on distinct trace states
		//List alreadyAppliedTraceStateNames = new ArrayList();

		for (Iterator it = parentStates.iterator(); it.hasNext();) {
			TraceState parentState = (TraceState) it.next();
			//	String parentStateName = parentState.getSgn().getTagName();
			//	if (!alreadyAppliedTraceStateNames.contains(parentStateName)) {
			//	if (!statesAlreadyApplied.contains(parentState)){
			//have not apply before yet
			//	alreadyAppliedTraceStateNames.add(parentStateName);
			//	statesAlreadyApplied.add(parentState);

				if (statesAlreadyApplied.contains(parentState))
			//already applied before
					return matchings;

			//List prevStates = new ArrayList();
			applyOnState(parentState, noPotentialRedun, matchings);
			//}
		}

		return matchings;
	}

	/*
	 * @param startingState TraceState the trace state of the starting state
	 * @param noPotentialRedun boolean whether allows potentially redundant
	 * checking
	 */
	public boolean applyOnState(TraceState startingState,
			boolean noPotentialRedun, List matchings) {
		

		boolean allChildHasSureEndingMark = true;

		boolean foundSureOne = false;

		QueryGraphNode parent = startingState.getQgn();
		SchemaGraphNode parentSgn = startingState.getSgn();

		List destQgns = parent.getAllDestQgns();

		for (int i = 0; i < destQgns.size(); i++) {
			QueryGraphNode destQgn = (QueryGraphNode) destQgns.get(i);

			if (destQgn.getTraceStates().size() == 0)
				//an EMPTY node does not any tracestates
				continue;

			if (destQgn.getType() == QueryGraphNode.NONFILTER)
				//a non-filtering child needs not to be considered
				continue;

			List destQgnStates = startingState.getDescStates(destQgn);
			TraceState destQgnState = (TraceState) destQgnStates.get(0);

			if (destQgnStates.size() > 1
					|| destQgn.getChildrenSize() == 0
					|| (destQgnStates.size() == 1 && parentSgn
							.getMaxOccurrence(destQgnState.getSgn()
									.getTagName()) > 1)) {
				//1. there are more than two states of the destination node
				//2. the destQgn is a leaf
				//3. there is only one state of the destination node but the
				// state can appear in starting state multiple times
				for (Iterator it = ruleList.iterator(); it.hasNext();) {
					Rule rule = (Rule) it.next();

					if (rule instanceof UncertainBoundaryPattern) {
						//when applying occurrence rule, we first need to
						//make sure destQgn needs an ending mark
						//for other rules, whether they are applicable needs to
						// only look at the local range
						//i.e., no need to look at descendatnt
						// a
						// |
						// b
						//if b occurs in a multiple times, and the tree rooted
						//at b must occur, then no need to find an ending mark
						// for b
						boolean minOccurPositive = startingState
								.minOccurPositive(destQgnState);
						//whether this dest state is a definite appearing one
						// within its starting state
						if (minOccurPositive) {
							//the last child state must occur within the
							// starting state when potentialRedun is not allowed
							//or potentialRedun is allowed
							//check the subtree rooted at lastChildState to
							// find the ending marks
							statesAlreadyApplied.add(destQgnState);

							List marksWithinDestQgnState = new ArrayList();
							boolean sureMark = applyOnState(destQgnState,
									noPotentialRedun, marksWithinDestQgnState);

							matchings.addAll(marksWithinDestQgnState);

							if (sureMark //&& marksWithinDestQgnState.size() ==
										 // 0
									&& !destQgn.getIsSel()) {
								//must occur, no ending mark is needed
								foundSureOne = true;
								break;
							}
						}
					}//end of if rule instanceof OccurrenceRule

					foundSureOne = rule.fire(startingState, destQgn,
							noPotentialRedun, matchings);
					if (foundSureOne)
						//if find sure one, no need to futher apply the rule on
						// this leaf node
						break;
				}
				//endinf mark of the current state is not found
				if (!foundSureOne)
					allChildHasSureEndingMark = false;

			} //end of if (destQgnState.size() > 1 || destQgn.getChildrenSize()
			// == 0)
			else {
				//recursively apply on the children of destQgnState
				boolean minOccurPositive = startingState
						.minOccurPositive(destQgnState);
				//whether this dest state is a definite appearing one within
				// its starting state
				if (minOccurPositive || !noPotentialRedun) {
					//the last child state must occur within the starting state
					// when potentialRedun is not allowed
					//or potentialRedun is allowed
					//check the subtree rooted at lastChildState to find the
					// ending marks
					statesAlreadyApplied.add(destQgnState);

					List marksWithinDestQgnState = new ArrayList();
					boolean sureMark = applyOnState(destQgnState,
							noPotentialRedun, marksWithinDestQgnState);

					matchings.addAll(marksWithinDestQgnState);

					if (!(sureMark && minOccurPositive)) {

						//non surely appearing ending mark exist on its
						// children
						//so we have to find an ending mark on the node itself
						for (Iterator it = ruleList.iterator(); it.hasNext();) {
							Rule rule = (Rule) it.next();
							foundSureOne = rule.fire(startingState, destQgn,
									noPotentialRedun, matchings);
							if (foundSureOne)
								//if find sure one, no need to futher apply the
								// rule on this leaf node
								break;
						}

						if (!foundSureOne)
							allChildHasSureEndingMark = false;
					} //end of if (applyOnChild)
				} //end of if (minOccur > 0 || !noPotentialRedun)
			} //end of else
		}//end of for

		return allChildHasSureEndingMark;
	}

	/*
	 * apply rules on a query graph travese the graph tree bottom up, for each
	 * node, then apply rules @param qg QueryGraph @param noPoentialRedun
	 * boolean
	 *  
	 */
	public void applyOnGraph(QueryGraph qg, boolean noPotentialRedun) {
		Vector entries = qg.getTopEntries();

		//start top-down traversal of the query graph

		Vector curLevel = entries;

		while (curLevel.size() > 0) {
			Vector nextLevel = new Vector();
			for (Iterator it = curLevel.iterator(); it.hasNext();) {
				QueryGraphNode qgn = (QueryGraphNode) it.next();

				//find whether qgn (1) is an entry node (2) has parallel
				// computations
				if (qgn.isNonSelDest())
					//if indicating the application should not go on within
					//the descendant of this node
					//this can happen when
					applyOnParent(qgn, noPotentialRedun);

				GraphNode[] qgnChildren = qgn.getChildren();
				for (int i = 0; i < qgnChildren.length; i++)
					nextLevel.add(qgnChildren[i]);

			}
			curLevel = nextLevel;

		} //end of while (curLevel.size()>0)
	}

}