/*
 * we comment out any xmlfrags.add for saving memory space, so that the the storage size is not longer accurrate
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager;

import org.w3c.dom.*;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeperMgr;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;

import java.io.Serializable;
import java.util.List;

/**
 * manage a single stream storage the storage is smart enough not to store any
 * data twice e.g., if both a parent and a child is to stored, the child data
 * will not be stored twice instead a pointer to the middle of the parent gives
 * the child data Creation date: (1/17/2003 5:30:20 PM)
 * 
 * @author: Hong Su
 */

public class Finer_StreamStoreMgr implements Serializable {
	//object can be either stringbuffer or node
	private Element currentUnitToPutDom = null;

	private StringBuffer currentUnitToPutTypedVal = new StringBuffer();

	private int typedValCount;

	//	a stack to dom node parent
	private Element[] parentStack = new Element[100];

	//reserve one for the fake root
	private int stackPosition = 1;
	
//	if the current element must be stored, it is exectuing the extractUnderEffect
	private ExtractStream extractUnderEffect;

	//begin offset relative to the beginning of individual object for string
	private int beginOffset = 0;
	
	
	
	public Element getCurrentUnitToPutDom() {
		return currentUnitToPutDom;
	}

	public StringBuffer getCurrentUnitToPutTypedVal() {
		return currentUnitToPutTypedVal;
	}

	public int getTypedValCount() {
		return typedValCount;
	}

	/**
	 * reset the storage manager remove all storage and reset the positions
	 * Creation date: (5/12/2003 11:30:24 AM)
	 */
	public void clear() {

		
		currentUnitToPutDom = null;

		beginOffset = 0;
		typedValCount = 0;
		currentUnitToPutTypedVal = null;

		stackPosition = 1;
	}

	/**
	 * @return the positions
	 * @param storingMethod
	 *            int the storing method, i.e., complete tree/node etc
	 */
	public int getTypedValBegin() {
		return beginOffset;
	}

	/**
	 * get the ending position of an object this is only good for a string
	 * object it is not necessary for a dom object since the starting position
	 * implies the ending positions (the tree leaves) we simply retrieve the
	 * subtree rooted at the starting positions 
	 * @return int
	 * @param storingMethod
	 */
	public int getTypedValEnd() {
		return currentUnitToPutTypedVal.length();
	}

	public void putAsDomIntoStoragePush(String unit) {

		//create the element
		Element ele = new LightElement(unit);

		/*
		 * if (attributes != null) { for (int i = 0; i < attributes.getLength();
		 * i++) { ele.setAttribute( attributes.getLocalName(i),
		 * attributes.getValue(i)); } }
		 */

		//associate this newly creately element with its parent
		Element parent = (Element) currentUnitToPutDom;
		if (parent != null)
			parent.appendChild(ele);

		currentUnitToPutDom = ele;

		parentStack[stackPosition] = ele;
		

	}

	public void putAsDomIntoStoragePop() {

		currentUnitToPutDom = parentStack[stackPosition - 1];

	}

	public void putAsDomIntoStorageSlip(String unit) {

		Text text = new LightText(unit);
		currentUnitToPutDom.appendChild(text);

	}

	public void putAsTypedValIntoStoragePush(String unit) {

		if (typedValCount == 0) {
			//a new XML fragment is to be formed
			currentUnitToPutTypedVal = new StringBuffer();
		}

		//set the begin offset before the first tag is put in
		beginOffset = currentUnitToPutTypedVal.length();

		typedValCount++;
	}

	public void putAsTypedValIntoStoragePop() {

		typedValCount--;

	}

	public void putAsTypedValIntoStorageSlip(String unit) {

		//attach to the position right after where last unit storage happens
		//wrap the unit in the format corresponding to the stack action

		currentUnitToPutTypedVal.append(unit);

	}

	public void resetTypedValCount() {

		typedValCount = 0;

	}
	
	public void descreaseStackPosition(){
		stackPosition--;
	}
	
	public void increaseStackPosition(){
		stackPosition++;
	}

}