/*
 * this class manipulates the plan, such as copy, merge
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automatainout;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.FindOpVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.InTimeSJStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.SelectStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.StructuralJoinStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractByUnnestStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavUnnestStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.logicaloperator.StructuralJoin;

/**
 * @author suhong
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class PlanManipulator {

	/*
	 * @return DagNode find the nearest sj ancestor of tokenNav @param tokenNav
	 * PureNavigateAutoStream
	 */
	public static DagNode findNearestSJ(XATStreamOperator op) {
		DagNode parentNode = (DagNode) op.getNode();
		XATStreamOperator parentOp = (XATStreamOperator) parentNode
				.getOperator();
		while (!(parentOp instanceof StructuralJoinStream)) {
			parentNode = (DagNode) parentNode.getParent();
			if (parentNode ==null)
				break;
			parentOp = (XATStreamOperator) parentNode.getOperator();

		}

		if (parentOp instanceof StructuralJoinStream)
			//op does not have an ancestor SJ
			return parentNode;
		else
			return null;
	}

	/*
	 * @return DagNode find the nearest tokennav descendant of tokenNav @param
	 * nav NavigateStream
	 */

	public static DagNode findNearestTokenNav(NavigateStream nav) {
		DagNode childNode = (DagNode) nav.getNode().getChild(0);
		while (!(childNode.getOperator() instanceof PureNavigateAutoStream))
			childNode = (DagNode) childNode.getChild(0);

		return childNode;
	}

	/*
	 * @param head DagNode an entry operator of a branch @return DagNode find
	 * the tail of this branch
	 */
	public static List findEndingPointsFromEntry(DagNode entryNode) {
		List endingPoints = new ArrayList();
		

		ArrayList curParents = new ArrayList();
		ArrayList lastParents = new ArrayList();
		lastParents.add(entryNode);

		while (lastParents.size() > 0) {
            Iterator it = lastParents.iterator();
			while (it.hasNext()) {
				DagNode parentNode = (DagNode) it.next();
				if (parentNode.getParent().getOperator() instanceof StructuralJoinStream)
					endingPoints.add(parentNode);
				else
					for (Iterator it2 = parentNode.getParents(); it2.hasNext();)
						curParents.add(it2.next());
			}
			lastParents = (ArrayList) curParents.clone();
			curParents.clear();
		}

		return endingPoints;
	}

	/*
	 * group operators that share the same nearest ancestor structural join
	 * operator @return Map given a structural join operator, give a list of
	 * operators that are in the same group
	 */
	static public Map grouping(List tokenNavs) {
		Map sjToList = new HashMap();

		int tokenNavNum = tokenNavs.size();

		for (int i = 0; i < tokenNavNum; i++) {
			PureNavigateAutoStream tokenNav = (PureNavigateAutoStream) tokenNavs
					.get(i);
			DagNode sjAncestorNode = findNearestSJ(tokenNav);
			List group = (List) sjToList.get(sjAncestorNode);
			if (group == null)
				group = new ArrayList();
			group.add(tokenNav);
			sjToList.put(sjAncestorNode, group);
		}

		return sjToList;

	}

	/*
	 * tokenNav identifies a branch find the structural join operator this *
	 * branch is an input branch @param tokenNav PureNavigateAutoStream
	 * identifies an input branch
	 */
	public static StructuralJoinStream findInputSj(
			PureNavigateAutoStream tokenNav) {
		StructuralJoinStream fanInSj = (StructuralJoinStream) RuntimeOptUtil.getTokenNavToFanInSj()
				.get(tokenNav);
		if (fanInSj == null) {
			XATStreamOperator extractOp = (XATStreamOperator) tokenNav
					.getNode().getParent().getOperator();
			return (StructuralJoinStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(extractOp);
		} else
			return (StructuralJoinStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(fanInSj);
	}

	/*
	 * @param tokenNavNode DagNode the node whose parents are to be serialized
	 * @return Map return a map, given a node identifying a branch, give all the
	 * other branches that can be serialized with this branch serialize a
	 * subplan identified by a bottom tokenNavnode and a top structuraljoin node
	 */
	static Map findSerializableBranches(DagNode tokenNavNode) {

		//given the first branch, find all the branches that can be serialized
		// with it Map groups = new ArrayList();
		Map branchToGroup = new HashMap();

		int parentSize = tokenNavNode.getParentsSize();

		//this indicates the positions of the branches that can be merged to
		// some previous branches
		int[] mergedBranch = new int[parentSize];
		int mergedBranchPos = 0;

		for (int i = 0; i < parentSize - 1; i++) {

			boolean merged = false;

			for (int k = 0; k < parentSize; k++)
				if (mergedBranch[k] == i) {
					//this branch has been merged
					merged = true;
					break;
				}

			if (merged)
				continue;

			//two branches that has unnest cannot be serialized
			//assume each branch is optimal
			//i.e., the nodes are already in optimal order
			//so two branches, if seralizable, one branch will be inserted into
			// the other branch
			//while the other branch is fixed
			DagNode activeParentNode = (DagNode) tokenNavNode.getParent(i);
			List activeBranch = nodeToSj(activeParentNode);
			//check whether operator on this branch consumes destination node
			for (int j = i + 1; j < parentSize - 1; j++) {
				DagNode passiveParentNode = (DagNode) tokenNavNode.getParent(j);
				List passiveBranch = nodeToSj(passiveParentNode);
				if (!(conflict(activeBranch, passiveBranch))) {
					mergedBranch[mergedBranchPos] = j;
					mergedBranchPos++;
					activeBranch.addAll(passiveBranch);

					//register in the map that the passive branch can be
					// serialized
					//with the active branch
					List group = (List) branchToGroup.get(activeBranch);
					if (group == null)
						group = new ArrayList();

					group.addAll(passiveBranch);
					branchToGroup.put(activeParentNode, group);
				}
				//then check whether unnests on passive branch conflicts with
				// active branch
			}
		}

		return branchToGroup;
	}

	/*
	 * @node DagNode the source node @return a list of operators from the source
	 * node to the nearest structural join
	 */
	static List nodeToSj(DagNode node) {
		List operators = new ArrayList();
		operators.add(node);

		DagNode parentNode = (DagNode) node.getParent();
		XATStreamOperator parentOp = (XATStreamOperator) parentNode
				.getOperator();
		while (!(parentNode.getOperator() instanceof StructuralJoin)) {
			parentNode = (DagNode) parentNode.getParent(0);
			parentOp = (XATStreamOperator) parentNode.getOperator();
			operators.add(parentOp);
		}

		return operators;

	}

	/*
	 * @node DagNode the source node @return a list of navigateunnest operators
	 * on the branch
	 */
	static List unnestOpsOnBranch(DagNode node) {
		List operators = new ArrayList();

		DagNode parentNode = (DagNode) node.getParent();
		XATStreamOperator parentOp = (XATStreamOperator) parentNode
				.getOperator();
		while (!(parentOp instanceof StructuralJoin)) {
			if (parentOp instanceof NavUnnestStream)
				operators.add(parentOp);
			parentNode = (DagNode) parentNode.getParent(0);
			parentOp = (XATStreamOperator) parentNode.getOperator();
		}

		return operators;
	}

	/*
	 * @List sortedOpAndCosts a list of OpSta in sorted order connect these
	 * operators in the specified order @param dest the end of the list of
	 * operator should point to dest which is a structural join
	 */
	static void connect(List sortedOpStas, DagNode dest, CopiedDag tempPlan) {
		int num = sortedOpStas.size();

		XATStreamOperator lastOp = (XATStreamOperator) ((OpStatistics) sortedOpStas
				.get(0)).getOp();
		DagNode lastNode = tempPlan.getNode(lastOp);
		DagNode curNode = null;
		for (int i = 1; i < num; i++) {
			OpStatistics opSta = (OpStatistics) sortedOpStas.get(i);
			XATStreamOperator curOp = opSta.getOp();
			curNode = tempPlan.getNode(curOp);

			for (Iterator it = lastNode.getParents(); it.hasNext();) {
				DagNode lastNodeParent = (DagNode) it.next();
				lastNodeParent.deleteChild(lastNode);
			}
			lastNode.deleteAllParents();
			lastNode.addParent(curNode);

			XATNode[] children = new XATNode[1];
			children[0] = lastNode;
			curNode.setChildren(children);

			lastNode = curNode;
		}

		while (curNode.getParentsSize() > 0
				&& !(curNode.getParent(0).getOperator() instanceof StructuralJoinStream)) {
			curNode = (DagNode) curNode.getParent(0);
		}
		if (curNode.getParentsSize() == 0) {
			//curNode is now the last one
			curNode.addParent(dest);
			dest.addChild(curNode);
		}

	}

	/*
	 * @param op XATStreamOperator @return String get the columns the operator
	 * generates
	 */
	static String getColumnGenerated(XATStreamOperator op) {
		String columnGenerated = "";

		if (op instanceof NavigateStream)
			columnGenerated = ((NavigateStream) op).getOutputColumnName();

		return columnGenerated;
	}

	/*
	 * @param List activeOps @param passiveOp XATStreamOperator @return boolean
	 * whether the activeOps conflict with the branch starting from passiveOp
	 */
	static boolean conflict(List activeOps, List passiveOps) {

		int activeOpNum = activeOps.size();

		for (int i = 0; i < activeOpNum; i++) {
			XATStreamOperator curActiveOp = (XATStreamOperator) activeOps
					.get(i);
			if (conflict(curActiveOp, passiveOps))
				return true;

		}

		return false;
	}

	/*
	 * @param op XATStreamOperator @return List get the columns the operator
	 * consumes
	 */
	static List getColumnsConsumed(XATStreamOperator op) {
		List columnsConsumed = new ArrayList();

		if (op instanceof NavigateStream)
			columnsConsumed.add(((NavigateStream) op).getEntryPoint()[0]
					.getStepValue());

		else if (op instanceof SelectStream)
			//the select only defines on one variable
			columnsConsumed.addAll(((SelectStream) op).getExpression()
					.getTerminalColumnExprs());

		return columnsConsumed;
	}

	/*
	 * check whether the activeOp conflicts with any node on passiveOps i.e.,
	 * node on passiveOp is in the form of NavUnnest_{\$a, p}\$b where the
	 * active node consumes \$a @param activeOp XATStreamOperator activeOp
	 * @param passiveOps List
	 */
	static boolean conflict(XATStreamOperator activeOp, List passiveOps) {

		List columnsConsumed = getColumnsConsumed(activeOp);

		int passiveOpNum = passiveOps.size();
		for (int i = 0; i < passiveOpNum; i++) {
			XATStreamOperator passiveOp = (XATStreamOperator) passiveOps.get(i);
			if (passiveOp instanceof NavUnnestStream) {
				String entryName = ((NavUnnestStream) passiveOp)
						.getEntryPoint()[0].getStepValue();
				for (int j = 0; j < columnsConsumed.size(); j++)
					//if the navunnest's entry column is going to be consumed
					//then activeOp cannot be mereged with the passive branch
					if (columnsConsumed.get(j).equals(entryName))
						return true;
			}

		}

		return false;

	}

	/*
	 * @param tokenNavNode DagNode a tokenNavNode that must have multiple
	 * parents @return StructuralJoinStream the fan in strucutral join stream
	 * operator
	 */
	static StructuralJoinStream getFanInSj(DagNode node) {
		XATStreamOperator op = (XATStreamOperator)node.getOperator();
		
		DagNode curParentNode = (DagNode) node.getParent(0);
		XATStreamOperator curParent = (XATStreamOperator) curParentNode
				.getOperator();
		while (curParent != null) {
			if (curParent instanceof StructuralJoinStream){
				if (op instanceof PureNavigateAutoStream&& (((StructuralJoinStream) curParent).getBranchNavOp() == op))
				return (StructuralJoinStream) curParent;
				else if (!(op instanceof PureNavigateAutoStream))
					return (StructuralJoinStream)curParent;	
			}

			if (curParentNode.getParentsSize()>0){
			curParentNode = (DagNode) curParentNode.getParent();
			curParent = (XATStreamOperator) curParentNode.getOperator();
			}else
				break;
		}

		return null;
	}

	/*
	 * @param tokenNavNode DagNode @return the structural join that confines the
	 * to-be-moved tokenNavNode
	 */
	static StructuralJoinStream getConfineSj(DagNode tokenNavNode) {
		StructuralJoinStream confinedSj = null;

		InTimeSJStream sj;
		DagNode sjNode;
		String joinOn;

		PureNavigateAutoStream tokenNav = (PureNavigateAutoStream) tokenNavNode
				.getOperator();

		if (tokenNavNode.getParentsSize() > 1) {
			StructuralJoinStream fanInSj = (StructuralJoinStream) RuntimeOptUtil.getTokenNavToFanInSj()
					.get(tokenNavNode.getOperator());
			sj = (InTimeSJStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(fanInSj);
			joinOn = tokenNav.getOutputColumnName();
		} else {
			sj = (InTimeSJStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(tokenNavNode.getParent(0).getOperator());
			joinOn = tokenNav.getEntryPoint()[0].getStepValue();
		}

		while (sj != null) {
			sjNode = (DagNode) sj.getNode();
			//find whether there are sibling branches that can have multiple
			// outputs
			//within sj
			/*List sortedBranches = ((SjToBranchStaMapContent) RuntimeOptUtil.sjToBranchStatistics
					.get(sj)).getSortedBranchStas();
			int branchNum = sortedBranches.size();*/
			List entryOps = sj.getEntryOps();
			int entryOpNum = entryOps.size();

			for (int i = 0; i < entryOpNum; i++) {
				/*BranchStatistics branchSta = (BranchStatistics) sortedBranches
						.get(i);
				DagNode curNode = ((DagNode)branchSta.getEndingPoints().get(0)).getParent(0);*/
				XATStreamOperator curOp = (XATStreamOperator) entryOps.get(i);
				DagNode curNode = (DagNode)curOp.getNode();
				while (curNode != sjNode) {
					//extractbyunnest, navunnest, structural join duplciate the
					// entry point, i.e., joinOn
					if (((curOp instanceof ExtractByUnnestStream) && (((ExtractByUnnestStream) curOp)
							.getEntryPoint()[0].getStepValue().equals(joinOn)))
							|| ((curOp instanceof NavUnnestStream) && (((NavUnnestStream) curOp)
									.getEntryPoint()[0].equals(joinOn)))
							|| (curOp instanceof StructuralJoinStream)) {
						confinedSj = sj;
						return confinedSj;
					} else {
						curNode = curNode.getParent(0);
						curOp = (XATStreamOperator) curNode.getOperator();
					}

				}
			}

			//go up to the next sj, its corresponding fan out tokenna
			//is the child of the current fan out tokennav
			sj = (InTimeSJStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(sjNode.getOperator());
			tokenNavNode = (DagNode) tokenNavNode.getChild(0);
			tokenNav = (PureNavigateAutoStream) tokenNavNode.getOperator();
			joinOn = tokenNav.getEntryPoint()[0].getStepValue();

		}

		return confinedSj;

	}

	/*
	 * @param DagNode tokenNavNode @return get the sj that confines tokenNavNode
	 * as well as its sibling
	 */
	static StructuralJoinStream getCommonConfineSj(DagNode tokenNavNode) {
		DagNode childTokenNavNode = (DagNode) tokenNavNode.getChild(0);

		Iterator siblingNodes = childTokenNavNode.getParents();

		StructuralJoinStream commonSj = null;
		for (Iterator it = siblingNodes; it.hasNext();) {
			DagNode sibling = (DagNode) it.next();
			StructuralJoinStream curSj = getConfineSj(sibling);

			List curSjAncestors = FindOpVisitor.findAncestorOps((DagNode) curSj
					.getNode());
			if (!curSjAncestors.contains(commonSj))
				//commonsj is not ancestor of curSj
				commonSj = curSj;
		}

		return commonSj;

	}

	/*
	 * @param sortedResults List the list of sorted simulateMoveOut @param
	 * newResult SimulateMoveOutResult insert the newResult into the appropriate
	 * place in the list the earlier, the cost cut moving out of the operator
	 * leads to, in the order of descending cost cut
	 */
	static void insertIntoSortedList(List sortedResults,
			SimulateMoveResult newResult) {
		int num = sortedResults.size();
		/*if (num == 0) {
			sortedResults.add(newResult);
			return;
		}*/
		

		double newCostCut = newResult.getCostCut();
		for (int i = 0; i < num; i++) {
			SimulateMoveResult result = (SimulateMoveResult) sortedResults
					.get(i);
			double curCostCut = result.getCostCut();
			if (newCostCut > curCostCut) {
				sortedResults.add(i, newResult);
				return;
			}
		}

		//everything in the sortedResutls has a smaller cost than the newResult
		//insert the newResult at the end
		sortedResults.add(newResult);
	}

	/*
	 *@param curNav XATStreamOperator
	 */
	static boolean needRecompute(XATStreamOperator curNav,
			XATStreamOperator movedNav) {
		//movedTokenNavNode must be now associated with a navigate operator
		//we find the old tokenNav operator it is originally with
		if (!(curNav instanceof PureNavigateAutoStream))
			curNav = (NavigateStream)RuntimeOptUtil.nodeNavToTokenNav.get(curNav);
		StructuralJoinStream confiningSjForCurNav = (StructuralJoinStream) RuntimeOptUtil.getTokenNavToConfiningSj()
		.get(curNav);
		PureNavigateAutoStream branchForCurNav = confiningSjForCurNav.getBranchNavOp();
        DagNode branchForCurNavNode = (DagNode)branchForCurNav.getNode();
		
		if (!(movedNav instanceof PureNavigateAutoStream))
				movedNav = (NavigateStream)RuntimeOptUtil.nodeNavToTokenNav.get(movedNav);
		StructuralJoinStream confiningSjForMovedNav = (StructuralJoinStream) RuntimeOptUtil.getTokenNavToConfiningSj()
		.get(movedNav);
		PureNavigateAutoStream branchForMovedNav = confiningSjForMovedNav.getBranchNavOp();
        DagNode branchForMovedNavNode = (DagNode)branchForMovedNav.getNode();
		
		//if branchForMovedNav is branchForCurNav's ancestor or the other way
		//then need recompute
		
		if (ancesContain(branchForMovedNavNode, branchForCurNavNode)
				|| ancesContain(branchForCurNavNode, branchForMovedNavNode))
			return true;
		else return false;		
		
	}


	/*
	 * @param tokenNav XATStreamOperator 
	 * @param targetTokenNav XATStreamOperator
	 * @return whether the targetTokenNav is an ancestor of tokenNav
	 */
	static private boolean ancesContain(DagNode tokenNavNode, DagNode targetTokenNavNode){
		if (tokenNavNode == targetTokenNavNode)
			return true;
		
		XATStreamOperator targetTokenNav = (XATStreamOperator)targetTokenNavNode.getOperator();		
				
		for (Iterator it = tokenNavNode.getParents(); it.hasNext();){
			DagNode parentNode = (DagNode)it.next();
			XATStreamOperator parentOp = (XATStreamOperator)parentNode.getOperator();
			if (parentOp instanceof PureNavigateAutoStream){
				if (parentOp == targetTokenNav)
					return true;
				else
					return ancesContain(parentNode, targetTokenNavNode);
			}				
		} 
		
		return false;		
		
	}
	
	/*
	 * @param tokenNav PureNavigteAutoStream @return boolean whether its
	 * confining sj = input sj
	 */
	static boolean confineSjSameAsInputSj(PureNavigateAutoStream tokenNav) {
		StructuralJoinStream confiningSj = (StructuralJoinStream) RuntimeOptUtil.getTokenNavToConfiningSj()
				.get(tokenNav);
		StructuralJoinStream inputSj = getInputSj(tokenNav);
		if (confiningSj == inputSj)
			return true;
		else
			return false;  

	}  

	/*
	 * @param tokenNav PureNavigateAutoStream @return StructuralJoinStream to
	 * which tokenNav lies on an input branch
	 */
	static StructuralJoinStream getInputSj(PureNavigateAutoStream tokenNav) {
		//                            structuraljoin1
		//                         / \
		//                  structuraljoin2 |
		//                    / \ |
		//                    | extract2 |
		//                     \ \ |
		//                extract1 tokenNav2 extract
		//                      \ / |
		//                    tokenNav1 tokenNav
		//                             \ /
		//                              tokenNav
		// tokenNav1's input sj is structural join 1
		//tokennav2's input sj is structural join 2, but the starting point is
		// extract2

		DagNode tokenNavNode = (DagNode) tokenNav.getNode();
		StructuralJoinStream inputSj = null;
		if (tokenNavNode.getParentsSize() == 1)
			inputSj = (StructuralJoinStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(tokenNavNode.getParent(0).getOperator());
		else
			inputSj = (StructuralJoinStream) RuntimeOptUtil.getEntryOpToInputSj()
					.get(tokenNav);

		return inputSj;
	}

	/*
	 * @param tokenNode DagNode @return XATStreamOperator find the op of the
	 * starting point if tokenNode has multiple parents, then itself identifies
	 * a starting point otherwise its parent extract identifies the starting
	 * point
	 */
	public static XATStreamOperator getEntryOpFromTokenNav(DagNode tokenNavNode) {
		if (tokenNavNode.getParentsSize() == 1)
			return (XATStreamOperator) tokenNavNode.getParent(0).getOperator();
		else {
			XATStreamOperator fanInSj = (XATStreamOperator) RuntimeOptUtil.getTokenNavToFanInSj()
					.get(tokenNavNode.getOperator());
			return fanInSj;
		}

	}

	/*
	 * @param nodeNav NavigateStream @return XATStreamOperator return the entry
	 * operator that identifies an input branch nodeNav lies on
	 */
	public static XATStreamOperator getEntryOpFromNodeNav(DagNode nodeNavNode) {
		DagNode entryNode = nodeNavNode;
		XATStreamOperator entryOp = (XATStreamOperator) nodeNavNode
				.getOperator();
		while (entryOp instanceof ExtractStream
				|| entryOp instanceof StructuralJoinStream) {
			entryNode = (DagNode) entryNode.getChild(0);
			entryOp = (XATStreamOperator) entryNode.getOperator();
		}

		return entryOp;

	}

}