/*
 * Created on Dec 28, 2004n
 *
 * This class implements an algorithm that
 * performs an one-time optimization to find the optimal plan
 * assumption: the environment is rather stable
 * the optimal plan should work for all time
 *
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automatainout;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.lang.Math;


import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.InTimeSJStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.SelectStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.StructuralJoinStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.TaggerStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.XMLScanStream;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.AffectedPart;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.AffectedSj;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.SpecifiedNavPulloutRewriteRule;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.SpecifiedNavPushinRewriteRule;
import edu.wpi.cs.dsrg.xmldb.xat.component.statistics.StatisticsCollector;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.DagTraversal;

/**
 * @author suhong
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class RuntimeOptUtil {

	static XATDag inputPlan;

	static DagNode sourceTokenNavNode;

	//give a structural join operator, get the statistics about its input
	// branches
	public static Map sjToBranchStatistics = new HashMap();

	//give a tokenNav or an extract operator which identifies a branch
	// find the statistics of the branch
	// the node must have a single parent
	//otherwise it cannot identify a branch
	private static Map branchToStatistics = new HashMap();

	//given an operator, find its statistics
	private static Map opToStatistics = new HashMap();

	//a tokenNav which has multiple parent branches (the parents are its fan
	// out)
	//maps to a structural join operator that all fan out from tokenNav fans in
	private static Map tokenNavToFanInSj = new HashMap();

	//given a token nav, find the structural join
	//the token nav is a starting point of an input branch to the structural
	// join
	private static Map entryOpToInputSj = new HashMap();

	private static Map tokenNavToConfiningSj = new HashMap();

	//given a node nav, find out which token nav it is mapped from
	public static Map nodeNavToTokenNav = new HashMap();

	//given a tokenNav,find the node nav it turns into
	public static Map tokenNavToNodeNav = new HashMap();  

	//given a tokenNav, find its extract nav, it does not necessarily have one
	public static Map tokenNavToExtract = new HashMap();

	//the copy of the final plan
	static CopiedDag finalPlan;

	static public void setInputPlan(XATDag newInputPlan) {
		inputPlan = newInputPlan;
	}

	static public Map getBranchToStatistics() {
		return branchToStatistics;
	}
	
	static public Map getOpToStatistics(){
		return opToStatistics;
	}
	
	static public Map getEntryOpToInputSj(){
		return entryOpToInputSj;
	}
	
	static public Map getTokenNavToConfiningSj(){
		return tokenNavToConfiningSj;
	}
	
	static public Map getTokenNavToFanInSj(){
		return tokenNavToFanInSj;
	}

	static public void initialize(XATDag originalPlan) {
		inputPlan = originalPlan;
		ContinuousOpt.tokenNavsInPlan.clear();

		//first, set up the tokenNav
		//must be bottom up traverse because the later program assumes the
		// first node
		//in the list to be the bottommost one
		List allNodes = DagTraversal.dagBottomUpTraverse(originalPlan);
		int nodeNum = allNodes.size();
		for (int i = 0; i < nodeNum; i++) {
			DagNode node = (DagNode) allNodes.get(i);
			XATStreamOperator op = (XATStreamOperator) node.getOperator();
			if (op instanceof PureNavigateAutoStream) {
				ContinuousOpt.tokenNavsInPlan.add(op);
				StatisticsCollector
						.initializeTokenNavtoStartCount((PureNavigateAutoStream) op);
				StatisticsCollector
						.initializeTokenNavtoActivateCount((PureNavigateAutoStream) op);
			}

			if (!(op instanceof XMLScanStream) && !(op instanceof TaggerStream))
				StatisticsCollector.initializeOpSta(op);
		}

		//second, set up
		// tokenNavToConfiningSj/tokenNavToFanInSj/startingPointToInputSj
		int tokenNavNum = ContinuousOpt.tokenNavsInPlan.size();
		for (int i = 0; i < tokenNavNum; i++) {
			PureNavigateAutoStream tokenNav = (PureNavigateAutoStream) ContinuousOpt.tokenNavsInPlan
					.get(i);
			DagNode tokenNavNode = (DagNode) tokenNav.getNode();

			if (tokenNavNode.getChild(0).getOperator() instanceof XMLScanStream)
				sourceTokenNavNode = tokenNavNode;

			ExtractStream parentExtract = null;
			StructuralJoinStream fanInSj = null;
			DagNode fanInSjNode = null;

			if (tokenNavNode.getParentsSize() > 1) {
				//should set its fanin sj
				fanInSj = PlanManipulator
						.getFanInSj(tokenNavNode);
				fanInSjNode = (DagNode) fanInSj.getNode();
				tokenNavToFanInSj.put(tokenNav, fanInSj);

				for (Iterator it = tokenNavNode.getParents(); it.hasNext();) {
					DagNode curParentTokenNavNode = (DagNode) it.next();
					XATStreamOperator curParentTokenNav = (XATStreamOperator) curParentTokenNavNode
							.getOperator();
					if (curParentTokenNav instanceof ExtractStream) {
						//it is a starting point
						entryOpToInputSj.put(curParentTokenNav, fanInSj);

						parentExtract = (ExtractStream) curParentTokenNav;
						//set up tokenNavToExtractNav
						tokenNavToExtract.put(tokenNav, curParentTokenNav);
						break;
					}
				}					
				//also set up entryOpToInputSj for fanInSj
				StructuralJoinStream inputSjofFanInSj = PlanManipulator.getFanInSj(fanInSjNode);
				entryOpToInputSj.put(fanInSj, inputSjofFanInSj);
			} 
				else {

				//find the input structural join
				DagNode extractNode = tokenNavNode.getParent(0);
				DagNode inputSjNode = PlanManipulator
						.findNearestSJ((XATStreamOperator) extractNode
								.getOperator());
				if (inputSjNode != null) {
					XATStreamOperator inputSj = (StructuralJoinStream) inputSjNode
							.getOperator();
					entryOpToInputSj.put(extractNode.getOperator(), inputSj);

					//we also find the input sj of the parentSj
				/*	if (entryOpToInputSj.get(fanInSj) == null) {
						if (fanInSj.getBranchNavOp() != ContinuousOpt.tokenNavsInPlan
								.get(0)) {
							StructuralJoinStream inputSj = (StructuralJoinStream) PlanManipulator
									.findNearestSJ(fanInSj).getOperator();
							entryOpToInputSj.put(fanarentSj, inputSj);
						}
					}*/

				}
			}
			

			//set up tokenNavToConfiningSj
			StructuralJoinStream confiningSj = PlanManipulator.getConfineSj(tokenNavNode);
			tokenNavToConfiningSj.put(tokenNav, confiningSj);
		}  

		StatisticsCollector.setBottommostTokenNav((PureNavigateAutoStream) ContinuousOpt.tokenNavsInPlan
						.get(0));
	}

	/*
	 * @param oldResult SimulateMoveResult the plan in which a tokenNav will be
	 * pulled out @param bestResult the best result so far compute how much cost
	 * gain the plan is compared against the best plan
	 *  
	 */
	static public void estimateMoveOut(SimulateMoveResult oldResult) {
		double overallCostCut = 0;
		CopiedDag tempPlan = oldResult.getTempPlan();
		DagNode movedNodeInTemp = oldResult.getMovedNodeInTemp();
		PureNavigateAutoStream nav = (PureNavigateAutoStream) movedNodeInTemp
				.getOperator();
		XATStreamOperator movedBranchEntryOp = PlanManipulator
				.getEntryOpFromTokenNav(movedNodeInTemp);
		BranchStatistics removedBranchSta = (BranchStatistics) branchToStatistics
				.get(movedBranchEntryOp);

		moveOut(oldResult);
		AffectedPart affectedPart = oldResult.getDeltaAffectedPart();

		List affectedSjs = affectedPart.getAffectedSjs();
		int affectedSjNum = affectedSjs.size();

		double costCut = 0;
		for (int i = 0; i < affectedSjNum; i++) {
			AffectedSj affectedSj = (AffectedSj) affectedSjs.get(i);
			InTimeSJStream sj = (InTimeSJStream) affectedSj.getSj();
			
			Map curSjToBranchStatistics = oldResult.getAffectedPart().
			getNewSjToBranchStatistics();
			SjToBranchStaMapContent curSjToBranchStaMapContent = (SjToBranchStaMapContent)curSjToBranchStatistics.get(sj);

			List sortedBranchStas = null;
			if (curSjToBranchStaMapContent!=null)
				sortedBranchStas = curSjToBranchStaMapContent.getSortedBranchStas();
			else
				sortedBranchStas = ((SjToBranchStaMapContent)sjToBranchStatistics.get(sj)).getSortedBranchStas();

			XATStreamOperator entryOp = affectedSj.getEntryOp();
			BranchStatistics beforeBranchSta = null;
			if (affectedSj.isSjRemoved())
				beforeBranchSta = (BranchStatistics)branchToStatistics.get(RuntimeOptUtil.entryOpToInputSj.get(entryOp));
			else
				beforeBranchSta = (BranchStatistics) branchToStatistics
					.get(entryOp);
					
			
			if (beforeBranchSta == null)
				//unit cost =0, sel = 1, probNotEmpty = 1
				beforeBranchSta = new BranchStatistics(null, null, 1, 0, 1);
			BranchStatistics afterBranchSta = CostComp
					.computeBranchStatistics(tempPlan.getNode(entryOp));

			costCut = searchSortedBranchStaForMoveOut(oldResult, affectedSj,
					nav, sortedBranchStas, beforeBranchSta, afterBranchSta,
					removedBranchSta);

			overallCostCut += costCut;
		}

		//overallCostCut also includes the cost of the structural joins that
		// have been removed
		List removedSjs = affectedPart.getRemovedSjs();
		int removedSjNum = removedSjs.size();
		for (int i = 0; i < removedSjNum; i++) {
			StructuralJoinStream removedSj = (StructuralJoinStream) removedSjs
					.get(i);
			List sortedBranchSta = ((SjToBranchStaMapContent) sjToBranchStatistics
					.get(removedSj)).getSortedBranchStas();
			int removedBranchNum = sortedBranchSta.size();
			for (int j = 0; j < removedBranchNum; j++) {
				BranchStatistics removedBranch = (BranchStatistics) sortedBranchSta
						.get(j);
				DagNode entryNode = removedBranch.getEntryNode();
				XATStreamOperator entryOp = (XATStreamOperator) entryNode
						.getOperator();
				OpStatistics extractOpSta = (OpStatistics) opToStatistics
						.get(entryOp);
				overallCostCut += extractOpSta.getCost();

				if (entryOp instanceof ExtractStream) {
					DagNode tokenNavNode = (DagNode) entryNode.getChild(0);
					OpStatistics tokenNavSta = (OpStatistics) opToStatistics
							.get(tokenNavNode.getOperator());
					overallCostCut += tokenNavSta.getCost();
				}
			}
		}

		//current gain over the best plan = current plan - last plan + last
		// plan - best plan
		oldResult.setCostCut(overallCostCut + oldResult.getCostCut());

	}

	/*
	 * @param nodeNav NavigateStream move in the operator @param tempPlan the
	 * plan on which the nodeNav is pushed in @return SimulateMoveOutResult a
	 * new plan with nodeNav pushed in
	 */
	static void estimateMoveIn(SimulateMoveResult oldResult) {

		double overallCostCut = 0;
		CopiedDag tempPlan = oldResult.getTempPlan();
		DagNode movedNodeInTemp = oldResult.getMovedNodeInTemp();
		NavigateStream nodeNav = (NavigateStream) movedNodeInTemp
				.getOperator();
		
		moveIn(oldResult);
		AffectedPart affectedPart = oldResult.getAffectedPart();

		XATStreamOperator addedBranchEntryOp = null;
		BranchStatistics addedBranchSta = null;
		List addedSjs = affectedPart.getAddedSjs();
		int addedSjNum = addedSjs.size();
		if (addedSjNum > 0) {
			//new sjs are created
			addedBranchEntryOp = (StructuralJoinStream) addedSjs.get(0);
			addedBranchSta = (BranchStatistics) branchToStatistics
					.get(addedBranchEntryOp);
		} else {
			//no sjs are created
			addedBranchEntryOp = PlanManipulator
					.getEntryOpFromTokenNav(movedNodeInTemp);
			addedBranchSta = (BranchStatistics) branchToStatistics
					.get(addedBranchEntryOp);
		}

		List affectedSjs = affectedPart.getAffectedSjs();
		int affectedSjNum = affectedSjs.size();

		for (int i = 0; i < affectedSjNum; i++) {
			AffectedSj affectedSj = (AffectedSj) affectedSjs.get(i);
			InTimeSJStream sj = (InTimeSJStream) affectedSj.getSj();

			List sortedBranchStas = ((SjToBranchStaMapContent) sjToBranchStatistics
					.get(sj)).getSortedBranchStas();
			//the children size of the new sj
			//a branch is moved

			XATStreamOperator entryOp = affectedSj.getEntryOp();

			BranchStatistics beforeBranchSta = (BranchStatistics) branchToStatistics
					.get(entryOp);

			BranchStatistics afterBranchSta = null;
			if (!affectedSj.isEntryOpRemoved())
				afterBranchSta = CostComp.computeBranchStatistics(tempPlan
						.getNode(entryOp));

			double costCut = searchSortedBranchStaForMoveIn(oldResult,
					affectedSj, nodeNav, sortedBranchStas, beforeBranchSta,
					afterBranchSta, addedBranchSta);

			overallCostCut += costCut;
		}

		//overallCostCut also includes the cost of the structural joins that
		// have been added
		for (int i = 0; i < addedSjNum; i++) {
			StructuralJoinStream addedSj = (StructuralJoinStream) addedSjs
					.get(i);
			List sortedBranchSta = ((SjToBranchStaMapContent) sjToBranchStatistics
					.get(addedSj)).getSortedBranchStas();
			int addedBranchNum = sortedBranchSta.size();
			for (int j = 0; j < addedBranchNum; j++) {
				BranchStatistics addedBranch = (BranchStatistics) sortedBranchSta
						.get(j);
				DagNode entryNode = addedBranch.getEntryNode();
				XATStreamOperator entryOp = (XATStreamOperator) entryNode
						.getOperator();
				OpStatistics extractOpSta = (OpStatistics) opToStatistics
						.get(entryOp);
				overallCostCut -= extractOpSta.getCost();

				if (entryOp instanceof ExtractStream) {
					DagNode tokenNavNode = (DagNode) entryNode.getChild(0);
					OpStatistics tokenNavSta = (OpStatistics) opToStatistics
							.get(tokenNavNode.getOperator());
					overallCostCut -= tokenNavSta.getCost();
				}
			}
		}

		//		current gain over the best plan = current plan - last plan + last
		// plan - best plan
		oldResult.setCostCut(overallCostCut + oldResult.getCostCut());

	}

	/*
	 * @param sortedBranchStas List a list of branches in the original plan
	 * @param beforeRank double the old rank of a modified branch when
	 * NEGAINFINITY, a newly inserted branch @param afterRank double the new
	 * rank of a modified branch @param removedBranch BranchStatistics the
	 * branch statistics of the removed branch @param moveOut boolean whether
	 * this corresponds to be move-out ar an move-in @param oldEvalOrder the old
	 * evaluation order of input subplans to the affected sj @param newEvalOrder
	 * the new evaluation order of input subplans to the affected sj the
	 * newEvalOrder will be filled in this function
	 *  
	 */
	static double searchSortedBranchStaForMoveOut(SimulateMoveResult oldResult,
			AffectedSj affectedSj, PureNavigateAutoStream tokenNav,
			List sortedBranchStas, BranchStatistics beforeBranchSta,
			BranchStatistics afterBranchSta, BranchStatistics removedBranchSta) {
        List evalOrder = new ArrayList();
		
		double costCut = 0;
		double beforeBranchRank = beforeBranchSta.getRankVal();
		double beforeBranchProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		double afterBranchRank = afterBranchSta.getRankVal();
		double afterBranchProbNotEmpty = afterBranchSta.getProbNotEmpty();
		double removedBranchRank = removedBranchSta.getRankVal();
		double removedBranchProbNotEmpty = removedBranchSta.getProbNotEmpty();
		XATStreamOperator removedBranchEntry = (XATStreamOperator) removedBranchSta
				.getEntryNode().getOperator();
		double movedNavProbNotEmpty = 0;

		//tokenNav op is pulled out, the subplan between it and its fan in sj
		// is moved out
		StructuralJoinStream sj = (StructuralJoinStream) tokenNavToFanInSj
				.get(tokenNav);
		OpStatistics opSta = null;
		if (sj == null)
			opSta = (OpStatistics) opToStatistics.get(tokenNav);
		else
			opSta = (OpStatistics) opToStatistics.get(sj);
		movedNavProbNotEmpty = opSta.getProbNotEmpty();

		//the smaller rank between removedBranchRank and beforeRank
		double lowerBetweenRemovedAndBefore = removedBranchRank;
		double higherBetweenRemovedAndBefore = beforeBranchRank;
		boolean removedBranchLower = true;
		if (removedBranchRank > beforeBranchRank) {
			lowerBetweenRemovedAndBefore = beforeBranchRank;
			higherBetweenRemovedAndBefore = removedBranchRank;
			removedBranchLower = false;
		}

		double factor = CostComp.INFINITY;

		if (movedNavProbNotEmpty != 0)
			factor = 1 / movedNavProbNotEmpty;

		double overallProbBranchNotEmpty = 1;
		double overallSel = CostComp.getOverallSelFromInputSJ(affectedSj
				.getSj());
		double localCostCut = 0;
		boolean afterBranchInserted = false;
		int modifiedBranchPosSorted = -1;
		int removedBranchPosSorted = -1;

		List newSortedBranchStas = new ArrayList();
		int size = sortedBranchStas.size();
		//the position of the removed branch

		for (int i = 0; i < size; i++) {
			//visit old and new sortedBranchStas synchornizely

			BranchStatistics branchSta = (BranchStatistics) sortedBranchStas
					.get(i);
			BranchStatistics newBranchSta = (BranchStatistics) branchSta
					.clone();
			DagNode entryNode = branchSta.getEntryNode();

			double curBranchRank = branchSta.getRankVal();
			double branchCost = branchSta.getCost();

			if (branchSta.getEntryNode().getOperator() == removedBranchSta.getEntryNode().getOperator() || 
					(beforeBranchSta.getEntryNode()!=null &&
					branchSta.getEntryNode().getOperator() == beforeBranchSta.getEntryNode().getOperator())) {
				localCostCut = branchCost;
				//also remove the removed branch from the newSortedBranchStas
				if (branchSta == beforeBranchSta)
					modifiedBranchPosSorted = i;
				
				if (branchSta == removedBranchSta)
					removedBranchPosSorted = i;
				
				costCut += localCostCut;

				continue;
			}

			//adjust the new evaluation order
			newSortedBranchStas.add(newBranchSta);
			evalOrder.addAll(newBranchSta.getEndingOps());

			if (factor == CostComp.INFINITY) {
				//movedNavProbNotEmpty =0
				//before branchSta.cost = 0
				localCostCut = branchSta.getUnitCost()
						* overallProbBranchNotEmpty * overallSel;
			} else if (modifiedBranchPosSorted == -1 && removedBranchPosSorted == -1) {
				//the current branch is evaluated before both removed and before branches
				if (curBranchRank <= afterBranchRank)
					//\sigma_{op \not\Rightarrow \emptyset}
					localCostCut = (1 - factor) * branchCost;
				else {
					//\sigma_{newbranch \not\Rightarrow \emptyset}
					localCostCut = (1 - factor * afterBranchProbNotEmpty)
							* branchCost;
					//curBranchRank>=afterBranchRankVal
					if (!afterBranchInserted)
						newSortedBranchStas.add(newSortedBranchStas.size() - 1,
							afterBranchSta);
					evalOrder.addAll(afterBranchSta.getEndingOps());
					afterBranchInserted = true;

					//also compute the cost
					afterBranchSta.setCost(afterBranchSta.getUnitCost()
							* overallProbBranchNotEmpty * overallSel);
				}
			} else if ((modifiedBranchPosSorted == -1 && removedBranchPosSorted != -1)
					|| (modifiedBranchPosSorted!=-1 && removedBranchPosSorted == -1)) {
				//this branch is evaluated in between
				if (curBranchRank <= afterBranchRank)
					if (removedBranchLower)
						localCostCut = (1 - factor / removedBranchProbNotEmpty)
								* branchCost;
					else
						localCostCut = (1 - factor / beforeBranchProbNotEmpty)
								* branchCost;
				else {
					//curBranchRank>afterBranchRankVal
					//readjust the position of the changed branch
					if (!afterBranchInserted)
						newSortedBranchStas.add(newSortedBranchStas.size() - 1,
							afterBranchSta);
					evalOrder.addAll(afterBranchSta.getEndingOps());
					afterBranchInserted = true;

					//also compute the cost
					afterBranchSta.setCost(afterBranchSta.getUnitCost()
							* overallProbBranchNotEmpty * overallSel);

					if (removedBranchLower)
						localCostCut = (1 - factor * afterBranchProbNotEmpty
								/ removedBranchProbNotEmpty)
								* branchCost;
					else
						localCostCut = (1 - factor * afterBranchProbNotEmpty
								/ beforeBranchProbNotEmpty)
								* branchCost;
				}
			} else if (modifiedBranchPosSorted != -1 && removedBranchPosSorted!=-1) {
				//evaluated after both
				if (curBranchRank <= afterBranchRank)
					localCostCut = (1 - factor / removedBranchProbNotEmpty
							/ beforeBranchProbNotEmpty)
							* branchCost;
				else {
					localCostCut = (1 - factor * afterBranchProbNotEmpty
							/ removedBranchProbNotEmpty
							/ afterBranchProbNotEmpty)
							* branchCost;

					if (!afterBranchInserted)
						newSortedBranchStas.add(newSortedBranchStas.size() - 1,
							afterBranchSta);
					evalOrder.addAll(afterBranchSta.getEndingOps());

					afterBranchInserted = true;

					//also compute the cost
					afterBranchSta.setCost(afterBranchSta.getUnitCost()
							* overallProbBranchNotEmpty * overallSel);
				}
			}

			//update the cost of these branches previously in the old plan
			newBranchSta.setCost(branchSta.getCost() - localCostCut);
			overallProbBranchNotEmpty = overallProbBranchNotEmpty
					* newBranchSta.getProbNotEmpty();
			costCut += localCostCut;

		}

		if (!afterBranchInserted) {
			//also compute the cost
			afterBranchSta.setCost(afterBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(afterBranchSta);
			evalOrder.addAll(afterBranchSta.getEndingOps());
		}

		//		now substract the cost of the modified new branch
		costCut -= afterBranchSta.getCost();
		
		//also add the cost of the removed tokenNav
		OpStatistics tokenNavSta = (OpStatistics)opToStatistics.get(tokenNav);
		costCut += tokenNavSta.getCost();

		//update the deltaAffectedPart
		Map newSjToBranchStatistics = new HashMap();
		SjToBranchStaMapContent sjToBranchStaMapContent = new SjToBranchStaMapContent(
				newSortedBranchStas, evalOrder);
		newSjToBranchStatistics
				.put(affectedSj.getSj(), sjToBranchStaMapContent);
		oldResult.getDeltaAffectedPart().setNewSjToBranchStatistics(
				newSjToBranchStatistics);

		return costCut;
	}

	/*
	 * @param sortedBranchStas List a list of branches in the original plan
	 * @param beforeRank double the old rank of a modified branch when
	 * NEGAINFINITY, a newly inserted branch @param afterRank double the new
	 * rank of a modified branch @param removedBranch BranchStatistics the
	 * branch statistics of the removed branch @param moveOut boolean whether
	 * this corresponds to be move-out ar an move-in @param oldEvalOrder the old
	 * evaluation order of input subplans to the affected sj @param newEvalOrder
	 * the new evaluation order of input subplans to the affected sj the
	 * newEvalOrder will be filled in this function
	 *  
	 */
	/*static double searchSortedBranchStaForMoveIn(SimulateMoveResult oldResult,
			AffectedSj affectedSj, NavigateStream nodeNav,
			List sortedBranchStas, BranchStatistics beforeBranchSta,
			BranchStatistics afterBranchSta, BranchStatistics addedBranchSta,
			List oldEvalOrder, List newEvalOrder) {

		double costCut = 0;
		double beforeBranchRank = beforeBranchSta.getRankVal();
		double beforeBranchProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		double afterBranchRank = CostComp.INFINITY;
		double afterBranchProbNotEmpty = 1;
		double higherProbNotEmpty = 1;
		if (afterBranchSta != null) {
			afterBranchRank = afterBranchSta.getRankVal();
			afterBranchProbNotEmpty = afterBranchSta.getProbNotEmpty();
			higherProbNotEmpty = afterBranchSta.getProbNotEmpty();
		}
		double addedBranchRank = addedBranchSta.getRankVal();
		double addedBranchProbNotEmpty = addedBranchSta.getProbNotEmpty();
		XATStreamOperator addedBranchEntry = (XATStreamOperator) addedBranchSta
				.getEntryNode().getOperator();

		double movedNavProbNotEmpty = 0;
		//tokenNav op is pulled out, the subplan between it and its fan in sj
		// is moved out
		StructuralJoinStream sj = (StructuralJoinStream) tokenNavToFanInSj
				.get(nodeNav);
		OpStatistics opSta = null;
		if (sj == null)
			opSta = (OpStatistics) opToStatistics.get(nodeNav);
		else
			opSta = (OpStatistics) opToStatistics.get(sj);
		movedNavProbNotEmpty = opSta.getProbNotEmpty();
		double factor = movedNavProbNotEmpty;

		//the smaller rank between removedBranchRank and beforeRank
		double lowerBetweenAddedAndAfter = addedBranchRank;
		double higherBetweenAddedAndAfter = afterBranchRank;
		double lowerProbNotEmpty = addedBranchSta.getProbNotEmpty();
		double beforeProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		boolean addedLower = true;
		if (addedBranchRank > afterBranchRank) {
			lowerBetweenAddedAndAfter = afterBranchRank;
			higherBetweenAddedAndAfter = addedBranchRank;
			lowerProbNotEmpty = afterBranchSta.getProbNotEmpty();
			higherProbNotEmpty = addedBranchSta.getProbNotEmpty();
			addedLower = false;
		}

		double overallProbBranchNotEmpty = 1;
		double overallSel = CostComp.getOverallSelFromInputSJ(affectedSj
				.getSj());
		double localCostCut = 0;
		int modifiedBranchPos = -1;
		int removedBranchPos = -1;

		List newSortedBranchStas = new ArrayList();
		int size = sortedBranchStas.size();

		boolean afterBranchInserted = false;
		boolean addedBranchInserted = false;
		//the position of the removed branch

		//6 cases:
		// a branch: before beforeSta before both afterSta and addedSta
		//                             between
		//                             after both
		//           after beforeSta before both afterSta and addedSta
		//                             between
		//                             after both

		for (int i = 0; i < size; i++) {
			//visit old and new sortedBranchStas synchornizely

			BranchStatistics branchSta = (BranchStatistics) sortedBranchStas
					.get(i);
			BranchStatistics newBranchSta = (BranchStatistics) branchSta
					.clone();
			DagNode entryNode = branchSta.getEntryNode();

			double curBranchRank = branchSta.getRankVal();
			double branchCost = branchSta.getCost();

			if (branchSta == beforeBranchSta && afterBranchSta == null) {
				removedBranchPos = i;
				//branchSta has been removed
				localCostCut = branchCost;
				//also remove the removed branch from the newSortedBranchStas
				removedBranchPos = ((Integer) oldEvalOrder.get(i)).intValue();
				costCut += localCostCut;

				continue;
			}

			//adjust the new evaluation order
			int pos = ((Integer) oldEvalOrder.get(i)).intValue();
			if (pos > removedBranchPos)
				newEvalOrder.add(new Integer(pos - 1));
			else
				newEvalOrder.add(new Integer(pos));
			newSortedBranchStas.add(newBranchSta);

			if (curBranchRank < beforeBranchRank) {
				if (curBranchRank < lowerBetweenAddedAndAfter)
					//\sigma_{op \not\Rightarrow \emptyset}
					localCostCut = (1 - factor) * branchCost;
				else {
					if (lowerBetweenAddedAndAfter < curBranchRank
							&& curBranchRank < higherBetweenAddedAndAfter) {
						//\sigma_{newbranch \not\Rightarrow \emptyset}
						localCostCut = (1 - factor * lowerProbNotEmpty)
								* branchCost;
						if (addedLower) {
							newSortedBranchStas.add(addedBranchSta);
							addedBranchInserted = true;
							if (afterBranchSta == null)
								//the new branch is added at the last
								newEvalOrder.add(new Integer(sortedBranchStas
										.size()));
							else
								newEvalOrder.add(new Integer(sortedBranchStas
										.size() + 1));
						} else {
							//is after the modified branch
							if (afterBranchSta != null) {
								afterBranchInserted = true;
								newSortedBranchStas.add(afterBranchSta);
								newEvalOrder.add(new Integer(removedBranchPos));
							}
						}
					} else
						//curBranch > both
						localCostCut = (1 - factor * lowerProbNotEmpty
								* higherProbNotEmpty)
								* branchCost;

				}
			} else {
				//curBranchRank > beforeBranchRank
				if (curBranchRank < lowerBetweenAddedAndAfter)
					localCostCut = (1 - factor / beforeProbNotEmpty)
							* branchCost;
				else {
					if (lowerBetweenAddedAndAfter < curBranchRank
							&& curBranchRank < higherBetweenAddedAndAfter) {
						localCostCut = (1 - factor * lowerProbNotEmpty
								/ beforeProbNotEmpty)
								* branchCost;
						if (addedLower) {
							addedBranchInserted = true;
							newSortedBranchStas.add(addedBranchSta);

							//also compute the cost
							addedBranchSta.setCost(addedBranchSta.getUnitCost()
									* overallProbBranchNotEmpty * overallSel);
							if (afterBranchSta == null)
								//the new branch is added at the last
								newEvalOrder.add(new Integer(sortedBranchStas
										.size()));
							else
								newEvalOrder.add(new Integer(sortedBranchStas
										.size() + 1));
						} else {
							//is after the modified branch
							if (afterBranchSta != null) {
								afterBranchInserted = true;
								newSortedBranchStas.add(afterBranchSta);
								newEvalOrder.add(new Integer(removedBranchPos));

								//also compute the cost
								afterBranchSta.setCost(afterBranchSta
										.getUnitCost()
										* overallProbBranchNotEmpty
										* overallSel);
							}
						}
					} else
						//curBranch>both
						localCostCut = (1 - factor * lowerProbNotEmpty
								* higherProbNotEmpty / beforeProbNotEmpty)
								* branchCost;

				}

			}
			//			update the cost of these branches previously in the old plan
			newBranchSta.setCost(branchSta.getCost() - localCostCut);
			overallProbBranchNotEmpty = overallProbBranchNotEmpty
					* newBranchSta.getProbNotEmpty();
			costCut += localCostCut;
		}

		if (!afterBranchInserted && afterBranchSta != null) {
			//also compute the cost
			afterBranchSta.setCost(afterBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(afterBranchSta);
			newEvalOrder.add(new Integer(newSortedBranchStas.size() - 1));
		}

		if (!addedBranchInserted) {
			//			also compute the cost
			addedBranchSta.setCost(addedBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(addedBranchSta);
			newEvalOrder.add(new Integer(newSortedBranchStas.size() - 1));

		}

		//		now add the cost of the modified new branch
		if (afterBranchSta != null)
			costCut = costCut - afterBranchSta.getCost();

		//update the deltaAffectedPart
		Map newSjToBranchStatistics = new HashMap();
		SjToBranchStaMapContent sjToBranchStaMapContent = new SjToBranchStaMapContent(
				newSortedBranchStas, newEvalOrder);
		newSjToBranchStatistics
				.put(affectedSj.getSj(), sjToBranchStaMapContent);
		oldResult.getDeltaAffectedPart().setNewSjToBranchStatistics(
				newSjToBranchStatistics);

		return costCut;
	}*/
	
	/*static double searchSortedBranchStaForMoveIn(SimulateMoveResult oldResult,
			AffectedSj affectedSj, NavigateStream nodeNav,
			List sortedBranchStas, BranchStatistics beforeBranchSta,
			BranchStatistics afterBranchSta, BranchStatistics addedBranchSta,
			List newChildren) {

		double costCut = 0;
		double beforeBranchRank = beforeBranchSta.getRankVal();
		double beforeBranchProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		double afterBranchRank = CostComp.INFINITY;
		double afterBranchProbNotEmpty = 1;
		double higherProbNotEmpty = 1;
		if (afterBranchSta != null) {
			afterBranchRank = afterBranchSta.getRankVal();
			afterBranchProbNotEmpty = afterBranchSta.getProbNotEmpty();
			higherProbNotEmpty = afterBranchSta.getProbNotEmpty();
		}
		double addedBranchRank = addedBranchSta.getRankVal();
		double addedBranchProbNotEmpty = addedBranchSta.getProbNotEmpty();
		XATStreamOperator addedBranchEntry = (XATStreamOperator) addedBranchSta
				.getEntryNode().getOperator();

		double movedNavProbNotEmpty = 0;
		//tokenNav op is pulled out, the subplan between it and its fan in sj
		// is moved out
		StructuralJoinStream sj = (StructuralJoinStream) tokenNavToFanInSj
				.get(nodeNav);
		OpStatistics opSta = null;
		if (sj == null)
			opSta = (OpStatistics) opToStatistics.get(nodeNav);
		else
			opSta = (OpStatistics) opToStatistics.get(sj);
		movedNavProbNotEmpty = opSta.getProbNotEmpty();
		double factor = movedNavProbNotEmpty;

		//the smaller rank between removedBranchRank and beforeRank
		double lowerBetweenAddedAndAfter = addedBranchRank;
		double higherBetweenAddedAndAfter = afterBranchRank;
		double lowerProbNotEmpty = addedBranchSta.getProbNotEmpty();
		double beforeProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		boolean addedLower = true;
		if (addedBranchRank > afterBranchRank) {
			lowerBetweenAddedAndAfter = afterBranchRank;
			higherBetweenAddedAndAfter = addedBranchRank;
			lowerProbNotEmpty = afterBranchSta.getProbNotEmpty();
			higherProbNotEmpty = addedBranchSta.getProbNotEmpty();
			addedLower = false;
		}

		double overallProbBranchNotEmpty = 1;
		double overallSel = CostComp.getOverallSelFromInputSJ(affectedSj
				.getSj());
		double localCostCut = 0;
		int modifiedBranchPos = -1;
		int removedBranchPos = -1;

		List newSortedBranchStas = new ArrayList();
		int size = sortedBranchStas.size();

		boolean afterBranchInserted = false;
		boolean addedBranchInserted = false;
		//the position of the removed branch

		//6 cases:
		// a branch: before beforeSta before both afterSta and addedSta
		//                             between
		//                             after both
		//           after beforeSta before both afterSta and addedSta
		//                             between
		//                             after both

		for (int i = 0; i < size; i++) {
			//visit old and new sortedBranchStas synchornizely

			BranchStatistics branchSta = (BranchStatistics) sortedBranchStas
					.get(i);
			BranchStatistics newBranchSta = (BranchStatistics) branchSta
					.clone();
			DagNode entryNode = branchSta.getEntryNode();

			double curBranchRank = branchSta.getRankVal();
			double branchCost = branchSta.getCost();

			if (branchSta == beforeBranchSta && afterBranchSta == null) {
				removedBranchPos = i;
				//branchSta has been removed
				localCostCut = branchCost;
				costCut += localCostCut;

				continue;
			}

			newSortedBranchStas.add(newBranchSta);
			newChildren.add(newBranchSta.getEndingPoint());

			if (curBranchRank < beforeBranchRank) {
				if (curBranchRank < lowerBetweenAddedAndAfter)
					//\sigma_{op \not\Rightarrow \emptyset}
					localCostCut = (1 - factor) * branchCost;
				else {
					if (lowerBetweenAddedAndAfter < curBranchRank
							&& curBranchRank < higherBetweenAddedAndAfter) {
						//\sigma_{newbranch \not\Rightarrow \emptyset}
						localCostCut = (1 - factor * lowerProbNotEmpty)
								* branchCost;
						if (addedLower) {
							newSortedBranchStas.add(addedBranchSta);
							addedBranchInserted = true;
						} else {
							//is after the modified branch
							if (afterBranchSta != null) {
								afterBranchInserted = true;
								newSortedBranchStas.add(afterBranchSta);
								newChildren.add(newBranchSta.getEndingPoint());
							}
						}
					} else
						//curBranch > both
						localCostCut = (1 - factor * lowerProbNotEmpty
								* higherProbNotEmpty)
								* branchCost;

				}
			} else {
				//curBranchRank > beforeBranchRank
				if (curBranchRank < lowerBetweenAddedAndAfter)
					localCostCut = (1 - factor / beforeProbNotEmpty)
							* branchCost;
				else {
					if (lowerBetweenAddedAndAfter < curBranchRank
							&& curBranchRank < higherBetweenAddedAndAfter) {
						localCostCut = (1 - factor * lowerProbNotEmpty
								/ beforeProbNotEmpty)
								* branchCost;
						if (addedLower) {
							addedBranchInserted = true;
							newSortedBranchStas.add(addedBranchSta);

							//also compute the cost
							addedBranchSta.setCost(addedBranchSta.getUnitCost()
									* overallProbBranchNotEmpty * overallSel);							
						} else {
							//is after the modified branch
							if (afterBranchSta != null) {
								afterBranchInserted = true;
								newSortedBranchStas.add(afterBranchSta);
								newChildren.add(newBranchSta.getEndingPoint());;

								//also compute the cost
								afterBranchSta.setCost(afterBranchSta
										.getUnitCost()
										* overallProbBranchNotEmpty
										* overallSel);
							}
						}
					} else
						//curBranch>both
						localCostCut = (1 - factor * lowerProbNotEmpty
								* higherProbNotEmpty / beforeProbNotEmpty)
								* branchCost;

				}

			}
			//			update the cost of these branches previously in the old plan
			newBranchSta.setCost(branchSta.getCost() - localCostCut);
			overallProbBranchNotEmpty = overallProbBranchNotEmpty
					* newBranchSta.getProbNotEmpty();
			costCut += localCostCut;
		}

		if (!afterBranchInserted && afterBranchSta != null) {
			//also compute the cost
			afterBranchSta.setCost(afterBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(afterBranchSta);
			newChildren.add(afterBranchSta.getEndingPoint());
		}

		if (!addedBranchInserted) {
			//			also compute the cost
			addedBranchSta.setCost(addedBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(addedBranchSta);
			newChildren.add(addedBranchSta.getEndingPoint());

		}

		//		now add the cost of the modified new branch
		if (afterBranchSta != null)
			costCut = costCut - afterBranchSta.getCost();

		//update the deltaAffectedPart
		Map newSjToBranchStatistics = new HashMap();
		SjToBranchStaMapContent sjToBranchStaMapContent = new SjToBranchStaMapContent(
				newSortedBranchStas, newChildren);
		newSjToBranchStatistics
				.put(affectedSj.getSj(), sjToBranchStaMapContent);
		oldResult.getDeltaAffectedPart().setNewSjToBranchStatistics(
				newSjToBranchStatistics);

		return costCut;
	}*/
	
	static double searchSortedBranchStaForMoveIn(SimulateMoveResult oldResult,
			AffectedSj affectedSj, NavigateStream nodeNav,
			List sortedBranchStas, BranchStatistics beforeBranchSta,
			BranchStatistics afterBranchSta, BranchStatistics addedBranchSta) {
        List evalOrder = new ArrayList();
		
		double costCut = 0;
		double beforeBranchRank = beforeBranchSta.getRankVal();
		double beforeBranchProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		double afterBranchRank = CostComp.INFINITY;
		double afterBranchProbNotEmpty = 1;
		double higherProbNotEmpty = 1;
		if (afterBranchSta != null) {
			afterBranchRank = afterBranchSta.getRankVal();
			afterBranchProbNotEmpty = afterBranchSta.getProbNotEmpty();
			higherProbNotEmpty = afterBranchSta.getProbNotEmpty();
		}
		double addedBranchRank = addedBranchSta.getRankVal();
		double addedBranchProbNotEmpty = addedBranchSta.getProbNotEmpty();
		XATStreamOperator addedBranchEntry = (XATStreamOperator) addedBranchSta
				.getEntryNode().getOperator();

		String nodeNavEntry = nodeNav.getEntryPoint()[0].getStepValue();
		double factor = 1;
		//nodeNav op is pushed in, any operators it relies on are all pushed in
		List addedTokenNavs = oldResult.getAffectedPart().getAddedTokenNavs();
		XATStreamOperator beforeBranchEntryOp = (XATStreamOperator)beforeBranchSta.getEntryNode().getOperator();
		if (beforeBranchEntryOp instanceof ExtractStream)
			//no navs that nodeNav depends on are outside before
			factor = getSimpleMoveInFactor(addedTokenNavs);
		else
			//must be an SJoin
			factor = getComplexMoveInFactor(addedTokenNavs, (InTimeSJStream)beforeBranchEntryOp);
		         
		//the smaller rank between removedBranchRank and beforeRank
		double lowerBetweenAddedAndAfter = addedBranchRank;
		double higherBetweenAddedAndAfter = afterBranchRank;
		double lowerProbNotEmpty = addedBranchSta.getProbNotEmpty();
		double beforeProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		boolean addedLower = true;
		if (addedBranchRank > afterBranchRank) {
			lowerBetweenAddedAndAfter = afterBranchRank;
			higherBetweenAddedAndAfter = addedBranchRank;
			lowerProbNotEmpty = afterBranchSta.getProbNotEmpty();
			higherProbNotEmpty = addedBranchSta.getProbNotEmpty();
			addedLower = false;
		}

		double overallProbBranchNotEmpty = 1;
		double overallSel = CostComp.getOverallSelFromInputSJ(affectedSj
				.getSj());
		double localCostCut = 0;
		int modifiedBranchPos = -1;
		int removedBranchPos = -1;

		List newSortedBranchStas = new ArrayList();
		int size = sortedBranchStas.size();

		boolean afterBranchInserted = false;
		boolean addedBranchInserted = false;
		//the position of the removed branch

		//6 cases:
		// a branch: before beforeSta before both afterSta and addedSta
		//                             between
		//                             after both
		//           after beforeSta before both afterSta and addedSta
		//                             between
		//                             after both

		for (int i = 0; i < size; i++) {
			//visit old and new sortedBranchStas synchornizely

			BranchStatistics branchSta = (BranchStatistics) sortedBranchStas
					.get(i);
			BranchStatistics newBranchSta = (BranchStatistics) branchSta
					.clone();
			DagNode entryNode = branchSta.getEntryNode();

			double curBranchRank = branchSta.getRankVal();
			double branchCost = branchSta.getCost();

			if (branchSta.getEntryNode().getOperator() == beforeBranchSta.getEntryNode().getOperator() && afterBranchSta == null) {
				removedBranchPos = i;
				//branchSta has been removed
				localCostCut = branchCost;
				costCut += localCostCut;

				continue;
			}

			newSortedBranchStas.add(newBranchSta);
			List newBranchEndingPoints = newBranchSta.getEndingPoints();
			for (int j=0; j<newBranchEndingPoints.size(); j++)
				evalOrder.add(((DagNode)newBranchEndingPoints.get(j)).getOperator());

			if (curBranchRank < beforeBranchRank) {
				if (curBranchRank < lowerBetweenAddedAndAfter)
					//\sigma_{op \not\Rightarrow \emptyset}
					localCostCut = (1 - factor) * branchCost;
				else {
					if (lowerBetweenAddedAndAfter < curBranchRank
							&& curBranchRank < higherBetweenAddedAndAfter) {
						//\sigma_{newbranch \not\Rightarrow \emptyset}
						localCostCut = (1 - factor * lowerProbNotEmpty)
								* branchCost;
						if (addedLower) {
							newSortedBranchStas.add(addedBranchSta);
							addedBranchInserted = true;
						} else {
							//is after the modified branch
							if (afterBranchSta != null) {
								afterBranchInserted = true;
								newSortedBranchStas.add(afterBranchSta);
								evalOrder.addAll(newBranchSta.getEndingOps());
							}
						}
					} else
						//curBranch > both
						localCostCut = (1 - factor * lowerProbNotEmpty
								* higherProbNotEmpty)
								* branchCost;

				}
			} else {
				//curBranchRank > beforeBranchRank
				if (curBranchRank < lowerBetweenAddedAndAfter)
					localCostCut = (1 - factor / beforeProbNotEmpty)
							* branchCost;
				else {
					if (lowerBetweenAddedAndAfter < curBranchRank
							&& curBranchRank < higherBetweenAddedAndAfter) {
						localCostCut = (1 - factor * lowerProbNotEmpty
								/ beforeProbNotEmpty)
								* branchCost;
						if (addedLower) {
							addedBranchInserted = true;
							newSortedBranchStas.add(addedBranchSta);

							//also compute the cost
							addedBranchSta.setCost(addedBranchSta.getUnitCost()
									* overallProbBranchNotEmpty * overallSel);							
						} else {
							//is after the modified branch
							if (afterBranchSta != null) {
								afterBranchInserted = true;
								newSortedBranchStas.add(afterBranchSta);
								evalOrder.addAll(newBranchSta.getEndingOps());

								//also compute the cost
								afterBranchSta.setCost(afterBranchSta
										.getUnitCost()
										* overallProbBranchNotEmpty
										* overallSel);
							}
						}
					} else
						//curBranch>both
						localCostCut = (1 - factor * lowerProbNotEmpty
								* higherProbNotEmpty / beforeProbNotEmpty)
								* branchCost;

				}

			}
			//			update the cost of these branches previously in the old plan
			newBranchSta.setCost(branchSta.getCost() - localCostCut);
			overallProbBranchNotEmpty = overallProbBranchNotEmpty
					* newBranchSta.getProbNotEmpty();
			costCut += localCostCut;
		}

		if (!afterBranchInserted && afterBranchSta != null) {
			//also compute the cost
			afterBranchSta.setCost(afterBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(afterBranchSta);
			evalOrder.addAll(afterBranchSta.getEndingOps());
		}

		if (!addedBranchInserted) {
			//			also compute the cost
			addedBranchSta.setCost(addedBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(addedBranchSta);
			evalOrder.addAll(addedBranchSta.getEndingOps());

		}

		//		now substract the cost of the modified new branch
		if (afterBranchSta != null)
			costCut -= afterBranchSta.getCost();
		//now substract the cost of the new tokenNav
		PureNavigateAutoStream tokenNav = (PureNavigateAutoStream)oldResult.getMovedNodeInTemp().getOperator();
		OpStatistics tokenNavSta = (OpStatistics)opToStatistics.get(tokenNav);
		costCut -= tokenNavSta.getCost();
		

		//update the deltaAffectedPart
		Map newSjToBranchStatistics = new HashMap();
		SjToBranchStaMapContent sjToBranchStaMapContent = new SjToBranchStaMapContent(
				newSortedBranchStas, evalOrder);
		newSjToBranchStatistics
				.put(affectedSj.getSj(), sjToBranchStaMapContent);
		oldResult.getDeltaAffectedPart().setNewSjToBranchStatistics(
				newSjToBranchStatistics);

		return costCut;
	}
	
	static private double getSimpleMoveInFactor(List addedTokenNavs){
		int addedTokenNavNum = addedTokenNavs.size();
		double factor = 0;
		for (int i = addedTokenNavNum-1; i >=0; i--){
			PureNavigateAutoStream addedTokenNav = (PureNavigateAutoStream)addedTokenNavs.get(i);
			OpStatistics opSta = (OpStatistics)opToStatistics.get(addedTokenNav);
			double probNotEmpty = opSta.getProbNotEmpty();
			double sel = opSta.getSel();
			if (i==addedTokenNavNum-1){
				if (addedTokenNavNum>1)
					factor = 1 - probNotEmpty;
				else
					factor = probNotEmpty;
			}
			else 
				factor = 1 - java.lang.Math.pow(factor, sel);
		}		
		
		return factor;
		
	}
	
	static private double getComplexMoveInFactor(List addedTokenNavs, InTimeSJStream sj){
		double probNotEmpty = 1;
		
		double deltaProbNotEmpty = getSimpleMoveInFactor(addedTokenNavs);

        List entryOps = sj.getEntryOps();
        int entryOpNum = entryOps.size();
        
        PureNavigateAutoStream tokenNav = sj.getBranchNavOp();
        OpStatistics opSta = (OpStatistics)opToStatistics.get(tokenNav);
        double sel = opSta.getSel();
        
        for (int i=0; i<entryOpNum; i++){
        	XATStreamOperator entryOp = (XATStreamOperator)entryOps.get(i);
        	BranchStatistics branchSta = (BranchStatistics)branchToStatistics.get(entryOp);
        	probNotEmpty *= branchSta.getProbNotEmpty();
        }
        
        double factor = (1 - Math.pow(1 - probNotEmpty, sel))/(1 - Math.pow(1 - probNotEmpty*deltaProbNotEmpty, sel));
        
        return factor;
		
	}

	/*
	 * @double get the cost of the navigate operator that tokenNav will be
	 * converted into @param tokenNav PureNavigateAutoStream the tokenNav
	 * operator @param collector StatisticsCollector
	 */
	static double getUnitCost(PureNavigateAutoStream tokenNav,
			StatisticsCollector collector) {
		double unitCost = 0;

		unitCost = ((Integer) StatisticsCollector.tokenNavtoStartCountSum
				.get(tokenNav)).intValue()
				* StatisticsCollector.NODENAV_VISIT_COST;

		return unitCost;

	}

	/*
	 * move out the tokenNav, @PureNavigateAuotStream tokenNav @return
	 * BranchStatistics, i.e., the branch that is affected by the moving out of
	 * the tokenNav i.e., the branch that the moved-out operator is on object[0] =
	 * starting point identifying a branch object[1] = selectivity of the
	 * starting point object[2] = cost of the branch object[3] = probability of
	 * the branch yields something object[4] = rank value
	 *  
	 */
	static void moveOut(SimulateMoveResult oldResult) {
		moveOut(oldResult, oldResult.getMovedNodeInTemp());
	}

	/*
	 * move out the tokenNav, @PureNavigateAuotStream tokenNav @return
	 * BranchStatistics, i.e., the branch that is affected by the moving out of
	 * the tokenNav i.e., the branch that the moved-out operator is on object[0] =
	 * starting point identifying a branch object[1] = selectivity of the
	 * starting point object[2] = cost of the branch object[3] = probability of
	 * the branch yields something object[4] = rank value
	 *  
	 */
	static void moveOut(SimulateMoveResult oldResult, DagNode tokenNavNode) {

		CopiedDag tempPlan = oldResult.getTempPlan();
		PureNavigateAutoStream tokenNav = (PureNavigateAutoStream) tokenNavNode
				.getOperator();
		DagNode tokenNavChildNode = (DagNode) tokenNavNode.getChild(0);
		XATStreamOperator tokenNavChild = (XATStreamOperator) tokenNavChildNode
				.getOperator();

		//apply rewrite rule to push nodeNav down
		//we start from applying rule on any nodeNav depended on by the one
		// that is to be moved out
		//e.g., NavNest_{\$b, //profile}\$e
		//                    |
		//      NavUnnest_{\$a, /seller}\$b
		//if nodeNav = NavNest_{\$b, //profile}\$e,
		// NavUnnest_{\$a, /seller}\$b will be pushed in first
		//and then NavNest_{\$b, //profile}\$e is pushed in
		//now find the list of nodeNav operators that are dependent on

		SpecifiedNavPulloutRewriteRule rewriter = new SpecifiedNavPulloutRewriteRule(
				oldResult, tokenNav);
		AffectedPart affectedPart = (AffectedPart) rewriter.visit(tokenNav,
				tokenNavChild);
		oldResult.setDeltaAffectedPart(affectedPart);

	}

	static void moveIn(SimulateMoveResult oldResult) {
		moveIn(oldResult, oldResult.getMovedNodeInTemp());
	}

	/*
	 * move in the nodeNav, @param nodeNav NavigateStream the nodeNav operator
	 * to be pushed into @param tempPlan CopiedDag the temporary plan to push in
	 * the nodeNav @param existingScript ChangeScript the already adopted script
	 * of modifying the original plan
	 *  
	 */
	static void moveIn(SimulateMoveResult oldResult, DagNode nodeNavNode) {

		NavigateStream nodeNav = (NavigateStream) nodeNavNode.getOperator();
		CopiedDag tempPlan = oldResult.getTempPlan();
		DagNode nodeNavChildNode = (DagNode) nodeNavNode.getChild(0);
		XATStreamOperator nodeNavChild = (XATStreamOperator) nodeNavChildNode
				.getOperator();

		//apply rewrite rule to push nodeNav down
		//we start from applying rule on any nodeNav depended on by the one
		// that is to be moved out
		//e.g., NavNest_{\$b, //profile}\$e
		//                    |
		//      NavUnnest_{\$a, /seller}\$b
		//if nodeNav = NavNest_{\$b, //profile}\$e,
		// NavUnnest_{\$a, /seller}\$b will be pushed in first
		//and then NavNest_{\$b, //profile}\$e is pushed in
		//now find the list of nodeNav operators that are dependent on

		SpecifiedNavPushinRewriteRule rewriter = new SpecifiedNavPushinRewriteRule(
				oldResult, nodeNav);
		AffectedPart affectedPart = (AffectedPart) rewriter.visit(nodeNav,
				nodeNavChild);
		oldResult.setAffectedPart(affectedPart);

	}

	/*
	 * @param tokenNavNode DagNode the node identifes a branch @return int the
	 * probability of this branch would generate some result within the scope of
	 * the structural join assume operators on this branch all have data
	 * dependency for example, navnest_{a, p1} b1 has no data dependency with
	 * navnest_{a, p2}b2
	 */
	static double computeNotEmptyProb(DagNode tokenNavNode) {
		DagNode nodeOnBranch = tokenNavNode;
		XATStreamOperator opOnBranch = (XATStreamOperator) nodeOnBranch
				.getOperator();

		double notEmptyProb = 1;
		double sel = 1;

		while (!(opOnBranch instanceof StructuralJoinStream)) {
			if (opOnBranch instanceof ExtractStream
					|| opOnBranch instanceof NavigateStream) {
				OpStatistics opOnBranchSta = (OpStatistics) opToStatistics
						.get(opOnBranch);
				//find how many tuples are there be, i.e., the selectivity
				sel = sel * opOnBranchSta.getSel();
			} else if (opOnBranch instanceof SelectStream) {
				OpStatistics opOnBranchSta = (OpStatistics) opToStatistics
						.get(opOnBranch);
				double localNotEmptyProb = opOnBranchSta.getProbNotEmpty();
				double localSel = opOnBranchSta.getSel();
				//notEmptyProbe = 1 - (1 - localNotEmptyProb)^{localSel}
				notEmptyProb = 1 - Math.pow(1 - localNotEmptyProb, localSel);
			}

			nodeOnBranch = nodeOnBranch.getParent(0);
			opOnBranch = (XATStreamOperator) nodeOnBranch.getOperator();

		}
		return notEmptyProb;

	}

	/*
	 * @param movedTokenNavNode DagNode the node to be moved out where
	 * structuraljoin_{1} not= structuraljoin_{2} @return double if the operator
	 * is moved out, the probablity the moved out part would not generate empty
	 * result within structuraljoin_{2}
	 */
	static double getProbNotEmptyForMovedOutNode(DagNode movedTokenNavNode) {

		double probNotEmpty = 1;

		PureNavigateAutoStream movedTokenNav = (PureNavigateAutoStream) movedTokenNavNode
				.getOperator();

		OpStatistics opSta = (OpStatistics) opToStatistics.get(movedTokenNav);
		double localProbNotEmpty = opSta.getProbNotEmpty();

		/*
		 * StructuralJoin_{$b} / \ ExtractNest_{$b}$g ExtractNest_{$b}$h | |
		 * TokenNav_{$b, /name}$g TokenNav_{$b, /address}$h \ / TokenNav_{$a,
		 * //seller} $b
		 */

		if (movedTokenNavNode.getParentsSize() == 1) {
			//move out this operator has no side effect
			//e.g., TokenNav_{$b, /address}$h
			DagNode childNode = (DagNode) movedTokenNavNode.getChild(0);
			OpStatistics childOpSta = (OpStatistics) opToStatistics
					.get(childNode.getOperator());
			double sel = childOpSta.getSel();

			probNotEmpty = 1 - java.lang.Math.pow(1 - localProbNotEmpty, sel);
		} else {
			//move out this operator has side effect,
			//i.e., all children patterns have to be moved out as well
			//e.g., TokenNav_{$a, //seller}$b

			StructuralJoinStream sj = (StructuralJoinStream) tokenNavToFanInSj
					.get(movedTokenNav);
			OpStatistics sjSta = (OpStatistics) opToStatistics.get(sj);

			probNotEmpty = 1 - java.lang.Math.pow(1 - sjSta.getProbNotEmpty(),
					opSta.getSel());

		}
		return probNotEmpty;
	}

	/*
	 * @param DagNode structuralJoinNode @double the prob of the subplan
	 * identified by the structuraljoin node not generating empty result within
	 * an output tuple of the fan in node
	 */
	static double computeProbNotEmptyForSj(DagNode structuralJoinNode) {
		double probNotEmpty = 1;

		List sortedBranches = ((SjToBranchStaMapContent) sjToBranchStatistics
				.get(structuralJoinNode.getOperator())).getSortedBranchStas();
		int branchNum = sortedBranches.size();

		for (int i = 0; i < branchNum; i++) {
			BranchStatistics branchSta = (BranchStatistics) sortedBranches
					.get(i);

			double branchProbNotEmpty = branchSta.getProbNotEmpty();

			probNotEmpty = probNotEmpty * branchProbNotEmpty;
		}

		return probNotEmpty;
	}

	/*
	 * @param navs List
	 * @param firstTime boolean whether this is the first time reasoning
	 *the difference is that the first time, navs is the complete list of all the tokenNavs in the plan
	 */
	static List initializeOpsNeedRecompute(List navs, SimulateMoveResult bestResult) {
		List opsNeedRecompute = new ArrayList();

		int tokenNavNum = navs.size();

		for (int i = 0; i < tokenNavNum; i++) {
			//can be either tokenNav or nodeNav
			NavigateStream nav = (NavigateStream) navs.get(i);
			DagNode navNode = (DagNode) nav.getNode();
			CopiedDag copyOfInput = new CopiedDag(inputPlan);
			SimulateMoveResult result = new SimulateMoveResult(copyOfInput
					.getNewNodeFromOldNode(navNode), 0, copyOfInput);
			opsNeedRecompute.add(result);
			if (bestResult!=null)
				result.setTokenNavsInPlan(bestResult.getTokenNavsInPlan());
			else
				result.setTokenNavsInPlan(navs);
		}

		return opsNeedRecompute;
	}

	/*
	 * @param deltaAffectedPart AffectedPart the delta change over the oldResult
	 * incorporate these changes on branchToStatistics, entryOpToInputSj,
	 * tokenNavToFanInSj, tokenNavToFanInSj
	 *  
	 */
	static void adoptDeltaAffectedPart(AffectedPart deltaAffectedPart) {
		Map newSjToBranchStatistics = deltaAffectedPart
				.getNewSjToBranchStatistics();
		Set sjs = newSjToBranchStatistics.keySet();
		for (Iterator it = sjs.iterator(); it.hasNext();) {
			InTimeSJStream sj = (InTimeSJStream) it.next();
			SjToBranchStaMapContent sjToBranchStaMapContent = (SjToBranchStaMapContent) newSjToBranchStatistics
					.get(sj);
			sj.setBranchEvalOrder(sjToBranchStaMapContent.getEvalOrder());
		}

		sjToBranchStatistics.putAll(newSjToBranchStatistics);
		newSjToBranchStatistics.clear();
		
		Map newTokenNavToExtract = deltaAffectedPart.getNewTokenNavToExtract();
		tokenNavToExtract.putAll(newTokenNavToExtract);
		newTokenNavToExtract.clear();
		
		Map newEntryOpToInputSj = deltaAffectedPart.getNewEntryOpToInputSj();
		entryOpToInputSj.putAll(newEntryOpToInputSj);
		newEntryOpToInputSj.clear();
		
		Map newBranchToStatistics = deltaAffectedPart.getNewBranchToStatistics();
		branchToStatistics.putAll(newBranchToStatistics);
		newBranchToStatistics.clear();
		
		   
	}
	
	static public void cleanOpSta(NavigateStream nav){
		opToStatistics.remove(nav);
	}
}