/*
 * Created on Dec 28, 2004n
 *
 * This class implements an algorithm that
 * performs an one-time optimization to find the optimal plan
 * assumption: the environment is rather stable
 * the optimal plan should work for all time
 *
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.automatainout;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.lang.Math;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.InTimeSJStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.SelectStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.sqloperator.StructuralJoinStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.ExtractStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.NavigateStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.PureNavigateAutoStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.TaggerStream;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.xmloperator.XMLScanStream;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.AffectedPart;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.AffectedSj;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.SpecifiedNavPulloutRewriteRule;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.constructor.SpecifiedNavPushinRewriteRule;
import edu.wpi.cs.dsrg.xmldb.xat.component.statistics.StatisticsCollector;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.DagTraversal;

/**
 * @author suhong
 * 
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class RuntimeOptUtilBak {

	static XATDag inputPlan;

	static DagNode sourceTokenNavNode;

	//give a structural join operator, get the statistics about its input
	// branches
	public static Map sjToBranchStatistics = new HashMap();

	//give a tokenNav or an extract operator which identifies a branch
	// find the statistics of the branch
	// the node must have a single parent
	//otherwise it cannot identify a branch
	public static Map branchToStatistics = new HashMap();

	//given an operator, find its statistics
	public static Map opToStatistics = new HashMap();

	//a tokenNav which has multiple parent branches (the parents are its fan
	// out)
	//maps to a structural join operator that all fan out from tokenNav fans in
	public static Map tokenNavToFanInSj = new HashMap();

	//given a token nav, find the structural join
	//the token nav is a starting point of an input branch to the structural
	// join
	public static Map entryOpToInputSj = new HashMap();

	public static Map tokenNavToConfiningSj = new HashMap();

	//given a node nav, find out which token nav it is mapped from
	public static Map nodeNavToTokenNav = new HashMap();

	//given a tokenNav,find the node nav it turns into
	public static Map tokenNavToNodeNav = new HashMap();

	//given a tokenNav, find its extract nav, it does not necessarily have one
	public static Map tokenNavToExtract = new HashMap();

	//the copy of the final plan
	static CopiedDag finalPlan;

	static public void setInputPlan(XATDag newInputPlan) {
		inputPlan = newInputPlan;
	}

	static public void initialize(XATDag originalPlan) {
		inputPlan = originalPlan;
		ContinuousOpt.tokenNavsInPlan.clear();

		//first, set up the tokenNav
		//must be bottom up traverse because the later program assumes the
		// first node
		//in the list to be the bottommost one
		List allNodes = DagTraversal.dagBottomUpTraverse(originalPlan);
		int nodeNum = allNodes.size();
		for (int i = 0; i < nodeNum; i++) {
			DagNode node = (DagNode) allNodes.get(i);
			XATStreamOperator op = (XATStreamOperator) node.getOperator();
			if (op instanceof PureNavigateAutoStream) {
				ContinuousOpt.tokenNavsInPlan.add(op);
				StatisticsCollector
						.initializeTokenNavtoStartCount((PureNavigateAutoStream) op);
				StatisticsCollector
						.initializeTokenNavtoActivateCount((PureNavigateAutoStream) op);
			}

			if (!(op instanceof XMLScanStream) && !(op instanceof TaggerStream))
				StatisticsCollector.initializeOpSta(op);
		}

		//second, set up
		// tokenNavToConfiningSj/tokenNavToFanInSj/startingPointToInputSj
		int tokenNavNum = ContinuousOpt.tokenNavsInPlan.size();
		for (int i = 0; i < tokenNavNum; i++) {
			PureNavigateAutoStream tokenNav = (PureNavigateAutoStream) ContinuousOpt.tokenNavsInPlan
					.get(i);
			DagNode tokenNavNode = (DagNode) tokenNav.getNode();

			if (tokenNavNode.getChild(0).getOperator() instanceof XMLScanStream)
				sourceTokenNavNode = tokenNavNode;

			ExtractStream parentExtract = null;

			//set up tokenNavToConfiningSj
			StructuralJoinStream confiningSj = PlanManipulator
					.getConfineSj(tokenNavNode);
			tokenNavToConfiningSj.put(tokenNav, confiningSj);

			if (tokenNavNode.getParentsSize() > 1) {
				//should set its fanin sj
				StructuralJoinStream fanInSj = PlanManipulator
						.getFanInSj(tokenNavNode);
				DagNode fanInSjNode = (DagNode) fanInSj.getNode();
				tokenNavToFanInSj.put(tokenNav, fanInSj);

				for (Iterator it = tokenNavNode.getParents(); it.hasNext();) {
					DagNode curParentTokenNavNode = (DagNode) it.next();
					XATStreamOperator curParentTokenNav = (XATStreamOperator) curParentTokenNavNode
							.getOperator();
					if (curParentTokenNav instanceof ExtractStream) {
						//it is a starting point
						entryOpToInputSj.put(curParentTokenNav, fanInSj);

						parentExtract = (ExtractStream) curParentTokenNav;
						//set up tokenNavToExtractNav
						tokenNavToExtract.put(tokenNav, curParentTokenNav);
					}
				}
			} else {

				//find the input structural join
				DagNode extractNode = tokenNavNode.getParent(0);
				StructuralJoinStream parentSj = (StructuralJoinStream) PlanManipulator
						.findNearestSJ(
								(XATStreamOperator) extractNode.getOperator())
						.getOperator();
				entryOpToInputSj.put(extractNode.getOperator(), parentSj);

				//we also find the input sj of the parentSj
				if (entryOpToInputSj.get(parentSj) == null) {
					if (parentSj.getBranchNavOp() != ContinuousOpt.tokenNavsInPlan
							.get(0)) {
						StructuralJoinStream inputSj = (StructuralJoinStream) PlanManipulator
								.findNearestSJ(parentSj).getOperator();
						entryOpToInputSj.put(parentSj, inputSj);
					}
				}

			}
		}

		StatisticsCollector
				.setBottommostTokenNav((PureNavigateAutoStream) ContinuousOpt.tokenNavsInPlan
						.get(0));
	}

	/*
	 * @param oldResult SimulateMoveResult the plan in which a tokenNav will be
	 * pulled out @param bestResult the best result so far compute how much cost
	 * gain the plan is compared against the best plan
	 *  
	 */
	static public void estimateMoveOut(SimulateMoveResult oldResult,
			ChangeScript existingScript) {
		double overallCostCut = 0;
		CopiedDag tempPlan = oldResult.getTempPlan();
		DagNode movedNodeInTemp = oldResult.getMovedNodeInTemp();
		PureNavigateAutoStream nav = (PureNavigateAutoStream) movedNodeInTemp
				.getOperator();
		XATStreamOperator movedBranchEntryOp = PlanManipulator
				.getEntryOpFromTokenNav((PureNavigateAutoStream) movedNodeInTemp
						.getOperator());
		BranchStatistics removedBranchSta = (BranchStatistics) branchToStatistics
				.get(movedBranchEntryOp);

		moveOut(oldResult);
		AffectedPart affectedPart = oldResult.getDeltaAffectedPart();

		List affectedSjs = affectedPart.getAffectedSjs();
		int affectedSjNum = affectedSjs.size();

		double costCut = 0;
		for (int i = 0; i < affectedSjNum; i++) {
			AffectedSj affectedSj = (AffectedSj) affectedSjs.get(i);
			InTimeSJStream sj = (InTimeSJStream) affectedSj.getSj();

			List sortedBranchStas = ((SjToBranchStaMapContent) sjToBranchStatistics
					.get(sj)).getSortedBranchStas();
			List evalOrder = sj.getBranchEvalOrder();

			XATStreamOperator entryOp = affectedSj.getEntryOp();

			//rank of the subplan with entryOp before and after
			double beforeRank, afterRank;
			//when the entryOp is a newly added extract operator
			//beforeBranchSta is null
			BranchStatistics beforeBranchSta = (BranchStatistics) branchToStatistics
					.get(entryOp);
			if (beforeBranchSta == null) {
				//unit cost =0, sel = 1, probNotEmpty = 1
				beforeBranchSta = new BranchStatistics(null, null, 1, 0, 1);
				//remove one branch and add one branch, so size is the same

			}

			List newEvalOrder = new ArrayList();

			BranchStatistics afterBranchSta = CostComp
					.computeBranchStatistics(tempPlan.getNode(entryOp));

			costCut = searchSortedBranchSta(oldResult, affectedSj, nav,
					sortedBranchStas, beforeBranchSta, afterBranchSta,
					removedBranchSta, true, evalOrder, newEvalOrder);

			overallCostCut += costCut;
		}

		//overallCostCut also includes the cost of the structural joins that
		// have been removed
		List removedSjs = affectedPart.getRemovedSjs();
		int removedSjNum = removedSjs.size();
		for (int i = 0; i < removedSjNum; i++) {
			StructuralJoinStream removedSj = (StructuralJoinStream) removedSjs
					.get(i);
			List sortedBranchSta = ((SjToBranchStaMapContent) sjToBranchStatistics
					.get(removedSj)).getSortedBranchStas();
			int removedBranchNum = sortedBranchSta.size();
			for (int j = 0; j < removedBranchNum; j++) {
				BranchStatistics removedBranch = (BranchStatistics) sortedBranchSta
						.get(j);
				DagNode entryNode = removedBranch.getEntryNode();
				XATStreamOperator entryOp = (XATStreamOperator) entryNode
						.getOperator();
				OpStatistics extractOpSta = (OpStatistics) opToStatistics
						.get(entryOp);
				overallCostCut += extractOpSta.getCost();

				if (entryOp instanceof ExtractStream) {
					DagNode tokenNavNode = (DagNode) entryNode.getChild(0);
					OpStatistics tokenNavSta = (OpStatistics) opToStatistics
							.get(tokenNavNode.getOperator());
					overallCostCut += tokenNavSta.getCost();
				}
			}
		}

		//current gain over the best plan = current plan - last plan + last
		// plan - best plan
		oldResult.setCostCut(overallCostCut + oldResult.getCostCut());

	}

	/*
	 * @param nodeNav NavigateStream move in the operator @param tempPlan the
	 * plan on which the nodeNav is pushed in @return SimulateMoveOutResult a
	 * new plan with nodeNav pushed in
	 */
	static void estimateMoveIn(SimulateMoveResult oldResult,
			ChangeScript changeScript) {
		double overallCostCut = 0;
		CopiedDag tempPlan = oldResult.getTempPlan();
		DagNode movedNodeInTemp = oldResult.getMovedNodeInTemp();
		NavigateStream nav = (NavigateStream) movedNodeInTemp.getOperator();
		XATStreamOperator movedBranchEntryOp = PlanManipulator
				.getEntryOpFromNodeNav(nav);
		BranchStatistics removedBranchSta = (BranchStatistics) branchToStatistics
				.get(movedBranchEntryOp);

		moveIn(oldResult);
		AffectedPart affectedPart = oldResult.getAffectedPart();

		List affectedSjs = affectedPart.getAffectedSjs();
		int affectedSjNum = affectedSjs.size();

		for (int i = 0; i < affectedSjNum; i++) {
			AffectedSj affectedSj = (AffectedSj) affectedSjs.get(i);
			InTimeSJStream sj = (InTimeSJStream) affectedSj.getSj();

			List sortedBranchStas = (List) sjToBranchStatistics.get(sj);
			List evalOrder = sj.getBranchEvalOrder();
			//the children size of the new sj
			//a branch is moved

			XATStreamOperator entryOp = affectedSj.getEntryOp();

			//rank of the subplan with entryOp before and after
			double beforeRank, afterRank;
			BranchStatistics beforeBranchSta = (BranchStatistics) branchToStatistics
					.get(entryOp);
			BranchStatistics afterBranchSta = CostComp
					.computeBranchStatistics(tempPlan.getNode(entryOp));
			
            List newEvalOrder = new ArrayList();
			double costCut = searchSortedBranchSta(oldResult, affectedSj, nav,
					sortedBranchStas, beforeBranchSta, afterBranchSta,
					removedBranchSta, false, evalOrder, newEvalOrder);

			overallCostCut += costCut;
		}

		//		current gain over the best plan = current plan - last plan + last
		// plan - best plan
		oldResult.setCostCut(overallCostCut + oldResult.getCostCut());

	}

	/*
	 * @param sortedBranchStas List a list of branches in the original plan
	 * @param beforeRank double the old rank of a modified branch when
	 * NEGAINFINITY, a newly inserted branch @param afterRank double the new
	 * rank of a modified branch @param removedBranch BranchStatistics the
	 * branch statistics of the removed branch @param moveOut boolean whether
	 * this corresponds to be move-out ar an move-in @param oldEvalOrder the old
	 * evaluation order of input subplans to the affected sj @param newEvalOrder
	 * the new evaluation order of input subplans to the affected sj the
	 * newEvalOrder will be filled in this function
	 *  
	 */
	static double searchSortedBranchSta(SimulateMoveResult oldResult,
			AffectedSj affectedSj, NavigateStream nav, List sortedBranchStas,
			BranchStatistics beforeBranchSta, BranchStatistics afterBranchSta,
			BranchStatistics removedBranchSta, boolean moveOut,
			List oldEvalOrder, List newEvalOrder) {

		double costCut = 0;
		double beforeBranchRank = beforeBranchSta.getRankVal();
		double beforeBranchProbNotEmpty = beforeBranchSta.getProbNotEmpty();
		double afterBranchRank = afterBranchSta.getRankVal();
		double afterBranchProbNotEmpty = afterBranchSta.getProbNotEmpty();
		double removedBranchRank = removedBranchSta.getRankVal();
		double removedBranchProbNotEmpty = removedBranchSta.getProbNotEmpty();
		XATStreamOperator removedBranchEntry = (XATStreamOperator) removedBranchSta
				.getEntryNode().getOperator();
		double movedNavProbNotEmpty = 0;

		//tokenNav op is pulled out, the subplan between it and its fan in sj
		// is moved out
		StructuralJoinStream sj = (StructuralJoinStream) tokenNavToFanInSj
				.get(nav);
		OpStatistics opSta = null;
		if (sj == null)
			opSta = (OpStatistics) opToStatistics.get(nav);
		else
			opSta = (OpStatistics) opToStatistics.get(sj);
		movedNavProbNotEmpty = opSta.getProbNotEmpty();

		//the smaller rank between removedBranchRank and beforeRank
		double lowerBetweenRemovedAndBefore = removedBranchRank;
		double higherBetweenRemovedAndBefore = beforeBranchRank;
		boolean removedBranchLower = true;
		if (removedBranchRank > beforeBranchRank) {
			lowerBetweenRemovedAndBefore = beforeBranchRank;
			higherBetweenRemovedAndBefore = removedBranchRank;
			removedBranchLower = false;
		}

		double factor = CostComp.INFINITY;

		if (moveOut) {
			if (movedNavProbNotEmpty != 0)
				factor = 1 / movedNavProbNotEmpty;
		} else
			factor = movedNavProbNotEmpty;

		double overallProbBranchNotEmpty = 1;
		double overallSel = CostComp.getOverallSelFromInputSJ(affectedSj
				.getSj());
		double localCostCut = 0;
		boolean afterBranchInserted = false;
		int modifiedBranchPos = -1;
		int removedBranchPos = 0;

		List newSortedBranchStas = new ArrayList();
		int size = sortedBranchStas.size();
		//the position of the removed branch

		for (int i = 0; i < size; i++) {
			//visit old and new sortedBranchStas synchornizely

			BranchStatistics branchSta = (BranchStatistics) sortedBranchStas
					.get(i);
			BranchStatistics newBranchSta = (BranchStatistics) branchSta
					.clone();
			DagNode entryNode = branchSta.getEntryNode();

			double curBranchRank = branchSta.getRankVal();
			double branchCost = branchSta.getCost();

			if (branchSta == removedBranchSta || branchSta == beforeBranchSta) {
				localCostCut = branchCost;
				//also remove the removed branch from the newSortedBranchStas
				if (branchSta == beforeBranchSta)
					modifiedBranchPos = ((Integer) oldEvalOrder.get(i))
							.intValue();
				if (branchSta == removedBranchSta)
					removedBranchPos = ((Integer) oldEvalOrder.get(i))
							.intValue();
				costCut += localCostCut;

				continue;
			}

			//adjust the new evaluation order
			int pos = ((Integer) oldEvalOrder.get(i)).intValue();
			if (pos > removedBranchPos)
				newEvalOrder.add(new Integer(pos - 1));
			else
				newEvalOrder.add(new Integer(pos));
			newSortedBranchStas.add(newBranchSta);

			if (factor == CostComp.INFINITY) {
				//movedNavProbNotEmpty =0
				//before branchSta.cost = 0
				localCostCut = branchSta.getUnitCost()
						* overallProbBranchNotEmpty * overallSel;
			} else if (curBranchRank < lowerBetweenRemovedAndBefore) {
				if (curBranchRank < afterBranchRank)
					//\sigma_{op \not\Rightarrow \emptyset}
					localCostCut = (1 - factor) * branchCost;
				else {
					//\sigma_{newbranch \not\Rightarrow \emptyset}
					localCostCut = (1 - factor * afterBranchProbNotEmpty)
							* branchCost;
					//curBranchRank>=afterBranchRankVal
					newSortedBranchStas.add(newSortedBranchStas.size() - 1,
							afterBranchSta);
					if (modifiedBranchPos == -1)
						//a new branch with an extract is added at the end of
						// all children
						newEvalOrder.add(newSortedBranchStas.size() - 1,
								new Integer(oldEvalOrder.size() - 1));
					else {
						if (modifiedBranchPos > removedBranchPos)
							newEvalOrder.add(newEvalOrder.size() - 1,
									new Integer(modifiedBranchPos - 1));
						else
							newEvalOrder.add(newEvalOrder.size() - 1,
									new Integer(modifiedBranchPos));
					}

					afterBranchInserted = true;

					//also compute the cost
					afterBranchSta.setCost(afterBranchSta.getUnitCost()
							* overallProbBranchNotEmpty * overallSel);
				}
			} else if (lowerBetweenRemovedAndBefore < curBranchRank
					&& curBranchRank < higherBetweenRemovedAndBefore) {
				if (curBranchRank < afterBranchRank)
					if (removedBranchLower)
						localCostCut = (1 - factor / removedBranchProbNotEmpty)
								* branchCost;
					else
						localCostCut = (1 - factor / beforeBranchProbNotEmpty)
								* branchCost;
				else {
					//curBranchRank>=afterBranchRankVal
					//readjust the position of the changed branch
					newSortedBranchStas.add(newSortedBranchStas.size() - 1,
							afterBranchSta);
					if (modifiedBranchPos == -1)
						//a new branch with an extract is added at the end of
						// all children
						newEvalOrder.add(new Integer(oldEvalOrder.size() - 1));
					else {
						if (modifiedBranchPos > removedBranchPos)
							newEvalOrder.add(newEvalOrder.size() - 1,
									new Integer(modifiedBranchPos - 1));
						else
							newEvalOrder.add(newEvalOrder.size() - 1,
									new Integer(modifiedBranchPos));
					}

					afterBranchInserted = true;

					//also compute the cost
					afterBranchSta.setCost(afterBranchSta.getUnitCost()
							* overallProbBranchNotEmpty * overallSel);

					if (removedBranchLower)
						localCostCut = (1 - factor * afterBranchProbNotEmpty
								/ removedBranchProbNotEmpty)
								* branchCost;
					else
						localCostCut = (1 - factor * afterBranchProbNotEmpty
								/ beforeBranchProbNotEmpty)
								* branchCost;
				}
			} else if (curBranchRank >= higherBetweenRemovedAndBefore) {
				if (curBranchRank < afterBranchRank)
					localCostCut = (1 - factor / removedBranchProbNotEmpty
							/ beforeBranchProbNotEmpty)
							* branchCost;
				else {
					localCostCut = (1 - factor * afterBranchProbNotEmpty
							/ removedBranchProbNotEmpty
							/ afterBranchProbNotEmpty)
							* branchCost;

					newSortedBranchStas.add(newSortedBranchStas.size() - 1,
							afterBranchSta);
					if (modifiedBranchPos == -1)
						//a new branch with an extract is added at the end of
						// all children
						newEvalOrder.add(newEvalOrder.size() - 1, new Integer(
								oldEvalOrder.size() - 1));
					else {
						if (modifiedBranchPos > removedBranchPos)
							newEvalOrder.add(newEvalOrder.size() - 1,
									new Integer(modifiedBranchPos - 1));
						else
							newEvalOrder.add(newEvalOrder.size() - 1,
									new Integer(modifiedBranchPos));
					}

					afterBranchInserted = true;

					//also compute the cost
					afterBranchSta.setCost(afterBranchSta.getUnitCost()
							* overallProbBranchNotEmpty * overallSel);
				}
			}

			//update the cost of these branches previously in the old plan
			newBranchSta.setCost(branchSta.getCost() - localCostCut);
			overallProbBranchNotEmpty = overallProbBranchNotEmpty
					* newBranchSta.getProbNotEmpty();
			costCut += localCostCut;

		}

		if (!afterBranchInserted) {
			//also compute the cost
			afterBranchSta.setCost(afterBranchSta.getUnitCost()
					* overallProbBranchNotEmpty * overallSel);
			newSortedBranchStas.add(afterBranchSta);
			newEvalOrder.add(new Integer(newSortedBranchStas.size() - 1));
		}

		//		now add the cost of the modified new branch
		costCut = costCut - afterBranchSta.getCost();

		//update the deltaAffectedPart
		Map newSjToBranchStatistics = new HashMap();
		SjToBranchStaMapContent sjToBranchStaMapContent = new SjToBranchStaMapContent(
				newSortedBranchStas, newEvalOrder);
		newSjToBranchStatistics
				.put(affectedSj.getSj(), sjToBranchStaMapContent);
		oldResult.getDeltaAffectedPart().setNewSjToBranchStatistics(
				newSjToBranchStatistics);

		return costCut;
	}

	/*
	 * @double get the cost of the navigate operator that tokenNav will be
	 * converted into @param tokenNav PureNavigateAutoStream the tokenNav
	 * operator @param collector StatisticsCollector
	 */
	static double getUnitCost(PureNavigateAutoStream tokenNav,
			StatisticsCollector collector) {
		double unitCost = 0;

		unitCost = ((Integer) StatisticsCollector.tokenNavtoStartCountSum
				.get(tokenNav)).intValue()
				* StatisticsCollector.NODENAV_VISIT_COST;

		return unitCost;

	}

	/*
	 * move out the tokenNav, @PureNavigateAuotStream tokenNav @return
	 * BranchStatistics, i.e., the branch that is affected by the moving out of
	 * the tokenNav i.e., the branch that the moved-out operator is on object[0] =
	 * starting point identifying a branch object[1] = selectivity of the
	 * starting point object[2] = cost of the branch object[3] = probability of
	 * the branch yields something object[4] = rank value
	 *  
	 */
	static void moveOut(SimulateMoveResult oldResult) {
		moveOut(oldResult, oldResult.getMovedNodeInTemp());
	}

	/*
	 * move out the tokenNav, @PureNavigateAuotStream tokenNav @return
	 * BranchStatistics, i.e., the branch that is affected by the moving out of
	 * the tokenNav i.e., the branch that the moved-out operator is on object[0] =
	 * starting point identifying a branch object[1] = selectivity of the
	 * starting point object[2] = cost of the branch object[3] = probability of
	 * the branch yields something object[4] = rank value
	 *  
	 */
	static void moveOut(SimulateMoveResult oldResult, DagNode tokenNavNode) {

		CopiedDag tempPlan = oldResult.getTempPlan();
		PureNavigateAutoStream tokenNav = (PureNavigateAutoStream) tokenNavNode
				.getOperator();
		DagNode tokenNavChildNode = (DagNode) tokenNavNode.getChild(0);
		XATStreamOperator tokenNavChild = (XATStreamOperator) tokenNavChildNode
				.getOperator();

		//apply rewrite rule to push nodeNav down
		//we start from applying rule on any nodeNav depended on by the one
		// that is to be moved out
		//e.g., NavNest_{\$b, //profile}\$e
		//                    |
		//      NavUnnest_{\$a, /seller}\$b
		//if nodeNav = NavNest_{\$b, //profile}\$e,
		// NavUnnest_{\$a, /seller}\$b will be pushed in first
		//and then NavNest_{\$b, //profile}\$e is pushed in
		//now find the list of nodeNav operators that are dependent on

		SpecifiedNavPulloutRewriteRule rewriter = new SpecifiedNavPulloutRewriteRule(
				oldResult, tokenNav);
		AffectedPart affectedPart = (AffectedPart) rewriter.visit(tokenNav,
				tokenNavChild);

		/** ****make the change script******* */
		//for a move out: remove one place and insert multiple places
		/*
		 * ChangeScript deltaScript = new ChangeScript(); ChangePrimitive delPri =
		 * new ChangePrimitive(ChangePrimitive.DELETE);
		 * delPri.setOnNode((DagNode) tokenNav.getNode());
		 * deltaScript.addDelPrimitive(delPri);
		 * 
		 * //also the insert primivitives, adding the subplan containging the //
		 * nodeNav List affectedSjs = affectedPart.getAffectedSjs(); int
		 * affectedSjNum = affectedSjs.size(); for (int i = 0; i <
		 * affectedSjNum; i++) { AffectedSj affectedSj = (AffectedSj)
		 * affectedSjs.get(i); if (affectedSj.contentChanged()) {
		 * ChangePrimitive insertPri = new ChangePrimitive(
		 * ChangePrimitive.INSERT); deltaScript.addInsertPri(insertPri);
		 * XATStreamOperator entryOp = affectedSj.getEntryOp(); if
		 * (affectedSj.entryOpAdded()) { DagNode ontoNode = (DagNode) ((DagNode)
		 * entryOp.getNode()) .getChild(0).getOperator().getNode();
		 * insertPri.setOnNode(ontoNode); insertPri.setBranchHeadNode((DagNode)
		 * entryOp.getNode()); } else { //extract op is there before //if this
		 * extract has a non-empty input subplan to the // structural join if
		 * (!(entryOp.getNode().getParent().getOperator() instanceof
		 * StructuralJoinStream)) { ChangePrimitive moreDelPri = new
		 * ChangePrimitive( ChangePrimitive.DELETE);
		 * moreDelPri.setOnNode(((DagNode) entryOp.getNode()) .getParent(0));
		 * deltaScript.addDelPrimitive(moreDelPri); } DagNode
		 * entryNodeInOriginal = (DagNode) entryOp.getNode();
		 * insertPri.setOnNode(entryNodeInOriginal); DagNode entryNodeInTemp =
		 * tempPlan.getNode(entryOp); insertPri.setBranchHeadNode((DagNode)
		 * entryNodeInTemp .getParent()); }
		 *  } }
		 * 
		 * oldResult.setChangeScript(deltaScript);
		 */

		oldResult.setDeltaAffectedPart(affectedPart);

	}

	static void moveIn(SimulateMoveResult oldResult) {
		moveIn(oldResult, oldResult.getMovedNodeInTemp());
	}

	/*
	 * move in the nodeNav, @param nodeNav NavigateStream the nodeNav operator
	 * to be pushed into @param tempPlan CopiedDag the temporary plan to push in
	 * the nodeNav @param existingScript ChangeScript the already adopted script
	 * of modifying the original plan
	 *  
	 */
	static void moveIn(SimulateMoveResult oldResult, DagNode nodeNavNode) {

		NavigateStream nodeNav = (NavigateStream) nodeNavNode.getOperator();
		CopiedDag tempPlan = oldResult.getTempPlan();
		DagNode nodeNavChildNode = (DagNode) nodeNavNode.getChild(0);
		XATStreamOperator nodeNavChild = (XATStreamOperator) nodeNavChildNode
				.getOperator();

		//apply rewrite rule to push nodeNav down
		//we start from applying rule on any nodeNav depended on by the one
		// that is to be moved out
		//e.g., NavNest_{\$b, //profile}\$e
		//                    |
		//      NavUnnest_{\$a, /seller}\$b
		//if nodeNav = NavNest_{\$b, //profile}\$e,
		// NavUnnest_{\$a, /seller}\$b will be pushed in first
		//and then NavNest_{\$b, //profile}\$e is pushed in
		//now find the list of nodeNav operators that are dependent on

		SpecifiedNavPushinRewriteRule rewriter = new SpecifiedNavPushinRewriteRule(
				oldResult, nodeNav);
		AffectedPart affectedPart = (AffectedPart) rewriter.visit(nodeNav,
				nodeNavChild);

		//if any ancestor node of newBranchHead has appeared in a primitive
		//in the previous changescript as a head of a new branch
		//then we only need do not need to make any change to the change script
		//otherwise we add the local change to the global change script
		/*
		 * List insertPris = existingScript.getInsertPrimitives(); int
		 * insertPriSize = insertPris.size();
		 * 
		 * for (int i=0; i <insertPriSize; i++){ ChangePrimitive insertPri =
		 * (ChangePrimitive)insertPris.get(i); DagNode branchHeadNode =
		 * insertPri.getBranchHeadNode(); XATStreamOperator branchHeadOp =
		 * (XATStreamOperator)branchHeadNode.getOperator();
		 * 
		 * DagNode curChildNode = (DagNode)newBranchHead.getChild(0);
		 * XATStreamOperator curChildOp =
		 * (XATStreamOperator)curChildNode.getOperator(); while (!(curChildOp
		 * instanceof StreamSource)){ //curChildOp must be a tokenNav if
		 * (curChildOp == branchHeadOp){ //must update the primitive
		 * changeToChangeScript.deletePris(insertPri); ChangePrimitive
		 * newInsertPri = new ChangePrimitive(insertPri);
		 * newInsertPri.setBranchHeadNode(curChildNode);
		 * changeToChangeScript.addPris(newInsertPri); return
		 * changeToChangeScript; } curChildNode =
		 * (DagNode)curChildNode.getChild(0); curChildOp =
		 * (XATStreamOperator)curChildNode.getOperator(); } }
		 */

		/** *******make the change script ************ */
		//for a move in: modify multiple places
		List affectedSjs = affectedPart.getAffectedSjs();
		int affectedSjNum = affectedSjs.size();

		ChangeScript deltaScript = new ChangeScript();
		for (int i = 0; i < affectedSjNum; i++) {
			AffectedSj affectedSj = (AffectedSj) affectedSjs.get(i);
			if (affectedSj.contentChanged()) {
				XATStreamOperator entryOp = affectedSj.getEntryOp();
				ChangePrimitive delPrimitive = new ChangePrimitive(
						ChangePrimitive.DELETE);
				deltaScript.addDelPrimitive(delPrimitive);
				if (affectedSj.entryOpRemoved())
					delPrimitive.setOnNode((DagNode) entryOp.getNode());
				else {
					delPrimitive.setOnNode((DagNode) entryOp.getNode()
							.getParent());
					ChangePrimitive insertPrimitive = new ChangePrimitive(
							ChangePrimitive.INSERT);
					insertPrimitive.setOnNode((DagNode) entryOp.getNode());
					insertPrimitive.setBranchHeadNode((DagNode) tempPlan
							.getNode(entryOp).getParent());
					deltaScript.addInsertPri(insertPrimitive);
				}
			}
		}

		oldResult.setChangeScript(deltaScript);
		oldResult.setAffectedPart(affectedPart);

	}

	/*
	 * @param tokenNavNode DagNode the node identifes a branch @return int the
	 * probability of this branch would generate some result within the scope of
	 * the structural join assume operators on this branch all have data
	 * dependency for example, navnest_{a, p1} b1 has no data dependency with
	 * navnest_{a, p2}b2
	 */
	static double computeNotEmptyProb(DagNode tokenNavNode) {
		DagNode nodeOnBranch = tokenNavNode;
		XATStreamOperator opOnBranch = (XATStreamOperator) nodeOnBranch
				.getOperator();

		double notEmptyProb = 1;
		double sel = 1;

		while (!(opOnBranch instanceof StructuralJoinStream)) {
			if (opOnBranch instanceof ExtractStream
					|| opOnBranch instanceof NavigateStream) {
				OpStatistics opOnBranchSta = (OpStatistics) opToStatistics
						.get(opOnBranch);
				//find how many tuples are there be, i.e., the selectivity
				sel = sel * opOnBranchSta.getSel();
			} else if (opOnBranch instanceof SelectStream) {
				OpStatistics opOnBranchSta = (OpStatistics) opToStatistics
						.get(opOnBranch);
				double localNotEmptyProb = opOnBranchSta.getProbNotEmpty();
				double localSel = opOnBranchSta.getSel();
				//notEmptyProbe = 1 - (1 - localNotEmptyProb)^{localSel}
				notEmptyProb = 1 - Math.pow(1 - localNotEmptyProb, localSel);
			}

			nodeOnBranch = nodeOnBranch.getParent(0);
			opOnBranch = (XATStreamOperator) nodeOnBranch.getOperator();

		}
		return notEmptyProb;

	}

	/*
	 * @param movedTokenNavNode DagNode the node to be moved out where
	 * structuraljoin_{1} not= structuraljoin_{2} @return double if the operator
	 * is moved out, the probablity the moved out part would not generate empty
	 * result within structuraljoin_{2}
	 */
	static double getProbNotEmptyForMovedOutNode(DagNode movedTokenNavNode) {

		double probNotEmpty = 1;

		PureNavigateAutoStream movedTokenNav = (PureNavigateAutoStream) movedTokenNavNode
				.getOperator();

		OpStatistics opSta = (OpStatistics) opToStatistics.get(movedTokenNav);
		double localProbNotEmpty = opSta.getProbNotEmpty();

		/*
		 * StructuralJoin_{$b} / \ ExtractNest_{$b}$g ExtractNest_{$b}$h | |
		 * TokenNav_{$b, /name}$g TokenNav_{$b, /address}$h \ / TokenNav_{$a,
		 * //seller} $b
		 */

		if (movedTokenNavNode.getParentsSize() == 1) {
			//move out this operator has no side effect
			//e.g., TokenNav_{$b, /address}$h
			DagNode childNode = (DagNode) movedTokenNavNode.getChild(0);
			OpStatistics childOpSta = (OpStatistics) opToStatistics
					.get(childNode.getOperator());
			double sel = childOpSta.getSel();

			probNotEmpty = 1 - java.lang.Math.pow(1 - localProbNotEmpty, sel);
		} else {
			//move out this operator has side effect,
			//i.e., all children patterns have to be moved out as well
			//e.g., TokenNav_{$a, //seller}$b

			StructuralJoinStream sj = (StructuralJoinStream) tokenNavToFanInSj
					.get(movedTokenNav);
			OpStatistics sjSta = (OpStatistics) opToStatistics.get(sj);

			probNotEmpty = 1 - java.lang.Math.pow(1 - sjSta.getProbNotEmpty(),
					opSta.getSel());

		}
		return probNotEmpty;
	}

	/*
	 * @param DagNode structuralJoinNode @double the prob of the subplan
	 * identified by the structuraljoin node not generating empty result within
	 * an output tuple of the fan in node
	 */
	static double computeProbNotEmptyForSj(DagNode structuralJoinNode) {
		double probNotEmpty = 1;

		List sortedBranches = ((SjToBranchStaMapContent) sjToBranchStatistics
				.get(structuralJoinNode.getOperator())).getSortedBranchStas();
		int branchNum = sortedBranches.size();

		for (int i = 0; i < branchNum; i++) {
			BranchStatistics branchSta = (BranchStatistics) sortedBranches
					.get(i);

			double branchProbNotEmpty = branchSta.getProbNotEmpty();

			probNotEmpty = probNotEmpty * branchProbNotEmpty;
		}

		return probNotEmpty;
	}

	/*
	 * @param tokenNavs List @param sourceTokenNavNode DagNode initialize the
	 * opsNeedRecompute list
	 */
	static List initializeOpsNeedRecompute(List navs) {
		List opsNeedRecompute = new ArrayList();

		int tokenNavNum = navs.size();

		for (int i = 0; i < tokenNavNum; i++) {
			//can be either tokenNav or nodeNav
			NavigateStream nav = (NavigateStream) navs.get(i);
			DagNode navNode = (DagNode) nav.getNode();
			CopiedDag copyOfOriginal = new CopiedDag(inputPlan);
			BranchStatistics branchSta = null;
			SimulateMoveResult result = new SimulateMoveResult(copyOfOriginal
					.getNewNodeFromOldNode(navNode), 0, copyOfOriginal);
			opsNeedRecompute.add(result);

		}

		return opsNeedRecompute;
	}

	/*
	 * @param deltaAffectedPart AffectedPart the delta change over the oldResult
	 * incorporate these changes on branchToStatistics, entryOpToInputSj,
	 * tokenNavToFanInSj, tokenNavToFanInSj
	 *  
	 */
	static void adoptDeltaAffectedPart(AffectedPart deltaAffectedPart) {
		Map newSjToBranchStatistics = deltaAffectedPart
				.getNewSjToBranchStatistics();
		Set sjs = newSjToBranchStatistics.keySet();
		for (Iterator it = sjs.iterator(); it.hasNext();) {
			InTimeSJStream sj = (InTimeSJStream) it.next();
			SjToBranchStaMapContent sjToBranchStaMapContent = (SjToBranchStaMapContent) newSjToBranchStatistics
					.get(sj);
			sj.setBranchEvalOrder(sjToBranchStaMapContent.getEvalOrder());
		}

		sjToBranchStatistics.putAll(deltaAffectedPart
				.getNewSjToBranchStatistics());
		tokenNavToExtract.putAll(deltaAffectedPart.getNewTokenNavToExtract());
		entryOpToInputSj.putAll(deltaAffectedPart.getNewEntryOpToInputSj());
	}

}