package edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.plaintext;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * This class uses the SAX Parser to parse the config file
 * @author Luping Ding
 *  
 */
public class DataConfigFileParser extends DefaultHandler {

	/**Stores the name of the input File*/
	String inputFile = null;
	/**Stores the reference to the sax parser*/
	SAXParser saxParser = null;
	/**Stores the reference to the handler*/
	DefaultHandler dh = null;
	/**Stores the reference to the buffered reader used to lines from the file*/
	BufferedReader br = null;
	/**Stores the reference to the root of the tree*/
	Element rootElement = null;
	/**Stores the reference to the current element*/
	Element element = null;
	/**Stores the current element name note this is used as a global
	 * variable inside the context of this class */
	String elementName = null;
	/** Added by Yali Zhu: Stored the current range list (streamname, attrname, rangelist, streamname, attrname, rangelist ...*/
	Element rootRangeElement = null;
	Element rangeElement = null;
	/**
	 * Constructs a new StreamConfigFileParser
	 */
	public DataConfigFileParser(String fileName) throws IOException {
		this.inputFile = fileName;

		File file = new File(fileName);

		SAXParserFactory factory = SAXParserFactory.newInstance();

		dh = this;

		try {
			saxParser = factory.newSAXParser();
			saxParser.parse(file, this);

		} catch (ParserConfigurationException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		}
	}

	/**
	 * @see org.xml.sax.ContentHandler#startDocument()
	 */
	public void startDocument() {
		System.out.println("Start of document");
	}

	/** 
	 * @see org.xml.sax.ContentHandler#endDocument()
	 */
	public void endDocument() {
		System.out.println("End of document");
	}

	/**
	 * @see org.xml.sax.ContentHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	public void startElement(String namespaceURI, String lName, // local name
	String qName, // qualified name
	Attributes attrs) throws SAXException {
		Element e = null;
		RangeElement rangeE = null;
		String eName = qName; // element name
		//This variable uses this name in the global context		
		elementName = eName;
		//be careful what to use lName or qName
		if (eName.toLowerCase().compareTo("streams") == 0) {
			element = new StreamsElement(eName, null);
			rootElement = element;
		} else if (eName.toLowerCase().compareTo("stream") == 0) {
			e = new StreamElement(eName, element);
			//add another element to the current element
			element.addChildElement(e);

			//make the current element this element
			//we do the reverse when the element ends
			element = e;
		} else if (eName.toLowerCase().compareTo("tuple") == 0) {
			e = new TupleElement(eName, element);
			element.addChildElement(e);
			//make the current element this element
			//we do the reverse when the element ends
			element = e;
			//Ignore this element
			//Can be that we need a root for the element
		} else if (eName.toLowerCase().compareTo("distribution") == 0) {
			e = new DistributionElement(elementName, element);
			element.addChildElement(e);
			//							make the current element this element
			//we do the reverse when the element ends
			element = e;
			//Ignore this element
			//Can be that we need a root for the element
		} else if (eName.toLowerCase().compareTo("attribute") == 0) {
			e = new AttributeElement(elementName, element);
			element.addChildElement(e);
			//							make the current element this element
			//we do the reverse when the element ends
			element = e;
			//Ignore this element
			//Can be that we need a root for the element
		} else if (eName.toLowerCase().compareTo("reductionfactors") == 0) {
			rangeElement = new RangesElement(eName, null);
			rootRangeElement = rangeElement;
		} else if (eName.toLowerCase().compareTo("reductionfactor") == 0) {
			rangeE = new RangeElement(eName, rangeElement);
			rangeElement.addChildElement(rangeE);
			rangeElement = rangeE;
		}
	}

	/**
	 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
	 */
	public void characters(char buf[], int offset, int len)
		throws SAXException {
		String tempName = null;
		try {
			/*Hack I am using because this same method is being called more than thrice
			 * I only want to assign the value for the first time
			 */
			if(elementName != null) {			
				tempName = elementName.toLowerCase();
				elementName = null;
			} else {
				return;
			}
							
		 	if (tempName.compareTo("outputfilename") == 0) {
				((StreamElement) element).setOutputFileName(
					new String(buf, offset, len));
			}

			else if (tempName.compareTo("name") == 0) {
				if (element instanceof AttributeElement)
					((AttributeElement) element).setAttrName(
						new String(buf, offset, len));
				if (element instanceof StreamElement)
					((StreamElement) element).setStreamName(
						new String(buf, offset, len));
				if (element instanceof DistributionElement)
					((DistributionElement) element).setDistributionName(
						new String(buf, offset, len));
			}

			else if (tempName.compareTo("type") == 0) {			
				((AttributeElement) element).setType(new String(buf, offset, len));
			}
			else if (tempName.compareTo("category") == 0) {
				((DistributionElement) element).setCategory(
					new String(buf, offset, len));
			}
			else if (tempName.compareTo("range") == 0) {
				((DistributionElement) element).setRange(
					new String(buf, offset, len));
			}
			else if (tempName.compareTo("order") == 0) {
				((DistributionElement) element).setOrder(
					new String(buf, offset, len));
			}
			else if (tempName.compareTo("numvalues") == 0) {
				((DistributionElement) element).setNumValues(
					new String(buf, offset, len));
			}
			else if (tempName.compareTo("granularity") == 0) {
				((DistributionElement) element).setGranularity(
					new String(buf, offset, len));
			}
			else if (tempName.compareTo("avgexceptionpercent") == 0) {
				((DistributionElement) element).setAvgExceptionPercent(
					new String(buf, offset, len));
			}
			else if (tempName.compareTo("avgdistinctvalues") == 0) {
				((DistributionElement) element).setAvgDistinctValues(
					new String(buf, offset, len));
			}
			/** following are added for reduction factor settings */
			else if (tempName.compareTo("leftstreamname") == 0) {
				((RangeElement) rangeElement).setLeftStreamName(
					new String(buf, offset, len));
			} 
			else if (tempName.compareTo("leftcolumnname") == 0) {
				((RangeElement) rangeElement).setLeftColumnName(
					new String(buf, offset, len));
			} 
			else if (tempName.compareTo("rightstreamname") == 0) {
				((RangeElement) rangeElement).setRightStreamName(
					new String(buf, offset, len));
			} 
			else if (tempName.compareTo("rightcolumnname") == 0) {
				((RangeElement) rangeElement).setRightColumnName(
					new String(buf, offset, len));
			} 
			else if (tempName.compareTo("jointype") == 0) {
				((RangeElement) rangeElement).setJoinType(
					new String(buf, offset, len));
			} 
			else if (tempName.compareTo("factorvalue") == 0) {
				((RangeElement) rangeElement).setFactorValue(
					new String(buf, offset, len));
			} 
			else if (tempName.compareTo("starttuplenum") == 0) {
				((RangeElement) rangeElement).setStartTupleNum(
					new String(buf, offset, len));
			}
	
		} catch(RuntimeException re) {
			re.printStackTrace();	
		}
	}

	/**
	 * @see org.xml.sax.ContentHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	public void endElement(String namespaceURI, String sName, // simple name
	String qName // qualified name
	) throws SAXException {
	
		
		//go one level above recursion
		boolean flag = false;
				
		String endElementName = qName.toLowerCase();
				
		if(endElementName.compareTo("streams") == 0)
			flag = true;
		if(endElementName.compareTo("stream")== 0)
			flag = true;
		if(endElementName.compareTo("tuple")== 0)
			flag = true;
		if(endElementName.compareTo("distribution")==0)	
			flag = true;
		if(endElementName.compareTo("attribute")==0)	
			flag = true;
			
		if(flag)
		{
			if(element.getParent() != null)
			{				
				element = element.getParent();
				elementName = element.getName();
			}
		}		
		flag = false;
		if (endElementName.compareTo("reductionfactors") == 0)
			flag = true;
		if (endElementName.compareTo("reductionfactor") == 0)
			flag = true;
		if(flag)
		{
			if (rangeElement.getParent() != null)
			{
				rangeElement = rangeElement.getParent();
				elementName = rangeElement.getName();
			}
		}		
	}
	/**
	 * Returns the root of the element tree
	* @return Element
	*/
	public Element getRootElement() {
		return (this.rootElement);
	}

	/**
	* @return
	*/
	public TupleGenerator[] getTupleGenerators() {
		return (((StreamsElement) rootElement).makeTupleGenerators());
	}
	
	/** added by Yali Zhu
	* set range list
	*/
	public void setRangeList(TupleGenerator[] tg) {
		int i = 0, j = 0, k = 0;
		String leftStreamID = null;
		String rightStreamID = null;
		String leftColumnID = null;
		String rightColumnID = null;
		int dataRange = 0;
		int startTupleNum = 0;
		RangeElement e = null;
		AttributeGenerator[] attributeGenerators = null;
		if (rootRangeElement != null) {
			while ((e = (RangeElement) rootRangeElement.getElement(i))
				!= null) {
				i++;
				leftStreamID = e.getLeftStreamName();
				rightStreamID = e.getRightStreamName();
				leftColumnID = e.getLeftColumnName();
				rightColumnID = e.getRightColumnName();
				dataRange = e.getDataRange();
				startTupleNum = e.getStartTupleNum();

				for (j = 0; j < tg.length; j++) {
					if (tg[j].getStreamName().compareTo(leftStreamID) == 0) {
						attributeGenerators = tg[j].getAttributeGenerator();
						for (k = 0; k < attributeGenerators.length; k++) {
							if (attributeGenerators[k]
								.getAttributeName()
								.compareTo(leftColumnID)
								== 0) {
								attributeGenerators[k].setNextRange(
									startTupleNum,
									dataRange);
								break;
							}
						}

					} else if (
						tg[j].getStreamName().compareTo(rightStreamID) == 0) {
						attributeGenerators = tg[j].getAttributeGenerator();
						for (k = 0; k < attributeGenerators.length; k++) {
							if (attributeGenerators[k]
								.getAttributeName()
								.compareTo(rightColumnID)
								== 0) {
								attributeGenerators[k].setNextRange(
									startTupleNum,
									dataRange);
								break;
							}
						}
					}
				}
			}
		}
	}
	}
