package edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.plaintext;


/**
 * Represents a distribution element in the XML file.
 * Stores all the relevant information to the distribution
 * @author Luping Ding
 *
 */
public class DistributionElement extends Element {
	/**Stores the category of distribution*/
	protected String category;
	/**Stores the name of the distribution*/
	protected String distributionName; 
	protected String startValue ="0";
	/**Stores the range of the distribution*/
    protected String range;
  	/**Stores the order*/
  	protected String order;
	/**Stores the number of values to be generated*/
	protected String numValues;
	protected String granularity;
	protected String avgExceptionPercent;
	protected String avgDistinctValues;
  	
	/**
	 * Construct a new Distribution Element
	 * @param name
	 * @param parent
	 */
	public DistributionElement(String name, Element parent) {
		super(name, parent);				
	}
	
	/**
	 * Genereates the attribute depending on the field values intiaized
	 * @return
	 */
	
	public AttributeGenerator makeAttributeGenerator() {
		AttributeGenerator ag = null;
		int intStartValue = Integer.parseInt(startValue);
		int intRange =Integer.parseInt(range);
		int intNumValues = Integer.parseInt(numValues);
			
		if (category.toLowerCase().equals("random")) {
			if(distributionName.toLowerCase().equals("uniform")) {
				ag = new RandomUniformAttributeGenerator(intRange, intNumValues);		
			}			
		}else if (category.toLowerCase().equals("unique")) {
			int intOrder = 0;
			if (order.toLowerCase().equals("asc")) {
				intOrder = AttributeGenerator.ASC;
			} else if (order.toLowerCase().equals("desc")) {
				intOrder = AttributeGenerator.DESC;
			} else if (order.toLowerCase().equals("random")) {
				intOrder = AttributeGenerator.RANDOM;
			}
			ag = new UniqueAttributeGenerator(intStartValue, intRange, intOrder, intNumValues);
		} else if(category.toLowerCase().equals("cluster")) {
			int intOrder = 0;
			if (order.toLowerCase().equals("asc")) {
				intOrder = AttributeGenerator.ASC;
			} else if (order.toLowerCase().equals("desc")) {
				intOrder = AttributeGenerator.DESC;
			} else if (order.toLowerCase().equals("random")) {
				intOrder = AttributeGenerator.RANDOM;
			}
			int intGran = Integer.parseInt(granularity);
			ag = new ClusterAttributeGenerator(intStartValue, intRange, intOrder, intNumValues, intGran);
		} else if(category.toLowerCase().equals("roughcluster")) {
			int intGran = Integer.parseInt(granularity);
			double doubleAvgException = Double.parseDouble(avgExceptionPercent);
			ag = new RoughClusterAttributeGenerator(intStartValue, intRange, intNumValues, doubleAvgException, intGran);
		} else if(category.toLowerCase().equals("auction")) {
			int intGran = Integer.parseInt(granularity);
			int intAvgDistinctVal = Integer.parseInt(avgDistinctValues);	
			ag = new AuctionPunctAttributeGenerator(intStartValue, intRange, intNumValues, intAvgDistinctVal, intGran);		
		}
		return(ag);		
	}
	
	public void setCategory(String category) {
		this.category = category;
	}
	
	/**
	 * @param string
	 */
	public void setDistributionName(String disName) {
		distributionName = disName;
	}

	/**
	 * @param string
	 */
	public void setRange(String string) {
		range = string;
	}

	/**
	 * @param string
	 */
	public void setOrder(String string) {
		order = string;
	}

	/**
	 * @param string
	 */
	public void setNumValues(String numVals) {
		numValues = numVals;		
	}
	
	public void setGranularity(String gran) {
		granularity = gran;
	}
	
	public void setAvgExceptionPercent(String avgException) {
		avgExceptionPercent = avgException;
	}
	
	public void setAvgDistinctValues(String avgDistinctVals) {
		avgDistinctValues = avgDistinctVals;
	}
}
