package edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.plaintext;

import java.util.Random;
import java.util.ArrayList;
import java.util.Arrays;
import edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.exceptions.StreamEmptyException;

/**
 * Used to generate the Attribute values with punctuatios
 * @author: Luping Ding
 */
public class RoughClusterAttributeGenerator extends ClusterAttributeGenerator{
	/** ratio of exceptional tuples within a punctated segment. */
	double avgExceptionPercent;
	/** random generator for choosing the index inside the 
	 *  punctuated segment where the exceptional tuples locate.
	 */
	Random exceptionIndexRandom;
	/** random generator for choosing the exceptional values. */
	Random exceptionValueRandom;
	/** index inside the segmentValue indicating the tuple to be returned. */
	int retValueIndex;
	/** stores values generated for the current punctuated segment. */
	ArrayList segmentValue;
	
	int numSegValues;
	
	int segValueCount;
	
	int currentValue;
		
	/**
	 * Constructs a new PunctAttributeGenerator
	 */
	public RoughClusterAttributeGenerator() {
		super();
		this.avgExceptionPercent = 0.5;
		exceptionIndexRandom = new Random();
		exceptionValueRandom = new Random();
	}

	public RoughClusterAttributeGenerator(int startValue, int range, int numValues, double avgExceptionPercent, int granularity) {
		super(startValue, range, AttributeGenerator.ASC, numValues, granularity);
		this.avgExceptionPercent = avgExceptionPercent;
		exceptionIndexRandom = new Random();
		exceptionValueRandom = new Random();
	}

	/**
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.plaintext.ControlledAttributeGenerator#getNextValue()
	 */
	public Object getNextValue() throws StreamEmptyException {
		if (numValueCount == numValues) {
			throw new StreamEmptyException();
		}

		if (count == 0) {
			if (index == range) {
				throw new StreamEmptyException();
			}
			retValueIndex = 0;
			while (count == 0) {
				count = (int)segmentSizeRandom.generateRandom();
			}
			if (count > (numValues - numValueCount)) {
				count = numValues - numValueCount;
			}
			segmentValue = new ArrayList(count);
			currentValue = vals[index];
			int exceptionCount = (int) (count * avgExceptionPercent);
			numSegValues = count - exceptionCount;
			
			for (int i=0; i<numSegValues; i++) {
				segmentValue.add(new Integer(currentValue));
			}
			
			int[] exceptionIndexArray = new int[exceptionCount];
			for (int i=0; i<exceptionCount; i++) {
				while (true) {
					boolean conflict = false;
					//int nextIndex = exceptionIndexRandom.nextInt(exceptionCount);
					int nextIndex = exceptionIndexRandom.nextInt(count);
					for (int j=0; j<i; j++) {
						if (nextIndex == exceptionIndexArray[j]) {
							conflict = true;
							break;
						}
					}
					if (!conflict) {
						exceptionIndexArray[i] = nextIndex;
						break;
					}
				}
			}
			
			Arrays.sort(exceptionIndexArray);
			
			for (int i=0; i<exceptionCount; i++) {
				int exceptionValue = index + 1 + exceptionValueRandom.nextInt(range);
				segmentValue.add(exceptionIndexArray[i], new Integer(exceptionValue));
			}
			index ++;
			segValueCount = 0;
		} 
		
		Integer retValue = (Integer)segmentValue.get(retValueIndex);
		retValueIndex ++;
		count --;
		numValueCount ++;
		
		String retString = null;
		if (retValue.intValue() == currentValue) {
			segValueCount ++;
			if (segValueCount == numSegValues) {
				return new String(retValue + "#");
			} else {
				return retValue;
			}
		} else {
			return retValue;
		}
	}		   
}


