package edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.plaintext;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;

import edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.exceptions.StreamEmptyException;
import edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server.Schema;

/**
 * This class genrates tuples. Where each tuple is 
 * a collection of attributes. Thus it contains a collection
 * of attribute generators.
 * @author Luping Ding
 */
public class TupleGenerator {
	/**An array of attribute generators ued to generate the tuples*/
	AttributeGenerator[] arrAttributeGenerator = null;
	/** Added by Yali zhu
	 * Stores the name of the streamID
	 */
	String streamName = null;
	/**Stores the name of the file to where the tuples are writeen to*/
	String fileName = null;
	/**Used to write the tuples to the output file*/
	BufferedWriter out = null;
/**
 * Constructs a new TupleGenerator
 * @param arrAttributeGenerator
 * @param fileName
 */
	public TupleGenerator(
		AttributeGenerator[] arrAttributeGenerator,
		String fileName)
		{
			
		this.arrAttributeGenerator = arrAttributeGenerator;
		this.fileName = fileName;
		
		try {
			out = new BufferedWriter(new FileWriter(fileName));
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void setStreamName(String name)
	{
		this.streamName = name;
	}
	
	public String getStreamName()
	{
		return this.streamName;
	}
	
	public AttributeGenerator[] getAttributeGenerator() 
	{
		return arrAttributeGenerator;
	}

	/**
	 * Writes a tuple onto the file
	 * Note each time we get a constraint it is
	 * meta data so we do not consider it as tuple and
	 * we write the constraint as an individual row. 
	 * Into the file
	 * @throws IOException
	 */
	public void writeTuples() throws StreamEmptyException, IOException {
		//While the any of the attribute streams are not empty
		//keep writing tuples on the disk
		while(true) {		
			int punctAttIndex = -1;
			int punctValue = 0;
			String value = "";
			for (int i = 0; i < arrAttributeGenerator.length; i++) {
				try {				
					Object o = arrAttributeGenerator[i].getNextValue();
					if (o instanceof String) {
						String str = (String) o;
						punctValue = Integer.parseInt(str.substring(0, str.length()-1));
						value = value + " " + punctValue;
						punctAttIndex = i;
					} else if (o instanceof Integer) {
						value = value + o + " ";													
					} 
				} catch (StreamEmptyException e) {
					out.close();				
					System.out.println("Attribute " + i + "th Empty ");
					throw(e);
				}			
			}
			out.write(value+"\n");
			//writing the constraint for the ith Attribute
			//Note this constraint is tagged appropriately
			if (punctAttIndex >=0) {
				String punct = new String(Schema.PUNCT_TAG);
				for (int i=0; i<arrAttributeGenerator.length; i++) {
					if (i == punctAttIndex) {
						punct = punct + "" + punctValue + " ";
					} else {
						punct += "* ";
					}
				}
				out.write(punct +"\n");
			}
		}
	}
}
