package edu.wpi.cs.dsrg.xmldb.xat.component.datagenerators.xml;

import java.io.FileReader;

import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Take an XML file, generate a file consisting of tokens Creation date:
 * (5/29/2003 6:11:40 PM)
 * 
 * @author: Hong Su
 */
public class TokenFileGenerator {
	/**
	 * TokenFileGenerator constructor comment.
	 */
	public TokenFileGenerator() {
		super();
	}

	/**
	 * parse raw file
	 */
	private static void initializeTokenQueueToFile(String rawFile,
			String tokenFile) {

		try {
			XMLReader parser = XMLReaderFactory
					.createXMLReader("org.apache.xerces.parsers.SAXParser");

			EmptySAXHandler exec = new EmptySAXHandler(tokenFile);

			parser.setContentHandler(exec);

			parser.setFeature("http://xml.org/sax/features/validation", false);
			//The only way that the parser can determine if text is ignorable
			// is by reading the associated grammar and having a content model
			// for the document.
			//a dtd is needed always
			// parser.setFeature(
			//     "http://apache.org/xml/features/nonvalidating/load-external-dtd",
			//     true);

			System.out.println("parsing " + rawFile);
			parser.parse(new InputSource(new FileReader(rawFile)));

			//free space of exec
			exec = null;

		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

	}

	/*
	 * @param rawFiles String[] the file names of xml files @param tokenFile
	 * String the target token file name tokenFile is the token format of an xml
	 * file which is the concatenation of the xml files each xml file has the
	 * same root element, these root elements however are stripped off, a root
	 * element is finally added to the concatenated token file
	 */
	private static void concatenateRawFiles(String[] rawFiles, String tokenFile) {
		try {
			XMLReader parser = XMLReaderFactory
					.createXMLReader("org.apache.xerces.parsers.SAXParser");

			ConcatEmptySAXHandler exec = new ConcatEmptySAXHandler(tokenFile);

			parser.setContentHandler(exec);

			parser.setFeature("http://xml.org/sax/features/validation", false);
			//The only way that the parser can determine if text is ignorable
			// is by reading the associated grammar and having a content model
			// for the document.
			//a dtd is needed always
			// parser.setFeature(
			//     "http://apache.org/xml/features/nonvalidating/load-external-dtd",
			//     true);

			int rawFileNum = rawFiles.length;
			for (int i = 0; i < rawFileNum; i++) {
				String rawFile = rawFiles[i];
				System.out.println("parsing " + rawFile);

				if (i == 0)
					exec.setFirstFile(true);
				else {
					exec.setFirstFile(false);
					if (i == rawFileNum-1)
						exec.setLastFile();
				}

				parser.parse(new InputSource(new FileReader(rawFile)));

			}

			//free space of exec
			exec = null;

		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

	}

	/**
	 * 
	 * Creation date: (5/29/2003 6:13:08 PM)
	 * 
	 * @param args
	 *            java.lang.String[]
	 */
	public static void main(String[] args) {
		//xml file, use regular file location rather than UDP file:///
		if (args.length == 2) {
			String rawFile = args[0];

			//target token file
			String targetFile = args[1];

			initializeTokenQueueToFile(rawFile, targetFile);

		} else {
			int rawFileNum = args.length - 1;
			String[] rawFiles = new String[rawFileNum];
			for (int i=0; i<rawFileNum; i++)
				rawFiles[i] = args[i];
				
			concatenateRawFiles(rawFiles, args[args.length-1]);		

		}
	}
}