package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration;

import java.io.Serializable;
import java.util.Enumeration;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDagImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATPropertiesImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator.QueryPlanGenerator;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Oct 27, 2003
 * 
 * Insert comments here.
 */
public class QueryPlan extends XATPropertiesImp implements Serializable{
	
	private XATDag queryPlan;
	private QueryPlanGenerator qpg;
	private int treeID;
	
	/**
	 * 
	 */
	public QueryPlan(int treeID) {
		super();
		queryPlan = null;
		qpg = null;
		this.treeID = treeID;
		setDefaultProperties();
	}
	
	public void setDefaultProperties(){
		super.setDefaultProperties();
		addProperty("FILE_NAME","none");
		addProperty("GENERATOR","none");
		addProperty("STREAMS","none");
		addProperty("QUERY_ID", "1");
		addProperty("edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator","none");
		
	}
	
	public void setQueryPlanGenerator(QueryPlanGenerator g) throws Exception{
		
		if(qpg != null) throw new Exception("The Queryplan generator has already been set!");
		this.qpg = g;
	}
	
	public void generateQueryPlan(){
		try{
			queryPlan = XATDagImp.convertTreeToDag(qpg.makeQueryPlan());	
			queryPlan.setProperty("TREE_ID",""+treeID);
			XATNode[] temp = queryPlan.convertToArray();
			for(int i=0; i< temp.length;i++){
				temp[i].setTreeID(""+treeID);
			}
		}catch(Exception e){ 
			e.printStackTrace(); 
		}

	}
	
	public int numberQueryPlanOperators(int startingNumber){
		int count = startingNumber;
		XATNode[] nodes = queryPlan.convertToArray();
		
		try{
			for(int i = 0; i < nodes.length ; i++){
				nodes[i].getOperator().setProperty("OBJECT_ID",Integer.toString(count));
				count++;
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return count;
		
	}
	
	public XATDag getQueryPlan() throws Exception{
		if(queryPlan == null) throw new Exception("The query plan has not been created yet!");
		return queryPlan;
	}
	
	public String toString(){
		String ret = new String();
		
		ret+= "Query Plan: " + "\n";
		ret+= "\t Properties: \t\t\t\t";
		Enumeration e = this.getProperties();
		while(e.hasMoreElements()){
			String name = (String) e.nextElement();
			String prop = this.getProperty(name);
			ret+= name + "=" + prop + ", ";
		}
		ret+="\n";
		ret+="\t XATTree: \n";
		ret += queryPlan.toString();
		return ret;
		
	}
	
	
}
