package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Mar 2, 2004
 * 
 * This class is an interface for a Connection between query objects. Tuples will 
 * be passed amongst this connection.
 * 
 * There are several possibilities for a connection.
 * 
 * - Local Machine
 * - Stream to Machine
 * - Machine to Machine
 * - Machine to Application
 * 
 * Because of this we keep track of the local and remote operator that
 * we will be connecting to.
 * 
 * From Location
 * --------------
 * 	IP Address
 *  Port
 *  Operator id
 *  Output Queue number
 * 
 * To Location
 * --------------
 * 	IP Address
 * 	Port
 *  Operator id
 * 	Input Queue number
 * 
 */
public interface Connection {


/** 
 * Returns the name of the connection class.
 * @return
 */
public String getClassName();

/**
 * Returns the IP address where the data will come from.
 * 
 * @return String -ipAddress
 */
public String getFromIPAddress() throws ConnectionException;

/**
 * Returns the ID for the operator where the data will come from.
 * 
 * @return String operatorID
 */

public String getFromOperator() throws ConnectionException;

/**
 * Returns the port where the data will come from.
 * 
 * @return int port
 */
public int getFromPort() throws ConnectionException;


/**
 * Returns the queue number from where the data will come from.
 * 
 * @return int queueNumber
 */
public int getFromQueueNumber() throws ConnectionException;

/**
 * Returns the amount of time for which the stream should be run.
 * 
 * @return
 * @throws ConnectionException
 */
public int getStreamLength() throws ConnectionException;

/**
 * Returns the stream name that will connect to the "to" operator
 * 
 * @return the Stream Name
 * @throws ConnectionException
 */
public String getStreamName() throws ConnectionException;

/**
 * Returns the IP address where the data will go to.
 * 
 * @return String -ipAddress
 */
public String getToIPAddress() throws ConnectionException;

/**
 * Returns the ID for the operator to where the data will go.
 * 
 * @return String operatorID
 */
public String getToOperator() throws ConnectionException;

/**
 * Returns the port where the data will go to.
 * 
 * @return int port
 */
public int getToPort() throws ConnectionException;

/**
 * Returns the queue number to where the data will go.
 * 
 * @return int queueNumber
 */
public int getToQueueNumber() throws ConnectionException;

/**
 * Returns the String representation of the connection
 * @return
 */
public String toString();
}
