package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection;

import java.io.ObjectOutputStream;
import java.util.Enumeration;
import java.util.Hashtable;
import java.io.Serializable;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 12, 2003
 * 
 * The connection table class is similar to a hash table but stores
 * specific information about the types of connections
 * in Raindrop There are two types of connections that are monitored:
 * 
 * 1. Incoming
 * 2. Outgoing
 * 
 * Incoming connections know the Operator ID from which a tuple came, and
 * the Operator for which that tuple is supposed to go.
 * 
 * Outgoing connections know all of the operators that are supposed
 * to output across the network and the socket connection to physically send
 * the tuples.
 * 
 */
public class ConnectionTable implements Serializable {

	private Hashtable table;
	/**
	 * Default Constructor
	 */
	public ConnectionTable() {
		this.table = new Hashtable();
	}
	
	// These are all simple get and set methods.

	public synchronized void addConnection(String tuplename,OperatorQueue oq){
		table.put(tuplename,oq);
	}
	
	public synchronized void addConnection(XATStreamOperator operator, ObjectOutputStream os){
		table.put(operator,os);
	}
	
	public synchronized int connectionCount(){
		return this.table.size();
	}
	
	public synchronized OperatorQueue getConnection(String tuplename){
		return (OperatorQueue) table.get(tuplename);
	}
	
	public synchronized ObjectOutputStream getConnection(XATStreamOperator operator){
		return (ObjectOutputStream) table.get(operator);
	}
	
	public synchronized Enumeration getKeys(){
		return this.table.keys();
	}
	
	public synchronized Enumeration getValues(){
		return this.table.elements();
	}
	
	public synchronized void removeConnection(String tuplename){
		this.table.remove(tuplename);
	}
	
	public synchronized void removeConnection(XATStreamOperator operator){
		this.table.remove(operator);
	}
	
	public synchronized void clearTable(){
		this.table.clear();
	}
	
	public String toString(){
		String ret = new String();
		
		Enumeration e= table.keys();
		
		ret+="    Connection Table     \n";
		ret+="-------------------------\n";
		
		while(e.hasMoreElements()){
			Object o = e.nextElement();
			ret+= o +  "=>" + table.get(o) +"\n";
		}
		
		return ret;
	}
}
