/*
 * Created on Mar 13, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel;


import java.util.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.*;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * The abstract class cost model represents the cost model calculation for a machine. 
 * Given certain statistics about a machine, a concrete CostModel class will
 * calculate the "cost" or fullness of the server based on the particular model of the class.
 * 
 */
public abstract class CostModel {

	public final int MAXIMIZE = 1;
	public final int MINIMIZE = 0;
	
	protected ArrayList requiredStats;
	protected StatisticsTable st;
	protected DistributionTable dt;
	
	/**
	 * Returns an iterator of statistics that are required for the cost model.
	 * @return Iterator -statistics required by the cost model.
	 * 
	 */
	public abstract Iterator requiredStatistics();
	public abstract CostTable calculateCost(StatisticsTable st, DistributionTable dt);
	protected void setStatistics(){}
	
	public CostTable linearLormalize(CostTable ct, int quantifier){
		CostTable c = ct;
		boolean maximize = false;
		double maxCost = 0, minCost = 0;
		
		if(quantifier == MAXIMIZE)
			maximize = true;
			
		if(maximize){
			maxCost = c.maxRawCost();
			double normalizedCost;
			Enumeration e = ct.getMachines();
			while(e.hasMoreElements()){
				Machine m = (Machine) e.nextElement();
				double rawCost = c.getRawCost(m);
				if(maxCost == 0){
					normalizedCost = 0;
				}
				else {
					normalizedCost = rawCost/maxCost;
				}
				
				c.put(m,normalizedCost);
			}
		}
		else{
		
			minCost = c.minRawCost();
		}
			
			
		
		
		
		return c;
		
	}
}
