package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel;

import java.util.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Mar 16, 2004
 * 
 * Insert comments here.
 */
public class NumTuplesInQueue extends CostModel {

	/**
	 * Default Constructor
	 * @param st -The statistics table from which to calculate the cost from
	 */
	public NumTuplesInQueue(StatisticsTable st){
		this.st = st;
		this.requiredStats = new ArrayList();
		setStatistics();
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.CostModel#requiredStatistics()
	 */
	public Iterator requiredStatistics() {
		return this.requiredStats.iterator();	
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.CostModel#setStatistics()
	 */
	protected void setStatistics() {
		super.setStatistics();
		//this.requiredStats.add("TOTAL_TUPLES_IN_QUEUES");
		this.requiredStats.add("NUMBER_OF_TUPLES_IN_INPUT_QUEUES");
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.CostModel#calculateCost()
	 */
	public CostTable calculateCost(StatisticsTable st, DistributionTable dt) {
		this.dt = dt;
		this.st = st;
		CostTable ct = new CostTable(st.size());
		
		try{
		
		for(Enumeration e = st.keys();e.hasMoreElements();){
			Machine m = (Machine) e.nextElement();
			LightWeightStatistic lws = st.get(m);
			double tuples = 0;
			
			//First, calculate the raw load on each machine.
			for(Enumeration en = lws.getKeys(); en.hasMoreElements();){
				LightWeightQueryObject xqo = (LightWeightQueryObject) en.nextElement();
				if(xqo.isActive()){
					tuples+= Double.parseDouble(lws.get(xqo,"NUMBER_OF_TUPLES_IN_INPUT_QUEUES"));
					ct.putRawCost(m,tuples);
				}
				
			}
			
			//Second, calculate the normalized load.
			ct = this.linearLormalize(ct,MAXIMIZE);
			
			//	Finally calculate the shared load for each operator on the machine.

			ArrayList nodes = dt.getOperatorsOnMachine(m);
			
			for(Iterator i = nodes.iterator(); i.hasNext();){
				XATNode node = (XATNode) i.next();
				LightWeightQueryObject xqo = lws.getQueryObject(node);
				
				double thisQueue = Double.parseDouble(lws.get(xqo,"NUMBER_OF_TUPLES_IN_INPUT_QUEUES"));
				double cost = thisQueue/tuples;
				NodeCost nc = new NodeCost(node,cost);
				ct.put(m,nc);
			}
		}
		
		}catch (Exception e){
			e.printStackTrace();
		}
		
		return ct;
	}

}
