package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel;

import java.util.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Mar 16, 2004
 * 
 * Insert comments here.
 */
public class PctMemoryUsed extends CostModel {

	/**
	 * Default Constructor
	 * @param st -The statistics table from which to calculate the cost from
	 */
	public PctMemoryUsed(StatisticsTable st){
		this.st = st;
		this.requiredStats = new ArrayList();
		setStatistics();
	}
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.CostModel#requiredStatistics()
	 */
	public Iterator requiredStatistics() {
		return this.requiredStats.iterator();	
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.CostModel#setStatistics()
	 */
	protected void setStatistics() {
		super.setStatistics();
		this.requiredStats.add("USED_MEMORY_PERCENTAGE");
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.CostModel#calculateCost()
	 */
	public CostTable calculateCost(StatisticsTable st, DistributionTable dt) {
		this.dt = dt;
		this.st = st;
		CostTable ct = new CostTable(st.size());
		
		for(Enumeration e = st.keys();e.hasMoreElements();){
			Machine m = (Machine) e.nextElement();
			LightWeightStatistic lws = st.get(m);
			double mem = 0;
			
			//First, calculate the raw load on each machine.
			for(Enumeration en = lws.getKeys(); en.hasMoreElements();){
				LightWeightQueryObject xqo = (LightWeightQueryObject) en.nextElement();
				mem = Double.parseDouble(lws.get(xqo,"USED_MEMORY_PERCENTAGE"));
				ct.putRawCost(m,mem);
				
			}
			
			//Second, calculate the normalized load.
			ct = this.linearLormalize(ct,MAXIMIZE);
			
			//	Finally calculate the shared load for each operator on the machine.

			ArrayList nodes = dt.getOperatorsOnMachine(m);
			int count = nodes.size();
			
			double avgCost = 0;
			if(count > 0)
				avgCost = ct.getMachineCost(m)/count;
				
				System.out.println(avgCost + " = " + ct.getMachineCost(m) +"/"+count + "(" +m.getMachineName()+")");
				
			for(Iterator i = nodes.iterator(); i.hasNext();){
				XATNode node = (XATNode) i.next();
				NodeCost nc = new NodeCost(node,avgCost);
				ct.put(m,nc);
			}
		}
		
		return ct;
	}

}
