package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.manager;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.ObjectOutputStream;
import java.lang.reflect.Constructor;
import java.net.ConnectException;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Date;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;
import jxl.Workbook;
import jxl.write.Label;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.DistributionConfiguration;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Stream;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.ConnectionManager;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.ConnectionManagerImp;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.DistributionTable;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.DistributionValidator;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.XATDesignPattern;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.redistribution.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Oct 28, 2003
 * 
 * The DistributionManager is responsible for all of the distribution
 * for Query Execution. It knows all of the query plans, machines to use,
 * streams to connect to, and applications that are listening for results.
 * 
 * The manager is responsible for deciding how to distribute the query plan
 * and tell the machines who does work, and where to send
 * the information. These are the main components:
 * 
 * --Distribution Creator
 * --Connection Sender
 * --Query Plan Validator
 * @TODO fill this in
 */
public class DistributionManager implements Runnable {

	private DistributionConfiguration config;

	/**
	 * Default Constructor.
	 * 
	 * @param DistributionConfiguration - The configuration object.
	 */
	public DistributionManager(DistributionConfiguration config) {
		this.config = config;
		init();
	}

	/**
	 *  
	 * Initializes the Manager.
	 *
	 */

	private void init() {

		ArrayList cantConnect = new ArrayList();

		// Send the configuration information to all of the other servers
		System.out.println(
			"Sending configuration information to "
				+ config.machineCount()
				+ " machine(s).");
		for (Iterator i = config.getMachines(); i.hasNext();) {
			Machine m = (Machine) i.next();
			String ipAddress = m.getIPAddress();
			int port = m.getPort();
			System.out.println("Connecting to " + ipAddress + ":" + port);
			try {

				Socket socket = new Socket(ipAddress, port);
				ObjectOutputStream out =
					new ObjectOutputStream(socket.getOutputStream());

				out.writeObject(config);
				System.out.println(
					"Successfully sent distribution information to:"
						+ m.getMachineName()
						+ " @ "
						+ m.getIPAddress()
						+ ":"
						+ m.getPort());
				socket.close();

			} catch (ConnectException ce) {

				System.out.println(
					"Cannot connect to "
						+ ipAddress
						+ ":"
						+ port
						+ "\n"
						+ ce.getMessage());

				// add this machine to be removed after the loop
				// we cant remove them here because we would get
				// a concurrent modification exception.
				cantConnect.add(m);
			} catch (Exception e) {
				e.printStackTrace();
			}

		}
		// Remove the machines we couldnt connect to
		if (cantConnect.size() > 0)
			System.out.println(
				"Could not connect to " + cantConnect.size() + " machine(s)!");
		for (int i = 0; i < cantConnect.size(); i++) {
			Machine r = (Machine) cantConnect.get(i);
			config.removeMachine(r.getIPAddress(), r.getPort());
			System.out.println(
				"Removed: " + r.getIPAddress() + ":" + r.getPort());
		}
	}

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	public void run() {

		if (config.machineCount() == 0) {
			System.out.println("We have no machines to do work. Now Exiting.");
			return;
		}

		try {
			System.out.println("Starting Distribution Manager.");
			
			//Initialize
			//Threads
			ResultReceiver rr;
			StatisticsReceiver sr;
			
			//Tables
			Hashtable appTable = new Hashtable(4);
			Hashtable serverTable = new Hashtable(4);
			DistributionTable table;
			CostTable costTable = new CostTable(config.getMachineArray());
			StatisticsTable statisticsTable = new StatisticsTable(config.getMachineArray());
			
			//Properties
			int reDistDelay = Integer.parseInt(config.getProperty("REDISBRIBUTION_TIME"));
			int executionTime = Integer.parseInt(config.getProperty("EXECUTION_DURATION"));
			int distDelay = Integer.parseInt(config.getProperty("DISTRIBUTION_DELAY"));
			String distributionPatten = config.getProperty("DISTRIBUTION_PATTERN");
			String costModel = config.getProperty("WORKLOAD_COST_MODEL");
			int streamTime = Integer.parseInt(config.getProperty("STREAM_DURATION"));
			double reDistPct = Double.parseDouble(config.getProperty("REDISTRIBUTION_PERCENT")) / 100.0;
			int maxStateSize = Integer.parseInt(config.getProperty("STATE_SIZE_THRESHOLD"));
			String scope = config.getProperty("REDISTRIBUTION_SCOPE");
			String redistributionPolicy = config.getProperty("REDISTRIBUTION_POLICY");
			
			long startTime = System.currentTimeMillis();
			long currentTime = startTime;
			executionTime -= distDelay;
			boolean global = false;
			ArrayList lightStats = new ArrayList();
			
			//Distribution Objects
			ConnectionManager cm;
			XATDesignPattern pattern = null;
			DistributionValidator validate;
			XATRedistributionPolicy rp = null;
			CostModel c = null;
			
			//Spreadsheet Objects
			Date d;
			WritableWorkbook workbook;
			WritableSheet[] sheets;
			
			if(maxStateSize < 0){
				//there is no maximum threshold for state size
				maxStateSize = Integer.MAX_VALUE;
			}
			
			if(scope.equals("global")){
				global = true;
			}
			
			// Start the result receiver
			
			rr = new ResultReceiver(22000, config.machineCount());
			
			Thread tT = new Thread(rr);
			tT.setName("Result Receiver");
			tT.start();
			
			// Start the statistics receiver
			
			sr = new StatisticsReceiver(23000, statisticsTable, config.getMachineArray());
			
			Thread sT = new Thread(sr);
			sT.setName("Statistics Receiver");
			sT.start();
			
			// Setup the cost model
			Constructor[] conf = Class.forName(costModel).getConstructors();
			Constructor cmodelConstructor;

			for(int i = 0; i < conf.length; i++){
				cmodelConstructor = conf[i];
				Class[] parameters = cmodelConstructor.getParameterTypes();
				
				if(parameters.length == 1){
					c = (CostModel) cmodelConstructor.newInstance(new Object[] {statisticsTable});
				}
			}
			
			if(c == null){
				throw new Exception("The Cost Model could not be instantiated.");
			}
			
			//Find out what statistics to send back to the machines
			Iterator iter = c.requiredStatistics();
			while(iter.hasNext()){
				String temp = (String) iter.next();
				lightStats.add(temp);
			}

			// Sleep while waiting to distribute
			// this is done to allow the clients sufficient time to start up.
			Thread.sleep(distDelay);
			
			// Send the statistics	
			LightWeightStats lw = new LightWeightStats(lightStats);
			Packet p = new Packet("lws",lw);
			System.out.print("Sending statistics to machines...");
			for(Iterator it = config.getMachines(); it.hasNext();){
				Machine m = (Machine) it.next();
				Socket s = new Socket(m.getIPAddress(),m.getConnectionListenerPort());
				ObjectOutputStream out = new ObjectOutputStream(s.getOutputStream());
				out.writeObject(p);
				out.close();
				s.close();
			}
			System.out.print("done.\n");
			
			// Put all of the available applications in the table
			// with the key being the queryplan to which it is to connect
			
			Application[] a = config.getApplicationArray();
			for(int i = 0; i < a.length; i++){
				appTable.put(a[i].getProperty("CONNECTS_TO"),a[i]);
			}
			
			//Find out waht all the servers are by streamname
			//put them in another table
			Stream[] s = config.getStreamArray();
			for(int i = 0; i < s.length; i++){
				serverTable.put(s[i].getName(),s[i].getServer());
			}	

			// Create the distribution pattern
			Constructor[] con = Class.forName(distributionPatten).getConstructors();
			Constructor patternConstructor;

			for(int i = 0; i < con.length; i++){
				patternConstructor = con[i];
				Class[] parameters = patternConstructor.getParameterTypes();
				
				if(parameters.length == 2){
					pattern = (XATDesignPattern) patternConstructor.newInstance(new Object[] {config.getMachineArray(), config.getQueryPlanArray()});
				}
			}
			
			if(pattern == null){
				throw new Exception("The Redistribution Policy could not be instantiated.");
			}
			
			//DISTRIBUTE THE OPERATORS
			
			table = pattern.distribute();
			//Validate the distribution
			validate = new DistributionValidator(config.getQueryPlanArray());
			
			if (validate.validate(table)) {
				//Connect the operators to the machines
				cm = new ConnectionManagerImp(table,serverTable,appTable,streamTime);
				cm.connect();
			}
			else{
				System.out.println("This is an illegal distribution.");
				return;
			}
			
			System.out.println("Completed Initial distribution.");
			
			// Create the redistribution policy
			Constructor[] cons = Class.forName(redistributionPolicy).getConstructors();
			Constructor reDistConstructor;
			
			for(int i = 0; i < cons.length; i++){
				reDistConstructor = cons[i];
				Class[] parameters = reDistConstructor.getParameterTypes();
				
				if(parameters.length == 0){
					rp = (XATRedistributionPolicy) reDistConstructor.newInstance(new Object[] {});
				}
			}
			
			if(rp == null){
				throw new Exception("The Redistribution Policy could not be instantiated.");
			}
			
			System.out.println(table);
			
			int count = 0;
			Machine[] machineArray = config.getMachineArray();
			
			do{
				System.out.println(count);
				double max,min;
				System.out.print("Redistributing..");
				Thread.sleep(reDistDelay);
				System.out.println(statisticsTable);
				
				costTable = c.calculateCost(statisticsTable, table);
				System.out.println(costTable);
				max = costTable.maxCost();
				min = costTable.minCost();
				
				if(reDistPct < max - min){
					System.out.println("The reDist is big enough to redistirbute");
					
					//Comment out for now.
					//Until testing is done.
					//DistributionTable newTable = rp.redistribute(table,costTable,null,config.getMachineArray());
					
					DistributionTable newTable = (DistributionTable) table.clone();
					XATNode nodeToMove = newTable.getNode("2");
					Machine tempMachine = machineArray[count];
					newTable.add(nodeToMove,tempMachine);
					
					System.out.println("New Distribution:");
					System.out.println(newTable);
					
					
					cm.reconnect(newTable);	
					table = newTable;				
				}
				
				System.out.print("done.\n");
				
				
				
				currentTime = System.currentTimeMillis();
				
				if(count >= machineArray.length-1){
					count = 0;
				}
				else{
					count++;
				}
			}while(currentTime - startTime < executionTime);
			
			System.out.println("Waiting for results from " + config.machineCount() + " machines.");
			
			while(!rr.isDone())
					Thread.sleep(2000);
			
			//Request that the Result Receiver stop.
			rr.requestStop();
					
			System.out.print("Received all of the results files, now creating master file...");
			d = new java.util.Date();
			
			
			// CREATE THE EXCEL WORKBOOK
			workbook = Workbook.createWorkbook(new File("Result-" + d.getMonth() + "-" + d.getDate() + "-" + d.getHours() + "-" + d.getMinutes() + ".xls"));
			sheets = new WritableSheet[rr.getNumFileNames() +2];
			
			//Add the Parameter worksheet
			sheets[0] = workbook.createSheet("Experiment Parameters",0);
			
			StringTokenizer st = new StringTokenizer(config.toString(),"\n");
			int i=0;
			while(st.hasMoreTokens()){
				String temp = st.nextToken();
				Label label = new Label(0, i, temp); 
				sheets[0].addCell(label); 
				i++;
			}
						
			int j=1;
			int k=0;
			int l=0;
			
			//Add one worksheet for each machine.
			
			for(Iterator it = rr.getFileNames();it.hasNext();){
				String name = (String) it.next();
				String line;
				sheets[j] = workbook.createSheet(name,j);
			
				//Now add in the data
				
				BufferedReader br = new BufferedReader(new FileReader(name));
				k=0;
				while((line = br.readLine()) != null){
					StringTokenizer tr = new StringTokenizer(line,",");
					
					l =0;
					while(tr.hasMoreTokens()){
						String token = tr.nextToken();
						Label label = new Label(l,k,token);
						sheets[j].addCell(label);
						l++;
					}
					k++;
				}
				
				j++;
			}
			//now make the experiment worksheet.
			sheets[j] = workbook.createSheet("Charts",j);

			workbook.write(); 
			workbook.close(); 
			
			// END EXCEL WORKBOOK
			
			System.out.print("done.\n");
			System.exit(0);
			return;
			

		} catch (Exception e) {
			e.printStackTrace();
		}

	}
}
