package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.manager;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.StatisticsTable;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.LightWeightStatistic;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 8, 2003
 * 
 * Insert comments here.
 */
public class StatisticsReceiver extends Thread implements Runnable {

	/**Stores the reference to the port number*/
	private int port = -1;
	/** The packet pool we will add all incoming tuples to. */

	private ArrayList p;
	private boolean keepGoing;
	private boolean done = false;
	private StatisticsTable table;
	private Machine[] m;
	
	/**
	 * Creates a new Package receiver for the execution controller.
	 */
	public StatisticsReceiver(int port, StatisticsTable sTable, Machine[] m) throws IOException {
		super();
		this.port = port;
		this.p = new ArrayList();
		keepGoing = true;
		this.m = m;
		table = sTable;
		done = false;
	}

	/* Runs the Packet Receiver
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			ServerSocket socket = new ServerSocket(this.port);
			System.out.println("Starting StatisticsReceiver on port: " + port);
			int i = 0;
			while (keepGoing) {
				Socket inc = socket.accept();
				StatisticReceiverHandler tmp =
					new StatisticReceiverHandler(inc, i, m, table);
				tmp.setName("StatisticsReceiverHandler" + i);
				tmp.setDaemon(true);
				tmp.start();
				p.add(tmp);
				i++;
			}

		} catch (IOException ioe) {
			System.out.println(
				"Could not start the Statistics Receiver: " + ioe.getMessage());
		}
	}

	public void requestStop() {

		for(Iterator i = p.iterator();i.hasNext();){
			StatisticReceiverHandler stop = (StatisticReceiverHandler) i.next();
			stop.requestStop();
		}

		keepGoing = false;
		interrupt();

	}

}
/**
 * 
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 8, 2003
 * 
 * The PacketReceiverHandler class is responsible for
 * maintaining a connection between the local execution engine
 * and a remote engine. The PacketReceiver will spawn one of these threads
 * whenever a Packet from a new location is sent.
 * 
 */
class StatisticReceiverHandler extends Thread {

	private Socket incoming;

	private int handlerNumber;
	private boolean enable;
	private StatisticsReceiver parent;
	private String machineName;
	private Machine[] m;
	private Machine thisMachine;
	private StatisticsTable table;
	
	public StatisticReceiverHandler(
		Socket i,
		int handlerNumber, Machine[] m, StatisticsTable table) {
		this.incoming = i;
		this.handlerNumber = handlerNumber;
		this.enable = true;
		this.machineName = new String("empty");
		this.table = table;
		this.m = m;
	}
	
	public void requestStop(){
		this.enable = false;
	}

	public synchronized void run() {

		try {

			ObjectInputStream in =
				new ObjectInputStream(incoming.getInputStream());

			while (enable) {
				
				Object o = in.readObject();

				LightWeightStatistic lws = (LightWeightStatistic) o;
				
				
				if(this.machineName.equals("empty")){
					this.machineName = lws.getMachineName();
					for(int i=0;i<m.length;i++){
						if(m[i].getMachineName().equals(this.machineName))
							thisMachine = m[i];
					}
				}
	
				table.put(thisMachine,lws);
				
			}
			
			return;

		} catch (EOFException eofe) {
			try {

				incoming.close();

			} catch (Exception e) {
				System.out.println(e.getMessage());
			}

		} catch (SocketException se) {
			System.out.println(
				"Connection Failed: Cannot get stream data from "
					+ incoming.getInetAddress()
					+ ":"
					+ incoming.getPort());
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

}
