package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet;

import java.io.Serializable;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Jan 30, 2004
 * 
 * The network packet class is simply an array of Packet objects to send
 * across the network.
 */
public class NetworkPacket implements Serializable {
	private int cursor;

	private Packet[] packets;
	private int size;
	
	/**
	 * Default constructor, a network packet of size 1000 will be created
	 */
	public NetworkPacket() {
		this.size = 100;
		this.cursor = 0;
		packets = new Packet[size];
	}
	
	/**
	 * Default constructor
	 * 
	 * @param size -The number of Packets this object can hold.
	 */
	
	public NetworkPacket(int size){
		
		this.size = size;
		this.cursor = 0;
		packets = new Packet[this.size];
	}
	
	/**
	 * Adds a packet to the Network Packet.
	 * 
	 * @param p - The packet to add.
	 * @throws ArrayIndexOutOfBoundsException
	 */
	public void addPacket(Packet p) throws ArrayIndexOutOfBoundsException{
		
		packets[cursor] = p;
		this.cursor++;
	}
	
	/**
	 * Gets a packet from the Network Packet from the specific location.
	 * 
	 * @param location -The location of the packet we wish to get.
	 * @return The packet from the location.
	 */
	public Packet getPacket(int location){
		return packets[location];
	}
	
	
	/**
	 * Returns the size of the network packet.
	 * @return The size of the network packet.
	 */
	public int size(){
		return this.cursor;
	}
	
	public int totalSize(){
		return this.size;
	}
	
}
