package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet;

import java.io.NotSerializableException;
import java.io.Serializable;
import java.util.Enumeration;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.ConnectionTable;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 15, 2003
 * 
 * 
 * 
 */
public class TupleDistributor implements Runnable, Serializable {

	private ConnectionTable table;
	private boolean enable;
	
	/**
	 * @param port
	 * @param receivePool
	 * @param tree
	 * @throws IOException
	 */
	public TupleDistributor(ConnectionTable ct){
		this.table = ct;	
		this.enable = true;	
	}
	
	public void run(){

		XATStreamOperator op = null;
		Packet p = null;
		NetworkPacket np = null;
		XATTuple[] tuples;
		Enumeration e;
		int queueSize;
		int dQ;
		XATQueueSchema xqs;
		
		while(enable){
		
			try{
				// Scan through the operators that are supposed to output and
				// take the tuples out of their queues, turn them into packets, and
				// put them into the tuple pool
				
				
				// Get the current keys in the table
				// Remember that the table will change so we will need to get
				// the keys after every iteration to get the changes.
				
				e = table.getKeys();
				
				while(e.hasMoreElements()){
				
					//Try to fetch the value, if it cannot be fetched
					//It may have been removed by another object, simply disregard it.
					try{
	
						op = (XATStreamOperator) e.nextElement();	
						queueSize = op.getOutputQueue().getQueueSize(this);
						
						// Register to remove stuff from the output queue
						if(!op.getOutputQueue().isRegistered(this)){
							op.getOutputQueue().register(this);
						}
						
						if(queueSize > 0){	
							dQ = (int) (queueSize);
							tuples = op.getOutputQueue().dequeue(dQ, this);
							np = new NetworkPacket(dQ+1);
							
							// Put the schema in.
							
							xqs = op.getOutputQueue().getSchema();
							p = new Packet(op.getProperty("OBJECT_ID"),xqs,"schema");
							np.addPacket(p);
							
							// Add each tuple to a network packet
							for(int i=0; i< tuples.length; i++){
								p = new Packet(op.getProperty("OBJECT_ID"),tuples[i],"");
								np.addPacket(p);
							}
							
							table.getConnection(op).reset();
							table.getConnection(op).writeObject(np);
						}
						
						
					} catch (NullPointerException ne) {

						System.out.println(
							"The key "
								+ op.toString()
								+ " no longer exists in the output table.");
						continue;
					} catch (IllegalArgumentException iag) {
						System.out.println("============= IllegalArgumentException ============");
						System.out.println(iag.getLocalizedMessage());
						System.out.println(iag.getMessage());
						System.out.println(op.toString());
						System.out.println(op.getOutputQueue().getSchema());
						iag.printStackTrace();
						System.out.println("====================================");
					}
				}
				}
			catch(XATQueueException xqe){
				System.out.println("============= XATQueueException ============");
				System.out.println(op);
				xqe.printStackTrace();
				System.out.println("====================================");
			}
			catch(NotSerializableException nse){
				System.out.println("===== NotSerializableException =====");
				System.out.println(nse.getMessage());
				nse.printStackTrace();
				System.out.println("====================================");
				
			}
			catch(Exception ex){
				System.out.println("============= Exception ============");
			 	ex.printStackTrace();
				System.out.println("====================================");
			}
		}
	}
	
	public void requestStop(){
		this.enable = false;
	}
}
