package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet;
import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
//import java.io.*;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import java.util.ArrayList;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 8, 2003
 * 
 * Insert comments here.
 */
public class TupleReceiver extends Thread implements Runnable {

	/**Stores the reference to the port number*/
	private int port = -1;
	/** The packet pool we will add all incoming tuples to. */
	private PacketPool receivePool = null;
	private ArrayList tree;
	private ArrayList p;
	private boolean keepGoing;

	/**
	 * Creates a new Package receiver for the execution controller.
	 */
	public TupleReceiver(int port, PacketPool receivePool, ArrayList tree)
		throws IOException {
		super();
		this.port = port;
		this.receivePool = receivePool;
		this.tree = tree;
		this.p = new ArrayList();
		keepGoing = true;
	}

	/* Runs the Packet Receiver
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			ServerSocket socket = new ServerSocket(this.port);
			System.out.println("Starting TupleReceiver on port: " + port);
			int i = 0;
			while (keepGoing) {
				Socket inc = socket.accept();
				
				PacketReceiverHandler tmp =  new PacketReceiverHandler(inc, receivePool, i);
				tmp.setName("PacketReceiverHandler" + i);
				tmp.setDaemon(true);
				tmp.start();
				p.add(tmp);
				i++;
			}
		} catch (IOException ioe) {
			System.out.println(
				"Could not start the Tuple Receiver: " + ioe.getMessage());
		}
	}

	public void requestStop() {
		System.out.println("Trying to interrupt the threads");

		//Request that each sub-thread is stopped.
		for (int j = 0; j < p.size(); j++) {
			
			PacketReceiverHandler stopHand = (PacketReceiverHandler) p.get(j);
			
			stopHand.requestStop();
		}

		keepGoing = false;
		interrupt();

	}
}
/**
 * 
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 8, 2003
 * 
 * The PacketReceiverHandler class is responsible for
 * maintaining a connection between the local execution engine
 * and a remote engine. The PacketReceiver will spawn one of these threads
 * whenever a Packet from a new location is sent.
 * 
 */
class PacketReceiverHandler extends Thread {

	private Socket incoming;
	private PacketPool pool;
	private int handlerNumber;
	private boolean keepGoing;

	public PacketReceiverHandler(Socket i, PacketPool p, int handlerNumber) {
		this.incoming = i;
		this.pool = p;
		this.handlerNumber = handlerNumber;
		this.setPriority(MAX_PRIORITY);
		this.keepGoing = true;
	}

	public synchronized void run() {
		// 	Listen to incoming packets and place them into the Packet Pool.
		Packet p = null;
		Object o = null;
		NetworkPacket np = null;

		try {
			
			//File f = new File("received_from_stream_" + handlerNumber + ".txt");
			//FileWriter outWriter = new FileWriter(f);

			ObjectInputStream in =
				new ObjectInputStream(incoming.getInputStream());
			while (keepGoing) {

				//Receive the new Packets				
				o = in.readObject();
				np = (NetworkPacket) o;
				//System.out.println("Received a packet with" + np.size() + " tuples.");

				for(int j=0; j < np.size();j++){
					
					p = np.getPacket(j);
					
					pool.addPacket(p);
					//outWriter.write(p.toString() + "\n");
				}
			}
			
			incoming.close();
		}catch (EOFException eofe){
			try{
			
			 incoming.close();
			 this.requestStop();
			 
			}catch(Exception e){
				System.out.println(e.getMessage());
			}
			 
		}
		catch (SocketException se) {
			System.out.println(
				"Connection Failed: Cannot get stream data from "
					+ incoming.getInetAddress()
					+ ":"
					+ incoming.getPort());
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		
	}

	public void requestStop() {
		keepGoing = false;
	}
}
