package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.ConnectException;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Application;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Server;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.Activate;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.Connection;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.DeActivate;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.ReceiveData;
//import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.ReceiveStream;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.SendData;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.SendState;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.SendStream;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.StopSend;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.Packet;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Feb 17, 2004
 * 
 * Insert comments here.
 */
public class ConnectionManagerImp implements ConnectionManager {

	private DistributionTable currentTable;
	private Hashtable serverTable;
	private Hashtable appTable;
	private int streamTime;
	
	/**
	 * 
	 */
	public ConnectionManagerImp(DistributionTable table, Hashtable servers, Hashtable applications, int st) {
		currentTable = table;
		serverTable = servers;
		appTable = applications;
		streamTime = st;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.ConnectionManager#connect()
	 */
	public boolean connect() {
		
		Socket currSocket = null;
		ObjectOutputStream currOut;
		
		try{
		
		Enumeration e = currentTable.getNodeEnumeration();
		
		//Connect all of the nodes
		while(e.hasMoreElements()){
			//Thread.sleep(100);
			XATNode currentNode = (XATNode) e.nextElement();
			currSocket = new Socket(currentTable.getMachine(currentNode).getIPAddress(),currentTable.getMachine(currentNode).getConnectionListenerPort());
			currOut = new ObjectOutputStream(currSocket.getOutputStream());
			
			//See if it connects to a stream
			if(currentNode.getStreamSize() > 0){
				//Get all of the streams it is connected to
				Iterator i = currentNode.getStreamNames();
				while(i.hasNext()){
					String streamName = (String) i.next();
					Server server = (Server) this.serverTable.get(streamName);
				
					int queueNum = 0;

					XATQueue q = ((XATStreamOperator)currentNode.getOperator()).getInputQueue(streamName);
					queueNum = q.getQueueNumber();
					
					//Create the connection object
					Connection c = new SendStream(streamName,streamTime,currentTable.getMachine(currentNode),currentNode.getOperator().getProperty("OBJECT_ID"),queueNum);
					Packet p = new Packet("connect",c);
					
					//	Send the packet to the stream generator
					Socket sgSocket = new Socket(server.getIPAddress(),server.getPort());
					ObjectOutputStream sgOut = new ObjectOutputStream(sgSocket.getOutputStream());
					sgOut.writeObject(p);
					sgOut.close();
					sgSocket.close();
				}
			}
			
			// Now activate the node on the machine
			
			Connection c = new Activate(currentNode.getOperator().getProperty("OBJECT_ID"));
			Packet p = new Packet("local",c);
			
			currOut.writeObject(p);
							
			// Now look and see where the parent is and connect it to the parent.
			
			Iterator it = currentNode.getParents();
	
			while(it.hasNext()){
			
				//Remember there may be more than 1 parent
				XATNode parentNode = (XATNode) it.next();
				
				if(currentTable.getMachine(currentNode).equals(currentTable.getMachine(parentNode))){
					//The parent is on the same machine as the child, so this will be connected later.
				}
				else{
					//The parent is on another machine so tell our machine how to connect it, and then
					//let the other machine know that it will be getting tuples from the child.
					
					//@TODO RIGHT NOW WE ONLY SUPPORT TREES, HERE IS WHERE THE CHANGE WILL HAVE TO BE MADE TO SUPPORT DAG's
					
					int queueNum = ((XATStreamOperator) currentNode.getOperator()).getOutputQueue().getQueueNumber();
					
					Connection fc = new SendData(currentNode.getOperator().getProperty("OBJECT_ID"),queueNum,currentTable.getMachine(parentNode));
					Packet np = new Packet("machines",fc);
					
					//Send it to the childs machine

					currOut.writeObject(np);
				
					//Now tell the parent about the tuples it will be receiving
					Connection tc = new ReceiveData(currentNode.getOperator().getProperty("OBJECT_ID"),parentNode.getOperator().getProperty("OBJECT_ID"),queueNum);

					Packet mp = new Packet("machines",tc);
					
				
					//Send it to the parents machine
					Socket parentSocket = new Socket(currentTable.getMachine(parentNode).getIPAddress(),currentTable.getMachine(parentNode).getConnectionListenerPort());
					ObjectOutputStream parentOut = new ObjectOutputStream(parentSocket.getOutputStream());
					parentOut.writeObject(mp);
					parentOut.close();
					parentSocket.close();						

				}
				
			}

			//If the iterator is null, there are no parents so connect it to an application
			if(currentNode.getParentSize() == 0){
				

				String currID = currentNode.getTreeID();
				Application currApp = (Application) appTable.get(currID);
				Connection ac = new SendData(currentNode.getOperator().getProperty("OBJECT_ID"),0,currApp.getIPAddress(),currApp.getPort());
				Packet ap = new Packet("connect",ac);
				//Send this info the the stream generator

				currOut.writeObject(ap);
			}
			
			currOut.close();
			currSocket.close();
			
		} //finished with all of the nodes 
		
		}catch(ConnectException ce){
		System.out.println("=======ConnectException=========");
		System.out.println("Message: " + ce.getMessage());
		System.out.println("Cause " + ce.getCause());
		System.out.println(currSocket);
		ce.printStackTrace();
		System.out.println("=========================");
	}
		catch(Exception e){
			System.out.println("=======Exception=========");
			e.printStackTrace();
			System.out.println("=========================");
		}
		
		//PHEW DONE
		return true;
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.ConnectionManager#reconnect(edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.DistributionTable)
	 */
	public boolean reconnect(DistributionTable newTable) {
		
		try {
			DistributionTable diffTable = currentTable.difference(newTable);
			
			System.out.println("Difference:");
			System.out.println(diffTable);
			
			//For every node we will move, do the following steps:
			// NOTATION: (N) = New Machine, (O) = Old Machine, OQ = Output Queue
			//1. Connect the XATNode(N)'s OQ to Parent Node
			//2. Connect XATNode(O)'s Childrens OQ's to XATNode(N)
			//3. Deactivate XATNode(O) when not running
			//4. Send XATNode(O)'s state to XATNode(N)
			//5. Stop XATNode(0)'s oq's when they are empty
			//6. Activate XATNode(N)
			for(Enumeration e = diffTable.getNodeEnumeration(); e.hasMoreElements();){
			
				// Get the node and other essential information
				XATNode currNode = (XATNode) e.nextElement();
				XATNode parentNode = currNode.getParent();
				XATNode children[] = currNode.getChildren();
				Machine parentMachine = currentTable.getMachine(parentNode);
				Machine currentMachine = currentTable.getMachine(currNode);
				Machine newMachine = newTable.getMachine(currNode);
				String opID = currNode.getStreamOperator().getProperty("OBJECT_ID");
				String parID = parentNode.getStreamOperator().getProperty("OBJECT_ID");
						
				System.out.println("Moving " + currNode + " from " + currentMachine.getMachineName() + " to " + newMachine.getMachineName());
				
				//1.
				//Only need to do this step if the node and parent are on different machines
				if(!parentMachine.equals(newMachine)){
				
					int queueNum = currNode.getStreamOperator().getOutputQueue().getQueueNumber();

					Connection c = new SendData(opID,queueNum,parentMachine);
					Connection d = new ReceiveData(opID,parID,queueNum);
					Packet p = new Packet("reconnection",c);
					Packet q = new Packet("reconnection",d);
					Socket firstStep = new Socket(newMachine.getIPAddress(), newMachine.getConnectionListenerPort());
					ObjectOutputStream firstOut = new ObjectOutputStream(firstStep.getOutputStream());
					firstOut.writeObject(p);
					firstOut.close();
					firstStep.close();
					
					firstStep = new Socket(parentMachine.getIPAddress(),parentMachine.getConnectionListenerPort());
					firstOut = new ObjectOutputStream(firstStep.getOutputStream());
					firstOut.writeObject(q);
					firstOut.close();
					firstStep.close();
				}
				
				//2.
				//Now reconnect the OQ of each child
				for(int i = 0; i < children.length; i++){
					Machine childMachine = currentTable.getMachine(children[i]);
					String childID = children[i].getStreamOperator().getProperty("OBJECT_ID");
					//If the child and parent are on different machines
					//we have to connect them, just like Step 1.
					if(!childMachine.equals(newMachine)){
						int queueNum = children[i].getStreamOperator().getOutputQueue().getQueueNumber();
						
						Connection c = new SendData(childID,queueNum,newMachine);
						Connection d = new ReceiveData(childID,opID,queueNum);
						Packet p = new Packet("reconnection",c);
						Packet q = new Packet("reconnection",d);
					
						Socket secondStep = new Socket(childMachine.getIPAddress(), childMachine.getConnectionListenerPort());
						ObjectOutputStream firstOut = new ObjectOutputStream(secondStep.getOutputStream());
						firstOut.writeObject(p);
						firstOut.close();
						secondStep.close();
					
						secondStep = new Socket(newMachine.getIPAddress(),newMachine.getConnectionListenerPort());
						firstOut = new ObjectOutputStream(secondStep.getOutputStream());
						firstOut.writeObject(q);
						firstOut.close();
						secondStep.close();
					}
					//else remove a the connection if it is on the same machine.
					else{
						
						Connection c = new StopSend(childID);
						Packet p = new Packet("reconnection", c);
						Socket secondStep = new Socket(childMachine.getIPAddress(), childMachine.getConnectionListenerPort());
						ObjectOutputStream firstOut = new ObjectOutputStream(secondStep.getOutputStream());
						firstOut.writeObject(p);
						firstOut.close();
						secondStep.close();
					}
				}
				
				//3.
				// Now deactivate the operator on the old machine
				Connection c = new DeActivate(opID);
				Packet p = new Packet("reconnection",c);
				Socket thirdStep = new Socket(currentMachine.getIPAddress(),currentMachine.getConnectionListenerPort());
				ObjectOutputStream thirdOut = new ObjectOutputStream(thirdStep.getOutputStream());
				thirdOut.writeObject(p);
				thirdOut.close();
				thirdStep.close();
				
				//4.
				//Now send the state from old machine to new machine
				c = new SendState(opID,newMachine);
				p = new Packet("reconnection",c);
				Socket fourthStep = new Socket(currentMachine.getIPAddress(),currentMachine.getConnectionListenerPort());
				ObjectOutputStream fourthOut = new ObjectOutputStream(fourthStep.getOutputStream());
				fourthOut.writeObject(p);
				fourthOut.close();
				fourthStep.close();
				
				//5.
				//Stop sending data from the old queue when it is empty. 
				c = new StopSend(opID);
				p = new Packet("reconnection",c);
				Socket fifthStep = new Socket(currentMachine.getIPAddress(),currentMachine.getConnectionListenerPort());
				ObjectOutputStream fifthOut = new ObjectOutputStream(fifthStep.getOutputStream());
				fifthOut.writeObject(p);
				fifthOut.close();
				fifthStep.close();
				
				//6. 
				//Activate the operator on the new machine
				c = new Activate(opID);
				p = new Packet("reconnection",c);
				Socket sixthStep = new Socket(newMachine.getIPAddress(),newMachine.getConnectionListenerPort());
				ObjectOutputStream sixthOut = new ObjectOutputStream(sixthStep.getOutputStream());
				sixthOut.writeObject(p);
				sixthOut.close();
				sixthStep.close();
				
				//Finally update the currentTable to reflect the change
				currentTable.add(currNode,newMachine);
			}
			
		} catch (UnknownHostException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;
	}

}
