package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.ArrayList;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.*;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Feb 17, 2004
 * 
 * Insert comments here.
 */
public class DistributionTable {

	private Hashtable ht;
	
	/**
	 * 
	 */
	public DistributionTable() {
		
		ht = new Hashtable();
	}
	
	public Enumeration getNodeEnumeration(){
		return ht.keys();
	}
	
	public boolean contains(XATNode node){
		if(ht.containsKey(node)) return true;
		return false;	
	}
	
	public ArrayList getOperatorsOnMachine(Machine m){
		ArrayList ret = new ArrayList();
		Enumeration e = this.getNodeEnumeration();
		while(e.hasMoreElements()){
			XATNode n = (XATNode) e.nextElement();
			Machine ma = this.getMachine(n);
			if(ma.equals(m))
				ret.add(n);
		}
		
		return ret;
	}
	
	public XATNode[] getNodes(){
		
		XATNode[] nodes;
		Enumeration e = ht.keys();
		nodes = new XATNode[ht.size()];
		int counter = 0;
		while(e.hasMoreElements()){
			
			nodes[0] = (XATNode) e.nextElement();
			counter++;
		}
		
		return nodes;
	}
	
	public Machine getMachine(XATNode node){
	
		return (Machine) ht.get(node);
	}
	
	public void add(XATNode node, Machine m){
		
		ht.put(node,m);
	}
	
	public void remove(XATNode node){
		ht.remove(node);
	}
	
	public DistributionTable difference(DistributionTable newTable){
		
		DistributionTable differenceTable = new DistributionTable();
		
		Enumeration e1 = this.getNodeEnumeration();
		Enumeration e2 = newTable.getNodeEnumeration();
		
		while(e2.hasMoreElements()){
			
			XATNode node = (XATNode) e2.nextElement();
			
			if(this.contains(node)){
				//If the node is in the first table
				//compare to see if the machines are the same, if they arent
				// add it to the difference table
				
				if( !this.getMachine(node).equals(newTable.getMachine(node))){
					differenceTable.add(node, newTable.getMachine(node));
				}
				
			}
			else{
				//The node is in the newer table but not the original
				differenceTable.add(node, newTable.getMachine(node));	
				
			}
		}
		
		return differenceTable;
		
		
	}
	
	public String toString(){
		int maxLength = 10;
		String ret = new String("\n");
		ret+="--------------------------\n";
		ret+="|   Distribution Table   |\n";
		ret+="--------------------------\n";
		ret+="| Operator   |  Machine  |\n";
		ret+="--------------------------\n";
		for(Enumeration e = ht.keys(); e.hasMoreElements();){
			
			XATNode node = (XATNode) e.nextElement();
			Machine m = (Machine) ht.get(node);
			String operatorName = ((XATStreamOperator)node.getOperator()).toShortString();
			String machineName = m.getMachineName();
			
			if(operatorName.length() > maxLength)
				operatorName = operatorName.substring(0,maxLength);
			if(operatorName.length() < maxLength){
				operatorName = operatorName.concat(BlankString.createWhiteSpace(maxLength-operatorName.length()));
			}
			
			if(machineName.length() > maxLength)
				machineName = machineName.substring(0,maxLength);
			if(machineName.length() < maxLength){
				machineName = machineName.concat(BlankString.createWhiteSpace(maxLength-machineName.length()));
			}
			
			ret +="| " + operatorName + " | " + machineName +"|\n";
			
		}
		ret+="--------------------------\n";
		return ret;
	}

	public XATNode getNode(String operatorNumber){
		
		Enumeration e = this.getNodeEnumeration();
		
		while(e.hasMoreElements()){
			
			XATNode temp = (XATNode) e.nextElement();
			if(temp.getStreamOperator().getProperty("OBJECT_ID").equals(operatorNumber))
				return temp;
		}
		
		return null;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() throws CloneNotSupportedException {
		DistributionTable ret = new DistributionTable();
		
		Enumeration e = this.getNodeEnumeration();
		while(e.hasMoreElements()){
			XATNode temp = (XATNode) e.nextElement();
			Machine m = (Machine) this.getMachine(temp);
			
			ret.add(temp,m);
		}
		
		return ret;
	}

}
