package edu.wpi.cs.dsrg.xmldb.xat.component.distribution.redistribution;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.costmodel.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.pattern.DistributionTable;
import java.util.*;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 */
public class Balance extends XATRedistributionPolicy {

	public Balance(){
		super();
	}
	
	public DistributionTable redistribute(
		DistributionTable currTable,
		CostTable currCost,
		XATNode[] ineligibleNodes,
		Machine[] ineligibleMachines) {
		boolean canBalance = true;
		
		if(ineligibleNodes != null){
			for(int i = 0; i < ineligibleNodes.length ; i++){
				this.inEligibleNodes.add(ineligibleNodes[i]);
			}
		}
		try {
			int count = 1;
			while(count <= currCost.size()){
				//Take the heaviest node and move some of its work
				// to the lowest node.
				Machine maxMachine = currCost.NthMaxMachine(count);
				Machine minMachine = currCost.minMachine();
				double maxCost = currCost.getMachineCost(maxMachine);
				double minCost = currCost.minCost();
				count++;
				
				//System.out.println("The heaviest loaded machine is: " + maxMachine.getMachineName());
				//System.out.println("The lightest loaded machine is: " + minMachine.getMachineName());
				
				double diffInCost = (currCost.getMachineCost(maxMachine) - currCost.getMachineCost(minMachine)) / 2;
				//System.out.println("The avg diff in cost is: " + diffInCost);
				// now find the operator that has this difference
				// and move it over!
				
				ArrayList maxNodes = currCost.getNodeCost(maxMachine);
				Iterator i = maxNodes.iterator();
				double closest = Double.MAX_VALUE;
				XATNode closestNode = null;
				
				while(i.hasNext()){
					NodeCost nc = (NodeCost) i.next();
					if(Math.abs(nc.getCost() - diffInCost) <= Math.abs(closest - diffInCost)){
						if(isEligible(nc.getNode())){
							closestNode = nc.getNode();
							closest = nc.getCost();
						}
					}	
				}
				
				//System.out.println("The closest node is: " + closestNode);
				
				//if we can find something small enough to move, then do it
				if(closest <= (maxCost - minCost) / 2 && closest != 0){
					
					//Now update teh cost table
								
					//Update the invidual node costs on the min machine
				
					double newCost = currCost.getMachineCost(minMachine) + closest;
					double oldCost = currCost.getMachineCost(minMachine);
					double totalNewCost = 0;
					ArrayList nodeCosts = currCost.getNodeCost(minMachine);
					Iterator it = nodeCosts.iterator();
					while(it.hasNext()){
						NodeCost nc = (NodeCost) it.next();
						double oldNodeCost = nc.getCost();
						XATNode tempNode = nc.getNode();
						double newNodeCost = oldNodeCost * oldCost / newCost;
						currCost.remove(minMachine, tempNode);
						currCost.put(minMachine,new NodeCost(tempNode,newNodeCost));
						totalNewCost += newNodeCost;
					}
				
					//move the node
					currCost.remove(maxMachine,closestNode);
					currCost.put(minMachine,new NodeCost(closestNode,1-totalNewCost));
				
					currCost.put(maxMachine,currCost.getMachineCost(maxMachine) - closest);
					currCost.put(minMachine,currCost.getMachineCost(minMachine) + closest);
				
					//System.out.println(currCost);
					
					count = 1;
					
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println(currCost);
		return currCost.convertToDistributionTable();
	}

}
