package edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.NotSerializableException;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Enumeration;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.DistributionConfiguration;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerException;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.LightWeightStatistic;
import edu.wpi.cs.cs4432.project1.*;

/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Mar 22, 2004
 * 
 * Insert comments here.
 */
public class CS4432ExecutionController extends DistributedExecutionController {

	/**
	 * @param dc
	 * @param clientName
	 * @throws Exception
	 */
	public CS4432ExecutionController(
		DistributionConfiguration dc,
		String clientName)
		throws Exception {
		super(dc, clientName);
		
	}
	
	//Override the run method
	public void run() {
		System.out.println("Starting the CS4432 Execution Controller.");
		//Initially try to run the thread...
		try {
		} catch (Exception e) {
			System.out.println(e.toString());
			return;
		}    
		// Here is where we start execution.
		boolean keepGoing = true;
		XATSchedulerDecision d = null;
		int totalTuples;
		boolean notInterrupted = true;
		long startTime = System.currentTimeMillis();
		long currentTime = System.currentTimeMillis();
		boolean timeUp = false;
		boolean runSuccess = false;
		long lastRedistTime = currentTime;
		Socket sStat = null;
		ObjectOutputStream outStat = null;
		
		// Set up the queue manager
		//*************************
		QueueManager qm = QueueManager.getQueueManager();
    	int maxQSize = Integer.parseInt(config.getProperty("MAX_QUEUE_SIZE"));
    	qm.setCapacity(maxQSize);
    	System.out.println("The maximum queue size will be: " + maxQSize);
    	//*************************
    	
		try{
			sStat = new Socket(config.getManagerIPAddress(),23000);
		
		}catch(Exception e){
			e.printStackTrace();
		}
		//Find out how long to run the machine.
		int reDistTime = Integer.parseInt(config.getProperty("REDISBRIBUTION_TIME"));
		int runTime = Integer.parseInt(config.getProperty("EXECUTION_DURATION"));
		System.out.println("Will process data for " + runTime + " milliseconds.");
		while (keepGoing && !timeUp) {
			try {

				currentTime = System.currentTimeMillis();
				if(currentTime - startTime > runTime){
					timeUp = true;
				}
			
				if(currentTime - lastRedistTime > reDistTime){
					if(outStat == null){
						outStat = new ObjectOutputStream(sStat.getOutputStream());
					}

					lastRedistTime = currentTime;

					if(lightStats != null){
						LightWeightStatistic lws = new LightWeightStatistic(this.clientName,lightStats);
						outStat.writeObject(lws);
					}
				}
				/** have the scheduler make the next decision */
				d = currentScheduler.makeNextDecision();
        	
				/** 
				  perform some pre exec tasks like configuring operator, calculating statistics 
				  and whatnot.  If this method returns false, then we should not run the operator
				  for one reason or another (it could have no work to do or there could be
				  starvation or something)
				*/
				if (performPreExecTasks(d)) {
            	
					XATStreamOperator operator = d.getNextOperator();                
					int oldOutput =
						Integer.parseInt(stats.getValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator));
					String timeBefore = "" + System.currentTimeMillis();
                
					//run the operator
					//System.out.println("===== RUNNING " + operator);
              
					operator.setRunning(true);
             
					runSuccess = operator.run();
			
					operator.setRunning(false);
					//System.out.println("===========================");
                
					String timeAfter = "" + System.currentTimeMillis();
					String timeRun = "" + (Long.parseLong(timeAfter) - Long.parseLong(timeBefore));

					//now update the stats related to the operator and how it performed
					if(runSuccess){
					 	 updatePostExecStats(operator,timeAfter,timeRun,oldOutput);
					
						//OPTIMIZE THE QUEUES
						qm.optimize();
						
					}
					//check the status, but don't force a change. 
					//This is where we may change scheduling algorithms.
					checkSchedulerStatus(); 
				} else {
					//don't run the operator 
				}
			}catch(XATSchedulerException xse){ 
				// If we have no work to do, we will sleep for a bit
				// and then try again. This will primarily happen when we
				// are setting up the distribution and nobody is connected yet.
				try{
					Thread.sleep(1000);
				}   
				catch (Exception f){
					f.printStackTrace();     	
				}
        	
			}
			catch (NotSerializableException nse){
				FileWriter fw;
				PrintWriter p;
				try{
				fw = new FileWriter("nse-client.txt");
				p = new PrintWriter(fw);
				System.out.println("---Not Serializable Exception---");
				nse.printStackTrace(p);
				System.out.println(nse.getMessage());
				System.out.println("--------------------------------");
				}catch(Exception e){}
			}
			catch (Exception e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
				System.exit(1);
			} 
		} // end of while
   
		System.out.println("Completed Execution, will now  print and send statistics.");
		System.out.print("Printing Statistics....");
 
		/** 
		* now the we are done running, update the periodic properties one last time
		* to ensure the up to date value is printed out
		*/
		for(int count=0; count < tree.size(); count++){
			XATNode[] nodes = ((XATTree)tree.get(count)).convertToArray();
			for (int j = 0; j < nodes.length; j++) {
				for (int i = 0; i < periodicOperatorStatistics.length; i++) {
				stats.updateValue(periodicOperatorStatistics[i], nodes[j].getOperator());
				}
			}
		}
		for (int i = 0; i < periodicTreeStatistics.length; i++) {
			for(int j=0; j < tree.size();j++){
				stats.updateValue(periodicTreeStatistics[i], (XATTree) tree.get(j));
			}
		}

		/** stop the printing because execution is done */
		printer.interrupt();
		for (Enumeration e = stats.getCategories(); e.hasMoreElements();) {
			Object o = e.nextElement();
			if (!new String("GENERIC").equals(o)) {
				printer.setCategory(o);
				printer.gatherStatistics();
			}
		}
    
		printer.print();
		System.out.print("done.\n");
		String resultFilename = printer.getFileName(false);
    
		System.out.println("Printing results stats to " + resultFilename);
		if (debug) {
			for (Enumeration e = stats.getProperties(); e.hasMoreElements();) {
				String next = (String) e.nextElement();
				System.out.println(next + ": " + stats.getProperty(next));
			}
		}
    
		System.out.println("Sending result stats to Distribution Manager...");
	
		try{
	
		Socket s = new Socket(this.config.getManagerIPAddress(),22000);

		BufferedInputStream in = new BufferedInputStream(new FileInputStream(resultFilename));
		BufferedOutputStream out = new BufferedOutputStream(s.getOutputStream());
	
		out.write(this.clientName.getBytes());
		out.write((int) '@');
	
		char ch;
		do{
			ch = (char) in.read();
			out.write(ch);
		
		}while((int)ch != 65535);
		outStat.close();
		sStat.close();
		out.write(-1);
		out.close();
		s.close();
	
		System.out.print("done.\n");
  
		}catch(Exception e){
			e.printStackTrace();
		}
		return;
	}

}
