package edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATPreference;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.*;
/**
 * Responsible for parsing the xml configuration file that will be used
 * by the ExecutionController
 */
public class DistributedConfigFileParser {
	private org.w3c.dom.Document document;
/**
 * ConfigFileParser constructor comment.
 */
public DistributedConfigFileParser(String fileName) throws Exception{
    super();
    try {
        String key = "javax.xml.parsers.DocumentBuilderFactory";
        String value = "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl";
        //set the DocumentBuilderFactory to use
        java.lang.System.setProperty(key, value);
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //DocumentBuilderFactory factory = new DocumentBuilderFactoryImpl();
        DocumentBuilder builder = factory.newDocumentBuilder();
        document = builder.parse(fileName);
    } catch (Exception e) {
		throw e;
    }

}

public Iterator getMultipleAttributes(String name) throws IOException{
	NodeList nodes = (NodeList) document.getElementsByTagName(name);
	ArrayList list = new ArrayList();
	for(int i=0;i < nodes.getLength();i++){
		list.add(this.getAttributes(name,i));	
	}
	
	return list.iterator();
}

public Iterator getMachineSchedulingAlgorithms(int machineNumber) throws IOException{
	
	return getAttributes("algorithms",machineNumber);
	
}
private Iterator getAttributes(String elementName) throws IOException {
	return getAttributes(elementName,0);
}
/**
 * Returns an Iterator of the name / value pairs of properties
 * for a given element name 
 * @return Iterator
 * @param elementName java.lang.String
 * @param attributeName java.lang.String[]
 */
private Iterator getAttributes(String elementName, int number) throws IOException {
    NodeList nodes = (NodeList) document.getElementsByTagName(elementName);
    Node node = nodes.item(number);
    ArrayList list = new ArrayList();
    if (node != null) {
        NodeList propertiesList = node.getChildNodes();
        //System.out.println(statsPropertiesList.getLength());
        Node nextNode = node.getFirstChild();
        /** traverses sibling to sibling and add properties found to the vector*/
        while (nextNode != null) {
            if (nextNode instanceof Element) {
                /**
                * check the nodeName, should = property
                * if it doesnt, then do nothing
                */

                if (nextNode.getNodeName().equals("property")) {
                    /** now we've found the properties element*/
                    NamedNodeMap attrs = nextNode.getAttributes();
                    String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
                    String value = ((Node) attrs.getNamedItem("value")).getNodeValue();
                    list.add(new NameValuePair(name, value));
                }
            }
            nextNode = nextNode.getNextSibling();
        }
    }

    return list.iterator();
}
/**
 * Get the properties related to the columns to printout
 * @return Iterator
 */
public Iterator getColumnNamesForOutput() throws IOException {
	return getAttributes("outputColumnNames");
}
/**
 * Return all statistics to gather every time after execution relating to an operator
 * @return Iterator
 */
public Iterator getEveryTimeOperatorStatistics() throws IOException {
	return getAttributes("everyTimeOperator");
}

/**
 * Return all statistics to gather every time after execution relating to an operator
 * @return Iterator
 */
public Iterator getEveryTimeSystemStatistics() throws IOException {
	return getAttributes("everyTimeSystem");
}

/**
 * Return all of the properties of the ExecutionAdapter
 * @return Iterator
 */
public Iterator getExecutionAdapterProperties() throws IOException {
	return getAttributes("ExecutionAdapter");
}
/**
 * Return all of the properties of the ExecutionController
 * @return Iterator
 */
public Iterator getExecutionControllerProperties() throws IOException {
	return getAttributes("ExecutionController");
}
/**
 * Return all properties relating to the execution
 * @return Iterator
 */
public Iterator getExperimentProperties() throws IOException {
	return getAttributes("experiment");
}
/**
 * Get the properties related to the output format
 * @return Iterator
 */
public Iterator getOutputFormatProperties() throws IOException {
	return getAttributes("outputFormat");
}
/**
 * Return all statistics to gather Periodically after execution relating to an operator
 * @return Iterator
 */
public Iterator getPeriodicOperatorStatistics() throws IOException {
	return getAttributes("periodicOperator");
}
/**
 * Return all statistics to gather Periodically after execution relating to the XATTree
 * @return Iterator
 */
public Iterator getPeriodicTreeStatistics() throws IOException {
	return getAttributes("periodicTree");
}

public Iterator getPreferences() throws Exception{
	return getPreferences(0);
}
/**
 * Get the user's preferences.  Unfortunately, the preference
 * requires 3 attributes, therefore the rest of the parsing code
 * cannot be reused.  
 * @return Iterator
 */
public Iterator getPreferences(int number) throws Exception {
	String elementName = "preferences";
	NodeList nodes = (NodeList) document.getElementsByTagName(elementName);
    Node node = nodes.item(number);
    /** iterate through the properties of the statisticsToGather element*/
    NodeList preferenceList = node.getChildNodes();
    //System.out.println(statsPropertiesList.getLength());
    ArrayList list = new ArrayList();
    Node nextNode = node.getFirstChild();
    double totalWeight = 0;
    /** traverses sibling to sibling and add properties found to the vector*/
    while (nextNode != null) {
        if (nextNode instanceof Element) {
            /**
            * check the nodeName, should = preference
            * if it doesnt, then do nothing
            */

            if (nextNode.getNodeName().equals("preference")) {                
            	/** now we've found the properties element*/
            	NamedNodeMap attrs = nextNode.getAttributes();
            	String statistic = ((Node) attrs.getNamedItem("statistic")).getNodeValue();
            	String quantifier = ((Node) attrs.getNamedItem("quantifier")).getNodeValue();
            	double weight = Double.parseDouble(((Node) attrs.getNamedItem("weight")).getNodeValue());
            	XATPreference p = new XATPreference();
            	p.setStatistic(statistic);
            	if(quantifier.equalsIgnoreCase("min")){
					p.setQuantifier(XATPreference.MIN);
            	} else if(quantifier.equalsIgnoreCase("max")){
	            	p.setQuantifier(XATPreference.MAX);
            	} else{
					throw new IllegalArgumentException("The quanitifier must be min or max");
            	}            	
            	p.setWeight(weight);
            	totalWeight += weight;
            	list.add(p);
            }            
        }
        nextNode = nextNode.getNextSibling();
    }
    if(totalWeight != 1.0){
		throw new IllegalArgumentException("The sum of the weights must be exactly 1. The current total is:" + totalWeight);
    }
	return list.iterator();
}

public Iterator getServers() throws Exception {
	String elementName = "servers";
	NodeList nodes = (NodeList) document.getElementsByTagName(elementName);
	Node node = nodes.item(0);
	/** iterate through the properties of the statisticsToGather element*/
	NodeList preferenceList = node.getChildNodes();
	//System.out.println(statsPropertiesList.getLength());
	ArrayList list = new ArrayList();
	Node nextNode = node.getFirstChild();
	/** traverses sibling to sibling and add properties found to the vector*/
	while (nextNode != null) {
		if (nextNode instanceof Element) {
			/**
			* check the nodeName, should = preference
			* if it doesnt, then do nothing
			*/

			if (nextNode.getNodeName().equals("server")) {                
				/** now we've found the server element*/
				NamedNodeMap attrs = nextNode.getAttributes();
				String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
				String ipAddress = ((Node) attrs.getNamedItem("ip_address")).getNodeValue();
				int port = Integer.parseInt(((Node) attrs.getNamedItem("port")).getNodeValue());
				Server s = new Server();
				s.setName(name);
				s.setIPAddress(ipAddress);
				s.setPort(port);
				list.add(s);
			}            
		}
		nextNode = nextNode.getNextSibling();
	}
	return list.iterator();
}

public Iterator getStreams() throws Exception {
	String elementName = "streams";
	NodeList nodes = (NodeList) document.getElementsByTagName(elementName);
	Node node = nodes.item(0);
	/** iterate through the properties of the statisticsToGather element*/
	NodeList preferenceList = node.getChildNodes();
	//System.out.println(statsPropertiesList.getLength());
	ArrayList list = new ArrayList();
	Node nextNode = node.getFirstChild();
	/** traverses sibling to sibling and add properties found to the vector*/
	while (nextNode != null) {
		if (nextNode instanceof Element) {
			/**
			* check the nodeName, should = preference
			* if it doesnt, then do nothing
			*/

			if (nextNode.getNodeName().equals("stream")) {                
				/** now we've found the server element*/
				NamedNodeMap attrs = nextNode.getAttributes();
				String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
				String server = ((Node) attrs.getNamedItem("server")).getNodeValue();
				Stream s = new Stream();
				s.setName(name);
				s.setProperty("server",server);
				list.add(s);
			}            
		}
		nextNode = nextNode.getNextSibling();
	}
	return list.iterator();
}
/**
 * Return all properties relating to available machines in the system.
 * @return Iterator
 */
public Iterator getDistributionProperties() throws IOException {
	return getAttributes("distribution");
}

public Iterator getMachineProperties() throws IOException {
	return getAttributes("machine");
}
/**
 * Return all properties relating to the query plan generator
 * @return Iterator
 */
public Iterator getQueryPlanGeneratorProperties() throws IOException {
	return getAttributes("queryPlanGenerator");
}
/**
 * Return all of the scheduling algorihms
 * @return Iterator
 */
public Iterator getSchedulingAlgorithms() throws IOException {
	return getAttributes("algorithms");
}
/**
 * Return all properties relating to the scheduling algorihms
 * @return Iterator
 */
public Iterator getSchedulingProperties() throws IOException {
	return getAttributes("scheduling");
}
/**
 * Returns all of the system properties
 * @return Iterator
 */
public Iterator getSystemProperties() throws IOException {
	return getAttributes("system");
}
/**
 * Returns all of the tree properties
 * @return Iterator
 */
public Iterator getTreeProperties() throws IOException {
	return getAttributes("treeProperties");
}
}
