package edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.NotSerializableException;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.net.Socket;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.DistributionConfiguration;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.QueryPlan;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.ConnectionListener;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.ConnectionTable;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.PacketPool;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.TupleDistributor;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.TupleFeeder;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.TupleReceiver;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.TupleSender;
import edu.wpi.cs.dsrg.xmldb.xat.component.gui.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATExecutionAdapter;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATScheduler;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerException;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.LightWeightStatistic;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter;


/**
 * This controls everything about execution including asking the scheduler
 * which operator to run next and for how long, running the operator, updating
 * the statistics for the operator, replanning, and changing the scheduling algorithm.  
 *  
 * @author: Timothy M. Sutherland
 */
public class DistributedExecutionController extends ExecutionController implements Runnable {
	protected ArrayList tree; // an arraylist of XATTree's
	protected StatisticsPrinter printer;
	protected TupleReceiver packetReceiver;
	protected ConnectionListener connectionListener;
	
	protected XATScheduler currentScheduler;
	protected Machine machine;
	/** array of statistics to periodically gatherer about operators */
	protected String[] periodicOperatorStatistics;
	/** array of statistics to periodically gatherer about trees */
	protected String[] periodicTreeStatistics;
	/** array of statistics to gatherer about operators everytime the operator is run*/
	protected String[] everyTimeOperatorStatistics;
	
	protected String[] everyTimeSystemStatistics;
	
	/** 
	 * this defines the last time that certain properties about this operator were updated.
	 * In the ExecController, we do not want to update every property everytime an operator
	 * has run, but would prefer to do it periodically.  This property will keep track of the
	 * last time those properties were updated. The key is the operator, the value is the lastTime
	 * it was updated. 
	*/
	protected java.util.Hashtable lastTimeOperatorUpdated;
	/** 
	 * this defines the last time that the status of execution was checked.  That is,
	 * when did we last determine how well execution is performing.  This value is
	 * tied to the STATUS_CHECK_FREQUENCY property.  
	*/
	protected long lastTimeStatusChecked;
	/** 
	 * this defines the last time that the tree statistics were updated.  
	*/
	protected long lastTimeTreesUpdated;
	/** 
	 * this defines the last time that tuples were uploaded to the application.  
	*/
	protected boolean debug;
	/** this adapter manages all of the scheduling algorithms and decides which one to run*/
	protected XATExecutionAdapter adapter;
	protected java.util.Hashtable schedulerCount;
	protected boolean debug_scheduling_notice;
	protected DistributionConfiguration config;
	protected String clientName;
	protected PacketPool inPool;
	protected PacketPool outPool;
	protected ConnectionTable inConnections;
	protected ConnectionTable outConnections;
	protected TupleFeeder tupleFeeder;
	protected TupleDistributor tupleDistributor;
	protected TupleSender tupleSender;
	protected CAPE_GUI gui;
	protected XATDag[] treeArray;
	protected SystemObject system;
	protected ArrayList lightStats;


/**
 * 
 */
public DistributedExecutionController(DistributionConfiguration dc, String clientName) throws Exception {
	super();
	config = dc;			
	this.clientName = clientName;
	System.out.println("My Name is: " + clientName);
	tree = new ArrayList();
	system = new SystemObject();
	initExecution();	
	this.inPool = new PacketPool();
	this.outPool = new PacketPool();
	this.inConnections = new ConnectionTable();
	this.outConnections = new ConnectionTable();
	
	packetReceiver = new TupleReceiver(machine.getTupleReceiverPort(),inPool,tree);
	connectionListener = new ConnectionListener(machine.getConnectionListenerPort(),clientName, inConnections, outConnections, this);
	tupleFeeder = new TupleFeeder(inPool,inConnections);
	tupleDistributor = new TupleDistributor(outConnections);
	tupleSender = new TupleSender(outPool,outConnections);
	
	
}
/**
 * If the time since the status was checked is greater than STATUS_CHECK_FREQUENCY,
 * then this will check the status of execution, otherwise it will just return.  
 * 
 */
protected void checkSchedulerStatus() {

	long frequency = Long.parseLong(this.getProperty("STATUS_CHECK_FREQUENCY"));
	//if we are not forcing a change and if it isnt time to do so, we won't check the exec status
	if((System.currentTimeMillis() - lastTimeStatusChecked) < frequency){		
			/** not time to check the status of execution */
			return;		
	} else{		
		/** 
		 * ok, time to check the status of execution 
		 * first we will update the tree and operator statistics b/c these contain a lot of
		 * the metrics that will evaluate the execution status
		*/
		
		XATTree[] treeArray = new XATTree[tree.size()];
			for(int i = 0; i < tree.size();i++){
				treeArray[i] = (XATTree) tree.get(i);
		}
		
		for(int count=0; count < tree.size(); count++){
			XATNode[] nodes = ((XATTree)tree.get(count)).convertToArray(); 
    		for (int j = 0; j < nodes.length; j++) {
     		   for (int i = 0; i < periodicOperatorStatistics.length; i++) {
      	      stats.updateValue(periodicOperatorStatistics[i], nodes[j].getOperator());
       	   	}
     	   lastTimeOperatorUpdated.put(nodes[j].getOperator(), new Long(System.currentTimeMillis()));
    		}	
		}									
		for (int i = 0; i < periodicTreeStatistics.length; i++){
			stats.updateValue(periodicTreeStatistics[i], treeArray);
		}
		lastTimeTreesUpdated = System.currentTimeMillis();
		if(debug){
			System.out.println("*****Time to check Execution Status*****");
		}

		currentScheduler = adapter.getNextScheduler();
	
		stats.setCurrentCategory(adapter.getCategory());
		if(debug){
			System.out.println("***** Next Scheduler is: " + adapter.getCategory().toString() + " *****");
		}
		printer.setCategory(stats.getCurrentCategory());
		lastTimeStatusChecked = System.currentTimeMillis();
		updateSchedulerCount();
	}	
}
/**
 * Returns the current scheduling algorithm
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATScheduler
 */
public edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATScheduler getCurrentScheduler() {
	return currentScheduler;
}
/**
 * Returns the handle to the statistics printer
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter
 */
public StatisticsPrinter getPrinter() {	
	return printer;
}

public void setLightweightStats(ArrayList stats){
	this.lightStats = stats;
}

/**
 * Returns the handle to the packet receiver
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter
 */
public TupleReceiver getTupleReceiver() {	
	return packetReceiver;
}
/**
 * Returns the handle to the packet receiver
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter
 */
public ConnectionListener getConnectionListener() {	
	return connectionListener;
}

public TupleFeeder getTupleFeeder(){
	return tupleFeeder;
}

public TupleDistributor getTupleDistributor(){
	return tupleDistributor;
}

public TupleSender getTupleSender(){
	return tupleSender;
}

public CAPE_GUI getGUI(){
	gui = new CAPE_GUIQueryEngine(treeArray, this.clientName,this.system);
	return this.gui;
}

/**
 * Insert the method's description here.
 * Creation date: (4/6/2003 8:46:07 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree[] getTree() {
	return (XATTree[])tree.toArray();
}

/**
 * Initializes various parts of the execution including
 * query plan, input, schedulers, and statistics table
 */
protected void initExecution() throws Exception {
	
	
	/** next set up the ExecutionController properties */
	Enumeration e = config.getProperties();
	while(e.hasMoreElements()){
		String name = (String) e.nextElement();
		String value = config.getProperty(name);
		if(this.isValidStatistic(name))
			this.setProperty(name, value);
	}

	// Get the machine properties
	machine = config.getMachine(clientName);
	
	Enumeration en = machine.getProperties();
	while(en.hasMoreElements()){
		String name = (String) en.nextElement();
		String property = machine.getProperty(name);
		if(this.isValidPropertyName(name))
			this.setProperty(name, property);
	}
	
	// Get the query plans	
	int c = 0;
	for(Iterator i= config.getQueryPlans(); i.hasNext();){
		QueryPlan qp = (QueryPlan) i.next();
		tree.add(qp.getQueryPlan());
		c++;
	}
          
	//set up the rest of the class variables
	debug = (new Boolean(this.getProperty("EXECUTION_DEBUG"))).booleanValue();
	debug_scheduling_notice =  (new Boolean(this.getProperty("SCHEDULING_NOTIFICATION_DEBUG"))).booleanValue();
	lastTimeOperatorUpdated = new Hashtable();
	lastTimeStatusChecked = System.currentTimeMillis();
	lastTimeTreesUpdated = System.currentTimeMillis();
	schedulerCount = new Hashtable();
	
	stats.setProperty("TIMER", this.getProperty("STATS_TABLE_STATS"));	
	
	XATTree[] statsTree = new XATTree[tree.size()];
	for(int i = 0; i < tree.size(); i++){
		statsTree[i] = (XATTree) tree.get(i);
	}

    stats.initStatsTable(statsTree);  
    stats.addNewQueryObject(system);   
    
         	             		
    if(debug){
	    System.out.println(tree);
	}
	
    /**
      init the last time some of its properties were updated
      set the last updated time to 0, that way the properties will be updated the first time.    
      This helps with experiments 
    */
    
    for(int count=0; count < tree.size(); count++){
		XATNode[] nodes = ((XATTree)tree.get(count)).convertToArray();
    	for (int i = 0; i < nodes.length; i++){
    		if(nodes[i].getOperator() instanceof XATStreamOperator){
				XATStreamOperator operator = (XATStreamOperator)nodes[i].getOperator();
				lastTimeOperatorUpdated.put(operator, new Long(0));
    		}
  		}
    }
    
    //set up schedulers
    String className = this.getProperty("ADAPTIVE_HEURISTIC"); 
	treeArray = new XATDag[tree.size()];
	for(int i = 0; i < tree.size();i++){
		treeArray[i] = (XATDag) tree.get(i);
	}

    Constructor[] c1 = Class.forName(className).getConstructors();
    Constructor construct;

    
	for(int i = 0; i < c1.length; i++){
		construct = c1[i];
		Class[] parameters = construct.getParameterTypes();
		System.out.println(parameters[0].getName());
		if(parameters.length == 2){
			if(parameters[0].getName().equals("edu.wpi.cs.dsrg.xmldb.xat.component.distribution.configuration.Machine")){
				adapter = (XATExecutionAdapter)construct.newInstance(new Object[] { machine, treeArray});
			}
			
		}
	}
			
	if(adapter == null){
		throw new Exception("The Adapter couldd not be instantiated.");
	}
    //start at scheduler 0 
    currentScheduler = adapter.getCurrentScheduler();
    updateSchedulerCount();
    
	//set up some of the statistics to gather
	initStatsToGather();
    //set up the printer
    initStatisticsPrinter();

    //set the starting category in the statistics gatherer to be the first scheduler
	stats.setCurrentCategory(adapter.getCategory());
	printer.setCategory(stats.getCurrentCategory());	
	System.out.println("Initialization...done.");
}

/**
 * Initializes the statistics printer by call the config file parser and setting all of the
 * properties that are returned
 * 
 */
protected void initStatisticsPrinter() throws Exception {
    printer = new StatisticsPrinter();
    for (Iterator i = config.getPrinterOutputFormat(); i.hasNext();) {
        NameValuePair pair = (NameValuePair) i.next();
        printer.setProperty(pair.getName(), pair.getValue());        
    }
   ArrayList columnList = new ArrayList();
   for (Iterator i = config.getPrinterColumnNames();i.hasNext();) {
	   columnList.add(i.next());   	
   }
   
   printer.setColumnHeaders(columnList);

}
/**
 * Calls config file parser and gathers all of the statistics that 
 * should be gatherered for an operator both every time it is run and perodically
 *
 * This will also do the same for tree statistics
 */
protected void initStatsToGather() throws Exception {

	/** call populate statistics to do the real work */
	everyTimeOperatorStatistics = populateStatistics(config.getEveryTimeOperator());
	periodicOperatorStatistics = populateStatistics(config.getPeriodicOperator());
	periodicTreeStatistics = populateStatistics(config.getPeriodicTree());
	everyTimeSystemStatistics = populateStatistics(config.getEveryTimeSystem());
	
}

/**
 * This will update the statistics for operators and take care of any additional
 * book keeping before the operator is allowed to run.
 * @param decision XATSchedulerDecision
 * @return boolean true if the operator should continue to run, false if it should not 
 */
protected boolean performPreExecTasks(XATSchedulerDecision decision)
    throws Exception {
    boolean returnValue = true;
    
    /**
     * if the workload is 0, then we dont want to run the operator,
     * but rather note that we detected starvation.  If
     * we get too many starvations in a row, we will force a change of algorithm
     * If that doesnt work, the execution will terminate.
     * Note: some algorithms have built in starvation checks, therefore, this
     * checking is mearly a little overhead.  However, other algorithms are
     * inherently prone to starvation and thus, this check is necessary, otherwise
     * we dont get anywhere.  
     */	
    
    if (decision.getNumberOfTuples() < 1) {
    	// There is no work to do.
        return false;
    } else {
        XATStreamOperator operator = (XATStreamOperator) decision.getNextOperator();
	
        if (debug) {
	        System.out.println("Next operator to run: " + operator);
          	System.out.println("Dequeuing this many tuples: " + decision.getNumberOfTuples() + "\n");
        }
        /** set the operator's property */
        operator.setProperty("NUMBER_OF_TUPLES_TO_DEQUEUE", "" + decision.getNumberOfTuples());
        stats.updateValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator);
        /** 
         * figure out how many tuples are available for this operator
         * and update the TUPLE_COUNT statistic.  This will be used
         * later to figure out the number of possible outputs.
         */
        for (Iterator i = operator.getInputQueues(); i.hasNext();) {
            XATQueue queue = (XATQueue) i.next();
            int work = queue.getQueueSize(operator);
            stats.updateValue("TUPLE_COUNT", queue, "" + work);
        }
        returnValue = true;
    }
    return returnValue;
}
/**
 * Takes an iterator of NameValuePairs and adds the names to the given String array.
 * This is used for taking the names of the statisticsToGather and putting them into the appropriate array 
 * @param iterator Iterator
 * @return array java.lang.String[]
 */
protected String[] populateStatistics(Iterator iterator) {

    ArrayList list = new ArrayList();
    for (; iterator.hasNext();) {
        list.add(iterator.next());
    }
    //initialize the Array size
    String[] array = new String[list.size()];
    int index = 0;
    //initialize the properties in the array
    for (Iterator i = list.iterator(); i.hasNext();) {
        NameValuePair pair = (NameValuePair) i.next();
        array[index] = pair.getName();
        index++;
    }
    return array;
}
/**
 * Runs the actual execution
 *
 */
public void run() {
	System.out.println("Starting a new Distributed Execution Controller.");
	//Initially try to run the thread...
    try {
    } catch (Exception e) {
        System.out.println(e.toString());
        return;
    }    
    // Here is where we start execution.
    boolean keepGoing = true;
    XATSchedulerDecision d = null;
    int totalTuples;
    int memCount = 0;
    boolean notInterrupted = true;
    long startTime = System.currentTimeMillis();
    long currentTime = System.currentTimeMillis();
    boolean timeUp = false;
    boolean runSuccess = false;
    long lastRedistTime = currentTime;
    Socket sStat = null;
	ObjectOutputStream outStat = null;
	DecimalFormat df = new DecimalFormat("0.00#");
	XATStreamOperator operator;
	int oldOutput;
	String timeBefore;
	int reDistTime = Integer.parseInt(config.getProperty("REDISBRIBUTION_TIME"));
	int runTime = Integer.parseInt(config.getProperty("EXECUTION_DURATION"));
	Runtime rt = Runtime.getRuntime();
	String timeAfter;
	String timeRun;
	LightWeightStatistic lws;
	//XATQueueManager qm = XATQueueManager.getQueueManager(this);
	rt.gc();
	
    try{
    	Thread.sleep(2000);
	    sStat = new Socket(config.getManagerIPAddress(),23000);
		
    }catch(Exception e){
	    e.printStackTrace();
    }
    //Find out how long to run the machine.

    System.out.println("Will process data for " + runTime + " milliseconds.");
    try {
    while (keepGoing && !timeUp) {
        try {
			currentTime = System.currentTimeMillis();
			if(currentTime - startTime > runTime){
				timeUp = true;
			}
			
			if(currentTime - lastRedistTime > reDistTime){
				if(outStat == null){
					outStat = new ObjectOutputStream(sStat.getOutputStream());
				}

				lastRedistTime = currentTime;
				if(lightStats != null){
					lws = new LightWeightStatistic(this.clientName,lightStats);
					outStat.writeObject(lws);
				}
				
				System.out.println("-------------Memory Information " + memCount++ +" --------------");
				System.out.println("Tuples in input pool: " + this.inPool.size());
				System.out.println(this.stats.displaySize());
				System.out.println("Free memory: " + df.format(((double)rt.freeMemory()/1024.0/1024.0)) + "MB");
				System.out.println("Total memory: " + df.format(((double)rt.totalMemory()/1024.0/1024.0)) + "MB");
				System.out.println("Max memory: " + df.format(((double)rt.maxMemory()/1024.0/1024.0)) + "MB");
				System.gc();
				System.out.println("Free memory: " + df.format(((double)rt.freeMemory()/1024.0/1024.0)) + "MB");
				System.out.println("Total memory: " + df.format(((double)rt.totalMemory()/1024.0/1024.0)) + "MB");
				System.out.println("Max memory: " + df.format(((double)rt.maxMemory()/1024.0/1024.0)) + "MB");	
				System.out.println("----------------------------------------------");

			}
            /** have the scheduler make the next decision */
            d = currentScheduler.makeNextDecision();
        	
            /** 
              perform some pre exec tasks like configuring operator, calculating statistics 
              and whatnot.  If this method returns false, then we should not run the operator
              for one reason or another (it could have no work to do or there could be
              starvation or something)
            */
            
            if (performPreExecTasks(d)) {
                operator = d.getNextOperator();                
                oldOutput =
                    Integer.parseInt(stats.getValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator));
                
                timeBefore = "" + System.currentTimeMillis();
                
                //run the operator
                //System.out.println("===== RUNNING " + operator);
              
                operator.setRunning(true);
             
                runSuccess = operator.run();
			
                operator.setRunning(false);
                //System.out.println("===========================");
                
				//System.out.println("------------- Queue Manager Information -------");
				//System.out.println("Queues Managed by QM: " + operator.getInputQueue().getQueueManager().getNumQueues());
				//System.out.println("Queues: ");
				//operator.getInputQueue().getQueueManager().printQueues();
				//System.out.println("Total Tuples: " + operator.getInputQueue().getQueueManager().getTotalTuples());
				//System.out.println("Total Tuples on Disk: " + operator.getInputQueue().getQueueManager().getTotalOnDisk());
				//System.out.println("----------------------------------------------");
                
                timeAfter = "" + System.currentTimeMillis();
                timeRun = "" + (Long.parseLong(timeAfter) - Long.parseLong(timeBefore));

                //now update the stats related to the operator and how it performed
                if(runSuccess){
              	  	updatePostExecStats(operator,timeAfter,timeRun,oldOutput);
                }
                else {
                	//System.out.println("WARNING: OPERATOR DID NOT RUN");
                	//System.out.println("  " + operator.toString());
                }
                //check the status, but don't force a change. 
                //This is where we may change scheduling algorithms.
                checkSchedulerStatus(); 
            } else {
                //don't run the operator 
            }
        }catch(XATSchedulerException xse){ 
        	// If we have no work to do, we will sleep for a bit
        	// and then try again. This will primarily happen when we
        	// are setting up the distribution and nobody is connected yet.
        	try{
				Thread.sleep(1000);
        	}   
        	catch (Exception f){
        		f.printStackTrace();     	
        	}
        	
        }
        catch (NotSerializableException nse){
        	FileWriter fw;
        	PrintWriter p;
        	try{
			fw = new FileWriter("nse-client.txt");
        	p = new PrintWriter(fw);
        	System.out.println("---Not Serializable Exception---");
        	nse.printStackTrace(p);
			System.out.println(nse.getMessage());
        	System.out.println("--------------------------------");
			}catch(Exception e){}
        }
        catch(OutOfMemoryError oome){
        	System.out.println("Out of memory. Sending whatever statistics we can.");
        	Runtime.getRuntime().gc();
        	
        }
        catch (Exception e) {
        	System.out.println(e.getMessage());
        	e.printStackTrace();
        	System.exit(1);
        } 
    } // end of while
    } catch (Exception ex) {
		try {
			FileWriter out = new FileWriter("execution_exceptions.txt", true);
			out.write(ex.getStackTrace().toString());
			out.close();
		} catch (IOException e) {
			System.out.println(e);
			e.printStackTrace();
		}
    }
   
    System.out.println("Completed Execution, will now  print and send statistics.");
    System.out.print("Printing Statistics....");
 
    /** 
    * now the we are done running, update the periodic properties one last time
    * to ensure the up to date value is printed out
    */
	for(int count=0; count < tree.size(); count++){
		XATNode[] nodes = ((XATTree)tree.get(count)).convertToArray();
 	    for (int j = 0; j < nodes.length; j++) {
     	    for (int i = 0; i < periodicOperatorStatistics.length; i++) {
            	stats.updateValue(periodicOperatorStatistics[i], nodes[j].getOperator());
       		}
   		}
	}
    for (int i = 0; i < periodicTreeStatistics.length; i++) {
    	for(int j=0; j < tree.size();j++){
			stats.updateValue(periodicTreeStatistics[i], (XATTree) tree.get(j));
    	}
    }

    /** stop the printing because execution is done */
    printer.interrupt();
    for (Enumeration e = stats.getCategories(); e.hasMoreElements();) {
        Object o = e.nextElement();
        if (!new String("GENERIC").equals(o)) {
            printer.setCategory(o);
            printer.gatherStatistics();
        }
    }
    
    printer.print();
    System.out.print("done.\n");
    String resultFilename = printer.getFileName(false);
    
    System.out.println("Printing results stats to " + resultFilename);
    if (debug) {
        for (Enumeration e = stats.getProperties(); e.hasMoreElements();) {
            String next = (String) e.nextElement();
            System.out.println(next + ": " + stats.getProperty(next));
        }
    }
    
	System.out.println("Sending result stats to Distribution Manager...");
	
	try{
	
	Socket s = new Socket(this.config.getManagerIPAddress(),22000);

	BufferedInputStream in = new BufferedInputStream(new FileInputStream(resultFilename));
	BufferedOutputStream out = new BufferedOutputStream(s.getOutputStream());
	
	out.write(this.clientName.getBytes());
	out.write((int) '@');
	
	char ch;
	do{
		ch = (char) in.read();
		out.write(ch);
		
	}while((int)ch != 65535);
	outStat.close();
	sStat.close();
	out.write(-1);
	out.close();
	s.close();
	
    System.out.print("done.\n");
    
    //Stop the tuple feeder
    //tupleFeeder.requestStop();
    //Stop the tuple sender
    //tupleSender.requestStop();
    
    //Stop Distributing
    //this.outConnections.clearTable();
    //tupleDistributor.requestStop();
    
	}catch(Exception e){
		e.printStackTrace();
	}
    return;
}
	/**
    * called by the constructor.  this will set the default properties
    * that the ALL query objects will support. Each subclass should override
    * this method and add their own unique properties, but should also call
    * super.setDefaultProperties() so that the common properties may be set.
    */   
	protected void setDefaultProperties(){
		super.setDefaultProperties();
		/** 
		 * this defines how often some of the properties should be updated
		 * some properties, such as operator output rate do not need to be updated
		 * every time an operator is run, but rather, they can be updated periodically
		 * This will define how often (in seconds) these properties should be updated.  
		 *
		 * The tree frequency serves the same purpose, except for tree properties
		 */
		addProperty("UPDATE_OPERATOR_PROPERTY_FREQUENCY","5000");
		addProperty("UPDATE_TREE_PROPERTY_FREQUENCY","5000");
		/** 
		 * similar to the previous property, this defines how often
		 * the ExecController should check the execution status and determine if
		 * the scheduling algorithm should change.  This value should be set sufficently high
		 * such that any strategy has a chance to work (or not work).
		*/		
		addProperty("STATUS_CHECK_FREQUENCY","10000");
		addProperty("STATS_TABLE_STATS","false");
		addProperty("EXECUTION_DEBUG","false");
		//true if you want notification of the next scheduling algorithm and statistics
		//about which algorithm ran to be printed
		addProperty("SCHEDULING_NOTIFICATION_DEBUG","false");		
		/** the default adapter will always stick with the first algorithm */
		addProperty("ADAPTIVE_HEURISTIC","edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.NeverRotateAdapter");
		/** how many times will an operator be run (in a row) without any work before the algorithm is changed */
		addProperty("STARVATION_CHECK","3");
		/** how long, in milleseconds, that the query should be executed */
		addProperty("EXECUTION_DURATION","10000");
		addProperty("NO_WORK_SLEEP_TIME","100");
	}

/**
 * Insert the method's description here.
 * Creation date: (1/27/03 1:26:53 PM)
 * @return java.lang.String
 */
public String toString() {
	String rs = new String();

	String sName = this.getClass().getName();
	//find the last dot.
	int index = 0;
	for(index = sName.length()-1; index >=0; index--)
		if(sName.charAt(index) == '.')
			break;
	rs = sName.substring(index + 1) + " " + hashCode(); // + "  data:";	
    return rs;
}
/**
 * This will update the statistics for operators. This is broken into 2 phases,
 * the first phase will update those properties that should be updated every time.  The
 * rest define those that are updated periodically 
 * @param operator edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator
 */
protected void updatePostExecStats(XATStreamOperator operator,String timeAfter, String timeRun, int oldOutput) {

	stats.updateValue("LAST_TIME_RUN", operator, timeAfter);
	stats.updateValue("TIME_TOOK_TO_RUN", operator, timeRun);
	stats.updateValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator);
	stats.updateValue("NUMBER_OF_TUPLES_OUTPUTTED", operator, "" + oldOutput);
	
    for (int i = 0; i < everyTimeOperatorStatistics.length; i++) {
        stats.updateValue(everyTimeOperatorStatistics[i], operator);
    }
    
    for(int i = 0; i < everyTimeSystemStatistics.length; i++){
    	stats.updateValue(everyTimeSystemStatistics[i], system);
    }
    /**
     * can't really generalize the next 2 properties
     *
     * also update the NUMBER_OF_TUPLES_IN_INPUT_QUEUES property for the operator's
     * parent(s)
     */
    try {
	    XATNode parent = operator.getNode().getParent();
	    if(parent != null){
		    XATOperator parentOp = parent.getOperator();
	        stats.updateValue("NUMBER_OF_TUPLES_IN_INPUT_QUEUES", parentOp);
        }
             
    } catch (Exception e) {
        //no parent so nothing to worry about 
    }
    try {
        /** update the number of tuples in output queues for all the children operators */
        for (int i = 0; i < operator.getNode().getChildren().length; i++) {
            XATOperator child = operator.getNode().getChild(i).getOperator();
            stats.updateValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", child);
        }
    } catch (
        edu.wpi.cs.dsrg.xmldb.xat.common.exception.XATNodeDoesNotExistException e) {
        //no child so nothing to worry about 
    }

    /** now determine if we should update the periodic properties for the operator*/
    long frequency =
        Long.parseLong(this.getProperty("UPDATE_OPERATOR_PROPERTY_FREQUENCY"));
    long lastTime = ((Long) lastTimeOperatorUpdated.get(operator)).longValue();
    long currentTime = System.currentTimeMillis();
    /** if it has been long enough, then update these statistics */
    if ((currentTime - lastTime) > frequency) {
        for (int i = 0; i < periodicOperatorStatistics.length; i++) {
            stats.updateValue(periodicOperatorStatistics[i], operator);
        }
    }
    /** update the operator's lastTimeUpdated so we stay consistent */
    lastTimeOperatorUpdated.put(operator, new Long(currentTime));

    frequency = Long.parseLong(this.getProperty("UPDATE_TREE_PROPERTY_FREQUENCY"));
    /** determine if we should update the periodic properties for the tree*/
    if ((currentTime - lastTimeTreesUpdated) > frequency) {
		XATTree[] treeArray = new XATTree[tree.size()];
			for(int i = 0; i < tree.size();i++){
				treeArray[i] = (XATTree) tree.get(i);
			}
        for (int i = 0; i < periodicTreeStatistics.length; i++) {
            stats.updateValue(periodicTreeStatistics[i], treeArray);
        }
        lastTimeTreesUpdated = System.currentTimeMillis();
    }
}
/**
 * Increments the count for the number of times each algorithm was run
 */
protected void updateSchedulerCount() {

	//if we havent seen the currentScheduler yet, then add it
	//to the scheduler count
	if(!schedulerCount.containsKey(currentScheduler)){
		schedulerCount.put(currentScheduler, new Integer(0));
	}
	int count = ((Integer)(schedulerCount.get(currentScheduler))).intValue() + 1;
	schedulerCount.put(currentScheduler, new Integer(count));
	
	}
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject("ExecutionController",this.getClass().getName(),true);
	}
}
