package edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Random;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObjectImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator.QueryPlanGenerator;
import edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator.QueryPlanGeneratorImp;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATExecutionAdapter;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATScheduler;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision;
import edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerException;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter;
import edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.XATStreamController;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.*;
//import edu.wpi.cs.dsrg.xmldb.xat.component.migrationcontroller.*;
//import edu.wpi.cs.dsrg.xmldb.xat.component.optimizer.rewritingrules.*;
//import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
//import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;

/**
 * This controls everything about execution including asking the scheduler
 * which operator to run next and for how long, running the operator, updating
 * the statistics for the operator, replanning, and changing the scheduling algorithm.  
 *  
 * @author: Brad Pielech
 */
public class MigrationExecutionController extends XATQueryObjectImp implements Runnable {
	private edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree tree;
	private edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter printer;
	private XATScheduler currentScheduler;
	private boolean neverRotateScheduler = false;
	/** array of statistics to periodically gatherer about operators */
	private String[] periodicOperatorStatistics;
	/** array of statistics to periodically gatherer about trees */
	private String[] periodicTreeStatistics;
	/** array of statistics to gatherer about operators everytime the operator is run*/
	private String[] everyTimeOperatorStatistics;
	private ConfigFileParser config = null;
	private int migrationAlgorithm = -1; // -1: no migration; 0- parallel track; 1- moving state
	
	/** 
	 * this defines the last time that certain properties about this operator were updated.
	 * In the ExecController, we do not want to update every property everytime an operator
	 * has run, but would prefer to do it periodically.  This property will keep track of the
	 * last time those properties were updated. The key is the operator, the value is the lastTime
	 * it was updated. 
	*/
	private java.util.Hashtable lastTimeOperatorUpdated;
	/** 
	 * this defines the last time that the status of execution was checked.  That is,
	 * when did we last determine how well execution is performing.  This value is
	 * tied to the STATUS_CHECK_FREQUENCY property.  
	*/
	private long lastTimeStatusChecked;
	/** 
	 * this defines the last time that the tree statistics were updated.  
	*/
	private long lastTimeTreesUpdated;
	/** 
	 * this defines the last time that tuples were uploaded to the application.  
	*/
	private long lastTimeApplicationRun;
	
	/**
	 * 
	 */
	//private RewriteRule rewriter = new PushDownJoinRule(); 
	
	/** this defines how long the execution will run */
	private long timeLeftToRun;
	private boolean debug;
	/** this adapter manages all of the scheduling algorithms and decides which one to run*/
	private XATExecutionAdapter adapter;
	/** how many times in a row an operator has tried to run, but has no work to do */
	private int starveCount = 0;
	/** how many times we will let an operator starve before we force an algorithm change */
	private int starveForce;
	private Thread systemThread;
	private double yield;
	private java.util.Random rand;
	private long sleepTime;
	/** signal to the execution that the streams have finished */
	private boolean streamsDone;
	/** signal to the outside that execution has finished */
	private boolean execDone;
	private XATStreamApplication application;
	private java.util.Hashtable schedulerCount;
	private boolean debug_scheduling_notice;
	private edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.XATStreamController controller;
/**
 * AnotherTester constructor comment.
 */
public MigrationExecutionController(ConfigFileParser parser, Thread t, XATStreamController control) throws Exception {
	super();			
	systemThread = t;
	controller = control;
	initExecution(parser);
	
}

/**
 * This method first checks if the query plan need to be optimized or not. If yes, it then locates the 
 * part of the query plan that need to be optimized and apply rewriting rules to the sub-plan. 
 * It then creates the old and new boxes and passes them to the migration controller to actually finish the migration.
 * For moving state, it's straight-forward, but for parallel states, the old sub-query plan has to be removed
 * after it is done purging all the old tuples. 
 *
 */
//public void optimizeQueryPlan() {
//	// method to check potential optimization spots, figuring out the sub-query plans.
//	// Method to apply rewriting rules to the sub-query plan.
//	// For now assume the sub-query plan contains the whole query plan.
//	// Need to be modified later.
//
//	//first update the PERFORMANCE_FACOTR property in each existing operators
//	XATNode[] oldNodes = tree.convertToArray();
//	double performance_facotr = 0;
//	try {
//		for (int i = 0; i < oldNodes.length; i++) {
//			XATStreamOperator oldOp = (XATStreamOperator) oldNodes[i].getOperator();
//			oldOp.setProperty("PERFORMANCE_FACTOR", stats.getValue("REDUCTION_FACTOR", oldOp));
//		}
//	} catch (Exception e) {
//		System.out.println("Caught exception when try to update PERFORMANCE_FACTOR: " + e);
//	}
//	
//	if (rewriter.CheckRule(tree)) {
//		long startTime = System.currentTimeMillis();
//		XATTree newTree = (XATTree) tree.clone();
//		System.out.println(
//			"Tree Clone takes " + (System.currentTimeMillis() - startTime));
//
//		// clear tuples in newTree
//		XATNode[] newNodes = newTree.convertToArray();
//		for (int i = 0; i < newNodes.length; i++)
//			 ((XATStreamOperator) newNodes[i].getOperator()).clearAllTuples();
//
//		System.out.println("Applying rewriting rules to the new query plan");
//	//if (rewriter.ApplyRule(newTree)) {
//		rewriter.ApplyRule(newTree);
//		
//		/** setup oldbox */
//		startTime = System.currentTimeMillis();
//		XATBox oldBox = new XATBox((DagNode)tree.getRoot(), (XATDag)tree);
//
//		/*init migration controller and clean accumulated tuples from old box */
//		MigrationController mcontrol = new MigrationController((XATDag)tree, oldBox);
//		mcontrol.cleanAccumulatedTuples();
//
//		/* setup new box. connect it to current dag, recompute states and disconnect old box from current dag */
//		XATBox newBox = new XATBox((DagNode)newTree.getRoot(), (XATDag)newTree);
//		mcontrol.setNewBox(newBox);
//		mcontrol.setupMSMigration(); //moving state migration
//
//		/* add operators in new box and remove operators in new box from exec controller */
//		//XATStreamOperator[] newOperators = newBox.getAllOperators();
//		//XATStreamOperator[] newRootOperators = new XATStreamOperator[1];
//		//newRootOperators[0] = newBox.getBoxRootOperator();
//		//exec.removeOperatorsToRun(oldOperators, oldRootOperators);
//		//exec.removeQueuesToCollect(oldBox.getInterQs());
//		//exec.removeStatesToCollect(oldBox.getAllStates());
//		//exec.addOperatorsToRun(newOperators, newRootOperators);
//		//exec.addQueuesToCollect(newBox.getInterQs());
//		//exec.addStatesToCollect(newBox.getAllStates());
//
//		/**  Restart execController thread. */
//		System.out.println(
//			"Migration takes " + (System.currentTimeMillis() - startTime));
//
//		/** create a new adaptor due to the change to the tree. */
//		try {
//			String className = this.getProperty("ADAPTIVE_HEURISTIC");
//			if (className
//				.compareTo("edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.NeverRotateAdapter")
//				== 0)
//				neverRotateScheduler = true;
//
//			Constructor[] c1 = Class.forName(className).getConstructors();
//			Constructor construct = c1[1];
//			adapter = (XATExecutionAdapter) construct.newInstance(new Object[] { config, newTree });
//			//start at scheduler 0 
//			currentScheduler = adapter.getCurrentScheduler();
//			updateSchedulerCount();
//			for (int i = 0; i < oldNodes.length; i++) {
//				XATStreamOperator oldOp = (XATStreamOperator) oldNodes[i].getOperator();
//				stats.removeQueryObject(oldNodes[i].getOperator());
//				lastTimeOperatorUpdated.remove(oldNodes[i].getOperator());
//				Iterator oldQueues = oldOp.getInputQueues();
//				while (oldQueues.hasNext()) {
//					stats.addNewQueryObject((XATQueue) oldQueues.next());
//				}
//			}
//			newNodes = tree.convertToArray();
//			for (int i = 0; i < newNodes.length; i++) {
//				XATStreamOperator newOp =
//					(XATStreamOperator) newNodes[i].getOperator();
//				stats.addNewQueryObject(newOp);
//				lastTimeOperatorUpdated.put(newNodes[i].getOperator(), new Long(0));
//				Iterator newQueues = newOp.getInputQueues();
//				while (newQueues.hasNext()) {
//					stats.addNewQueryObject((XATQueue) newQueues.next());
//				}
//			}
//		} catch (Exception e) {
//			System.out.println(
//				"Exception catched during query plan optimization: " + e);
//		}
//	} else
//		System.out.println("The query plan is not changed.");
//}

/**
 * If the time since the status was checked is greater than STATUS_CHECK_FREQUENCY,
 * then this will check the status of execution, otherwise it will just return.  
 * 
 */
private void checkExecStatus(boolean forceChange) {

	long frequency = Long.parseLong(this.getProperty("STATUS_CHECK_FREQUENCY"));
	//if we are not forcing a change and if it isnt time to do so, we won't check the exec status
	if((!forceChange) && ((System.currentTimeMillis() - lastTimeStatusChecked) < frequency)){		
			/** not time to check the status of execution */
			return;		
	} else{		
		/** 
		 * ok, time to check the status of execution 
		 * first we will update the tree and operator statistics b/c these contain a lot of
		 * the metrics that will evaluate the execution status
		*/
		
		XATNode[] nodes = tree.convertToArray();
    	for (int j = 0; j < nodes.length; j++) {
     	   for (int i = 0; i < periodicOperatorStatistics.length; i++) {
      	      stats.updateValue(periodicOperatorStatistics[i], nodes[j].getOperator());
       	   }
     	   lastTimeOperatorUpdated.put(nodes[j].getOperator(), new Long(System.currentTimeMillis()));
    	}
    	// the following has been updated in method updatePostExecStats()								
//		for (int i = 0; i < periodicTreeStatistics.length; i++){
//			stats.updateValue(periodicTreeStatistics[i], tree);
//		}
//		lastTimeTreesUpdated = System.currentTimeMillis();
		if(debug){
			System.out.println("*****Time to check Execution Status*****");
		}
		//Thread.currentThread().yield();
		if(forceChange && !neverRotateScheduler){
			currentScheduler = adapter.forceChange();
		} else{
			currentScheduler = adapter.getNextScheduler();
		}
				
		//only gather statistics if we changed the scheduling algorithm
//		System.out.println("*****Going to change scheduling algorithm *****");
//			printer.getStatistics();
		stats.setCurrentCategory(adapter.getCategory());
		if(debug){
			System.out.println("***** Next Scheduler is: " + adapter.getCategory().toString() + " *****");
		}
		printer.setCategory(stats.getCurrentCategory());
		lastTimeStatusChecked = System.currentTimeMillis();
		updateSchedulerCount();
	}	
	
		
}
/**
 * First initializes the query plan, generates 1 tree and returns the tree
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
private XATTree generateQueryPlan(ConfigFileParser config) throws Exception{
	return initQueryPlanGenerator(config).makeQueryPlan();    
}
/**
 * Returns the current scheduling algorithm
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATScheduler
 */
public edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATScheduler getCurrentScheduler() {
	return currentScheduler;
}
/**
 * Returns the handle to the statistics printer
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter
 */
public edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter getPrinter() {	
	return printer;
}
/**
 * Insert the method's description here.
 * Creation date: (4/6/2003 8:46:07 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree getTree() {
	return tree;
}
/**
 * Initializes some of the class variables of this execution controller
 */
private void initClassVariables() throws Exception {
	
	debug = (new Boolean(this.getProperty("EXECUTION_DEBUG"))).booleanValue();
	debug_scheduling_notice =  (new Boolean(this.getProperty("SCHEDULING_NOTIFICATION_DEBUG"))).booleanValue();
	Thread execThread = new Thread(this);
	execThread.setName("Execution Controller");
	rand = new Random();
	yield = Double.parseDouble(this.getProperty("YIELD_PROBABILITY"));
	starveForce = Integer.parseInt(this.getProperty("STARVATION_CHECK"));
	timeLeftToRun = Long.parseLong(this.getProperty("EXECUTION_DURATION"));
	sleepTime = Long.parseLong(this.getProperty("NO_WORK_SLEEP_TIME"));	
	/** 
     * init the hashtable containing each operator  
    */
	
	lastTimeOperatorUpdated = new Hashtable();
    lastTimeStatusChecked = System.currentTimeMillis();
    lastTimeTreesUpdated = System.currentTimeMillis();
    lastTimeApplicationRun = System.currentTimeMillis();
    application = XATStreamApplication.getXATStreamApplication();
    setStreamsDone(false);
    setExecDone(false);

    schedulerCount = new Hashtable();
}
/**
 * Initializes various parts of the execution including
 * query plan, input, schedulers, and statistics table
 */
private void initExecution(ConfigFileParser config) throws Exception {
	/** next set up the ExecutionController properties */
	this.config = config;
	for (Iterator i = config.getExecutionControllerProperties();i.hasNext();) {
	    	NameValuePair pair = (NameValuePair)i.next();
	    	setProperty(pair.getName(), pair.getValue());
	}
	if(this.getProperty("MIGRATION_ALGORITHM").compareToIgnoreCase("pt") == 0) {
		migrationAlgorithm = 0;
	}
	else if(this.getProperty("MIGRATION_ALGORITHM").compareToIgnoreCase("ms") == 0) {
		migrationAlgorithm = 1;
	}
		
	tree = generateQueryPlan(config);		      
    for(Iterator i = config.getTreeProperties(); i.hasNext();){
	    NameValuePair pair = (NameValuePair)i.next();
		tree.setProperty(pair.getName(), pair.getValue());
    }
    
	//set up the rest of the class variables
	initClassVariables();		
	application.connectApplicationToTree(this.getTree());
	
	stats.setProperty("TIMER", this.getProperty("STATS_TABLE_STATS"));			
    stats.initStatsTable(tree);          	             		
    if(debug){
	    System.out.println(tree);
	}
	
    controller.connectStreamsWithTree(tree); 
    /**
      init the last time some of its properties were updated
      set the last updated time to 0, that way the properties will be updated the first time.    
      set up the OBJECT_ID's of the operators.  This helps with experiments 
    */
	XATNode[] nodes = tree.convertToArray();
	int id = 1;
	for (int i = 0; i < nodes.length; i++) {
		XATStreamOperator operator = (XATStreamOperator) nodes[i].getOperator();
		
		// if it is of type WBinaryJoinOperator, (not the dummy XATWBinaryJoinOperator)
		//  init the operator to set the necessary parameters: window, predicates, etc.
		if (operator instanceof WBinaryJoinOperator) {
			((WBinaryJoinOperator) operator).init();
		}
		lastTimeOperatorUpdated.put(operator, new Long(0));
		operator.setProperty("OBJECT_ID", "" + id);
		operator.setActive(true);
		id++;
	}
     
    //set up schedulers including the adaptive scheduler
    String className = this.getProperty("ADAPTIVE_HEURISTIC"); 
    if(className.compareTo("edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.NeverRotateAdapter") == 0)
    	neverRotateScheduler = true;
    Constructor[] c1 = Class.forName(className).getConstructors();
    Constructor construct = c1[1];
    adapter = (XATExecutionAdapter)construct.newInstance(new Object[] { config, tree});
    //start at scheduler 0 
    currentScheduler = adapter.getCurrentScheduler();
    updateSchedulerCount();
    //set up the printer
    initStatisticsPrinter(config);
    //set up some of the statistics to gather
    initStatsToGather(config);
    //set the starting category in the statistics gatherer to be the first scheduler
	stats.setCurrentCategory(adapter.getCategory());
	printer.setCategory(stats.getCurrentCategory());	
}
/** 
 * traverse through the query plan generator properties.  The most
 * important one to find is equal to QueryPlanGeneratorImp.factoryID because
 * this is a system property, the rest belong to the queryplangenerator
 * store items in a temporary ArrayList until we find that property, then
 * just instantiate the queryplangenerator and set the rest of the properties. 
 */
private QueryPlanGenerator initQueryPlanGenerator(ConfigFileParser config) throws Exception{
    QueryPlanGenerator gen = null;;
        ArrayList temp = new ArrayList();			
        for(Iterator i = config.getQueryPlanGeneratorProperties(); i.hasNext();){
			NameValuePair pair = (NameValuePair)i.next();
			if(pair.getName().equals(QueryPlanGeneratorImp.factoryID)){
				System.setProperty(pair.getName(), pair.getValue());
				gen = QueryPlanGeneratorImp.newInstance();
			} else{
				/** if the generator has not been instantiated yet, store the pair in the temp list*/
				if(gen == null){
					temp.add(pair);
				} else{
					/** otherwise, just set that property */
					gen.setProperty(pair.getName(), pair.getValue());
				}
			}
        }
        //add in the rest of the properties
        for(Iterator i = temp.iterator();i.hasNext();){
			NameValuePair pair = (NameValuePair)i.next();
			gen.setProperty(pair.getName(), pair.getValue());
        }
	return gen;
}
/**
 * Initializes the statistics printer by call the config file parser and setting all of the
 * properties that are returned
 * 
 */
private void initStatisticsPrinter(ConfigFileParser config) throws Exception {
    printer = new StatisticsPrinter();
    for (Iterator i = config.getOutputFormatProperties(); i.hasNext();) {
        NameValuePair pair = (NameValuePair) i.next();
        printer.setProperty(pair.getName(), pair.getValue());        
    }
   ArrayList columnList = new ArrayList();
   for (Iterator i = config.getColumnNamesForOutput();i.hasNext();) {
	   columnList.add(i.next());   	
   }
   printer.setColumnHeaders(columnList);

}
/**
 * Calls config file parser and gathers all of the statistics that 
 * should be gatherered for an operator both every time it is run and perodically
 *
 * This will also do the same for tree statistics
 */
private void initStatsToGather(ConfigFileParser config) throws Exception {

	/** call populate statistics to do the real work */
	everyTimeOperatorStatistics = populateStatistics(config.getEveryTimeOperatorStatistics());
	periodicOperatorStatistics = populateStatistics(config.getPeriodicOperatorStatistics());
	periodicTreeStatistics = populateStatistics(config.getPeriodicTreeStatistics());
}
/**
 * returns whether or not the query execution has finished
 * @return boolean
 */
public boolean isExecDone() {
	return execDone;
}
/**
 * Insert the method's description here.
 * Creation date: (5/1/2003 2:51:05 PM)
 * @return boolean
 */
public boolean isStreamsDone() {
	return streamsDone;
}
/**
 * This will update the statistics for operators and take care of any additional
 * book keeping before the operator is allowed to run.
 * @param decision XATSchedulerDecision
 * @return boolean true if the operator should continue to run, false if it should not 
 */
private boolean performPreExecTasks(XATSchedulerDecision decision)
    throws Exception {
    boolean returnValue = true;
    
    /**
     * if the workload is 0, then we dont want to run the operator,
     * but rather note that we detected starvation.  If
     * we get too many starvations in a row, we will force a change of algorithm
     * If that doesnt work, the execution will terminate.
     * Note: some algorithms have built in starvation checks, therefore, this
     * checking is mearly a little overhead.  However, other algorithms are
     * inherently prone to starvation and thus, this check is necessary, otherwise
     * we dont get anywhere.  
     */	
    if (decision.getNumberOfTuples() < 1) {
        starveCount++;
        returnValue = false;
        if (starveCount == starveForce) {
            checkExecStatus(true);            
        }
    } else {
        starveCount = 0;        
        XATStreamOperator operator = (XATStreamOperator) decision.getNextOperator();
        if (debug) {
	        System.out.println("Next operator to run: " + operator);
          	System.out.println("Dequeuing this many tuples: " + decision.getNumberOfTuples() + "\n");
        }
        /** set the operator's property */
        operator.setProperty("NUMBER_OF_TUPLES_TO_DEQUEUE", "" + decision.getNumberOfTuples());
        //stats.updateValue("NUMBER_OF_TUPLES_TO_DEQUEUE", operator, "" + workload);
        stats.updateValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator);
        /** 
         * figure out how many tuples are available for this operator
         * and update the TUPLE_COUNT property.  This will be used
         * later to figure out the number of possible outputs.
         */
        for (Iterator i = operator.getInputQueues(); i.hasNext();) {
            XATQueue queue = (XATQueue) i.next();
            int work = queue.getQueueSize(operator);
            stats.updateValue("TUPLE_COUNT", queue, "" + work);
        }

        returnValue = true;
    }
    return returnValue;
}
/**
 * Takes an iterator of NameValuePairs and adds the names to the given String array.
 * This is used for taking the names of the statisticsToGather and putting them into the appropriate array 
 * @param iterator Iterator
 * @return array java.lang.String[]
 */
private String[] populateStatistics(Iterator iterator) {

    ArrayList list = new ArrayList();
    for (; iterator.hasNext();) {
        list.add(iterator.next());
    }
    //initialize the Array size
    String[] array = new String[list.size()];
    int index = 0;
    //initialize the properties in the array
    for (Iterator i = list.iterator(); i.hasNext();) {
        NameValuePair pair = (NameValuePair) i.next();
        array[index] = pair.getName();
        index++;
    }
    return array;
}
/**
 * Runs the actual execution
 *
 */
public void run() {
    try {
    } catch (Exception e) {
        System.out.println(e.toString());
        systemThread.interrupt();
        return;
    }    
    long optimizeInterval = Long.parseLong(this.getProperty("OPTIMIZATION_INTERVAL"));
    long totalTime = 0, timeSpentSleeping = 0;
    long startTime = System.currentTimeMillis();
    long timeHasRun = 0;
    long lastTimeOptimized = startTime;
    boolean keepGoing = true;
    XATSchedulerDecision d = null;
	totalTime = System.currentTimeMillis();
	
    while (keepGoing) {
        try {
            /** have the scheduler make the next decision */

            d = currentScheduler.makeNextDecision();
            /** 
              perform some pre exec tasks like configuring operator, calculating statistics 
              and whatnot.  If this method returns false, then we should not run the operator
              for one reason or another (it could have no work to do or there could be
              starvation or something)
            */
            if (performPreExecTasks(d)) {
                XATStreamOperator operator = d.getNextOperator();                
                int oldOutput =
                    Integer.parseInt(stats.getValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator));
                String timeBefore = "" + System.currentTimeMillis();
                //run the operator
                operator.run();
                String timeAfter = "" + System.currentTimeMillis();
                String timeRun = "" + (Long.parseLong(timeAfter) - Long.parseLong(timeBefore));
                //System.out.println("Time run for "+operator+"is "+timeRun);
                /**
                  these stats should really be lumped with updatePostExec, but
                  that would require passing tooo many parameters to the method, so
                  they are just here.  Yeah, I know...
                */
                stats.updateValue("LAST_TIME_RUN", operator, timeAfter);
                stats.updateValue("TIME_TOOK_TO_RUN", operator, timeRun);
                stats.updateValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", operator);
                stats.updateValue("NUMBER_OF_TUPLES_OUTPUTTED", operator, "" + oldOutput);
                //now update the stats related to the operator and how it performed
                updatePostExecStats(operator);
                //check the status, but don't force a change
                //checkExecStatus(true);
                //yield with a certain probibility
                /*
                if (rand.nextDouble() < yield) {
                    //System.out.println("** Execution Thread Yielding **");
                    Thread.currentThread().yield();
                }
                */
                //figure out how long we ran for last time
              	totalTime = (System.currentTimeMillis() - totalTime); 
				//System.out.println("Time left to Run: " + timeLeftToRun); 
                timeLeftToRun -= totalTime;                
                System.out.println("Time left to Run: " + timeLeftToRun);
				totalTime = System.currentTimeMillis();
                //if we have reached the end, just terminate
                if (timeLeftToRun <= 0) {
                    System.out.println("Execution has run for the allotted time, now terminiating");
                    keepGoing = false;
                }
            } else {
                //don't run the operator 
                
            }
			//check if it's time to try to optimize
			if((migrationAlgorithm != -1) && (System.currentTimeMillis() - lastTimeOptimized) > optimizeInterval) {
				System.out.println("\nExec has run for "+(System.currentTimeMillis() - lastTimeOptimized));
				System.out.println("Now try to optimize query plan.");
				lastTimeOptimized = System.currentTimeMillis();
				long opstarttime = lastTimeOptimized;
				//optimizeQueryPlan();
				System.out.println("Optimization takes " + (System.currentTimeMillis() - opstarttime));
			}
			

        } catch (XATSchedulerException e1) {
            /**
               No work for schedulers so just first see if the streams are done.  If
               they are, exit, otherwise, sleep for a bit and hope there is work for us when we wake up	
            */
            try {
                //System.out.println("No work to do so sleeping");
                if(isStreamsDone()){
					System.out.println("Execution has no work and streams are done, now terminiating");
                    keepGoing = false;
                } else { 
                //Thread.currentThread().sleep(sleepTime);
                Thread.sleep(sleepTime);
                timeLeftToRun -= sleepTime;
                timeSpentSleeping += sleepTime;
                //System.out.println("Time left to Run: " + timeLeftToRun);
                if (timeLeftToRun <= 0) {
                    System.out.println("Execution has run for the allotted time, now terminiating");
                    keepGoing = false;
                }
                }
            } catch (InterruptedException e) {
                //interrupted, but just continue working 
            }

        } catch (Exception e) {
            /** 
            * this comes from setting a property or running an operator.
            * if it comes from running an operator it is bad, but I'll figure out how to
            * deal with it later
            */
            System.out.println(e.toString());
            systemThread.interrupt();
            return;
        }
    }
    
    totalTime = System.currentTimeMillis() - startTime;
    System.out.println("Execution ran for " + totalTime);
    System.out.println("Execution slept for " + timeSpentSleeping);
    //System.out.println("Application removed " + application.getCount() + " tuples from the output queues");
    /** 
    * now the we are done running, update the periodic properties one last time
    * to ensure the up to date value is printed out
    */
    XATNode[] nodes = tree.convertToArray();
    for (int j = 0; j < nodes.length; j++) {
        for (int i = 0; i < periodicOperatorStatistics.length; i++) {
            stats.updateValue(periodicOperatorStatistics[i], nodes[j].getOperator());
        }
    }

    for (int i = 0; i < periodicTreeStatistics.length; i++) {
        stats.updateValue(periodicTreeStatistics[i], tree);
    }
    /** stop the printing because execution is done */
    printer.interrupt();
    for (Enumeration e = stats.getCategories(); e.hasMoreElements();) {
        Object o = e.nextElement();
        if (!new String("GENERIC").equals(o)) {
            printer.setCategory(o);
            printer.gatherStatistics();
        }
    }
    printer.print();
    if (debug) {
        for (Enumeration e = stats.getProperties(); e.hasMoreElements();) {
            String next = (String) e.nextElement();
            System.out.println(next + ": " + stats.getProperty(next));
        }
    }
    //print out the number of times each algorithm was run
	if(debug_scheduling_notice){
    	System.out.println("\n---");
    	System.out.println("Number of times algorithms chosen:");
    	for(Enumeration e = schedulerCount.keys();e.hasMoreElements();){
			Object next = e.nextElement();
			String className = next.getClass().getName();
			int index = 0;
			for(index = className.length()-1; index >=0; index--) {
				if(className.charAt(index) == '.')
					break;
			}
			className = className.substring(index + 1);		
			System.out.println(className + ":\t" + schedulerCount.get(next));
    	}
	}
	setExecDone(true);
    systemThread.interrupt();
    
    return;
}
	/**
    * called by the constructor.  this will set the default properties
    * that the ALL query objects will support. Each subclass should override
    * this method and add their own unique properties, but should also call
    * super.setDefaultProperties() so that the common properties may be set.
    */   
	protected void setDefaultProperties(){
		super.setDefaultProperties();
		/** 
		 * this defines how often some of the properties should be updated
		 * some properties, such as operator output rate do not need to be updated
		 * every time an operator is run, but rather, they can be updated periodically
		 * This will define how often (in seconds) these properties should be updated.  
		 *
		 * The tree frequency serves the same purpose, except for tree properties
		 */
		addProperty("UPDATE_OPERATOR_PROPERTY_FREQUENCY","5000");
		addProperty("UPDATE_TREE_PROPERTY_FREQUENCY","5000");
		/** 
		 * similar to the previous property, this defines how often
		 * the ExecController should check the execution status and determine if
		 * the scheduling algorithm should change.  This value should be set sufficently high
		 * such that any strategy has a chance to work (or not work).
		*/		
		
		addProperty("STATUS_CHECK_FREQUENCY","10000");
		addProperty("STATS_TABLE_STATS","false");
		addProperty("EXECUTION_DEBUG","false");
		//true if you want notification of the next scheduling algorithm and statistics
		//about which algorithm ran to be printed
		addProperty("SCHEDULING_NOTIFICATION_DEBUG","false");		
		/** the default adapter will always stick with the first algorithm */
		addProperty("ADAPTIVE_HEURISTIC","edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.NeverRotateAdapter");
		/** how many times will an operator be run (in a row) without any work before the algorithm is changed */
		addProperty("STARVATION_CHECK","3");
		addProperty("YIELD_PROBABILITY", "0.1");
		/** how long, in milleseconds, that the query should be executed */
		addProperty("EXECUTION_DURATION","10000");
		addProperty("NO_WORK_SLEEP_TIME","100");
		addProperty("APPLICATION_UPLOAD_FREQUENCY","1000");
		addProperty("MIGRATION_ALGORITHM","off");
		addProperty("OPTIMIZATION_INTERVAL", "5000");
	}
/**
 * Insert the method's description here.
 * Creation date: (5/1/2003 10:37:49 PM)
 * @param newExecDone boolean
 */
private void setExecDone(boolean newExecDone) {
	execDone = newExecDone;
}
/**
 * Insert the method's description here.
 * Creation date: (5/1/2003 2:51:05 PM)
 * @param newStreamsDone boolean
 */
public synchronized void setStreamsDone(boolean newStreamsDone) {
	streamsDone = newStreamsDone;
}
/**
 * Insert the method's description here.
 * Creation date: (1/27/03 1:26:53 PM)
 * @return java.lang.String
 */
public String toString() {
	String rs = new String();

	String sName = this.getClass().getName();
	//find the last dot.
	int index = 0;
	for(index = sName.length()-1; index >=0; index--)
		if(sName.charAt(index) == '.')
			break;
	rs = sName.substring(index + 1) + " " + hashCode(); // + "  data:";	
    return rs;
}
/**
 * This will update the statistics for operators. This is broken into 2 phases,
 * the first phase will update those properties that should be updated every time.  The
 * rest define those that are updated periodically 
 * @param operator edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator
 */
private void updatePostExecStats(XATStreamOperator operator) {

    for (int i = 0; i < everyTimeOperatorStatistics.length; i++) {
        stats.updateValue(everyTimeOperatorStatistics[i], operator);
    }
    /**
     * can't really generalize the next 2 properties
     *
     * also update the NUMBER_OF_TUPLES_IN_INPUT_QUEUES property for the operator's
     * parent(s)
     */
    try {
	    XATNode parent = operator.getNode().getParent();
	    if(parent != null){
		    XATOperator parentOp = parent.getOperator();
	        stats.updateValue("NUMBER_OF_TUPLES_IN_INPUT_QUEUES", parentOp);
        }
        
        
    } catch (Exception e) {
        //no parent so nothing to worry about 
    }
    try {
        /** update the number of tuples in output queues for all the children operators */
        for (int i = 0; i < operator.getNode().getChildren().length; i++) {
            XATOperator child = operator.getNode().getChild(i).getOperator();
            stats.updateValue("NUMBER_OF_TUPLES_IN_OUTPUT_QUEUES", child);
        }
    } catch (
        edu.wpi.cs.dsrg.xmldb.xat.common.exception.XATNodeDoesNotExistException e) {
        //no child so nothing to worry about 
    }

    /** now determine if we should update the periodic properties for the operator*/
    long frequency =
        Long.parseLong(this.getProperty("UPDATE_OPERATOR_PROPERTY_FREQUENCY"));
    long lastTime = ((Long) lastTimeOperatorUpdated.get(operator)).longValue();
    long currentTime = System.currentTimeMillis();
    /** if it has been long enough, then update these statistics */
    if ((currentTime - lastTime) > frequency) {
        for (int i = 0; i < periodicOperatorStatistics.length; i++) {
            stats.updateValue(periodicOperatorStatistics[i], operator);
        }
    }
    /** update the operator's lastTimeUpdated so we stay consistent */
    lastTimeOperatorUpdated.put(operator, new Long(currentTime));

    frequency = Long.parseLong(this.getProperty("UPDATE_TREE_PROPERTY_FREQUENCY"));
    /** determine if we should update the periodic properties for the tree*/
    if ((currentTime - lastTimeTreesUpdated) > frequency) {
        for (int i = 0; i < periodicTreeStatistics.length; i++) {
            stats.updateValue(periodicTreeStatistics[i], tree);
        }
        lastTimeTreesUpdated = System.currentTimeMillis();
    }
}
/**
 * Increments the count for the number of times each algorithm was run
 */
private void updateSchedulerCount() {

	//if we havent seen the currentScheduler yet, then add it
	//to the scheduler count
	if(!schedulerCount.containsKey(currentScheduler)){
		schedulerCount.put(currentScheduler, new Integer(0));
	}
	int count = ((Integer)(schedulerCount.get(currentScheduler))).intValue() + 1;
	schedulerCount.put(currentScheduler, new Integer(count));
	
	}

	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject("ExecutionController",this.getClass().getName(),true);
	}
}
