package edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller;

import java.util.Enumeration;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATPropertiesImp;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsGatherer;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsPrinter;
import edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.Listener;
import edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.StreamClient;
/**
 * This is the Experiment Environment setup.  It will parse the
 * configuration file and run the stream controller and execution controller.
 * It also supports batched experiments.  After the exec controller is done, the
 * whole experiment can be performed again.  
 * @author: Brad Pielech
 */
public class SystemSetup extends XATPropertiesImp implements Runnable, Listener{
	private java.lang.String configName;

	/**Stores the reference to the thread in which the execution controller runs*/
	private Thread systemThread = null;


/**
 * ExperimentSetup constructor comment.
 */
public SystemSetup(String configFileName) {
	super();

	configName = configFileName;
	this.systemThread = Thread.currentThread();	

}
/**
 * Insert the method's description here.
 * Creation date: (4/10/03 12:07:38 PM)
 * @return java.lang.Thread
 */
public java.lang.Thread getSystemThread() {
	return Thread.currentThread();
}
/**
 * When an object implementing interface <code>Runnable</code> is used 
 * to create a thread, starting the thread causes the object's 
 * <code>run</code> method to be called in that separately executing 
 * thread. 
 * <p>
 * The general contract of the method <code>run</code> is that it may 
 * take any action whatsoever.
 *
 * @see     java.lang.Thread#run()
 */
public void run() {

    try {

        ConfigFileParser config = new ConfigFileParser(configName);
        /** first set up the system properties */
        for (Iterator i = config.getSystemProperties(); i.hasNext();) {
            NameValuePair pair = (NameValuePair) i.next();
            System.setProperty(pair.getName(), pair.getValue());
        }
        for (Iterator i = config.getExperimentProperties(); i.hasNext();) {
            NameValuePair pair = (NameValuePair) i.next();
            this.setProperty(pair.getName(), pair.getValue());
        }
        

        int howMany = Integer.parseInt(this.getProperty("TIMES_TO_RUN"));
        for (int i = 0; i < howMany; i++) {

            System.out.println("Still have " + (howMany - i) + " experiments left");
 
            StreamClient control =
                new StreamClient(
                    this.getProperty("STREAM_CONFIG_FILE_NAME"));
			control.addListener(this);
            MigrationExecutionController exec =
                new MigrationExecutionController(config, Thread.currentThread(), control);
            //control.connectStreamsWithTree(exec.getTree());
		
            String s = new String();
            boolean printMeta =
                new Boolean(this.getProperty("PRINT_OUT_META_INFORMATION")).booleanValue();
            /** now set up the meta information of the printer */
            if (printMeta) {
                StatisticsPrinter printer = exec.getPrinter();
                String newLine = "\n";
                if (printer.getFormat() == StatisticsPrinter.HTML) {
                    newLine = "|";
                }
                s = s + "Execution Controller properties:" + newLine;
                for (Enumeration e = exec.getProperties(); e.hasMoreElements();) {
                    String name = (String) e.nextElement();
                    s = s + name + ": " + exec.getProperty(name) + newLine;
                }

                s = s + "SchedulerProperties: " + newLine;
                for (Enumeration e = exec.getCurrentScheduler().getProperties();
                    e.hasMoreElements();
                    ) {
                    String name = (String) e.nextElement();
                    s = s + name + ": " + exec.getCurrentScheduler().getProperty(name) + newLine;
                }
                s = s + "Stream Generator Properties: " + newLine;
                for (Enumeration e = control.getProperties(); e.hasMoreElements();) {
                    String name = (String) e.nextElement();
                    s = s + name + ": " + control.getProperty(name) + newLine;
                }
                printer.setMetaInformation(s);
            }
                   	

			Thread streamThread = new Thread(control);
			streamThread.setDaemon(true);
			streamThread.setName("Stream Receiver");
			streamThread.start();
			// begin
//			try {
//				streamThread.join();
//			} catch(Exception e) {
//				e.printStackTrace();
//			}
			//end
            Thread execThread = new Thread(exec);
            execThread.setName("Execution");

            Thread printerThread = new Thread(exec.getPrinter());
            printerThread.setName("Statistics Printer");
            printerThread.setDaemon(true);
			
            execThread.start();
            //////////////////////
            printerThread.start();

            /**
             * this has to sleep indefinately
             * and wait for either: 
             * the exec controller or stream controller thread to notify this thread to continue to run 
             */
            boolean keepGoing = true;
            while (keepGoing) {
                try {
                    //getSystemThread().sleep(10000);
                    Thread.sleep(10000);
                } catch (InterruptedException e) {
                    //see which thread is still running and interrupt it
                    //but wait for all 3 threads to die before continuing
                    while (//streamThread.isAlive() ||
                         printerThread.isAlive()
                        || execThread.isAlive()) {
                        if (printerThread.isAlive()) {
                            printerThread.interrupt();
                        }
//                       if (streamThread.isAlive()) {
//                            streamThread.interrupt();
//                        }
                        if (execThread.isAlive()) {
                            exec.setStreamsDone(true);
                            //execThread.interrupt();
                        }
                        try {
                            //getSystemThread().sleep(500);
                            Thread.sleep(500);
                        } catch (InterruptedException e2) {
                            //keep looping while the execution is not done
                        }

                    }
                    /** reset the statistics gatherer */
                    StatisticsGatherer.getStatisticsGather().reset();
                    /** reset the xat application */
                    XATStreamApplication.getXATStreamApplication().reset();                    
                 
                    keepGoing = false;                  
                }
            }
            System.out.println("*************\n");
			System.gc();
        }

    } catch (Exception e) {
        System.out.println(e.toString());
        return;
    }
    System.out.println("Experiments are over.");
    return;

}
	/**
    * called by the constructor.  this will set the default properties
    * that the ALL query objects will support. Each subclass should override
    * this method and add their own unique properties, but should also call
    * super.setDefaultProperties() so that the common properties may be set.
    */   
	protected void setDefaultProperties(){
		super.setDefaultProperties();
		addProperty("TIMES_TO_RUN", "1");
		addProperty("PRINT_OUT_META_INFORMATION", "1");
		addProperty("STREAM_CONFIG_FILE_NAME", "1");				
	}
	

	/**
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.Listener#endOfStream()
	 */
	public void endOfStream() {
		this.systemThread.interrupt();		
	}
	
}
