package edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATEndUserApplicationQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATEndUserApplicationQueueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObjectImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
/**
 * This represents an end user application.  This will remove tuples
 * from the output queues of the query plan and help to ease the memory requirements
 * of the system by deleting the tuples it dequeues. <p>
 * This class basically has 1 function and that is to compute the average tuple
 * age of all tuples in the output queues of the tree.  This class has changed
 * from the previous version:
 * <ol>
 *  <li> it is never told to run by the exec controller, but rather, when the
 * 	statistics gatherer wishes to update the average tuple age, it will invoke
 *  the calculateTupleAge method. This method will calculate the average age of the tuples
 *  and also clear those queues.  Those queues now use the XATEndUserApplicationQueues
 *  that do not actually store tuples, but rather the time the tuples spent in the system.
 *	<li> Application does still keep track of how many tuples have been outputted. 
 * </ol>
 * @deprecated since 1.2
 * @author: Brad Pielech
 */
public class XATStreamApplication extends XATQueryObjectImp{
	private XATEndUserApplicationQueue[] queues;
	private java.util.Random rand;
	private long count;
	private static XATStreamApplication app = null;
/**
 * XATStreamApplication constructor comment.
 */
private XATStreamApplication() {
	super();
	rand = new Random();
	count = 0;
}
/**
 * This method will iterate over the known queues, dequeue
 * all available tuples and then record the average time that
 * they spent in the system.  By making use of XATEndUserApplicationQueues,
 * when we dequeue, we are just returned the time they spent in the system.
 * Therefore, we will just be returned a list of Longs (representing the time
 * they spent in the system) and not tuples!  
 *
 * @param removeTuples true if the XSA should empty the tuples, false if it should
 * leave the queues alone
 * @return double
 */
public double calculateAverageTupleAge(boolean removeTuples) {
	long ageCount = 0;
	int totalSizes = 0;
	int size = 0;
	for (int i = 0; i < queues.length; i++) {
        //Thread.yield();
        size = queues[i].getQueueSize(this);
        totalSizes += size;
        if (size > 0) {
            
            List ages = queues[i].peekN(size, this);
            for(int j = 0; j < ages.size(); j++){
				ageCount += ((Long)ages.get(j)).longValue();
            }
            //System.out.println("App just removed " + size + " tuples from the output queues");
        }
        if(removeTuples) {
	        queues[i].removeAllTuples();
	        count += size;
        }
	}
	double avg = (totalSizes > 0) ? (ageCount / totalSizes):(0);
	return avg;

}
/**
 * Connects the appliction to the Tree by replacing the output queues
 * of the root operators with XATEndUserApplicationQueues 
 * @param tree XATTree the tree with the roots whose queues will be
 * connected
 */
public void connectApplicationToTree(XATTree tree) {
	ArrayList tempQ = new ArrayList();
	//iterate through the roots of the tree
	for(Iterator i = tree.getRoots(); i.hasNext();){		
		XATStreamOperator operator = (XATStreamOperator)((XATNode)i.next()).getOperator();
		
		XATEndUserApplicationQueue endQ = new XATEndUserApplicationQueueImp(operator);
		//add the endQ to the statistics gatherer
		//edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsGatherer.getStatisticsGather().addNewQueryObject(endQ);
		//and set the output queue to be an XATEndUserApplicationQueue
		operator.setOutputQueue(endQ);
		for(Iterator q = operator.getOutputQueues(); q.hasNext();){
			tempQ.add(q.next());
		}			
		
	}
	queues = new XATEndUserApplicationQueueImp[tempQ.size()];
	for (int i = 0; i < tempQ.size(); i++){
		queues[i] = (XATEndUserApplicationQueue)tempQ.get(i);
		queues[i].register(this);
	}	
}
/**
 * Returns the count of how many tuples were removed from the output queues thus far
 * @return long
 */
public long getCount() {
	return count;
}
/**
 * Returns the instance of the XATStreamApplication
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.executioncontroller.XATStreamApplication
 */
public static XATStreamApplication getXATStreamApplication() {
	if(app == null){
		app = new XATStreamApplication();	
	}
	return app;
}
/**
 * Reset the application
 */
public void reset() {
	app = null;
	
	}
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject("XATStreamApplication",this.getClass().getName(),true);
	}
}
