package edu.wpi.cs.dsrg.xmldb.xat.component.gui;

/**
 * Class that provides access to the data model of the nodes in the graph.
 * Creation date: (5/11/2002 2:36:00 PM)
 * @author: Michal Andrzej Dobosz
 */

import java.util.*;
import java.util.LinkedList;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
 
public class NodeContainer {
	private LinkedList nodes = null;
	
	/**
	 * NodeContainer constructor comment.
	 */
	public NodeContainer() {
		nodes = new LinkedList();
	}

	/**
	 * Adds a node to the list of nodes.
	 */
	public void addNode(XATOperator xatNode, String dotName) {
		nodes.add(new GraphNode(xatNode, dotName));
	}

	/**
	 * Finds the XATOperator to its corresponding DOT node name.
	 */
	public XATOperator findNode(String dotName) {
		GraphNode gn = null;

		for(Iterator i = nodes.iterator(); i.hasNext();) {
			gn = (GraphNode)i.next();
    		if(dotName.equals(gn.dot)) {
				return gn.xat;
    		}
		}

		return null;
	}

	/**
	 * Finds a particular node and returns its output data model.
	 */
	public XATTable getOutput(String dotName) {
		XATTable xattable = null;
		
		xattable = (XATTable)(findNode(dotName)).getOutputDataSource();
		if(xattable == null) {
			System.err.println("No output data model for this node.");
			XATTable newTable = new XATTableImp();
			return newTable;
		}
		else {
			return xattable;
		}
	}

	/**
	 * Finds a particular node and returns its input data model.
	 */
	public XATTable getInput(String dotName) {
		XATTable xattable = null;
		
		try {
			xattable = (XATTable)(findNode(dotName)).getPrimaryDataSource();
		} catch(Exception e) {
			System.err.println("No input data model for this node.");
			XATTable newTable = new XATTableImp();
			return(newTable);
		}

		if(xattable == null) {
			XATTable newTable = new XATTableImp();
			return newTable;
		} else {
			return xattable;
		}
	}
}
