package edu.wpi.cs.dsrg.xmldb.xat.component.gui;

import javax.swing.JSplitPane;
/**
 * Pops up a Frame with two JTables when a user clicks on a node.
 * Creation date: (5/12/2002 1:13:44 PM)
 * @author: Michal Andrzej Dobosz
 */

import att.grappa.*;
import att.grappa.Subgraph;
import java.util.Vector;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.BorderFactory; 
import javax.swing.JPanel;
import javax.swing.JFrame;
import java.awt.Window;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;

public class NodeListener implements GrappaConstants, GrappaListener {
	protected GrappaAdapter ga = null;
	protected NodeContainer nodes = null;
	private String nodeName = null;
	private JTable outputTable = null;
	private JTable inputTable = null;
	protected Vector frameList = null;
	
    /**
     * NodeListener constructor comment.
     */
    public NodeListener(NodeContainer nodes_in) {
		nodes = nodes_in;
        ga = new GrappaAdapter();
        frameList = new Vector();
    }
    /**
     * This method closes all of the frames that have been created from the user clicking on nodes.
     */
    public void closeFrames() {
	    int size = frameList.size();
		for(int i = 0; i < size; i++) {
			((JFrame)(frameList.get(i))).dispose();
		}
    }
    /**
     * The method called when a single mouse click occurs on a displayed subgraph.
     *
     * @param subg displayed subgraph where action occurred
     * @param elem subgraph element in which action occurred
     * @param pt the point where the action occurred (graph coordinates)
     * @param modifiers mouse modifiers in effect
     * @param panel specific panel where the action occurred
     */
    public void grappaClicked(
        Subgraph subg,
        Element elem,
        GrappaPoint pt,
        int modifiers,
        int clickCount,
        GrappaPanel panel) {
			ga.grappaClicked(subg, elem, pt, modifiers, clickCount, panel);
			if(ga.getClick()) {
				// get the name of the node that was clicked
				nodeName = ga.getName();

				// get the output table of that node
				TableCreator tc1 = new TableCreator(nodes.getOutput(nodeName));
				tc1.debugJTable();
				outputTable = tc1.getTable();
				outputTable.setPreferredScrollableViewportSize(new Dimension(250, 100));

				// get the input table of that node
				TableCreator tc2 = new TableCreator(nodes.getInput(nodeName));
				tc2.debugJTable();
				inputTable = tc2.getTable();
				inputTable.setPreferredScrollableViewportSize(new Dimension(250, 100));

				// create the scroll panes for the tables
				JScrollPane outputJSP = new JScrollPane(outputTable);
				JScrollPane inputJSP = new JScrollPane(inputTable);

				// create panels with borders for the scroll panes
				JPanel outputJP = new JPanel();
				JPanel inputJP = new JPanel();
				outputJP.setBorder(BorderFactory.createCompoundBorder(
                        BorderFactory.createTitledBorder("Output"),
                        BorderFactory.createEmptyBorder(5,5,5,5)));
				inputJP.setBorder(BorderFactory.createCompoundBorder(
                        BorderFactory.createTitledBorder("Input"),
                        BorderFactory.createEmptyBorder(5,5,5,5)));
				outputJP.setLayout(new BorderLayout());
				inputJP.setLayout(new BorderLayout());
				outputJP.add(outputJSP, BorderLayout.CENTER);
				inputJP.add(inputJSP, BorderLayout.CENTER);

				// create a new frame
				JFrame frame = new JFrame("Data Model - " + nodeName);
				frameList.addElement(frame);
				frame.setDefaultCloseOperation(frame.DISPOSE_ON_CLOSE);
            	frame.setLocation(100, 100);

            	// add JSplit Panel.
            	JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT, outputJP, inputJP);
            	splitPane.setDividerLocation(0.5);
            	splitPane.setDividerLocation(150);
            	splitPane.setOneTouchExpandable(true);
            	
            	// add the two JTables to the frame
            	(frame.getContentPane()).add(splitPane, BorderLayout.CENTER);

            	// reveal the constructed frame
            	frame.pack();
        		frame.setVisible(true);
			}
        }
    /**
     * The method called when a mouse drag occurs on a displayed subgraph.
     *
     * @param subg displayed subgraph where action occurred
     * @param currentPt the current drag point
     * @param currentModifiers the current drag mouse modifiers
     * @param pressedElem subgraph element in which the most recent mouse press occurred
     * @param pressedPt the point where the most recent mouse press occurred (graph coordinates)
     * @param pressedModifiers mouse modifiers in effect when the most recent mouse press occurred
     * @param outline enclosing box specification from the previous drag position (for XOR reset purposes)
     * @param panel specific panel where the action occurred
     */
    public void grappaDragged(
        Subgraph subg,
        GrappaPoint currentPt,
        int currentModifiers,
        Element pressedElem,
        GrappaPoint pressedPt,
        int pressedModifiers,
        GrappaBox outline,
        GrappaPanel panel) {
			ga.grappaDragged(subg, currentPt, currentModifiers, pressedElem, pressedPt, pressedModifiers, outline, panel);
        }
    /**
     * The method called when a mouse press occurs on a displayed subgraph.
     *
     * @param subg displayed subgraph where action occurred
     * @param elem subgraph element in which action occurred
     * @param pt the point where the action occurred (graph coordinates)
     * @param modifiers mouse modifiers in effect
     * @param panel specific panel where the action occurred
     */
    public void grappaPressed(
        Subgraph subg,
        Element elem,
        GrappaPoint pt,
        int modifiers,
        GrappaPanel panel) {
			ga.grappaPressed(subg, elem, pt, modifiers, panel);
        }
    /**
     * The method called when a mouse release occurs on a displayed subgraph.
     *
     * @param subg displayed subgraph where action occurred
     * @param elem subgraph element in which action occurred
     * @param pt the point where the action occurred (graph coordinates)
     * @param modifiers mouse modifiers in effect
     * @param pressedElem subgraph element in which the most recent mouse press occurred
     * @param pressedPt the point where the most recent mouse press occurred (graph coordinates)
     * @param pressedModifiers mouse modifiers in effect when the most recent mouse press occurred
     * @param outline enclosing box specification from the previous drag position (for XOR reset purposes)
     * @param panel specific panel where the action occurred
     */
    public void grappaReleased(
        Subgraph subg,
        Element elem,
        GrappaPoint pt,
        int modifiers,
        Element pressedElem,
        GrappaPoint pressedPt,
        int pressedModifiers,
        GrappaBox outline,
        GrappaPanel panel) {
			ga.grappaReleased(subg, elem, pt, modifiers, pressedElem, pressedPt, pressedModifiers, outline, panel);
        }
    /**
     * The method called when a element tooltip is needed.
     *
     * @param subg displayed subgraph where action occurred
     * @param elem subgraph element in which action occurred
     * @param pt the point where the action occurred (graph coordinates)
     * @param modifiers mouse modifiers in effect
     * @param panel specific panel where the action occurred
     *
     * @return the tip to be displayed or null
     */
    public String grappaTip(
        Subgraph subg,
        Element elem,
        GrappaPoint pt,
        int modifiers,
        GrappaPanel panel) {
			return ga.grappaTip(subg, elem, pt, modifiers, panel);
        }
}
