package edu.wpi.cs.dsrg.xmldb.xat.component.gui;

import java.awt.*;
import java.awt.Dimension;
import java.awt.geom.Dimension2D;
import java.util.*;

import javax.swing.*;
import att.grappa.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.utils.BindingTable;
/**
 * Insert the type's description here.
 * @author: Brad Pielech
 */
public abstract class ProcessStep {
	protected XATTree userTree;
	protected edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree viewTree;
	protected NodeListener userNodeListener;
	protected NodeListener viewNodeListener;
	protected NodeContainer userNodeContainer;
	protected NodeContainer viewNodeContainer;
	public att.grappa.GrappaPanel userGrappaPanel;
	protected att.grappa.GrappaPanel viewGrappaPanel;
	protected boolean userDone;
	protected boolean viewDone;
	protected edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable table;
	protected java.lang.String tabName;
	protected java.lang.String tabTip;
	protected javax.swing.JComponent contentPane;
	protected final static int USER_QUERY = 0;
	protected final static int VIEW_QUERY = 1;
	protected BindingTable userBindingTable;
	protected BindingTable viewBindingTable;
	public boolean DEBUG=false;
/**
 * ProcessStep constructor comment.
 */
protected ProcessStep() {
	this(null);
}
/**
 * ProcessStep constructor comment.
 */
public ProcessStep(XATTree userXAT) {
	userTree = userXAT;
	viewTree = null;
	userGrappaPanel = null;
	viewGrappaPanel = null;
	userNodeListener = null;
	viewNodeListener = null;
	userNodeContainer = null;
	viewNodeContainer = null;
	viewDone = false;
	userDone = false;
	table = null;
	
}
/**
 * ProcessStep constructor comment.
 */
public ProcessStep(XATTree userXAT, XATTree viewXAT) {
	userTree = userXAT;
	viewTree = viewXAT;
	userGrappaPanel = null;
	viewGrappaPanel = null;
	userNodeListener = null;
	viewNodeListener = null;
	userNodeContainer = null;
	viewNodeContainer = null;
	viewDone = false;
	userDone = false;
	table = null;
	
}
/**
 * remove all of the frames created from this tree
 * try to close the userNodeListener, if it hasnt been initialized, there
 *  will be a NullPointerException, in which case, the ViewListener wouldnt have
 *  been initialized either
 *  If that is successful, then try to close the ViewNodeListener
 * 
 */
public void destroy() {
 
	try {
		getUserNodeListener().closeFrames();
		getViewNodeListener().closeFrames();
	} catch (NullPointerException e) {
		return ;	
	}
}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 * @param tree edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
protected abstract JPanel execute(int queryType);
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 */
public JPanel executeUser(){
	if(!isUserDone()){
		
		setUserDone(false);
		return execute(USER_QUERY);
	}
	else 
	{
		return getUserGrappaPanel();
	}
}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 */
public JPanel executeView(){
	return getViewGrappaPanel();	

}
/**
 * Insert the method's description here.
 * 
 * @return javax.swing.JComponent
 */
public javax.swing.JComponent getContentPane() {
	return contentPane;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable
 */
public edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable getTable() {
	return table;
}
/**
 * Insert the method's description here.
 * 
 * @return java.lang.String
 */
public java.lang.String getTabName() {
	return tabName;
}
/**
 * Insert the method's description here.
 * 
 * @return java.lang.String
 */
public java.lang.String getTabTip() {
	return tabTip;
}
/**
 * Insert the method's description here.
 * 
 * @return java.util.Hashtable
 */
public BindingTable getUserBindingTable() {
	return userBindingTable;
}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 */
public att.grappa.GrappaPanel getUserGrappaPanel() {
	return userGrappaPanel;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeContainer
 */
public NodeContainer getUserNodeContainer() {
	return userNodeContainer;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeListener
 */
public NodeListener getUserNodeListener() {
	return userNodeListener;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree getUserTree() {
	return userTree;
}
/**
 * Insert the method's description here.
 * 
 * @return java.util.Hashtable
 */
public BindingTable getViewBindingTable() {
	return viewBindingTable;
}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 */
public att.grappa.GrappaPanel getViewGrappaPanel() {
	return viewGrappaPanel;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeContainer
 */
public NodeContainer getViewNodeContainer() {
	return viewNodeContainer;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeListener
 */
public NodeListener getViewNodeListener() {
	return viewNodeListener;
}
/**
 * Insert the method's description here.
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree getViewTree() {
	return viewTree;
}
/**
 * Insert the method's description here.
 * 
 * @return boolean
 */
public boolean isUserDone() {
	return userDone;
}
/**
 * Insert the method's description here.
 * 
 * @return boolean
 */
public boolean isViewDone() {
	return viewDone;
}
/**
 * Insert the method's description here.
 * 
 * @param newContentPane javax.swing.JComponent
 */
public void setContentPane(javax.swing.JComponent newContentPane) {
	contentPane = newContentPane;
}
/**
 * Insert the method's description here.
 * 
 * @param newTable edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable
 */
public void setTable(edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATTable newTable) {
	table = newTable;
}
/**
 * Insert the method's description here.
 * 
 * @param newTabName java.lang.String
 */
public void setTabName(java.lang.String newTabName) {
	tabName = newTabName;
}
/**
 * Insert the method's description here.
 * 
 * @param newTabTip java.lang.String
 */
public void setTabTip(java.lang.String newTabTip) {
	tabTip = newTabTip;
}
/**
 * Insert the method's description here.
 * 
 * @param newBindingTable java.util.Hashtable
 */
public void setUserBindingTable(BindingTable newBindingTable) {
	userBindingTable = newBindingTable;
}
/**
 * Insert the method's description here.
 * 
 * @param newUserDone boolean
 */
public void setUserDone(boolean newUserDone) {
	userDone = newUserDone;
}
/**
 * Insert the method's description here.
 * 
 * @param newUserGrappaPanel att.grappa.GrappaPanel
 */
public void setUserGrappaPanel(att.grappa.GrappaPanel newUserGrappaPanel) {
	userGrappaPanel = newUserGrappaPanel;
}
/**
 * Insert the method's description here.
 * 
 * @param newUserNodeContainer edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeContainer
 */
public void setUserNodeContainer(NodeContainer newUserNodeContainer) {
	userNodeContainer = newUserNodeContainer;
}
/**
 * Insert the method's description here.
 * 
 * @param newUserNodeListener edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeListener
 */
public void setUserNodeListener(NodeListener newUserNodeListener) {
	userNodeListener = newUserNodeListener;
}
/**
 * Insert the method's description here.
 * 
 * @param newUserTree edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public void setUserTree(edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree newUserTree) {
	userTree = newUserTree;
}
/**
 * Insert the method's description here.
 * 
 * @param newViewBindingTable java.util.Hashtable
 */
public void setViewBindingTable(BindingTable newViewBindingTable) {
	viewBindingTable = newViewBindingTable;
}
/**
 * Insert the method's description here.
 * 
 * @param newViewDone boolean
 */
public void setViewDone(boolean newViewDone) {
	viewDone = newViewDone;
}
/**
 * Insert the method's description here.
 * 
 * @param newViewGrappaPanel att.grappa.GrappaPanel
 */
public void setViewGrappaPanel(att.grappa.GrappaPanel newViewGrappaPanel) {
	viewGrappaPanel = newViewGrappaPanel;
}
/**
 * Insert the method's description here.
 * 
 * @param newViewNodeContainer edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeContainer
 */
public void setViewNodeContainer(NodeContainer newViewNodeContainer) {
	viewNodeContainer = newViewNodeContainer;
}
/**
 * Insert the method's description here.
 * 
 * @param newViewNodeListener edu.wpi.cs.dsrg.xmldb.xat.component.gui.NodeListener
 */
public void setViewNodeListener(NodeListener newViewNodeListener) {
	viewNodeListener = newViewNodeListener;
}
/**
 * Insert the method's description here.
 * 
 * @param newViewTree edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public void setViewTree(edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree newViewTree) {
	viewTree = newViewTree;
}
/**
 * Updates the Scroll Pane by setting the viewport view and revalidating
 * 
 * @param grappaPanel att.grappa.GrappaPanel
 */
public void updateContentPane(JPanel grappaPanel, int mappingWay, int flagNum, String userQ, int paraSet) {

    // get the current content pane.
    JComponent theContent = this.getContentPane();
    theContent.setDoubleBuffered(true);

    JScrollPane js = ((JScrollPane) this.getContentPane());

    JPanel jMergePanel = new JPanel();
    jMergePanel.setLayout(new BorderLayout());
    
	JLabel jMergeLabel = new JLabel("");
	int flagset = flagNum;
	
	if(flagset == 0)
	 //if(this instanceof TreeGenerationStep)
	 {
	 JFrame fa = new JFrame("XQuery");
	 Container cont = fa.getContentPane();
	 cont.setLayout (new FlowLayout());
          cont.add(new JTextArea(userQ));
	 fa.setSize(800,200);
	 
	 fa.show();
	 
 
	jMergeLabel = new JLabel("\n  XQuery Display ");
	 }
	
	if(flagset == 1)
	jMergeLabel = new JLabel("\n  XAT Plan Display");
	
	if(flagset == 4)
	   jMergeLabel = new JLabel("\n  Executing");
	
	
	if(mappingWay == 0 && flagset ==3)
	jMergeLabel = new JLabel("\n  Physical Plan based on Logical Max Pushdown");
	else 
	if(mappingWay == 1 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical All Binding");
	else 
	if(mappingWay == 2 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical Main Binding");
	else
	if(mappingWay == 3 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical Source");
	else
	if(mappingWay == 4 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical Nav with Selection"+"    Parameter: " + paraSet);
	else
	if(mappingWay == 5 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical NavUnnest with Selection"+"    Parameter: " + paraSet);
	else
	if(mappingWay == 6 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical Navigation"+"    Parameter: " + paraSet);
	else
	if(mappingWay == 7 && flagset == 3)
	jMergeLabel = new JLabel("\n  Physical Plan Logical Select Partially"+"    Parameter: " + paraSet);
	
	
	
    
    if(mappingWay == 0 && flagset ==2)
    jMergeLabel = new JLabel("\n  Logical Mapping: Max Pushdown");
    else 
	if(mappingWay == 1 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: All Binding");
	else 
	if(mappingWay == 2 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: Main Binding");
	else
	if(mappingWay == 3 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: Source");
	else
	if(mappingWay == 4 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: Nav with Selection"+"    Parameter: " + paraSet);
	else
	if(mappingWay == 5 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: NavUnnest with Selection"+"    Parameter: " + paraSet);
	else
	if(mappingWay == 6 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: Navigation"+"    Parameter: " + paraSet);
	else
	if(mappingWay == 7 && flagset == 2)
	jMergeLabel = new JLabel("\n  Logical Mapping: Select Partially"+"    Parameter: " + paraSet);

    JPanel jmPanel = new JPanel();
    grappaPanel.repaint();
    jmPanel.add(grappaPanel);
    jMergePanel.add(jMergeLabel, BorderLayout.NORTH);
    jMergePanel.add(jmPanel, BorderLayout.CENTER);

    js.setViewportView(jMergePanel);
    theContent.revalidate();

}
/**
 * Updates the Scroll Pane by setting the viewport view and revalidating
 * 
 * @param grappaPanel att.grappa.GrappaPanel
 */
public void updateContentPane(JPanel grappaPanel, JPanel grappaPanel2) {

		// get the current content pane.
		JComponent theContent = this.getContentPane();
		theContent.setDoubleBuffered(true);

		// depends on its type call the different setup.
		if(theContent instanceof JSplitPane){
			
		  JScrollPane jp1 = new JScrollPane();
		  JPanel jUserPanel = new JPanel();
		  jUserPanel.setLayout(new BorderLayout());
		  JPanel t1Panel = new JPanel();
		  JLabel jUserLabel = new JLabel("User Query");

		  t1Panel.add(grappaPanel);
		  jUserPanel.add(jUserLabel, BorderLayout.NORTH);
		  jUserPanel.add(t1Panel, BorderLayout.CENTER);
		  
	      jp1.setViewportView(jUserPanel);

	      JScrollPane jp2 = new JScrollPane();
	      JPanel jViewPanel = new JPanel();
	      jViewPanel.setLayout(new BorderLayout());
	      JPanel t2Panel = new JPanel();
		  JLabel jViewLabel = new JLabel("View Query");

		  t2Panel.add(grappaPanel2);
		  jViewPanel.add(jViewLabel, BorderLayout.NORTH);
		  jViewPanel.add(t2Panel, BorderLayout.CENTER);
		  
	      jp2.setViewportView(jViewPanel);
	      
	      JSplitPane jsp =  (JSplitPane) this.getContentPane();
		  jsp.setLeftComponent(jp1);
		  jsp.setRightComponent(jp2);
	      jsp.setDividerLocation(0.5);
	      jsp.setDividerLocation(250);
	      theContent.revalidate();
	      
		}else{
	      	
		  JScrollPane js = ((JScrollPane)this.getContentPane());

		  JPanel jMergePanel = new JPanel();
		  jMergePanel.setLayout(new BorderLayout());
		  JLabel jMergeLabel = new JLabel("Merged Query");

		  JPanel jmPanel = new JPanel();
		  jmPanel.add(grappaPanel);
		  jMergePanel.add(jMergeLabel, BorderLayout.NORTH);
		  jMergePanel.add(jmPanel, BorderLayout.CENTER);
		  
		  js.setViewportView(jMergePanel);
		  theContent.revalidate();
		}
 
		
	}
}
