package edu.wpi.cs.dsrg.xmldb.xat.component.gui;

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import java.util.Hashtable;
import javax.swing.*;

import edu.wpi.cs.dsrg.xmldb.xat.component.xatdaggeneration.DagParserTraverse;
import edu.wpi.cs.dsrg.xmldb.xat.component.xatdaggeneration.DagQueryDecorrelation;
import edu.wpi.cs.dsrg.xmldb.xat.component.xattreegeneration.*;
import xacute.quilt.*;
import att.grappa.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.testfiles.TestSXE;
/**
 * Insert the type's description here.
 * @author: Brad Pielech
 */
public class TreeGenerationStep extends ProcessStep {
	protected java.lang.String userQuery;
	protected java.lang.String viewQuery;
	protected java.lang.String queryToUse;
/**
 * GenerationStep constructor comment.
 * @param userXAT edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 * @param viewXAT edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public TreeGenerationStep(XATTree userXAT, XATTree viewXAT, String userQ, String viewQ) {
	super(userXAT, viewXAT);
	userQuery = userQ;
	viewQuery = viewQ;
	setTabName("Generate XAT");
	setTabTip("Generates the algebra tree");
	setContentPane(new JSplitPane(JSplitPane.VERTICAL_SPLIT));
	JSplitPane thisPane = (JSplitPane)getContentPane();
	thisPane.setOneTouchExpandable(true);
	thisPane.setDoubleBuffered(true);
}
/**
 * GenerationStep constructor comment.
 * @param userXAT edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public TreeGenerationStep(XATTree userXAT, String userQ) {
	super(userXAT);
	userQuery = userQ;
	setTabName("Generate XAT");
	setTabTip("Generates the algebra tree");
	setContentPane(new JScrollPane());
}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 * @param tree edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
protected JPanel execute(int queryType) {

    QuiltParser parser = new QuiltParser();
    QuiltQuery qq = null;
    
    // create a dummy tree Used to handle only one query is available.
    XATTree tree = new XATTreeImp();
	
    NodeContainer nc;
    NodeListener nl;
    GrappaPanel gp;
    DagParserTraverse pt1 = new DagParserTraverse();

    /** set which operator factory to use */
    /** this is to generate persistent operators */
    System.setProperty(
        edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.OperatorFactory.factoryID,
        "edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.PersistentOperatorFactory");
    /** this generates stream operators (not yet implemented) */
    //System.setProperty(OperatorFactory.factoryID, "edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.StreamOperatorFactory");
		
    // a flag to decide wether we will skip the Tree generation of this current query (e.g., user/view).
    boolean skip = false; //default is not.

    // if the getQueryToUse() is empty, or null, skip the parsing and generate an empty tree.
    String queryToUse = getQueryToUse();
    if (queryToUse == null) {
        skip = true;
    } else if (queryToUse.length() == 0) {
        skip = true;
    }

    try {

        if (!skip) {
            qq = parser.parseQuery(getQueryToUse());
            pt1.visit(qq);
            tree = pt1.getTree();
        }

		DagQueryDecorrelation qd = new DagQueryDecorrelation((XATDag)tree);
		tree = qd.getDecorTree();
		// compute the binding table after decorrelation
		BindingTableComputer btc = new BindingTableComputer();
		BindingTable bt = btc.computeBindingTable( tree );
		tree.setBindingTable(bt);		
		tree = TestSXE.treeRewrite(tree);
		tree = TestSXE.treeSchema(tree);



        // create a new GrappaPanel from this XAT
		System.out.println("Getting the XAT Tree here:\n"+tree+"\n");
		//System.out.println(".............");
        
        GrappaCreator gc = new GrappaCreator(tree, true);
        gp = gc.getGrappaPanel();
        nc = gc.getNodeContainer();
        nl = gc.getNodeListener();

        if (queryType == this.USER_QUERY) {
            if (!skip)
                setUserBindingTable(pt1.getTree().getBindingTable());
            else { // create an empty hashtable.
                setUserBindingTable(new BindingTable());
            }
            setUserTree(tree);
            setUserGrappaPanel(gp);
            setUserNodeContainer(nc);
            setUserNodeListener(nl);
        } else {
            if (!skip)
                setViewBindingTable(pt1.getTree().getBindingTable());
            else { // create an empty hashtable.
                setViewBindingTable(new BindingTable());
            }
            setViewTree(tree);
            setViewGrappaPanel(gp);
            setViewNodeContainer(nc);
            setViewNodeListener(nl);
        }
        return gp;
    } catch (Exception e) {
        e.printStackTrace(System.out);
    }

    return null;

}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 */
public JPanel executeUser() {
	if(!isUserDone()){	
		setUserDone(true);
		setQueryToUse(userQuery);
		return execute(USER_QUERY);	
	}
	else 
	{
	return getUserGrappaPanel();
	}
}
/**
 * Insert the method's description here.
 * 
 * @return att.grappa.GrappaPanel
 */
public JPanel executeView() {
	if(!isViewDone()){
		setViewDone(true);
		setQueryToUse(viewQuery);
		return execute(VIEW_QUERY);
	} else 
	{
		return getViewGrappaPanel();
		} 
}
/**
 * Insert the method's description here.
 * 
 * @return java.lang.String
 */
protected java.lang.String getQueryToUse() {
	return queryToUse;
}
/**
 * Insert the method's description here.
 * 
 * @param newQueryToUse java.lang.String
 */
protected void setQueryToUse(java.lang.String newQueryToUse) {
	queryToUse = newQueryToUse;
}
}
