package edu.wpi.cs.dsrg.xmldb.xat.component.gui;

/**
 * XAT to DOT parser.
 * Creation date: (2/26/2002 7:45:04 PM)
 * @author: Michal Andrzej Dobosz
 */

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.updateoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dompattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

import java.lang.reflect.Array;

public class XATParser extends InheritableVisitor {

    protected XATTree tree = null;
    protected String dottree = null;
    protected NodeContainer nodes = null;

    protected int numnode;
    public XATParser(XATTree xattree) {
        // copy in the XAT tree for our methods to access
        tree = xattree;

        // set up the first part of the DOT tree
        dottree = "digraph XAT {\n\tgraph [ratio = \"compress\", size = \"8, 8\", ranksep = \"0.5\"];\n\tnode [shape = \"record\", fontsize = \"12\"];\n\tedge [dir = \"back\", style = \"bold\"];\n";

		// initialize the node container
		nodes = new NodeContainer();

        // initialize the number of each type of node
        numnode = 0;
    }
    /**
     * Converts characters that DOT considers reserved into harmless ones
     */
    protected String deReserve(String dotString) {
	    int i;

		String finalString = new String("");

		if(dotString == null)
	    	return finalString;
	    
		for(i=0; i<dotString.length();i++){
			char dts = dotString.charAt(i);
			switch (dts){
				case '<': finalString += "\\<";   break;
				case '>': finalString += "\\>";   break;
				case '"': finalString += "\\\"";  break;
				case '\n': finalString += "\\n";  break; // new line.
				default : finalString += dts;    break;
			} 
		}
		return finalString;
    }
    public Object dispatch() {
        // System.out.println(indent() + "NULL");
        return null;
    }
    public Object dispatch(Object obj) {
        System.out.println("Dispatch!! " + obj.getClass().toString());
        return null;
    }
    /**
     * Get the DOT tree
     */
    public String getDOT() {
        return dottree;
    }
/**
 * Return a unique ID for Grappa node.
 * Creation date: (6/1/2003 11:06:44 PM)
 * @return java.lang.String
 */
protected String getNextID(Object obj) {
	numnode ++;
	return "node" + numnode;
}
    /**
     * Get the nodes
     */
    public NodeContainer getNodes() {
        return nodes;
    }
    /**
     * Get the XML algebra tree
     */
    public XATTree getTree() {
        return tree;
    }
    /**
     * Converts the XAT into DOT format and prints it out onto the screen
     */
    public void printDOT() {
        printNode(tree.getRoot(), null);
        System.out.println("\n");
         
        // finish off the DOT format
        dottree += "}\n";

        // print out the XAT tree in DOT format
        System.out.println(dottree);
    }
    /**
     * Prints a node in the XAT
     * If printNode is sent the root node of an XAT as input,
     * the whole tree is printed out.
     */
    protected void printNode(XATNode node, String parent) {
	    if(node == null){
		    // do nothing.
		    return;
	    }
        XATNode[] children = node.getChildren();
        String child = (String)visit(node.getOperator());
        if(parent == null) {
	        parent = child;
        }
        else if(child != null) {
			dottree += "\t" + parent + " -> " + child + ";\n";
        }
        for(int i = 0; i < Array.getLength(children); i++) {
	        if(node.getOperator() instanceof LeftOuterJoin && i == 0){
		        printNode(children[i], child, " [label=\"Left\"] ");
	        }else
	            printNode(children[i], child);
        }
    }
    /**
     * Prints a node in the XAT with different style.
     * If printNode is sent the root node of an XAT as input,
     * the whole tree is printed out.
     */
    protected void printNode(XATNode node, String parent, String style) {
	    if(node == null){
		    // do nothing.
		    return;
	    }
        XATNode[] children = node.getChildren();
        String child = (String)visit(node.getOperator());
        if(parent == null) {
	        parent = child;
        }
        else if(child != null) {
			dottree += "\t" + parent + " -> " + child + style + ";\n";
        }
        for(int i = 0; i < Array.getLength(children); i++) {
            printNode(children[i], child);
        }
    }
    /**
     * Set the XML algebra tree
     */
    public void setTree(XATTree xattree) {
        tree = xattree;
    }
    /**
     * Output of the select node in an XAT
     */
    public Object visit(Expose expVisit) {
	    String childID = getNextID(expVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(expVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Expose:"+expVisit.getNode().hashCode()+"|"
            + "{expose column|" + expVisit.getColumnName() + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the for node in an XAT
     */
    public Object visit(FOR forVisit) {
	    String childID = getNextID(forVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(forVisit, childID);

        String[] binding = forVisit.getColumnNames();
        
        dottree += "\t" + child
            + " [label = \"{For:"+forVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(forVisit.getOutputColumnName()) + "}|"
            + "{binding|" + deReserve(binding[0]) + "}"
            + "}\",style=filled,color=pink];\n";

	    printNode(forVisit.getXtreFunctions().getRoot(), child, " [style=\"dashed\" , arrowhead=\"odot\", label=\"Inner\\nTree\"] ");

        return child;
    }
    /**
     * Output of the function node in an XAT
     */
    public Object visit(Function funVisit) {
	    String childID = getNextID(funVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(funVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Function:"+funVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(funVisit.getOutputColumnName()) + "}|"
            + "{name|" + deReserve(funVisit.getName()) + "}|";

        
	    Object[] params = funVisit.getParams();

		String navigation = new String("");
	    for(int i = 0; i < params.length; i++) {
			if(params[i] instanceof NavigationStepImp) {
				NavigationStep navStep = (NavigationStep)params[i];
			    navigation += navStep.getStepValue() + "/";
		    }
		    else {
				navigation += params[i].toString() + "/";
	        }
    	}

	    if(navigation != ""){
		    if(navigation.endsWith("/")){navigation = navigation.substring(0,navigation.length()-1);}
		    dottree += "{navigation|" + deReserve(navigation) + "}"
	    		+ "}\"];\n";
		}else{
			dottree += "];\n";
		}

	    return child;
    }
    /**
     * Output of the join node in an XAT
     */
    public Object visit(IfExpr ifVisit) {
	    String childID = getNextID(ifVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(ifVisit, childID);

        dottree += "\t" + child
            + " [label = \"{If:"+ifVisit.getNode().hashCode()+"|"
            + "{output column|" + ifVisit.getOutputColumnName() + "}|"
            + "{condition|" + deReserve(ifVisit.getIfExpr().toString()) + "}"
            + "}\"];\n";

	    printNode(ifVisit.getThenXAT().getRoot(), child, " [style=dashed , arrowhead=odot, label=\"Then\\nTree\"] ");
	    printNode(ifVisit.getElseXAT().getRoot(), child, " [style=dashed , arrowhead=odot, label=\"Else\\nTree\"] ");
            
        return child;
    }
	/**
     * Output of the intersect node in an XAT
     */
    public Object visit(Merge mergeVisit) {
	    String childID = getNextID(mergeVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(mergeVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Merge:"+mergeVisit.getNode().hashCode()+"|"
            + "}\"];\n";

        return child;
    }
/**
 * Output of the column name node in an XAT
 */
public Object visit(NameColumn colVisit) {
    String childID = getNextID(colVisit);
    String child = "\"" + childID + "\"";
    nodes.addNode(colVisit, childID);

    StringBuffer sb = new StringBuffer();

    if (colVisit.getBlockDesc() != null) {
        // then it's a block identifer.
        dottree += "\t"
            + child
            + " [label = \"{Block:"
            + colVisit.getNode().hashCode()
            + "|"
            + "{column name|"
            + colVisit.getOriginalColumnName()
            + "}|"
            + "{desc |"
            + colVisit.getBlockDesc()
            + "}"
            + "}\",style=filled,color=orange];\n";
    } else {

        dottree += "\t"
            + child
            + " [label = \"{Name Column:"
            + colVisit.getNode().hashCode()
            + "|"
            + "{original name|"
            + colVisit.getOriginalColumnName()
            + "}|"
            + "{new name|"
            + colVisit.getNewColumnName()
            + "}"
            + "}\",style=filled,color=grey];\n";
    }

    return child;
}
    /**
         * Output of the select node in an XAT
         */
    public Object visit(Or orVisit) {
	    String childID = getNextID(orVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(orVisit, childID);

        dottree += "\t" + child + " [label = \"OR:"+orVisit.getNode().hashCode()+"\", color=purple, style=filled];\n";

        return child;
    }
    /**
     * Output of the source node in an XAT
     */
    public Object visit(Source souVisit) {
	    String childID = getNextID(souVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(souVisit, childID);
        
        dottree += "\t" + child
            + " [label = \"{Source:"+souVisit.getNode().hashCode()+"|"
            + "{output column|" + souVisit.getOutputColumnName() + "}|"
            + "{data|" + souVisit.getName() + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the SQL node in an XAT
     */
    public Object visit(SQLStmt sqlVisit) {
	    String childID = getNextID(sqlVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(sqlVisit, childID);

        dottree += "\t" + child
            + " [label = \"{SQL:"+sqlVisit.getNode().hashCode()+"|"
            + "{SQL statement|" + deReserve(sqlVisit.getStmt()) + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the XQuery node in an XAT
     */
    public Object visit(XQueryStmt xquVisit) {
	    String childID = getNextID(xquVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(xquVisit, childID);

        dottree += "\t" + child
            + " [label = \"{XQuery Statement:"+xquVisit.getNode().hashCode()+"|"
            + deReserve(xquVisit.getStmt())
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the cartesian product node in an XAT
     */
    public Object visit(CartesianProduct carVisit) {
	    String childID = getNextID(carVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(carVisit, childID);

        dottree += "\t" + child
            + " [label = \"Cartesian Product:"+carVisit.getNode().hashCode()+"\"];\n";

        return child;
    }
    /**
     * Output of the distinct node in an XAT
     */
    public Object visit(Distinct disVisit) {
	    String childID = getNextID(disVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(disVisit, childID);

        dottree += "\t" + child
            + " [label = \"Distinct:"+disVisit.getNode().hashCode()+"\"];\n";

        return child;
    }
/**
 * Output of the group by node in an XAT
 */
public Object visit(GroupBy groVisit) {

    String childID = getNextID(groVisit);
    String child = "\"" + childID + "\"";
    nodes.addNode(groVisit, childID);

    String[] cols = groVisit.getColumnNames();

    dottree += "\t"
        + child
        + " [label = \"{Group By:"
        + groVisit.getNode().hashCode()
        + "|"
        + "{output column|"
        + deReserve(groVisit.getOutputColumnName())
        + "}|";

    dottree += "{grouping column(s)|";
    for (int i = 0; i < +cols.length; i++)
        dottree += (cols[i] + " ");

    dottree += "}}\",style=filled,color=cadetblue1]\n";

    try {
        printNode(
            groVisit.getXATTree().getRoot(),
            child,
            " [style=\"dashed\" , arrowhead=\"odot\", label=\"Inner\\nTree\"] ");
    } catch (Exception e) {
    }

    return child;
}
	/**
     * Output of the intersect node in an XAT
     */
    public Object visit(Intersection intVisit) {
	    String childID = getNextID(intVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(intVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Intersect:"+intVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(intVisit.getOutputColumnName()) + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the join node in an XAT
     */
    public Object visit(Join joiVisit) {
	    String childID = getNextID(joiVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(joiVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Join:"+joiVisit.getNode().hashCode()+"|"
            + "{output column|" + joiVisit.getOutputColumnName() + "}|"
            + "{condition|" + deReserve((joiVisit.getExpression()).toString()) + "}"
            + "}\"];\n";

        return child;
    }
	/**
	 * Output of the join node in an XAT
	 */
	public Object visit(LeftOuterJoin joiVisit) {
		String childID = getNextID(joiVisit);
		String child = "\"" + childID + "\"";
		nodes.addNode(joiVisit, childID);

		dottree += "\t" + child
			+ " [label = \"{Left Outer Join:"+joiVisit.getNode().hashCode()+"|"
			+ "{output column|" + joiVisit.getOutputColumnName() + "}|"
			//+ "{condition|" + deReserve((joiVisit.getExpression()).toString()) + "}"
			+ "}\",style=filled,color=red];\n";

		return child;
	}
	/**
	 * Output of the join node in an XAT
	 */
	public Object visit(NatureJoin joiVisit) {
		String childID = getNextID(joiVisit);
		String child = "\"" + childID + "\"";
		nodes.addNode(joiVisit, childID);

		dottree += "\t" + child
			+ " [label = \"{Nature Join:"+joiVisit.getNode().hashCode()+"|"
			+ "{output column|" + joiVisit.getOutputColumnName() + "}|"
			//+ "{condition|" + deReserve((joiVisit.getExpression()).toString()) + "}"
			+ "}\",style=filled,color=blue];\n";

		return child;
	}
    /**
     * Output of the order node in an XAT
     */
    public Object visit(OrderBy ordVisit) {
	    String childID = getNextID(ordVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(ordVisit, childID);

        String[] cols = ordVisit.getColumnNames();

        dottree += "\t" + child
            + " [label = \"{Order:"+ordVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(ordVisit.getOutputColumnName()) + "}|"
            + "{column names|";

        int length = cols.length - 1;
	    for(int i = 0; i < length; i++) {
	      dottree += deReserve(cols[i]) + ", ";
	    }
        dottree += deReserve(cols[length]) + "}|";
	     

        dottree += "}\"];\n";

        return child;
    }
    /**
     * Output of the select node in an XAT
     */
    public Object visit(Project projVisit) {
	    String childID = getNextID(projVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(projVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Project:"+projVisit.getNode().hashCode()+"|"
            + "{column names|" ;
            
        String [] list = projVisit.getColumnNames();
		for(int i=0;i<list.length; i++)
			dottree += list[i] + " ";
        
        dottree += "}" + "}\"];\n";

        return child;
    }
    /**
     * Output of the select node in an XAT
     */
    public Object visit(Select selVisit) {
	    String childID = getNextID(selVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(selVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Select:"+selVisit.getNode().hashCode()+"|"
            + "{output column|" + selVisit.getOutputColumnName() + "}|"
            + "{expression|" + deReserve((selVisit.getExpression()).toString()) + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the union node in an XAT
     */
    public Object visit(Union uniVisit) {
	    String childID = getNextID(uniVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(uniVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Union:"+uniVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(uniVisit.getOutputColumnName()) + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the delete node in an XAT
     */
    public Object visit(Delete delVisit) {
	    String childID = getNextID(delVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(delVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Delete:"+delVisit.getNode().hashCode()+"|"
            + "{context|" + deReserve(delVisit.getContext()) + "}|"
            + "{variable|" + deReserve(delVisit.getVariable()) + "}"
            + "}\"];\n";
        
        return child;
    }
    /**
     * Output of the insert node in an XAT
     */
    public Object visit(Insert insVisit) {
	    String childID = getNextID(insVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(insVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Insert:"+insVisit.getNode().hashCode()+"|"
            + "{context|" + deReserve(insVisit.getContext()) + "}";
        
		// if there is a variable in the insert node	
	    if(insVisit.getVariable() != null) {
		  dottree += "|{variable|" + deReserve(insVisit.getVariable()) + "}";
	    }

	    // if there appears an update function in the insert node     
		if(insVisit.getUpdateFunction() != 2) {
		  dottree += "|{update function|";
		
		  String[] params = insVisit.getFuncArgs();
		   
		  if(insVisit.getUpdateFunction() == 0) {
			dottree += "new_attribute(" + deReserve(params[0]) + "," + deReserve(params[1]) + ")}";
		  }
		  else {
			dottree += "new_ref(" + deReserve(params[0]) + "," + deReserve(params[1]) + ")}";		 
		  }
		}
		 
		// if there is a relative variable in the insert node
		if(insVisit.getRelativeVariable() != null) {
		  dottree += "|{";
		  if(insVisit.getOrderPredicate() == 0) {
		    dottree += "before ";
		  }
		  else if(insVisit.getOrderPredicate() == 1) {
		 	dottree += "after ";
		  }
		  dottree += "relative variable|" + deReserve(insVisit.getRelativeVariable()) + "}";
		}

		dottree += "}\"];\n";

        return child;
    }
    /**
     * Output of the rename node in an XAT
     */
    public Object visit(Rename renVisit) {
	    String childID = getNextID(renVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(renVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Rename:"+renVisit.getNode().hashCode()+"|"
            + "{context|" + deReserve(renVisit.getContext()) + "}|"
            + "{variable|" + deReserve(renVisit.getName()) + "}|"
            + "{content|" + deReserve(renVisit.getVariable()) + "}"
            + "}\"];\n";

        return child;
    }
    /**
     * Output of the replace node in an XAT
     */
    public Object visit(Replace repVisit) {
	    String childID = getNextID(repVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(repVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Replace:"+repVisit.getNode().hashCode()+"|"
            + "{context|" + deReserve(repVisit.getContext()) + "}|"
            + "{variable|" + deReserve(repVisit.getVariable()) + "}|"
            + "{replacing var.|" + deReserve(repVisit.getReplacingVariable()) + "}";

        // if there appears an update function in the insert node     
		if(repVisit.getUpdateFunction() != 2) {
		  dottree += "|{update function|";
		
		  String[] params = repVisit.getFuncArgs();
		   
		  if(repVisit.getUpdateFunction() == 0) {
			dottree += "new_attribute(" + deReserve(params[0]) + "," + deReserve(params[1]) + ")}";
		  }
		  else {
			dottree += "new_ref(" + deReserve(params[0]) + "," + deReserve(params[1]) + ")}";		 
		  }
		}
		
		// if there is content in the replace node
	    if(repVisit.getPCDATAContent() != null) {
		  dottree += "|{content|" + deReserve(repVisit.getPCDATAContent()) + "}";
	    }

	    dottree += "}\"];\n";
		     
        return child;
    }
	/**
     * Output of the aggregate node in an XAT
     */
    public Object visit(Combine aggVisit) {
	    String childID = getNextID(aggVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(aggVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Combine:"+aggVisit.getNode().hashCode()+"|"
            + "{focus column|" + aggVisit.getOutputColumnName() + "}"
            + "}\"];\n";

        return child;
    }
/**
 * Output of the navigate node in an XAT
 */
public Object visit(Navigate navVisit) {
    String childID = getNextID(navVisit);
    String child = "\"" + childID + "\"";
    nodes.addNode(navVisit, childID);

    int eLength = 0, dLength = 0;

    if (navVisit instanceof NavCollection)
        dottree += "\t"
            + child
            + " [label = \"{NavCollection:"
            + navVisit.getNode().hashCode()
            + "";
    else
        dottree += "\t"
            + child
            + " [label = \"{NavUnnest:"
            + navVisit.getNode().hashCode()
            + "";

    StringBuffer sb = new StringBuffer();

    try {
        NavigationStep[] entry = navVisit.getEntryPoint();
        eLength = entry.length - 1;
        if (eLength > -1) {
            sb.append("|{entry|");
            for (int i = 0; i < eLength; i++) {
                sb.append(deReserve(entry[i].getStepValue()) + ", ");
            }
            sb.append(deReserve(entry[eLength].getStepValue()) + "}");
        }

    } catch (NullPointerException e) {
        sb.append("|{entry|");
        sb.append("null}");
    }

    NavigationStep[] destination = navVisit.getDestination();
    if (destination != null) {
        dLength = destination.length - 1;
        if (dLength > -1) {
            sb.append("|{destination|");
            for (int i = 0; i < dLength; i++) {
                sb.append(deReserve(destination[i].getStepValue()) + ", ");
            }
            sb.append(deReserve(destination[dLength].getStepValue()) + "}");
        }

    } else {
        sb.append("|{destination|");
        sb.append("null}");
    }

    dottree += sb.toString();

    dottree += "|{binding|"
        + deReserve(navVisit.getOutputColumnName())
        + "}}\",style=filled,color=springgreen];\n";

    return child;
}
    /**
     * Output of the tagger node in an XAT
     */
    public Object visit(Tagger tagVisit) {
	    String childID = getNextID(tagVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(tagVisit, childID);

        dottree += "\t" + child
            + " [label = \"{Tagger:"+tagVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(tagVisit.getOutputColumnName()) + "}|";
        
      	DOMPattern tagTree = (DOMPattern)tagVisit.getPattern();
      	dottree += "{tagger tree |" + deReserve(tagTree.toIndentString()) + "}}\",style=filled,color=gold];\n";
    
        return child;
    }
    /**
     * Output of the xml union node in an XAT
     */
    public Object visit(XMLUnion uniVisit) {
	    String childID = getNextID(uniVisit);
        String child = "\"" + childID + "\"";
        nodes.addNode(uniVisit, childID);

        dottree += "\t" + child
            + " [label = \"{XMLUnion:"+uniVisit.getNode().hashCode()+"|"
            + "{output column|" + deReserve(uniVisit.getOutputColumnName()) + "}"
            + "}\"];\n";

        return child;
    }
    
    
    
    
    
    
    
    
}
