package edu.wpi.cs.dsrg.xmldb.xat.component.gui;

import java.awt.print.*;
/**
 * XAT drawing demonstration.
 * Creation date: (2/13/2002 2:11:50 PM)
 * @author: Michal Andrzej Dobosz
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import att.grappa.*;

import xacute.quilt.*;
import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;

import edu.wpi.cs.dsrg.xmldb.xat.component.sqlgeneration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xattreegeneration.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.*;
import edu.wpi.cs.dsrg.demo.ReWriteGUI;

public abstract class XQueryGUI
    extends JFrame
    implements ActionListener, GrappaConstants {

    protected static XQueryGUI gui = null;
    protected Graph graph = null;

    // the current status of the XQueryGUI, for single query or double queries.
    protected boolean single;

    // information stored 
    protected XATTree uxat;
    protected XATTree vxat;
    protected static String dot = null;

    // information passed in through the constructor
    protected String userQuery;
    protected String viewQuery;
    protected String username;
    protected String password;
    protected String URI;

    protected JTabbedPane tabbedpane = null;
    // create a file chooser
    protected JFileChooser fc = new JFileChooser();

    // define what to do when someone clicks a tab
    public class TabListener implements ChangeListener {
        public void stateChanged(ChangeEvent ce) {
            SingleSelectionModel model = (SingleSelectionModel) ce.getSource();
            int tabDex = model.getSelectedIndex();
            ProcessStep ps = (ProcessStep) getStepVector().elementAt(tabDex);

            if (!single) {
                ps.updateContentPane(ps.executeUser(), ps.executeView());
                setUxat(ps.getUserTree());
                setVxat(ps.getViewTree());
                setStepBindingTable(ps.getUserBindingTable(), ps.getViewBindingTable());
            } else {
                ps.updateContentPane(ps.executeUser(), tabDex, tabDex, userQuery, tabDex);
                setUxat(ps.getUserTree());
                setStepBindingTable(ps.getUserBindingTable());
            }
            /* 
            * pretty ineffcient, sets the bindingtable of each step every time this event is fired
            * a better solution would involve a hack to make sure it runs only once.  
            */
            setStepBindingTable(ps.getUserBindingTable(), ps.getViewBindingTable());
            System.out.println("Tab " + tabDex);
            grappapanel1 = ps.getUserGrappaPanel();
            updateTrees();
        }
    }

    // define the item types
    protected final int ITEM_PLAIN = 0;
    protected final int ITEM_CHECK = 1;
    protected final int ITEM_RADIO = 2;

    protected JMenuBar menuBar;
    protected JMenu menuFile;
    protected JMenu menuEdit;
    protected JMenu menuGraph;
    protected JMenu menuProperty;
    protected JMenu menuScale;
    protected JMenuItem menuPropertySystem;
    protected JMenuItem menuPropertyEditor;
    protected JMenuItem menuPropertyDisplay;
    protected JMenuItem menuScaleToFit;
    protected JMenuItem menuScale100;
    protected JMenuItem menuScale75;
    protected JMenuItem menuScale50;
    protected JMenuItem menuScale25;
    protected JMenuItem menuFileNew;
    protected JMenuItem menuFileOpen;
    protected JMenuItem menuFileSave;
    protected JMenuItem menuFileSaveAs;
    protected JMenuItem menuFileExit;
    protected JMenuItem menuEditCopy;
    protected JMenuItem menuEditCut;
    protected JMenuItem menuEditPaste;
    protected JMenuItem menuGraphRepaint;
    protected JMenuItem menuGraphPrint;
    protected JMenuItem menuGraphReset;
    protected JMenuItem menuGraphResync;

    protected Vector stepVector;
    protected att.grappa.GrappaPanel grappapanel1;
    /**
    * XQuery GUI constructor.
    *
    * @param userQuery_in the user query
    * @param username_in the username of an account on the Oracle server
    * @param password_in the password of the account on the Oracle server
    * @param URI_in the Universal Resource Indicator of the Oracle server
    * @param type the type of the GUI. XQueryGUI.QUERY works on queries. XQueryGUI.UPDATE works on updates.
    */
    public XQueryGUI(
        String userQuery_in,
        String username_in,
        String password_in,
        String URI_in)
        throws OracleConnectionException {
        userQuery = userQuery_in;
        if (username_in == null | password_in == null | URI_in == null) {
            throw new OracleConnectionException("Must specify the username, password, and URI.");
        }
        username = username_in;
        password = password_in;
        URI = URI_in;
        stepVector = new Vector();
        uxat = new XATTreeImp();
        single = true;
    }
    /**
    * XQuery GUI constructor.
    *
    * @param userQuery_in the user query
    * @param viewQuery_in the view query
    * @param username_in the username of an account on the Oracle server
    * @param password_in the password of the account on the Oracle server
    * @param URI_in the Universal Resource Indicator of the Oracle server
    * @param type the type of the GUI. XQueryGUI.QUERY works on queries. XQueryGUI.UPDATE works on updates.
    */
    public XQueryGUI(
        String userQuery_in,
        String viewQuery_in,
        String username_in,
        String password_in,
        String URI_in)
        throws OracleConnectionException {
        userQuery = userQuery_in;
        viewQuery = viewQuery_in;
        if (username_in == null | password_in == null | URI_in == null) {
            throw new OracleConnectionException("Must specify the username, password, and URI.");
        }
        username = username_in;
        password = password_in;
        URI = URI_in;
        stepVector = new Vector();
        uxat = new XATTreeImp();
        vxat = new XATTreeImp();
        single = false;
    }
    // define what to do when someone selects a menu item
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() instanceof JMenuItem) {
            JMenuItem menu = (JMenuItem) evt.getSource();
            if (menu == menuFileExit) {
                System.exit(0);
            } else
                if (menu == menuFileOpen) {
                    int returnVal = fc.showOpenDialog(menu);

                    if (returnVal == JFileChooser.APPROVE_OPTION) {
                        File file = fc.getSelectedFile();
                        System.out.println("Opening: \"" + file.getPath()+ "\".");

                        try {
                            userQuery = readFile(file.getPath());
                            /**
                            * not sure what happens here
                            */
                            //newSteps();
                            tabbedpane.setSelectedIndex(0);
                        } catch (Exception e) {
                            e.printStackTrace(System.out);
                            System.exit(-1);
                        }
                    } else {
                        System.out.println("Open command cancelled by user.");
                    }
                } else
                    if (menu == menuFileSave) {
                        int returnVal = fc.showSaveDialog(menu);

                        if (returnVal == JFileChooser.APPROVE_OPTION) {
                            File file = fc.getSelectedFile();
                            // this is where a real application would save the file.
                            System.out.println("Saving: \"" + file.getName() + "\".");
                        } else {
                            System.out.println("Save command cancelled by user.");
                        }
                    } else
                        if (menu == menuGraphRepaint) {
                            grappapanel1.repaint();
                        } else
                            if (menu == menuGraphPrint) {
                                // Get a PrinterJob     
                                PrinterJob job = PrinterJob.getPrinterJob();
                                // Specify the Printable is an instance of SimplePrint
                                job.setPrintable(grappapanel1);
                                // Put up the dialog box     
                                if (job.printDialog()) {
                                    // Print the job if the user didn't cancel printing 
                                    try {
                                        job.print();
                                    } catch (Exception e) { /* handle exception */
                                        System.out.println("Print Error: " + e.getMessage());
                                    }
                                }
                            } else
                                if (menu == menuScale100) {
                                    grappapanel1.setScaleToFit(false);
                                    grappapanel1.resetZoom();
                                    grappapanel1.repaint();
                                } else
                                    if (menu == menuScaleToFit) {
                                        grappapanel1.setScaleToFit(true);
                                        grappapanel1.repaint();
                                    }
        }
    }
    protected void createMenu() {
        // Create the menu bar
        menuBar = new JMenuBar();

        // Set this instance as the application's menu bar
        setJMenuBar(menuBar);

        // Build the property sub-menu
        menuProperty = new JMenu("Properties");
        menuProperty.setMnemonic('P');

        // Create property items
        menuPropertySystem =
            CreateMenuItem(menuProperty, ITEM_PLAIN, "System...", null, 'S', null);
        menuPropertyEditor =
            CreateMenuItem(menuProperty, ITEM_PLAIN, "Editor...", null, 'E', null);
        menuPropertyDisplay =
            CreateMenuItem(menuProperty, ITEM_PLAIN, "Display...", null, 'D', null);

        // Create the file menu
        menuFile = new JMenu("File");
        menuFile.setMnemonic('F');
        menuBar.add(menuFile);

        // Create the file menu
        // Build a file menu items
        menuFileNew = CreateMenuItem(menuFile, ITEM_PLAIN, "New", null, 'N', null);
        menuFileOpen =
            CreateMenuItem(
                menuFile,
                ITEM_PLAIN,
                "Open...",
                new ImageIcon("open.gif"),
                'O',
                "Open a new file");
        menuFileSave =
            CreateMenuItem(
                menuFile,
                ITEM_PLAIN,
                "Save",
                new ImageIcon("save.gif"),
                'S',
                "Save this file");
        menuFileSaveAs =
            CreateMenuItem(
                menuFile,
                ITEM_PLAIN,
                "Save As...",
                null,
                'A',
                "Save this data to a new file");
        // Add the property menu to the file menu
        menuFile.addSeparator();
        menuFile.add(menuProperty);
        menuFile.addSeparator();
        menuFileExit =
            CreateMenuItem(menuFile, ITEM_PLAIN, "Exit", null, 'x', "Exit the program");

        // Create the edit menu
        menuEdit = new JMenu("Edit");
        menuEdit.setMnemonic('E');
        menuBar.add(menuEdit);

        // Create edit menu options
        menuEditCut =
            CreateMenuItem(
                menuEdit,
                ITEM_PLAIN,
                "Cut",
                new ImageIcon("cut.gif"),
                't',
                "Cut data to the clipboard");
        menuEditCopy =
            CreateMenuItem(
                menuEdit,
                ITEM_PLAIN,
                "Copy",
                new ImageIcon("copy.gif"),
                'C',
                "Copy data to the clipboard");
        menuEditPaste =
            CreateMenuItem(
                menuEdit,
                ITEM_PLAIN,
                "Paste",
                new ImageIcon("paste.gif"),
                'P',
                "Paste data from the clipboard");

        // Build the scale sub-menu
        menuScale = new JMenu("Scale");
        menuScale.setMnemonic('S');

        // Create scale items
        menuScaleToFit =
            CreateMenuItem(
                menuScale,
                ITEM_PLAIN,
                "Scale To Fit",
                null,
                'S',
                "Scale the graph to fit the panel");
        menuScale100 =
            CreateMenuItem(
                menuScale,
                ITEM_PLAIN,
                "100%",
                null,
                '1',
                "Scale the graph to 100%");
        menuScale75 =
            CreateMenuItem(
                menuScale,
                ITEM_PLAIN,
                "75%",
                null,
                '7',
                "Scale the graph to 75%");
        menuScale50 =
            CreateMenuItem(
                menuScale,
                ITEM_PLAIN,
                "50%",
                null,
                '5',
                "Scale the graph to 50%");
        menuScale25 =
            CreateMenuItem(
                menuScale,
                ITEM_PLAIN,
                "25%",
                null,
                '2',
                "Scale the graph to 25%");

        // Create the graph menu
        menuGraph = new JMenu("Graph");
        menuGraph.setMnemonic('G');
        menuBar.add(menuGraph);

        // Create graph menu options
        menuGraphRepaint =
            CreateMenuItem(
                menuGraph,
                ITEM_PLAIN,
                "Repaint",
                null,
                'p',
                "Repaint the graph");
        menuGraphPrint =
            CreateMenuItem(menuGraph, ITEM_PLAIN, "Print", null, 'P', "Print the graph");
        menuGraphReset =
            CreateMenuItem(menuGraph, ITEM_PLAIN, "Reset", null, 's', "Reset the graph");
        menuGraphResync =
            CreateMenuItem(menuGraph, ITEM_PLAIN, "Resync", null, 'y', "Resync the graph");
        // Add the scale menu to the graph menu
        menuGraph.addSeparator();
        menuGraph.add(menuScale);
    }
    protected JMenuItem CreateMenuItem(
        JMenu menu,
        int iType,
        String sText,
        ImageIcon image,
        int acceleratorKey,
        String sToolTip) {
        // Create the item
        JMenuItem menuItem;

        switch (iType) {
            case ITEM_RADIO :
                menuItem = new JRadioButtonMenuItem();
                break;

            case ITEM_CHECK :
                menuItem = new JCheckBoxMenuItem();
                break;

            default :
                menuItem = new JMenuItem();
                break;
        }

        // Add the item test
        menuItem.setText(sText);

        // Add the optional icon
        if (image != null) {
            menuItem.setIcon(image);
        }

        // Add the accelerator key
        if (acceleratorKey > 0) {
            menuItem.setMnemonic(acceleratorKey);
        }

        // Add the optional tool tip text
        if (sToolTip != null) {
            menuItem.setToolTipText(sToolTip);
        }

        // Add an action handler to this menu item
        menuItem.addActionListener(this);

        menu.add(menuItem);

        return menuItem;
    }
    public void displayGraph() {
        this.setTitle("Raindrop - XQuery Evaluation over Streaming XML");
        this.setLocation(0, 0);

        // set the file filter for the file chooser
        KweeltFilter kf = new KweeltFilter();
        fc.setFileFilter(kf);

        // create the tabs
        tabbedpane = new JTabbedPane();
        Vector steps = getStepVector();
        //iterate through the steps in the vector and add them to the tabbedPane
        for (int i = 0; i < steps.size(); i++) {
            ProcessStep ps = (ProcessStep) steps.elementAt(i);
            tabbedpane.addTab(
                ps.getTabName(),
                new ImageIcon("images/icon.gif"),
                ps.getContentPane(),
                ps.getTabTip());
            System.out.println(ps.toString());
        }

        // well, select it the second panel.
        tabbedpane.setSelectedIndex(1);
        tabbedpane.getModel().addChangeListener(new TabListener());
        // select the first tab, so we can have the first tab pane executed.
        tabbedpane.setSelectedIndex(0);

        JPanel tabPanel = new JPanel();
        tabPanel.setLayout(new BorderLayout());
        tabPanel.add(tabbedpane, BorderLayout.CENTER);

        getContentPane().add("Center", tabPanel);

        // create menu.
        this.createMenu();
        this.setSize(800, 600);

        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                //System.exit(0);
                 ((JFrame) e.getSource()).setVisible(false);
            }
        });

        this.setVisible(true);
    }
    /**
     * Insert the method's description here.
     * 
     * @return java.lang.String
     */
    protected java.lang.String getPassword() {
        return password;
    }
    /**
     * Insert the method's description here.
     * 
     * @return java.util.Vector
     */
    public java.util.Vector getStepVector() {
        return stepVector;
    }
    /**
     * Insert the method's description here.
     * 
     * @return javax.swing.JTabbedPane
     */
    public javax.swing.JTabbedPane getTabbedpane() {
        return tabbedpane;
    }
    /**
     * Insert the method's description here.
     * 
     * @return java.lang.String
     */
    protected java.lang.String getURI() {
        return URI;
    }
    /**
     * Insert the method's description here.
     * 
     * @return java.lang.String
     */
    protected java.lang.String getUsername() {
        return username;
    }
    /**
     * Insert the method's description here.
     * 
     * @return java.lang.String
     */
    protected java.lang.String getUserQuery() {
        return userQuery;
    }
    /**
     * Insert the method's description here.
     * 
     * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
     */
    public edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree getUxat() {
        return uxat;
    }
    /**
     * Insert the method's description here.
     * 
     * @return java.lang.String
     */
    protected java.lang.String getViewQuery() {
        return viewQuery;
    }
    /**
     * Insert the method's description here.
     * 
     * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
     */
    public edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree getVxat() {
        return vxat;
    }
    /**
    * Reads in the contents of a file and converts it to a string object
    */
    protected static String readFile(String file) throws Exception {
        BufferedReader in =
            new BufferedReader(new InputStreamReader(new FileInputStream(file)));
        StringBuffer buf = new StringBuffer();
        String s;
        while ((s = in.readLine()) != null) {
            buf.append(s + "\n");
        }
        in.close();
        return buf.toString();
    }
    /**
     * Insert the method's description here.
     * Creation date: (5/21/2002 1:19:05 PM)
     */
    public void setParameters(
        String uname,
        String upasswd,
        String uURI,
        String uQuery,
        String vQuery) {
        username = uname;
        password = upasswd;
        URI = uURI;
        userQuery = uQuery;
        viewQuery = vQuery;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newPassword java.lang.String
     */
    protected void setPassword(java.lang.String newPassword) {
        password = newPassword;
    }
    /**
     * sets the binding hashtable for each step
     * 
     */
    public void setStepBindingTable(BindingTable userBindingTable) {
        for (int i = 0; i < stepVector.size(); i++) {
            ((ProcessStep) stepVector.elementAt(i)).setUserBindingTable(userBindingTable);
        }
    }
    /**
     * sets the binding hashtable for each step
     * 
     */
    public void setStepBindingTable(
        BindingTable userBindingTable,
        BindingTable viewBindingTable) {
        for (int i = 0; i < stepVector.size(); i++) {
            ((ProcessStep) stepVector.elementAt(i)).setUserBindingTable(userBindingTable);
            ((ProcessStep) stepVector.elementAt(i)).setViewBindingTable(viewBindingTable);
        }
    }
    /**
     * Insert the method's description here.
     * 
     * @param newStepVector java.util.Vector
     */
   public void setStepVector(java.util.Vector newStepVector) {
        stepVector = newStepVector;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newURI java.lang.String
     */
    protected void setURI(java.lang.String newURI) {
        URI = newURI;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newUsername java.lang.String
     */
    protected void setUsername(java.lang.String newUsername) {
        username = newUsername;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newUserQuery java.lang.String
     */
    protected void setUserQuery(java.lang.String newUserQuery) {
        userQuery = newUserQuery;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newUxat edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
     */
    public void setUxat(
        edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree newUxat) {
        uxat = newUxat;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newViewQuery java.lang.String
     */
    public void setViewQuery(java.lang.String newViewQuery) {
        viewQuery = newViewQuery;
    }
    /**
     * Insert the method's description here.
     * 
     * @param newVxat edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
     */
    public void setVxat(
        edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree newVxat) {
        vxat = newVxat;
    }
    /**
     * Updates the user and view trees of each step
     * 
     */
    public void updateTrees() {

        for (int i = 0; i < stepVector.size(); i++) {
            ProcessStep ps1 = (ProcessStep) stepVector.elementAt(i);
            ps1.setUserTree(this.getUxat());
            ps1.setViewTree(this.getVxat());
        }

    }
}
