package edu.wpi.cs.dsrg.xmldb.xat.component.gui.listeners;

import diva.graph.JGraph;
import diva.graph.basic.BasicNodeRenderer;
import diva.graph.basic.BasicEdgeRenderer;
import diva.graph.modular.Node;
import diva.graph.modular.Edge;

import java.util.Iterator;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.gui.CAPE_GUI;

/** The listener for the update node color timer.
 * Updates the node color to:
 * <ul>
 * <li>gray - if the node is inactive
 * <li>red - if the node is active and
 * <li>green - if the node is running.
 * </ul>
 */
public class UpdateNodeColorListener implements ActionListener{
	
	/** The CAPE_GUI */
	private CAPE_GUI _gui;
	
	/** Constructor associates the given <code>gui</code> with this listener. */
	public UpdateNodeColorListener(CAPE_GUI gui){
		_gui = gui;
	}
		
	public void actionPerformed(ActionEvent evt){
		try{//for each graph
			for(Iterator it = _gui.getGraphManager().getGraphs(); it.hasNext();){
				JGraph graph = (JGraph)it.next();
				//for each node
				for(Iterator nodes = graph.getGraphPane().getGraphModel().getNodes(); nodes.hasNext();){
					Node node = (Node)nodes.next();
					
					//get the semantic object(dag node) of this node
					DagNode dagNode = (DagNode)node.getSemanticObject();
					
					//get the node renderer for this node
					BasicNodeRenderer nrend = (BasicNodeRenderer)graph.getGraphPane()
						.getGraphController().getNodeController(node).getNodeRenderer();
						
					//if the node represents an input/output stream(s) node, then paint blue
					if(((XATStreamOperator)dagNode.getOperator()) == null){
						nrend.setNodeFill(Color.blue);
					}//otherwise the node is an actual node
					else{	
						//if the node is running, then paint green
						if(((XATStreamOperator)dagNode.getOperator()).isRunning()){
							nrend.setNodeFill(Color.green);
						}//if the node is active, then paint red
						else if(((XATStreamOperator)dagNode.getOperator()).isActive()){
							nrend.setNodeFill(Color.red);
						}//otherwise the node is inactive and paint gray
						else{nrend.setNodeFill(Color.gray);}
					}				
					graph.getGraphPane().getGraphController().getNodeController(node).setNodeRenderer(nrend);
					graph.getGraphPane().getGraphController().getNodeController(node).drawNode(node);
				}//for each edge
				for(Iterator edges = graph.getGraphPane().getGraphModel().getEdges(); edges.hasNext();){
					Edge edge = (Edge)edges.next();
					
					//get the edge renderer for this edge
					BasicEdgeRenderer erend = (BasicEdgeRenderer)graph.getGraphPane().getGraphController().
						getEdgeController(edge).getEdgeRenderer();
					
					//erend.setLabelTextSize(10);
					if(((XATQueue)edge.getSemanticObject()).getSchema() == null){
						erend.setLabelTextColor(Color.gray);
					}
					else{erend.setLabelTextColor(Color.black);}
					
					graph.getGraphPane().getGraphController().getEdgeController(edge).setEdgeRenderer(erend);	
					graph.getGraphPane().getGraphController().getEdgeController(edge).drawEdge(edge);
				}
				_gui.getGraphManager().redrawGraph(graph);
			}
		}catch(Exception e){System.err.println("UpdateNodeColorListener: Error updating node color.");}
	}
}