package edu.wpi.cs.dsrg.xmldb.xat.component.gui.listeners;

import java.util.Iterator;
import java.util.List;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import diva.canvas.toolbox.LabelFigure;
import diva.graph.JGraph;
import diva.graph.modular.Edge;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.component.gui.CAPE_GUI;

/** The listener for the update queues timer.
 * Update all queues every <code>UPDATE_QUEUES_SECONDS</code> milliseconds. 
 * 
 * <p>Future Work: Modify Diva's repaint to properly repaint the changed region only rather
 * than redrawing the entire graph. */
public class UpdateQueuesListener implements ActionListener{
	
	/** The CAPE_GUI */
	private CAPE_GUI _gui;
	
	/** Constructor associates the given <code>gui</code> with this listener.
	 * @param gui gui to associate with this listener */
	public UpdateQueuesListener(CAPE_GUI gui){
		_gui = gui;
	}
	
	public void actionPerformed(ActionEvent evt){
		try{ //for each graph
			for(Iterator it = _gui.getGraphManager().getGraphs(); it.hasNext();){
				JGraph graph = (JGraph)it.next();
				//for each edge
				for(Iterator edges = graph.getGraphPane().getGraphModel().getEdges(); edges.hasNext();){
					Edge edge = (Edge) edges.next();
					XATQueue queue = (XATQueue)edge.getSemanticObject();
					int queueNum = queue.getQueueNumber();
					int lastDQed = queue.size();
					List owners = queue.getQueueOwner();
					
					//if this queue is an input stream then make the label:
					//QueueName+"_O"+OBJECT_ID+":"size  Example: Nile_O2:249
					LabelFigure label = new LabelFigure();
					if(queue.getQueueName().length() > 0){
						label = new LabelFigure(queue.getQueueName()+"_O"+
							((XATStreamOperator)owners.get(0)).getProperty("OBJECT_ID")
							+"_Q"+queueNum+":"+lastDQed);
					}
					else{
						label = new LabelFigure("O"+((XATStreamOperator)owners.get(0))
							.getProperty("OBJECT_ID")+"_Q"+queueNum+":"+lastDQed);
					}
					edge.setLabelFigure(label);
				}
			}
		}catch(Exception e){
			e.printStackTrace();
		}
	}
}