package edu.wpi.cs.dsrg.xmldb.xat.component.gui.utils;

import javax.swing.Icon;
import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import javax.swing.ImageIcon;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.component.gui.CAPE_GUI;

/** Tabs for the tabbed panes with an 'X' (close icon) to close the tab.
 * Each tab can be associated with a query object:
 * <ul>
 * <li>XATTree
 * <li>XATQueue
 * <li>XATStreamOperator
 * <li>SystemObject
 * </ul>
 * and a pane type:
 * <ul>
 * <li><code>STATS</code>: the tabbed pane to hold statistic table(s)
 * <li><code>QUERY</code>: the tabbed pane to hold graphs, or query plans
 * </ul>
 * @author Maylene Waltz
 */
public class CAPETab extends MouseAdapter implements Icon {
	private int x_pos = 0;
	private int y_pos = 0;
	private int width = 15;
	private int height = 17;
	
	/** The type of tabbed pane this tab is in (a statistics tabbed pane 
	 * or graph tabbed pane). */
	private String _paneType;
	
	/** The <code>XATQueryObject</code> associated with this tab. */
	private XATQueryObject _queryObject;
	
	/** Constructor to make a new tab. */
	public CAPETab(){
		_queryObject = null;
		_paneType = null;
	}
	
	/** Constructor to make a new tab associated with the given <code>queryObject</code>.
	 * @param queryObject the query object associated with this tab */
	public CAPETab(Object queryObject){
		if(queryObject instanceof XATQueryObject){
			_queryObject = (XATQueryObject)queryObject;
		}else _queryObject = null;
		_paneType = null;
	}
	
	/** Constructor to make a new tab associated with the given <code>paneType</code>.
	 * @param paneType the type of tabbed pane this tab belongs to */
	public CAPETab(String paneType){
		_queryObject = null;
		_paneType = paneType;
	}
	
	/** Constructor to make a new tab associated with the given
	 * <code>queryObject</code> and <code>paneType</code>.
	 * @param queryObject the query object to associate with this tab
	 * @param paneType the type of tabbed pane this tab belongs to */
	public CAPETab(Object queryObject, String paneType){
		if(queryObject instanceof XATQueryObject){
			_queryObject = (XATQueryObject)queryObject;
		}else _queryObject = null;
		_paneType = paneType;
	}
	
	/** Paints the 'X' icon on the tab. */
	public void paintIcon(Component c, Graphics g, int x, int y){
		this.x_pos = x;
		this.y_pos = y;
		int y_p = y+2;
		
		java.net.URL exitIconURL = CAPE_GUI.class.getResource("images/exit_small.png");
		ImageIcon exitIcon = new ImageIcon(exitIconURL, "Close Tab");

		g.drawImage(exitIcon.getImage(), x, y_p, 16, 16, c);
		//Draws the 'X's outlining square to close the tab
		//g.drawRoundRect(x, y_p, 13, 13, 2, 2);
		//g.drawOval(x, y_p, 13, 13);
		
		//Draws the 'X' to close the tab
		//g.drawLine(x+3, y_p+3, x+10, y_p+10);
		//g.drawLine(x+3, y_p+4, x+9, y_p+10);
		//g.drawLine(x+4, y_p+3, x+10, y_p+9);
		//g.drawLine(x+10, y_p+3, x+3, y_p+10);
		//g.drawLine(x+10, y_p+4, x+4, y_p+10);
		//g.drawLine(x+9, y_p+3, x+3, y_p+9);
	}
	
	/** Returns the bounds of the 'X' on the tab. 
	 * @return the rectangular bounds of the tab's icon(s) */
	public Rectangle getBounds(){
		return new Rectangle(x_pos, y_pos, width, height);
	}
	
	/** Returns the icon's width. 
	 * @return the icon's width in pixels */
	public int getIconWidth(){
		return width;
	}
	
	/** Returns the icon's height.
	 * @return the icon's height in pixels */
	public int getIconHeight(){
		return height;
	}
	
	/** Returns the pane type. 
	 * @return the type of tabbed pane this tab belongs to */
	public String getPaneType(){
		return _paneType;
	}
	
	/** Returns the <code>XATQueryObject</code> associated with this tab. 
	 * @return the query object associated with this tab */
	public XATQueryObject getXATQueryObject(){
		return _queryObject;
	}
}