package edu.wpi.cs.dsrg.xmldb.xat.component.gui.utils;

import java.awt.Shape;
import java.awt.Color;
import java.awt.geom.Rectangle2D;

/** Utility functions for the CAPE_GUI. */
public class CAPEUtils{
	
	/** Determines the text color based on the selected index
	 * of 'Node Text Color' combo box in the customize panel.
	 * @param index the selected index of the 'Node Text Color' combo box
	 * @return the color at the selected index of the 'Node Text Color' combo box */
	static public Color determineColor(int index){
		Color color = new Color(0,0,0);		
		switch(index){
			case 0: color = Color.BLACK; break;
			case 1: color = Color.WHITE; break;
			case 2: color = Color.DARK_GRAY; break;
			case 3: color = Color.GRAY; break;
			case 4: color = Color.RED; break;
			case 5: color = Color.ORANGE; break;
			case 6: color = Color.YELLOW; break;
			case 7: color = Color.GREEN; break;
			case 8: color = Color.CYAN; break;
			case 9: color = Color.BLUE; break;
			default: return color;
		}
		return color;
	}
	
	/** Determines the node size based on the selected index
	 * of the 'Node Size' combo box in the customize panel.
	 * @param index the selected index of the 'Node Size' combo box
	 * @return a rectangle relative in size to the selected index of the 'Node Size' combo box */
	static public Shape determineNodeSize(int index){
		Rectangle2D size = new Rectangle2D.Double(0.0,0.0,10.0,10.0); //default size
		switch(index){
			case 0: size.setRect(0.0,0.0,10.0,10.0); break; //Extra Small
			case 1: size.setRect(0.0,0.0,20.0,20.0); break; //Small
			case 2: size.setRect(0.0,0.0,30.0,30.0); break; //Medium
			case 3: size.setRect(0.0,0.0,40.0,40.0); break; //Large
			case 4: size.setRect(0.0,0.0,50.0,50.0); break; //Extra Large
			default: return size;
		}
		return size;
	}
	
	/** Determines the text size based on the selected index
	 * of the 'Node Text Size' combo boxe in the customize panel.
	 * @param index the selected index of the 'Node Text Size' combo box
	 * @return the text size relative to the selected index of the 'Node Text Size' combo box */
	static public int determineTextSize(int index){
		int size = 12; //default size
		switch(index){
			case 0: size = 12; break; //Extra Small
			case 1: size = 16; break; //Small
			case 2: size = 22; break; //Medium
			case 3: size = 28; break; //Large
			case 4: size = 36; break; //Extra Large
			default: return size;
		}	
		return size;
	}
}