package edu.wpi.cs.dsrg.xmldb.xat.component.gui.utils;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;

/** This class creates sample trees to view in the CAPE_GUI.
 * No statistics for table(s) or plot(s) are available using these
 * sample trees since they are not part of an actual running CAPE system.
 * 
 * @author Maylene Waltz
 */
public class CreateSampleTrees {
	public XATDag[] trees; 
	
	/** Constructor initializes an array of tree(s). */
	public CreateSampleTrees(){
		trees = new XATDag[1];
	}

	/**
	 * Creates sample XATDag tree(s).
	 * @return an array of sample tree(s)
	 */
	public XATDag[] createSampleTrees(){
		trees[0] = createTree1();
		//trees[1] = createTree2();
		//trees[2] = createTree3();
		//trees[3] = createTree4();
		//trees[4] = createTree5();
		//trees[1] = createTree6();
		return trees;
	}
	
	/**
	 * XATDag tree1:
	 * 		C-B-A
	 * @return a sample tree
	 */
	protected XATDag createTree1(){
		XATDag tree1 = new XATDagImp();
		DagNode A = new DagNodeImp();
		DagNode B = new DagNodeImp();
		DagNode C = new DagNodeImp();
		
		tree1.setRoot(A);
		A.setChild(0,B);
		B.setChild(0,C);
		B.setParent(A);
		C.setParent(B);
		
		//create dummy operators for each node
		ProbabilisticMultiStreamOperator Aop = new ProbabilisticMultiStreamOperatorImp(A);
		ProbabilisticMultiStreamOperator Bop = new ProbabilisticMultiStreamOperatorImp(B);
		ProbabilisticMultiStreamOperator Cop = new ProbabilisticMultiStreamOperatorImp(C);
		try{
			tree1.setProperty("TREE_ID", "1");
			Aop.setProperty("OBJECT_ID", "1");
			Bop.setProperty("OBJECT_ID", "2");
			Cop.setProperty("OBJECT_ID", "3");
		}catch(Exception e){}
		
		//Aq's dummy tuples
		XATTuple Atuple1 = new XATTupleImp();
		XATTuple Atuple2 = new XATTupleImp();
		XATTuple Atuple3 = new XATTupleImp();
		XATTuple Atuple4 = new XATTupleImp();
		//Bq's dummy tuples
		XATTuple Btuple1 = new XATTupleImp();
		XATTuple Btuple2 = new XATTupleImp();
		XATTuple Btuple3 = new XATTupleImp();
		//IA's dummy tuples
		XATTuple IAtuple1 = new XATTupleImp();
		XATTuple IAtuple2 = new XATTupleImp();
		XATTuple IAtuple3 = new XATTupleImp();
		XATTuple IAtuple4 = new XATTupleImp();
		XATTuple IAtuple5 = new XATTupleImp();
		
		//Create the queues
		XATQueue Aq = new XATMemoryQueueImp(); // B -> A
		XATQueue Bq = new XATMemoryQueueImp(); // C -> B
		XATQueue IAq1 = new XATMemoryQueueImp(); //Input -> C
		//XATQueue Aq = new XATQueueImp(); // B -> A
		//XATQueue Bq = new XATQueueImp(); // C -> B
		//XATQueue IAq1 = new XATQueueImp(); //Input -> C
		//XATQueue Aq; 	// B -> A
		//XATQueue Bq; 	// C -> B
		//XATQueue IAq1;	//Input -> C
		
		if(System.getProperty("DATA_MODEL").endsWith("XATMemoryQueueImp")) {
			Aq = new XATMemoryQueueImp(); 
			Bq = new XATMemoryQueueImp(); 
			IAq1 = new XATMemoryQueueImp(); 
		}
		else if (System.getProperty("DATA_MODEL").endsWith("XATPersistentQueueImp")) {
			Aq = new XATPersistentQueueImp(); 
			Bq = new XATPersistentQueueImp(); 
			IAq1 = new XATPersistentQueueImp();
		}
		else {
			System.out.println("!!! Error: Unknow type of DATA_MODEL !!!");
			System.exit(-1);
			return null;
		}
		
		//Set the queues owners
		Aq.addNewOwner(Bop);
		Bq.addNewOwner(Cop);
		//IAq1.addNewOwner(Cop); //who is input queues owner? needed?
		
		//enqueue Aq's tuples
		Aq.enqueue(Atuple1, B);
		Aq.enqueue(Atuple2, B);
		Aq.enqueue(Atuple3, B);
		Aq.enqueue(Atuple4, B);
		//enqueue Bq's tuples
		Bq.enqueue(Btuple1, C);
		Bq.enqueue(Btuple2, C);
		Bq.enqueue(Btuple3, C);
		//enqueue inputAQ1's tuples
		/*IAq1.enqueue(IAtuple1, A);
		IAq1.enqueue(IAtuple2, A);
		IAq1.enqueue(IAtuple3, A);
		IAq1.enqueue(IAtuple4, A);
		IAq1.enqueue(IAtuple5, A);*/
		
		
		Aop.setInputQueue(Aq);
		Bop.setInputQueue(Bq);
		Bop.setOutputQueue(Aq);
		//Cop.setInputQueue(IAq1);
		Cop.setOutputQueue(Bq);
		
		return tree1;
	}
	
	/** 
	 * XATDag tree2:
	 * 		A
	 * 	   / \
	 *    B   C
	 * @return a sample tree
	 */
	protected XATDag createTree2(){
		XATDag tree2 = new XATDagImp();
		DagNode A = new DagNodeImp();
		DagNode B = new DagNodeImp(); 
		DagNode C = new DagNodeImp(); 
		
		tree2.setRoot(A);
		A.setChild(0,B);
		A.setChild(1,C);
		B.setParent(A);
		C.setParent(A);
		
		return tree2;
	}
	
	/** 
	 * XATDag tree3:
	 * 		A
	 * 	   / \
	 *    B   E
	 *   / \
	 *  C   D
	 * @return a sample tree
	 */
	protected XATDag createTree3(){
		XATDag tree3 = new XATDagImp();
		DagNode A = new DagNodeImp();
		DagNode B = new DagNodeImp();
		DagNode C = new DagNodeImp();
		DagNode D = new DagNodeImp();
		DagNode E = new DagNodeImp();
		
		tree3.setRoot(A);
		A.setChild(0,B);
		A.setChild(1,E);
		B.setParent(A);
		E.setParent(A);
		B.setChild(0,C);
		B.setChild(1,D);
		C.setParent(B);
		D.setParent(B);
		
		return tree3;
	}
	
	/** 
	 * XATDag tree4:
	 * 		A
	 * 	   / \
	 *    B   E
	 *   / \ /
	 *  C   D
	 * @return a sample tree
	 */
	protected XATDag createTree4(){
		XATDag tree4 = new XATDagImp();
		DagNode A = new DagNodeImp();
		DagNode B = new DagNodeImp();
		DagNode C = new DagNodeImp();
		DagNode D = new DagNodeImp();
		DagNode E = new DagNodeImp();
		
		tree4.setRoot(A);
		A.setChild(0,B);
		A.setChild(1,E);
		B.setParent(A);
		E.setParent(A);
		B.setChild(0,C);
		B.setChild(1,D);
		E.setChild(0,D);
		C.setParent(B);
		D.setParent(B);
		D.addParent(E);
		
		return tree4;
	}
	
	/**
	 * XATDag tree5:
	 * 	     A   B
	 *      / \ / \
	 *     C   D   E
	 *      \ /   / \
	 *       F   G   H
	 * @return a sample tree
	 */
	protected XATDag createTree5(){
		XATDag tree5 = new XATDagImp();
		DagNode A = new DagNodeImp();
		DagNode B = new DagNodeImp();
		DagNode C = new DagNodeImp();
		DagNode D = new DagNodeImp();
		DagNode E = new DagNodeImp();
		DagNode F = new DagNodeImp();
		DagNode G = new DagNodeImp();
		DagNode H = new DagNodeImp();
		
		tree5.setRoot(A);
		tree5.addRoot(B);
		A.setChild(0,C);
		A.setChild(1,D);
		C.setParent(A);
		D.setParent(A);
		B.setChild(0,D);
		B.setChild(1,E);
		D.addParent(B);
		E.setParent(B);
		C.setChild(0,F);
		F.setParent(C);
		D.setChild(0,F);
		F.addParent(D);
		E.setChild(0,G);
		E.setChild(1,H);
		G.setParent(E);
		H.setParent(E);
		
		return tree5;
	}
	
	/**
	 * XATDag tree6:
	 * A-B-C-D-E-F-G-H-I-J-K-L-M-N-O-P
	 * @return a sample tree
	 */
	protected XATDag createTree6(){
		XATDag tree6 = new XATDagImp();
		DagNode A = new DagNodeImp();
		DagNode B = new DagNodeImp();
		DagNode C = new DagNodeImp();
		DagNode D = new DagNodeImp();
		DagNode E = new DagNodeImp();
		DagNode F = new DagNodeImp();
		DagNode G = new DagNodeImp();
		DagNode H = new DagNodeImp();
		DagNode I = new DagNodeImp();
		DagNode J = new DagNodeImp();
		DagNode K = new DagNodeImp();
		DagNode L = new DagNodeImp();
		DagNode M = new DagNodeImp();
		DagNode N = new DagNodeImp();
		DagNode O = new DagNodeImp();
		DagNode P = new DagNodeImp();
		
		tree6.setRoot(A);
		A.setChild(0,B);
		B.setChild(0,C);
		C.setChild(0,D);
		D.setChild(0,E);
		E.setChild(0,F);
		F.setChild(0,G);
		G.setChild(0,H);
		H.setChild(0,I);
		I.setChild(0,J);
		J.setChild(0,K);
		K.setChild(0,L);
		L.setChild(0,M);
		M.setChild(0,N);
		N.setChild(0,O);
		O.setChild(0,P);
		
		B.setParent(A);
		C.setParent(B);
		D.setParent(C);
		E.setParent(D);
		F.setParent(E);
		G.setParent(F);
		H.setParent(G);
		I.setParent(H);
		J.setParent(I);
		K.setParent(J);
		L.setParent(K);
		M.setParent(L);
		N.setParent(M);
		O.setParent(N);
		P.setParent(O);
		
		ProbabilisticMultiStreamOperator Aop = new ProbabilisticMultiStreamOperatorImp(A);
		ProbabilisticMultiStreamOperator Bop = new ProbabilisticMultiStreamOperatorImp(B);
		ProbabilisticMultiStreamOperator Cop = new ProbabilisticMultiStreamOperatorImp(C);
		ProbabilisticMultiStreamOperator Dop = new ProbabilisticMultiStreamOperatorImp(D);
		ProbabilisticMultiStreamOperator Eop = new ProbabilisticMultiStreamOperatorImp(E);
		ProbabilisticMultiStreamOperator Fop = new ProbabilisticMultiStreamOperatorImp(F);
		ProbabilisticMultiStreamOperator Gop = new ProbabilisticMultiStreamOperatorImp(G);
		ProbabilisticMultiStreamOperator Hop = new ProbabilisticMultiStreamOperatorImp(H);
		ProbabilisticMultiStreamOperator Iop = new ProbabilisticMultiStreamOperatorImp(I);
		ProbabilisticMultiStreamOperator Jop = new ProbabilisticMultiStreamOperatorImp(J);
		ProbabilisticMultiStreamOperator Kop = new ProbabilisticMultiStreamOperatorImp(K);
		ProbabilisticMultiStreamOperator Lop = new ProbabilisticMultiStreamOperatorImp(L);
		ProbabilisticMultiStreamOperator Mop = new ProbabilisticMultiStreamOperatorImp(M);
		ProbabilisticMultiStreamOperator Nop = new ProbabilisticMultiStreamOperatorImp(N);
		ProbabilisticMultiStreamOperator Oop = new ProbabilisticMultiStreamOperatorImp(O);
		ProbabilisticMultiStreamOperator Pop = new ProbabilisticMultiStreamOperatorImp(P);
		
		return tree6;
	}
}