package edu.wpi.cs.dsrg.xmldb.xat.component.gui.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import diva.graph.JGraph;
import diva.graph.GraphController;
import diva.graph.GraphModel;
import diva.graph.GraphPane;
import diva.graph.basic.BasicGraphController;
import diva.graph.basic.BasicLayoutTarget;
import diva.graph.layout.GlobalLayout;
import diva.graph.layout.LevelLayout;

import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;

/** The <code>GraphManager</code> manages open tree graphs and the drawing of the
 * graphs to the tabbed pane. It manages the tabbed pane for tree graphs and the
 * tabbed pane for statistic tables.
 * 
 * @author Maylene Waltz
 */
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.component.gui.CAPE_GUI;

public class GraphManager{
	
	/** The CAPE_GUI */
	private CAPE_GUI _gui;
	
	/** An <code>ArrayList</code> of the open<code>JGraph</code>s. */
	private ArrayList _graphs;
	
	/** The tabbed pane to display the trees as graphs. Each tab contains a graph of a tree. */
	private JTabbedPaneCAPE _graphTabbedPane;
	
	/** The tabbed pane to display statistic tables. */
	private JTabbedPaneCAPE _statsTabbedPane;
	
	/** Constructor initializes the <code>ArrayList</code> that holds the open <code>JGraph</code>s and
	 * associates the given <code>gui</code> with this <code>GraphManager</code>
	 * @param gui the <code>CAPE_GUI</code> to associate with this <code>GraphManager</code> */
	public GraphManager(CAPE_GUI gui){
		_gui = gui;
		_graphs = new ArrayList();
	}
	
	/** Add the given <code>graph</code> to the <code>ArrayList</code> of open graphs.
	 * @param the graph to add to known open graphs */
	public void addGraph(JGraph graph){
		_graphs.add(graph);
	}
	
	/** Draws the given <code>model</code> as a new <code>JGraph</code> and places the graph
	 * in the graph tabbed pane. The given <code>queryObject</code> is associated with the
	 * tab of this tree.
	 * @param model the graph model of the tree
	 * @param queryObject the query object to associate with the tab of this tree */
	public void drawModel(GraphModel model, XATQueryObject queryObject){
		JGraph graph = new JGraph(new GraphPane(new BasicGraphController(_gui), model));	
		graph.getGraphPane().getGraphController().setStatsTabbedPane(_statsTabbedPane);
		_graphs.add(graph);
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(graph);
		String title = new String("");
		if(queryObject instanceof XATTree) title = new String("T"+((XATTree)queryObject).getProperty("TREE_ID"));
		_graphTabbedPane.addTab(title, queryObject, panel, "QUERY");
	}
	
	/** Returns the <code>JGraph</code> located at the given <code>index</code>.
	 * @return the graph at the given <code>index</code> */
	public JGraph getGraph(int index){
		return (JGraph)_graphs.get(index);
	}
	
	/** Returns an <code>Iterator</code> of all the open graphs.
	 * @return an <code>Iterator</code> of all the open graphs */
	public Iterator getGraphs(){
		return _graphs.iterator();
	}
	
	/** Returns the tabbed pane that holds the open graphs.
	 * @return the tabbed pane that holds the tree graphs */
	public JTabbedPaneCAPE getGraphTabbedPane(){
		return _graphTabbedPane;
	}
	
	/** Returns the tabbed pane that holds the statistic tables.
	 * @return the tabbed pane that holds the statistic tables */
	public JTabbedPaneCAPE getStatsTabbedPane(){
		return _statsTabbedPane;
	}
	
	/** Creates an empty <code>JTabbedPaneCAPE</code>.
	 * @return an empty tabbed pane */
	protected java.awt.Component initializeTabbedPaneRaindrop(){
		return new JTabbedPaneCAPE(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT, _gui);
	}
	
	/** Redraws the given <code>graph</code> using the <code>LevelLayout</code> in the diva package. 
	 * @param graph the graph to be redrawn */
	public void redrawGraph(JGraph graph){
		GraphController graphController = graph.getGraphPane().getGraphController();
		BasicLayoutTarget target = new BasicLayoutTarget(graphController);
		GlobalLayout graphLayout = new LevelLayout(target);
		graphLayout.layout(graph.getGraphPane().getGraphModel().getRoot());
	}
	
	/** Redraws the graph at the given <code>index</code>.
	 * @param the index of the graph to be redrawn */
	public void redrawGraph(int index){
		redrawGraph((JGraph)_graphs.get(index));
	}
	
	/** Removes the given <code>graph</code> from the <code>ArrayList</code> of open graphs.
	 * @param the graph to be removed from the known open graphs */
	public void removeGraph(JGraph graph){
		_graphs.remove(graph);
	}
	
	/** Removes the graph at the given <code>index</code>.
	 * @param the index of the graph to be removed from the known open graphs */
	public void removeGraph(int index){
		_graphs.remove(index);
	}
	
	/** Sets the graph tabbed pane to the given <code>graphTabbedPane</code>.
	 * @param a tabbed pane to be used to display tree graph(s) */
	public void setGraphTabbedPane(JTabbedPaneCAPE graphTabbedPane){
		_graphTabbedPane = graphTabbedPane;
	}
	
	/** Sets the statistics tabbed pane to the given <code>statsTabbedPane</code>.
	 * @param a tabbed pane to be used to display statistic table(s) */
	public void setStatsTabbedPane(JTabbedPaneCAPE statsTabbedPane){
		_statsTabbedPane = statsTabbedPane;
	}
}