package edu.wpi.cs.dsrg.xmldb.xat.component.gui.utils;

import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableColumn;

import edu.wpi.cs.dsrg.xmldb.xat.component.gui.CAPE_GUI;

/** The <code>TableMaker</code> handles making customized tables for specific query objects.
 * It makes the table and places the table in a scrollable pane.
 * Table properties such as column width and auto-resize mode are set here.
 * Scroll pane properties such as the unit of incrememnt are set here as well.
 * 
 * @author Maylene Waltz
 */
public class TableMaker{
	
	/** The <code>CAPE_GUI</code>. */
	private CAPE_GUI _gui;
	
	/** The table. */
	private JTable _table;
	
	/** Constructor associates the given <code>gui</code> with this table maker. */
	public TableMaker(CAPE_GUI gui){
		_gui = gui;
	}
	
	/** Returns this <code>TableMaker</code>'s table. */
	public JTable getTable(){
		return _table;
	}
	
	/** Makes the table for the given <code>queryObject</code> 
	 * and sets table properties such as:
	 * <ul>
	 * <li>column width and
	 * <li>auto-resize mode.
	 * </ul><p>
	 * The table is placed inside a customized <code>JScrollPane</code>.
	 * @param queryObject the query object that this table is for. For statistics tables
	 * this parameter is needed for gathering statistics.
	 * @return the <code>JScrollPane</code> that contains the table.
	 */
	public JScrollPane makeTable(Object queryObject){
		StatsTableModel statsTableModel = new StatsTableModel(_gui);
		JTable table = new JTable(statsTableModel.update(queryObject), statsTableModel.getColumnHeaders());
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		TableColumnModel tcm = table.getColumnModel();
		TableColumn column = null;
		for(int i=0; i<tcm.getColumnCount(); i++){
			column = tcm.getColumn(i);
			if(i == 0){
				column.setPreferredWidth(280);
			}else{
				column.setPreferredWidth(80);
			}
		}
		JScrollPane scrollPane = new JScrollPane(table);
		scrollPane.setWheelScrollingEnabled(true);
		scrollPane.getHorizontalScrollBar().setUnitIncrement(80);
		return scrollPane;
	}
}