package edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatestructure;

import java.io.Serializable;
import java.util.ArrayList;

import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.ComparableTupleValues.KeyDouble;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.ComparableTupleValues.KeyInt;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.ComparableTupleValues.KeyInterface;

/**
 * Describes the structure of predicates. Window and join predicates are made up of 2 PredicatePart.
 * Abstract predicates are made up of 1 PredicatePart and a constant.
 * Creation date: (1/6/2003 11:04:03 PM)
 * @author: Asima Silva
 */
public class PredicatePart implements Serializable{
	protected ArrayList streamName;
	protected java.lang.String column;
	protected KeyInterface factor;
	protected KeyInterface add;
	protected KeyInterface sub;
	protected int columnID;
/**
 * PredicatePart constructor comment.
 */
public PredicatePart() {
	super();
	column = "";
	factor = new KeyInt(1);
	sub = new KeyInt(0);
	add = new KeyInt(0);
}
/**
 * Insert the method's description here.
 * Creation date: (1/30/2003 11:13:38 PM)
 * @param streamName ArrayList
 * @param column java.lang.String
 */
public PredicatePart(ArrayList streamName, String column) {
	this.setStreamName(streamName);
	this.setColumn(column);
	factor = new KeyDouble(1);
	add = new KeyDouble(0);
	sub = new KeyDouble(0);
	}
/**
 * Insert the method's description here.
 * Creation date: (2/11/2003 12:33:00 PM)
 * @param nstreamName ArrayList
 * @param ncolumnName java.lang.String
 * @param ncolumnID int
 * @param nfactor double
 * @param nadd double
 * @param nsub double
 */
public PredicatePart(ArrayList nstreamName, String ncolumnName, int ncolumnID, double nfactor, double nadd, double nsub) {
	this.setStreamName(nstreamName);
	this.setColumn(ncolumnName);
	this.setColumnID(ncolumnID);
	this.setFactor(new KeyDouble(nfactor));
	this.setAdd(new KeyDouble(nadd));
	this.setSub(new KeyDouble(nsub));
	}
/**
 * Compares two objects for equality. Returns a boolean that indicates
 * whether this object is equivalent to the specified object. This method
 * is used when an object is stored in a hashtable.
 * @param obj the Object to compare with
 * @return true if these Objects are equal; false otherwise.
 * @see java.util.Hashtable
 */
public boolean equals(Object obj) {
    // Insert code to compare the receiver and obj here.
    // This implementation forwards the message to super.  You may replace or supplement this.
    // NOTE: obj might be an instance of any class
    //return super.equals(obj);

    PredicatePart p = (PredicatePart) obj;
    if (p.getAdd().equals(this.getAdd())
        && p.getColumn() == this.getColumn()
        && p.getFactor().equals(this.getFactor())
        && this.getStreamName() == getStreamName()
        && p.getSub().equals(this.getSub()))
        return true;
    else
        return false;
}
/**
 * Get the addition factor of this predicate.
 * Creation date: (1/8/2003 11:52:51 PM)
 * @return KeyInterface
 */
public KeyInterface getAdd() {
	return add;
}
/**
 * Returns the column name of the predicate part
 * Creation date: (1/6/2003 11:11:18 PM)
 * @return java.lang.String
 */
public java.lang.String getColumn() {
	return column;
}
/**
 * Insert the method's description here.
 * Creation date: (2/9/2003 10:27:07 PM)
 * @return int
 */
public int getColumnID() {
	return columnID;
}
/**
 * Returns the mulitplicative factor of the predicate part.
 * Creation date: (1/8/2003 11:52:27 PM)
 * @return keyInterface
 */
public KeyInterface getFactor() {
	return factor;
}
/**
 * Returns the stream name of the predicate part.
 * Creation date: (1/6/2003 11:09:13 PM)
 * @return ArrayList
 */
public ArrayList getStreamName() {
	return streamName;
}
/**
 * Returns the subtractive factor of the predicate part.
 * Creation date: (1/8/2003 11:53:07 PM)
 * @return KeyInterface
 */
public KeyInterface getSub() {
	return sub;
}
/**
 * Sets the additive factor of the predicate part.
 * Creation date: (1/8/2003 11:52:51 PM)
 * @param newAdd KeyInterface
 */
public void setAdd(KeyInterface newAdd) {
	add = newAdd;
}
/**
 * Sets the column name of the predicate part's stream.
 * Creation date: (1/6/2003 11:11:18 PM)
 * @param newColumn java.lang.String
 */
public void setColumn(java.lang.String newColumn) {
	column = newColumn;
}
/**
 * Insert the method's description here.
 * Creation date: (2/9/2003 10:27:07 PM)
 * @param newID int
 */
public void setColumnID(int newID) {
	columnID = newID;
}
/**
 * Sets the multiplicative factor of the predicate part.
 * Creation date: (1/8/2003 11:52:27 PM)
 * @param newFactor int
 */
public void setFactor(KeyInterface newFactor) {
	factor = newFactor;
}
/**
 * Sets the stream name of the predicate part's stream.
 * Creation date: (1/6/2003 11:09:13 PM)
 * @param newStreamName ArrayList
 */
public void setStreamName(ArrayList newStreamName) {
	streamName = newStreamName;
}
/**
 * Sets the subtractive factor of the predicate part.
 * Creation date: (1/8/2003 11:53:07 PM)
 * @param newSub KeyInterface
 */
public void setSub(KeyInterface newSub) {
	sub = newSub;
}
/**
 * Returns a String that represents the value of this object.
 * @return a string representation of the receiver
 */
public String toString() {
	// Insert code to print the receiver here.
	// This implementation forwards the message to super. You may replace or supplement this.
	String pp = this.getStreamName() +"."+ this.getColumn()+ "*" + this.getFactor() + "+"+ this.getAdd()+"-"+this.getSub();
	return pp;
}
}
