package edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.algorithms;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.basics.JoinState;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperatorImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValueImp;

/**
 * Metadata-aware join operator. If the join cardinality is 1:N,
 * then the left stream is 1, and the right stream is N.
 * Creation date: (1/22/2003 9:26:09 AM)
 * @author: Luping Ding
 */
public abstract class JoinOperatorImp extends XATMultiQueueStreamOperatorImp 
		implements JoinOperator {
	/** indication. */
	public static final int LEFT = 0;
	public static final int RIGHT = 1;
	/** join algorithm. */
	public static final int HASH = 0;
	public static final int NLOOP = 1;
	/** punctuation type. */
	public static final int SINGLEVALUE = 0;
	public static final int RANGE = 1;
	/** purge rule. */
	public static final int EAGERPURGE = 0;
	public static final int LAZYPURGE = 0;
	/** further optimization of PWJoin. */
	public static final int DIRECTDROP = 0;
	public static final int PROBEANDDROP = 1;
	public static final int INSERT = 2;
	
	protected JoinState leftSynop;
	protected JoinState rightSynop;
	protected long leftWindow;
	protected long rightWindow;
		
	protected int leftPos; /** Position of join attribute of left stream. */
	protected int rightPos; /** Position of join attribute of right stream. */
	protected int outPos; 
	
	protected int hashSize; /** The number of hash buckets. */
	protected long bucketSize;
	
	/** number of output tuples. */
	protected long numOutTuples = 0;
	/** number of output punctuations. */
	protected long numOutPuncts = 0;
	/** maximum number of tuples allowed in both states. */
	protected long maxNumTuplesInState = 1000;
	/** maximum number of output tuples before Join stops. */
	protected long maxNumOutTuples = 1000;
	
	protected long[] timeOutTuples = new long[10];
	protected long[] timeOutPuncts = new long[10];
	protected long[] timeTuplesInState = new long[10];

	/** keeps track of the current count of punctuations 
	 *  on the left (right) side, call the right (left) 
	 *  state purger every time the purge threshold is reached.
	 *  will be reset every time the threshold is reached
	 */	 
	protected int countLeftPurge = 0;
	protected int countRightPurge = 0;

	/** thresholds for deciding when to purge */
	protected int countLeftPurgeThreshold = 10;
	protected int countRightPurgeThreshold = 10;
	
	/** keeps track of the current count of punctuations 
	 *  on the left (right) side, call the propagator 
	 *  every time the propagate threshold is reached.
	 *  will be reset every time the threshold is reached
	 */	 
	protected int countLeftPropagate = 0;
	protected int countRightPropagate = 0;
	
	 /** thresholds for deciding when to propogate
	  *  Note this parameters indiacate the threshold 
	  *  value after only if the number of punctuations 
	  *  received from both sides equal this value will 
	  *  the punctuations be propagated 
	  */
	protected int countLeftPropagateThreshold = 10;
	protected int countRightPropagateThreshold = 10;
	
	protected boolean eagerWindowPurge;
	
	public JoinOperatorImp(XATNode node) {
		super(node);
	}
	public JoinState getLeftSynop() {
		return leftSynop;
	}
	public JoinState getRightSynop() {
		return rightSynop;
	}
	public long getLeftWindow() {
		return leftWindow;
	}
	public long getRightWindow() {
		return rightWindow;
	}
	public int getLeftPos() {
		return leftPos;
	}
	public int getRightPos() {
		return rightPos;
	}
	public int getOutPos() {
		return outPos;
	}
	public int getHashSize() {
		return hashSize;
	}
	public long getBucketSize() {
		return bucketSize;
	}
	public long getNumTuplesInState() {
		//return rightSynop.getNumTuples();
		return (leftSynop.getNumTuples()+rightSynop.getNumTuples());
	}
	public long getNumOutTuples() {
		return numOutTuples;
	}
	public long getNumOutPuncts() {
		return numOutPuncts;
	}
	public long getMaxNumTuplesInState() {
		return maxNumTuplesInState;
	}
	public long getMaxNumOutTuples() {
		return maxNumOutTuples;
	}
	public int getCountLeftPurge() {
		return countLeftPurge;	
	}
	public int getCountRightPurge() {
		return countRightPurge;
	}
	public int getCountLeftPurgeThreshold() {
		return countLeftPurgeThreshold;
	}
	public int getCountRightPurgeThreshold() {
		return countRightPurgeThreshold;
	}
	public int getCountLeftPropagate() {
		return countLeftPropagate;
	}
	public int getCountRightPropagate() {
		return countRightPropagate;
	}
	public int getCountLeftPropagateThreshold() {
		return countLeftPropagateThreshold;
	}
	public int getCountRightPropagateThreshold() {
		return countRightPropagateThreshold;
	}
	public void setLeftSynop(JoinState lSynop) {
		leftSynop = lSynop;
	}
	public void setRightSynop(JoinState rSynop) {
		rightSynop = rSynop;
	}
	public void setLeftWindow(long leftWin) {
		leftWindow = leftWin;
	}
	public void setRightWindow(long rightWin) {
		rightWindow = rightWin;
	}
	public void setLeftPos(int pos) {
		leftPos = pos;
	}
	public void setRightPos(int pos) {
		rightPos = pos;
	}
	public void setOutPos(int op) {
		outPos = op;
	}
	public void setHashSize(int hs) {
		hashSize = hs;
	}
	public void setBucketSize(long bs) {
		bucketSize = bs;
	}
	public void setMaxNumTuplesInState(long mntis) {
		maxNumTuplesInState = mntis;
	}
	public void setMaxNumOutTuples(long mnot) {
		maxNumOutTuples = mnot;
	}
	public void setCountLeftPurge(int clp) {
		countLeftPurge = clp;
	}
	public void setCountRightPurge(int crp) {
		countRightPurge = crp;
	}
	public void setCountLeftPurgeThreshold(int i) {
		countLeftPurgeThreshold = i;
	}
	public void setCountRightPurgeThreshold(int i) {
		countRightPurgeThreshold = i;
	}
	public void setCountLeftPropagate(int clp) {
		countLeftPropagate = clp;
	}
	public void setCountRightPropagate(int crp) {
		countRightPropagate = crp;
	}
	public void setCountLeftPropagateThreshold(int leftThreshold) {
		countLeftPropagateThreshold = leftThreshold;
	}
	public void setCountRightPropagateThreshold(int rightThreshold) {
		countRightPropagateThreshold = rightThreshold;
	}
	public void setEagerWindowPurge(boolean eagerWinPurge) {
		eagerWindowPurge = eagerWinPurge;
	}
	public void increNumOutTuples(long delta) {
		numOutTuples += delta;
	}
	public void decreNumOutTuples(long delta) {
		numOutTuples -= delta;
	}
	public void increNumOutPuncts(long delta) {
		numOutPuncts += delta;
	}
	public void decreNumOutPuncts(long delta) {
		numOutPuncts -= delta;
	}
	public void increCountLeftPurge() {
		countLeftPurge ++;
	}
	public void increCountRightPurge() {
		countRightPurge ++;
	}
	public void increCountLeftPropagate() {
		countLeftPropagate ++;
	}
	public void increCountRightPropagate() {
		countRightPropagate ++;
	}
	public boolean isStateFull(){
		return (getNumTuplesInState() >= maxNumTuplesInState);
	}
	public boolean ifStop() {
		return (numOutTuples >= maxNumOutTuples);
	}
	public boolean isStreamEmpty() {
		boolean empty = false;	
		if (this.getLeftInputQueue().getQueueSize(this) == 0 
			&& this.getRightInputQueue().getQueueSize(this) == 0) {		
			empty = true;							
		}
		return(empty);
	}
	public XATTuple joinTuples(XATTuple lTuple, XATTuple rTuple) {
		int lLen = lTuple.getLength();
		int rLen = rTuple.getLength();
		XATTuple xatTuple = new XATTupleImp();
	    
		for (int i=0; i<lLen; i++) {
		 xatTuple.addCell(new XATDataValueImp(lTuple.getCell(i)));
		}
		for (int i = 0; i < rLen; i++) {        
		xatTuple.addCell(new XATDataValueImp(rTuple.getCell(i)));
		}
    
		return(xatTuple);
	}

	public Object hash(Object data) {
		return new Integer(((Integer) data).intValue() % hashSize);
	}
	public abstract boolean run();
	
	protected void setDefaultStatistics() {
		super.setDefaultStatistics();
		addStatistic("NUMBER_OF_TUPLES_OUTPUTTED_BY_JOIN");
		addStatistic("NUMBER_OF_PUNCTS_OUTPUTTED_BY_JOIN");
		addStatistic("NUMBER_OF_TUPLES_IN_JOIN_STATE");
	}
 	
}
