package edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.algorithms;

import edu.wpi.cs.dsrg.xmldb.xat.common.punctuations.Punct;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStampException;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.algorithms.JoinOperatorImp;

/**
 * Metadata-aware join operator. If the join cardinality is M:N,
 * then the left stream is M, and the right stream is N.
 * Creation date: (1/22/2003 9:26:09 AM)
 * @author: Luping Ding
 */
public class PWJoinNNSingle extends JoinOperatorImp {
	public PWJoinNNSingle(XATNode node) {
		super(node);
	}
	public boolean run() {
		XATTuple newObj = null;
		int indication = 0;
		long queueEmptyTime = 0;
		int howMany = 600;
		int tupleCount = 0;
		
		while (true) {
			if (tupleCount == howMany) {
				return true;
			}
			try {
				if (this.getLeftInputQueue().getQueueSize(this) == 0) {
					newObj = this.getRightInputQueue().dequeue(this);
					indication = JoinOperatorImp.RIGHT;
				} else if (this.getRightInputQueue().getQueueSize(this) == 0) {
					newObj = this.getLeftInputQueue().dequeue(this);
					indication = JoinOperatorImp.LEFT;
				} else {
					long leftTimeStamp = 0;
					long rightTimeStamp = 0;
			
					leftTimeStamp =
						this.getLeftInputQueue().peek(this).getTimeStamp().getMinTimeStamp();
					rightTimeStamp = 
						this.getRightInputQueue().peek(this).getTimeStamp().getMinTimeStamp();
							
					if(leftTimeStamp < rightTimeStamp) {
						newObj = this.getLeftInputQueue().dequeue(this);
						indication = JoinOperatorImp.LEFT;
					} else {
						newObj = this.getRightInputQueue().dequeue(this);
						indication = JoinOperatorImp.RIGHT;
					}					
				}
			} catch(XATQueueException e1) {
				if (queueEmptyTime == 0) {
					queueEmptyTime = System.currentTimeMillis();
				} else {
					if (System.currentTimeMillis() - queueEmptyTime > 1000) {
						return true;
					}
				}
				continue;
			} catch(XATTimeStampException e2) {
				e2.printStackTrace();
			}
			
			if (indication == JoinOperatorImp.LEFT) {
				Object value = newObj.getCell(leftPos).getValue();
				if (value instanceof Punct) {
					Object punctVal = ((Punct) value).getValue();
					Punct rpe = (Punct)rightSynop.getPunctSet().get(punctVal);
					if (rpe != null) {
						//propagate this punctuation
						numOutPuncts ++;
						rightSynop.removePunct(punctVal);
					} else {
						leftSynop.addPunct(punctVal, (Punct) value);
					}
					rightSynop.purgeByPunct(punctVal);
				} else {
					tupleCount ++;
					try {
						leftSynop.setLatestTimestamp(newObj.getTimeStamp().getMinTimeStamp());
					} catch(Exception e) {
						e.printStackTrace();
					}
					rightSynop.probe(value, newObj);
					if (!rightSynop.ifOnFlyDropByPunct(value)) {
						leftSynop.addTuple(value, newObj);
					}
				}
			} else {
				Object value = newObj.getCell(rightPos).getValue();
				if (value instanceof Punct) {
					Object punctVal = ((Punct) value).getValue();
					Punct lpe = (Punct)leftSynop.getPunctSet().get(punctVal);
					if (lpe != null) {
						//propagate this punctuation
						numOutPuncts ++;
						leftSynop.removePunct(punctVal);
					} else {
						rightSynop.addPunct(punctVal, (Punct) value);
					}
					leftSynop.purgeByPunct(punctVal);
				} else {
					tupleCount ++;
					try {
						rightSynop.setLatestTimestamp(newObj.getTimeStamp().getMinTimeStamp());
					} catch(Exception e) {
						e.printStackTrace();
					}
					leftSynop.probe(value, newObj);
					if (!leftSynop.ifOnFlyDropByPunct(value)) {
						rightSynop.addTuple(value, newObj);
					}
				}
			}
		}
	}	
	
	public void init() {}
	public String toToolTipString() {
		return new String();
	}		

}
