package edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.basics;

import java.util.ArrayList;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;

/**
 * The hash bucket of the hash table in synopsis
 * Assume data in hash bucket is ordered by timestamp
 * Creation date: (3/11/2003 1:19:47 PM)
 * @author: Luping Ding
 */
public class HashBucket {
	HashState hashState;
	//the tuples contained by this bucket
	ArrayList tupleList;
	int sizeOfTupleList;
	long recentProbeTime;
	
	/**
	 * Bucket constructor comment.
	 */
	public HashBucket(HashState hashState) {
		super();
		this.hashState = hashState;
		tupleList = new ArrayList();
		sizeOfTupleList = 0;
		recentProbeTime = System.currentTimeMillis();
	}
	public XATTuple getLastTuple() {
		return (XATTuple) tupleList.get(sizeOfTupleList-1);
	}
	public void addTuple(XATTuple t) {		
		tupleList.add(t);
		sizeOfTupleList ++;
		//hashState.increNumTuples();
	}
	
	/**
	 * Returns the size of the bucket in number of tuples
	 * @return
	 */
	public int sizeOfTupleList() {
		return sizeOfTupleList;
	}

	/**
	 * Returns the tuple at the specified index
	 * @param index
	 * @return
	 */
	public XATTuple getTuple(int index) {
		return (XATTuple) (tupleList.get(index));
	}

	public long getRecentProbeTime() {
		return recentProbeTime;
	}
	/**
	 * Removes the tuple from the bucket at the specified index posistion
	 * @param index
	 * @return
	 */
	public XATTuple removeTuple(int index) {
		sizeOfTupleList --;
		hashState.decreNumTuples();
		return (XATTuple) (tupleList.remove(index));
	}

	public XATTupleImp[] toArray() {
		return (
			(XATTupleImp[]) (this
				.tupleList
				.toArray(new XATTupleImp[0])));
	}

	/**
	 * Deletes the contents of the bucket
	 */
	public void clear() {
		tupleList.clear();
		sizeOfTupleList = 0;
	}
	
	public void setRecentProbeTime(long recentProbeTime) {
		this.recentProbeTime = recentProbeTime;
	}
}
