package edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.basics;

import java.util.HashMap;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.algorithms.JoinOperator;

/**
 * 
 * @author Luping Ding
 * Creates a new Synopsis the synopsis maintains the state of join
 * There is one instance of each synopsis each for the right hand side and
 * the left hand side.
 */

public class HashState extends JoinStateImp {
	protected HashMap state;
	
	/**
	 * Creates a new instance of the synopsis
	 * The last parameter is the output stream used by the
	 * statistics gatherer to output data
	 * @param n
	 * @param sName
	 * @param os
	 */
	public HashState() {
		super();
		state = new HashMap();
	}
	
	public HashState(JoinOperator op) {
		super(op);
		state = new HashMap(op.getHashSize());
	}

	public void addTuple(Object value, XATTuple tuple){
		//Note this sets the time stamp in the time stamp field for the MJoinTuple and not in XATTuple
		//((XATMJoinTuple)tuple).setTimeStamp(System.currentTimeMillis());
		Object key = joinOp.hash(value);
		Object bucket = state.get(key);
		if (bucket != null) {
			((HashBucket) bucket).addTuple(tuple);		
		} else {
			HashBucket b = new HashBucket(this);
			b.addTuple(tuple);
			state.put(key, b);
		}
		this.increNumTuples();
	}
	
	public void probe(Object value, XATTuple newTuple) {
		Object key = joinOp.hash(value);
		Object tBucket = state.get(key);
				
		if (tBucket == null) {
			return;
		}
		HashBucket bucket = (HashBucket) tBucket;
		int vLen = bucket.sizeOfTupleList();
		for (int cursor = 0; cursor < vLen; cursor++) {
			XATTuple t = (XATTuple) bucket.getTuple(cursor);
			if (t.getCell(joinPos).getValue().equals(value)) {
				this.joinOp.increNumOutTuples(1);
				//System.out.println("Output: "+value);
				/*
				if (indication == JoinOperatorImp.LEFT) {
					XATTuple xt = joinOp.joinTuples(t, newTuple);
					joinOp.getOutputQueue().enqueue(xt, this);
				} else {
					XATTuple xt = joinOp.joinTuples(newTuple, t);
					joinOp.getOutputQueue().enqueue(xt, this);
				}
				*/
			}
		}
	}

	public void purgeByPunct(Object punctVal) {
		//System.out.println("Punct Value: "+punctVal);

		Object partitionNumber = joinOp.hash(punctVal);
		Object ro = state.get(partitionNumber);
		if (ro != null) {
			HashBucket bucket = (HashBucket) ro;
			int tCursor = 0;
			while (tCursor < bucket.sizeOfTupleList()) {
				XATTuple t = bucket.getTuple(tCursor);
				Object joinValue = t.getCell(joinPos).getValue(); 
				if (joinValue.equals(punctVal)) {
					bucket.removeTuple(tCursor);
					//System.out.println("Purge: "+joinValue);
				} else {
					tCursor++;
				}
			}
		}
	}

	public void optimizedWindowProbe(Object value, XATTuple newTuple) {
	}

	public void setLastTuple(Object value) {
		Object key = joinOp.hash(value);
		Object tBucket = state.get(key);
				
		if (tBucket == null) {
			return;
		}
		HashBucket bucket = (HashBucket) tBucket;
		int vLen = bucket.sizeOfTupleList();
		for (int cursor = vLen-1; cursor >= 0; cursor--) {
			XATTuple t = (XATTuple) bucket.getTuple(cursor);
			if (t.getCell(joinPos).getValue().equals(value)) {
				t.setIfLastTuple(true);
				break;
			}
		}
	}
	
}
