package edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.basics;

import edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.algorithms.JoinOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.pjoin.algorithms.JoinOperatorImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.punctuations.Punct;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import java.util.HashMap;

/**
 * 
 * @author Luping Ding
 * Creates a new Synopsis the synopsis maintains the state of join
 * There is one instance of each synopsis each for the right hand side and
 * the left hand side.
 */

public abstract class JoinStateImp implements JoinState {
	/** The join node. */
	protected JoinOperator joinOp;
	/** The position of join attribute in the schema. */
	protected int joinPos;
	/** The list containing all the arrived but unpropagated punctuations. */
	protected HashMap punctSet;
	
	protected long window;
	
	protected int numTuples = 0;
	protected int memThreshold;
	
	protected long latestTimestamp;
	protected JoinState oppositeState;
	protected int indication;
	
	/**
	 * Creates a new instance of the synopsis
	 * The last parameter is the output stream used by the
	 * statistics gatherer to output data
	 * @param n
	 * @param sName
	 * @param os
	 */
	public JoinStateImp() {
		punctSet = new HashMap();
		numTuples = 0;
	}
	public JoinStateImp(JoinOperator op) {
		joinOp = op;
		punctSet = new HashMap();
		numTuples = 0;
	}
	
	public JoinOperator getOperator() {
		return joinOp;
	}
	public int getJoinPos() {
		return joinPos;
	}
	public HashMap getPunctSet() {
		return punctSet;
	}
	public long getWindow() {
		return window;
	}
	public int getNumTuples() {
		return numTuples;
	}
	public int getMemThreshold() {
		return memThreshold;
	}
	public long getLatestTimestamp() {
		return latestTimestamp;
	}
	public JoinState getOppositeState() {
		return oppositeState;
	}
	public int getIndication() {
		return indication;
	}
	public void setOperator(JoinOperator op) {
		joinOp = op;
	}
	public void setJoinPos(int jPos) {
		joinPos = jPos;
	}
	public void setWindow(long window) {
		this.window = window;
	}
	public void setMemThreshold(int memThreshold) {
		this.memThreshold = memThreshold;
	}
	public void setLatestTimestamp(long latestTimestamp) {
		this.latestTimestamp = latestTimestamp;
	}
	public void setOppositeState(JoinState js) {
		oppositeState = js;
	}
	public void setIndication(int indication) {
		this.indication = indication;
	}
	public abstract void addTuple(Object value, XATTuple tuple);
	
	public void addPunct(Object key, Punct punct) {		
		punctSet.put(key, punct);
	}
	public Punct getPunct(Object key) {
		return (Punct)punctSet.get(key);
	}
	public void removePunct(Object punctVal) {
		punctSet.remove(punctVal);
	}
	public void increNumTuples() {
		numTuples ++;
	}
	public void decreNumTuples() {
		numTuples --;
	}
	public void decreNumTuples(int delta) {
		numTuples -= delta;
	}
	public boolean isFull() {
		return (numTuples == memThreshold);
	}
	public void removePunct(long pid) {
		punctSet.remove(new Long(pid));
	}
	public abstract void probe(Object value, XATTuple newTuple);
	public abstract void optimizedWindowProbe(Object value, XATTuple newTuple);
	public abstract void purgeByPunct(Object punctVal);
	
	//further optimization for window join
	public boolean ifOnFlyDropByPunct(Object value) {
		if (punctSet.get(value) == null) {
			return false;
		} else {
			return true;
		}
	}
	
	public int ifOnFlyDropByWindow(Object value) {
		Object hashObj = punctSet.get(value);
		if (hashObj == null) {
			return JoinOperatorImp.INSERT;
		} else {
			if (((Punct) hashObj).getValid()) {
				return JoinOperatorImp.PROBEANDDROP;
			} else {
				return JoinOperatorImp.DIRECTDROP;
			}
		}
	}
	
	public abstract void setLastTuple(Object value);
}
