package edu.wpi.cs.dsrg.xmldb.xat.component.operatorexec;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.FOR;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATMultiParentOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Or;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Source;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.SourceImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.CartesianProduct;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.CartesianProductImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.GroupBy;
import edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup.*;
import java.util.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.utils.DeepCopyMaker;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.ExecutionInvalidXATTreeException;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.XATException;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.XATTableColumnNoExistException;
import edu.wpi.cs.dsrg.xmldb.xat.testfiles.StatisticsGatheringVisitor;

/**
 * ExecuteVisitor for MQ
 * Creation date: (5/15/2003 12:45:09 AM)
 * @author: Song Wang
 */
public class DagExecuteVisitor
    extends edu.wpi.cs.dsrg.xmldb.xat.component.operatorexec.ExecuteVisitor {

private Hashtable clonedChildXATTabs = new Hashtable(); 
/**
 * DagExecuteVisitor constructor comment.
 */
public DagExecuteVisitor() {
	super();
}
public DagExecuteVisitor(String db, String username, String password) {
	super(db,username,password);
}
private boolean isShared(DagNode node) {
	if (node.getParentsSize()>1) return true;
	else return false;
}
/**
 * Executes a tree with a given input.
 * @param xtree tree containing a mix of SQL and XML nodes
 * @param input XATTable is the input XAT table.      
 * @return the resulting XATTable after all nodes have been executed
 * @since   1.0            
 */
public XATTable execute(XATTree xtree, XATTable input) {
	XATTree xtClone = (XATTree) xtree.clone();

	// add a temporary source node to this given tree.
	DagNode sourceNode = new DagNodeImp();
	Source sourceTmp = new SourceImp(sourceNode);
	sourceTmp.setType(Source.TEMP);
	sourceNode.setOperator(sourceTmp);
	sourceTmp.setOutput(input);

	xtClone.appendXATNode(sourceNode);

	visit(xtClone);
	return (XATTable) xtClone.getRoot().getOperator().getOutputDataSource();
}

/*
 * Execution for an XAT tree, part of the visitor pattern.     
 * @param tree is the XATTree. 
 * @return Object
 * @since   1.0         
*/
public Object visit(XATDag tree) {
	return visit(tree.getRoot());
}

public Object visit(XATNode xnoObj) {
    if (isDebug())
        System.out.println(xnoObj.getOperator());
    //if this node is already evaluated.
    if (xnoObj.getOperator().getOutputDataSource() != null&&isShared((DagNode)xnoObj)) {
        if (isDebug()) {
            System.out.println("This Node already executed!");
        }
        return null;
    }


    XATNode[] xnaChildren = xnoObj.getChildren();
    for (int i = 0; i < xnaChildren.length; i++) {
        visit(xnaChildren[i]);
    }

    // A vector of cloned output table.
    Vector cloneXATTables = new Vector();
    if (isKeepClone()) {
        // clone the input table for this operator.
        for (int i = 0; i < xnaChildren.length; i++) {
            Object datasource =
                DeepCopyMaker.makeDeepCopy(xnaChildren[i].getOperator().getOutputDataSource());
            if (datasource != null)
                cloneXATTables.add(datasource);
            else
                cloneXATTables.add(new XATTableImp());
        }
    }
    
	for (int i = 0; i < xnaChildren.length; i++) {
		if (isShared((DagNode)xnaChildren[i])){
			Object datasource =
				DeepCopyMaker.makeDeepCopy(xnaChildren[i].getOperator().getOutputDataSource());
			if (datasource != null)
				clonedChildXATTabs.put((DagNode)xnaChildren[i],datasource);
			else
				clonedChildXATTabs.put((DagNode)xnaChildren[i],new XATTableImp());
		}
	}
   
    visit(xnoObj.getOperator());
	for (int i = 0; i < xnaChildren.length; i++) {
		if (isShared((DagNode)xnaChildren[i])){
			xnaChildren[i].getOperator().setOutput(
				(XATDataSource) clonedChildXATTabs.get((DagNode)xnaChildren[i]));
		} 
	}

	if (isKeepClone()) {
		// restore the XATtable for its input table.
		for (int i = 0; i < xnaChildren.length; i++) {
			xnaChildren[i].getOperator().setOutput(
				(XATDataSource) cloneXATTables.elementAt(i));
		}
	}

    if (isDebug() && isShowTable())
        System.out.println(xnoObj.getOperator().getOutputDataSource());
    return null;
}
/**
 * Execution for the GroupBy operator, part of the visitor pattern.         
 * Groupby GB(col1..n, F1..m(col))[s]
 * Input: table s.
 * Output: table s.
 * Logic: Same as SQL.
 * Order Handling: The schema order of the output table is col1..n followed by F1..m(col).
 * The tuple order of the output table is no longer kept.
 * Requirement: col1..n and all the col in the F1..m should be in table s.
 * @param gb is the GroupBy opertor. 
 * @return null
 * @since   1.0              
 */
public Object visit(GroupBy gb) throws ExecutionInvalidXATTreeException {
	// First we need to get the input table.
	XATTable xtbNewGroupby = new XATTableImp();
	XATTable xtbOrgGroupby;
	try {
		xtbOrgGroupby = (XATTable) gb.getPrimaryDataSource();
		Vector groups = new Vector();
		//if the table is empty, the groupby should return a table with one tuple
		//one tuple filled with empty collections.
		if (xtbOrgGroupby.getNumberOfTuples() == 0) {
			XATTable group = xtbOrgGroupby.cloneSchema();
			groups.add(group);
		} else {
			// Now, we break it into groups.
			groups = breakDown(gb.getColumnNames(), xtbOrgGroupby);
		}
		// Get the subqueries.
		XATDag subquery = (XATDag)gb.getXATTree();

		// for each group, we evaluate the subquery.
		for (int i = 0; i < groups.size(); i++) {
			XATTable group = (XATTable) groups.elementAt(i);
			group.setName(xtbOrgGroupby.getName());
			// clone the subquery.
			XATDag subqueryclone = (XATDag) subquery.clone();
			StatisticsGatheringVisitor.addTempTree(subqueryclone);
			XATNode[] leaves = subqueryclone.getLeaves();
			// append this source to the end of each leaf.
			// Note: There suppose to be only one leaf
			DagNode sourceNode = new DagNodeImp();
			Source sourceTmp = new SourceImp(sourceNode);
			sourceTmp.setOutput(group);
			sourceTmp.setType(Source.TEMP);
			sourceNode.setOperator(sourceTmp);

			for (int j = 0; j < leaves.length; j++) {
				XATNode leaf = (XATNode) leaves[j];
				// if the leaf is a source node,
				// then create a cartesian product,
				// put both the source node below it.
				if (leaf.getOperator() instanceof Source) {
					DagNode cpNode = new DagNodeImp();
					CartesianProduct cpOp = new CartesianProductImp(cpNode);
					// insert the catesian there.
					cpNode.addChild(sourceNode);
					sourceNode.setParent(cpNode);
					cpNode.addChild(leaf);
					cpNode.setParent(leaf.getParent());
					leaf.setParent(cpNode);
					cpNode.getParent().deleteChild(leaf);
					cpNode.getParent().addChild(cpNode);
				} else { // if the leaf is not a source operator, then just appened this operator.
					subqueryclone.addNode(sourceNode, leaf);
					sourceNode.setParent(leaf);
				}
			}
			DagExecuteVisitor exec = new DagExecuteVisitor();
			exec.visit(subqueryclone);
			XATTable xtbTmp =
				(XATTable) subqueryclone
					.getRoot()
					.getOperator()
					.getOutputDataSource();
			if (xtbTmp != null) {
				// if the result is not empty, then put it back to the groupby's result.
				if (i == 0) { // if it's the first table,
					// initialize the schema of the xtbNewGroupBy
					xtbNewGroupby = xtbTmp.cloneSchema();
				}
				// append the result to the output table.
				xtbNewGroupby.appendTable(xtbTmp);
				if (isDebug() && isShowTable())
					System.out.println("**" + xtbNewGroupby + "**");
			}
		}
	} catch (XATException e) {
		throw new ExecutionInvalidXATTreeException();
	}

	// Takes the new table and appends the columns after the functions are called on the Original table
	xtbNewGroupby.setName("aftergroupby");
	gb.setOutput(xtbNewGroupby);

	return null;
}
/*
 * Execution for the FOR operator, part of the visitor pattern.     
 * For each tuple, we execute the sub query:
 *  - we create the temp source for the current tuple in the outer query.
 *  - execute the subtree.
 *  - get the result and keep it.
 *  - update the data in the temp source.
 *  - work for the next tuple.
 * @param forop is the FOR opertor. 
 * @return null
 * @since   1.0         
*/
public Object visit(FOR forop) {
	// First we need to get the input table.

	XATTable xtbNewFor = new XATTableImp();
	XATTable xtbOrgFor;
	try {
		xtbOrgFor = (XATTable) forop.getPrimaryDataSource();

		// Get the subqueries.
		XATDag subquery = (XATDag)forop.getXtreFunctions();

		// clone the subquery.
		XATDag subqueryclone = (XATDag) subquery.clone();
		StatisticsGatheringVisitor.addTempTree(subqueryclone);
		DagNode[] leaves = (DagNode[])subqueryclone.getLeaves();

		// append this source to the end of each leaf if the subquery has no other sources, otherwise make the cartesian product of the two sources.
		// Note: There suppose to be only one leaf.
		int iLengthleaves = leaves.length;

		DagNode sourceNode = new DagNodeImp();
		Source sourceTmp = new SourceImp(sourceNode);
		sourceTmp.setType(Source.TEMP);
		sourceNode.setOperator(sourceTmp);

		for (int j = 0; j < iLengthleaves; j++) {
			DagNode leaf = (DagNode) leaves[j];
			// check if the leaf is a source operator.
			if (!(leaf.getOperator() instanceof Source)) {
				subqueryclone.addNode(sourceNode, leaf);
				sourceNode.setParent(leaf);
			} else // it is source operator.
				{
				// add a cartesian product.
				DagNode xtnodeCP = new DagNodeImp();
				CartesianProduct cp = new CartesianProductImp(xtnodeCP);
				xtnodeCP.setOperator(cp);
				//subqueryclone.addNode(xtnodeCP, leaf.getParent());
				//subqueryclone.deleteNode(leaf);
				subqueryclone.replaceNode(leaf,xtnodeCP);
				subqueryclone.addNode(sourceNode, xtnodeCP);
				subqueryclone.addNode(leaf, xtnodeCP);
			}
		}
		DagExecuteVisitor exec = new DagExecuteVisitor();

		// for each tuple, we evaluate the subquery.
		if (xtbOrgFor.getNumberOfTuples() == 0) {
			//fixing for the case: select in outter tree of for.
			//inner tree should not be evaluated
			//here we visit inner tree is only for the schema.

			// create the temporary table for the subquery.
			XATTable xtbSource = xtbOrgFor.cloneSchema();
			xtbSource.setName(xtbOrgFor.getName());
			// make the data ready for the subquery.
			sourceTmp.setOutput(xtbSource);
			// execute the subquery.
			exec.visit(subqueryclone);
			XATTable xtbTmp =
				(XATTable) subqueryclone
					.getRoot()
					.getOperator()
					.getOutputDataSource();
			// initialize the schema of the xtbNewGroupBy
			xtbNewFor = xtbTmp.cloneSchema();
		}

		for (int i = 1; i <= xtbOrgFor.getNumberOfTuples(); i++) {
			// create the temporary table for the subquery.
			XATTable xtbSource = xtbOrgFor.cloneSchema();
			xtbSource.appendTuple(xtbOrgFor.getTuple(i));
			xtbSource.setName(xtbOrgFor.getName());
			// make the data ready for the subquery.
			sourceTmp.setOutput(xtbSource);
			// execute the subquery.
			exec.visit(subqueryclone);
			XATTable xtbTmp =
				(XATTable) subqueryclone
					.getRoot()
					.getOperator()
					.getOutputDataSource();
			if (i == 1) {
				// initialize the schema of the xtbNewGroupBy
				xtbNewFor = xtbTmp.cloneSchema();
			}
			// append the result to the output table.
			 ((XATTableImp) xtbNewFor).appendTable(xtbTmp);
		}

		//Deal with select case of deleting outer for variable binding context.
		// By: Song Wang
		if ((xtbNewFor.getNumberOfTuples() == 0)
			&& (xtbOrgFor.getNumberOfTuples() > 0)) {
			//empty table for this variable.
			XATDag tempTree = new XATDagImp();
			tempTree.setRoot(forop.getNode());
			DagNode[] forleaves = (DagNode[])tempTree.getLeaves();
			for (int k = 0; k < forleaves.length; k++) {
				DagNode forleaf = (DagNode) forleaves[k];
				if ((forleaf.getOperator() instanceof Source)
					&& (((Source) forleaf.getOperator()).getType()
						== Source.TEMP)) {
					Source forsourceTmp = (Source) forleaf.getOperator();
					XATTable forTmpTable =
						(XATTable) forsourceTmp.getOutputDataSource();
					Vector forsourceTmpTuple = forTmpTable.getTuple(1);
					Vector extendedForSource = new Vector();
					for (Iterator it = xtbNewFor.getColNames().iterator();
						it.hasNext();
						) {
						try {
							int l =
								forTmpTable.getColumnPosition(
									(String) it.next());
							extendedForSource.add(
								forsourceTmpTuple.get(l - 1));
						} catch (XATTableColumnNoExistException e) {
							extendedForSource.add(new XATDataValueImp());
						}
					}
					xtbNewFor.appendTuple(extendedForSource);
					break;
				}
			}
		}
	} catch (XATException e) {
		e.printStackTrace(System.err);
	}

	// Takes the new table and appends the columns after the functions are called on the Original table
	forop.setOutput(xtbNewFor);

	return null;
}
public Object visit( XATMultiParentOperator MPO) throws ExecutionInvalidXATTreeException {
	
	XATTable xTabInputMPO = new XATTableImp();
	//	step1: take input XAT table
	xTabInputMPO =(XATTable)MPO.getPrimaryDataSource();
	
	//step2: set input XAT table as output XAT table
	MPO.setOutput(xTabInputMPO);
	
	
	return null;
}

}
