package edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory;

import java.lang.reflect.Method;
/**
 * Insert the type's description here.
 * @author: Brad Pielech
 */
public abstract class OperatorFactory {
	public static final java.lang.String factoryID = "edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory";
/**
 * OperatorFactory constructor comment.
 */
protected OperatorFactory() {
	super();
}
/**
 * <pre>
 * Return the correct factory as specified by a System parameter called
 * "edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory"
 * The property name can be accessed through OperatorFactory.factoryID 
 * if no property is set then an exception is thrown and tree generation cannot continue
 * <pre>
 * @return OperatorFactory a handle to an OperatorFactory that can be used to create an OperatorBuilder
 */
public static OperatorFactory newInstance() throws FactoryException {
	
	String systemProp = System.getProperty(OperatorFactory.factoryID);
    try {
        if (systemProp != null) {
            Class factoryClass = Class.forName(systemProp);
            return (OperatorFactory) factoryClass.newInstance();
        } else {
            throw new FactoryException("ERROR: The edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory system property was not set");
        }
    } catch (ClassNotFoundException e) {
        throw new FactoryException("The specified Factory specified cannot be found");
	} catch (IllegalAccessException e1) {
		/** 
		 * if we cannot access the constructor in factoryClass.newInstance
		 * try to call the method newInstance in that class instead 
		 */
		try {
			Class factoryClass = Class.forName(systemProp);
		    Method newInstanceMethod = factoryClass.getMethod("newInstance", null);
  		  	return (OperatorFactory)newInstanceMethod.invoke(null, null);    	
		} catch (Exception e2) {
			throw new FactoryException(e2.getMessage());
		}		
    } catch (Exception e3) {
        throw new FactoryException(e3.getMessage());
    }
}
/**
 * Creates a new instance of OperatorBuilder that will be used to generate operators
 * The various subclasses of this class will return different OperatorBuilders
 * based on the subclass 
 * 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.OperatorBuilder
 */
public abstract OperatorBuilder newOperatorBuilder();
}
