/*
 * Created on Feb 5, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.optimizer.rewriterules;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObjectImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.WBinaryJoinOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.Stream;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATEndUserApplicationQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATEndUserApplicationQueueImp;
/**
 * @author yaliz
 * 
 * This rewrite rule pushes down join operators with lower selectivity (= higher reduction factor).
 * The input is a query tree and the output is the same tree object with different orders of joins, 
 * depending on the join operator's current selectivity gathered by StatsGatherer.
 * 
 */
public class PushDownJoinRule  extends XATQueryObjectImp implements RewriteRule{
	//XATTree origTree = null;
	XATTree newTree = null;
	boolean changed = false;
	public PushDownJoinRule() {
		super();
	}

	/** 
	 * The current implementation of this rule assumes that the query plan is a tree, 
	 * not a DAG. Need to modify it to support DAG as well.
	 */	
	public boolean ApplyRule(XATTree newTree){
		changed = false;
		//this.origTree = origTree;
		this.newTree = newTree;
		boolean noMoreRound = true;
		while (true) {
			//XATNode origRootNode = origTree.getRoot();
			XATNode newRootNode = newTree.getRoot();
			System.out.println("One round of applying push down rule.");
			OneRound(newRootNode, noMoreRound);
			if(noMoreRound)
				break;
		}
		return changed;
	}
	
	/**
	 * Check to see if the current rule needs to be applied to this tree.
	 * @param tree
	 * @return true if rule need to be applied, this will be followed by a calling of ApplyRule(tree) method.
	 *         false otherwise, then the optimization by this rule is not necessary.
	 */
	public boolean CheckRule(XATTree oldTree) {
		changed = false;
		XATNode oldRootNode = oldTree.getRoot();
		CheckOneRound(oldRootNode);
		return changed;
	}
	
	private void CheckOneRound(XATNode node) {
		if (node.getChildrenSize() == 0) {
			return;
		}
		XATStreamOperator operator = (XATStreamOperator) node.getOperator();
		if (operator instanceof WBinaryJoinOperator) {
			// get operator's performance-factor (which equals reduction factor for now).
			double rootFactor =
				Double.parseDouble(operator.getProperty("PERFORMANCE_FACTOR"));
			XATNode[] childrenNodes = node.getChildren();

			//find the cnode with the smallest reduction factor
			double minFactor = Double.MAX_VALUE;
			int minIndex = 0;
			for (int i = 0; i < childrenNodes.length; i++) {
				XATStreamOperator childOperator =
					(XATStreamOperator) childrenNodes[i].getOperator();
				double crFactor =
					Double.parseDouble(
						childOperator.getProperty("PERFORMANCE_FACTOR"));
				if (crFactor < minFactor) {
					minFactor = crFactor;
					minIndex = i;
				}
			}
			// if the smallest reduction factor of the children is smaller than the root operator
			// switch the root operator with the child that has the smallest reduction factor
			if (minFactor < rootFactor) {
				changed = true;
				return; // if switch is needed, return right away.
			} else {
				for (int j = 0; j < childrenNodes.length; j++) {
					if(changed == false) 
						CheckOneRound(childrenNodes[j]);
					else
						return;
				}
			}
		}
	}
	
	
	
	private void OneRound(XATNode node, boolean noMoreRound) {

			if (node.getChildrenSize() == 0) {
				return;
			}
			else {
				XATStreamOperator operator = (XATStreamOperator) node.getOperator();
				if (operator instanceof WBinaryJoinOperator) {
					// get operator's performance-factor (which equals reduction factor for now).
					double rootFactor = Double.parseDouble(operator.getProperty("PERFORMANCE_FACTOR"));
					XATNode[] childrenNodes = node.getChildren();
					

					//find the cnode with the smallest reduction factor
					double minFactor = Double.MAX_VALUE;
					int minIndex = 0;
					for (int i = 0; i < childrenNodes.length; i++) {
						XATStreamOperator childOperator = (XATStreamOperator) childrenNodes[i].getOperator();
						double crFactor = Double.parseDouble(childOperator.getProperty("PERFORMANCE_FACTOR"));
						if (crFactor < minFactor) {
							minFactor = crFactor;
							minIndex = i;
						}
					}
					// if the smallest reduction factor of the children is smaller than the root operator
					// switch the root operator with the child that has the smallest reduction factor
					if (minFactor < rootFactor) {
						changed = true;
						System.out.println("Switching nodes: "+node+" w/ rf = "+rootFactor
						                   +" and "+childrenNodes[minIndex]+" w/ rf = "+minFactor);
						SwitchNode(node, childrenNodes[minIndex]);
						noMoreRound = false;

						OneRound(node, noMoreRound);
					}
					else {
						for(int j=0; j<childrenNodes.length; j++) {
							OneRound(childrenNodes[j], noMoreRound);
						}
					}
				}
			}
	}
	
	private void SwitchNode(XATNode upperNode, XATNode lowerNode) {
		// two things to each for each node: parent, children
		// four things to change for each operator: input queue, output queue and 
		// state schema for window join operator (run init) again to achieve this goal,
		// this will take care of other things that need to be re-initialized.
		boolean isRightChild = true; // indicates if lowerNode is the right child of upperNode.
		
		XATStreamOperator upper = (XATStreamOperator)upperNode.getOperator();
		XATStreamOperator lower = (XATStreamOperator)lowerNode.getOperator();
		//if(upperNode.getChild(0).equals(lowerNode)) {

		
		/** first change the input and output queues of involved operators */
		if((lower instanceof XATMultiQueueStreamOperator) && (upper instanceof XATMultiQueueStreamOperator)) {
			
			if(((XATMultiQueueStreamOperator)upper).getLeftInputQueue().getQueueOwner().get(0).equals(lower)) {
				isRightChild = false; 
			}
			System.out.println("lower is upper's right child: "+isRightChild);
			
			XATMultiQueueStreamOperator upperOperator = (XATMultiQueueStreamOperator)upper;
			XATMultiQueueStreamOperator lowerOperator = (XATMultiQueueStreamOperator)lower;
			
			XATQueue lowerLeftInputQueue = lowerOperator.getLeftInputQueue();
			Object lowerLeftChildOperator = lowerLeftInputQueue.getQueueOwner().get(0);
			XATQueue lowerRightInputQueue = lowerOperator.getRightInputQueue();
			Object lowerRightChildOperator = lowerRightInputQueue.getQueueOwner().get(0);
			XATQueue lowerOutputQueue = lowerOperator.getOutputQueue();
			XATQueue upperOutputQueue = upperOperator.getOutputQueue();
			XATQueue newUpperOutputQueue = upperOutputQueue;
			//newUpperOutputQueue.removeOwner(upperOperator);
			if(upperOutputQueue instanceof XATEndUserApplicationQueue) {
				newUpperOutputQueue = new XATQueueImp(upperOperator);
				upperOperator.setOutputQueue(newUpperOutputQueue);
			}
			
			if(isRightChild) {
				lowerOperator.replaceInputQueue(lowerLeftInputQueue, newUpperOutputQueue);
				upperOperator.replaceInputQueue(lowerOutputQueue, lowerLeftInputQueue);
			}
			else {
				lowerOperator.replaceInputQueue(lowerRightInputQueue, newUpperOutputQueue);
				upperOperator.replaceInputQueue(lowerOutputQueue, lowerRightInputQueue);
			}
			if ((XATStreamOperator) upperNode.getParent() != null) {
				XATStreamOperator upperParentOperator = (XATStreamOperator) upperNode.getParent().getOperator();
				upperOutputQueue.unregister(upperParentOperator);
				lowerOutputQueue.unregister(upperOperator);
				upperParentOperator.replaceInputQueue(upperOutputQueue, lowerOutputQueue);
			}
			else {
				lowerOperator.setOutputQueue(new XATEndUserApplicationQueueImp(lowerOperator));
			}
			/** then take care of the parent-child relation of involved XATNodes */
			lowerNode.addChild(upperNode);
			upperNode.deleteChild(lowerNode);

			if(isRightChild) {
				if(!(lowerLeftChildOperator instanceof Stream)) {
					lowerNode.deleteChild(((XATOperator)lowerLeftChildOperator).getNode());
					upperNode.addChild(((XATOperator)lowerLeftChildOperator).getNode());
				}
			}
			else {
				if(!(lowerRightChildOperator instanceof Stream)) {
					lowerNode.deleteChild(((XATOperator)lowerRightChildOperator).getNode());
					upperNode.addChild(((XATOperator)lowerRightChildOperator).getNode(), 0);
				}
			}
			XATNode upperParentNode = upperNode.getParent();
			if (upperParentNode != null) {
				int position = upperParentNode.getIndexOf(upperNode);
				upperParentNode.addChild(lowerNode, position);
				upperParentNode.deleteChild(upperNode);
				upperNode.setParent(lowerNode);
			} else {
				lowerNode.setParent(null);
				upperNode.setParent(lowerNode);
				newTree.setRoot(lowerNode);
			}
			
			// as the last step. reinitialize these two operators, 
			// starting from the new lower op (orig upper op), MUST FOLLOW THIS ORDER!
			((WBinaryJoinOperator)upper).reinit();
			((WBinaryJoinOperator)lower).reinit();
		}
		else {
			System.out.println("ERROR: cannot switch nodes, not the correct MultiQueueStreamOperator type!");
		}
	}
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject("PushDownRule",this.getClass().getName(),true);
	}
}