package edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator;

import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.SchemaNameChangeOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATSingleQueueStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.BinBoolExpressionStr;
import edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.BinCOMPExpressionStr;
import edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.ExpressionStr;
import edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.TerminalExpressionStr;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.queryplan.XATJugglerOperator;
import edu.wpi.cs.cs4432.project2.operator.*;


/**
 * This class contains visit methods that will help the FromXMLFileQueryPlanGenerator
 * to parse class variables.  Every operator type that has a class variable should
 * have a visit here that will parse the XML.  This allows each operator to have its
 * own encoding b/c making the encoding generic is too hard and verbose. 
 * @author: Brad Pielech
 */
public class ClassVariableVisitor
	extends edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.TwoParamVisitor
	implements Serializable {
	/** To visit a null object. */
	public Object dispatch() {
		return null;
	}
	/** To visit any object. */
	public Object dispatch(Object obj) {
		return null;
	}
	/** To visit two any objects. */
	public java.lang.Object dispatch(
		java.lang.Object obj1,
		java.lang.Object obj2) {
		return null;
	}
	/**
	 * Parses the expression that is encoded in the XML and returns an Expression
	 * object.  Only simple expressions are supported currently- no Arithmetic expressions
	 * are parsed, only terminal, comparison, and, or expressions are supported.  The root
	 * of the expression tree is returned.  
	 * @return Expression
	 * @param node org.w3c.dom.Node
	 */
	private ExpressionStr parseExpression(Node node) throws Exception {

		/**
		<expression>
		    <exptype id="1" type = "edu.wpi.cs.dsrg.xmldb.xat.common.expression.TerminalExpressionImp" value="$col1" valtype ="columnName" leftid = "" rightid = ""/>
		    <exptype id="2" type = "edu.wpi.cs.dsrg.xmldb.xat.common.expression.TerminalExpressionImp" value="$col3" valtype ="columnName" leftid = "" rightid = ""/>
		    <exptype id="3" type = "edu.wpi.cs.dsrg.xmldb.xat.common.expression.BinCOMPExpressionImp" value="" valtype="4" leftid = "1" rightid = "2"/>
		</expression>
		*/
		Hashtable expTable = new Hashtable();
		NodeList steps = node.getChildNodes();
		ExpressionStr root = null;
		boolean twoterminalseen = false;
		/** iterate through the variables */
		for (int i = 0; i < steps.getLength(); i++) {
			Node next = steps.item(i);
			if (next.getNodeType() == Node.ELEMENT_NODE) {
				NamedNodeMap attrs = next.getAttributes();
				String id = ((Node) attrs.getNamedItem("id")).getNodeValue();
				String leftid =
					((Node) attrs.getNamedItem("leftid")).getNodeValue();
				String rightid =
					((Node) attrs.getNamedItem("rightid")).getNodeValue();
				String valType =
					((Node) attrs.getNamedItem("valtype")).getNodeValue();
				String value =
					((Node) attrs.getNamedItem("value")).getNodeValue();
				/** 
				 * create expression, if it is terminal exp then set the type the type's value 
				 * then store in hashtable
				 * then if it is a bin exp, connect left and right by looking in hashtable			
				*/
				String type =
					((Node) attrs.getNamedItem("type")).getNodeValue();
				Object o1 = Class.forName(type).newInstance();
				expTable.put(id, o1);
				//if we have a terminal expression, then store it in hashtable and set up value
				if (o1 instanceof TerminalExpressionStr) {
					TerminalExpressionStr e1 = (TerminalExpressionStr) o1;

					if (valType.equalsIgnoreCase("columnname")) {
						e1.setValue(
							new edu
								.wpi
								.cs
								.dsrg
								.xmldb
								.xat
								.common
								.operator
								.ColumnNameImp(
								value));
					}
					if (valType.equalsIgnoreCase("double")) {
						e1.setValue(Double.parseDouble(value));
					}
					if (valType.equalsIgnoreCase("string")) {
						e1.setValue(value);
					}
					if (valType.equalsIgnoreCase("boolean")) {
						e1.setValue(Boolean.getBoolean(value));
					}
				} else if (o1 instanceof BinBoolExpressionStr) {
					BinBoolExpressionStr bb = (BinBoolExpressionStr) o1;
					ExpressionStr left = (ExpressionStr) expTable.get(leftid);
					bb.setLeft(left);
					ExpressionStr right = (ExpressionStr) expTable.get(rightid);
					bb.setRight(right);
					/** 
					 * try to figure out how to set the root
					 * if we have 2 terminal expressions as the left and right
					 * and we havent set the root to another Expression with 2 terminals
					 * as left and right, set this to the root
					 *
					 * if there are not 2 terminals as the left and right, set this to the root.
					 * and then don't allow something with 2 terminals to be set as root
					*/
					if ((left instanceof TerminalExpressionStr)
						&& (right instanceof TerminalExpressionStr)) {
						if (!twoterminalseen) {
							root = bb;
							twoterminalseen = true;
						}
					} else {
						root = bb;
						twoterminalseen = true;
					}
					root = bb;

					if (o1 instanceof BinCOMPExpressionStr) {
						BinCOMPExpressionStr bc = (BinCOMPExpressionStr) o1;
						bc.setType(Integer.parseInt(valType));
					}

				}

			}
		}
		return root;
	}

	public Object visit(XATWBinaryJoinOperator op, Node node) {
		NodeList classList = node.getChildNodes();
		for (int i = 0; i < classList.getLength(); i++) {
			Node next = classList.item(i);

			if (next.getNodeType() == Node.ELEMENT_NODE) {
				NamedNodeMap attrs = next.getAttributes();
				Node idN = attrs.getNamedItem("size");
				String id = idN.getNodeValue();

				op.setTimeWindow(Long.parseLong(id));
			}
		}

		return null;
	}

	public Object visit(XATJugglerOperator jug, Node node) {
		System.out.println("Visiting the Juggler Operator");
		NodeList classList = node.getChildNodes();
		String numQueues = null;
		String queries = null;
		String attrPredicates = null;
		String joinPredicates = null;
		String windowPredicates = null;
		for (int i = 0; i < classList.getLength(); i++) {
			Node next = classList.item(i);
			

			if (next.getNodeType() == Node.ELEMENT_NODE) {
	
				NamedNodeMap attrs = next.getAttributes();
				Node idN;
				if ((idN = attrs.getNamedItem("num")) != null) {
					numQueues = idN.getNodeValue();
				}
				if ((idN = attrs.getNamedItem("queries")) != null) {
					queries = idN.getNodeValue();
				}
				if ((idN = attrs.getNamedItem("apred")) != null) {
					attrPredicates = idN.getNodeValue();
				}
				if ((idN = attrs.getNamedItem("wpred")) != null) {
					windowPredicates = idN.getNodeValue();
				}
				if ((idN = attrs.getNamedItem("jpred")) != null) {
					joinPredicates = idN.getNodeValue();
				}
			}

		}
		System.out.println("Initializing( " + numQueues + ", " + queries + ", " + attrPredicates + ", " + windowPredicates + ", " + joinPredicates);
		jug.visitInit(numQueues, queries,attrPredicates, joinPredicates,windowPredicates);

		return null;
	}
	
	public Object visit(XATSingleQueueStreamWindowOperator operator, Node node) {
		long increment = 0;
		long time_window = 0;
		Vector col_pos = new Vector();
		
		NodeList variableList = node.getChildNodes();
		for (int i = 0; i < variableList.getLength(); i++){
			Node next = variableList.item(i);
			if(next.getNodeType() == Node.ELEMENT_NODE){
				NamedNodeMap attrs = next.getAttributes();
				String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
			
				if (name.equals("increment")) {
					long value = Long.parseLong(((Node) attrs.getNamedItem("value")).getNodeValue());
					operator.setIncrement(value);
				} else if (name.equals("time_window")) {
					long value = Long.parseLong(((Node) attrs.getNamedItem("value")).getNodeValue());
					operator.setTimeWindow(value);
				} else if (name.equals("column_pos")) {
					String value = ((Node) attrs.getNamedItem("value")).getNodeValue();
					col_pos.add(new Integer(value));
				}
			}
		}
		
		if (operator instanceof StreamGroupbyWindowOperatorImp) {
			((StreamGroupbyWindowOperatorImp) operator).setGroupAttrPos(((Integer)col_pos.elementAt(0)).intValue());
		} else if (operator instanceof StreamAggregateSumWindowOperatorImp) {
			((StreamAggregateSumWindowOperatorImp) operator).setAggAttrPos(((Integer)col_pos.elementAt(0)).intValue());
		}
		return null;
	}

	public Object visit(XATSingleQueueStreamOperator operator, Node node) {
		Vector col_pos = new Vector();
		
		NodeList variableList = node.getChildNodes();
		for (int i = 0; i < variableList.getLength(); i++){
			Node next = variableList.item(i);
			if(next.getNodeType() == Node.ELEMENT_NODE){
				NamedNodeMap attrs = next.getAttributes();
				String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
			
				if (name.equals("column_pos")) {
					String value = ((Node) attrs.getNamedItem("value")).getNodeValue();
					col_pos.add(new Integer(value));
				}
			}
		}
		
		int[] col_pos_int = new int[col_pos.size()];
		for (int i=0; i<col_pos.size(); i++) {
			col_pos_int[i] = ((Integer) col_pos.get(i)).intValue();
		}
		
		if (operator instanceof StreamProjectOperatorImp) {
			((StreamProjectOperatorImp) operator).setProjAttrPos(col_pos_int);
		}
		return null;
	}

	public Object visit(XATMultiQueueStreamWindowOperator operator, Node node) {
		long increment = 0;
		long time_window = 0;
		
		NodeList variableList = node.getChildNodes();
		for (int i = 0; i < variableList.getLength(); i++){
			Node next = variableList.item(i);
			if(next.getNodeType() == Node.ELEMENT_NODE){
				NamedNodeMap attrs = next.getAttributes();
				String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
			
				if (name.equals("increment")) {
					long value = Long.parseLong(((Node) attrs.getNamedItem("value")).getNodeValue());
					operator.setIncrement(value);
				} else if (name.equals("time_window")) {
					long value = Long.parseLong(((Node) attrs.getNamedItem("value")).getNodeValue());
					operator.setTimeWindow(value);
				}
			}
		}

		return null;
	}

	public Object visit(SchemaNameChangeOperator con, Node node) {
			System.out.println("Visiting the Schema Operator");
			NodeList classList = node.getChildNodes();
			String streamName = null;
			
			for (int i = 0; i < classList.getLength(); i++) {
				Node next = classList.item(i);
			

				if (next.getNodeType() == Node.ELEMENT_NODE) {
	
					NamedNodeMap attrs = next.getAttributes();
					Node idN = attrs.getNamedItem("name");
					if (idN != null) {
						streamName = idN.getNodeValue();
					}
				}

			}

			con.setStreamName(streamName);

			return null;
		}
}
