package edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator;

import java.io.IOException;
import java.lang.reflect.Constructor;
import java.util.Hashtable;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNodeImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.XATNodeDoesNotExistException;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Source;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.SourceImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.WBinaryJoinOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.XATWBinaryJoinOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;

/**
 * Generate an XAT (or dag) from an XML file.  It is probably easiest to
 * work out the query plan on paper and then encode it in the XML.  
 * There are not many assumptions in the format of the XML file, but here they are:
 * <ol>
 * <li> The xml file does not encode both Stream and NonStream operators.  While
 * care has been taken to ensure this will work with both of them, it is not
 * logically correct to inter-twine them in one plan
 * <li> The XML file should not remove any elements even if they are unused.  That is,
 * the operator element should still have the children element even if it is a leaf.  Otherwise,
 * the xml parser will complain
 * </ol>
 *
 * More to come 
 * @author: Brad Pielech
 */
public class FromXMLFileQueryPlanGenerator extends QueryPlanGeneratorImp  {
	private java.util.Hashtable operatorTable;
	private ClassVariableVisitor variableVisitor;
	private java.lang.String dataModel;
/**
 * FromXMLFileQueryPlanGenerator constructor comment.
 */
public FromXMLFileQueryPlanGenerator() {
	super();	
	operatorTable = new Hashtable();
	variableVisitor = new ClassVariableVisitor();
}
/**
 * This will connect the children of the given node to those
 * that were specificed in the XML document.  It iterates through
 * the Node, finds all elements with the id attribute, checks
 * the operator table to see if this dagNode exists.  If it does,
 * they are connected.  If the operator does not exist, then it is created
 * and the parent / child relationship is established.  The operator of the
 * other dag node will be added later 
 * @param children org.w3c.dom.Node
 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
private void addChildren(Node children, DagNode node) {

	NodeList childList = children.getChildNodes();
	for (int i = 0; i < childList.getLength(); i++) {
		Node next = childList.item(i);
		if (next.getNodeType() == Node.ELEMENT_NODE) {
			NamedNodeMap attrs = next.getAttributes();
			Node idN = attrs.getNamedItem("id");
			String id = idN.getNodeValue();
			DagNode child;
			if (operatorTable.containsKey(id)) {
				child = (DagNode) operatorTable.get(id);
			} else {
				child = new DagNodeImp();
				operatorTable.put(id, child);
			}
			node.addChild(child);
			child.addParent(node);
		}
	}
}
/**
 * This method will call the visit method in variableVisitor.  It is too
 * hard to generalize the creation of variables, so each operator
 * type is handled in its own visit method.  The visit method will
 * pass the operator and the Node (DOM, not XAT node) and each
 * visit method can be customized per operator type.  
 * @param children org.w3c.dom.Node
 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
private void addClassVariables(Node classVariables, DagNode node) {
	variableVisitor.visit(node, classVariables);

	
	NodeList classList = classVariables.getChildNodes();
	for (int i = 0; i < classList.getLength(); i++){
		Node next = classList.item(i);
		if(next.getNodeType() == Node.ELEMENT_NODE){
			NamedNodeMap attrs = next.getAttributes();
			if (node.getOperator() instanceof XATWBinaryJoinOperator) {

				Node idN = attrs.getNamedItem("size");
				String id = idN.getNodeValue();

				XATWBinaryJoinOperator xat =
					(XATWBinaryJoinOperator) node.getOperator();

				xat.setTimeWindow(Long.parseLong(id));
			}
			else if (node.getOperator() instanceof WBinaryJoinOperator) {

				Node idN = attrs.getNamedItem("size");
				String id = idN.getNodeValue();

				WBinaryJoinOperator xat =
					(WBinaryJoinOperator) node.getOperator();

				xat.setTimeWindow(Long.parseLong(id));
			}
		}
	}
}
/**
 * Parses the parents element and connects parents.  If the parent
 * does not exist in the hash table, then just create a new DagNode with
 * the corresponding ID.  The operator will be connected later
 * Creation date: (2/1/03 12:53:59 PM)
 * @param parents org.w3c.dom.Node
 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
private void addParents(Node parents, DagNode node) {
	
	NodeList parentList = parents.getChildNodes();
	for (int i = 0; i < parentList.getLength(); i++){
		Node next = parentList.item(i);
		if(next.getNodeType() == Node.ELEMENT_NODE){
			NamedNodeMap attrs = next.getAttributes();
			Node idN = attrs.getNamedItem("id");
			String id = idN.getNodeValue();
			DagNode parent;
			if(operatorTable.containsKey(id)){
				parent = (DagNode)operatorTable.get(id);	
			} else{
				parent = new DagNodeImp();
				operatorTable.put(id,parent);
			}
			node.addParent(parent);
			parent.addChild(node);
		}
	}
	
}
/**
 * Adds the properties of the DagNode as specified in the XML document
 * This is pretty straight forward- parse the properties element,
 * get the name and value attributes, then set the property
 * @param children org.w3c.dom.Node
 * @param node edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
private void addProperties(Node properties, DagNode node) throws Exception {
	
	NodeList propertiesList = properties.getChildNodes();
	for (int i = 0; i < propertiesList.getLength(); i++){
		Node next = propertiesList.item(i);
		if(next.getNodeType() == Node.ELEMENT_NODE){
			NamedNodeMap attrs = next.getAttributes();
            String name = ((Node) attrs.getNamedItem("name")).getNodeValue();
            String value = ((Node) attrs.getNamedItem("value")).getNodeValue();
            node.getOperator().setProperty(name, value);
		}
	}
	
}
/**
 * If we are working with XATStreamOperators, we can connect all of the
 * operators together with this recursive method 
 * @param XATStreamOperator root.  
 */
private void connectQueues(XATStreamOperator root) throws Exception {

	
    XATNode[] children = root.getNode().getChildren();
    for (int i = 0; i < children.length; i++) {
	    try{
        XATNode node = children[i];
        
        /** if current node is not Source node, which means that it is an XATStreamOperator */
		if (node.getOperator() instanceof XATStreamOperator) {

			XATStreamOperator child = (XATStreamOperator) node.getOperator();
			XATQueue queue = ((XATStreamOperator) child).getOutputQueue();
			/** set the input queue.  If we are a multi queue operator, then add the queue */
			if (root instanceof XATMultiQueueStreamOperator) {
				((XATMultiQueueStreamOperator) root).addInputQueue(queue);
			} else {
				((XATStreamOperator) root).setInputQueue(queue);
			}
			connectQueues(child);
		}
    	} catch (XATNodeDoesNotExistException e){
			/**
			 this is thrown when an operator does not have
			 a child.  This shouldnt happen because we only iterate
			 through the available children, but this catch will prevent
			 the error from being propagated higher
			*/
    	}
	} 
    return;
}
/**
 * Traverses the XML document and finds an element by a given name with the given sibling.
 * If the first element encountered does not have the same name as the parameter name, an exception is thrown
 * @return org.w3c.dom.Node
 * @param name java.lang.String
 * @param sibling org.w3c.dom.Node
 */
private Node getNodeByName(String name, Node sibling) throws IOException{
	while (sibling != null) {
                /**
                * check the nodeName, should = edges
                * if it doesnt, then throw exception
                */
                if (sibling instanceof Element) {
                    if (!sibling.getNodeName().equals(name)) {
                        throw new IOException("The XML document does not conform to the default Schema");
                    }
                    /** now we've found the edges element*/
                    return sibling;
                }
                sibling = sibling.getNextSibling();
            }
	throw new IOException("The element was not found");
}
/**
 * Parses an XML document and returns an XATTree data structure.
 * This will first parse the file, find all operator elements, then create
 * operators one by one.  Then the data models (table or queue) will be connected
 *
 * Dags will be created if the dag attribute of the query plan element is set to true.  Otherwise,
 * XATNodes will be created.
 * @return XATTree
 * @exception QueryPlanGeneratorException 
 */
public XATTree makeQueryPlan() throws QueryPlanGeneratorException {
    try {
        String key = "javax.xml.parsers.DocumentBuilderFactory";
        String value = "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl";
        //set the DocumentBuilderFactory to use
        java.lang.System.setProperty(key, value);
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //DocumentBuilderFactory factory = new DocumentBuilderFactoryImpl();
        DocumentBuilder builder = factory.newDocumentBuilder();
        Document document = builder.parse(this.getProperty("FILE_NAME"));
		/*
        	Element node = document.getDocumentElement();
        	NamedNodeMap attrs = node.getAttributes();
        	dataModel = ((Node) attrs.getNamedItem("name")).getNodeValue();
		*/
        NodeList operators = document.getElementsByTagName("operator");
        for (int i = 0; i < operators.getLength(); i++) {
            parseOperator(operators.item(i));
        }

        /** 
         if we have XATStreamOperators, then we will connect the input and output
         queues together with this method.  We will also connect the tree
         to the Streams
        */
		XATOperator operator = getDag().getRoot().getOperator();
		if(operator instanceof XATStreamOperator){
			connectQueues((XATStreamOperator) operator);
			connectTreeToStreams();
		}
		
		return getDag();
	} catch (Exception e) {
			throw new QueryPlanGeneratorException(e.toString());
		}
}
/**
 * Takes an element as a parameter, parses the element, creates
 * an XATNode and operator, connects parents and children (if possible)
 * and then stores the operator in the operatorTable
 * @param operatorElement org.w3c.dom.Node
 */
private void parseOperator(Node operatorElement) throws Exception {
    /** first make sure we were given an element */
    if (operatorElement.getNodeType() != Node.ELEMENT_NODE) {
        throw new DOMException(
            DOMException.INVALID_ACCESS_ERR,
            "The parameter was not a DOM Element");
    }
    /** next make sure we were given an operator element */
    if (!operatorElement.getNodeName().equalsIgnoreCase("operator")) {
        throw new DOMException(
            DOMException.INVALID_ACCESS_ERR,
            "The parameter was not an operator Element");
    }
    NamedNodeMap nodeAttributes = operatorElement.getAttributes();

    Attr nodeIdentifier = (Attr) nodeAttributes.getNamedItem("id");
    if (nodeIdentifier == null) {
        throw new DOMException(
            DOMException.SYNTAX_ERR,
            "The ID attribute was not set properly");
    }
    String id = nodeIdentifier.getValue();
    DagNode dagNode;
    /** 
     * first check the hashtable to see if we have a DagNode matching this operator's ID 
     * if it was, grab it and we will use it, otherwise, create a new one.
     */
    if (operatorTable.containsKey(id)) {
        dagNode = (DagNode) operatorTable.get(id);
    } else {
        dagNode = new DagNodeImp();
        operatorTable.put(id, dagNode);
    }

    /** see if this operator is a root or not */
    nodeIdentifier = (Attr) nodeAttributes.getNamedItem("root");
    Boolean isRoot = new Boolean(nodeIdentifier.getValue());
    if (isRoot.booleanValue()) {
        getDag().addRoot(dagNode);
    }

    /** first get the className attribute */
    nodeIdentifier = (Attr) nodeAttributes.getNamedItem("className");
    if (nodeIdentifier == null) {
        throw new DOMException(
            DOMException.SYNTAX_ERR,
            "The className attribute was not set properly");
    }
    String className = nodeIdentifier.getValue();
    //Class[] c1 = new Class[] { DagNode.class};
    //Constructor construct = Class.forName(className).getConstructor(c1);
    Constructor[] c1 = Class.forName(className).getConstructors();
    Constructor construct = c1[0];
    XATOperator operator = null;
 	if(className.equalsIgnoreCase("edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.SourceImp"))
 	{
		construct = c1[1];
		operator = (Source) construct.newInstance(new Object[] { dagNode });
		((SourceImp)operator).setName(id);
 	}
	else {
		operator = (XATOperator) construct.newInstance(new Object[] { dagNode });
	}
    dagNode.setOperator(operator);
 

	/** 
	 set the data model for this operator.
	 This is a small hack, it assumes it is a stream operator.  This will
	 have to be altered if it is an XATTable
	*/
    
    /** now get the classVariables, properties, parents, and children of the operator */
    Node nextNode = operatorElement.getFirstChild(),
        classVariables = null,
        properties = null,
        parents = null,
        children = null;
    classVariables = getNodeByName("classVariables", nextNode);
    nextNode = classVariables.getNextSibling();
    properties = getNodeByName("properties", nextNode);
    nextNode = properties.getNextSibling();
    parents = getNodeByName("parents", nextNode);
    nextNode = parents.getNextSibling();
    children = getNodeByName("children", nextNode);

    addChildren(children, dagNode);
    addParents(parents, dagNode);
    addProperties(properties, dagNode);
    addClassVariables(classVariables, dagNode);
    
    return;

}
/**
 * Sets the properties and default values that are available for this
 * particular QueryPlanGenerator
 */
protected void setDefaultProperties() {
	super.setDefaultProperties();    
    /**
    * This is the name of the xml file to parse 
    */
    addProperty("FILE_NAME", "queryplan.xml");    
}
/**
* sets the given property to the new value.  This method is responsible
* for validating the given property.  An exception is thrown if
* the value is not correct.
* Validation should be done in the following manner: 
* First the method should check if the object supports the given property by calling
* <tt>isValidPropertyName(String)<tt>.  If it is not a valid property name, then an exception should
* be thrown. <p>
*
* Once it is known that the object supports the property, the value must be validated.  Each implementor of
* XATStatistics has a set of properties that are common to all of its types and some that are specific to types.  
* For example, all XATOperators (which implement XATStatistics) have certain properties while an individual XATOperator
* has specific properties. The indivual types should only validate their own specific types.  If the caller is trying
* to set a common type, it should be validated by the super class by calling <tt>super.setProperty(name, value)<tt>.  This
* will reduce duplicate code. <p>
*
* When it is determined the name / value pair is correct, the pair is updated using <tt>setNewPropertyValue()<tt> <p>
* <tt>XATStatisticsImp<tt> provides default validation by checking to see if the property's value is
* >=0.  propertyValues that are Strings will just be set.  If the property requires more strict constraints than this, 
* it is up to the subclass that created the property to validate it (as described above).
* @param name the name of the property to set
* @param value the new value of the property
* @throws Exception if the given property cannot be found or
* if the value is not valid for the given property.
*/
public void setProperty(java.lang.String name, java.lang.String value)
    throws java.lang.Exception {
    super.setProperty(name, value);
}
}
