package edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator;

import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNodeImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDagImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Source;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.SourceImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.windowstreamoperator.XATWBinaryJoinOperatorImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.streamexpression.BinCOMPExpressionStr;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTreeImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNodeImp;
/**
 * Insert the type's description here.
 * Creation date: (3/28/2003 2:33:17 PM)
 * @author: Yali Zhu
 */
public class ManualQueryPlanGenerator extends QueryPlanGeneratorImp {
/**
 * ManualQueryPlanGenerator constructor comment.
 */
protected ManualQueryPlanGenerator() {
	super();
}
/**
 * Makes a single query plan.  Each QueryPlanGenerator implements
 * this method in order to generate a query plan in their own manner
 * 
 * @return XATTree
 * @exception QueryPlanGeneratorException 
 */
public XATTree makeQueryPlan() throws QueryPlanGeneratorException {
	//return makeSingleJoinPlan();
	return makeQueryPlan1();
}
/**
 * Insert the method's description here.
 * Creation date: (4/6/2003 3:08:27 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
private XATDag makeQueryPlan1() {

    XATDag dag = new XATDagImp();

    DagNode node1 = new DagNodeImp();
    XATWBinaryJoinOperatorImp wbjoinop1 =
        new XATWBinaryJoinOperatorImp(node1);
    wbjoinop1.setTimeWindow(1000);
    wbjoinop1.setJoinPredicate("Col1", "Col2", BinCOMPExpressionStr.EQ);
    node1.setOperator(wbjoinop1);

    DagNode sourceNode1 = new DagNodeImp();
    Source sourceOp1 = new SourceImp(sourceNode1);
    sourceOp1.setName("A");
    sourceNode1.setOperator(sourceOp1);
    sourceNode1.setParent(node1);
    node1.addChild(sourceNode1);

    DagNode sourceNode2 = new DagNodeImp();
    Source sourceOp2 = new SourceImp(sourceNode2);
    sourceOp2.setName("B");
    sourceNode2.setOperator(sourceOp2);
    sourceNode2.setParent(node1);
    node1.addChild(sourceNode2);

    XATQueueImp outputQ1 = new XATQueueImp(wbjoinop1);
    Vector cols1 = new Vector();
    cols1.add("Col1");
    cols1.add("Col2");

    // set state schema for each join operators

    XATQueueSchema schema1 = new XATQueueSchema(cols1);
    outputQ1.setSchema(schema1);

    try {
        wbjoinop1.setOutputQueue(outputQ1);
        outputQ1.register(this);
    } catch (XATQueueException e) {
        System.out.println(e);
    }

    wbjoinop1.setDebug(false);
    //wbjoinop.setDebug(true);

    /** put another window join node on top */
    DagNode node2 = new DagNodeImp();
    XATWBinaryJoinOperatorImp wbjoinop2 =
        new XATWBinaryJoinOperatorImp(node2);
    wbjoinop2.setTimeWindow(1000);
    wbjoinop2.setJoinPredicate("Col1", "Col3", BinCOMPExpressionStr.EQ);
    node2.setOperator(wbjoinop2);
	node1.setParent(node2);
	node2.addChild(node1);
	
    DagNode sourceNode3 = new DagNodeImp();
    Source sourceOp3 = new SourceImp(sourceNode3);
    sourceOp3.setName("C");
    sourceNode3.setOperator(sourceOp3);
    sourceNode3.setParent(node2);
    node2.addChild(sourceNode3);

    XATQueueImp outputQ2 = new XATQueueImp(wbjoinop2);
    Vector cols2 = new Vector();
    cols2.add("Col1");
    cols2.add("Col2");
    cols2.add("Col3");

    XATQueueSchema schema2 = new XATQueueSchema(cols2);
    outputQ2.setSchema(schema2);

    try {
        wbjoinop2.setOutputQueue(outputQ2);
        outputQ2.register(this);
    } catch (XATQueueException e) {
        System.out.println(e);
    }

    wbjoinop2.addInputQueue(outputQ1);
    wbjoinop2.setDebug(false);
    //wbjoinop.setDebug(true);

    dag.setRoot(node2);

    return dag;
}
/**
 * Insert the method's description here.
 * Creation date: (4/6/2003 3:08:51 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
private XATDag makeQueryPlan2() {
    XATDag dag = new XATDagImp();

    DagNode node1 = new DagNodeImp();
    XATWBinaryJoinOperatorImp wbjoinop1 =
        new XATWBinaryJoinOperatorImp(node1);
    wbjoinop1.setTimeWindow(1000);
    wbjoinop1.setJoinPredicate("Col1", "Col3", BinCOMPExpressionStr.EQ);
    node1.setOperator(wbjoinop1);

    DagNode sourceNode1 = new DagNodeImp();
    Source sourceOp1 = new SourceImp(sourceNode1);
    sourceOp1.setName("A");
    sourceNode1.setOperator(sourceOp1);
    sourceNode1.setParent(node1);
    node1.addChild(sourceNode1);

    DagNode sourceNode2 = new DagNodeImp();
    Source sourceOp2 = new SourceImp(sourceNode2);
    sourceOp2.setName("C");
    sourceNode2.setOperator(sourceOp2);
    sourceNode2.setParent(node1);
    node1.addChild(sourceNode2);

    XATQueueImp outputQ1 = new XATQueueImp(wbjoinop1);
    Vector cols1 = new Vector();
    cols1.add("Col1");
    cols1.add("Col3");

    XATQueueSchema schema1 = new XATQueueSchema(cols1);
    outputQ1.setSchema(schema1);

    try {
        wbjoinop1.setOutputQueue(outputQ1);
        outputQ1.register(this);
    } catch (XATQueueException e) {
        System.out.println(e);
    }

    wbjoinop1.setDebug(false);
    //wbjoinop.setDebug(true);

    /** put another window join node on top */
    DagNode node2 = new DagNodeImp();
    XATWBinaryJoinOperatorImp wbjoinop2 =
        new XATWBinaryJoinOperatorImp(node2);
    wbjoinop2.setTimeWindow(1000);
    wbjoinop2.setJoinPredicate("Col1", "Col2", BinCOMPExpressionStr.EQ);
    node2.setOperator(wbjoinop2);
    node1.setParent(node2);
    node2.addChild(node1);

    DagNode sourceNode3 = new DagNodeImp();
    Source sourceOp3 = new SourceImp(sourceNode3);
    sourceOp3.setName("B");
    sourceNode3.setOperator(sourceOp3);
    sourceNode3.setParent(node2);
    node2.addChild(sourceNode3);

    XATQueueImp outputQ2 = new XATQueueImp(wbjoinop2);
    Vector cols2 = new Vector();
    cols2.add("Col1");
    cols2.add("Col3");
    cols2.add("Col2");

    XATQueueSchema schema2 = new XATQueueSchema(cols2);
    outputQ2.setSchema(schema2);

    try {
        wbjoinop2.setOutputQueue(outputQ2);
        outputQ2.register(this);
    } catch (XATQueueException e) {
        System.out.println(e);
    }

    wbjoinop2.addInputQueue(outputQ1);
    wbjoinop2.setDebug(false);
    //wbjoinop.setDebug(true);

    dag.setRoot(node2);

    return dag;
}
/**
 * Insert the method's description here.
 * Creation date: (4/6/2003 12:01:27 AM)
 */
private XATTree makeSingleJoinPlan() {
   
	XATTree tree = new XATTreeImp();

    XATNode node1 = new XATNodeImp();
    XATWBinaryJoinOperatorImp wbjoinop =
        new XATWBinaryJoinOperatorImp(node1);
    wbjoinop.setTimeWindow(1000);
    wbjoinop.setJoinPredicate("Col1", "Col2", BinCOMPExpressionStr.EQ);
    node1.setOperator(wbjoinop);

    DagNode sourceNode1 = new DagNodeImp();
    Source sourceOp1 = new SourceImp(sourceNode1);
    sourceOp1.setName("A");
    sourceNode1.setOperator(sourceOp1);
    sourceNode1.setParent(node1);
    node1.addChild(sourceNode1);

    DagNode sourceNode2 = new DagNodeImp();
    Source sourceOp2 = new SourceImp(sourceNode2);
    sourceOp2.setName("B");
    sourceNode2.setOperator(sourceOp2);
    sourceNode2.setParent(node1);
    node1.addChild(sourceNode2);

    XATQueueImp outputQ = new XATQueueImp(wbjoinop);
    Vector cols3 = new Vector();
    cols3.add("Col1");
    cols3.add("Col2");
  
    XATQueueSchema schema3 = new XATQueueSchema(cols3);
    outputQ.setSchema(schema3);

    try {
        wbjoinop.setOutputQueue(outputQ);
        outputQ.register(this);
    } catch (XATQueueException e) {
        System.out.println(e);
    }

    wbjoinop.setDebug(false);
    //wbjoinop.setDebug(true);
    tree.setRoot(node1);

    return tree;
}
}
