package edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator;

import java.util.Enumeration;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
/**
 * Provides a way to create random graphs that have a certain set of properties.
 * In order to specify the type of random graphs to generate, the RandomGraphGenManager
 * should be queried.  
 * @author: Brad Pielech
 */
public interface QueryPlanGenerator extends XATQueryObject {

	public final static java.lang.String factoryID = "edu.wpi.cs.dsrg.xmldb.xat.component.queryplangenerator";
	/**
	* <pre>
	* creates N query plans according to the properties supported by the QueryPlanGenerator
	* <b>pre</b>:
	* the properties of the query plan have been initialized accordingly
	* <b>post</b>:
	* The Enumeration returned will consist of randomly generated IQueryPlans
	* </pre>
	* @return Enumeration of IQueryPlans
	* @param int howMany how many IQueryPlans to generate
	* @exception QueryPlanGeneratorException if either a property of the generator was not initialized or
	* if there is a conflict in values ex: max < min
	*/	
	Enumeration makeNQueryPlans(int howMany) throws QueryPlanGeneratorException;
	/**
	* <pre>
	* creates N query plans according to the properties supported by the QueryPlanGenerator
	* <b>pre</b>:
	* the properties of the query plan have been initialized accordingly
	* <b>post</b>:
	* The Enumeration returned will consist of randomly generated IQueryPlans
	* </pre>
	* @return Enumeration of IQueryPlans
	* @param int howMany how many IQueryPlans to generate
	* @exception QueryPlanGeneratorException if either a property of the generator was not initialized or
	* if there is a conflict in values ex: max < min
	*/	
	public abstract XATTree makeQueryPlan() throws QueryPlanGeneratorException;
	public void setDataModel(String model);
}
