package edu.wpi.cs.dsrg.xmldb.xat.component.scheduler;

import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;

/**
 * The Chain class represents a set of operators that are grouped together
 * by a common slope built from a <tt> ProgressChart </tt>.
 * Creation date: (4/15/2003 4:20:50 PM)
 * @author: Tim Sutherland
 * @version: 1.1.1
 */
public class Chain {

/** A vector of Operators in the Chain */	
private Vector operator;
/** The slope of the Chain Object */
private double slope;
/** Default Constructor. Creates an Empty Chain */
public Chain(){

	operator = new Vector();
	slope = 0;

}
/** Constructor. Creates a chain with the given array
 * of <tt>XATStreamOperators </tt>
 */
public Chain(XATStreamOperator[] xso){

	for(int i = 0 ; i < xso.length ; i++){
		operator.addElement(xso[i]);
	}
	slope = 0;

}
/**
 * Adds an operator to the Chain
 * Creation date: (4/17/2003 10:38:02 AM)
 * @param XATStreamOperator
 */
public void addOperator(XATStreamOperator xso) {
	operator.addElement(xso);
	}
/**
 * Returns the number of operators in the chain
 * Creation date: (4/20/2003 12:04:50 PM)
 * @return int- the number of operators
 */
public int getNumberOfOperators() {
	return operator.size();
}
/** Gets the operator at the specified location
 * @param int the location of the operator in the chain
 * @return XATStreamOperator- the operator at the specified location
 */
 
public XATStreamOperator getOperator(int location){

	return (XATStreamOperator) operator.elementAt(location);
}
/**
 * Returns the slope of the chain
 * Creation date: (4/15/2003 7:03:47 PM)
 * @return double- the slope of the chain
 */
public double getSlope() {
	return slope;
}
/**
 * Sets the slope of the chain
 * Creation date: (4/15/2003 7:03:47 PM)
 * @param newSlope double the new slope of the chain
 */
public void setSlope(double newSlope) {
	slope = newSlope;
}
/**
 * Returns a representation of the Chain as a string.
 * Creation date: (4/22/2003 6:11:42 PM)
 * @return java.lang.String- a representation of the Chain
 */
public String toString() {

	String s = "Chain: \nSlope- " + slope + "\nOperators- \n";

	String t = new String();

	for(int i = 0 ; i < getNumberOfOperators(); i++){

		t += getOperator(i).toString() + "\n";

	}
	return s + t;
}
}
