package edu.wpi.cs.dsrg.xmldb.xat.component.scheduler;

import java.util.ArrayList;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsGatherer;

/**
 * This is a GreedyScheduler that will run the operator with the highest
 * GREEDY_PRIORITY.  This priority is based on the ratio between (1 - SELECTIVITY) and
 * AVERAGE_TUPLE_PROCESSING_TIME.  This method may be prone to starvation because the operator
 * with the highest priority may not have any work to do.  
 */
public class GreedyPriorityScheduler extends XATScheduler {
/**
 * GreedyPriorityScheduler constructor comment.
 */
public GreedyPriorityScheduler() {
	super();
}
/**
 * This is called by the Execution Controller.  This method decides
 * which operator to run next and how many tuples that operator should dequeue.
 * This will just pick the operator with the highest priority.  If that operator
 * does not have any work to do, then it will pick the 2nd and so on.  If nothing
 * has any work to do, then throw the exception 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision
 */
public XATSchedulerDecision makeNextDecision() throws XATSchedulerException {

    XATSchedulerDecision decision = new XATSchedulerDecision();
    ArrayList operatorsChecked = new ArrayList();
    int totalTuples = 0;
    int length = this.getTree().convertToArray().length;
    for (int i = length - 1; i >= 0; i--) {
        /** get the next highest priority operator*/
        XATStreamOperator operator =
            (XATStreamOperator) StatisticsGatherer.getStatisticsGather().getQueryObject(
                "GREEDY_PRIORITY", i);
        int workload = calculateWorkLoad(operator);
        if (workload > 0 && operator.isActive()) {
            decision.setNextOperator(operator);
            decision.setNumberOfTuples(workload);
            return decision;
        }
    }

    // if we looked at all operators in the for loop, then there is nothing left to do 
	throw new XATSchedulerException("Looked at all operators and there is no work left to do");
          
}
}
