package edu.wpi.cs.dsrg.xmldb.xat.component.scheduler;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsGatherer;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsReference;
/**
 * Schedules the operator with the most tuples in its input queues.  This scheduler
 * may not have the best throughput, and may be prone to starvation, but the intermediate
 * queues sizes should be smaller 
 * @author: Brad Pielech
 */
public class MostTuplesInQueueScheduler extends XATScheduler {
/**
 * MostTuplesInQueueScheduler constructor comment.
 */
public MostTuplesInQueueScheduler() {
	super();
}
/**
 * This is called by the Execution Controller.  This method decides
 * which operator to run next and how many tuples that operator should dequeue.
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision
 */
public XATSchedulerDecision makeNextDecision() throws XATSchedulerException {
    XATSchedulerDecision decision = new XATSchedulerDecision();
    int length = this.getTree().convertToArray().length;
    int operatorsChecked = 0;
    while (true) {
        XATStreamOperator operator =
            (XATStreamOperator) StatisticsGatherer.getStatisticsGather().getQueryObject(
                "NUMBER_OF_TUPLES_IN_INPUT_QUEUES",
                StatisticsReference.HIGHEST, true);
        operatorsChecked++;
        int workload = calculateWorkLoad(operator);
        if (workload > 0 && operator.isActive()) {
            decision.setNextOperator(operator);
            decision.setNumberOfTuples(workload);
            return decision;
        } else {
            /** 
             * if there is no work, then check to see if we have looked at all operators
             * if we have, then break
             */
            if (operatorsChecked >= length) {
	            int temp = 0;
                throw new XATSchedulerException("No more work to do");
            }
        }
    }
}
}
