package edu.wpi.cs.dsrg.xmldb.xat.component.scheduler;

import java.util.ArrayList;
import java.util.Random;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsGatherer;
import edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsReference;
/**
 * This is a GreedyScheduler that will run each operator with a probibility equal to its 
 * GREEDY_PRIORITY.  This priority is based on the ratio between (1 - SELECTIVITY) and
 * AVERAGE_TUPLE_PROCESSING_TIME.  This method may be prone to starvation because the operator
 * with the chosen may not have any work to do.  
 */
public class ProbabilisticGreedyScheduler extends XATScheduler {
	private java.util.Random rand;
/**
 * GreedyPriorityScheduler constructor comment.
 */
public ProbabilisticGreedyScheduler() {
	super();
	rand = new Random();
}
/**
 * This is called by the Execution Controller.  This method decides
 * which operator to run next and how many tuples that operator should dequeue.
 * This will just pick the operator with the highest priority.  Starvation may
 * occur, but it is up to the Execution Controller to detect this and change the
 * algorithm 
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision
 */
public XATSchedulerDecision makeNextDecision() throws XATSchedulerException {

    XATSchedulerDecision decision = new XATSchedulerDecision();
    ArrayList operatorsChecked = new ArrayList();
    int totalTuples = 0;
    XATNode[] nodes = this.getTree().convertToArray();
	StatisticsGatherer stats = StatisticsGatherer.getStatisticsGather();
    double count = 0;
    //sum the scores
    try {    	    
    for (int i = 0; i < nodes.length; i++) {
        XATStreamOperator next = (XATStreamOperator) nodes[i].getOperator();
        count += Double.parseDouble(stats.getValue("GREEDY_PRIORITY", next, StatisticsReference.AVERAGE));
    }
    } catch (Exception e) {
    	System.out.println(e.toString());
    }
	//System.out.println("Priority Count: " + count);
    //get the random variable between 0 and count
    double nextValue = rand.nextDouble() * count;
    count = 0;
    XATStreamOperator operator = null;
    boolean keepGoing = true;
    boolean activeOperator = false;
    
    int i = 0;    
    
    while(!activeOperator){
    
    	//now walk over the scores again and see which one makes the sum greater than our target
    	while(keepGoing){
        	operator = (XATStreamOperator) nodes[i].getOperator();
        	count += Double.parseDouble(stats.getValue("GREEDY_PRIORITY", operator, StatisticsReference.AVERAGE));
        	if (count >= nextValue) {
            	keepGoing = false;
        	}
        	i++;
    	}
    activeOperator = operator.isActive();

    }
	int workload = calculateWorkLoad(operator);
	/** 
	 * with greedy, if there is no work for an operator to do,
	 * that is ok b/c greedy is inherently prone to starvation 
	 */
	decision.setNextOperator(operator);
	decision.setNumberOfTuples(workload); 
    return decision;
}
}
