package edu.wpi.cs.dsrg.xmldb.xat.component.scheduler;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
/**
 * This is basic scheduling algorithm,
 * it will run the operator at the <tt>index</tt> position in the nodes array.  index
 * is then incremented, unless it is > nodes.length, then it is set to 0. <p>
 * Initially, this strategy may not seem like round robin b/c operators are run out of order,
 * but that is b/c not all operators have work to do.  If an operator does not have work (tuples in
 * input queue), it is skipped.  After running the operators in the bottom of the 
 * query plan a few times, this scheduler will the appear to run operators in a traditional RR fashion.
 */
public class RoundRobinScheduler extends XATScheduler {
	private XATNode[] nodes;
	private int index;
/**
 * RoundRobinScheduler constructor comment.
 */
public RoundRobinScheduler() {
	super();
}
/**
 * Insert the method's description here.
 * Creation date: (2/22/03 1:51:50 PM)
 */
private void init() {
	
	XATTree[] trees = getTrees();
	XATNode[] temp;
	int c = 0;
	int size=0;
	
	for(int i=0; i < trees.length; i++){
		size += getTree(i).size();
	}
	nodes = new XATNode[size];

	for(int i=0; i < trees.length; i++){
		temp = getTree(i).convertToArray();
		for(int j=0; j < temp.length;j++){
			nodes[c] = temp[j];
			c++;
		}
	}
	
	//index = 0;
	// Set index this way so schduler starts from leaf node, instead of root node.
	index = size-1;
	}
/**
 * make the next scheduling decision.   
 * If the operator is chosen but has workload of 0, then
 * we will choose another. If we have checked all operators and there is still no work, then
 * we will break.  This prevents
 * starvation, that is, when the scheduler decides to run an operator with no tuples in its
 * input buffer.  
 * Creation date: (1/22/03 4:05:38 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.scheduler.XATSchedulerDecision
 * @throws XATSchedulerException when there is no work to do
 */
public XATSchedulerDecision makeNextDecision() throws XATSchedulerException {
    XATSchedulerDecision decision = new XATSchedulerDecision();
    XATStreamOperator operator;
    int totalTuples = 0;
    int count = 0;

	do{
		operator = (XATStreamOperator)nodes[index].getOperator();
		//index++; count++;
		//if(index >= nodes.length){
		//	index = 0;
		//}	
		
		index--; count++;
		if(index < 0){
		  index = nodes.length-1;;
		}	
		
		// Throw an exception if we have gone through all of the
		// operators and all are inactive.
		if(count > nodes.length) throw new XATSchedulerException();
		
	}while(!operator.isActive());

    int workload = calculateWorkLoad(operator);
     	        
	decision.setNextOperator(operator);		
    decision.setNumberOfTuples(workload);
        
    return decision;		  
}
/**
 * Insert the method's description here.
 * Creation date: (2/22/03 1:46:59 PM)
 * @param newTree edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 */
public void setTree(edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree newTree) {
	super.setTree(newTree);
	init();
}

public void setTrees(edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree[] newTree) {
	super.setTrees(newTree);
	init();
}
/**
 * Insert the method's description here.
 * Creation date: (2/26/2003 6:22:23 PM)
 * @return java.lang.String
 */
public String toString() {
	String rs = new String();

	String sName = this.getClass().getName();
	//find the last dot.
	int index = 0;
	for(index = sName.length()-1; index >=0; index--)
		if(sName.charAt(index) == '.')
			break;
	rs = sName.substring(index + 1) + " " + hashCode(); // + "  data:";	
    return rs;
}
}
