package edu.wpi.cs.dsrg.xmldb.xat.component.scheduler;

import java.io.*;
/**
 * An instance of this class will encode a user's preference.  There
 * are 3 parts to a preference: the statistic, the quantifier (min or max) and
 * the weight.  The weights of all preferences in the system should sum to 1, if
 * they do not, an error will occur and execution will stop.  
 * @author: Brad Pielech
 */
public class XATPreference implements Serializable{
	/** constant for maximizing */
	public static int MAX = 1;
	/** constant for minimizing */
	public static int MIN = -1;
	/** the statistic related to this preference */
	private String statistic;
	/** 
	 * the quantifer for this preference.  This is either 1 (maximize)
	 * or -1 (minimize)
	*/
	private int quantifier;
	/** the relative weight assigned to this preference */
	private double weight;
	
/**
 * XATPreference constructor comment.
 */
public XATPreference() {
	super();
}
/**
 * Insert the method's description here.
 * Creation date: (3/29/2003 10:39:27 PM)
 * @return int
 */
public int getQuantifier() {
	return quantifier;
}
/**
 * Insert the method's description here.
 * Creation date: (3/29/2003 10:39:27 PM)
 * @return java.lang.String
 */
public java.lang.String getStatistic() {
	return statistic;
}
/**
 * Insert the method's description here.
 * Creation date: (3/29/2003 10:39:27 PM)
 * @return double
 */
public double getWeight() {
	return weight;
}
/**
 * Set the new quantifier, this should be 1 or -1.
 * @param newQuantifier int
 */
public void setQuantifier(int newQuantifier) {
	if((newQuantifier == XATPreference.MAX) || (newQuantifier == XATPreference.MIN)){
		quantifier = newQuantifier;
		return ;
	} else{
		throw new IllegalArgumentException("The quantifier should be 1 or -1");
	}
	
}
/**
 * set the statistic associated with this preference.  An error
 * is thrown if the statistic name is not supported by the statistics gatherer.
 * @param newStatistic java.lang.String
 */
public void setStatistic(java.lang.String newStatistic) {
	//@TODO fix this to work with distribution
	//if(!StatisticsGatherer.getStatisticsGather().isStatisticSupported(newStatistic)){
	//	throw new IllegalArgumentException("The statistic " + newStatistic + " is not supported by the Stats Gatherer");
	//}
	statistic = newStatistic;
}
/**
 * set the weight, this must be between 0 and 1
 * @param newWeight double
 */
public void setWeight(double newWeight) {
	if((weight < 0) || (weight > 1)){
		throw new IllegalArgumentException("The weight must be between 0 and 1 inclusive");
	}
	weight = newWeight;
}
/**
 * Insert the method's description here.
 * Creation date: (3/30/2003 4:15:42 PM)
 * @return java.lang.String
 */
public String toString() {
	String s = "";
	s = "XATPreference: ";
	s = s + "\t Statistic: " + statistic;
	s = s + "\t Quantifier: " + quantifier;
	s = s + "\t Weight: " + weight;
	return s;
}
}
