package edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.InheritableVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

/**
 * Compute the Consumed columns for each type of operator.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */
public class ConsumedVisitor extends InheritableVisitor
{
/**
 * Put all computed consumed columns together.
 * @return Set The set contains all consumed column by given operator.
 * @param node The XATNode computing for.
 * @since 1.0
 */
private Set aggregateConsumed(XATNode node) {
    Set s = new HashSet((Set) visit(node));
    XATNode[] children = node.getChildren();

    for (int i = 0; i < children.length; i++) {
        s.addAll(aggregateConsumed(children[i]));
    }
    return s;
}
public Object dispatch() {
    return new HashSet();
}
public Object dispatch(Object o) {
    return new HashSet();
}
/**
 * Compute all consumed columns by a given Expression.
 * @return Set The set including all consumed columns of a given expression.
 * @param  e   The Expression for computing consumed columns.
 * @since 1.0
 * @see Expression
 * @see UniExpression
 * @see TerminalExpression
 * @see BinExpression
 */
private Set expressionConsumed(Expression e) {
    if (e instanceof BinExpression) {
        BinExpression be = (BinExpression) e;
        Set s = new HashSet(expressionConsumed(be.getLeft()));
        s.addAll(expressionConsumed(be.getRight()));
        return s;
    } else
        if (e instanceof UniExpression) {
            return expressionConsumed(((UniExpression) e).getExpression());
        } else
            if (e instanceof TerminalExpression) {
                if (((TerminalExpression) e).getType() == TerminalExpression.NAME) {
                    ColumnName cn = (ColumnName) ((TerminalExpression) e).getValue();
                    return oneElementSet(cn.getColumnName());
                } else {
                    return new HashSet();
                }
            } else {
                return null;
            }
}
/**
 * Construct an set object using given object o.
 * @return Set The constructed object.
 * @param  o   The object used to construct the Set object.
 * @since 1.0
 */

private Set oneElementSet(Object o) {
    Set s = new HashSet();
    s.add(o);
    return s;
}
/**
 * Compute the consumed columns for Expose operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  e	The Expose operator computing for.
 * @since 1.0
 * @see Expose
 */
 
public Object visit(Expose e) {
    return oneElementSet(e.getColumnName());
}
/**
 * Compute the consumed columns for FOR operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  f	The FOR operator computing for.
 * @since 1.0
 * @see FOR
 */
 
public Object visit(FOR f) {
    /*
    XATTree tree = f.getXtreFunctions();
    return aggregateConsumed(tree.getRoot());
    */

    // assume FOR always got one for binding in each FOR operator.
    return oneElementSet(f.getColumnNames()[0]);
}
/**
 * Compute the consumed columns for Function operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  f	The Function operator computing for.
 * @since 1.0
 * @see Function
 */

public Object visit(Function f) {
    Object[] o = f.getParams();
    Set s = new HashSet();
    for (int i = 0; i < o.length; i++) {
        if (o[i] instanceof ColumnName) {
            s.add(((ColumnName) o[i]).getColumnName());
        } else
            if (o[i] instanceof Expression) {
                s.addAll(expressionConsumed((Expression) o[i]));
            }
    }
    return s;
}
/**
 * Compute the consumed columns for Merge operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  m	The Merge operator computing for.
 * @since 1.0
 * @see Merge
 */

public Object visit(Merge m) {
    Set s = new HashSet();

    return s;
}
/**
 * Compute the consumed columns for NameColumn operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  nc	The NameColumn operator computing for.
 * @since 1.0
 * @see NameColumn
 */

public Object visit(NameColumn nc) {
    return oneElementSet(nc.getOriginalColumnName());
}
/**
 * Compute the consumed columns for Source operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  s	The Source operator computing for.
 * @since 1.0
 * @see Source
 */

public Object visit(Source s) {
    return new HashSet();
}
/**
 * Compute the consumed columns for SQLStmt operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  sqlstmt The SQLStmt operator computing for.
 * @since 1.0
 * @see SQLStmt
 */

public Object visit(SQLStmt sqlstmt) {
    return new HashSet();
}
/**
 * Compute the consumed columns for CartesianProduct operator.
 * @return Object The HashSet constining consumed columns of given operator.
 * @param  cp	The CartesianProduct operator computing for.
 * @since 1.0
 * @see CartesianProduct
 */

public Object visit(CartesianProduct cp) {
    return new HashSet();
}
/**
 * Compute the consumed columns for Distinct operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  d	The Distinct operator computing for.
 * @since 1.0
 * @see Distinct
 */
 
public Object visit(Distinct d) {
    return oneElementSet(d.getInputColumnName());
}
/**
 * Compute the consumed columns for GroupBy operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  gb	The GroupBy operator computing for.
 * @since 1.0
 * @see GroupBy
 */
 
public Object visit(GroupBy gb) {
    Set s = new HashSet();

    String[] strings = gb.getColumnNames();

    for (int i = 0; i < strings.length; i++) {
        s.add(strings[i]);
    }

    XATTree tree = gb.getXATTree();

    s.addAll(aggregateConsumed(tree.getRoot()));
    return s;
}
/**
 * Compute the consumed columns for Intersection operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  i	The Intersection operator computing for.
 * @since 1.0
 * @see Intersection
 */

public Object visit(Intersection i) {
    /* unsure of this */
    return new HashSet();
}
/**
 * Compute the consumed columns for Join operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  j	The Join operator computing for.
 * @since 1.0
 * @see Join
 */

public Object visit(Join j) {
    return expressionConsumed(j.getExpression());
}

/**
 * Compute the consumed columns for NatureJoin operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  nj	The NatureJoin operator computing for.
 */
public Object visit(NatureJoin nj){
	XATNode njNode = nj.getNode();
	XATNode firstChild = njNode.getChild(0);
	XATNode secondChild = njNode.getChild(1);
	Vector firstChildVector = topDownTraverse(firstChild);
	Vector secondChildVector = topDownTraverse(secondChild);
	Object nextNode = null;
	for(int i=0; i<secondChildVector.size(); i++){
		nextNode = secondChildVector.elementAt(i); 
		if (firstChildVector.contains(nextNode)) break;
	}
	String str = new String("");
	str = ((XATNode)nextNode).getOperator().getOutputColumnName();
	Set s = new HashSet();
	s.add(str);
	return s;	
}

public static Vector topDownTraverse(XATNode root) {

  Vector resultSet = new Vector();

  try {

	//Do breath first traverse
	Set curSet = new HashSet();//contains nodes on current visiting level
	Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from root
	curSet.add(root);
	do {
			//for each node in current level
		for (Iterator it = curSet.iterator(); it.hasNext();) {
		XATNode curNode = (XATNode) it.next();

		//visit this node
		resultSet.add(curNode);

		if (curNode.getChildrenSize() > 0) {
		  XATNode[] children = curNode.getChildren();

		  //push each child in next level
		  for (int i = 0; i < children.length; i++) {
			XATNode curChild = (XATNode) children[i];

			if(!nextSet.contains(curChild))
			  nextSet.add(curChild);
		  }//push each child
		} //push next level

	  } //for

	  curSet = nextSet;
	  nextSet = new HashSet();
	} while (!curSet.isEmpty());

  } catch (Exception e) {
	e.printStackTrace();
  }

  return resultSet;
}


/**
 * Compute the consumed columns for OrderBy operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  ob	The OrderBy operator computing for.
 * @since 1.0
 * @see OrderBy
 */

public Object visit(OrderBy ob) {
    String[] columns = ob.getColumnNames();
    Set s = new HashSet();

    for (int i = 0; i < columns.length; i++) {
        s.add(columns[i]);
    }

    return s;
}
/**
 * Compute the consumed columns for OuterUnion operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  ou	The OuterUnion operator computing for.
 * @since 1.0
 * @see OuterUnion
 */

public Object visit(OuterUnion ou) {
    /* unsure of this */
    return new HashSet();
}
/**
 * Compute the consumed columns for Project operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  p	The Project operator computing for.
 * @since 1.0
 * @see Project
 */

public Object visit(Project p) {
    String[] columns = p.getColumnNames();
    Set s = new HashSet();

    for (int i = 0; i < columns.length; i++) {
        s.add(columns[i]);
    }

    return s;
}
/**
 * Compute the consumed columns for Select operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  s	The Select operator computing for.
 * @since 1.0
 * @see Select
 */

public Object visit(Select s) {
    return expressionConsumed(s.getExpression());
}
/**
 * Compute the consumed columns for Union operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  u The Union operator computing for.
 * @since 1.0
 * @see Union
 */

public Object visit(Union u) {
    /* unsure of this */
    return new HashSet();
}
/**
 * Compute the consumed columns for Combine operator.
 * @return Object The HashSet constining consumed columns of given operator.
 * @param  a	The Combine operator computing for.
 * @since 1.0
 * @see Combine
 */
 
public Object visit(Combine a) {
    return new HashSet();
}
/**
 * Compute the consumed columns for Composer operator.
 * @return Object 
 * @param  c The Composer operator computing for.
 * @since 1.0
 * @see Composer
 */

public Object visit(Composer comp) {
    return new Boolean(true);
}
/**
 * Compute the consumed columns for NavCollection operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  n	The NavCollection operator computing for.
 * @since 1.0
 * @see NavCollection
 */

public Object visit(NavCollection n) {
    NavigationStep ns = n.getEntryPoint()[0];
    if (ns instanceof VarStep) {
        return oneElementSet(ns.getStepValue());
    }
    return null;
}
/**
 * Compute the consumed columns for NavUnnest operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  n	The NavUnnest operator computing for.
 * @since 1.0
 * @see NavUnnest
 */
 
public Object visit(NavUnnest n) {
    NavigationStep ns = n.getEntryPoint()[0];
    if (ns instanceof VarStep) {
        return oneElementSet(ns.getStepValue());
    }
    return null;
}
/**
 * Compute the consumed columns for Tagger operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  t The Tagger operator computing for.
 * @since 1.0
 * @see Tagger
 */

public Object visit(Tagger t) {
    String[] names = t.getPattern().getColumnNames();
    Set s = new HashSet();
    for (int i = 0; i < names.length; i++) {
        s.add(names[i]);
    }
    return s;
}
/**
 * Compute the consumed columns for XMLUnion operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  xu The XMLUnion operator computing for.
 * @since 1.0
 * @see XMLUnion
 */

public Object visit(XMLUnion xu) {
    String[] columns = xu.getColumnNames();
    Set s = new HashSet();

    for (int i = 0; i < columns.length; i++) {
        s.add(columns[i]);
    }

    return s;
}
/**
 * Compute the consumed columns for XATNode operator.
 * @return Object The object constining consumed columns of given operator.
 * @param  node The XATNode operator computing for.
 * @since 1.0
 * @see XATNode
 */

public Object visit(XATNode node) {
    return visit(node.getOperator());
}

/**
 * Get the value of the propertity "DEBUG_SchemaCleanUp".
 * @return boolean True: With debug information. False: No debug information.
 * @since 1.0
 */
public static boolean isDebug() {
	return System.getProperty("DEBUG_SchemaCleanUp")!=null;
}
}
