package edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Hashtable;
import java.util.Set;
import java.util.HashSet;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.InheritableVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.IfExpr;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.FOR;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.GroupBy;

/**
 * Compute the Minimum Schema through the SchemaVisitor, the Required 
 * Columns through the RequiredVisitor, and do the cutting if the required 
 * columns not in produced and modified.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0  
 */
 
public class MinSchemaVisitor extends InheritableVisitor
{
    /**
     * The hash table to keep the map between XATNode and its corresponding SchemaCleanUpNode.
     * @since 1.0     
     */
    Hashtable hash;
	/**
	 * The visitor object used to compute the Required Schema.
     * @since 1.0     	 
	 */
    RequiredVisitor rvisitor;
	/**
	 * The visitor object used to compute the Minimum Schema.
     * @since 1.0     	 
	 */
    SchemaVisitor svisitor;
/**
 * MinSchemaVisitor Constructor.
 * @param  hash_init The hash table to keep the map between 
 * XATNode and its corresponding SchemaCleanUpNode.
 * @since 1.0
 * @see RequiredVisitor
 * @see SchemaVisitor
 */
 
public MinSchemaVisitor(Hashtable hash_init) {
    hash = hash_init;
    rvisitor = new RequiredVisitor(hash);
    svisitor = new SchemaVisitor(hash);
}
public Object dispatch() {
    return null;
}
public Object dispatch(Object o) {
    return null;
}
/**
 * Get the value of the propertity "DEBUG_SchemaCleanUp".
 * @return boolean True: With debug information. False: No debug information.
 * @since 1.0
 */
public static boolean isDebug() {
	return System.getProperty("DEBUG_SchemaCleanUp")!=null;
}
/**
 * This method recursively traverse the XATTree to execute the function 
 * of the given InheritableVisitor.
 * @return void
 * @param node The XATNode going to start the traversing.
 * @param v The functional visitor.
 * @since 1.0
 */

private void traverseHelper(XATNode node, InheritableVisitor v) {
    v.visit(node);
    XATNode[] children = node.getChildren();
    for (int i = 0; i < children.length; i++) {
        traverseHelper(children[i], v);
    }
}
/**
 * This method traversed the XATTree to compute the Produce, consume, and minimun Schema.
 * @return void
 * @param tree The XATTree going to traverse.
 * @param requiredAtRoot The set of required columns at root.
 * @since 1.0
 * @see XATTree
 * @see PCMVisitor
 * @see MinSchemaVisitor
 * @see traverseHelper(XATNode, InheritableVisitor) traverseHelper
 */
 
private void traverseSubtree(XATTree tree, Set requiredAtRoot) {
    Hashtable localhash = new Hashtable();
    PCMVisitor pcmvisitor = new PCMVisitor(localhash);
    XATNode root = tree.getRoot();
    traverseHelper(root, pcmvisitor);
    SchemaCleanupNode scn = (SchemaCleanupNode) localhash.get(root);
    scn.schema = requiredAtRoot;
    //Modified by Song Wang to fix bug for new generated column in this subtree.
    scn.schema.addAll(scn.produced);
    root.setSchema(scn.schema);
    //root.setSchema(requiredAtRoot);
    Set s = new HashSet(scn.schema);
    s.removeAll(scn.produced);
    s.addAll(scn.consumed);
    scn.required = s;
    XATNode[] children = root.getChildren();
    MinSchemaVisitor rscvisitor = new MinSchemaVisitor(localhash);
    for (int i = 0; i < children.length; i++) {
        traverseHelper(children[i], rscvisitor);
    }
}
/**
 * Compute all consumed columns by a given Expression.
 * @return Object 
 * @param  node   The XATNode being cleaned.
 * @since 1.0
 * @see SchemaCleanupNode
 * @see traverseSubtree(XATTree, Set) traverseSubtree
 */

public Object visit(XATNode node) {
    SchemaCleanupNode scn = (SchemaCleanupNode) hash.get(node);
    svisitor.visit(node);
    rvisitor.visit(node);
    //XATNode parent = node.getParent();
	java.util.Iterator parents = node.getParents();

    XATNode[] children = node.getChildren();
    SchemaCleanupNode scnParent;
    //if (parent == null) {
	if (parents==null||!parents.hasNext()||parents.next()==null){
        scnParent = new SchemaCleanupNode();
        scnParent.modified = new HashSet();
        scnParent.required = scn.schema;
        scnParent.allModified = false;
    } else {
        //scnParent = (SchemaCleanupNode) hash.get(parent);
    }

    // get the current operator.
    XATOperator op = node.getOperator();

    if (op instanceof FOR || op instanceof GroupBy || op instanceof IfExpr) {

        // get it's outer tree's schema.
        XATNode outerRoot;
        if (node.getChildren().length == 0) {
            outerRoot = null;
        } else {
            outerRoot = node.getChildren()[0];

            // disable the link of the parent node, hence it will not confuse the SchemaVisitor.
            outerRoot.setParent(null);

            // compute the outer tree's schema first.
            visit(outerRoot);

            // restore the parent link.
            outerRoot.setParent(node);
        }
        // get the computed schema.
        SchemaCleanupNode outerScn = null;
        if (outerRoot != null)
            outerScn = (SchemaCleanupNode) hash.get(outerRoot);
        else {
            outerScn = new SchemaCleanupNode();
            outerScn.modified = new HashSet();
            outerScn.required = new HashSet();
            outerScn.allModified = false;
        }

        if (op instanceof FOR) {
            Set s = new HashSet(outerScn.schema);
            s.retainAll(outerScn.produced);
            traverseSubtree(((FOR) op).getXtreFunctions(), s);
        } else
            if (op instanceof GroupBy) {
                Set s;
                if (scn.schema == null)
                    s = new HashSet();
                else
                    s = new HashSet(scn.schema);

                traverseSubtree(((GroupBy) op).getXATTree(), s);
            } else
                if (op instanceof IfExpr) {
                    Set s = new HashSet(outerScn.schema);
                    s.retainAll(outerScn.produced);
                    traverseSubtree(((IfExpr) op).getThenXAT(), s);
                    traverseSubtree(((IfExpr) op).getElseXAT(), s);
                }
                
                outerRoot.clearSchema();
    }
    return null;
}
}
