package edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.Hashtable;
import java.util.Set;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.InheritableVisitor;

/**
 * This class is used to compute the Produce, Consumed columns for each XATNode. 
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0  
 */

public class PCMVisitor extends InheritableVisitor
{
    /**
     * The hash table to keep the map between XATNode and its corresponding SchemaCleanUpNode.
     * @since 1.0     
     */	
    Hashtable hash;
	/**
	 * The visitor object used to compute the Produced column of XATNode.
     * @since 1.0     	 
	 */
    ProducedVisitor pvisitor = new ProducedVisitor();
	/**
	 * The visitor object used to compute the Consumed column of XATNode.
     * @since 1.0     	 
	 */
    ConsumedVisitor cvisitor = new ConsumedVisitor();
	/**
	 * The visitor object used to compute the Modified column of XATNode.
     * @since 1.0     	 
	 */
    ModifiedVisitor mvisitor = new ModifiedVisitor();   
/**
 * PCMVisitor constructor.
 * @param  hash_init The hash table to keep the map between 
 * XATNode and its corresponding SchemaCleanUpNode.
 * @since 1.0
 */
 
public PCMVisitor(Hashtable hash_init) {
    hash = hash_init;
}
public Object dispatch() {
    return null;
}
public Object dispatch(Object o) {
    return o;
}
/**
* Get the value of the propertity "DEBUG_SchemaCleanUp".
* @return boolean True: With debug information. False: No debug information.
* @since 1.0
*/
public static boolean isDebug() {
    return System.getProperty("DEBUG_SchemaCleanUp") != null;
}
/**
 * Compute the SchemaCleanupNode of given XATNode.
 * This method compute the SchemaCleanupNode for a given XATNode, and put the result
 * into hash table.
 * @return Object 
 * @param  node   The XATNode being cleaned.
 * @since 1.0
 * @see SchemaCleanupNode
 */
 
public Object visit(XATNode node) {
    SchemaCleanupNode scn = new SchemaCleanupNode();
    scn.produced = (Set) pvisitor.visit(node);
    scn.consumed = (Set) cvisitor.visit(node);
    Object m = mvisitor.visit(node);
    if (m instanceof Set) {
        scn.modified = (Set) m;
        scn.allModified = false;
    } else
        if (m instanceof Boolean) {
            scn.modified = null;
            scn.allModified = ((Boolean) m).booleanValue();
        } else {
            throw new ClassCastException();
        }
    return hash.put(node, scn);
}
}
