package edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.Hashtable;

import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.InheritableVisitor;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;

/**
 * This class is used to cleanup the tree with cutting.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0   
 */
public class TreeCleanupVisitor extends InheritableVisitor
{
    /**
      * The hash table to keep the map between XATNode and its corresponding SchemaCleanUpNode.
      * @since 1.0     
      */	
    Hashtable hash;
	/**
     * A visitor used to compute the Produce, Consumed columns for each XATNode.
     * @since 1.0
     */
    PCMVisitor pcmvisitor;
	/**
     * A visitor used to compute the minimun schema.
     * @since 1.0
     */        
    CutVisitor rscvisitor;

/**
 * The default TreeCleanupVisitor constructor.
 * @since 1.0
 * @see PCMVisitor
 * @see CutVisitor
 */
public TreeCleanupVisitor() {
    hash = new Hashtable();
    pcmvisitor = new PCMVisitor(hash);
    rscvisitor = new CutVisitor(hash);
}
/**
 * The TreeCleanupVisitor constructor.
 * @param  hash_init The hash table to keep the map between 
 * XATNode and its corresponding SchemaCleanUpNode. 
 * @since 1.0
 * @see PCMVisitor
 * @see CutVisitor
 */

public TreeCleanupVisitor(Hashtable hash_init) {
    hash = hash_init;
    pcmvisitor = new PCMVisitor(hash);
    rscvisitor = new CutVisitor(hash);
}
    public Object dispatch()
    {
	return null;
    }
    public Object dispatch(Object o)
    {
	return o;
    }
/**
 * Get the value of the propertity "DEBUG_SchemaCleanUp".
 * @return boolean True: With debug information. False: No debug information.
 * @since 1.0
 */
public static boolean isDebug() {
	return System.getProperty("DEBUG_SchemaCleanUp")!=null;
}
/**
 * Recursively traverse the tree from top down to do the tree cleanup.
 * @return void
 * @param node The current visited XATNode.
 * @param v    The visitor used to do cleanup function.
 * @since 1.0
 */

private void traverseTopDown(XATNode node, InheritableVisitor v) {
    v.visit(node);
    XATNode[] children = node.getChildren();
    for (int i = 0; i < children.length; i++) {
        traverseTopDown(children[i], v);
    }
}
/**
 * Visit a XATTree to clean up tree.
 * @return Object
 * @param tree The XATTree current cleaned.
 * @since 1.0
 */

public Object visit(XATTree tree) {

	// clean it up until no more nodes cutted.
	do{
    	XATNode root = tree.getRoot();
	    traverseTopDown(root, pcmvisitor);
	    // assume the tree is not cutted.
	    rscvisitor.setCutted(false);
		traverseTopDown(root, rscvisitor);
	}while(rscvisitor.getCutted());

    return null;
}
/**
 * Visit a XATNode to clean up tree.
 * @return Object
 * @param root The XATNode current visited.
 * @since 1.0
 */

public Object visit(XATNode root) {
    // keep the parent of this node.
    XATNode parent = root.getParent();
    // make the root of this node null, so that, it will not cofuse the pcm and rsc visitors.
    root.setParent(null);
    traverseTopDown(root, pcmvisitor);
    traverseTopDown(root, rscvisitor);
    // restore the original parent.
    root.setParent(parent);

    return null;
}
}
