package edu.wpi.cs.dsrg.xmldb.xat.component.sqlgeneration;

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;

/**
 * <pre>
 * IncrementalQuery is an ABC for the different query types.  It
 * contains the methods for adding a clause and getting the query statement.
 *
 * Each IncrementalQuery has one or more Vectors corresponding to the different
 * clauses in that query.  For example, an SQL statement will have SELECT, FROM, WHERE, etc
 * The query is built incrementally, that is, each operator will add one or more
 * clauses to the query via addClause.  
 * </pre>
 * @see IncrementalSQL
 * @see IncrementalXQuery
 * @author: Brad Pielech
 */
public abstract class IncrementalQuery {
 
	
	public edu.wpi.cs.dsrg.utils.BindingTable variableTable;
/**
 * SQLIncrementalStatement constructor comment.
 */
public IncrementalQuery(){
	
}
/**
 * SQLIncrementalStatement constructor comment.
 */
public IncrementalQuery(BindingTable varTable){
	variableTable = varTable;
}
/**
* <pre>
*  addClause accepts an QueryFragment.  It takes the
*  clauseText from the fragment and appends it to the
*  appropriate clause vector
* </pre>
* @param fragment the QueryFragment to add to the SQL statement
*/
public abstract void addClause(QueryFragment fragment);
/**
*  getSQLStatement returns 1 Query statement of the appropriate tyep that is the combination
*  of the various clause properties 
*
*/
public abstract String getStatement() throws SQLNoStatementExists; 
/**
 * Insert the method's description here.
 * Creation date: (1/9/2003 3:00:29 PM)
 * @return edu.wpi.cs.dsrg.utils.BindingTable
 */
public edu.wpi.cs.dsrg.utils.BindingTable getVariableTable() {
	return variableTable;
}
/**
 * initialize empties the vectors for each of the clauses.
 * 
 */
public abstract void initialize();
/**
 * Insert the method's description here.
 * Creation date: (1/9/2003 3:00:29 PM)
 * @param newVariableTable BindingTable
 */
public void setVariableTable(BindingTable newVariableTable) {
	variableTable = newVariableTable;
}
}
