package edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
/**
 * Extends the StatisticsObject to provide support for aggregate values of the statistic.
 * This also computes mean, min, and max values.  Statistics that wish to make use
 * of this functionality should derive from this class instead of StatisticsObject directly.<p>
 *
 * Statistics that calculate totals all at once (like THROUGHPUT or TOTAL_TUPLES_IN_QUEUES)
 * should derive from this class.  Statistics that want an aggregate, but only report
 * their values in increments (NUMBER_OF_TUPLES_DEQUEUED_TOTAL) should derive from AggregateDiffernce. 
 * <tt>getPropertyValue()</tt> returns the total for the statistic. 
 * For example, the TreeOutputRateCalculator
 * will want to keep track of how the output rate has changed over time.   
 * @author: Brad Pielech
 */
public abstract class AggregateStatisticsObject extends StatisticsObject {
	private double currentMean = 0;
	private double minValue = 10000;
	private double maxValue = -1;
	private double weight = -1;
	private java.lang.String mostRecent;
/**
 * MultiValueStatisticsObject constructor comment.
 */
public AggregateStatisticsObject() {
	super();
	mostRecent = "0";
}
/**
 * MultiValueStatisticsObject constructor comment.
 * @param name java.lang.String
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public AggregateStatisticsObject(String name, XATQueryObject queryObject) {
	super(name, queryObject);
	mostRecent = "0";
}
/**
 * Insert the method's description here.
 * Creation date: (4/2/2003 5:38:40 PM)
 * @return double
 */
public double getMaxValue() {
	return maxValue;
}
/**
 * Insert the method's description here.
 * Creation date: (4/2/2003 5:38:40 PM)
 * @return double
 */
public double getMinValue() {
	return minValue;
}
/**
 * Insert the method's description here.
 * Creation date: (4/18/2003 1:03:25 AM)
 * @return java.lang.String
 */
public java.lang.String getMostRecent() {
	return mostRecent;
}
/** 
 * Returns the mean of the statistic.  This is only relevant for statistics that
 * are aggregates where the property value represents a running sum.  Otherwise,
 * this value returned will be meaningless. 
 * @return java.lang.Object
 */
public double getPropertyValueMean() {	
	return currentMean;
}
/**
 * Insert the method's description here.
 * Creation date: (4/18/2003 1:03:25 AM)
 * @param newMostRecent java.lang.String
 */
public void setMostRecent(java.lang.String newMostRecent) {
	mostRecent = newMostRecent;
}
/**
 * Sets the property value.  For AggregateStatisticsObjects, this method
 * will calculate the new weighed mean and set the value to what was
 * passed in.  This assumes the newly passed in value represents the
 * total property value and not the difference from when it was calculated
 * previously. <p>
 *
 * For example, the THROUGHPUT calculator will report
 * how many tuples the tree has outputted in total, not just how many
 * were outputted since the last time it ran.  OTOH, the NUMBER_OF_TUPLES_OUTPUTTED
 * calculator will report how many tuples have been outputted since it last ran.  Therefore,
 * that calculator should derive from AggregateDifference
 * @param newPropertyValue java.lang.String
 */ 
public void setPropertyValue(String newPropertyValue) {
    if (getPropertyValue() != null) {
	    if(weight < 0){
			weight = Double.parseDouble(System.getProperty("AVERAGE_WEIGHT"));		
		} 
	    double newValue = Double.parseDouble(newPropertyValue); 
	    mostRecent = newPropertyValue;	        

        if(currentMean == 0){
			currentMean = newValue;
        } else {
			currentMean = (weight) * currentMean + ((1 - weight) * newValue);
        }        
        super.setPropertyValue(newPropertyValue);
        if(newValue > maxValue){
			maxValue = newValue;
        } else if (newValue < minValue){
			minValue = newValue;
        }
        /*
        //if(propertyName.equals("OUTPUT_RATE") && (this.category != StatisticsGatherer.OVERALL)){
        if(getPropertyName().equals("TOTAL_TUPLES_IN_QUEUES") && (getCategory() != StatisticsGatherer.OVERALL)){
	  		     
			System.out.println("New value: " + newPropertyValue);
			System.out.println("New mean: " + getPropertyValueMean());
			System.out.println("New updated value: " + getPropertyValue());			
			//System.out.println("UpdateCount: " + updateCount);
			System.out.println("\n");
			
        }
        */
    } else{
		super.setPropertyValue(newPropertyValue);
    }
}
}
