/*
 * Created on Mar 14, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer;


import java.io.Serializable;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
/**
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * The LightWeightStatistic class is responsible for containing
 * statistics about XATQueryObjects that will be sent across the netowrk
 * to the DistributionManager. 
 * 
 * Essentially the LightWeightStatistics is a Hash Table.
 * 
 * The key is the XATQueryObject for which the statistics are collected
 * The value is an Hashtable of statistics about the XATQueryObject.
 * 
 */
public class LightWeightStatistic implements Serializable{

	public Hashtable stats;
	private String machineName;
	private ArrayList statNames;

	public LightWeightStatistic(){
		this.machineName = "empty";
		stats = new Hashtable();
		statNames = new ArrayList();
	}
	
	public LightWeightStatistic(String machineName, ArrayList statsToCollect){
		this.machineName = machineName;
		stats = new Hashtable();
		statNames = statsToCollect;
		create();
	}
	
	public String getMachineName(){
		return this.machineName;
	}
	
	private void create(){
		
		StatisticsGatherer s = StatisticsGatherer.getStatisticsGather();
		Iterator i = s.getAllKnownQueryObjects();
		
		while(i.hasNext()){
			Hashtable temp = new Hashtable();
			XATQueryObject qo = (XATQueryObject) i.next();
			LightWeightQueryObject lwqo = qo.getLightWeightQueryObject();
			for(Iterator j = statNames.iterator();j.hasNext();){
				String statName = (String) j.next();
				if(qo.isValidStatistic(statName)){
					temp.put(statName,s.getValue(statName,qo));
				}
				
			}
		if(temp.size() >0)
			stats.put(lwqo, temp);
			
		}
		
	}
	
	public Enumeration getKeys() {
		return stats.keys();
	}	
	
	public Hashtable get(LightWeightQueryObject q){
		return (Hashtable) stats.get(q);
	}
	
	public String get(LightWeightQueryObject q, String statistic){
		Hashtable temp = this.get(q);
		return (String) temp.get(statistic);
	}
	
	public LightWeightQueryObject getQueryObject(XATNode n) throws Exception{
		XATStreamOperator op = (XATStreamOperator)n.getOperator();
		int id= op.getObjectID();
		
		Enumeration e = this.getKeys();
		while(e.hasMoreElements()){
			LightWeightQueryObject lwqo = (LightWeightQueryObject) e.nextElement();
			if(id == lwqo.getObjectID())
				return lwqo;
		}
		
		throw new Exception("I cannot find the LWQO representing " + n);
	}
	
	public String toString(){
		return this.machineName + " " + this.stats.toString();
	}

}
