package edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer;

import java.util.Hashtable;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;
/**
 * This class is an extention of an ArrayList that provides constant lookup time
 * by using a hashtable.  It also provides convience methods to get the first and last objects and
 * return the statistics object
 * @author: Brad Pielech
 */
public class ModifiedArrayList extends java.util.ArrayList {
	private Hashtable lookUpTable;
/**
 * ModifiedArrayList constructor comment.
 */
public ModifiedArrayList() {
	super();
	lookUpTable = new Hashtable();
}
/**
 * ModifiedArrayList constructor comment.
 * @param initialCapacity int
 */
public ModifiedArrayList(int initialCapacity) {
	super(initialCapacity);
	lookUpTable = new Hashtable();
}
/**
 * ModifiedArrayList constructor comment.
 * @param c java.util.Collection
 */
public ModifiedArrayList(java.util.Collection c) {
	super(c);
	lookUpTable = new Hashtable();
}
/**
 * If the object can be added to the array list, then
 * add it, but store it in the hash table with the QueryObject
 * as the key and the StatisticsObject as the value.  This will
 * allow for quicker look up.
 * @return boolean
 * @param o java.lang.Object
 */
public boolean add(Object o) {
	if(super.add(o)){
		StatisticsObject so = (StatisticsObject)o;
		lookUpTable.put(so.getAssociatedQueryObject(), o);
	}
	return true;
}
/**
 * Returns the object in the array who corresponds to the
 * given identifier.  The valid identifiers are found in <tt>StatisticsReference</tt>
 * @return java.lang.Object
 * @param identifier int
 */
public Object getByIdentifier(int identifier) {
	return getByIdentifier(identifier, false);
}
/**
 * Returns the object in the array who corresponds to the
 * given identifier.  The valid identifiers are found in <tt>StatisticsReference</tt>
 * @return java.lang.Object
 * @param identifier int
 * @param updateValues boolean true if the ModifiedArrayList should first update the statistitics, before
 * sorting and returning the value.  
 * 
 */
public Object getByIdentifier(int identifier, boolean updateValues) {
	//if we want to update the values first, then do so
	if(updateValues){
		//get the stats gatherer
		StatisticsGatherer stats = StatisticsGatherer.getStatisticsGather();
		Object oldCategory = stats.getCurrentCategory();
		Object[] objs = super.toArray();
		//iterate through the statistics objects stored in the array
		for (int i = 0; i < objs.length; i++){			
			StatisticsObject statsObj = (StatisticsObject)objs[i];
			stats.setCurrentCategory(statsObj.getCategory());
			//update the statistic for the given query object
			stats.updateValue(statsObj.getPropertyName(), statsObj.getAssociatedQueryObject());
		}
		stats.setCurrentCategory(oldCategory);
	}
	/** sort the list */	
    java.util.Collections.sort(this);
	switch (identifier) {		
		case StatisticsReference.LAST:{
			return getLast();
		}
		case StatisticsReference.LOWEST:{
			return getFirst();
		}
		case StatisticsReference.HIGHEST:{
			return getLast();
		}
		case StatisticsReference.AVERAGE:{			
			return get((this.size() / 2));
		}
		default: {
			return get(identifier);
		}
	}		
}
/**
 * Insert the method's description here.
 * Creation date: (1/7/2003 6:01:42 PM)
 * @return java.lang.Object
 */
public Object getFirst() {
	return get(0);
}
/**
 * Insert the method's description here.
 * Creation date: (1/7/2003 6:02:03 PM)
 * @return java.lang.Object
 */
public Object getLast() {
	return get(this.size() - 1);
}
/**
 * Insert the method's description here.
 * Creation date: (1/24/03 3:35:15 PM)
 * @return edu.wpi.cs.dsrg.xmldb.xat.component.statisticsgatherer.StatisticsObject
 * @param queryObject edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject
 */
public StatisticsObject getStatisticsObject(XATQueryObject queryObject) {
	return (StatisticsObject)lookUpTable.get(queryObject);
}
}
