package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.StringTokenizer;

import edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.Packet;

/**
 * @author 
 *
 * For Each server that we are going to receive a stream from has 
 * we have one instance of the receieve thread. This thread blocks on receive
 * and as soon as it gets a packet adds the packet to the packet pool
 */
public class RecvThread implements Runnable {
	/**Stores the reference to the packet pool*/
	PacketPool packetPool = null;
	/**Stores the reference to the client Socket*/
	Socket clientSocket = null;
	/**Stores the reference to the object input stream*/
	ObjectInputStream oInputStream = null;
	/**Indicates if the thread is enabled or disabled*/
	boolean enable = false;
	/**Stores qthe reference to the thread object*/
	Thread t = null;
	/**Stores the number of streams that this thread receives currently*/
	int countCurrentStreams = 0;
	/**Stores the name of the server*/
	String serverName  = null;
	
	/**
	 * @param packetPool
	 */
	public RecvThread(PacketPool packetPool,
					 String serverName,
					 String ipAddress,
					 String port,
					 String streamNames) 
					 throws CannotServeStreamException, IOException {
		this.packetPool = packetPool;
		int portNumber = Integer.parseInt(port);
		this.serverName = serverName;
		clientSocket = new Socket(ipAddress,portNumber);			
		System.out.println("Socket connected");
		OutputStream os = clientSocket.getOutputStream();
		
		System.out.println("Sending the streamnames "+streamNames);
		streamNames = streamNames +  "\n";
		this.countCurrentStreams = this.countStreams(streamNames);
				
		//Sending the name of the streams		
		os.write(streamNames.getBytes());
		os.flush();
		
		//Receving the acknowledgement from the server
		InputStream is = clientSocket.getInputStream();
		
		
		String temp = this.readLine(is);
		//System.out.println("Received ack "+temp);
		
		
		if(!(temp.compareTo("accepted\n") == 0))
		{
			throw (new CannotServeStreamException(temp));
		}
		else
		{		
			System.out.println("Acknowledgement Received");
			this.oInputStream = new ObjectInputStream(is);			
			enable = true;
			t = new Thread(this,"Recv Packet Thread");
			t.setDaemon(true);		
		}				 			
	}
	
	/** Blocks on recv for the server and then as soon as it receives a packet
	 * it sends the packet to the packet pool
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		while(enable)
		{
			//Receive the new Packets
			try{									
			Object o = this.oInputStream.readObject();			
			Packet p = (Packet) o;							
			if(p.getControlMessage().compareTo("end") == 0)
				{	
					System.out.println("End of the stream ="+p.getIdentifier());
					this.countCurrentStreams = this.countCurrentStreams - 1;
					if(this.countCurrentStreams == 0)
					{
						System.out.println("Shutting down Receive thread");					
						enable = false;
						continue;
					}				
				}			
			else			
				{			
					this.packetPool.addPacket(p);
					//System.out.println("Got a new packet with stream name :"+ ((Packet) o).getIdentifier() +" from server :"+this.serverName);
				}												
			}catch(IOException e)
			{	e.printStackTrace();
				if(enable == false)
					System.out.println("Thread stopped");
				else{								
					System.out.println("----Thread terminated----");
					System.out.println("----Receive thread Closed down-----");
					System.out.println("----Server shutdown----");
					//e.printStackTrace(); //no need for this line
					enable = false;
				}
			} catch (ClassNotFoundException e) {				
				//e.printStackTrace(); // no need for this line
			}
				
		}	
	}
	
	/**
	 * This method is used to stop the thread
	 */
	public void disable() throws IOException
	{
		this.enable = false;
		t.interrupt();
		this.clientSocket.close();		
	}

	/**
	 * 
	 */
	public void start() {		
		t.start();
	}

	/**
	 * 
	 */
	public void join() throws InterruptedException {
		t.join();	
	}
	
	/**
	 * This methid reads the 1st line of text from the input stream
	 * @param is
	 * @return
	 * @throws IOException
	 */
	private String readLine(InputStream is) throws IOException {
		String temp ="";				
		char c = 0;
		while(c!='\n')
		{
			c = (char) is.read();
			temp = temp + c;	
		}
		String streamNames = temp;		
		return streamNames;
	}
	
	/**
	 * Parses the stream names and then returns the number of streams
	 * @param streamNames
	 * @return
	 */
	int countStreams(String streamNames)
	{
		int count = 0;
		StringTokenizer st = new StringTokenizer(streamNames,",\n");
		while(st.hasMoreTokens())
		{
			st.nextToken();
			count = count + 1;
		}
		return(count);
	}

	/**
	 * Returns true if the thread is active
	 * @return boolean
	 */
	public boolean isAlive() {		
		return(t.isAlive());
	}
}
