package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client;

import java.io.IOException;
import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Iterator;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObjectImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.Source;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATMultiQueueStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.streamoperator.XATStreamOperator;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.LightWeightQueryObject;
/**
 * @author 
 * This is the main class that instantiates 
 * the components and then activates the components and 
 * co-ordinates the shutting down of the components 
 */

public class StreamClient extends XATQueryObjectImp implements XATStreamController,Runnable{
	/**Stores the reference to the threads that receive the packets from the server*/
	private RecvThread[] recvThreads= null;
	/**Stores the reference to the streams*/
	private HashMap streamTable = null;
	/**Stores the reference to the stream feeder*/
	private StreamFeeder sf = null;
	/**Stores the reference to the packet pool*/
	PacketPool pool = null;
	/**Stores the reference to the thread*/
	Thread t = null;
	/**Stores the reference to the Listener interface*/
	Listener listener = null;
	 
	/**
	 * Constructs the main Client
	 *@param inputFileName 
	 */
	public StreamClient(String inputFileName) throws InterruptedException, CannotServeStreamException, IOException, ServerNotFoundException {
		
		ClientConfigFileParser cfp = new ClientConfigFileParser(inputFileName);
		recvThreads = cfp.getRecvThreads();
		streamTable = cfp.getStreamTable();
		pool = cfp.getPacketPool();		
		sf = new StreamFeeder(streamTable,pool);
		t = new Thread(this,"StreamGenerator");
	}

/**
 * This connect the streams to the Query tree
 */
public void connectStreamsWithTree(XATTree queryTree) {

		XATNode[] leaves = queryTree.getLeaves();
		XATNode node, upperNode;
		XATOperator operator, upperOperator;
		XATQueue streamQ, queue;
		String streamID;
		int length = Array.getLength(leaves);

		if (length == 0) {
			System.out.println(
				"The query plan tree is empty. Stop processing ...\n");
		}

		if (length > streamTable.size()) {
			System.out.println(
				"The total number of stream is less than the leaves of query tree. Stop processing...\n");
		}

		/** attach each stream to the corresponding position in XATTree. 
		 *  To find the right position, start from leaves. */
		for (int i = 0; i < length; i++) {

			node = leaves[i];
			operator = node.getOperator();

			/** A XATTree should always starts from a Source node */
			if (operator instanceof Source) {
				streamID = ((Source) operator).getName();
				Stream currentStream = (Stream) streamTable.get(streamID);
				streamQ = currentStream.getStreamQueue();

				// The direct parent of the source node should be an instance of XATStreamOperator
				for (Iterator it = node.getParents(); it.hasNext();) {
					upperNode = (XATNode) it.next();
					upperOperator = upperNode.getOperator();

					if (upperOperator instanceof XATMultiQueueStreamOperator) {
						(
							(
								XATMultiQueueStreamOperator) upperOperator)
									.addInputQueue(
							streamQ);
					} else if (upperOperator instanceof XATStreamOperator) {
						((XATStreamOperator) upperOperator).setInputQueue(
							streamQ);
					} else {
						System.out.println(
							"Invalid operator or Tree only consists of Navigate operators!");
						return;
					}
					upperNode.deleteChild(node);
				}
				stats.addNewQueryObject(streamQ);
				/** clean up the statistics gatherer */
				stats.removeQueryObject(operator);

			} else {
				System.out.println("Invalid operator!");
			}
		}		
}
	
	/**
	 *  Starts the receiver and the stream feeder
	 */
	public void startGeneratingStreams()
	{
		t.start();		
	} 
	
	/**
	 * waits for the recv threads to end
	 * @throws InterruptedException
	 */
	public void join() throws InterruptedException
	{
		for(int i=0;i<recvThreads.length;i++)
			recvThreads[i].join();
	}
	
	
	/**
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		if(args.length != 1)
		{
			System.out.println("Usage: MainClient <clientconfig.xml>");
		}
		
		try {
			StreamClient mc = new StreamClient(args[0]);
		} catch (IOException e) {		
			e.printStackTrace();
		} catch (CannotServeStreamException e) {			
			e.printStackTrace();
		} catch (InterruptedException e) {			
			e.printStackTrace();
		}catch(ServerNotFoundException se)
		{
			System.out.println("----Shutting down the server-----");
		}
		
	}

	
	/**
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.XATStreamController#getStreamTotal()
	 */
	public int getStreamTotal() {
		return(streamTable.size());		
	}

	
	/**
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client.XATStreamController#getQueue(java.lang.String)
	 */
	public XATQueue getQueue(String streamName) {		
		Stream s = (Stream) this.streamTable.get(streamName);
		return(s.getStreamQueue());
	}

	/**
	 * @see java.lang.Runnable#run()
	 */
	public void run() {			
		for(int i=0;i<recvThreads.length;i++)
			recvThreads[i].start();			
		sf.start();
		try {
			join();
		} catch (InterruptedException e) {			
			e.printStackTrace();
		}
		sf.disable();	
		if(this.listener != null)
		{	
			listener.endOfStream();					
		}
	}
	
	/**
	 * Used to register the listener object.
	 * This interface is used to indicate endofStreams event
	 * @param l
	 */
	public void addListener(Listener l)
	{
		this.listener = l;
	}
	
	/* (non-Javadoc)
	 * @see edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject#getLightWeightQueryObject()
	 */
	public LightWeightQueryObject getLightWeightQueryObject() {
		return new LightWeightQueryObject("StreamClient",this.getClass().getName(),true);
	}
	
}
