package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.client;

import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueue;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.XATQueryObject;

/**
 * Defines an interface to the stream controller
 * @author: Yali Zhu
 */
public interface XATStreamController extends Runnable, XATQueryObject{
/**
 * Returns the total number of streams
 * @return int
 */
int getStreamTotal();

/**
 * Returns the Queue given that stores the tuples
 * coming from the stream. Note it takes in the stream name
 * as the parameter this stream name should be the same
 * as the one in the stream config file
 * @param name
 * @return XATQueue
 */
public XATQueue getQueue(String streamName);

/**
 * This method connects initialized streams (more precisely: stream queues) with 
 * leave operators (skip source and navigate operators) of a given XATTree.
 *
 * *****************************************************
 * The method requires that a XATTree will have source operators as its leaves.
 * No navigation operators are in the XATTree.
 * *****************************************************
 *
 * @param queryTree
 */
public void connectStreamsWithTree(XATTree queryTree);

/**
 * Starts the thread that generates the streams and 
 * that is starts placing the tuples in the queue
 */
public void startGeneratingStreams();

/**
 * This method is used to add the listener to listen to the
 * end of stream event.
 * Note when the stream client finishes receving the streams.
 * It uses this listener interface to inform that the streams are over
 * @param tester
 */
void addListener(Listener l);

/**
 * Blocks until the stream generator finishes its task.
 */
public void join() throws InterruptedException;

}
