package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.Socket;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.*;

/**
 * @author 
 * This class stores all the properties of the client
 * like the socket and the object output stream etc...
 */

public class Client {	
	/**Stores the reference to the socket*/
	private Socket socket;
	/**Stores the reference to the objectoutput stream*/
	private ObjectOutputStream outputStream = null;
	private int counter;
	private int clientNumber;
	private NetworkPacket np;
	private long timeToStream;
	private long startTime;
	
	/**
	 * Constructs a new Client
	 * @param socket
	 */
	public Client(Socket socket, long length, int clientNumber) throws IOException
	{
		this.socket = socket;
		this.outputStream = new ObjectOutputStream(socket.getOutputStream());	
		np = new NetworkPacket(300);
		counter = 0;		
		timeToStream = length;
		startTime = System.currentTimeMillis();
		this.clientNumber = clientNumber;
		System.out.println("Will stream to " + socket + " for " + timeToStream + " milliseconds");
	}
	
	/**
	 * Sends the packet to the client
	 * @param packet
	 */
	public boolean send(Packet packet) throws IOException {
		
		long currentTime = System.currentTimeMillis();
		
		//Only stream for as long as we are supposed to.
		if(currentTime - startTime > timeToStream){
			this.outputStream.writeObject(np);
			System.out.println("Sending " + np.size() + " tuples to " + this.socket.toString());
			System.out.println("Done Streaming to " + socket);
			return false;
		}
		// Fill up the NetworkPacket object
		if(counter < np.totalSize()){
			String id = packet.getIdentifier();
			Packet p = new Packet(id+clientNumber,packet.getObject());
			np.addPacket(p);
			counter++;
			return true;
		}
		//When it is full, send it and start collecting more packets.
		else{
			System.out.println("Sending " + np.size() + " tuples to " + this.socket.toString());
			this.outputStream.writeObject(np);	
			np = null;
			np = new NetworkPacket(300);
			counter=0;
			return true;
		}
			
	}

	
	/**
	 * Closes the socket
	 */
	public void close(){
		try {
			np = null;
			counter = 0;
			socket.close();
			outputStream.close();
		} catch (IOException e) {
			//System.out.println("---Got Exception when trying to close the socket---");			
		}
				
	}
	
	public String toString(){
		String ret = new String();
		
		ret+= this.socket.getInetAddress() + ":" + this.socket.getPort();
		
		return ret;
	}

}
