package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.io.EOFException;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.connection.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.Packet;

/**
 * @author 
 * Accepts new clients on a server socket creates a new
 * socket and then adds the client socket to the client pool
 * This class also implements the protocol that completes the hand shake with 
 * the client
 * 
 */
public class ClientAcceptor implements Runnable {
	/**The variable that can disable and enable the functioning of the acceptor*/
	private boolean enable = true;
	/**Stores the reference to the port number*/
	private int port = -1;
	/**Stores the reference to the server socket*/
	private ServerSocket socket = null;
	/**Stores the reference to the thread*/
	private Thread t = null;
	/**Stores the reference to the =ClientAdder interface of the client pool*/
	private PacketAdder adder = null;
	/**Stores the reference to the main class*/
	private AccessStream main;

	/**
	 * Constructs a new ClientAcceptor
	 * @param portNumber
	 * @param adder
	 * @throws IOException
	 */
	public ClientAcceptor(int portNumber, PacketAdder adder, AccessStream main)
		throws IOException {
		this.port = portNumber;
		this.adder = adder;
		enable = true;
		this.main = main;
		t = new Thread(this, "Acceptor");
		t.start();
	}

	/**
	 * This methosd accepts new clients and makes the entries into the client pool
	 * 1st it checks to see if the streams needed by the clients can be served
	 * then sends an ancknowledgement to the client. In case the streams can all be served
	 * then we send a new line as ack, or else we send the name of the 1st stream that cannot be
	 * served followed by the new line to the client
	 * @see java.lang.Runnable#run()
	 */
	public void run() {
		try {
			ServerSocket socket = new ServerSocket(port);
			System.out.println("Starting ClientAdder on port: " + port);
			int i = 1;
			for (;;) {
				Socket inc = socket.accept();
				System.out.println(
					"Connecting to a new distribution manager at"
						+ inc.getInetAddress());
				Thread t = new ClientAcceptorHandler(inc, adder, main,i);
				t.setName("PacketReceiverHandler" + i);
				t.setDaemon(true);
				t.start();
				i++;
			}
		} catch (IOException ioe) {
			System.out.println(
				"Could not start the Tuple Receiver: " + ioe.getMessage());
		}

	}

	/**
	 * This methid reads the 1st line of text from the input stream
	 * @param is
	 * @return
	 * @throws IOException
	 */
	private String readLine(InputStream is) throws IOException {
		String temp = "";
		char c = 0;
		while (c != '\n') {
			c = (char) is.read();
			temp = temp + c;
		}
		String streamNames = temp;
		return streamNames;
	}

	/**
	 * This method is used to stop the acceptor used by the 
	 * XATStreamGenerator when the stream generator shutdowns
	 */
	public void disableAcceptor() throws IOException {
		enable = false;
		try {
			socket.close();
		} catch (Exception e) {
			System.out.println("Caught socket exception:" + e.getLocalizedMessage());
		}
		t.interrupt();
	}

	/**
	 * 
	 */
	public void join() throws InterruptedException {
		this.t.join();

	}
}
/**
 * 
 * @author Timothy M. Sutherland
 * @author tims@cs.wpi.edu
 * @author http://www.tsutherland.com
 * @version 1.0
 * 
 * Dec 8, 2003
 * 
 * The CleintAccepterHandler class is responsible for
 * maintaining a connection between the stream generator
 * and a distribution manager. The ClientAcceptor will spawn one of these threads
 * whenever a Connection from a new location is sent.
 * 
 */
class ClientAcceptorHandler extends Thread {

	private Socket incoming;
	private PacketAdder adder;
	private AccessStream main;
	private int handlerNumber;
	
	public ClientAcceptorHandler(
		Socket i,
		PacketAdder adder,
		AccessStream main,int handlerNumber) {
		this.incoming = i;
		this.adder = adder;
		this.main = main;
		this.handlerNumber = handlerNumber;
		
	}

	public void run() {

		boolean enable = true;
		try {
			ObjectInputStream in =
				new ObjectInputStream(incoming.getInputStream());
			while (enable) {
				//Receive the new Packets

				//Get the connection packet

				Object o = in.readObject();
				Packet cp = (Packet) o;
				Connection c = (Connection) cp.getObject();
				String streamName = new String(c.getStreamName());

				// Get the connection object
				if (!(c instanceof SendStream))
					throw new Exception("The connection object sent to the stream generator is not of the type: Stream to Machine");
	
				SendStream tempSend = (SendStream) c;
				
				// Connect to the machine we will send the stream to.
				
				Socket toStream =
					new Socket(
						c.getToIPAddress(),c.getToPort());

				// Make the new client

				Client client = new Client(toStream, c.getStreamLength(), handlerNumber);
				System.out.println("Creating new client:" + toStream +handlerNumber);
				// Send the schema
				XATQueueSchema schema =
					main.getStream(streamName).getXATSchema();
				Packet p = new Packet(streamName, schema, "schema");
				client.send(p);

				// Add the client to the sender pool so we send the streams
				// to the client

				adder.addClient(client, streamName);
				
				// Tell the machine where to put the incoming tuples
				Socket s = new Socket(tempSend.getToIPAddress(),tempSend.getToMachine().getConnectionListenerPort());
				ObjectOutputStream os = new ObjectOutputStream(s.getOutputStream());
				Connection c2 = new ReceiveStream(streamName+this.handlerNumber,c.getToOperator(),c.getToQueueNumber());
				Packet p2 = new Packet("connect",c2);
				os.writeObject(p2);

			}
		} catch (EOFException eofe) {
			// do nothing	
		} catch (SocketException se) {
			System.out.println(se.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
