
package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

/**
 * @author
 *
 * This class maintains a registration table
 * Each client can reveive one or more than one stream
 * Given a stream this class maintains the list of clients
 */
public class ClientPool implements PacketAdder, Accessor {
	/**This stores the registration of the stream to the client
	 * Note the key is the stream Name and the value is an arraylist
	 * of client sockets 
	 */
	private HashMap streamTable = null;
	
	/**
	 * Creates a new Client pool 
	 */
	public ClientPool() {	
	 this.streamTable = new HashMap();
	}

	/**
	 * This method adds the clients to the stream Table
	 * @param c
	 * @param streamName
	 * @throws StreamNotFoundException
	 */
	public synchronized void addClient(Client c,String streamName) throws StreamNotFoundException
	{		
		if(c == null || streamName == null)
			throw new IllegalArgumentException();
			
		ArrayList al = (ArrayList) this.streamTable.get(streamName);	
		if(al == null)
		{
			throw new StreamNotFoundException("No such Stream: "+streamName);
		}
		
		al.add(c);									
	}
	
	/**
	 * This methods adds a new stream to the stream table
	 * @param streamName
	 * @throws DuplicateStreamNameException
	 */
	public synchronized void addStream(String streamName) throws DuplicateStreamNameException
	{	
		if(streamName == null)
		 throw new IllegalArgumentException();
		if(this.streamTable.get(streamName)!= null)
		{
			throw new DuplicateStreamNameException("2 Streams cannot have the same Name");
		}
		streamTable.put(streamName,new ArrayList());
	}
	
	/**
	 * This method returns an array of client Clients registered for 
	 * a single stream
	 * @param streamName
	 * @return
	 */
	public Client[] getClients(String streamName)
	{
		if(streamName == null)
			 throw new IllegalArgumentException();	
		ArrayList al= (ArrayList)this.streamTable.get(streamName);
		if(al == null)
		 return(null);
		else 
		 return((Client[])al.toArray(new Client[0]));
	}
	
	/**
	 * This method returns the name of the streams
	 * @return String[]
	 */
	public String[] getStream()
	{
		Set keySet = this.streamTable.keySet();
		return((String[])keySet.toArray(new String[0]));		
	}

 	/**
 	 * This method removes a client from the stream
 	 * Note this method will not be called frequently therefore
 	 * the scan of the hashMap is good enough
	 * @param s
	 */
	public synchronized void removeClient(Client client) 
 	{ 		
 		Collection c = this.streamTable.values();
 		
 		for (Iterator iter = c.iterator(); iter.hasNext();) {
			ArrayList list = (ArrayList) iter.next();
			for(int i=0;i<list.size();i++){
				if(list.get(i) == client)
				{
					list.remove(i);		
				}
			}			
		}
		client.close(); 		
  	}
 	
 	/**
 	 * Removes the stream from the table used in case the
 	 * stream gets over or the total time expires 	 
 	 * @param streamName
	 * @throws StreamNotFoundException
	 */
	public synchronized void removeStream(String streamName) throws StreamNotFoundException
 	{ 		
 		if(this.streamTable.get(streamName)== null)
 		 throw new StreamNotFoundException();
 		 
 		this.streamTable.remove(streamName);
 	}

	/**
	 * Cleans up all the clients 
	 */
	public void cleanup() throws IOException {		
		Collection c = this.streamTable.values();
 		
		for (Iterator iter = c.iterator(); iter.hasNext();) {
			ArrayList list = (ArrayList) iter.next();
			for(int i=0;i<list.size();i++){
				((Client) list.get(i)).close();
			}			
		}  			
	}
	
	/**
	 * @param streamName
	 * @return
	 */
	public boolean isStream(String streamName)
	{
		if(streamName == null)
		 throw(new IllegalArgumentException());
		 else
		 {		 
		 	if(streamTable.containsKey(streamName))
		 		return(true);
		 	else
		 		return(false);
		 }		
	}
		
	
}
