
package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValueImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.punctuations.Punct;
import edu.wpi.cs.dsrg.xmldb.xat.common.punctuations.SingleValuePunct;
import edu.wpi.cs.dsrg.xmldb.xat.common.punctuations.WildcardPunct;

/**
 * This stores the information about a particular column in the
 * the stream schema
 */
public class Column {
	/**Stores the reference to the coulmn name*/
	private String columnName = null;
	/**Stores the reference to the column type*/
	private String type = null;
	/**
	 * Constructs a new column
	 * @param id
	 * @param type
	 */
	public Column(String id,String type) {
		this.columnName = id;
		this.type = type;
	}
	


	/**
	 * @return
	 */
	public String getColumnName() {
		return columnName;
	}

	/**
	 * @return
	 */
	public String getType() {
		return type;
	}
	
	/**
	 * Makes a new XATDataValue depending on the type and then
	 * new the XATDataVAlue will help in making the tuples
	 * @param value
	 * @return
	 */
	public XATDataValue makeXATDataValue(String value) throws RecordParseException {	
		XATDataValue temp = null;
		try{
			if (value.startsWith("(") && value.endsWith(")")) { //punctuation
				String punctValue = value.trim().substring(1, value.length()-1);
				Punct punct = null;
				if (punctValue.equals("*")) {
					punct = new WildcardPunct();
				} else {
					punct = new SingleValuePunct(new Integer(punctValue));
				}
				temp = new XATDataValueImp(punct);
				return temp;
			} 
			
			if(type.toLowerCase().compareTo("string") == 0)
			{
				temp = new XATDataValueImp(value);				
			}
			if(type.toLowerCase().compareTo("int") == 0)
			{
				temp = new XATDataValueImp(new Integer(Integer.parseInt(value)));
			}
			if(type.toLowerCase().compareTo("long") == 0)
			{
				temp = new XATDataValueImp(new Long(Long.parseLong(value)));
			}
			if(type.toLowerCase().compareTo("float") == 0)
			{
				temp = new XATDataValueImp(new Float(Float.parseFloat(value)));
			}
			if(type.toLowerCase().compareTo("double") == 0)
			{
				temp = new XATDataValueImp(new Double(Double.parseDouble(value)));
			}
		} catch(Exception e) {
			//System.out.println("The value "+ value + " is not a valid" + this.type + ". Will store as a string.");
			temp = new XATDataValueImp(value);
			//throw(new RecordParseException(e));		
		}
		return(temp);
	}
}
