package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.util.ArrayList;
/**
 * Insert the type's description here.
 * Creation date: (2/6/2003 12:58:15 AM)
 * @author: Yali Zhu
 */
public abstract class RandomGenerator {   

	static final double PI = 3.141592654;
    static final int IA = 16807;
    static final long IM = 2147463647;
    static final double AM = (1.0 / IM);
    static final int IQ = 127773;
    static final int IR = 2836;
    static final int NTAB = 32;
    static final double NDIV = (1 + (IM - 1) / NTAB);
    static final double EPS = 1.0e-7;
    static final double RNMX = (1.0 - EPS);
    protected long iy = 0;
    //static protected long seed = -1;
    protected long seed = -1;
    protected long[] iv = new long[NTAB];
    static protected double[] cof = {
            76.18009172947146,
            -86.50532032941677,
            24.01409824083091,
            -1.231739572450155,
            0.1208650973866179e-2,
            -0.5395239384953e-5 };
    /**Stores the intervals for the start and the mean times*/
    double[][] interArrivals = null;
    /**Stores the current index with in the array*/
     int index = -1;
    
/**
 * @param vecInterArrivals
 */
public RandomGenerator(ArrayList vecInterArrivals) {

	this.interArrivals =(double[][]) vecInterArrivals.toArray(new double[0][0]);
}
 
/**
 * Constructs a new Random Generator
 * @param vecInterArrivals
 */
public RandomGenerator(ArrayList vecInterArrivals,long seed)
{
 	this.interArrivals = (double[][]) vecInterArrivals.toArray(new double[0][0]);
 	this.seed = seed;
}
    
    
            
/**
 * The function gammln is the natural log of the gamma function                                *
 * Returns the value ln(gamma(xx)) for xx > 0.                     
 * Creation date: (2/8/2003 10:35:44 PM)
 * @return double
 * @param xx double
 */
public double gammln(double xx) {
    /**
     * Internal arithmetic will be done in double precision, a nicety that you can omit if ve-gure
     * accuracy is good enough. 
     */

    double x, y, tmp, ser;
    int j;

    y = x = xx;
    tmp = x + 5.5;
    tmp -= (x + 0.5) * Math.log(tmp);
    ser = 1.000000000190015;

    for (j = 0; j <= 5; j++)
        ser += cof[j] / ++y;

    return -tmp + Math.log(2.5066282746310005 * ser / x);
}

/**
 * Insert the method's description here.
 * Creation date: (2/7/2003 4:09:41 PM)
 */
public void updateIntervals(long currentTimeStamp)
{
	try {
			//Note end of interval is a just a hack to get the code running
			//would like to recode the methid. No need of useing an arraylist to store the interval list
			//you could simlpy store it as a primitve array
			if(this.index+1 < this.interArrivals.length)
			{				
				if (this.interArrivals[this.index+1][0]<= currentTimeStamp) 
				{
				this.index = this.index + 1;
				double[] currentInterval = (double [])interArrivals[this.index];
			
				int avgInterval = (int) currentInterval[1];
				System.out.println("-----------------------");
				
				System.out.println("Setting the new mean to "+avgInterval);
				
				System.out.println("-----------------------");
				this.updateMeanInterval(avgInterval);														
				}		
			}
		} catch (IndexOutOfBoundsException e) {
			System.out.println(e.toString());
			e.printStackTrace();
		}
			
}

public abstract double generateRandom(long currentTimeStamp);

/**
 * Insert the method's description here.
 * Creation date: (2/8/2003 9:57:23 PM)
 * @return long
 */
public long getSeed() {
	return seed;
}
/**
 * Insert the method's description here.
 * Creation date: (5/7/2003 3:41:06 AM)
 * @param newSeed long
 */
public void setSeed(long newSeed) {
    seed = newSeed;
}
/**
 * Insert the method's description here.
 * Creation date: (2/6/2003 12:59:37 AM)
 */
public double uniformRand() {
    int j;
    long k;
    //static long iy = 0;
    //static long [] iv = new static long [NTAB];
    double temp;

    //Initialize seed.
    if ((seed <= 0) || (iy == 0)) {

        if (- (seed) < 1)
            seed = 1; // Be sure to prevent seed = 0.
        else
            seed = - (seed);

        // Load the shuffle table (after 8 warm-ups).
        for (j = NTAB + 7; j >= 0; j--) {
            k = (seed) / IQ;
            seed = IA * (seed - k * IQ) - IR * k;

            if (seed < 0)
                seed += IM;

            if (j < NTAB)
                iv[j] = seed;
        }

        iy = iv[0];
    }

    // Start here when not initializing.
    // Compute seed=(IA*seed) % IM without over-flows by Schrage's method.
    k = (seed) / IQ;						
    seed = IA * (seed - k * IQ) - IR * k;	

    
    if (seed < 0)
        seed += IM;
    j = (int) (iy / NDIV); 	// Will be in the range 0..NTAB-1.
    iy = iv[j]; 			// Output previously stored value and refill the shuffle table.

    iv[j] = seed;
    temp = AM * iy;

    if (temp > RNMX)
        return RNMX; // Because users don't expect endpoint values.
    else
        return temp;

}

/**
 * Updates the mean interarrival time
 * @param avgInterval
 */
public abstract void updateMeanInterval(int avgInterval);
	
	
}
