package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATDataValue;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTupleImp;

/**
 * @author
 * This stores the complete information about the schema of the
 * stream
 */
public class Schema {
	/**This stores an array of Columns
	 * Each cloumn contains a name and a type*/
	ArrayList columns = null;
	Object punctValue = null;
	String streamName;
	/**
	 * Makes the schema
	 */
	public Schema(String streamName) {
		columns =  new ArrayList();		
		this.streamName = streamName;
	}

	/**
	 * Adds a column to the schema
	 * @param a
	 */
	public void addColumn(Column a)
	{
		 columns.add(a);
	}
	
	/**
	 * Adds a column
	 * @param columnName
	 * @param type
	 */
	public void addColumn(String columnName,String type)
	{
		Column c = new Column(columnName,type);
		columns.add(c);
	}
	
	
	/**
	 * @param arrayValues
	 * @return
	 */
	private XATTuple makeTuple(String[] arrayValues) throws RecordParseException	
	{
		if(arrayValues == null || arrayValues.length != this.columns.size())
			throw(new IllegalArgumentException());
		 
		XATTuple xatTuple = new XATTupleImp();
		for(int i=0; i<arrayValues.length; i++) {
			XATDataValue xatDataValue = ((Column) this.columns.get(i)).makeXATDataValue(arrayValues[i]);
			xatTuple.addCell(xatDataValue);	
		}
		return xatTuple;
	}
	
	/**
	 * Converts a record read from the file to a Tuple using the
	 * schema
	 * @param record
	 * @param attrDelimeter
	 * @return
	 * @throws RecordParseException
	 */
	public XATTuple parse(String record,String attrDelimeter) throws RecordParseException
	{
		record = record.trim();
		String[] arrayStrings = new String[this.columns.size()];
		
		MyTokenizer st = new MyTokenizer(record,attrDelimeter);
		
		try{
		
			for(int i=0;st.hasMoreTokens();i++)
			{
				arrayStrings[i] = st.nextToken();
			}
		}catch(ArrayIndexOutOfBoundsException ae)
		{
			//System.out.println("Possible Corrupt Record: " + record +".");
		}
		return(makeTuple(arrayStrings));		
		
	}

	/**
	 * @param columnName
	 * @return
	 */
	public int getPosition(String columnName) {
		int i = 0;
		for(;i<columns.size();i++)
		{
			if(((Column) columns.get(i)).getColumnName().compareTo(columnName) == 0)
				break;
			
		}
		return(i);
	}
	
	/**
	 * @return
	 */
	public XATQueueSchema makeXATSchema()
	{
		Vector vecColNames = new Vector();
		for(int i=0;i<this.columns.size();i++)
		{
			vecColNames.add(((Column)this.columns.get(i)).getColumnName());
		}		
		XATQueueSchema schema = new XATQueueSchema(vecColNames);
		schema.setName(streamName);
		return(schema);
	}

}
/**
 * @author
 * This class uses the string tokenizer but considers the string within double quotes
 * the same token even if it has delimeters
 */
class MyTokenizer
{

	StringTokenizer st = null;
	/**
	 * @param record
	 * @param attrDelimeter
	 */
	public MyTokenizer(String record, String attrDelimeter) {		
		st = new StringTokenizer(record,attrDelimeter);
	}
		
	/**
	 * @return
	 */
	public boolean hasMoreTokens()
	{
		return(st.hasMoreTokens());
	}
	
	/**
	 * Returns the new token
	 * @return
	 */
	public String nextToken()
	{	
		String token = "";	
		String newToken = st.nextToken();		
		if(newToken.charAt(0) == '"')
		{	
			if(!(newToken.charAt((newToken.length()-1))== '"'))
			{
			
			while(true)
			{			
				newToken = newToken + " "+token;
				if((token = st.nextToken()).endsWith("\""))
				{	newToken = newToken + " "+ token;
					break;
				}
			}
		}
	}
		return(newToken);		
}
	
	
	
}

