package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.io.IOException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;
import edu.wpi.cs.dsrg.xmldb.xat.component.distribution.packet.*;

/**
 * @author 
 *
 * This class sends the tuple to the clients who have
 * registered for a particular stream
 */
public class Sender {
	
	/**This stores the referece to the client pool*/
	private Accessor accessor = null;
	
	/**
	 * Constructs a sender
	 * @param accessor
	 */
	public Sender(Accessor accessor)
	{
		if(accessor == null)
		 throw(new IllegalArgumentException());
		this.accessor = accessor;
	}
	
	/**
	 * This method constructs a new packet with the tuple
	 * and the stream name ahd then sends the packets to the
	 * clients
	 * @param streamName
	 * @param tuple
	 */
	public void send(String streamName,XATTuple tuple) throws IOException 	{
		if(streamName == null || tuple == null)
		{
			throw new IllegalArgumentException();
		}
		
		Client[] arrClient= this.accessor.getClients(streamName);
		
		Packet packet = new Packet(streamName,tuple,"tuple");
		if(arrClient.length != 0)
		{
			this.sendPacket(arrClient,packet); 			 
		}			
	}

	/**
	 *  
	 */
	public void cleanup() {

	}

	/**
	 * @param string
	 */
	public void sendEndNotification(String streamName)
	{		
		Client[] arrClient= this.accessor.getClients(streamName);
		Packet packet = new Packet(streamName,"end");
		if(arrClient != null)
			this.sendPacket(arrClient,packet);
	}
		
	
	
	/**
	 * Sends the packet to the other end
	 * @param arrClient
	 * @param packet
	 * @throws IOException
	 */
	private void sendPacket(Client[] arrClient,Packet packet) 
	{
		boolean isOK = true;
	
		if(arrClient ==null)
			throw(new IllegalArgumentException());
			for(int i=0;i<arrClient.length;i++)
			{			
				try {
			
					isOK = arrClient[i].send(packet);
					
					if(!isOK){
						accessor.removeClient(arrClient[i]);
					}
					
				} catch (IOException e) {
					accessor.removeClient(arrClient[i]);
					System.out.println("--Client " +arrClient[i].toString() +" disconnected from " + packet.getIdentifier() +" --");	
				}							
			}
		}					
	}

