package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATQueueSchema;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;

/**Note still left to code will code once we finish coding the new parser for the server config file

/**
 * A Parser that parses the file generated by the 
 * data generator
 * @author Luping Ding
 *
 */
public abstract class StreamTupleReaderTextImp  implements StreamTupleReader{
	/**Stores the name of the stream tuple reader*/
	private String identifier = null;
	/**Stores the name of the input File*/
	String[] inputFiles = null;
	/**Sores the attribute delimeter*/
	String attrDelimeter = null;
	/**Stores the record delimeter*/
	char recDelimeter;
	/**Stores the reference to the schema*/
	Schema schema  = null;
	/**Stores the reference to the buffered reader used to lines from the file*/
	BufferedInputStream in = null;		
		
	/**Maintains a count of the number of tuples read*/
	int count = 0;
	/**
	 * @param inputFileName
	 * @param attrDelimeter
	 * @param recDelimeter
	 * @param schema
	 *
	 */
	public StreamTupleReaderTextImp(String identifier,
									String[] inputFileNames, 
									String attrDelimeter,
									char recDelimeter,
									Schema schema) throws FileNotFoundException {
	this.identifier = identifier;
	this.inputFiles = inputFileNames;
	this.attrDelimeter = attrDelimeter;
	this.recDelimeter = recDelimeter;
	this.schema = schema;
	File f = new File(inputFiles[0]);
	double size= f.length()/1048576.0;
	System.out.println("Now generating stream " + identifier + " from file " + inputFiles[0]+".");
	System.out.println("The size of this file is "+size+"MB.");
	in = new BufferedInputStream(new FileInputStream(inputFiles[0]));
		
	}
		
	
	/**
	 * Parses the input file and returns a new Tuple. Note 
	 * each time the parsing done is tuple by tuple
	 * @return XATStreamTuple
	 */
	
	public XATTuple getNextTuple(long currentTimeStamp) throws IOException, RecordParseException
	{
		XATTuple tuple = null;		
			char ch;
			String strRecord="";

			while((ch = (char) in.read()) != this.recDelimeter)
			{
				if((int)ch == 65535){
					//The file is ended
					//for now repeat the file
					System.out.println("The file " + inputFiles[0] + " has no more entries. Will start from beginning.");
					in.close();
					in = new BufferedInputStream(new FileInputStream(inputFiles[0]));
					strRecord="";
					return null;
				}else{	
					strRecord = strRecord + ch;
				}
			}
			tuple = schema.parse(strRecord,this.attrDelimeter);
			
			this.setTimeStamp(currentTimeStamp,tuple);
						
			count = count + 1;									
					
		return(tuple);
	}
	
	public abstract void  setTimeStamp(long currentTimeStamp,XATTuple tuple) throws RecordParseException ;
	
	/**
	 * Returns the XATSchema
	 * @see edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server.StreamTupleReader#getXATSchema()
	 */
	public XATQueueSchema getXATSchema() {		
		return this.schema.makeXATSchema();
	}
	

}
