package edu.wpi.cs.dsrg.xmldb.xat.component.streamgenerators.server;
import java.io.EOFException;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.text.DecimalFormat;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStampException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTuple;

/**
 * @author
 * This is the main class that instantiates all the other components
 * and then starts the streams once the components are ready
 */
public class XATStreamGenerator implements AccessStream {
	
	/**Stores the reference to the list*/
	private Linearlist linearlist;
	/**Stores the name of the server config file Name*/
	private String configFileName = null;
	/**Stores the reference to the XATStreams*/
	private HashMap streams = null;
	/**Stores the reference to the client acceptor*/
	private ClientAcceptor clientAcceptor = null;
	/**Stores the reference to the Client pool*/
	private ClientPool clientPool = null;
	/**Stores the reference to the sender*/
	private Sender sender = null;
	/**Stores the total run time for the stream generator*/
	private long totalRunTime = -1;
	/**Stores the time at which the stream generator starts*/
	private long startTime = -1;
	private XATStream[] arrStreams;
	 
	/**
	 * Constructs a new XATStreamGenerator. 
	 * @param configFileName
	 * @throws IOException
	 */
	public XATStreamGenerator(int portNumber,String configFileName) throws DuplicateStreamNameException, IOException, ParsingException {
		if(configFileName == null ||portNumber < 1)
		 	throw new IllegalArgumentException();
		 			 	
		this.configFileName = configFileName;
		streams = new HashMap();
		ServerConfigFileParser sc = new ServerConfigFileParser(configFileName);
		arrStreams = sc.getStreams();
		this.totalRunTime = sc.getTotalTime();
		//Instantiating the linear list
		this.linearlist = new Linearlist();		
		//Loading the hash table
		for(int i=0;i<arrStreams.length;i++)									
			streams.put(arrStreams[i].getStreamName(),arrStreams[i]);
				
		clientPool = new ClientPool();
		//Add the streams to the client pool
		for(int i=0;i<arrStreams.length;i++)
		{
			clientPool.addStream(arrStreams[i].getStreamName());
		}
		
		sender = new Sender(clientPool);
	
		clientAcceptor = new ClientAcceptor(portNumber,clientPool,(AccessStream)this);		
				
		startTime = System.currentTimeMillis();
		
		makeList();		
		//System.out.println("Now generating streams on Port " + portNumber);
		this.run();
		this.clientAcceptor.disableAcceptor();
		this.clientPool.cleanup();
		this.sender.cleanup();
	}
	
	/**
* Actually generates the tuples. Pools each of the
* streams to generate the new Tuples
*/
public void run() {
	
	long time = System.currentTimeMillis();
	long updatedTime = System.currentTimeMillis();
	
	// Assume you stream no more than 4 streams.
	
	long[] numTuples = new long[4];
	numTuples[0]=0;
	numTuples[1]=0;
	numTuples[2]=0;
	numTuples[3]=0;
	
	// set debug flag of each stream to be the same as stream controller.
	//--long globaltime = System.currentTimeMillis() - startTime;	

	while (!(linearlist.isEmpty())&& (!Thread.currentThread().isInterrupted())) {
	
		updatedTime = System.currentTimeMillis();
				
		//waiting for the stream to be scheduled
		try {
			//Calculates the amount of time one can sleep						
			long margin = this.linearlist.getFirstKey() - (System.currentTimeMillis() - startTime);
		 
			if (margin > 100) {
				Thread.sleep(margin);
			}
			
		} catch (InterruptedException e1) {
			e1.printStackTrace();
		}

		XATStream currentStream = null;

		long temptime = linearlist.getFirstKey();		 		

			try {
				Object[] ob = (Object[]) linearlist.removeFirst();										
				currentStream = (XATStream) ob[0];
				DecimalFormat df = new DecimalFormat("0.00#");
				Runtime rt = Runtime.getRuntime();
				XATTuple tuple = (XATTuple) ob[1];						
				
				//send the new tuple
				sender.send(currentStream.getStreamName(),tuple);
				numTuples[currentStream.getStreamNumber()]++;
				if(updatedTime - time > 600000) { 
					long totalTuples = numTuples[0] + numTuples[1] + numTuples[2] + numTuples[3];
					
					System.out.println("------");
					System.out.println("Outputted " + totalTuples + " total tuples.");
					for(int j=0 ; j < streams.size() ; j++){
						System.out.println("Stream " + arrStreams[j].getStreamName() +": "+numTuples[j]);
					}
					System.out.println("The last tuple outputted was from " + currentStream.getStreamName() +": "+tuple);
					time = System.currentTimeMillis();	
					
					System.out.println("-------------Memory Information --------------");
					System.out.println("Free memory: " + df.format(((double)rt.freeMemory()/1024.0/1024.0)) + "MB");
					System.out.println("Total memory: " + df.format(((double)rt.totalMemory()/1024.0/1024.0)) + "MB");
					System.out.println("Max memory: " + df.format(((double)rt.maxMemory()/1024.0/1024.0)) + "MB");
					System.gc();
					System.out.println("Free memory: " + df.format(((double)rt.freeMemory()/1024.0/1024.0)) + "MB");
					System.out.println("Total memory: " + df.format(((double)rt.totalMemory()/1024.0/1024.0)) + "MB");
					System.out.println("Max memory: " + df.format(((double)rt.maxMemory()/1024.0/1024.0)) + "MB");	
					System.out.println("----------------------------------------------");
				}
				//next time the same tuple is to be sheduled
				long nextTime = -1;
				try {														
					 	tuple = currentStream.makeNewTuple(0);
						nextTime = tuple.getTimeStamp().getMaxTimeStamp();
						
					} catch (XATTimeStampException e2) {					
						System.out.println("Time Stamp Exception Removing the stream "+currentStream.getStreamName());
					} catch (RecordParseException e) {
						System.out.println("Record Parser Exception Removing the stream "+currentStream.getStreamName());
						//currentStream.cleanup();
						e.printStackTrace();	
						continue;				
					} catch (NullPointerException e3){
						
						// If the tuple comes back null, it means we are restarting
						// the file. So, we have to restart the timestamp as well.
						startTime = System.currentTimeMillis();
						try{
							tuple = currentStream.makeNewTuple(0);		 
							nextTime = tuple.getTimeStamp().getMaxTimeStamp();
						}catch(Exception e){
							System.out.println("We have a big problem!");
						}
					}
				 
				 ob = new Object[2];
				 ob[0] = currentStream;
				 ob[1] = tuple;
				 
				//insert this stream back into the list
				linearlist.insertIntoList(nextTime, ob);
				

			} catch (IOException e) {
				if (e instanceof EOFException) {					
						System.out.println(
							"The "
								+ currentStream.getStreamName()
								+ " is empty.");
						System.out.println(
							"Removing stream "
								+ currentStream.getStreamName());
					
					currentStream.cleanup();
					sender.sendEndNotification(currentStream.getStreamName());
					//U don't need to remove the stream from the list.
					//just do not insert it back again
					//linearlist.removeFirst();
				} else
					e.printStackTrace();
			}		
	}
	if (Thread.currentThread().isInterrupted()) {
		System.out.println("Streams interrupted.");
	} else {
		System.out.println("End of streams.");
		this.cleanup();
	}	
	return;
}

	/**
	 * Cleansup all the components 
	 */
	private void cleanup() {
		try {
			this.clientPool.cleanup();		
			this.sender.cleanup();
			this.clientAcceptor.disableAcceptor();
		} catch (IOException e) {
			System.out.println("--Error when cleaning up--");					
			e.printStackTrace();
		}		
	}

private void makeList() throws IOException{	
	XATStream tempStream;	
	long timer = System.currentTimeMillis() - startTime;
	Collection collection = streams.values();	
	for (Iterator iter = collection.iterator(); iter.hasNext();) {
		 tempStream = (XATStream) iter.next();			
		XATTuple tuple = null;
		long timeStamp = -1;
		try {
			tuple = tempStream.makeNewTuple(timer);
			timeStamp = tuple.getTimeStamp().getMaxTimeStamp();
		} catch (XATTimeStampException e) {
			e.printStackTrace();
		} catch (IOException e) {
			System.out.println("Removing the current Stream,IOExeption  "+tempStream.getStreamName());				
		} catch (RecordParseException e) {			
			System.out.println("Removing the current Stream, Record Parse Exception "+tempStream.getStreamName());
			e.printStackTrace();
		}		
		//Note change the list to use long from now on
		Object[] ob = new Object[2];
		ob[0] = tempStream;
		ob[1] = tuple;
		this.linearlist.insertIntoList(timeStamp, ob);
		
	}
}


/**
 * Returns the instance of the stream that has the stream name passed in as agrument
 * @param streamName
 * @return
 */
public XATStream getStream(String streamName)
{
	return((XATStream)streams.get(streamName));		
}


public static void main(String[] args) throws NumberFormatException, ParsingException {
	if(args.length != 2)
	{
		System.out.println("Usage: XATStreamGenerator <portnumber> <configfile.xml>");
		System.exit(0);
	}
	
	try {
		XATStreamGenerator streamGenerator = new XATStreamGenerator(Integer.parseInt(args[0]),args[1]);
	//	streamGenerator.run();
	
	} catch (IOException e) {		
		e.printStackTrace();
	}catch(DuplicateStreamNameException e){
		e.printStackTrace();
	}
	
}
}
