package edu.wpi.cs.dsrg.xmldb.xat.component.xatdaggeneration;
/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//	Rainbow 1.0 - XQuery Processing.
//
//	Copyright 2003, All Rights Reserved.
//
//	Permission to use, copy, modify and distribute this software and
//	its documentation for educational, research and non-profit
//	purposes, without fee, and without a written agreement is hereby
//	granted, provided that this copyright notice appear in all copies.
//
//	Permission to incorporate this software into commercial products
//	may be obtained directly from the authors.
//
//	This software is provided "as is" without express or implied warranty.
//	In no event will the authors be held liable for any damages arising
//	from the use of this software.
//
//	Authors:
//	Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//	Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//	Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//	Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//		Computer Science Department
//		Worcester Polytechnic Institute
//		Worcester, MA 01609 USA
//
//		http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//	Send bug reports, bug fixes, enhancements, requests, or comments to
//	the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import edu.wpi.cs.dsrg.utils.*;
import java.util.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.updateoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dompattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.operatorfactory.*;
import xacute.quilt.*;
import java.lang.reflect.Array;
import java.util.Vector;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;

/**
 * This class visits the XAT Tree constructed in ParserTraverse 
 * and get rid of For operaters. Thus we can get rid of inner loops
 * in the XAT Tree and make a flat algebra tree.
 * <p>
 * To understand this code, one should know how the decorrelation
 * works and what our algorithm is.  It would also be helpful to 
 * know what a visitor pattern is because that is what we use to
 * traverse through the query.
 * <p>
 * Currently, we need to 2 round of decorrelation to get rid of 
 * all For loops.
 * <p>
 * The Main issues in decorrelation is:
 * 1. empty collection problem
 * 2. count bugs.
 *
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 * @see edu.wpi.cs.dsrg.xmldb.xat.common.visitorpattern
 */

/**
 * songwang
 *
 * The new push down algorithm for LET clause.
 */

public class DagPushdownLET extends InheritableVisitor {

	/**
	* record the context Let bindings of the LET clause.
	* The Strings will be the for variables.
	* @since 1.0
	*/
	protected String[] contextLetBindings;

	/**
	* record the merged XAT Tree.
	* @since 1.0
	*/
	protected XATDag mergeTree;

	/**
	* A global point to denote where is the current parent we are working with.
	* 
	* @since 1.0
	*/
	protected DagNode currentParent;
	protected DagNode currentNode;
	protected XATDag upper;
	protected XATDag letTree;
	protected DagNode rootOfLetTree;

	/**
	* used to create operators
	* 
	* @since 1.0
	*/
	protected OperatorBuilder builder;

	protected ConsumedVisitor consumedVisitor;

	protected DagPushdownLET() {
		super();
	}
	/**
	 * Class Constructor with LET Tree to be pushdown.
	 * This is the operating method to be called by outside.
	 * @since 1.0
	 */
	public DagPushdownLET(XATDag upper, XATDag tree, XATDag letTree, String[] letBindings) {
		// check the validation of the two parameters.
		if (tree == null) {
			mergeTree = letTree;
			return;
		} else if (tree.getRoot() == null) {
			mergeTree = letTree;
			return;
		} else if (letTree == null) {
			mergeTree = tree;
			return;
		} else if (letTree.getRoot() == null) {
			mergeTree = tree;
			return;
		}

		try {
			/** create the factory */
			OperatorFactory factory = OperatorFactory.newInstance();
			/** instantiate the operator builder */
			builder = factory.newOperatorBuilder();
		} catch (FactoryException e) {
			/** 
			* there was a problem instantiating the factory
			* so we have to stop generating the tree. 
			*/
			System.err.println(e);
		}
		//used to check if the for binding variable is used by a DagNode
		this.consumedVisitor = new ConsumedVisitor();

		//prepare to pushdown
		this.contextLetBindings = letBindings;
		this.mergeTree = tree;
		this.currentParent = null;
		this.currentNode = (DagNode)tree.getRoot();
		this.letTree = letTree;
		this.rootOfLetTree = (DagNode)letTree.getRoot();
		this.upper = upper;
		
		upper.cleanup();
		tree.cleanup();
		letTree.cleanup();
		
		pushdown();
	}

	/**
	 * Completes the pushdown algorithm on the XAT
	 * @since 1.0
	 */
	private void pushdown() {
		if (!isConsumeLet()) {
			//basic cases:
			//at this point, there is no DagNode in the subtree of the currentParent
			//that consuming the binding variable.
			//so if the subtree is not null, make the CP otherwise connect the letTree
			if (this.currentNode == null) {
				mergeTree.addNode(this.rootOfLetTree, this.currentParent);
			} else {
				//create a Nature Join node
				//We do not use Cartesian Product because we do not know at this point
				//wether or not the let tree and the left subtree coming from same for variable.
				DagNode njNode = new DagNodeImp();
				NatureJoin natureJoin = builder.newNatureJoin(njNode,null);
				njNode.setOperator(natureJoin);

				// if this total branch does not use the let at all. need not merge
				if (this.currentParent !=null){
					// now, add the cartesian product to the tree
					mergeTree.addNode(njNode, this.currentParent);
					this.currentParent.deleteChild(this.currentNode);
					this.currentNode.deleteParent( this.currentParent);
					//set the child of the cartesian product
					mergeTree.addNode(this.rootOfLetTree, njNode);
					mergeTree.addNode(this.currentNode,	njNode);
				} else if (isConsumeLet((DagNode)this.upper.getRoot())){
					//Since the let variable maybe used by upper sort clause only
					//we need to check the upper tree also.
					// now, add the cartesian product to the tree
					mergeTree.setRoot(njNode);

					//set the child of the cartesian product
					mergeTree.addNode(this.rootOfLetTree, njNode);
					mergeTree.addNode(this.currentNode,	njNode);
					
				}

			}
		} else {
			//recursive case:
			//step 1:pushdown the FOR depending on different node.
			//step 2:recursively call decor()

			//visit this.currentNode
			visit((Object)this.currentNode.getOperator());
			//recursively call this function.
			//in above visit() statement. and pushdown() is recursively called.
		}
	}
	private boolean isConsumeLet() {
		HashSet consumed = new HashSet();
		DagNode rootOfSubTree = this.currentNode ;
		consumed = consumedCol(rootOfSubTree);
		//Here we assume a Let clause only has one binded variable.
		if (consumed.contains(this.contextLetBindings[0]))
			return true;
		else
			return false;
	}
	private boolean isConsumeLet(DagNode rootOfSubTree) {
		HashSet consumed = new HashSet();
		consumed = consumedCol(rootOfSubTree);
		//Here we assume a For clause only has one binded variable.
		if (consumed.contains(this.contextLetBindings[0]))
			return true;
		else
			return false;
	}

	//Return all the columns used in the XAT tree recursively.
	private HashSet consumedCol(DagNode node) {
		HashSet consumed = new HashSet();
		try {
			if (node == null)
				return consumed;
			if ((!(node.getOperator() instanceof LeftOuterJoinImp))
				&& (!(node.getOperator() instanceof NatureJoinImp)))
				consumed = (HashSet) consumedVisitor.visit(node);
			DagNode[] children = (DagNode[])node.getChildren();
			if (children.length > 0) {
				// append all the children's consumed columns.
				for (int i = 0; i < children.length; i++) {
					HashSet temp = (HashSet) consumedCol(children[i]);
					if (temp != null)
						consumed.addAll(temp);
				}
			}
			//for FOR op
			if (node.getOperator() instanceof FORImp)
				consumed.addAll(
					(HashSet) consumedCol((DagNode)((FOR) node.getOperator())
						.getXtreFunctions()
						.getRoot()));

			//for GB op
			if (node.getOperator() instanceof GroupByImp)
				consumed.addAll(
					(HashSet) consumedCol((DagNode)((GroupBy) node.getOperator())
						.getXATTree()
						.getRoot()));
		} catch (java.lang.NullPointerException e) {
			System.out.println();
		}

		return consumed;
	}
	/**
	 * Access method of field context of outter for bindings.
	 *
	 * @return array of String
	 * @since 1.0
	 */
	private String[] getLetContext() {
		return contextLetBindings;
	}

	public Object dispatch() {
		//System.out.println(indent() + "NULL");
		return null;
	}
	/**
	 * Called when an object hasn't been implemented
	 * by the visitor pattern
	 * default handling of the unknow operators.
	 */
	public Object dispatch(Object obj) {

		XATOperator opr = (XATOperator) obj;

		// do nothing here. Because we modify on the cloned input tree directly.	

		if (isDebug())
			System.out.println(
				"Operator of type "
					+ opr.getClass().getName()
					+ " is not handled!");
		if (isDebug())
			System.out.println(
				"This node is simply added to the Decorrelated Tree as is.");

		return null;
	}
	/**
	 * Access method of field decorrelate tree.
	 *
	 * @return XATDag
	 * @since 1.0
	 */
	public XATDag getMergeTree() {
		return this.mergeTree;
	}
	/**
	 * Get the value of the propertity 'DEBUG_LetPushdown'.
	 * @return boolean True: With debug information. False: No debug information.
	 *
	 * @since 1.0
	 */
	public boolean isDebug() {
		return System.getProperty("DEBUG_LetPushdown") != null;
	}
	/**
	 * Pushing down function for context insensitive operator having single child
	 * 
	 * @return java.lang.Object
	 * @param contextInsensetiveOp edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.XATOperator
	 * @since 1.0
	 */
	public Object visit(XATOperator xatOp) {
		//just push let tree down.
		if (isDebug())
			System.out.println("pushing let tree down\n");

		if ((this.currentParent!=null)&&(xatOp.getNode().getParent() !=this.currentParent)) return null;
		
		if (this.currentNode.getChildrenSize()>0){
			this.currentParent = this.currentNode;
			this.currentNode = (DagNode)this.currentNode.getChild(0);
			pushdown();
		} else {
			this.currentParent = this.currentNode;
			this.currentNode = null;
			pushdown();
		}
		return null;
	}
	/**
	 * Pushing down for operator having multiple children
	 * 
	 * @return java.lang.Object
	 * @param contextInsensetiveOp edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.XATMultiSourceOperator
	 * @since 1.0
	 */
	public Object visit(XATMultiSourceOperator xatOp) {
		//just push Let Tree down.
		if (isDebug())
			System.out.println("multiple pushing let tree down\n");

		if ((this.currentParent!=null)&&(xatOp.getNode().getParent() !=this.currentParent)) return null;

		//clone and push FOR under the subroot node.
		DagNode[] children = (DagNode[])this.currentNode.getChildren();
		for (int i = 0; i < children.length; i++) {
			DagNode child = children[i];
			if (isConsumeLet(child)) {
				XATDag newLetTree = (XATDag) this.letTree;//.clone();
				this.rootOfLetTree = (DagNode)newLetTree.getRoot();
				this.currentParent = (DagNode)xatOp.getNode();
				this.currentNode = child;
				pushdown();
			}
		}
		return null;
	}
}
