package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.schemacleanup.*;

/**
 * This Class is used to push down operators by traversing the XAT.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

 public class ComputationPushdownRules extends ReWriteRules {
/**
 * Default ComputationPushdownRules constructor.
 * Constructs a ComputationPushdownRules object.
 * @since 1.0   
 */

public ComputationPushdownRules() {
    super();
}



/**
 * <pre>
 * Select MultipleSource Operator Product Rule.
 * we want to determine what to do with the Select,  there are 4 possiblities.
 * 1) The Select should move to the left branch
 * 2) The Select should move to the right branch
 * 3) The Select should be merged with the Cartesian to form an OuterJoin
 * 4) The Select should stay on top of the Cartesian
 * to determine which case we have, we will try to findMatchingNavigateNode on both the
 * left and right branches of the Cartesian with both the left and right column names of the select
 * to accomplish this, we will use a state machine
 * two booleans, left and right are set to true.  if any column name does not go to that side,
 * then the boolean is set to false
 * after all column names have been checked against all branches, we can determine which of the 4
 * actions to take
 * 1) true / false
 * 2) false / true
 * 3) true / true
 * 4) false / false
 * </pre>
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Select).
 * @param child child XAT Operator (CartesianProduct).
 * @since 1.0   
 * @see Select
 * @see CartesianProduct
 * @see ReWriteResult
 * @see findMatchingNavigationNode(String, XATNode) findMatchingNavigationNode
 * @see moveNode(XATNode, XATNode) moveNode
 */
public Object visit(Select parent, CartesianProduct child) {
    if (isDebug()) {
        System.out.println("In Select / CartesianProduct (Join)");
    }
    ReWriteResult r1 = new ReWriteResult();
    //the bindings = null when they are not column names
    /*
    String leftBinding = getLeftColumn((BinExpression) parent.getExpression());
    String rightBinding = getRightColumn((BinExpression) parent.getExpression());
    */
    Vector columnNames = new Vector();
	RewriteUtilities.getColumnNames((BinExpression) parent.getExpression(), columnNames);
    boolean left = false;
    boolean right = false;
    //get all of the children of Cartesian
	Vector joinBranches = RewriteUtilities.getChildren((XATNode) child.getNode());
    for (int i = 0; i < columnNames.size(); i++) {
        /*we check only if the left still is true */
        if (!left) {
            /*if we cant find a matching node on the left, set it to false so we dont look again */
            if (RewriteUtilities.findMatchingNavigationNode((String) columnNames.elementAt(i),
                (XATNode) joinBranches.elementAt(0))) {
                left = true;
            }
        }
        if (!right) {
            if (RewriteUtilities.findMatchingNavigationNode((String) columnNames.elementAt(i),
                (XATNode) joinBranches.elementAt(1))) {
                right = true;
            }
        }

    }

    //now determine what to do
    //both are false so keep Select where it is
    if (!left && !right) {
        r1.setFinalPosition(true);
        r1.setSwapped(false);

    }
    //left = true, right = false so move to the left side
    if (left && !right) {
        RewriteUtilities.moveNode((XATNode) parent.getNode(), (XATNode) joinBranches.elementAt(0));
        if (isDebug()) {
            System.out.println("^^^^^^^^^^^^^ Moved Select to the left side of the MSO");
        }
        r1.setSwapped(false);
        r1.setMoved(true);
        r1.setFinalPosition(false);

    }
    //both = true so create a join
    if (left && right) {
        if (child instanceof Join) {
            // if it is already a join
            // do nothing.
            r1.setSwapped(false);
            r1.setFinalPosition(true);
        } else
            if (child instanceof CartesianProduct) {
                XATNode x1 = child.getNode();
                if (isDebug()) {
                    System.out.println("^^^^^^^^^^^^^ Created a Join");
                }
                Join j1 = new ThetaJoinImp(x1, (BinExpression) parent.getExpression());
                x1.setOperator(j1);
                //delete the parent
                r1.setDeleteParentNode(true);
                //the select is in its final position
                r1.setFinalPosition(true);
            }

    }
    /* 
    * the only case left is left = false, right = true
    */
    if (!left && right) {
        RewriteUtilities.moveNode((XATNode) parent.getNode(), (XATNode) joinBranches.elementAt(1));
        if (isDebug()) {
            System.out.println(
                "^^^^^^^^^^^^^ Moved Select to the right side of the XATMultiSourceOperator");
        }
        r1.setSwapped(false);
        r1.setMoved(true);
        r1.setFinalPosition(false);

    }
    return r1;
}
/**
 * Select GroupBy Rule.  Select cannot be pushed past the Groupby
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Select).
 * @param child child XAT Operator (GroupBy).
 * @since 1.0   
 * @see Select
 * @see GroupBy
 * @see ReWriteResult
 */
public Object visit(Select parent, GroupBy child) {
    if (isDebug()) {
        System.out.println("************In Select / GroupBy *********");
    }
    ReWriteResult r1 = new ReWriteResult();
    r1.setFinalPosition(true);
    r1.setSwapped(false);
    return r1;
}
/**
 * <pre>
 * Select MultipleSource Operator Product Rule.
 * MultipleSource are operators exclude CartesianProduct family operators.
 * For example: Merge, Set operator, IF_THEN_ELSE operator, "Or" operator.
 *
 * In this case, no push down will be applied. For the semantic will be changed.
 * </pre>
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Select).
 * @param child child XAT Operator (XATMultiSourceOperator).
 * @since 1.0   
 * @see Select
 * @see XATMultiSourceOperator
 * @see ReWriteResult
 */
public Object visit(Select parent, XATMultiSourceOperator child) {
    if (child instanceof CartesianProduct)
        return visit(parent, (CartesianProduct) child);

    if (isDebug()) {
        System.out.println("In Select / XATMultiSourceOperator");
    }

    // stop the pushdown of the select.

    ReWriteResult r1 = new ReWriteResult();
    r1.setFinalPosition(true);
    r1.setSwapped(false);

    return r1;
}
/**
 * Select XATOperator Rule.
 * Select can always be swapped with any XATOperator that doesnt have its
 * own rule.
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Select).
 * @param child child XAT Operator (XATOperator).
 * @since 1.0   
 * @see Select
 * @see XATOperator
 * @see ReWriteResult
 */
public Object visit(Select parent, XATOperator child) {
    if (isDebug()) {
        System.out.println("In Select / XATOperator");
    }
    ReWriteResult r1 = new ReWriteResult();
    // if the select depends on the child operator.

    if (child.getOutputColumnName() != null) {
        if (parent.getExpression().toString().indexOf(child.getOutputColumnName())
            != -1) {
            // stop here.
            r1.setFinalPosition(true);
            r1.setSwapped(false);
        } else {
            // keep going.
			RewriteUtilities.swap(parent, child);
            r1.setFinalPosition(false);
            r1.setSwapped(true);
        }
    } else {
        //the child operator does not generate anything, parent could not dependent on it
		RewriteUtilities.swap(parent, child);
        r1.setFinalPosition(false);
        r1.setSwapped(true);
    }
    return r1;
}
/**
 * Select Navigate Rule.
 * Select can swap with a Navigate if the bindings dont match
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Select).
 * @param child child XAT Operator (Navigate).
 * @since 1.0   
 * @see Select
 * @see Navigate
 * @see ReWriteResult
 */
public Object visit(Select parent, Navigate child) {
    if (isDebug()) {
        System.out.println("************In Select / Navigate *********");
    }
    ReWriteResult r1 = new ReWriteResult();

    boolean match = false;
    String navigateBinding = child.getOutputColumnName();
    //String selectBinding = getColumnName((BinExpression)parent.getExpression());
    ColumnName[] selectBindings =
        ((BinExpression) parent.getExpression()).getRelatedColumnNames();
    for (int i = 0; i < selectBindings.length; i++)
        if (selectBindings[i].getColumnName().equals(navigateBinding)) {
            match = true;
            break;
        }

    if (match) {
        r1.setFinalPosition(true);
        r1.setSwapped(false);
    } else {
		RewriteUtilities.swap(parent, child);
        r1.setSwapped(true);
        r1.setFinalPosition(false);
    }

    return r1;
}

/**
 * Rules for pushing a Navigate operator through a NameColumn operator
 * A Navigate node can always be pushed through a NameColumn operartor
 * This rule is the same as NameColumn / Navigate
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (NameColumn).
 * @since 1.0   
 * @see Navigate
 * @see GroupBy
 * @see ReWriteResult
 * @see deleteRedundantRename(NameColumn) deleteRedundantRename	 
 */
public Object visit(Navigate parent, NameColumn child) {
    if (isDebug()) {
        System.out.println("************In Navigate / NameColumn *********");
    }
    ReWriteResult p1 = new ReWriteResult();
    if (RewriteUtilities.isRedundantRename(child)) {
        p1.setDeleteChildNode(true);
        p1.setFinalPosition(false);
        if (isDebug()) {
            System.out.println("NameColumn" + child + " to be deleted!");
        }
        return p1;
    }

    String oldName = child.getOriginalColumnName();
    String newName = child.getNewColumnName();
    //check the binding of the Navigate
    if (parent.getOutputColumnName().equals(oldName)) {
        //if the old name exists in the table, update it.
        if (this.getVariableTable().contains(oldName))
            getVariableTable().updateVariableTable(newName, parent.getOutputColumnName());
        //parent.setOutputColumnName(newName);
        parent.setOutputColumnName(newName);

        //Changed here XXX to delete NameColumn as the OldColumn
        //	won't be used by any other operators under it for it was
        //	created by this Navigate operator
        p1.setFinalPosition(true);
        p1.setSwapped(false);
        p1.setDeleteChildNode(true);
        return p1;
    }
    //check the entryPoint
    NavigationStep[] entryPoint = parent.getEntryPoint();
    for (int i = 0; i < entryPoint.length; i++) {
        if (entryPoint[i].getStepValue().equals(oldName)) {
            if (isDebug()) {
                System.out.println(
                    "Old value: " + entryPoint[i].getStepValue() + " new value: " + newName);
            }
            entryPoint[i].setStepValue(newName);
            NavigationStep[] parentDestination = parent.getDestination();
            NavigationStep[] combinedParent = RewriteUtilities.combineSteps(entryPoint, parentDestination);
            getVariableTable().updateVariableTable(parent.getOutputColumnName(), entryPoint);
        }
    }

    if (!child
        .getOutputColumnName()
        .equals(parent.getEntryPoint()[0].getStepValue())) {
        p1.setFinalPosition(false);
        p1.setSwapped(true);
		RewriteUtilities.swap(parent, child);
    } else {
        p1.setFinalPosition(true);
    }
    return p1;

}
/**
 * Rules for pushing a Navigate operator through a Source operator
 * A Navigate can never go through a Source operator.  The Source operators
 * are on the very bottom of trees and swapping the 2 would cause errors
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (Source).
 * @since 1.0   
 * @see Navigate
 * @see Source
 * @see ReWriteResult
 */
public Object visit(Navigate parent, Source child) {

    ReWriteResult p1 = new ReWriteResult();
    if (isDebug()) {
        System.out.println("***********In Navigate, Source*********");
    }
    //if the source node is not default.xml, the swap may be performed.
    //otherwise, the navigate node is as far down in the tree as possible
    p1.setFinalPosition(true);
    p1.setSwapped(false);
    return p1;
}
/**
 * <pre>
 * Rules for pushing a Navigate node through a XATMultiSourceOperator .
 * This is also the same rule for Navigate
 * The XATMultiSourceOperator node will have n branches, the Navigate node must be moved to
 * the branch that contains a Navigate node whose destination equals parent.entryPoint
 *
 * findMatchingNavigationNode is used to determine which branch to move to
 *
 * This rules apply to cartesian product operators.
 * </pre> 
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (NameColumn).
 * @param child child XAT Operator (XATMultiSourceOperator).
 * @since 1.0   
 * @see Navigate
 * @see XATMultiSourceOperator
 * @see ReWriteResult
 */
public Object visit(Navigate parent, CartesianProduct child) {
    if (isDebug()) {
        System.out.println("In Navigate / CartesianProduct (Join)");
    }
    ReWriteResult p1 = new ReWriteResult();
    //in case nothing is found, just put the Navigate in its final position
    p1.setSwapped(false);
    p1.setFinalPosition(true);

    Vector childrenVector = new Vector();
    XATNode currentNode;

    //get all of the children of join
	Vector joinBranches = RewriteUtilities.getChildren((XATNode) child.getNode());

    //iterate over each child
    for (int x = 0; x < joinBranches.size(); x++) {
        //look at this branch for the navigate node
        VarStep v1 = (VarStep) parent.getEntryPoint()[0];
        // compute the full schema of this branch.
        CompleteSchemaVisitor completeSchema = new CompleteSchemaVisitor();
        Set schema = (Set) completeSchema.visit((XATNode) joinBranches.elementAt(x));
        // clean the schema
		RewriteUtilities.cleanSchema((XATNode) joinBranches.elementAt(x));

        if (schema.contains(v1.getStepValue())) {
            //if the node was found, copy the navigate under that branch
            boolean result =
			RewriteUtilities.copyNode((XATNode) parent.getNode(), (XATNode) joinBranches.elementAt(x));
            if (isDebug()) {
                System.out.println(
                    "I decided to copy the Navigate:"
                        + parent
                        + " to branch "
                        + x
                        + " of MSO:"
                        + child);
            }
            // delete the parent if it is copied.
            p1.setSwapped(false);
            p1.setDeleteParentNode(true);
            p1.setFinalPosition(true);
        }

    }

    return p1;
}
/**
 * Rules for pushing a Navigate operator through a Groupby operator
 * A Navigate can, but for the sake of order sensitive rewrite, is never
 *  pushed through a groupby (due to the way the tree is generated, this
 *  would normally only allow a navunnest of a text() to be pushed anyway).
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (GroupBy).
 * @since 1.0   
 * @see Navigate
 * @see GroupBy
 * @see ReWriteResult
 */
public Object visit(Navigate parent, GroupBy child) {

    ReWriteResult p1 = new ReWriteResult();
    if (isDebug()) {
        System.out.println("*********Navigate / GroupBy*********");
    }
    p1.setFinalPosition(true);
    p1.setSwapped(false);
    return p1;

}
/**
 * Rules for pushing a Navigate operator through a Project operator
 * A Navigate can always move through a Project, but the destination of the
 * Navigate node must be added to columns that are Projected
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (Project).
 * @since 1.0   
 * @see Navigate
 * @see Project
 * @see ReWriteResult
 * @depracated This rule should not happen in our expectation.
 */
public Object visit(Navigate parent, Project child) {

    ReWriteResult p1 = new ReWriteResult();
    p1.setFinalPosition(false);

    //the Navigate's destination must be added to the Project pattern
    if (parent.getOutputColumnName() == null) {
        String columnName = new String("");
        NavigationStep[] n1 = parent.getDestination();
        for (int i = 0; i < n1.length; i++) {
            columnName = columnName + n1[i] + "/";

        }
        child.insertColumnName(columnName, 0);
    } else {
        //if there is a variable binding, then use that instead of the destination
        child.insertColumnName(parent.getOutputColumnName(), 0);
    }
    //a Navigate and a Project can always be swapped
	RewriteUtilities.swap(parent, child);
    p1.setSwapped(true);
    return p1;

}
/**
 * Rules for pushing a Navigate operator through a Select operator
 * A Navigate can always move through a Select
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (Select).
 * @since 1.0   
 * @see Navigate
 * @see Select
 * @see ReWriteResult
 */
public Object visit(Navigate parent, Select child) {

    ReWriteResult p1 = new ReWriteResult();
    if (isDebug()) {
        System.out.println("*********Navigate / Select*********");
    }
    //a navigate and a select can always be swapped
	RewriteUtilities.swap(parent, child);
    p1.setFinalPosition(false);
    p1.setSwapped(true);
    return p1;

}
/**
 * <pre>
 * Rules for pushing a Navigate node through a XATMultiSourceOperator .
 * MultipleSource are operators exclude CartesianProduct family operators.
 * For example: Merge, Set operator, IF_THEN_ELSE operator, "Or" operator.
 *
 * In this case, no push down will be applied. For the semantic will be changed.
 * </pre> 
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (XATMultiSourceOperator).
 * @since 1.0   
 * @see Navigate
 * @see XATMultiSourceOperator
 * @see ReWriteResult
 */
public Object visit(Navigate parent, XATMultiSourceOperator child) {
    if (child instanceof CartesianProduct)
        return visit(parent, (CartesianProduct) child);

    if (isDebug()) {
        System.out.println("In Navigate / XATMultiSourceOperator Product");
    }
    ReWriteResult p1 = new ReWriteResult();
    //in case nothing is found, just put the Navigate in its final position
    p1.setSwapped(false);
    p1.setFinalPosition(true);

    return p1;
}
/**
 * Rules for pushing a Navigate operator through a generic XATOperator operator
 * A Navigate can always be pushed through a Distinct
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (XATOperator).
 * @since 1.0   
 * @see Navigate
 * @see XATOperator
 * @see ReWriteResult
 */
public Object visit(Navigate parent, XATOperator child) {
    if (isDebug()) {
        System.out.println("In Navigate / XATOperator: " + child.toString());
    }
    ReWriteResult p1 = new ReWriteResult();
    if (!child
        .getOutputColumnName()
        .equals(parent.getEntryPoint()[0].getStepValue())) {
			RewriteUtilities.swap(parent, child);
        p1.setFinalPosition(false);
        p1.setSwapped(true);
    } else {
        p1.setFinalPosition(true);
        p1.setSwapped(false);
    }
    return p1;

}
/**
 * Rules for pushing a Navigate operator through a Navigate operator.
 * If the entry point for the parent is the same as the complete path for the child,
 * the nodes cannot be swapped and the parent is pushedDown.  Otherwise, the swap is legal.
 * @return ReWriteResult. The swap flag in ReWriteResult false.
 * @param parent parent XAT operator (Navigate).
 * @param child child XAT Operator (Navigate).
 * @since 1.0   
 * @see Navigate
 * @see Navigate
 * @see ReWriteResult
 */
public Object visit(Navigate parent, Navigate child) {
    if (isDebug()) {
        System.out.println("************In Navigate / Navigate *********");
    }
    ReWriteResult p1 = new ReWriteResult();
    NavigationStep[] parentEntryPoint = parent.getEntryPoint();
    String parentEPString = parentEntryPoint[0].getStepValue();
    String childBinding = child.getOutputColumnName();
    /*
    NavigationStep[] childDestination = child.getDestination();
    NavigationStep[] childEntryPoint = child.getEntryPoint();
    NavigationStep[] combinedChild =
    combineSteps(childEntryPoint, childDestination);
    */
    try {

        //obtain a string representation of the destination and entry point

        //String childString = createString(combinedChild);

        //if they are the same, then they cannot be swapped
        if (!childBinding.equals(parentEPString)) {
            p1.setFinalPosition(false);
			RewriteUtilities.swap(parent, child);
            p1.setSwapped(true);
        } else {
            p1.setFinalPosition(true);
            p1.setSwapped(false);

        }

    } catch (Exception e1) {
        System.out.println("Error!!! " + e1.toString());
        System.out.println("Parent: " + parent.toString());
        System.out.println("Child: " + child.toString());
		RewriteUtilities.swap(parent, child);
        //System.out.println("done swapping nodes");
        p1.setSwapped(true);
        p1.setFinalPosition(true);
    }

    return p1;

}

}
