package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.updateoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.SelectImp;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;

/**
 * This class includes all static functions used in the rewrite functionality.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class RewriteUtilities {
	/**
	 * Print out a Hashtable in a good format.
	 * @return java.lang.String
	 * @param hash java.util.Hashtable
	 * @since 1.0
	 */
	public static String printHashtable(Hashtable hash) {
		StringWriter strWrt = new StringWriter();
		PrintWriter pout = new PrintWriter(strWrt);
	
		if(hash == null)
			return "NULL hashtable.";
	
		Enumeration keys = hash.keys();
		while(keys.hasMoreElements()){
			Object key = keys.nextElement();
			Object value = hash.get(key);
			pout.println(key + "\t|\t" + value);
		}
		
		return strWrt.toString();
	}
	/**
	 *  
	 * Swaps the operators of 2 nodes. This is used when the appropriate equivalance rule
	 * determines that the 2 nodes can be swapped. 
	 * @param operator1 
	 * @param operator2 
	 */
	public static void swap(XATOperator operator1, XATOperator operator2) {
	
		XATNode firstNode = operator1.getNode();
		XATNode secondNode = operator2.getNode();
	
		firstNode.setOperator(operator2);
		secondNode.setOperator(operator1);
	
		operator1.setNode(secondNode);
		operator2.setNode(firstNode);
	
	}
	/**
	 * <pre>
	 * MoveNode will alter the XATTree structue by moving the parent
	 * node directly under the child. The child's children will be altered to
	 * point to parent. This is primarily used to move a node under the correct Join / Union branch.
	 *
	 * <b> Structure of tree </b>
	 * parent = x2, child = x4
	 *   
	 *   	old structure
	 *   	x1 -> x2 -> x3 -> x4
	 *   
	 *   	new structure
	 *   	x1 -> x3 ->	x2 -> x4
	 * 
	 * Any other time you want to switch the positions of 2 nodes, swap(XATOperator, XATOperator) 
	 * should be used.
	 * <pre>
	 * @param parent   
	 * @param child 
	 * @see #swap(XATOperator, XATOperator) swap
	 */
	public static void moveNode(XATNode parent, XATNode child) {
	
		XATNode x1 = parent.getParent();
		XATNode x2 = parent;
		XATNode x3 = child.getParent();
		XATNode x4 = child;
	
		//disconnect children
		x1.deleteChild(x2);
		x2.deleteChild(x3);
		x3.deleteChild(x4);
	
		//add the new children
		x1.addChild(x3);
		x3.addChild(x2);
		x2.addChild(x4);
	
		//add the new parents		
		x3.setParent(x1);
		x2.setParent(x3);
		x4.setParent(x2);
	
	}
	/**
	 * Iterates over the BinExpression in order to find all of the column Names
	 * anytime a column name is found, it is added into the Vector.
	 * 
	 * @return boolean
	 * @param exp Expression
	 */
	public static void getColumnNames(BinExpression exp, Vector columnNames) {
		//check for a terminal expression
		if (exp instanceof TerminalExpression) {
			TerminalExpression te1 = (TerminalExpression) exp;
			//check for a column name terminal expression
			if (te1.getType() == TerminalExpression.NAME) {
				columnNames.addElement(
					(String) ((ColumnName) te1.getValue()).getColumnName());
			}
	
		} else {
			try {
				//try to call this on the left
				getColumnNames((BinExpression) exp.getLeft(), columnNames);
			} catch (ClassCastException e) {
				//if there is a CCE, then the left is a TerminalExpression
				TerminalExpression te1 = (TerminalExpression) exp.getLeft();
				//check for a column name terminal expression
				if (te1.getType() == TerminalExpression.NAME) {
					columnNames.addElement(
						(String) ((ColumnName) te1.getValue()).getColumnName());
	
				}
			}
			try {
				getColumnNames((BinExpression) exp.getRight(), columnNames);
			} catch (ClassCastException e1) {
				//same thing as above
				TerminalExpression te1 = (TerminalExpression) exp.getRight();
				//check for a column name terminal expression
				if (te1.getType() == TerminalExpression.NAME) {
					columnNames.addElement(
						(String) ((ColumnName) te1.getValue()).getColumnName());
				}
	
			}
	
		}
		return;
	}
	/**
	 * traverse up the tree, starting from the startingPoint parameter, 
	 * checking if columnToFind is used
	 * in any entry point in a Navigate, parameter in a Select, or
	 * column in a Tagger.
	 * It will be failed if the variable is not used in the inner tree, but
	 * used in the outer tree. In that case, for the FOR node in the tree,
	 * the Sub tree's root Node should have a link to the FOR/GB Node.
	 * This bug should be fixed in ParserTraverse. This bug is identified as
	 * BUG 698225 on Source Forge.net.
	 * Creation date: (7/25/2002 12:50:10 AM)
	 * @param column java.lang.String
	 */
	public static boolean isUsedAbove(XATNode startingPoint, String columnToFind) {
	
		XATNode n1 = startingPoint;
		XATNode parentNode;
		XATOperator x1;
	
		if (columnToFind == null)
			return false;
	
		while (true) {
			parentNode = n1.getParent();
			//if the parent is not null
			if (parentNode != null) {
				x1 = parentNode.getOperator();
				//check if x1 has a Navigate attached to it
				if (x1 instanceof Navigate) {
					NavigationStep[] entryPoint =
						((Navigate) x1).getEntryPoint();
					for (int i = 0; i < entryPoint.length; i++) {
						String stepValue = entryPoint[i].getStepValue();
						if (stepValue.equals(columnToFind)) {
							return true;
						}
					}
				} else if (
					x1 instanceof Tagger) {
					//if not, then check if x1 has a Tagger
	
					return ((Tagger) x1).getPattern().containsColumn(
						columnToFind);
	
				} else if (x1 instanceof SelectImp) { //try the Select
					Vector columnNames = new Vector();
					getColumnNames(
						(BinExpression) ((SelectImp) x1).getExpression(),
						columnNames);
					if (columnNames.contains(columnToFind)) {
						return true;
					}
				} else if (x1 instanceof Expose) {
					if (columnToFind.equals(((Expose) x1).getColumnName()))
						return true;
				} else if (x1 instanceof Combine) {
					if (columnToFind.equals(x1.getOutputColumnName()))
						return true;
				} else if (
					x1 instanceof ColumnNameList) {
					//try the MultiSource operators, e.g., XMLUnion, GroupBy.
					String[] columnNames =
						((ColumnNameList) x1).getColumnNames();
					for (int i = 0; i < columnNames.length; i++) {
						if (columnNames[i].equals(columnToFind)) {
							return true;
						}
					}
				} else if (x1 instanceof UpdateOperator) {
					String context = ((UpdateOperator) x1).getContext();
					String variable = ((UpdateOperator) x1).getVariable();
	
					if (columnToFind.equals(context)
						|| columnToFind.equals(variable))
						return true;
	
					if (x1 instanceof Replace) {
						String replace = ((Replace) x1).getReplacingVariable();
						if (columnToFind.equals(replace))
							return true;
					}
				} else if (x1 instanceof Function) {
					Object[] params = ((Function) x1).getParams();
					//check if the columnToFind is used in param
					for (int i = 0; i < params.length; i++) {
						if (params[i] instanceof ColumnName) {
							if (((ColumnName) params[i])
								.getColumnName()
								.equals(columnToFind)) {
								return true;
							}
						}
					}
				}
				//make n1 = its parent to recursively climb the tree
				n1 = parentNode;
			}
			//the parent was null so we reached the top of the tree, we can just return false
			else {
				return false;
			}
		}
	}
	/**
	 * An alternate definition for findMatchingNavigationNode.
	 * This method takes the binding parameter, converts it to a NavigationStep[] and calls
	 * the other findMatchingNavigationNode method.
	 * @return boolean True if a match is found. False: otherwise.
	 * @param binding The binding of a Navigate operator to the match.
	 * @param root The root node to start looking at.
	 * @since 1.0
	 * @see VarStepImp
	 * @see findMatchingNavigationNode(VarStep, XATNode) findMatchingNavigationNode
	 */
	public static boolean findMatchingNavigationNode(
		String binding,
		XATNode root) {
	
		VarStep ep = new VarStepImp(binding);
		return findMatchingNavigationNode(ep, root);
	
	}
	/**
	 * Get all children of a given root node.
	 * This method accepts a destination Vector and an XATNode and 
	 * all of the root's children are puts into destinationVector
	 * This is used to convert the root's children (stored as arrays) into a Vector.
	 * @return void
	 * @param destinationVector The vector to hold all children nodes.
	 * @param root The root XATNode to get the children from.
	 * @since 1.0
	 * @see XATNode
	 */
	public static Vector getChildren(XATNode root) {
	
		XATNode[] children = root.getChildren();
		//empty the destinationVector to ensure that only the root's children are in the Vector
		Vector destVector = new Vector();
		for (int x = 0; x < children.length; x++) {
			destVector.addElement((XATNode) children[x]);
		}
		return destVector;
	}
	/**
	 * Find a Navigate operator whose binding matches the entryPoint array.
	 * This method accepts a NavigationStep[] entryPoint 
	 * and will find a Navigate operator whose binding matches the entryPoint array.
	 * This is used to match a Navigate with a Navigate and is useful
	 * when trying to determine which side of a join a Navigate matches
	 * A Navigate's entryPoint is to be matched with the binding of another  
	 * @return boolean True: if a match is found. False: otherwise.
	 * @param entryPoint The EntryPoint to match.
	 * @param root The root node to start looking at.
	 * @since 1.0
	 * @see VarStep
	 */
	public static boolean findMatchingNavigationNode(
		VarStep entryPoint,
		XATNode root) {
	
		XATNode currentNode;
	
		String toMatchPath = entryPoint.getStepValue();
		//check the root first
		currentNode = root;
		//check for a source
		if (currentNode.getOperator() instanceof Source) {
			Source currentSource = (Source) currentNode.getOperator();
			if (toMatchPath.equals(currentSource.getOutputColumnName())) {
				return true;
			}
		}
		//check for a Navigate
		if (currentNode.getOperator() instanceof Navigate) {
	
			Navigate currentNav = (Navigate) currentNode.getOperator();
			//create a string corresponding to the destination
			String childBinding = currentNav.getOutputColumnName();
			//if the entryPoint matches the destination
			if (toMatchPath.equals(childBinding)) {
				return true;
			}
		}
	
		Vector childrenVector = getChildren(root);
	
		//loop recursively until there are no children of a node
		if (childrenVector.size() == 0) {
			//if there are no children, do nothing
			return false;
	
		} else {
			//call this method on each of the children
			for (int x = 0; x < childrenVector.size(); x++) {
				currentNode = (XATNode) childrenVector.elementAt(x);
				if (findMatchingNavigationNode(entryPoint, currentNode)) {
					return true;
				} else {
					//just keep going
				}
			}
	
		}
		return false;
	}	
/**
 * The default RewriteUtilities constructor.
 * @since 1.0
 */
public RewriteUtilities() {
	super();
}
/**
 * This method puts the entryPoint and destination step arrays of a Navigate 
 * Operator into 1 bigger array to make it easier to manipulate.
 * @return NavigationStep[] The array for combined Navigation Steps.
 * @param entryPoint  The entry point of Navigate operator.
 * @param destination The destination of Navigate operator. 
 * @since 1.0
 * @see NavigationStep
 */
public static NavigationStep[] combineSteps(
	NavigationStep[] entryPoint,
	NavigationStep[] destination) {

	int combinedLength = 0;
	int entryLength = 0, destLength = 0;
	try {
		entryLength = entryPoint.length;
	} catch (NullPointerException e) {
		//entry point is null
	}
	try {
		destLength = destination.length;
	} catch (NullPointerException e) {
		//dest is null

	}
	combinedLength = entryLength + destLength;
	NavigationStep[] combinedPath = new NavigationStep[combinedLength];
	//put in all of the entryPoint steps	
	for (int x = 0; x < entryLength; x++) {
		if (entryPoint[x] != null) {
			combinedPath[x] = entryPoint[x];
		}
	}
	//now put in all the destination steps
	for (int y = entryLength; y < combinedLength; y++) {
		if (destination[y - entryLength] != null) {
			combinedPath[y] = destination[y - entryLength];
		}
	}

	return combinedPath;
}

/**
 * Replacing the child node with parent node.
 * <pre>
 * This method will alter the XATTree structue by copying the parent
 * node directly replace child. The child's children will be altered to
 * point to parent. This is primarily used to copy a node under the correct 
 * Join / Union branch.
 *
 * <b> Structure of tree </b>
 * parent = x2, child = x4
 *   
 *   	old structure
 *   	x1 -> x2 ---> x3 -> x4
 *   
 *   	new structure
 *   	x1 -> x2 ---> x3 -> Clone of x2 -> x4
 * <pre>
 * @return boolean The boolean value indicates operation finished successfully.
 * @param parent The parent node.
 * @param child  The child node.
 * @since 1.0
 * @see #swap(XATOperator, XATOperator) swap
 */

public static boolean copyNode(XATNode parent, XATNode child) {

	XATNode x1 = parent.getParent();
	// clone the parent.
	XATNode x2 = (XATNode) parent.clone();
	// remove all children of the clone.
	x2.setChildren(new XATNode[] {
	});

	XATNode x3 = child.getParent();
	XATNode x4 = child;

	//get the index of x4 of x3.
	int index = x3.getIndexOf(x4);

	//replace x4 with x2.
	x3.setChild(index, x2);
	x2.setParent(x3);

	//add x4 as children of x2.
	x2.addChild(x4);
	x4.setParent(x2);

	// done. tree is modified.
	return true;
}

/**
 * Clean all the schema of XAT with a given root.
 * This method recursively clean the schema of XAT node. Set the schema of each node as null.
 * @return void 
 * @param root edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
 * @since 1.0   
 * @see   edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.XATTree
 * @see   edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode
 */
public static void cleanSchema(XATNode root) {
	root.clearSchema();
	// set all the children.
	XATNode[] children = root.getChildren();
	for (int i = 0; i < children.length; i++) {
		cleanSchema(children[i]);
	}

	// set all the subtree.
	if (root instanceof FOR) {
		XATTree subtree = ((FOR) root).getXtreFunctions();
		cleanSchema(subtree.getRoot());
	} else
		if (root instanceof GroupBy) {
			XATTree subtree = ((GroupBy) root).getXATTree();
			cleanSchema(subtree.getRoot());
		} else
			if (root instanceof IfExpr) {
				XATTree thenTree = ((IfExpr) root).getThenXAT();
				cleanSchema(thenTree.getRoot());
				XATTree elseTree = ((IfExpr) root).getElseXAT();
				cleanSchema(elseTree.getRoot());
			}
}

/**
 * Delete redundent Rename operator
 * This method compares the old column name and new column, if they are equal
 * or both of them are null, then return true, else return false.
 * @return boolean true: means can be deleted, because the new = old, otherwise, cannot be deleted.
 * @param ncOp edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.NameColumn
 * @since 1.0   
 * @see   edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.NameColumn
 */

public static boolean isRedundantRename(NameColumn ncOp) {
	if (ncOp.getNewColumnName() == null && ncOp.getOriginalColumnName() == null)
		return true;
	else
		return ncOp.getNewColumnName().equals(ncOp.getOriginalColumnName());
}

}
