package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.*;

import edu.wpi.cs.dsrg.utils.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;

/**
 * This class includes all static functions used in the rewrite functionality.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class XATCleanUp {	
/**
 * The default RewriteUtilities constructor.
 * @since 1.0
 */
public XATCleanUp() {
	super();
}
/**
 * This method will clean up the XATTree by doing following things:
 * Get rid of the rename Operator.
 *
 * @param tree The XATTree need cleanning.
 * @param mergeTagger True: Do tagger merge. False: otherwise.
 * @since 1.0
 */

public static void doCleanUp(XATTree tree) {

    // validate the tree. The tree has to be valid,
    // means, p-c consistent, and node-operator consistent.
    // tree.validate();
    Debugger db = new Debugger(tree);

    if (isDebug()) {
        System.out.println("!!!!!!!!!!!!!! Clean XAT !!!!!!!!!");
        System.out.println("!! Before Cleanup !!");
        db.printTree();
    }

    boolean again = true;
    int i = 1;
    while (again) {
        again = mergeOperatorStep(tree) || removeUselessOperatorStep(tree);
        if (isDebug()) {
            System.out.println("!! After cleanup " + i + " time(s) !!");
            // validate the tree. The tree has to be valid,
            // means, p-c consistent, and node-operator consistent.
            // tree.validate();
            db.printTree();
        }
        i++;
    }
}

/**
 * This method search the given ColumnName in the given expression.
 * If asked to replace, then found columnName is replaced with new column name provided.
 * This method can be useful in rewriting rules to check if any select, join etc node is refering to
 * a particular column generated below these nodes. You may want to modify the condition if you deleted
 * any nodes refered by expression in which case you want to replace the old column name with a new one.
 * @return boolean True: Found containing. False: otherwise.
 * @param exp The given expression searching inside.
 * @param cname The column name searching for.
 * @param replace Indicate if found do replacement or not.
 * @param newName The new column name used to replace the old column name.
 * @since 1.0
 */
private static boolean contains(
    Expression exp,
    String cname,
    boolean replace,
    String newName) {

    // if it is instance of a terminal expression then see if the column
    //   name in the expression matches the one that is been given
    if (exp instanceof TerminalExpression) {
        if (((TerminalExpression) exp).getValue() instanceof ColumnName)
            if (((ColumnName) ((TerminalExpression) exp).getValue())
                .getColumnName()
                .equals(cname))
                if (replace) {
                    ColumnName newColumn = new ColumnNameImp(newName);
                    ((TerminalExpression) exp).setValue(newColumn);
                }
        return true;
    }
    // if it is intance of binary expression then return true if any of its
    // left or right part of expression contains the column name we are looking for.
    else
        if (exp instanceof BinExpression) {
            Expression left = ((BinExpression) exp).getLeft();
            Expression right = ((BinExpression) exp).getRight();

            return contains(left, cname, true, newName)
                || contains(right, cname, true, newName);
        }
    // else it is a unary expression. What do we do here?
    else {
        return false;
    }

}
/**
 * This method will clean up the XATTree by doing following things:
 * Get rid of the rename Operator.
 *
 * @param tree The XATTree need cleanning.
 * @param mergeTagger True: Do tagger merge. False: otherwise.
 * @since 1.0
 */

private static boolean mergeOperatorStep(XATTree tree) {

    if (isDebug())
        System.out.println("**********Now merging operators************");
    //first merge the Navigates
    //first create a vector of the rule classes to look at
    Vector classList = new Vector();
    classList.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common.MergeOperatorRules");
    //instantiate the rules class 
    ReWriteRules rules = new ReWriteRules(classList);
    //now make the order of rules to look at
    Vector order = new Vector();
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.GroupByImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavCollectionImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.NavUnnestImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.DistinctImp");
	order.addElement(
		"edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.SelectImp");
	order.addElement(
			"edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.TaggerImp");

    //set the variable table in the rules class to be the hashtable from PT
    rules.setVariableTable(tree.getBindingTable());
    //create the traversal strategy
    ReWriteTraversal traverse = new ReWriteTraversal(tree, rules, order);
    //rewrite the tree.
    traverse.doRewrite(ReWriteTraversal.DOWN);
    // update binding table.
	tree.setBindingTable(new BindingTableComputer().computeBindingTable(tree));
	return traverse.getIsChanged();
}

private static boolean removeUselessOperatorStep(XATTree tree) {
    /** For the Rename Removal. **/
    if (isDebug())
        System.out.println(
            "**********Now Remove Useless Rename , Tagger, Navigate operators************");

    Vector classList = new Vector();
    classList.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common.RemoveUselessOperatorRules");
    //instantiate the rules class 
	ReWriteRules rules = new ReWriteRules(classList);

    //now make the order of rules to look at
    Vector order = new Vector();
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.NameColumnImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.TaggerImp");
    order.addElement(
        "edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.CombineImp");

    //set the variable table in the rules class to be the hashtable from PT
    rules.setVariableTable(tree.getBindingTable());

    //create the traversal strategy
	ReWriteTraversal traverse = new ReWriteTraversal(tree, rules, order);
    //rewrite the tree.
	traverse.doRewrite(ReWriteTraversal.DOWN);
    // update the binding table.
    tree.setBindingTable(new BindingTableComputer().computeBindingTable(tree));
    return traverse.getIsChanged();
}
/**
 * Get the value of the propertity 'DEBUG_DeepCopyMaker'.
 * @return boolean True: With debug information. False: No debug information.
 * @since 1.0
 */

public static boolean isDebug() {
    return System.getProperty("DEBUG_RewriteUtilities") != null;
}
}
