package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common.dag;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//	Rainbow 1.0 - XQuery Processing.
//
//	Copyright 2003, All Rights Reserved.
//
//	Permission to use, copy, modify and distribute this software and
//	its documentation for educational, research and non-profit
//	purposes, without fee, and without a written agreement is hereby
//	granted, provided that this copyright notice appear in all copies.
//
//	Permission to incorporate this software into commercial products
//	may be obtained directly from the authors.
//
//	This software is provided "as is" without express or implied warranty.
//	In no event will the authors be held liable for any damages arising
//	from the use of this software.
//
//	Authors:
//	Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//	Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//	Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//	Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//		Computer Science Department
//		Worcester Polytechnic Institute
//		Worcester, MA 01609 USA
//
//		http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//	Send bug reports, bug fixes, enhancements, requests, or comments to
//	the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dompattern.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;

/**
 * This Class is used to push down operators by traversing the XAT.
 * 
 * @author Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since 1.0
 */

public class DagRewriteRules extends ReWriteRules {

	public DagRewriteRules() {
		super();
	}

	public Object visit(NatureJoin parent, XATMultiParentOperator child) {

		if (isDebug()) {
			System.out
					.println("********* In NatureJoin / XATMultiParentOperator *************");
		}
		ReWriteResult p1 = new ReWriteResult();
		p1.setDeleteParentNode(true);
		p1.setFinalPosition(true);
		return p1;
	}

	/***************************************************************************
	 * Rewrite rules to get extra NaturalJoin as a linear tree NaturalJoin | |
	 * NavCollection, NavCollection | MPO
	 * 
	 * =====>
	 * 
	 * NavCollection | NavCollection, | MPO
	 **************************************************************************/
	public Object visit(NatureJoin parent, XATOperator child) {
		if (isDebug()) {
			System.out
					.println("********* In NatureJoin / XATOperator *************");
		}

		ReWriteResult p1 = new ReWriteResult();

		//From the Natural Join, trying to find the MPO from left and right
		// branch
		XATNode[] children = parent.getNode().getChildren();

		//to make sure that the child is always rightChild
		XATNode leftChild;
		XATNode rightChild;
		if ((child.getNode()).equals((XATNode) children[1])){
			leftChild = (XATNode) children[0];
			rightChild = (XATNode) children[1];
		}else{
			leftChild = (XATNode) children[1];
			rightChild = (XATNode) children[0];
		}

		//now start searching whether NatureJoin is rewritable
		//each branch should be a linear tree without NavUnnest
		XATNode leftCurParent = parent.getNode();
		XATNode leftCurChild = leftChild;
		while (!(leftCurChild.getOperator() instanceof XATMultiParentOperatorImp)) {
			if (!((leftCurChild.getOperator() instanceof XATMultiSourceOperatorImp)
					|| (leftCurChild.getOperator() instanceof NavUnnestImp) || (leftCurChild
					.getOperator() instanceof OuterNavUnnestImp))) {
				leftCurParent = leftCurChild;
				leftCurChild = leftCurChild.getChild(0);
			} else {
				return p1;
			}
		}
		XATNode leftFoundMPO = leftCurChild;

		XATNode rightCurParent = parent.getNode();
		XATNode rightCurChild = rightChild;
		while (!(rightCurChild.getOperator() instanceof XATMultiParentOperatorImp)) {
			if (!((rightCurChild.getOperator() instanceof XATMultiSourceOperatorImp)
					|| (rightCurChild.getOperator() instanceof NavUnnestImp) || (rightCurChild
					.getOperator() instanceof OuterNavUnnestImp))) {
				rightCurParent = rightCurChild;
				rightCurChild = rightCurChild.getChild(0);
			} else {
				return p1;
			}
		}
		XATNode rightFoundMPO = rightCurChild;

		//decide whether rewritable
		// if the MPO from left branch is equal to MPO in right brandh, then
		// rewritable
		if (leftFoundMPO.equals(rightFoundMPO)) {
			//connect the left branch and right branch
			((DagNode) leftChild).deleteParent(parent.getNode());
			leftChild.setParent(rightCurParent);
			int index = ((DagNode) rightCurParent)
					.getIndexOfChild((DagNode) rightFoundMPO);
			rightCurParent.setChild(index, leftChild);

			//clean the MPO
			((DagNode) rightFoundMPO).deleteParent(rightCurParent);

			//clean NaturalJoin
			((DagNode) parent.getNode()).deleteChild(leftChild);

		}
		p1.setDeleteParentNode(true);
		p1.setFinalPosition(true);
		return p1;
	}
}