package edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.sql;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.io.*;
import java.util.*;
import java.sql.*;
import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.function.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.navigation.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.xatrewrite.common.*;

/**
 * This Class is used to rewrite the XAT using position rules.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class ReWritePositionRules extends ReWriteRules {
    final static String PROPERTIES_FILE_NAME = "properties.txt";
/**
 * Default ReWritePositionRules constructor.
 * @since 1.0
 */
public ReWritePositionRules() {
	super();
}
/**
 * Rule to eliminate a Position Function node and replace it with a
 * global position function. In the instance that after decorrelation
 * a function node still exists, it is because it is a multi-step order
 * function with root context.  Otherwise it would be attached to a groupby
 * in the decorrelation of the for node.
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent Function operator.
 * @param child  The child Navigate operator.
 * @since 1.0
 */
public Object visit(Function parent, Navigate child) {

    ReWriteResult p1 = new ReWriteResult();

    if (isDebug())
        System.out.println(
            "****************** Function -- Navigate " + parent.getName());

    if (parent.getName().equals("position")) {
        if (isDebug())
            System.out.println("Function equals position");

        java.util.Properties defaultProperties = new Properties();

        try {
            FileInputStream fileIn = new FileInputStream(PROPERTIES_FILE_NAME);
            defaultProperties.load(fileIn);
        } catch (Exception e1) {
            System.out.println(e1.toString());
        }

        String username = new String(defaultProperties.getProperty("username"));
        String passwd = new String(defaultProperties.getProperty("password"));
        String uri = new String(defaultProperties.getProperty("uri"));
        String metadataTable = new String(defaultProperties.getProperty("metadataTable"));
        // Done

        if (isDebug())
        	System.out.println("Username = " + username + ", uri = " + uri);
        
        /* Get binding value, and full path */
        NavigationStep[] posNavStep = null;
		posNavStep =  getVariableTable().getVariableValue((String) child.getBinding());
	
        String posNavString = createString(posNavStep);
                
        /* Substract header from string*/
        String posNavStringMod = "", posNavStringPreMod = "";
        //Merged tree element string will resemble <root_tag>XXX</root_tag>/stuff/desired
	    posNavStringPreMod = posNavString.substring(posNavString.lastIndexOf(">")+1, posNavString.length());

	    if(posNavStringPreMod.endsWith("/")){
			posNavStringPreMod = posNavStringPreMod.substring(0,posNavStringPreMod.length()-1);
	    }
	    if(posNavStringPreMod.indexOf("/#text") > 0){
	        posNavStringMod = posNavStringPreMod.substring(0, posNavStringPreMod.indexOf("/#text"));
    	}else{
	    	posNavStringMod = posNavStringPreMod;
    	}
        String sqlstmt =
            "select * from " + metadataTable + " where xml_path like \'%" + posNavStringMod + "\'";

        if (isDebug())
        	System.out.println(sqlstmt);
        // Establishes the connection with db, executes SQL and
        // stores results in the XATTable for the node
        String query;
        ResultSet rs;
        PreparedStatement stmt;
        Connection con;
        Vector theResults = new Vector();


        //Establish connection with the database
        try {
            try {
                Class.forName("oracle.jdbc.driver.OracleDriver");
            } catch (ClassNotFoundException cnfe) {
	            System.err.println("unable to load Oracle JDBC driver");
                throw new RuntimeException("unable to load Oracle JDBC driver");
            }
            con = DriverManager.getConnection(uri, username, passwd);
        } catch (SQLException sqlex) {
	        System.err.println("could not establish connection");
            throw new RuntimeException("could not establish connection");
        }
        //Execute the query on the database
        try {
            stmt = con.prepareStatement(sqlstmt);
            if(isDebug()) System.out.println("SQL: " + sqlstmt);
            rs = stmt.executeQuery();
        } catch (SQLException sqlex) {
            System.err.println("could not execute query " + sqlstmt);
            throw new RuntimeException("could not execute query " + sqlstmt);
        }

        if(isDebug())System.out.println("Query executed");
        try {

            ResultSetMetaData rsmt = rs.getMetaData();
            int columnNum =  rsmt.getColumnCount();
            
            //Only one result is expected
            while (rs.next()){
	            //System.out.println("in the data loop");
                Vector column = new Vector();
                for (int i = 1; i <= columnNum; i++) {
	                //System.out.println("Data adding: " + rs.getObject(i));
                    column.addElement(rs.getString(i));
                }
                theResults.addElement(column);
            }
            rs.close();
            stmt.close();

        } catch (SQLException sqlex) {
            System.err.println("could not retrieve query results " + sqlstmt);
            sqlex.printStackTrace();
            throw new RuntimeException("could not retrieve query results " + sqlstmt);
        }

        
        Vector results = (Vector) theResults.firstElement();
        
        //XML Path will be the 0th entry of the 0th vector
        if (isDebug())System.out.println("XML: " + (String) results.elementAt(0));

        //DXV PATH
        if (isDebug())System.out.println("DXV: " + (String) results.elementAt(1));

        //Order Loading State
        if (isDebug())System.out.println("Metadata: " + (String) results.elementAt(2));

        //Context
        if (isDebug())System.out.println("Context: " + (String) results.elementAt(3));

        //AorN
        if (isDebug())System.out.println("AorN: " + (String) results.elementAt(4));

        Object[] newParams = new Object[results.size()-1];
        int j;
        for(j = 1; j<results.size(); j++){
			newParams[j-1] = results.elementAt(j);
        }
        
        XATNode funcNode = new XATNodeImp();
        Function funcSing = new FunctionImp(parent.getNode());
        funcSing.setName("multisteporder");
        funcSing.setParams(newParams);
        funcSing.setOutputColumnName(parent.getOutputColumnName());

        parent.getNode().setOperator(funcSing);
    } //else do nothing

    p1.setFinalPosition(true);
    p1.setSwapped(false);
    return p1;
}
/**
 * Rule to eliminate a Position Function node and replace it with either a single or
 * multi step position function.  It will be a single step if the binding of the nav
 * is equal to the parameter of the function.  Otherwise the binding of the nav will be
 * an ancestor, and it will be a multi step replace.
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent GroupBy operator.
 * @param child  The child GroupBy operator.
 * @since 1.0
 */
public Object visit(GroupBy parent, GroupBy child) {

    ReWriteResult p1 = new ReWriteResult();
    Function function = (Function) parent.getXATTree().getRoot().getOperator();
    if (isDebug())
        System.out.println(
            "****************** Groupby -- Navigate "
                + function.getName()
                + " child: "
                + child);

    if (function.getName().equalsIgnoreCase("position")) {
        String[] colsGroup = parent.getColumnNames();

        if (isDebug())
            System.out.println("Function equals position");

        java.util.Properties defaultProperties = new Properties();

        try {
            FileInputStream fileIn = new FileInputStream(PROPERTIES_FILE_NAME);
            defaultProperties.load(fileIn);
        } catch (Exception e1) {
            System.err.println(e1.toString());
        }

        String username = new String(defaultProperties.getProperty("username"));
        String passwd = new String(defaultProperties.getProperty("password"));
        String uri = new String(defaultProperties.getProperty("uri"));
        String metadataTable = new String(defaultProperties.getProperty("metadataTable"));
       
        // Done

        if (isDebug())System.out.println("++++" + username + passwd + uri);

        /* Get binding value, and full path */
        NavigationStep[] posNavStep = null;
        try {
            if (isDebug())System.out.println(
                "****" + ((Navigate) child.getNode().getChild(0)).getBinding());
            posNavStep =
                getVariableTable().getVariableValue(
                    (String) ((Navigate) child.getNode().getChild(0)).getBinding());
        } catch (XATNodeDoesNotExistException xat) {
            xat.printStackTrace();
        }
        String posNavString = createString(posNavStep);

        /* Substract header from string*/
        String posNavStringMod = "", posNavStringPreMod = "";
        //Merged tree element string will resemble <root_tag>XXX</root_tag>/stuff/desired
	    posNavStringPreMod = posNavString.substring(posNavString.lastIndexOf(">")+1, posNavString.length());

	    if(posNavStringPreMod.endsWith("/")){
			posNavStringPreMod = posNavStringPreMod.substring(0,posNavStringPreMod.length()-1);
	    }
	    if(posNavStringPreMod.indexOf("/#text") > 0){
	        posNavStringMod = posNavStringPreMod.substring(0, posNavStringPreMod.indexOf("/#text"));
    	}else{
	    	posNavStringMod = posNavStringPreMod;
    	}
        String sqlstmt =
            "select * from " + metadataTable + " where xml_path like \'%" + posNavStringMod + "\'";

		// Establishes the connection with db, executes SQL and
        // stores results in the XATTable for the node
        String query;
        ResultSet rs;
        PreparedStatement stmt;
        Connection con;
        Vector theResults = new Vector();
        
        //Establish connection with the database
        try {
            try {
                Class.forName("oracle.jdbc.driver.OracleDriver");
            } catch (ClassNotFoundException cnfe) {
	            System.err.println("unable to load Oracle JDBC driver");
                throw new RuntimeException("unable to load Oracle JDBC driver");
            }
            con = DriverManager.getConnection(uri, username, passwd);
        } catch (SQLException sqlex) {
	        System.err.println("could not establish connection");
            throw new RuntimeException("could not establish connection");
        }
        //Execute the query on the database
        try {
            stmt = con.prepareStatement(sqlstmt);
            if(isDebug()) System.out.println("SQL: " + sqlstmt);
            rs = stmt.executeQuery();
        } catch (SQLException sqlex) {
            System.err.println("could not execute query " + sqlstmt);
            throw new RuntimeException("could not execute query " + sqlstmt);
        }

        if(isDebug())System.out.println("Query executed");
        try {

            ResultSetMetaData rsmt = rs.getMetaData();
            int columnNum = rsmt.getColumnCount();

            //Only one result is expected
            while (rs.next()){
	            //System.out.println("in the data loop");
                Vector column = new Vector();
                for (int i = 1; i <= columnNum; i++) {
	                //System.out.println("Data adding: " + rs.getObject(i));
                    column.addElement(rs.getString(i));
                }
                theResults.addElement(column);
            }
            rs.close();
            stmt.close();

        } catch (SQLException sqlex) {
            System.err.println("could not retrieve query results " + sqlstmt);
            sqlex.printStackTrace();
            throw new RuntimeException("could not retrieve query results " + sqlstmt);
        }

        
        Vector results = (Vector) theResults.firstElement();

        //XML Path will be the 0th entry of the 0th vector
        if (isDebug())System.out.println("XML: " + (String) results.elementAt(0));

        //DXV PATH
        if (isDebug())System.out.println("DXV: " + (String) results.elementAt(1));

        //Order Loading State
        if (isDebug())System.out.println("Metadata: " + (String) results.elementAt(2));

        //Context
        if (isDebug())System.out.println("Context: " + (String) results.elementAt(3));

        //AorN
        if (isDebug())System.out.println("AorN: " + (String) results.elementAt(4));

        Object[] newParams = new Object[results.size() - 1 + colsGroup.length];
        int j;
        for (j = 1; j < results.size(); j++) {
            newParams[j - 1] = results.elementAt(j);
        }
        for (int k = 0; k < colsGroup.length; k++) {
            newParams[j - 1] = colsGroup[k];
            j++;
        }

        Function funcSing = new FunctionImp(parent.getNode());
        funcSing.setParams(newParams);
        funcSing.setName("multisteporder");
        funcSing.setOutputColumnName(function.getOutputColumnName());
        parent.getNode().setOperator(funcSing);

    } //else do nothing

    p1.setFinalPosition(true);
    p1.setSwapped(false);
    return p1;
}
/**
 * Rule to eliminate a Position Function node and replace it with either a single or
 * multi step position function.  It will be a single step if the binding of the nav
 * is equal to the parameter of the function.  Otherwise the binding of the nav will be
 * an ancestor, and it will be a multi step replace.
 * @return ReWriteResult. The swap flag in ReWriteResult true if the nodes 
 * were swapped, false otherwise.
 * @param parent The parent GroupBy operator.
 * @param child  The child Navigate operator.
 * @since 1.0
 */
public Object visit(GroupBy parent, Navigate child) {

    ReWriteResult p1 = new ReWriteResult();
    Function function = (Function) parent.getXATTree().getRoot().getOperator();
    if (isDebug())
        System.out.println(
            "****************** Groupby -- Navigate " + function.getName() + " child: " + child);

    if (function.getName().equalsIgnoreCase("position")) {
	    String[] colsGroup = parent.getColumnNames();
		
	    if (isDebug())
            System.out.println("Function equals position");

        java.util.Properties defaultProperties = new Properties();

        try {
            FileInputStream fileIn = new FileInputStream(PROPERTIES_FILE_NAME);
            defaultProperties.load(fileIn);
        } catch (Exception e1) {
            System.err.println(e1.toString());
        }

        String username = new String(defaultProperties.getProperty("username"));
        String passwd = new String(defaultProperties.getProperty("password"));
        String uri = new String(defaultProperties.getProperty("uri"));
        String metadataTable = new String(defaultProperties.getProperty("metadataTable"));
       
         // Done
        
        /* Get binding value, and full path */
        if(isDebug()) System.out.println("****" + child.getBinding());
        NavigationStep[] posNavStep = null;
		posNavStep =  getVariableTable().getVariableValue((String) child.getBinding());
	
        String posNavString = createString(posNavStep);
        if (isDebug()) System.out.println("++++" + posNavString);
        
        /* Substract header from string*/
        String posNavStringMod = "", posNavStringPreMod = "";
        //Merged tree element string will resemble <root_tag>XXX</root_tag>/stuff/desired
	    posNavStringPreMod = posNavString.substring(posNavString.lastIndexOf(">")+1, posNavString.length());

	    if(posNavStringPreMod.endsWith("/")){
			posNavStringPreMod = posNavStringPreMod.substring(0,posNavStringPreMod.length()-1);
	    }
	    if(posNavStringPreMod.indexOf("/#text") > 0){
	        posNavStringMod = posNavStringPreMod.substring(0, posNavStringPreMod.indexOf("/#text"));
    	}else{
	    	posNavStringMod = posNavStringPreMod;
    	}
        String sqlstmt =
            "select * from " + metadataTable + " where xml_path like \'%" + posNavStringMod + "\'";

        // Establishes the connection with db, executes SQL and
        // stores results in the XATTable for the node
        String query;
        ResultSet rs;
        PreparedStatement stmt;
        Connection con;
        Vector theResults = new Vector();
        
        //Establish connection with the database
        try {
            try {
                Class.forName("oracle.jdbc.driver.OracleDriver");
            } catch (ClassNotFoundException cnfe) {
	            System.err.println("unable to load Oracle JDBC driver");
                throw new RuntimeException("unable to load Oracle JDBC driver");
            }
            con = DriverManager.getConnection(uri, username, passwd);
        } catch (SQLException sqlex) {
	        System.err.println("could not establish connection");
            throw new RuntimeException("could not establish connection");
        }
        //Execute the query on the database
        try {
            stmt = con.prepareStatement(sqlstmt);
            if(isDebug()) System.out.println("SQL: " + sqlstmt);
            rs = stmt.executeQuery();
        } catch (SQLException sqlex) {
            System.err.println("could not execute query " + sqlstmt);
            throw new RuntimeException("could not execute query " + sqlstmt);
        }

        if(isDebug())System.out.println("Query executed");
        try {

            ResultSetMetaData rsmt = rs.getMetaData();
            int columnNum = rsmt.getColumnCount();

            //Only one result is expected
            while (rs.next()){
	            //System.out.println("in the data loop");
                Vector column = new Vector();
                for (int i = 1; i <= columnNum; i++) {
	                //System.out.println("Data adding: " + rs.getObject(i));
                    column.addElement(rs.getString(i));
                }
                theResults.addElement(column);
            }
            rs.close();
            stmt.close();

        } catch (SQLException sqlex) {
            System.err.println("could not retrieve query results " + sqlstmt);
            sqlex.printStackTrace();
            throw new RuntimeException("could not retrieve query results " + sqlstmt);
        }

        
        Vector results = (Vector) theResults.firstElement();
        
        if(isDebug())System.out.println("Results retrieved:" + results);

        //XML Path will be the 0th entry of the 0th vector
        if (isDebug())System.out.println("XML: " + (String) results.elementAt(0));

        //DXV PATH
        if (isDebug())System.out.println("DXV: " + (String) results.elementAt(1));
        
        //Order Loading State
        if (isDebug())System.out.println("Metadata: " + (String) results.elementAt(2));

        //Context
        if (isDebug())System.out.println("Context: " + (String) results.elementAt(3));

        //AorN
        if (isDebug())System.out.println("AorN: " + (String) results.elementAt(4));

        Object[] newParams = new Object[results.size()-1 + colsGroup.length];
        int j;
        for(j = 1; j<results.size(); j++){
			newParams[j-1] = results.elementAt(j);
        }		
        for(int k = 0; k < colsGroup.length; k++){
			newParams[j-1] = colsGroup[k];
			j++;
		}
 		
        Function funcSing = new FunctionImp(parent.getNode());
        funcSing.setParams(newParams);
		funcSing.setName("singlesteporder");
        funcSing.setOutputColumnName(function.getOutputColumnName());
        parent.getNode().setOperator(funcSing);
   
    } //else do nothing

    p1.setFinalPosition(true);
    p1.setSwapped(false);
    return p1;
}
}
