package edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////

import java.util.Vector;
import java.sql.*;

/**
 * This class clean the relational Database.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

public class CleanDB {
/**
 * Cleanup database with in specific user space.
 * @param uri The JDBC connection string.
 * @param user The user name of database.
 * @param pass The password of database.
 * @since 1.0
 */
public static void clean(String uri, String user, String pass) {
    if ("SYS".compareToIgnoreCase(user) == 0
        || "SYSTEM".compareToIgnoreCase(user) == 0) {
        System.out.println("Cannot delete tables for SYS or SYSTEM. Too dangerous");
        return;
    }
    boolean more = false;
    do {
        try {
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
            Class.forName("oracle.jdbc.driver.OracleDriver");
            Connection con = DriverManager.getConnection(uri, user, pass);
            Statement stmt = con.createStatement();
            // get all the tables of that user.
            ResultSet rs = stmt.executeQuery("select table_name from tabs");
            Vector tables = new Vector();
            while (rs.next()) {
                tables.addElement(rs.getString("table_name"));
            }
            char choice = 'N';
            if (tables.size() > 0)
                more = true;
            else
                more = false;
            // drop the tables.
            for (int i = 0; i < tables.size(); i++) {
                System.out.print("Drop table " + tables.elementAt(i) + " (Y/N/A/Q): ");
                if (choice != 'A') {
                    choice = Character.toUpperCase((char) System.in.read());
                    if (choice == 'Q') { //stop
                        return;
                    }
                    System.in.skip(2); // skip \r \n.
                }
                if (choice == 'A' || choice == 'Y') {
                    stmt.executeUpdate("drop table " + tables.elementAt(i));
                    System.out.println("Table " + tables.elementAt(i) + " dropped.");
                }
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }
    while (more);

}
/**
 * Cleanup database within specific user space.
 * @param uri The JDBC connection string.
 * @param user The user name of database.
 * @param pass The password of database.
 * @since 1.0
 */

public static void clean(String uri, String user, String pass, char choice) {
    if ("SYS".compareToIgnoreCase(user) == 0
        || "SYSTEM".compareToIgnoreCase(user) == 0) {
        System.out.println("Cannot delete tables for SYS or SYSTEM. Too dangerous");
        return;
    }
    boolean more = false;
    do {
        try {
            Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
            Class.forName("oracle.jdbc.driver.OracleDriver");
            Connection con = DriverManager.getConnection(uri, user, pass);
            Statement stmt = con.createStatement();
            // get all the tables of that user.
            ResultSet rs = stmt.executeQuery("select table_name from tabs");
            Vector tables = new Vector();
            while (rs.next()) {
                tables.addElement(rs.getString("table_name"));
            }
            if (tables.size() > 0)
                more = true;
            else
                more = false;
            // drop the tables.
            for (int i = 0; i < tables.size(); i++) {
                System.out.print("Drop table " + tables.elementAt(i) + " (Y/N/A/Q): ");
                if (choice != 'A') {
                    if (choice == 'Q') { //stop
                        return;
                    }
                }
                if (choice == 'A' || choice == 'Y') {
                    stmt.executeUpdate("drop table " + tables.elementAt(i));
                    System.out.println("Table " + tables.elementAt(i) + " dropped.");
                }
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }
    while (more);

}
/**
 * Clean the Database.
 * @param args The strings for connection establishing.
 * @since 1.0
 */
public static void main(String[] args) {
    // One sample usage to clean up the xml/xml@dsrg.
    //clean("jdbc:oracle:thin:@dsrg.wpi.edu:1521:ORCL", "xml", "xml");
}
}
