package edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils;

/**
 * Traverse the Dag plan
 * Creation date: (2002-12-24 13:25:18)
 * @author: sk
 */
 
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag;
import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.XATNode;

public class DagTraversal {
/**
 * Utility constructor comment.
 */
public DagTraversal() {
	super();
}
/**
 * This method traverse a DAG from Bottom-up.
 * the return set contains all nodes in traversal order.
 *
 * Creation date: (2003-1-29 19:40:06)
 * @return java.util.Vector
 * @param from edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 * @param to edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public static Vector dagBottomUpTraverse(DagNode leaf, DagNode root) {

  Vector resultSet = new Vector();

  try {

    //Do breath first traverse
    Set curSet = new HashSet();//contains nodes on current visiting level
    Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from the leaf
    curSet.add(leaf);
    do {
			//for each node in current level
	    for (Iterator it = curSet.iterator(); it.hasNext();) {
        DagNode curNode = (DagNode) it.next();

        //visit this node
        resultSet.add(curNode);

        if (curNode != root) {
          //DagNode[] parents = curNode.getParents();

          //push each parent in next level
          //for (int i = 0; i < parents.length; i++) {
					for(Iterator parentIt = curNode.getParents(); parentIt.hasNext();){
						//flag == true means all parents are visited
						DagNode curParent = (DagNode)parentIt.next();
            boolean flag = true;
            XATNode[] prevLevel = curParent.getChildren();
            for (int j = 0; j < prevLevel.length; j++) {
              if (!resultSet.contains(prevLevel[j])) {
                flag = false;
                break;
              }
            }

            if (flag) 
            	nextSet.add(curParent);
          }//push each parent
        } //push next level

      } //for

      curSet = nextSet;
      nextSet = new HashSet();
    } while (!curSet.isEmpty());

  } catch (Exception e) {
    e.printStackTrace();
  }

  return resultSet;
}
/**
 * This method traverse a DAG from Bottom-up.
 * the return set contains all nodes in traversal order.
 *
 * Creation date: (2003-1-29 19:40:06)
 * @return java.util.Vector
 * @param from edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 * @param to edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public static Vector dagBottomUpTraverse(XATDag xatdag) {
		
	Vector resultSet = new Vector();

  try {

	  DagNode root = (DagNode)xatdag.getRoot();
		DagNode leaf = root;
		while(leaf.getChildrenSize() > 0)
			leaf = (DagNode)leaf.getChild(0);

    //Do breath first traverse
    Set curSet = new HashSet();//contains nodes on current visiting level
    Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from the leaf
    curSet.add(leaf);
    do {
			//for each node in current level
	    for (Iterator it = curSet.iterator(); it.hasNext();) {
        DagNode curNode = (DagNode) it.next();

        //visit this node
        resultSet.add(curNode);

        if (curNode != root) {
          //DagNode[] parents = curNode.getParents();

          //push each parent in next level
          //for (int i = 0; i < parents.length; i++) {
					for(Iterator parentIt = curNode.getParents(); parentIt.hasNext();){
						//flag == true means all parents are visited
						DagNode curParent = (DagNode)parentIt.next();
            boolean flag = true;
            XATNode[] prevLevel = curParent.getChildren();
            for (int j = 0; j < prevLevel.length; j++) {
              if (!resultSet.contains(prevLevel[j])) {
                flag = false;
                break;
              }
            }

            if (flag) 
            	nextSet.add(curParent);
          }//push each parent
        } //push next level

      } //for

      curSet = nextSet;
      nextSet = new HashSet();
    } while (!curSet.isEmpty());

  } catch (Exception e) {
    e.printStackTrace();
  }

  return resultSet;
}
/**
 * This method traverse a DAG from top-dwon.
 * the return set contains all nodes in traversal order.
 *
 * Creation date: (2003-1-29 19:40:06)
 * @return java.util.Vector
 * @param from edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 * @param to edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public static Vector dagTopDownTraverse(DagNode root) {

  Vector resultSet = new Vector();

  try {

    //Do breath first traverse
    Set curSet = new HashSet();//contains nodes on current visiting level
    Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from root
    curSet.add(root);
    do {
			//for each node in current level
	    for (Iterator it = curSet.iterator(); it.hasNext();) {
        DagNode curNode = (DagNode) it.next();

        //visit this node
        resultSet.add(curNode);

        if (curNode.getChildrenSize() > 0) {
          XATNode[] children = curNode.getChildren();

          //push each child in next level
          for (int i = 0; i < children.length; i++) {
            DagNode curChild = (DagNode) children[i];

						//flag == true means all parents are visited
            boolean flag = true;
            //DagNode[] prevLevel = curChild.getParents();
            //for (int j = 0; j < prevLevel.length; j++) {
            for(Iterator prevIt = curChild.getParents(); prevIt.hasNext();){
	            DagNode curPrev = (DagNode)prevIt.next();
	            if (!resultSet.contains(curPrev)) {
                flag = false;
                break;
              }
            }

            if (flag)
              nextSet.add(curChild);
          }//push each child
        } //push next level

      } //for

      curSet = nextSet;
      nextSet = new HashSet();
    } while (!curSet.isEmpty());

  } catch (Exception e) {
    e.printStackTrace();
  }

  return resultSet;
}
/**
 * This method traverse a DAG from top-dwon.
 * the return set contains all nodes in traversal order.
 *
 * Creation date: (2003-1-29 19:40:06)
 * @return java.util.Vector
 * @param from edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 * @param to edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public static Vector dagTopDownTraverse(DagNode root, DagNode leaf) {

  Vector resultSet = new Vector();

  try {

    //Do breath first traverse
    Set curSet = new HashSet();//contains nodes on current visiting level
    Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from root
    curSet.add(root);
    do {
			//for each node in current level
	    for (Iterator it = curSet.iterator(); it.hasNext();) {
        DagNode curNode = (DagNode) it.next();

        //visit this node
        resultSet.add(curNode);

        if (curNode != leaf) {
          XATNode[] children = curNode.getChildren();

          //push each child in next level
          for (int i = 0; i < children.length; i++) {
            DagNode curChild = (DagNode) children[i];

						//flag == true means all parents are visited
            boolean flag = true;
            //DagNode[] prevLevel = curChild.getParents();
            //for (int j = 0; j < prevLevel.length; j++) {
            for(Iterator prevIt = curChild.getParents(); prevIt.hasNext();){
	            DagNode curPrev = (DagNode)prevIt.next();
	            if (!resultSet.contains(curPrev)) {
                flag = false;
                break;
              }
            }

            if (flag)
              nextSet.add(curChild);
          }//push each child
        } //push next level

      } //for

      curSet = nextSet;
      nextSet = new HashSet();
    } while (!curSet.isEmpty());

  } catch (Exception e) {
    e.printStackTrace();
  }

  return resultSet;
}
/**
 * This method traverse a DAG from top-dwon.
 * the return set contains all nodes in traversal order.
 *
 * Creation date: (2003-1-29 19:40:06)
 * @return java.util.Vector
 * @param from edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 * @param to edu.wpi.cs.dsrg.xmldb.xat.common.dag.DagNode
 */
public static Vector dagTopDownTraverse(XATDag xatdag) {

  Vector resultSet = new Vector();

  try {

    //Do breath first traverse
    Set curSet = new HashSet();//contains nodes on current visiting level
    Set nextSet = new HashSet();//contains nodes on next visiting level

		//traversal begins from root
    curSet.add(xatdag.getRoot());
    do {
			//for each node in current level
	    for (Iterator it = curSet.iterator(); it.hasNext();) {
        DagNode curNode = (DagNode) it.next();

        //visit this node
        resultSet.add(curNode);

        if (curNode.getChildrenSize() > 0) {
          XATNode[] children = curNode.getChildren();

          //push each child in next level
          for (int i = 0; i < children.length; i++) {
            DagNode curChild = (DagNode) children[i];

						//flag == true means all parents are visited
            boolean flag = true;
            //DagNode[] prevLevel = curChild.getParents();
            //for (int j = 0; j < prevLevel.length; j++) {
            for(Iterator prevIt = curChild.getParents(); prevIt.hasNext();){
	            DagNode curPrev = (DagNode)prevIt.next();
	            if (!resultSet.contains(curPrev)) {
                flag = false;
                break;
              }
            }

            if (flag)
              nextSet.add(curChild);
          }//push each child
        } //push next level

      } //for

      curSet = nextSet;
      nextSet = new HashSet();
    } while (!curSet.isEmpty());

  } catch (Exception e) {
    e.printStackTrace();
  }

  return resultSet;
}
/**
 * Insert the method's description here.
 * Creation date: (2003-2-4 20:05:35)
 * @param xatdag edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATDag
 */
public static void printDag(XATDag xatdag) {
 //do a breadth first traverse from top down
  try {
    //do a breadth first traverse from top down
    Vector traversal = dagTopDownTraverse(xatdag);
    for (Iterator it = traversal.iterator(); it.hasNext();) {
      DagNode curNode = (DagNode) it.next();
      //visit curNode
      System.out.println("( " + curNode.getOperator());

      for (int i = 0; i < curNode.getChildrenSize(); i++)
        System.out.println("    -->" + curNode.getChild(i).getOperator());
      System.out.println(")");
    }
  } catch (Exception e) {
    e.printStackTrace();
  }
}
}
