package edu.wpi.cs.dsrg.xmldb.xat.recursion;

/**
 * SAX parser, called by source operator, i.e., XMLScanStream
 * Creation date: (3/30/2003 2:29:49 PM)
 * @author: Hong Su
 */
import java.io.Serializable;
import java.util.List;

import edu.wpi.cs.dsrg.xmldb.xat.automata.utils.ClassExecTimeKeeper;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.automata.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automata.storemanager.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.ContinuousOpt;
import edu.wpi.cs.dsrg.xmldb.xat.component.automatainout.SimulateMoveResult;
import edu.wpi.cs.dsrg.xmldb.xat.component.statistics.PlanMigrator;
import edu.wpi.cs.dsrg.xmldb.xat.component.statistics.StatisticsCollector;

//import org.xml.sax.*;

public class TokenHandler implements Serializable {
	ExtendedAutomata automata;
	public Finer_StreamStoreMgr storeMgr;
	//id of start tokens
	protected int startTokenId = 0;
	//records the time that materializes the data   
	protected ClassExecTimeKeeper timeKeeperReasoning =
		new ClassExecTimeKeeper(this, "reasoning");
	protected ClassExecTimeKeeper timeKeeperSta = new ClassExecTimeKeeper(this, "statistics");
	protected ClassExecTimeKeeper timeKeeperMigrate = new ClassExecTimeKeeper(this, "migrate");
	
	private int bottommostDestCount = 0;
	
	int count=0;
	//best plan in current phase
	SimulateMoveResult bestResult = null;
	
	/**
	 * TokenHandler constructor comment.
	 */
	public TokenHandler() {
		super();
	}

	/**
	 * TokenHandler constructor comment.
	 */
	public TokenHandler(ExtendedAutomata automata) {
		super();
		this.automata = automata;

		//reset: add the initial state into the current active state set (curSet)
		automata.reset();

		storeMgr = automata.getStoreMgr();
		//automata.reset();

	}

	
	public void setAutomata(ExtendedAutomata automata) {
		this.automata = automata;

		automata.reset();

		this.storeMgr = automata.getStoreMgr();
		//automata.reset();
	}

	/**
	 * characters method comment.
	 */
	public void characters(String pcdata) {

		//timeKeeperStore.start();
		//check the run time stack of the automata to decide whether this pcdata must be stored in the storeMgr
		/*if (automata.getCompleteTreeCount() > 0
			&& automata.getDomCount() == 0) {
			//need to store current element as a tree
		
			//storeMgr.putAsCompleteTreeIntoStorageSlip(pcdata, null);
			storeMgr.putAsDomIntoStorageSlip(pcdata, null);
		}*/
		

		if (automata.getTypedValCount() > 0) {
			storeMgr.putAsTypedValIntoStorageSlip(pcdata);
		}

		if (automata.getDomCount() > 0) {
			storeMgr.putAsDomIntoStorageSlip(pcdata);
		}
		//timeKeeperStore.pause();

	}

	/**
	 * endElement method comment.
	 */
	public void endElement(int tokenId) {
	
		//timeKeeperStore.start();
		if (automata.getTypedValCount() > 0) {
			//just an inform rather than actually storing data
			storeMgr.putAsTypedValIntoStoragePop();
		}

		if (automata.getDomCount() > 0) {

			//just an inform rather than actually storing data
			storeMgr.putAsDomIntoStoragePop();
		}

	//	timeKeeperStore.pause();

	//	timeKeeperOthers.start();
		automata.triggerOpsToRunPop(tokenId, startTokenId);
	//	automata.triggerOpsToRunPopWithRecur(tokenId, startTokenId);
	//	timeKeeperOthers.pause();

		//***commented out for schema-blindless codes**//
		automata.triggerSuspensionPop();
		//****// 

	//	timeKeeperAutomata.start();
		//drive automata
		automata.backtrack();
			
	//	timeKeeperAutomata.pause();

	}
	
	/**
	 * endElement method comment.
	 * perform special actions: statistics collection and reasoning when the
	 * destination element of the bottommost tokenNav has been located
	 * desiredBottomDestCount times
	 */
	public void endElement(int tokenId, int desiredBottomDestCount) {
	
		//timeKeeperStore.start();
		if (automata.getTypedValCount() > 0) {
			//just an inform rather than actually storing data
			storeMgr.putAsTypedValIntoStoragePop();
		}

		if (automata.getDomCount() > 0) {
			//just an inform rather than actually storing data
			storeMgr.putAsDomIntoStoragePop();
		}

	//	timeKeeperStore.pause();

	//	timeKeeperOthers.start();
	//	automata.triggerOpsToRunPop(tokenId, startTokenId);
		// used for recursive query
		automata.triggerOpsToRunPopWithRecur(tokenId, startTokenId);
	//	timeKeeperOthers.pause();

		//***commented out for schema-blindless codes**//
		automata.triggerSuspensionPop();
		//****// 

	//	timeKeeperAutomata.start();
		//drive automata
		if (automata.backtrack()){
			//increment the count for the destination element
			//node of the bottommost tokenNav
			bottommostDestCount = bottommostDestCount+1;
							
		    if (desiredBottomDestCount!=0 && bottommostDestCount%desiredBottomDestCount==0){
		    	//need to collect statistics
		    	timeKeeperSta.start();
		    	double threshold = 0;
		    	List navsChanged = StatisticsCollector.continuousCollect(automata,
						threshold, desiredBottomDestCount);
		    	timeKeeperSta.pause();
		    	
		    	//begin optimization
		    	timeKeeperReasoning.start();
		    	bestResult = ContinuousOpt.reasoning(automata, threshold, navsChanged, bestResult);
		    	timeKeeperReasoning.pause();
		    	
				//now migrate
		    	timeKeeperMigrate.start();
		    	if (ContinuousOpt.getBetterPlanFound()){
		    		PlanMigrator.migrateToNew(bestResult);
		    		count++;
		    	}
				timeKeeperMigrate.pause();
		    }
		}
		
			

	}

	public ClassExecTimeKeeper getTimeKeeperSta() {
		return timeKeeperSta;
	}
	
	public ClassExecTimeKeeper getTimeKeeperReasoning(){
		return timeKeeperReasoning;
	}
	
	public ClassExecTimeKeeper getTimeKeeperMigrate(){
		return timeKeeperMigrate;
	}

	public void reset() {
		startTokenId = 0;

	}

	/**
	 * startElement method comment.
	 */
	public void startElement(String localName, int tokenId) {
		//timeKeeperAutomata.start();

		startTokenId++;

		//send current token id to the auto
		//automata.setCurTokenId(tokenId);
		//update nfa
		automata.move(localName);
		

		//must put this snippet before store
		//the extract may be suspended
		//****commented out for Schema-blindless case**//
		//automata.triggerSuspensionPush();
		//*** commented out for schema-blindless case*//

		//check the run time stack of the automata to decide whether this element must be stored in the storeMgr
//		System.out.println("TypeValCount " + automata.getTypedValCount());
		if (automata.getTypedValCount() > 0) 
			storeMgr.putAsTypedValIntoStoragePush(localName);
		
	//	System.out.println("DomCount " + automata.getDomCount());
		if (automata.getDomCount() > 0) 
			storeMgr.putAsDomIntoStoragePush(localName);

		//must put this snippet before store
		//the extract may be suspended
		//automata.triggerOpsToRunPush(tokenId, startTokenId);
		automata.triggerOpsToRunPushWithRecur(tokenId, startTokenId);
		

	}
	
	public String toString(){
		return "TokenHandler";
	}
}
