/**
 * RainDrop code.
 * 
 * Copyright of the Software is and will remain with The Trustees of
 * WPI and/or its employees and shall at no point transfer to Licensee. 
 * Any copyright notices on the Software shall be included on all copies 
 * of the Derivatives, or any parts or portions thereof, in any form, manner 
 * or substance, which are produced by the Licensee including but not 
 * limited to incorporation of the Software into any other program, 
 * technical data, documentation, firmware, or other information of the 
 * like kind, type or quality
 * 
 * (c) 2005, Shoushen Yang(shoushen@cs.wpi.edu)
 */

package edu.wpi.cs.dsrg.xmldb.xat.rsox;


/**
 * This class is used to parse the punctuations.
 * 
 * It only has a static method to parse. 
 * 
 * @author Shoushen Yang (shoushen@cs.wpi.edu) 
 */
public class PuncParser 
{
	/**
	 * The constructor uses the file name as the argument. 
	 * 
	 * @param rsi	the punctuation
	 * 
	 * @return String[]	the parsed Strings. It has six values. The meanings for it is as following:
	 * 
	 * 	0	-	ScopeType
	 *  1	-	ScopeLength
	 * 	2	-	ScopeLengthType
	 *  3	-	TargetType
	 *  4	-	TargetPosType
	 *  5	-	TargetCard
	 *  6	-	Action	
	 */	
	static public String[] parsePunc(String rsi) throws Exception
	{
		String[] ret = rsi.split(",");
		
		if (ret.length != 7)
		{
			throw new Exception("The format for the RSI is not right. It does not have the six values as needed.");
		}
		else
		{
			//first to trim all the strings
			for (int i = 0 ; i < 7 ; i ++)
				ret[i] = ret[i].trim();
			
			//we need to delete the first two "("s for ScopeType
			//first its length should be more than 2
			if ((ret[0].length() > 2) && ((ret[0].substring(0,2)).equals("(("))
				//ScopeLength
				&& (ret[1].length() > 0)
				//ScopeLengthType
				&& (ret[2].length() > 0) && (ret[2].charAt(ret[2].length() - 1) == ')')
				//TargetType
				&& (ret[3].length() > 0) && (ret[3].charAt(0) == '(')
				//TargetCard
				&& (ret[5].length() > 0) && (ret[5].charAt(ret[5].length() - 1) == ')')
				//Action
				&& (ret[6].length() > 0) && (ret[6].charAt(ret[6].length() - 1) == ')'))
			{
				//ScopeType
				ret[0] = ret[0].substring(2).trim();
				//ScopeLength
				//do nothing
				//ScopeLengthTYpe
				ret[2] = ret[2].substring(0,ret[2].length() - 1).trim();
				//TargetType
				ret[3] = ret[3].substring(1).trim();				
				//TargetPosType
				//do nothing			
				//TargetCard 
				ret[5] = ret[5].substring(0,ret[5].length() - 1).trim();			
				//Action
				ret[6] = ret[6].substring(0,ret[6].length() - 1).trim();				
				
				return ret;
			}
			else
				throw new Exception("The format for the RSI is not right, please check the items.");
			
			//now to do with the punctuation.
			
			//ideally first to update the schema graph, then query graph, then automata.
			
		}
	}
}
