package edu.wpi.cs.dsrg.xmldb.xat.testfiles;

/////////////////////////// COPYRIGHT /////////////////////////////////////
//
//  Rainbow 1.0 - XQuery Processing.
//
//  Copyright 2003, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//  Steffen Christ, Katica Dimitrova, Luping Ding, Michal A. Dobos,
//  Maged El-Sayed, Geoff Greene, Huzefa Hakimi, Justin Lutz, Guy D. Mann,
//  Mukesh Mulchandani, Brian Murphy, Bradford Pielech, Alex Proschitsky,
//  Elke A. Rundensteiner, Ling Wang, Song Wang, and Xin Zhang
//
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~dsrg/rainbow/index.htm
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to
//  the Rainbow Development Team: <rainbow@WPI.EDU>.
//
/////////////////////////// COPYRIGHT /////////////////////////////////////
import java.util.Vector;
import java.lang.reflect.Array;
import javax.swing.*;

import edu.wpi.cs.dsrg.xmldb.xat.common.xatnode.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.treestructure.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.expression.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.exception.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.specialoperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.xmloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.common.operator.sqloperator.*;
import edu.wpi.cs.dsrg.xmldb.xat.component.operatorexec.ExecuteVisitor;

/**
 * This Class is used to test XAT operators.
 * @author  Rainbow Group, DSRG, WPI
 * @version RainbowCore 1.0
 * @since   1.0
 */

class XATOperatorTest {
    private static XATTable xtbDefault1;
    private static XATTable xtbDefault2;
    private static XATTree xtrDefault1;
    private static XATTree xtrDefault2;
    JPanel jPanel1 = new JPanel();
    JPanel jPanel2 = new JPanel();
    JButton jButton1 = new JButton();

    private static void AggregateTest() throws XATNodeDoesNotExistException {
        Combine aAgtest;
        XATNode node = new XATNodeImp();
        XATTree tree = new XATTreeImp();
        System.out.println("Testing Combine.");
        aAgtest = new CombineImp(node);
        aAgtest = new CombineImp(node, tree);
        aAgtest.getXATTree();
        aAgtest.getNode();
        aAgtest.setXATTree(tree);
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new CombineImp(node.getChild(0)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }

        try {
            aAgtest.getPrimaryDataSource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    public static void CreateTables() {
        /*
              XATTable xtbDefault1;
              XATTable xtbDefault2;
        */
        try {
            xtbDefault1 = new XATTableImp();
            xtbDefault1.setName("Default 1");
            // column 1 in table 1
            String sColName = "Philosphers";
            Vector vCol2Add = new Vector();
            vCol2Add.addElement("Socrates");
            vCol2Add.addElement("Plato");
            vCol2Add.addElement("Aristotle");
            vCol2Add.addElement("DesCartes");
            xtbDefault1.appendColumn(sColName, vCol2Add);
            // column 2 in table 1
            sColName = "Writers";
            vCol2Add = new Vector();
            vCol2Add.addElement("Shakespeare");
            vCol2Add.addElement("Gibson");
            vCol2Add.addElement("Ginsberg");
            vCol2Add.addElement("Stephenson");
            xtbDefault1.appendColumn(sColName, vCol2Add);
            // column 3 in table 1
            sColName = "Times";
            vCol2Add = new Vector();
            vCol2Add.addElement("noon");
            vCol2Add.addElement("5:00");
            vCol2Add.addElement("2am");
            vCol2Add.addElement("6:47AM");
            xtbDefault1.appendColumn(sColName, vCol2Add);

            xtbDefault2 = new XATTableImp();
            xtbDefault2.setName("Default 2");
            // column 1 in table 1
            sColName = "Women";
            vCol2Add = new Vector();
            vCol2Add.addElement("Debbie");
            vCol2Add.addElement("Rachel");
            vCol2Add.addElement("Delilah");
            vCol2Add.addElement("Laura");
            xtbDefault2.appendColumn(sColName, vCol2Add);

            // column 2 in table 1
            sColName = "Integers";
            vCol2Add = new Vector();
            vCol2Add.addElement("5");
            vCol2Add.addElement("42");
            vCol2Add.addElement("187");
            vCol2Add.addElement("15346");
            xtbDefault2.appendColumn(sColName, vCol2Add);

        }
        catch (XATTableTupleNoMatchException e) {
        }

    }
    public static void CreateTrees() {
        /*
              XATTree xtrDefault1;
              XATTree xtrDefault2;
        */

        /*
             try
              {
                node.getChild(0).setOperator(new GroupByImp(node.getChild(0)));
              }
              catch (XATNodeDoesNotExistException e)
              {
                throw new XATNodeDoesNotExistException();
              }
        */

        System.out.println("Creating default tree one");
        xtrDefault1 = new XATTreeImp();

        System.out.println("Setting root");
        xtrDefault1.setRoot(new XATNodeImp());
        XATNode xnRootNode = xtrDefault1.getRoot();
        String sName = "/";
        Expose expTest = new ExposeImp(xnRootNode);
        expTest.setColumnName(sName);
        xnRootNode.setOperator(expTest);

        System.out.println("Adding first child");
        xtrDefault1.addNode(new XATNodeImp(), xtrDefault1.getRoot());
        try {
            XATNode xnSourceNode = xtrDefault1.getRoot().getChild(0);
            String sLocation = "XOptXMLDocTest.xml";
            Source sRcTest = new SourceImp(xnSourceNode, sLocation, Source.XML);
            xnSourceNode.setOperator(sRcTest);
        }
        catch (XATNodeDoesNotExistException e42) {
        }
        /*
        	// should be a root with 1 child
        	System.out.println("Adding second child");
        	xtrDefault1.addNode(new XATNodeImp(),xtrDefault1.getRoot());
        */

        System.out.println("Creating default tree two");
        xtrDefault2 = new XATTreeImp();
        System.out.println("Setting root");
        xtrDefault2.setRoot(new XATNodeImp());
        System.out.println("Adding first child");
        xtrDefault2.addNode(new XATNodeImp(), xtrDefault2.getRoot());
        /* should be a root with 1 child */
        System.out.println("Adding second child");
        xtrDefault2.addNode(new XATNodeImp(), xtrDefault2.getRoot());

    }
    private static void FORTest() throws XATNodeDoesNotExistException {
        FOR fFoTest;
        XATNode node = new XATNodeImp();
        String[] columnNames = {
        };
        String[] columnNames2 = { "name1", "name2" };
        System.out.println("Testing FOR");
        fFoTest = new FORImp(node);
        node.addChild(new XATNodeImp());
        fFoTest.getColumnNames();
        fFoTest.getInnerSource();
        fFoTest.getNode();
        fFoTest.getOuterSource();
        fFoTest.getOutputDataSource();
        fFoTest.setColumnNames(columnNames);
        fFoTest.setColumnNames(columnNames2);
        try {
            node.getChild(0).setOperator(new FORImp(node.getChild(0)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        String[] sColNames = fFoTest.getColumnNames();
        for (int i = 0; i < Array.getLength(sColNames); i++) {
            System.out.println(sColNames[i] + "\n");
        }

        try {
            fFoTest.getPrimaryDataSource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }
    }
    private static void GroupByTest() throws XATNodeDoesNotExistException {
        GroupBy gGbTest;
        String[] columnNames = { "name1", "name2" };
        XATNode node = new XATNodeImp();
        XATTree tree = new XATTreeImp();
        System.out.println("Testing GroupBy");
        gGbTest = new GroupByImp(node);
        gGbTest = new GroupByImp(node, tree);
        node.addChild(new XATNodeImp());
        gGbTest.getColumnNames();
        gGbTest.getNode();
        gGbTest.getOutputDataSource();
        gGbTest.getXATTree();
        gGbTest.setColumnNames(columnNames);
        gGbTest.setXATTree(tree);
        try {
            node.getChild(0).setOperator(new GroupByImp(node.getChild(0)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        String[] sColNames = gGbTest.getColumnNames();
        for (int i = 0; i < Array.getLength(sColNames); i++) {
            System.out.println(sColNames[i] + "\n");
        }
        try {
            gGbTest.getPrimaryDataSource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    private static void LeftOuterJoinTest() throws XATNodeDoesNotExistException {
        LeftOuterJoin lOjTest;
        XATNode node = new XATNodeImp();
        Expression expression = null;
        System.out.println("Testing LeftOuterJoin");
        lOjTest = new LeftOuterJoinImp(node, expression);
        node.addChild(new XATNodeImp());
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(
                new LeftOuterJoinImp(node.getChild(0), expression));
            node.getChild(1).setOperator(
                new LeftOuterJoinImp(node.getChild(1), expression));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        lOjTest.getExpression();
        lOjTest.getLeft();
        lOjTest.getNode();
        lOjTest.getOutput();
        lOjTest.getRight();
        lOjTest.setExpression(expression);
        Expression sColNames = lOjTest.getExpression();
        System.out.println(sColNames + "\n");
        XATTable left = lOjTest.getLeft();
        System.out.println("left: " + left);
        XATTable right = lOjTest.getRight();
        System.out.println("right: " + right);

        try {
            lOjTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    private static void LeftSemiJoinTest() throws XATNodeDoesNotExistException {
        LeftSemiJoin lSjTest;
        XATNode node = new XATNodeImp();
        Expression expression = null;
        System.out.println("Testing LeftSemiJoin");
        lSjTest = new LeftSemiJoinImp(node, expression);
        node.addChild(new XATNodeImp());
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(
                new LeftOuterJoinImp(node.getChild(0), expression));
            node.getChild(1).setOperator(
                new LeftOuterJoinImp(node.getChild(1), expression));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        lSjTest.getExpression();
        lSjTest.getLeft();
        lSjTest.getNode();
        lSjTest.getOutput();
        lSjTest.getRight();
        lSjTest.setExpression(expression);
        Expression sColNames = lSjTest.getExpression();
        System.out.println(sColNames + "\n");
        XATTable left = lSjTest.getLeft();
        System.out.println("left: " + left);
        XATTable right = lSjTest.getRight();
        System.out.println("right: " + right);
        try {
            lSjTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    public static void main(String[] argv)
        throws XATNodeDoesNotExistException, XATTreeNodeHasNoParentException {
        /* need to create multiple XATTrees and XATTables
            need to change the tests so that they take in an XATTree and an XATTable
        
            add menu setup for main so that it is easily testable and viewable
        
        */
        //initializing demo default tables
        //      CreateTables();
        //initializing demo default trees
        CreateTrees();
        ExecuteVisitor execTest1 = new ExecuteVisitor();
        execTest1.visit(xtrDefault1);
        XATNode xnoOutRoot = xtrDefault1.getRoot();
        XATOperator xopOutOperator = xnoOutRoot.getOperator();
        XATTable xtbOutTable = xopOutOperator.getOutput();
        String sOutput = xtbOutTable.toString();
        System.out.print(sOutput);

        //opening of demo
        System.out.println("Welcome to the XOpt demo");
        System.out.println("Please select a run option");
        System.out.println("1) Merging");
        System.out.println("2) Cutting");
        System.out.println("3) Executing");

        // here we should allow them to select

        System.out.println("Which XATTable would you like to use");
        System.out.println("1) Default1");
        System.out.println("2) Default2");
        System.out.println("3) ");

        System.out.println("Which XATTree would you like to use");
        System.out.println("1) Default1");
        System.out.println("2) Default2");
        System.out.println("3) ");

        //close of demo
        try {
            XATTreeTest();
            AggregateTest();
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        catch (XATTreeNodeHasNoParentException f) {
            throw new XATTreeNodeHasNoParentException();
        }
        FORTest();
        GroupByTest();
        LeftOuterJoinTest();
        LeftSemiJoinTest();
        NavigateTest();
        OrderByTest();
        ProjectTest();
        RightOuterJoinTest();
        RightSemiJoinTest();
        SelectTest();
        SourceTest();
        TaggerTest();
        ThetaJoinTest();
        UnionTest();
        XATTableTest();
    }
    private static void NavigateTest() throws XATNodeDoesNotExistException {
        Navigate nAtTest;
        String path = "testpath";
        String newpath = "newpath";
        XATNode node = new XATNodeImp();
        System.out.println("Testing Navigate");
        nAtTest = new NavUnnestImp(node);
        nAtTest.getNode();
        nAtTest.getDestination();
        nAtTest.getEntryPoint();
        nAtTest.setDestination(null);
        nAtTest.setEntryPoint(null);
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new NavUnnestImp(node.getChild(0)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        System.out.println("getNode: " + nAtTest.getNode());
        System.out.println(
            "getNode: " + nAtTest.getEntryPoint() + nAtTest.getDestination());
        //System.out.println("setNode: " + nAtTest.setXPath(newpath));
        try {
            nAtTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    private static void OrderByTest() throws XATNodeDoesNotExistException {
        OrderBy oRrTest;
        String[] columnNames = { "col1", "col2", "col3" };
        XATNode node = new XATNodeImp();
        System.out.println("Testing OrderBy");
        oRrTest = new OrderByImp(node);
        oRrTest.getColumnNames();
        oRrTest.getNode();
        oRrTest.getOutput();
        oRrTest.setColumnNames(columnNames);
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new NavUnnestImp(node.getChild(0)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }

        String[] cols = oRrTest.getColumnNames();
        for (int i = 0; i < cols.length; i++) {
            System.out.println("getColumnNames: " + cols[i]);
        }
        System.out.println("getNode: " + oRrTest.getNode());
        System.out.println("getOutput: " + oRrTest.getOutput());
        /*
              String[] cols2 = oRrTest.setColumnNames(columnNames);
              for (int i = 0; i < cols2.length; i++)
              {
                  System.out.println("setColumnNames: " + cols2[i]);
              }
        
        */
        try {
            oRrTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }
    }
    private static void printNode(String prefix, XATNode node) {
        XATNode[] children = node.getChildren();
        int i;
        System.out.println(prefix + node.hashCode());
        for (i = 0; i < Array.getLength(children); i++) {
            printNode(prefix + "\t", children[i]);
        }
    }
    private static void ProjectTest() throws XATNodeDoesNotExistException {
        Project pRjTest;
        String[] columnNames = { "proj1", "proj2", "proj3" };
        XATNode node = new XATNodeImp();
        System.out.println("Testing Project");
        pRjTest = new ProjectImp(node);
        pRjTest.getColumnNames();
        pRjTest.getNode();
        pRjTest.getOutput();
        pRjTest.setColumnNames(columnNames);
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new NavUnnestImp(node.getChild(0)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        String[] cols = pRjTest.getColumnNames();
        for (int i = 0; i < cols.length; i++) {
            System.out.println("getColumnNames: " + cols[i]);
        }
        System.out.println(pRjTest.getNode());
        System.out.println(pRjTest.getOutput());
        pRjTest.setColumnNames(columnNames);
        try {
            pRjTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    private static void RightOuterJoinTest() throws XATNodeDoesNotExistException {
        RightOuterJoin rOjTest;
        XATNode node = new XATNodeImp();
        Expression expression = null;
        System.out.println("Testing RightOuterJoin");
        rOjTest = new RightOuterJoinImp(node, expression);
        node.addChild(new XATNodeImp());
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(
                new RightOuterJoinImp(node.getChild(0), expression));
            node.getChild(1).setOperator(
                new RightOuterJoinImp(node.getChild(1), expression));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        rOjTest.getExpression();
        rOjTest.getLeft();
        rOjTest.getNode();
        rOjTest.getOutput();
        rOjTest.getRight();
        rOjTest.setExpression(expression);
        try {
            rOjTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }
    }
    private static void RightSemiJoinTest() throws XATNodeDoesNotExistException {
        RightSemiJoin rSjTest;
        XATNode node = new XATNodeImp();
        Expression expression = null;
        System.out.println("Testing RightSemiJoin");
        rSjTest = new RightSemiJoinImp(node, expression);
        node.addChild(new XATNodeImp());
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(
                new RightSemiJoinImp(node.getChild(0), expression));
            node.getChild(1).setOperator(
                new RightSemiJoinImp(node.getChild(1), expression));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        rSjTest.getExpression();
        rSjTest.getLeft();
        rSjTest.getNode();
        rSjTest.getOutput();
        rSjTest.getRight();
        rSjTest.setExpression(expression);
        try {
            rSjTest.getPrimarySource();
        }

        catch (Exception e) {
            System.out.println("Caught exception: " + e.getClass() + "-" + e.getMessage());
        }
    }
    private static void SelectTest() throws XATNodeDoesNotExistException {
        Select sLcTest;
        XATNode node = new XATNodeImp();
        BinCOMPExpression expression = null;
        System.out.println("Testing Select");
        sLcTest = new SelectImp(node, expression);
        sLcTest.getExpression();
        sLcTest.getNode();
        sLcTest.getOutput();
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new SelectImp(node.getChild(0), expression));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }

        sLcTest.getPrimarySource();
        sLcTest.setExpression(expression);
    }
    private static void SourceTest() throws XATNodeDoesNotExistException {
        Source sRcTest;
        XATNode node = new XATNodeImp();
        String string = "test";
        int i = 1;
        System.out.println("Testing Source");
        sRcTest = new SourceImp(node);
        sRcTest = new SourceImp(node, string, i);
        sRcTest.getName();
        sRcTest.getNode();
        sRcTest.getOutput();
        try {
            sRcTest.getPrimarySource();
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        sRcTest.getType();
        sRcTest.setName(string);
        sRcTest.setType(i);

    }
    private static void TaggerTest() throws XATNodeDoesNotExistException {
        Tagger tGrTest;
        XATNode node = new XATNodeImp();
        System.out.println("Testing Tagger");
        tGrTest = new TaggerImp(node, new DOMPatternImp());
        tGrTest.setPattern(new DOMPatternImp());
        tGrTest.getPattern();
        tGrTest.getOutputDataSource();
        tGrTest.getNode();
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new TaggerImp(node.getChild(0), new DOMPatternImp()));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        tGrTest.getPrimaryDataSource();
    }
    private static void ThetaJoinTest() throws XATNodeDoesNotExistException {
        ThetaJoin tTaTest;
        XATNode node = new XATNodeImp();
        Expression expression = null;
        System.out.println("Testing ThetaJoin");
        tTaTest = new ThetaJoinImp(node, expression);
        tTaTest.getExpression();
        node.addChild(new XATNodeImp());
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new ThetaJoinImp(node.getChild(0), expression));
            node.getChild(1).setOperator(new ThetaJoinImp(node.getChild(1), expression));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        tTaTest.getLeft();
        tTaTest.getNode();
        tTaTest.getOutput();
        tTaTest.getPrimarySource();
        tTaTest.getRight();
        tTaTest.setExpression(expression);

    }
    private static void UnionTest() throws XATNodeDoesNotExistException {
        Union uNnTest;
        int pos = 1;
        XATNode node = new XATNodeImp();
        System.out.println("Testing Union");
        uNnTest = new UnionImp(node);
        uNnTest.getNode();
        uNnTest.getOutputDataSource();
        node.addChild(new XATNodeImp());
        node.addChild(new XATNodeImp());
        try {
            node.getChild(0).setOperator(new UnionImp(node.getChild(0)));
            node.getChild(1).setOperator(new UnionImp(node.getChild(1)));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        uNnTest.getPrimaryDataSource();
        uNnTest.getDataSourceAt(pos);
        uNnTest.getDataSources();

    }
    private static void XATTableTest() {
        XATTable xTbTest1;
        XATTable xTbTest2;
        int pos = 1;
        String name = "Triple G's";
        Vector column = new Vector();
        Vector column2 = new Vector();
        Vector column3 = new Vector();
        column3.addElement("eskimo");
        column3.addElement("Hobbes");
        Object obj = new Object();
        System.out.println("Testing XATTable");
        xTbTest1 = new XATTableImp();
        xTbTest2 = new XATTableImp(xTbTest1);

        try {
            xTbTest1.appendColumn(name, (Vector) column.clone());
            xTbTest1.appendColumn(name, (Vector) column.clone());
        }

        catch (Exception e) {
            System.out.println(
                "Caught appendColumn exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println("appended 2 columns of " + column);
        System.out.println(xTbTest1.toString());
        try {
            column.add(name);
            xTbTest1.appendTuple((Vector) column.clone());
            xTbTest1.appendTuple((Vector) column.clone());

        }

        catch (Exception e) {
            System.out.println(
                "Caught appendTuple exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println("append tuple " + column);
        System.out.println(xTbTest1.toString());
        String sColName = "Misc";
        column2.add("home");
        column2.add("evil");
        column2.add("Gilligan");
        try {
            xTbTest1.appendColumn(sColName, column2);
        }

        catch (Exception e) {
            System.out.println(
                "Caught appendColumn exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println("append column " + column2);
        xTbTest1.print();
        try {
            System.out.println("column one: " + xTbTest1.getColumn(1));
        }
        catch (Exception e) {
            System.out.println(
                "Caught getColumn exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println(xTbTest1.toString());

        try {
            System.out.println("Cell (1,1): " + xTbTest1.getValue(1, 1));
            System.out.println("Cell (2,2): " + xTbTest1.getValue(2, 2));
        }
        catch (Exception e) {
            System.out.println(
                "Caught getValue exception: " + e.getClass() + "-" + e.getMessage());
        }

        System.out.println(xTbTest1.toString());

        try {
            xTbTest1.setColumnName("T3", 1);
            xTbTest1.setColumnName("home", 2);
            xTbTest1.setColumnName("words", 3);
        }
        catch (Exception e) {
            System.out.println(
                "Caught SetColumnName exception: " + e.getClass() + "-" + e.getMessage());
        }

        try {
            xTbTest1.setName("goethe");
        }
        catch (Exception e) {
            System.out.println(
                "Caught insertTuple exception: " + e.getClass() + "-" + e.getMessage());
        }
        try {
            System.out.println("tablename: " + xTbTest1.getName());
        }
        catch (Exception e) {
            System.out.println(
                "Caught getName exception: " + e.getClass() + "-" + e.getMessage());
        }
        try {
            System.out.println("column2: " + xTbTest1.getColumnName(2));
        }
        catch (Exception e) {
            System.out.println(
                "Caught getColumn exception: " + e.getClass() + "-" + e.getMessage());
        }

        try {
            System.out.println(
                "The Column number for Triple G's is: " + xTbTest1.getColumnPosition("home"));
        }
        catch (Exception e) {
            System.out.println(
                "Caught getColumnPosition exception: " + e.getClass() + "-" + e.getMessage());
        }

        System.out.println(xTbTest1.toString());

        try {
            System.out.println("num cols: " + xTbTest1.getNumberOfColumns());
        }
        catch (Exception e) {
            System.out.println(
                "Caught getNumberOfColumns exception: " + e.getClass() + "-" + e.getMessage());
        }
        try {
            System.out.println("num tuples: " + xTbTest1.getNumberOfTuples());
        }
        catch (Exception e) {
            System.out.println(
                "Caught getNumberOfTuples exception: " + e.getClass() + "-" + e.getMessage());
        }

        try {
            System.out.println("tuple 3: " + xTbTest1.getTuple(3));
        }
        catch (Exception e) {
            System.out.println(
                "Caught getTuple exception: " + e.getClass() + "-" + e.getMessage());
        }

        Vector colstuff = new Vector();
        sColName = "Crazy";
        colstuff.addElement("eskimo");
        colstuff.addElement("Hobbes");
        colstuff.addElement("Hume");

        try {
            System.out.println(
                "new col position: " + xTbTest1.insertColumn(2, sColName, colstuff));
        }
        catch (Exception e) {
            System.out.println(
                "Caught insertColumn exception: " + e.getClass() + "-" + e.getMessage());
        }

        System.out.println(xTbTest1.toString());
        try {
            System.out.println(
                "insert tuple: " + colstuff + " : " + xTbTest1.insertTuple(2, colstuff));
        }
        catch (Exception e) {
            System.out.println(
                "Caught insertTuple exception: " + e.getClass() + "-" + e.getMessage());
        }
        xTbTest1.print();

        try {
            System.out.println("move col  3 to 1");
            xTbTest1.moveColumn(3, 1);
        }
        catch (Exception e) {
            System.out.println(
                "Caught insertTuple exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println(xTbTest1.toString());

        try {
            System.out.println("move tup  1 to 3");
            xTbTest1.moveTuple(1, 3);
        }
        catch (Exception e) {
            System.out.println(
                "Caught moveTuple exception: " + e.getClass() + "-" + e.getMessage());
        }
        xTbTest1.print();

        try {
            xTbTest1.updateValue(1, 1, new XATDataValueImp("nitze"));
            System.out.println("Cell (1,1): " + xTbTest1.getValue(1, 1));
        }
        catch (Exception e) {
            System.out.println(
                "Caught updateCell exception: " + e.getClass() + "-" + e.getMessage());
        }
        try {
            Vector updateVect = new Vector();
            updateVect.addElement("Socrates");
            updateVect.addElement("Plato");
            updateVect.addElement("Aristotle");
            updateVect.addElement("DesCartes");

            System.out.println(
                "this column is replaced: " + xTbTest1.updateColumn(1, updateVect));
            System.out.println("this column replaces: " + updateVect);
        }
        catch (Exception e) {
            System.out.println(
                "Caught updateColumn exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println(xTbTest1.toString());

        try {
            Vector updatemonkeycun = new Vector();
            updatemonkeycun.addElement("GOA");
            updatemonkeycun.addElement("TraNCE");
            updatemonkeycun.addElement("Hamdbag");
            updatemonkeycun.addElement("night stick");
            System.out.println(
                "this column is replaced: " + xTbTest1.updateTuple(4, updatemonkeycun));
        }
        catch (Exception e) {
            System.out.println(
                "Caught updateTuple exception: " + e.getClass() + "-" + e.getMessage());
        }
        System.out.println(xTbTest1.toString());

        try {
            System.out.println("this tuple was deleted: " + xTbTest1.deleteTuple(1));
        }

        catch (Exception e) {
            System.out.println(
                "Caught deleteTuple exception: " + e.getClass() + "-" + e.getMessage());
        }

        xTbTest1.print();

        try {
            System.out.println("this column was deleted: " + xTbTest1.deleteColumn(3));
        }

        catch (Exception e) {
            System.out.println(
                "Caught deleteColumn exception: " + e.getClass() + "-" + e.getMessage());
        }
        xTbTest1.print();

        try {
            XATTable clone = new XATTableImp();
            clone = (XATTable)xTbTest1.deepCloneDataSource();
            Vector merchant = new Vector();
            merchant.addElement("Shylock");
            merchant.addElement("Portia");
            merchant.addElement("Jessica");
            clone.updateColumn(1, merchant);
            System.out.println("Original table: ");
            xTbTest1.print();
            System.out.println("Cloned (modified) table: ");
            clone.print();
            System.out.println("this column was deleted: " + clone.deleteColumn(2));
            System.out.println("Original table: ");
            xTbTest1.print();
            System.out.println("Cloned (modified) table: ");
            clone.print();

        }

        catch (Exception e) {
            System.out.println(
                "Caught deepClone exception: " + e.getClass() + "-" + e.getMessage());
        }

    }
    private static void XATTreeTest()
        throws XATNodeDoesNotExistException, XATTreeNodeHasNoParentException {
        XATTree tree;
        XATNode[] nodes;
        int i;

        System.out.println("Testing XATTable");
        System.out.println("Creating tree");
        tree = new XATTreeImp();
        System.out.println("Setting root");
        tree.setRoot(new XATNodeImp());
        printNode("", tree.getRoot());
        System.out.println("Adding first child");
        tree.addNode(new XATNodeImp(), tree.getRoot());
        printNode("", tree.getRoot());
        /* should be a root with 1 child */
        System.out.println("Adding second child");
        tree.addNode(new XATNodeImp(), tree.getRoot());
        printNode("", tree.getRoot());
        /* should be a root with 2 children */
        System.out.println("Moving child 1 to child 2");
        try {
            tree.moveNode(tree.getRoot().getChild(0), tree.getRoot().getChild(1));
        }
        catch (XATNodeDoesNotExistException e) {
            throw new XATNodeDoesNotExistException();
        }
        printNode("", tree.getRoot());
        /* should be a root with 1 child with 1 child */
        System.out.println("deleting child 1");
        try {
            tree.deleteNode(tree.getRoot().getChild(0));
        }
        catch (XATTreeNodeHasNoParentException f) {
            throw new XATTreeNodeHasNoParentException();
        }
        /* should be a root with 1 child */
        printNode("", tree.getRoot());
        System.out.println("Adding child 3");
        tree.addNode(new XATNodeImp(), tree.getRoot());
        printNode("", tree.getRoot());
        System.out.println("Adding child 1 of child 2");
        tree.addNode(new XATNodeImp(), tree.getRoot().getChild(0));
        printNode("", tree.getRoot());
        System.out.println("Moving child 3 to child 2");
        tree.moveNode(tree.getRoot().getChild(1), tree.getRoot().getChild(0), 0);
        printNode("", tree.getRoot());
        nodes = tree.getLeaves();
        System.out.println("Leaves:");
        for (i = 0; i < Array.getLength(nodes); i++) {
            System.out.println("\t" + nodes[i].hashCode());
        }

    }
}
