/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package xacute.common;

public interface Node
{
    /** to test for physical equality of two nodes */
    public boolean equals(Node n);

    /**
     * to compare two nodes two nodes are not always comparable (from different documents, attribute nodes, etc.)
     * the result can be: SAME BEFORE AFTER NOT_COMPARABLE
     */
    public int compareTo(Node n);

    /** to get the root of the document the node belongs to */
    public Node getDocumentRoot() throws QuiltException;

    /** to evaluate axis::note-test on a given node */
    public NodeList findAxis(int axis, NodeTest nt);

    /** to evaluate axis::note-test[predicate1]..[predicaten] on a given node */
    public NodeList findAxis(int axis, NodeTest nt, Predicate[] predicates) throws QuiltException;

    // to dereference an idref and get back the nodes pointed to
    public NodeList deref(String idref, Hint[] hints);

    /** functions that deal with the name of the node */
    public ExpandedName getExpandedName() throws QuiltException;

    public ExpandedName expand(String qName) throws QuiltException;

    public String getQName();

    /** functions that deal with the type of the node */
    public boolean isOfPrincipalType(int axis);

    public boolean isOfType(int type);

    /** output methods: 'shallow' means that children are ignored */
    public String stringValue();

    public String stringValue_shallow();

    public void toSAX(org.xml.sax.DocumentHandler handler) throws org.xml.sax.SAXException;

    public void toSAX_shallow(org.xml.sax.DocumentHandler handler) throws org.xml.sax.SAXException;
}
