/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.common;

import org.xml.sax.AttributeList;

import java.util.Vector;

public interface NodeFactory
{

    public String getFactoryName();

    public void clearDocumentCache();

    /**
     * transforming a string that represents a piece of XML into a XML
     * document, and returning its root node
     **/
    public Node getInLineXMLRoot(String xml)
    	throws QuiltException;
    
    /**
     * getting the root node of a given document
     **/
    public Node getDocumentRoot(String uri)
	throws Exception;

    /**
     * Node constructors for nodes that do not belong to a document
     **/
    public Node makeTextNode(String text);
    public Node makeConstructedNode(String name, AttributeList atts, Vector children);
    public Node makeShallowNode(Node n);

    /**
     * NodeList constructors
     **/
    public NodeList makeEmptyNodeList();
    public NodeList makeNodeList(Node n);
    public NodeList makeNodeList(NodeList n);
    
}




