/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.ksp;

import java.io.FileReader;

import org.w3c.dom.*;
import org.xml.sax.AttributeList;
import org.xml.sax.DocumentHandler;
import org.xml.sax.Locator;
import org.xml.sax.InputSource;

import org.apache.xerces.dom.DocumentImpl;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import com.microstar.xml.SAXDriver; 

/**
 *
 * This class computes consumes SAX events to produce a DOM tree.
 * input: document, top-level element
 * output: top-level element, with some new nodes underneath
 *
 * document is needed to grab DOM methods
 * element is needed to insert the nodes
 */

public class SAX2DOM 
    implements DocumentHandler
{
    Document doc;
    // Element top;
    Node currentNode;
    
    public SAX2DOM(Document doc, Element top)
    {
	this.doc = doc;
	// this.top = top;
	this.currentNode = top;
    }

    public void characters(char[] ch, int start, int length)
    {
	// we create a TEXT node and add it to the current node
	Node n = doc.createTextNode(new String(ch, start, length));
	currentNode.appendChild(n);
	return;
    }
    
    public void endDocument()
    {
	return;
    }
    
    public  void endElement(String name)
    {	
	currentNode = currentNode.getParentNode();
	return;
    }
    
    public void ignorableWhitespace(char[] ch, int start, int length)
    {
	return;
    }

    public void processingInstruction(String target, String data)
    {
	// we create a PI node and add it to the current node
	Node n = doc.createProcessingInstruction(target, data);
	currentNode.appendChild(n);
	return;
    }

    public void setDocumentLocator(Locator locator)
    {                                   
	return;
    }

    public void startDocument()
    {
	return;
    }

    public void startElement(String name, AttributeList atts)
    {
	Element el = doc.createElement(name);
	for (int i=0; i < atts.getLength(); i++)
	    {
		String attName = atts.getName(i);
		String attValue = atts.getValue(i);
		el.setAttribute(attName, attValue);
	    }
	currentNode.appendChild(el);
	currentNode = el;
	return;
    }


    public static void main(String args[])
	throws Exception
    {
	SAXDriver parser = new SAXDriver();
	DocumentImpl doc = new DocumentImpl();
	System.out.println( doc.toString());
	Element el = doc.createElement("ROOT");
	doc.appendChild(el);
	SAX2DOM handler = new SAX2DOM(doc, el);
	parser.setDocumentHandler(handler);
	parser.parse( new InputSource( new FileReader(args[0]) ) );
	System.out.println( doc.toString());
	XMLSerializer ser = new XMLSerializer(new java.io.PrintWriter( System.out), new OutputFormat());
	ser.serialize( el);
    }

/**
 * Insert the method's description here.
 * Creation date: (3/19/02 9:20:44 PM)
 */
public SAX2DOM() {
	DocumentImpl doc = new DocumentImpl();
	Element el = doc.createElement("ROOT");
	doc.appendChild(el);
	this.doc = doc;
	this.currentNode = el;
	System.err.println(el.toString());
}

/**
 * Insert the method's description here.
 * Creation date: (3/19/02 9:24:19 PM)
 * @return org.w3c.dom.Document
 */
public Document getDocument() {
	return this.doc;
}

/**
 * Insert the method's description here.
 * Creation date: (3/19/02 9:36:22 PM)
 * @return org.w3c.dom.Node
 */
public Node getNode() {
	return this.currentNode;
}
}                                    





