/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import java.util.Vector;

/**
 * This class represents any binary operator.
 * To make evaluation easier and faster, 
 * the operator applies to a list of arguments.
 * The first argument is stored separately.
 * The other arguments are stored in an array.
 */
public abstract class BinOp
    extends Operator
{
    
    /**
     * This field is the left-hand side 
     * of the binary operation.
     */
    protected QuiltExpression e;

    /**
     * The rest of the arguments.
     */
    protected QuiltExpression[] operands = new QuiltExpression[0];

    BinOp() {}
    
    BinOp( QuiltExpression e, Vector v)
    {
	this.e = e;
	this.operands = new QuiltExpression[v.size()];
	v.toArray(this.operands);
    }
    
    /**
     * Returns the expression
     * at a given index in the 
     * right-hand side of the
     * binary operation.
     */
    protected QuiltExpression getExpr(int i) { return this.operands[i]; }

/**
 * Insert the method's description here.
 * Creation date: (2/19/2002 11:03:01 PM)
 * @return xacute.quilt.QuiltExpression
 */
public QuiltExpression getE() {
	return e;
}

/**
 * Insert the method's description here.
 * Creation date: (2/19/2002 11:05:04 PM)
 * @return xacute.quilt.QuiltExpression[]
 */
public QuiltExpression[] getOperands() {
	return operands;
}
}


