/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.QuiltException;
import java.util.Vector;

/**
 * This class represents an arithmetic operation in <i>Quilt</i>.
 */
public class BinOpArith
    extends BinOp
{
   
    /**
     * This field is a constant representing the arithmetic operator.
     */
    private int op;

    BinOpArith(int op, QuiltExpression e, Vector v)
    {
	super(e, v);
	this.op = op;
    }
    
    Value eval(EvalContext con)
	throws QuiltException
    {
	ValueNum res = new ValueNum(this.e.eval(con).toNum(con));
	for (int i = 0; i < this.operands.length; i++)
	    {
		ValueNum val = new ValueNum(this.operands[i].eval(con).toNum(con));
		res.applyOperation(this.op, val, con);
	    }
	return res;
    }
    
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	buf.append(this.e.pprint());
	for (int i=0; i < this.operands.length; i++)
	    if ((char) this.op == '/')
		buf.append( " div " + this.operands[i].pprint());
	    else
		buf.append( " " + (char) this.op + " " + this.operands[i].pprint());
	return buf.toString();
    }

/**
 * Insert the method's description here.
 * Creation date: (8/28/02 3:47:49 PM)
 * @return int
 */
public int getOp() {
	return this.op;
}
}
