/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.QuiltException;

import java.util.Vector;

/**
 * This class represents any binary operator
 * that is evaluated into a boolean.
 */
public abstract class BinOpBool
    extends BinOp
{

    BinOpBool() {}
    
    BinOpBool(QuiltExpression e, Vector v)
    {
	super(e, v);
    }
    
    /**
     * Evaluates a <code>QuiltExpression</code> 
     * into a <code>ValueBool</code>.
     * @param con the evaluation context
     * @exception a <code>QuiltWrongTypeException</code> 
     *            is thrown in case <code>e</code>
     *            can not be evaluated into a 
     *            boolean value
     */
    protected static boolean computeBoolean(QuiltExpression e, EvalContext con)
	throws QuiltException
    {
	Value val = e.eval(con);
        return val.toBool(con);
    }

    protected Value evalAux(EvalContext con, int operator)
	throws QuiltException
    {
	boolean b, c;
	if (this.operands.length == 0)
	    return this.e.eval(con);
	else
	    {
		b = this.computeBoolean(this.e, con);
		for (int k=0; k < this.operands.length; k++)
		    {
			c = this.computeBoolean(this.operands[k], con);
			switch (operator)
			    {
			    case 0:
				b = b || c;
				break;
			    case 1:
				b = b && c;
			    }
		    }
		return new ValueBool(b);
	    }
    } 
    
}





