/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

import java.util.Vector;

public abstract class BinOpNodeLists
    extends BinOp
{

    protected int op;

    BinOpNodeLists(QuiltExpression e, Vector v, int op)
    {
	super(e, v);
	this.op = op;
    }

    Value eval(EvalContext con)
	throws QuiltException
    {
	return this.evalBinOpNodeLists(con, this.op);
    }

    /**
     * This auxiliary method evaluates a 
     * <code>BinOpNodeLists</code> into a basic
     * Quilt value.
     * @param env the environment of evaluation
     * @param op  a constant representing the operation
     */
    protected Value evalBinOpNodeLists(EvalContext con, int op) 
	throws QuiltException
    {
	Value val = this.e.eval(con);
	NodeList nl = val.toNodeList(con);
	for (int i=0; i<this.operands.length; i++)
	    {
		val = this.getExpr(i).eval(con);
		nl = this.performOperation(op, nl, val.toNodeList(con));
	    }
	return new ValueNodeList(nl);
    }

    
    /**
     * This private method calls
     * an operation function of
     * the class 
     * <code>xacute.xmltree.NodeList</code>.
     * @param op  a constant representing the 
     *            operation to perform
     * @param nl1 the left-hand side of the
     *            operation, which contains
     *            the result of the operation
     *            after the method returns
     * @param nl1 the right-hand side of the
     *            operation
     */
    private NodeList performOperation(int op, NodeList nl1, NodeList nl2)
	throws QuiltException
    {
	switch (op) {

	case Constants.KEEP_BEFORE:
	    return nl1.BEFORE(nl2);

	case Constants.KEEP_AFTER:
	    return nl1.AFTER(nl2);

	case Constants.KEEP_INTER:
	    return nl1.INTER(nl2);

	case Constants.KEEP_UNION:
	    // System.err.println("union");
	    return nl1.UNION(nl2);
	default:
	    throw new QuiltException("Invalid operator " + op + " in BinOpNodeLists.class");
	}
	
    }

    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	buf.append(e.pprint());
	if ( (this.operands.length > 0) && (op >= 0) && (op <= Constants.NODELISTOPS.length)) 
	    for (int i=0; i < this.operands.length; i++)
		buf.append(" " + Constants.NODELISTOPS[op] + " " + this.getExpr(i).pprint());
	return buf.toString();
    }
}





