package xacute.quilt;

import xacute.common.QuiltException;

import java.util.*;

/**
 * This class represents a <b>FOR</b> or a <b>LET</b> clause
 * in a FLWR expression in <i>Quilt</i>.
 */
public abstract class Binding
{

    /**
     * used for pretty-printing to avoid repeating LET/FOR
     */
    protected boolean isFirst;

    /**
     * This field is the name
     * of the variable to bind.
     */
    protected String name;

    /**
     * This field indicates if
     * the resulting node-list
     * must be freed from its
     * duplicates.
     */
    protected boolean distinct;

    /**
     * This field is the expression
     * whose evaluation result must
     * be bound to the variable.
     */
    protected QuiltExpression e;

    Binding(boolean isFirst, String name, boolean distinct, QuiltExpression e)
    {
	this.isFirst = isFirst;
	this.name = name;
	this.distinct = distinct;
	this.e = e;
    }
    /**
     * Returns the <code>Value</code> resulting of the
     * evaluation of a <code>Binding</code> in a
     * given <code>EvalContext</code>.
     */
    abstract Value eval(EvalContext con)
	throws QuiltException;
    public boolean getDistinct(){
	    return distinct;
    }
    /**
     * Accessor to the name of
     * the variable to bind.
     */
    public String getName() { return this.name; }
    public QuiltExpression getQE() {
      return e;
    }
    /**
     * Pretty-prints a
     * <code>Binding</code> object.
     */
    public abstract String pprint();
/**
 * Insert the method's description here.
 * Creation date: (2001-12-18 18:24:11)
 * @return xacute.quilt.QuiltQuery
 */
public QuiltExpression returnQuery() {
	return e;
}
    public String toString()
    {
	return "binding var $" + name;
    }
}
