/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package xacute.quilt;

import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;
import java.util.TreeMap;
import java.util.Vector;
import org.xml.sax.*;

/**
 * This class implements a tree structure representing the evaluation of the <b>FOR</b> and <b>LET</b> clauses
 * of a FLWR Expression in Quilt.
 */
public class BindingTree
{
    /** The value of the current variable to bind. */
    private Value value;

    /** Constructor for a <code>BindingTree</code> that can be attributed children. */
    BindingTree(Value v)
    {
        this.value = v;
    }

    BindingTree()
    {
    }

    static void buildTreeAndPaths(Binding[] bindings, EvalContext con, QuiltExpression q, SortClause[] sortClause,
        Vector tm) throws QuiltException
        {
            BindingTree t = new BindingTree();
            t.build(bindings, con, q, sortClause, tm, null);
            return;
    }

    private void build(Binding[] bindings, EvalContext con, QuiltExpression q, SortClause[] sortClause, Vector tm,
        BindingTreePath path) throws QuiltException
        {
            this.build(bindings, 0, con, q, sortClause, tm, path);
    }

    private void build(Binding[] bindings, int k, EvalContext con, QuiltExpression q, SortClause[] sortClause, Vector tm,
        BindingTreePath path) throws QuiltException
        {
            if (k < bindings.length)
            {
                Binding b = bindings[k];
                String name = b.getName();
                if (b instanceof ForBinding)
                {
                    // FOR binding
                    NodeList nl = b.eval(con).toNodeList(con);
                    int s = nl.size();
                    for (int c = 0; c < s; c++)
                    {
                        Node n = nl.get(c);
                        con.getEnv().setValue(name, new ValueNode(n));
                        BindingTree t = new BindingTree(new ValueNode(n));
                        t.build(bindings, k + 1, con, q, sortClause, tm, new BindingTreePath(new ValueNode(n), path));
                        // this.next.addElement(t);
                    }
                    if (s > 0)
                        con.getEnv().rmBinding(name);
                }
                else
                {
                    // LET binding
                    Value v = b.eval(con);
                    if (v == null)
                        v = new ValueNodeList(con.getNodeFactory().makeEmptyNodeList());
                    if (v != null)
                    {
                        con.getEnv().setValue(name, v);
                        BindingTree t = new BindingTree(v);
                        t.build(bindings, k + 1, con, q, sortClause, tm, new BindingTreePath(v, path));
                        // modified by Xin
                        //con.getEnv().rmBinding(name);
                        // end of modification
                    }
                }
            }
            else if ((q == null) || (q.isTrue(con))) // WHERE clause
                    tm.addElement(path);
    }
}
