/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.QuiltException;

/**
 * This class represents a path in a <code>BindingTree</code> object.
 * The more nested <code>Value</code> corresponds to the first variable to bind * so watch the order.
 */
public class BindingTreePath
{
    
    /**
     * This field is the tail 
     * value of the path. 
     */
    private Value v;

    /**
     * This field is a pointer to a
     * path constituting the head
     * of <code>this</code>.
     */
    private BindingTreePath path;

    /**
     * Returns a <code>BindingTreePath</code>
     * of length zero.
     */
    public BindingTreePath()
    { 
	this.v = null;
	this.path = null;
    }

    /**
     * Returns a new <code>BindingTreePath</code>.
     * @param v the tail of the path to create
     * @param path the head of the path to create
     */
    public BindingTreePath(Value v, BindingTreePath path)
    {
	this.v = v;
	this.path = path;
    }

    public Value getValue() { return this.v; }
    
    public BindingTreePath getNext() { return this.path; }
    
    public String toString() {
	return this.v.toString() + ((this.path != null) ? (" <-- " + this.path) : "");
    }
}



