/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeFactory;
import xacute.common.NodeList;
import xacute.common.QuiltException;
import org.xml.sax.helpers.AttributeListImpl;
import java.util.Vector;

public class ElementConstruct extends QuiltExpression
{
    /** This field is the name of the element to construct. */
    private TagNameable name;

    /** Let's support the Tag name expression. */
    private QuiltExpression ename;

    /** Let's support the Attribute List Expression. */
    private QuiltExpression eatts;
    private AttributeExpression[] atts;

    /** This array contains <code>QuiltExpression</code> objects to insert inside the constructed element. */
    private QuiltExpression[] content;

    public ElementConstruct(TagNameable name, Vector atts, Vector content)
    {
        this.name = name;
        this.ename = null;
        this.eatts = null;
        this.atts = new AttributeExpression[atts.size()];
        atts.toArray(this.atts);
        this.content = new QuiltExpression[content.size()];
        content.toArray(this.content);
    }

    public ElementConstruct(QuiltExpression exp, Vector atts, Vector content)
    {
        this.ename = exp;
        this.name = new QName("NOT EVALUATED YET!");
        this.eatts = null;
        this.atts = new AttributeExpression[atts.size()];
        atts.toArray(this.atts);
        this.content = new QuiltExpression[content.size()];
        content.toArray(this.content);
    }

    /** Used to manually set the attribute variable. */
    public void setEatts(QuiltExpression e)
    {
        this.eatts = e;
    }

    Value eval(EvalContext con) throws QuiltException
    {
        NodeFactory nf = con.getNodeFactory();
        // if my tag is an expression then, evaluate it and give me the tag name back!
        if (ename != null)
        {
            Value v = this.ename.eval(con);
            this.name = new QName(v.toString(con));
        }
        // processing attributes
        AttributeListImpl attList = new AttributeListImpl();
        // if my attributes is an expression then, evaluate it and give me the attribute list back! by Xinz
        if  (eatts != null)
        {
            Value v = this.eatts.eval(con);
            NodeList attNodeList = v.toNodeList(con);
            for (int j = 0; j < attNodeList.size(); j++)
                attList.addAttribute(attNodeList.get(j).getQName(), "CDATA", attNodeList.get(j).stringValue());
            
        }
        else{
	        for (int i = 0; i < this.atts.length; ++i)
	        {
	            AttributeExpression ae = this.atts[i];
	
	            /** the attribute is actually a list of attributes as in <A $tag/@* /> */
	            if (ae.getName() == null)
	            {
	                NodeList attNodeList = ae.eval(con).toNodeList(con);
	                for (int j = 0; j < attNodeList.size(); j++)
	                    attList.addAttribute(attNodeList.get(j).getQName(), "CDATA", attNodeList.get(j).stringValue());
	            }
	            else
	                attList.addAttribute(ae.getName(), "CDATA", ae.eval(con).toString(con));
	        }
        }
        // processing children
        Vector children = new Vector();
        for (int k = 0; k < this.content.length; k++)
        {
            Value v = this.content[k].eval(con);
            if (v instanceof ValueNodeList)
            {
                NodeList nl = v.toNodeList(con);
                for (int j = 0; j < nl.size(); j++)
                    children.addElement(nl.get(j));
            }
            else
                children.addElement(v.toNode(con));
        }
        return new ValueNode(nf.makeConstructedNode(this.name.getTagName(con), attList, children));
    }

    public String pprint()
    {
        StringBuffer buf = new StringBuffer();
        buf.append("<" + this.name.getTagName());
        for (int i = 0; i < this.atts.length; ++i)
            buf.append(" " + this.atts[i].pprint());
        if (this.content.length > 0)
        {
            buf.append(">\n");
            for (int i = 0; i < this.content.length; ++i)
                buf.append(this.content[i].pprint() + "\n");
            buf.append("</" + this.name.getTagName() + ">");
        }
        else
            buf.append("/>\n");
        return buf.toString();
    }
    public QuiltExpression[] getContent(){
	    return content;
    }

    public TagNameable getName(){
	    	return name;
    }

/**
 * Insert the method's description here.
 * Creation date: (8/9/02 4:46:29 PM)
 * @return xacute.quilt.AttributeExpression[]
 */
public AttributeExpression[] getAtts() {
	return this.atts;
}
}
