/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.NodeFactory;
import xacute.common.Semantics;


import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

public class Environment
    implements Cloneable
{

    /************ Attributes *************/

    /**
     * A <code>Hashtable</code> object whose keys are strings
     * identifying variables.
     */
    private Hashtable table;

    /**
     * A <code>Hashtable</code> object whose keys are strings
     * identifying function definitions.
     */
    private Hashtable funTable;

    /**
     * Constructs a new <code>Environment</code> 
     * object.
     */
    Environment()
    { 
	this.table = new Hashtable(); 
	this.funTable = new Hashtable();
    }

    public Object clone()
    {
	Environment env = new Environment();
	env.table = new Hashtable( this.table );
	env.funTable = new Hashtable( this.funTable );
	return env;
    }
    
    /**
     *
     * Methods used to manage the evaluation environment per se.
     *
     **/
    
    public boolean isBoundVar(String name)
    {
	return this.table.containsKey(name);
    }
    
    public boolean isBoundFun(String funName)
    {
	return this.table.containsKey(funName);
    }

    public void setValue(String name, Value value)
    {
	this.table.put(name, value);
    }
    
    public void setFunValue(String funName, FunDef funDef)
    {
	this.funTable.put(funName, funDef);
    } 
    
    public Value getValue(String name)
    {
	return (Value) this.table.get(name);
    }

    public FunDef getFunValue(String funName)
    {
	return (FunDef) this.funTable.get(funName);
    }

    public void rmBinding(String name)
    {
        this.table.remove(name);
    }
    
    public void setBindings(Binding[] bindings, BindingTreePath path)
    {
	this.setBindingsAux(bindings, bindings.length - 1, path);
    }
    
    public void rmBindings(Binding[] bindings)
    {
        for (int k = 0; k < bindings.length; k++)
            this.rmBinding(bindings[k].getName());
    }

    public String toString()
    {
	StringBuffer buf = new StringBuffer();
	buf.append("[");
        Enumeration names = this.table.keys();
	Enumeration values =  this.table.elements();
	while(names.hasMoreElements())
	    buf.append(names.nextElement() + "-->" + ((Value) values.nextElement()).toString(null) + "; ");
	buf.append("]");

	names = this.funTable.keys();
	values =  this.funTable.elements();
	while(names.hasMoreElements())
	    buf.append(names.nextElement() + "-->" + values.nextElement() + "; ");
	
	return buf.toString();
    }

    /**
     * This method is an auxiliary method
     * for <code>setBindings()</code>.
     */
    private void setBindingsAux(Binding[] bindings, int k, BindingTreePath path)
    {
	if (k >= 0) {
	    // System.err.println("Adding " + bindings[k].getName());
	    this.setValue(bindings[k].getName(), path.getValue());
	    this.setBindingsAux(bindings, k - 1, path.getNext());
	}
    }

}




