/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Node;
import xacute.common.NodeFactory;
import xacute.common.Semantics;

/**
 * An <code>EvalContext</code> carries the context information needed
 * for the evaluation of <code>QuiltExpression</code>. Among other
 * things it contains a triple (current node, current node position,
 * size of the outer nodelist).
 *
 * The <code>EvalContext</code> is also used to provide a pointer to
 * the <code>NodeFactory</code> being used for the evaluation of the
 * expression. The <code>NodeFactory</code> is useful to create new
 * nodes or new nodelists.
 *
 * <code>Semantics</code> is not used right now.
 *
 */

public class EvalContext
    implements Cloneable
{

    private Node node;
    private int pos;
    private int size;
    private Environment env;
    private Semantics sem;

    private NodeFactory nf;

    public EvalContext()
    {
	this.node = null;
	this.pos = -1;
	this.size = -1;
	this.env = new Environment();
	this.nf = null;
    }

    public Object clone()
    {
	EvalContext con = new EvalContext();
	con.env = (Environment) this.env.clone();
	return con;
    }
    
    public EvalContext(Node node, int pos, int size, Environment env)
    {
	this.node = node;
	this.pos = pos;
	this.size = size;
	this.env = env;
	this.nf = null;
    }

  public EvalContext(Node n, int pos, int size, Environment env, Semantics sem, NodeFactory nf)
    {
	this(n, pos, size, env);
	this.setSemantics(sem);
	this.setNodeFactory(nf);
    }
    
    public EvalContext(Environment env)
    {
	this(null, -1, -1, env);
    }
    
    public Node getNode() { return this.node; }
    public int getPos() { return this.pos; }
    public int getSize() { return this.size; }
    public Environment getEnv() { return this.env; }

    public NodeFactory getNodeFactory()
 	{
		return this.nf; 
	}
    public void setNodeFactory(NodeFactory nf) { this.nf = nf; }

    public Semantics getSemantics() { return this.sem; }
    public void setSemantics(Semantics sem) { this.sem = sem; }

    public String toString()
    {
	return this.getEnv().toString();
    }
    
}









