/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

import java.util.Vector;


/**
 * A <code>FilterExpression</code> is an expression followed by a predicate.
 */
public class FilterExpression
    extends QuiltExpression
{

    private QuiltExpression e;

    private QuiltExpression[] filters;
    
    FilterExpression(QuiltExpression e, Vector filters)
    {
	this.e = e;
	this.filters = new QuiltExpression[filters.size()];
	filters.toArray(this.filters);
    }

    Value eval(EvalContext con)
	throws QuiltException
    {
	return this.e.eval(con).filter(this.filters, con);
    }   
    
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	buf.append("(" + this.e.pprint() + ")");
	for (int i = 0; i < this.filters.length; ++i)
	    buf.append("[" + this.filters[i].pprint() + "]");
	return buf.toString();
    }

/**
 * Insert the method's description here.
 * Creation date: (5/2/2002 1:05:54 PM)
 * @return xacute.quilt.QuiltExpression
 */
public QuiltExpression getE() {
	return e;
}

/**
 * Insert the method's description here.
 * Creation date: (5/2/2002 1:06:27 PM)
 * @return xacute.quilt.QuiltExpression[]
 */
public QuiltExpression[] getFilters() {
	return filters;
}
}
