/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

import java.util.Vector;
import org.xml.sax.helpers.AttributeListImpl;

/**
 * This code should be moved somewhere else (probably to
 * xacute.util.NodeDefaultImpl).
 **/
   
class FilterHelper
{

    /**
     * This function takes a node and a predicate and returns a list of nodes:
     * - the node itself if the node is kept
     * - some (or zero) descendant nodes if the node is skipped
     *
     * For every node we keep, we need to make a copy of the node.
     **/
    static NodeList filter( Node n, Predicate p, EvalContext con )
	throws QuiltException
    {

	/**
	 *  When we apply the Predicate, position and size are not
	 *  relevant
	 **/
	
	if (p.isTrue( n, -1, -1 ))
	    {
		// we keep the node itself

		if (n.isOfType(Constants.TEXT_NODE))
		    {
			return con.getNodeFactory().makeNodeList( con.getNodeFactory().makeTextNode( n.stringValue() ) );
		    }

		Vector filteredChildNodes = new Vector();
		NodeList attributes = n.findAxis( Constants.ATTRIBUTE_AXIS, null );
		AttributeListImpl attList = new AttributeListImpl();
		for(int i=0; i < attributes.size(); i++)
		    attList.addAttribute( attributes.get(i).getQName(),
					  "",
					  attributes.get(i).stringValue() );
		NodeList childNodes = n.findAxis( Constants.CHILD_AXIS, null );
		for(int i=0; i < childNodes.size(); i++)
		    {
			NodeList nodes = filter( childNodes.get(i), p, con );
			for(int j=0; j < nodes.size(); j++)
			    filteredChildNodes.add( nodes.get(j) );
		    }
		return con.getNodeFactory().makeNodeList( con.getNodeFactory().makeConstructedNode( n.getQName(), attList, filteredChildNodes ) );
	    }
	else
	    {
		// we skip the node, but we may keep its descendants
		NodeList filteredChildNodes = con.getNodeFactory().makeEmptyNodeList();
		NodeList childNodes = n.findAxis( Constants.CHILD_AXIS, null );
		for(int i=0; i < childNodes.size(); i++)
		    filteredChildNodes.APPEND( filter( childNodes.get(i), p, con ) );
		return filteredChildNodes;
	    }
    }
    
}

