/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;


import xacute.common.Constants;
import xacute.common.NodeList;
import xacute.common.QuiltException;

/**
 * This class represents a <b>FOR</b> clause.
 */
public class ForBinding
    extends Binding 
{

    ForBinding(boolean isFirst, String name, boolean distinct, QuiltExpression e)
    {
	super(isFirst, name, distinct, e);
    }

    public String pprint()
    {
	return (isFirst?"FOR":",") + " $" + this.name 
	    + " IN " 
	    + ( this.distinct ? "DISTINCT " : "") 
	    + this.e.pprint();
    }

    Value eval(EvalContext con)
	throws QuiltException
    {
	Value v = this.e.eval(con);
	if (!(v instanceof ValueNodeList))
	    throw new QuiltWrongTypeException
		("A FOR clause can not be evaluated into a NodeList :\n" + this);
	else
	    {
		NodeList nl = v.toNodeList(con);
		if (this.distinct)
		    nl = nl.freeDuplicateValues();
		return new ValueNodeList(nl);
	}
    }
}








