package xacute.quilt;

import xacute.common.Node;
import xacute.common.QuiltException;
import java.util.Vector;

/**
 * This class implements a function
 * defined in the preamble of
 * a <i>Quilt</i> expression.
 * This is NOT a <code>QuiltExpression</code>.
 */
public class FunDef
{
    
    /**
     * The name of the function.
     */
    private String funName;

    private Var[] argList;

    /**
     * The body of the function.
     */
    private QuiltExpression body;

    FunDef( String funName, Vector argList, QuiltExpression body)
    {
	this.funName = funName;
	if (argList != null)
	    {
		this.argList = new Var[ argList.size() ];
		argList.toArray( this.argList );
	    }
	else this.argList = new Var[0];
	this.body = body;
    }
    Value evalFun(Value[] argValues, EvalContext con)
	throws QuiltException
    {
	if (argValues.length != this.argList.length)
	    throw new QuiltWrongArgNumberException
		(
		 this.funName
		 + "() requires " + this.argList.length
		 + " arguments and not " + argValues.length
		 );
	else
	    {
		// this.findFreshVars(con.gectEnv());
		EvalContext newCon = (EvalContext) con.clone();
		newCon.setNodeFactory( con.getNodeFactory() );
		
		for (int k=0; k < this.argList.length; k++)
		    newCon.getEnv().setValue( this.argList[k].getName(), argValues[k]);
		if (this.body == null)
		    throw new RuntimeException("No body for method " + getName());
		Value val = this.body.eval(newCon);
		//for (int k=0; k < this.argList.length; k++)
		//    con.getEnv().rmBinding(this.argList[k].getName());
		return val;
	    }
    }
    /**
    *@author Xin Zhang
    */
    public QuiltExpression getBody(){
	    return this.body;
    }
    /**
     * Returns the name of the function.
     */
    public String getName() {  return this.funName; }
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	buf.append("FUNCTION " + this.funName + "(");
	int s = this.argList.length;
	for (int k=0; k < s; k++)
	    buf.append( this.argList[k].pprint() + ((k == s - 1) ? "" : ", "));
	if (this.body != null)
	    buf.append(") {\n" + this.body.pprint() + "\n}");
	else
	    buf.append(") {}");
	return buf.toString();
    }
    /**
     * Prints the name of the function.
     */
    public String printResult()
    { // might implement Value ?
	return this.funName + "()";
    }
}
