package xacute.quilt;

import java.util.*;
import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;
import java.io.*;

/**
 * This class returns lexicographical order values that can be used for
 *  order values, or unique key values.
 * This function uses two static buffers to determine midpoint values,
 *	therefore, if this code is used within recursive calls, it will
 *	be necessary to call this function with "reset" as the parameter
 *	in order to zero out the static strings.
 * Written by Brian Murphy
 */
public class FunLEXICOGRAPHICALORDER extends FunPrimitive
{
    private static float key_counter = 1;
    private static char[] letterArr =
            {
                'a',
                'b',
                'c',
                'd',
                'e',
                'f',
                'g',
                'h',
                'i',
                'j',
                'k',
                'l',
                'm',
                'n',
                'o',
                'p',
                'q',
                'r',
                's',
                't',
                'u',
                'v',
                'w',
                'x',
                'y',
                'z' };
    private static String lowBuff = "bb";
    private static String highBuff = "zz";

    
/**
 * Insert the method's description here.
 * Creation date: (9/11/02 9:59:56 PM)
 */
public FunLEXICOGRAPHICALORDER() {}
/**
 * Insert the method's description here.
 * Creation date: (9/11/02 10:00:22 PM)
 * @return xacute.quilt.Value
 * @param con xacute.quilt.EvalContext
 * @exception xacute.common.QuiltException The exception description.
 */
Value eval(EvalContext con) throws xacute.common.QuiltException {
    String retString = new String("");

    if (this.args.length > 2 || this.args.length < 1)
        throw new QuiltWrongArgNumberException(this.getName());
    else if(this.args.length == 1){
	    //if one arguement, should be COUNT(children)

        NodeList childCountNode = this.args[0].eval(con).toNodeList(con);

        if(childCountNode.stringValue().toLowerCase().equals("reset")){
			lowBuff = "bb";
			highBuff = "zz";
			return new ValueString("");
        }

        Float f1 = new Float(childCountNode.stringValue());
        int childCount = f1.intValue();

        if(childCount == 0) childCount = 1;

        double strLength = 1;
        double twensix = 26.0;
        while (java.lang.Math.pow(twensix, strLength) < childCount) strLength++;

        //strLength += 1;
        
        String tempString;
        if (lowBuff.length() < strLength) {
            tempString = new String();
            for (int j = 0; j < strLength; j++) {
                tempString += 'b';
            }
        } else {
            tempString = new String(lowBuff);
        }

        int distance = (int) java.lang.Math.pow(twensix, strLength) / childCount;

        retString = incrementBuff(tempString, distance);

        key_counter += 1;
        //System.out.println(retString + " " + key_counter);
    }else{
	    //if two arguements, should be something like "aaa", "bbbb"
	    // assumes first arguement is "less" than second arguement
	    
		//two arguments, used for inserts, find the key inbetween the two
		NodeList lowBuffNode = this.args[0].eval(con).toNodeList(con);
		NodeList highBuffNode = this.args[1].eval(con).toNodeList(con);

		String lowBuffStr = new String(lowBuffNode.stringValue());
		String highBuffStr = new String(highBuffNode.stringValue());

		retString = findMedianOrder(lowBuffStr, highBuffStr);		
		
    }

    return new ValueString(retString);
}
/**
 * Insert the method's description here.
 * Creation date: (9/24/02 12:20:14 PM)
 * @return java.lang.String
 * @param inBuff java.lang.String
 * @param charIncPos int
 */
public String findMedianOrder(String lowEnd, String highEnd) {
    String newBuff = new String();
    char[] newBuffArr = new char[lowEnd.length()];
    double[] encodeLow = new double[lowEnd.length()];
    double[] encodeHigh = new double[highEnd.length()];
    int lastInd = 0;
    
    double countLow = 0;
    for (int i = lowEnd.length() - 1; i >= 0; i--) {
        int newInd = 0;
        for (int j = 0; j < letterArr.length; j++) {
            if (letterArr[j] == lowEnd.charAt(i)) {
                newInd = j; lastInd = j;
                break;
            }
        }
        //numeral form of the letter
        // a in the rightmost (1th) position = 0 * 26^0
        // a in the 10th position = 0 * 26^1
        encodeLow[i] = newInd * java.lang.Math.pow(26, countLow++);
    }

    double countHigh = 0;
    for (int i = highEnd.length() - 1; i >= 0; i--) {
        int newInd = 0;
        for (int j = 0; j < letterArr.length; j++) {
            if (letterArr[j] == highEnd.charAt(i)) {
                newInd = j;
                break;
            }
        }
        //numeral form of the letter
        // a in the rightmost (1th) position = 0 * 26^0
        // a in the 10th position = 0 * 26^1
        encodeHigh[i] = newInd * java.lang.Math.pow(26, countHigh++);
    }

    if (encodeHigh.length == encodeLow.length) {
        int pow = 0;
        for (int k = encodeLow.length - 1; k >= 0; k--) {
            Double val1 = new Double(java.lang.Math.pow(26, pow));
            Double locLow = new Double(encodeLow[k] / val1.intValue());
            Double locHigh = new Double(encodeHigh[k] / val1.intValue());
            int newInd = locLow.intValue() + (locHigh.intValue() - locLow.intValue()) / 2;
            newBuffArr[k] = letterArr[newInd];
            pow += 1;
        }

        //if there is no difference, say between "aa" and "ab"
        // append a middle value to the lowEnd buffer, "aam"
        if (newBuffArr.toString().equals(lowEnd.toString())) {
            newBuff = new String(newBuffArr);
        } else {
            newBuff = new String(lowEnd + 'm');
        }
    } else {
	    //if the lengths are different, tricky math, instead
	    // append an 'm' to the lowEnd if last character is greater than z
	    // or just take (z - lastCharNumeralVal) / 2 
	    int val = (26 - lastInd) / 2;
	    if(val < 25){
	        newBuff = new String(lowEnd + letterArr[val]);
	    }else{
			newBuff = new String(lowEnd + 'm');
		}
		    
    }

    //distance = newIndHigh - newIndLow;

    return newBuff;
}
/**
 * Insert the method's description here.
 * Creation date: (9/24/02 12:20:14 PM)
 * @return java.lang.String
 * @param inBuff java.lang.String
 * @param charIncPos int
 */
public String incrementBuff(String inBuff, int distance) {
    String newBuff = new String();
    char[] newBuffCharArr = new char[inBuff.length()];
    double[] encodeCharInt = new double[inBuff.length()];
    boolean addCharToEnd = false;

    if (distance == 0)
        distance += 1;
    //System.out.println(inBuff + "*");

    //encode the string as an interger array with base 26
    double count = 0;
    for (int i = inBuff.length() - 1; i >= 0; i--) {
        int newInd = 0;
        for (int j = 0; j < letterArr.length; j++) {
            if (letterArr[j] == inBuff.charAt(i)) {
                newInd = j;
                break;
            }
        }
        //numeral form of the letter
        // a in the rightmost (1th) position = 0 * 26^0
        // a in the 10th position = 0 * 26^1
        encodeCharInt[i] = newInd * java.lang.Math.pow(26, count++);
    }

    encodeCharInt[encodeCharInt.length - 1] += distance;

    char[] nextBuffArr = new char[inBuff.length()];

    //check to see if any array value "overflowed"
    count = 1;
    for (int j = encodeCharInt.length - 1; j >= 0; j--) {
        double highVal = java.lang.Math.pow(26, count);
        if (encodeCharInt[j] >= highVal) {
            if (j != 0) {
	            double overFlowVal = encodeCharInt[j] - highVal;
                encodeCharInt[j] = overFlowVal;
                encodeCharInt[j - 1] += java.lang.Math.pow(26, count);
            } else {
                encodeCharInt[j] = highVal-1;
                addCharToEnd = true;
            }
        }
        count += 1;
    }

	//encode new letter array
    int pow = 0;
    for (int k = encodeCharInt.length - 1; k >= 0; k--) {
        Double val1 = new Double(java.lang.Math.pow(26, pow));
        	    //System.out.println(encodeCharInt[k] + " " + val1.intValue());
        Double loc = new Double(encodeCharInt[k] / val1.intValue());
        nextBuffArr[k] = letterArr[loc.intValue()];
        pow += 1;
    }

    //if, for some reason, we come across 'zzzz...', append "bb"
    // this will still allow for space for insertion between 'zzzzz...' and 'zzzzz...bb'
    if (addCharToEnd) {
        char[] nextBuffTmp = new char[nextBuffArr.length + 2];
        for (int l = 0; l < nextBuffArr.length; l++)
            nextBuffTmp[l] = 'z';
        nextBuffTmp[nextBuffTmp.length - 1] = 'b';
        nextBuffTmp[nextBuffTmp.length - 2] = 'b';
        nextBuffArr = nextBuffTmp;
    }

    newBuff = new String(nextBuffArr);

    //update the low buffer for the next value
    lowBuff = newBuff;

    return newBuff;
}
}
