/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Node;
import xacute.common.QuiltException;

public abstract class FunPrimitive
    extends FunCall
{    
    Value eval(EvalContext con)
	throws QuiltException
    {
	throw new QuiltWrongTypeException(this.getClassName() + ": invalid eval");
     }
    
    protected final String getClassName()
    {
	try
	    {
		String rawName = this.getClass().getName();
		if (!rawName.startsWith("xacute")) return rawName;
		int dot = rawName.lastIndexOf('.');
		return rawName.substring(dot+1).substring(3).toLowerCase();
	    }
	catch (Exception e)
	    {
		return "Builtin";
	    }
    }

    public String getName() { return getClassName(); }
    
    /**
     * Overrides the corresponding
     * method in the superclass
     * <code>FunCall</code>.
     */
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
        try {
	    buf.append( getClassName()+ "(" );
	    if (this.args != null)
		{
		    int s = this.args.length;
		    for (int k = 0; k < s; ++k)
			buf.append( this.args[k].pprint() + ((k == s - 1) ? "" : ", "));		    
		}
	    buf.append(")");
	    return buf.toString();
	} catch (Exception e) {
	    System.err.println("Error for FunPrimitive " + e);
	    return "PRIMITIVE()";
	}
    }
}



