/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import java.util.Vector;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;


public abstract class FunStringTemplate
    extends FunPrimitive
{

    public FunStringTemplate()
    {
	super();
    }
    
    public abstract String[] exec( String[] args )
	throws QuiltException;
    
    Value eval(EvalContext con)
	throws QuiltException
    {
	String[] _args = new String[args.length];
	for(int i=0; i < args.length; i++)
	    _args[i] = args[i].eval(con).toString(con);
	String[] result = exec(_args);
	if (result.length > 1)
	    {
		NodeList nl = con.getNodeFactory().makeEmptyNodeList();
		for(int i=0; i < result.length; i++)
		    nl = nl.APPEND( con.getNodeFactory().makeTextNode( result[i] ) );		   
		return new ValueNodeList( nl );
	    }
	return new ValueNode( con.getNodeFactory().makeTextNode( result[0] ) );
    }
    
    /**
     * Pretty-prints a function call.
     * This method is overriden in the
     * subclass <code>FunPrimitive</code>.
     */
    public final String pprint()
    {
	StringBuffer buf = new StringBuffer();
	buf.append("&" + this.getName() + "(");
	int s = this.args.length;
	for (int k = 0; k < s; ++k)
	    buf.append( this.args[k].pprint()
			+ ((k == s - 1) ? "" : ", "));
	buf.append(")");
	return buf.toString();
    }
}



