/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.QuiltException;

import java.util.Vector;

/**
 * This class represents (inside 
 * a <i>Quilt</i> query) a call to
 * a function defined in the
 * preamble of the query.
 */
public class FunUserDefined
    extends FunCall
{

    private String name;
    /**
     * The unique constructor
     * for a user-defined function.
     */
    FunUserDefined(String name)
    {
	this.name = name;
	this.args = new QuiltExpression[0];
    }

    public String getName() { return this.name; }
    
    Value eval(EvalContext con)
	throws QuiltException
    {
	Value[] argValues = this.evalArgs(con);
	FunDef f = con.getEnv().getFunValue(this.name);
	return f.evalFun(argValues, con);
    }  

    /**
     * Evaluates the arguments of
     * the <code>FunUserDefined</code>.
     * @param n         the context-node (<b>null
     *                  the function-call is
     *                  evaluated with no
     *                  context)
     * @param position  the position of the
     *                  context-node in the
     *                  context-node-list
     * @param size      the context-node-list
     * @param env the environment of
     *            evaluation
     * @return    a <code>Vector</code>
     *            containing <code>Value</code>
     *            objects (in the same order
     *            as the arguments).
     */
    private Value[] evalArgs(EvalContext con)
	throws QuiltException
    {
	Vector argValues = new Vector();
	for (int k=0; k < this.args.length; k++)
	    argValues.addElement(this.args[k].eval(con));
	Value[] result = new Value[argValues.size()];
	argValues.toArray(result);
	return result;
    }

}








