/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;


import xacute.common.Constants;
import xacute.common.NodeList;
import xacute.common.QuiltException;

/**
 * This class represents a <b>LET</b> clause.
 */
public class LetBinding
    extends Binding
{

    LetBinding(boolean isFirst, String name, boolean distinct, QuiltExpression e)
    {
	super(isFirst, name, distinct, e);
    }
    
    public String pprint()
    {
      	return (isFirst?"LET":",") + " $" + this.name 
	    + " := " 
	    + ( this.distinct ? "DISTINCT " : "") 
	    + this.e.pprint();
    } 
    
    Value eval(EvalContext con)
	throws QuiltException
    {
	Value v = this.e.eval(con);
	if (v==null)
	    {
		System.err.println("LET value is null.");
		return new ValueNodeList(con.getNodeFactory().makeEmptyNodeList());
	    }
	
	if ( (v instanceof ValueNodeList) && (this.distinct) )
	    {
		NodeList nl =  v.toNodeList(con).freeDuplicateValues();
		return new ValueNodeList(nl);
	    }
	return v;
    }
}


