package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

import java.util.Vector;

public class LocationStep
    extends Step
{

    /**
     * This field is a constant
     * representing the
     * <code>AxisSpecifier</code>
     * of a <code>LocationStep</code>.
     */
    private int axis;

    /**
     * This field is the node-test of
     * the <code>LocationStep</code>.
     */
    private NodeTest test;

    LocationStep(int axis, NodeTest test)
    {
	this.axis = axis;
	this.test = test;
    }
    LocationStep(int axis, NodeTest test, Vector predicates)
    {
	this(axis, test);
	this.predicates = new QuiltExpression[predicates.size()];
	predicates.toArray(this.predicates);
    }
    NodeList evalStep(Node n, EvalContext con)
	throws QuiltException
    {
	NodeList nl = n.findAxis(this.axis, this.test);
	if (!nl.isEmpty())
	    for (int i=0; i < this.predicates.length; i++)
		nl = nl.applyPredicate( new Predicate( this.predicates[i], con ) );
	return nl;
    }
    /**
    *@author Xin Zhang
    **/
    public int getAxis(){
	    return this.axis;
    }
    /**
    *@author Xin Zhang
    **/
    public NodeTest getTest(){
	    return this.test;
    }
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	buf.append("/");
	if ((this.axis >= 0) && (this.axis < Constants.AXISSPECIFIERS.length))
	    buf.append(Constants.AXISSPECIFIERS[this.axis]);
	buf.append("::");
	if (this.test != null) buf.append(this.test.pprint());
	for(int i=0; i < this.predicates.length; i++)
	    buf.append("[" + this.predicates[i].pprint() + "]");
	return buf.toString();
    }
    public QuiltExpression[] returnQE() {
      return predicates;
    }
}
