/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;


/**
 * An implementation of class java.util.Comparator that handles
 * multi-key sort.  Every key is a String[].  Key are sorted in order
 * ([0],[1], etc.), according to sortOrders.  true means ascending
 * order. false descending.  When creating an instance, the boolean
 * are converted into integers (+1 for ascending, -1 for descending).
 **/

public class MyCompare
    implements java.util.Comparator
{

    private int[] sortOrders;
    
    public MyCompare( SortClause[] sortOrders )
    {
	this.sortOrders = new int[ sortOrders.length ];
	for(int i=0; i < sortOrders.length; i++)
	    this.sortOrders[i] = sortOrders[i].isDescending()? -1:+1;
    }

    public int compare( Object o1, Object o2 )
    {
	String[] s1 = (String[]) o1;
	String[] s2 = (String[]) o2;

	for(int i=0; i < sortOrders.length; i++)
	    {
		int comp = compare(s1[i], s2[i]);
		if (comp != 0) return comp * sortOrders[i];
	    }
	return 0;
    }

    public boolean equals( Object o1, Object o2 )
    {
	return ( (o1 == o2) ? true:( (compare(o1,o2) == 0) ) );
    }

    private static int compare( String s1, String s2 )
    {
	try
	    {
		float f1 = new Float(s1).floatValue();
		float f2 = new Float(s2).floatValue();
		if (f1 == f2) return 0;
		if (f1 < f2) return -1;
		else return +1;
	    }
	catch(Exception e)
	    {
		return s1.compareTo(s2);
	    }
    }
    
} // MyCompare
