/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Node;
import xacute.common.QuiltException;


/**
 * This class implements a n
 * <i>XPath</i> name-test, which
 * returns all the nodes with
 * type equal to the principal
 * type of the axis and with name
 * given by the name-test.
 */
public class NameTest
    extends NodeTest
{

    /**
     * The name to verify.
     *
     * This field is <b>null</b>
     * for the name-test
     * <code>*</code>.
     */
    private String name;

    /**
     * Constructor for an
     * empty name-test.
     */
    public NameTest() {	this.name = null; }

    /**
     * Constructor for a
     * non-empty name-test.
     */
     public NameTest(String name) { this.name = name; }

    public String pprint()
    {
        return (this.name == null) ? "*" : this.name;
    }

    public String toString() { return pprint(); }

    public boolean isTrue(Node n, int axis)
    {
        return ((n.isOfPrincipalType(axis))
                &&
                ((this.name == null)
                 ||
                 (n.getQName().equals(this.name))));
        // (n.getExpandedName().equals(this.name))));
    }

    public String getName() {
      return name;
    }
}



