/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;
import java.util.Vector;

public class Nav extends QuiltExpression
{
    /** This field is the expression to evaluate first. */
    private QuiltExpression e;

    /** This array contains <code>Step</code> objects. */
    private Step[] steps = new Step[0];

    Nav(QuiltExpression e, Vector v)
    {
        this.e = e;
        this.efunc = null;
        if (v != null)
        {
            this.steps = new Step[v.size()];
            v.toArray(this.steps);
        }
    }

    /** The expression for the last function call. it's optional. */
    QuiltExpression efunc;

    public QuiltExpression getEfunc()
    {
        return efunc;
    }

    public QuiltExpression getE()
    {
        return e;
    }

    public Step[] getSteps(){
	    return this.steps;
    }

    /**
     * @param e the first quilt expression.
     * @param f the last optional function call.
     */
    Nav(QuiltExpression e, Vector v, QuiltExpression f)
    {
        this.e = e;
        this.efunc = f;
        if (v != null)
        {
            this.steps = new Step[v.size()];
            v.toArray(this.steps);
        }
    }

    Value eval(EvalContext con) throws QuiltException
    {
	    Value result = this.evalAux(this.e.eval(con), con);
	    if (this.efunc != null)
			result = evalFunc(result.toNodeList(con), con);

		return result;
    }	

    public String pprint()
    {
        StringBuffer buf = new StringBuffer();
        buf.append(this.e.pprint());
        for (int i = 0; i < this.steps.length; i++)
        {
            buf.append(this.steps[i].pprint());
        }
        if(this.efunc != null)
	        buf.append("/" + this.efunc.pprint());
        return buf.toString();
    }

    private Value evalAux(Value value, EvalContext con) throws QuiltException
    {
        if (this.steps.length == 0)
            return value;
        if (!value.isNodeValue())
            throw new QuiltSyntaxException("A non-node-list expression followed by some steps : " + this);
        NodeList nl = value.toNodeList(con);
        for (int i = 0; i < this.steps.length; i++)
        {
            NodeList res = con.getNodeFactory().makeEmptyNodeList();
            for (int k = 0; k < nl.size(); k++)
                // res = res.APPEND( this.steps[i].evalStep(nl.get(k), con));
                // major change Arnaud
                    res = res.UNION(this.steps[i].evalStep(nl.get(k), con));
            nl = res;
        }
            return new ValueNodeList(nl);
    }

    Value evalFunc(NodeList nl, EvalContext con) throws QuiltException
    {
        NodeList cur = nl;
        NodeList new_nl = con.getNodeFactory().makeEmptyNodeList();
        for (int j = 0; j < cur.size(); j++)
        {
            EvalContext newCon = new EvalContext(cur.get(j), j, cur.size(), con.getEnv());
            newCon.setNodeFactory(con.getNodeFactory());
            Value val = this.efunc.eval(newCon);
            new_nl.APPEND(val.toNodeList(newCon));
        }
        cur = new_nl;
        return new ValueNodeList(cur);
    }
}
