/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

/**
 * This class implements the <i>Quilt</i> quantifiers
 * <b>SOME</b> and <b>EVERY</b>.
 */
public class Quantifier
    extends QuiltExpression
{
    
    /**
     * This field is a constant representing
     * the quantifier.
     */
    private int op;

    /**
     * The name of the variable that must
     * satisfy a condition.
     */
    private String var;

    /**
     * The expression in which the variable
     * varies.
     */
    private QuiltExpression in;

    /**
     * The predicate expression to satisfy.
     */
    private QuiltExpression cond;
    
    Quantifier(int op,	String var, QuiltExpression in,	QuiltExpression cond)
    {
	this.op = op;
	this.var = var;
	this.in = in;
	this.cond = cond;
    }

    private String printOp()
    {
	if ((this.op >= 0) && (this.op < Constants.QUANTIFIERS.length))
	    return Constants.QUANTIFIERS[this.op];
	return "Unknown quantifier";
    }
    
    Value eval(EvalContext con)
	throws QuiltException
    {
	Value val = this.in.eval(con);
	if (!(val instanceof ValueNodeList))
	    throw new QuiltWrongTypeException
		("A containing expression in a "
		 + this.printOp()
		 + "can not be evaluated into a node-list");
	NodeList nl = val.toNodeList(con);
	throw new RuntimeException("xacute.quilt.Quantifier.eval() : NYI");
    }
    
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	if ((this.op >= 0) && (this.op < Constants.QUANTIFIERS.length))
	    buf.append(Constants.QUANTIFIERS[this.op]);
	buf.append(" $" + var + " IN " + this.in
		   + " SATISFIES " + this.cond);
	return buf.toString();
    }
}
