/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.QuiltException;

import java.util.Vector;

/**
 * This class represents a <i>Quilt</i> query, with a preamble containing
 * function definitions.
 */
public class QuiltQuery
    implements PPrintable
{

    /**
     * This field is an array containing
     * <code>FunDef</code> representations of the functions
     * defined in the preamble of the <code>QuiltQuery</code>.
     */
    private FunDef[] funDefs;
    public FunDef[] getfunDefs() {
      return funDefs;
    }

    /**
     * This field is the top-level expression to evaluate
     * in the <code>QuiltQuery</code>.
     */
    private QuiltExpression body;

    public QuiltExpression getBody() { return body; }

    public QuiltQuery( Vector funDefs, QuiltExpression body)
    {
	this.funDefs = new FunDef[funDefs.size()];
	funDefs.toArray(this.funDefs);
	this.body = body;
    }

    /**
     * Gives the result of a
     * <code>Query</code> by triggering
     * SAX events.
     * @param handler a <code>DocumentHandler</code> on which the
     * event-methods will be called
     * @param con the <code>EvalContext</code>
     */
    public void eval(org.xml.sax.DocumentHandler handler, EvalContext con)
	throws QuiltException
    {
	try
	    {
		for (int k=0; k < this.funDefs.length; k++)
		    {
			FunDef f = this.funDefs[k];
			con.getEnv().setFunValue(f.getName(), f);
		    }
		handler.startDocument();
		this.body.eval(con).toSAX(handler);
		handler.endDocument();
	    } catch(org.xml.sax.SAXException e) {
		System.out.println("Error in passing the query result to an org.xml.sax.DocumentHandler: "
				   + e.getMessage());
	    }
    }

    /**
     * Pretty-prints a <code>QuiltQuery</code>.
     */
    public String pprint()
    {
	StringBuffer buf = new StringBuffer();
	for (int k=0; k < this.funDefs.length; k++)
	    buf.append(this.funDefs[k].pprint() + "\n");
	buf.append(this.body.pprint());
	return buf.toString();
    }
}






