/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;

/**
 * This class contains useful methods
 * for comparisons that occur in the
 * evaluation of a <i>Quilt</i> expression,
 * and other useful methods.
 */
public class Tools
{

    /**
     * Performs a comparison between 2 floating numbers.
     */
    public static boolean compare(int op, float f1, float f2) {
	switch (op) {
	case Constants.LT:
	    return (f1 < f2);
	case Constants.GT:
	    return (f1 > f2);
	case Constants.GEQ:
	    return (f1 >= f2);
	case Constants.LEQ:
	    return (f1 <= f2);
	case Constants.EQ:
	    return (f1 == f2);
	case Constants.NEQ:
	    return (f1 != f2);
	default:
	    return false;
	}
    }

    /**
     * Performs a comparison between 2 Strings.
     */
    public static boolean compare(int op, String s1, String s2) {
	int k = s1.compareTo(s2);
	switch (op) {
	case Constants.LT:
	    return (k < 0);
	case Constants.GT:
	    return (k > 0);
	case Constants.GEQ:
	    return (k >= 0);
	case Constants.LEQ:
	    return (k <= 0);
	case Constants.EQ:
	    return (k == 0);
	case Constants.NEQ:
	    return (k != 0);
	default:
	    return false;
	}
    }
    
    /**
     * Performs a comparison between 2 booleans.
     */
    public static boolean compare(int op, boolean b1, boolean b2) {
	throw new RuntimeException("xacute.quilt.Tools.compare(int, boolean, boolean) : NYI");
    }

    /**
     * Reports a <i>PcData</i> to a 
     * <code>org.xml.sax.DocumentHandler</code>
     *
     * @param handler the <code>DocumentHandler</code> on which
     *                events must be triggered
     * @param s       the content of the <i>PcData</i>
     */
    public static void reportString(org.xml.sax.DocumentHandler handler,
				    String s) {
	try {
	    handler.characters(s.toCharArray(), 0, s.length());
	} catch (org.xml.sax.SAXException e) {
	    return;
	}
    }

    /**
     * Returns the attribute-name
     * corresponding to a given
     * element-name in a <code>Vector</code> 
     * consisting of <code>DerefPair</code>.
     * @param hints   the non-<b>null</b>
     *                vector of pairs
     * @param eltName the element-name
     * @return        <b>null</b> if the vector does
     *                not contain this element-name.
     */
    public static String attOfElt(java.util.Vector hints,
				  String eltName) {
	for (int k = 0; k < hints.size(); ++k)
	    if (((DerefPair) hints.elementAt(k)).getElt().equals(eltName))
		return ((DerefPair) hints.elementAt(k)).getAtt();
	return null;
    }

    /**
     * Returns the prefix 
     * of a <code>QName</code>,
     * or <b>null</b> if it
     * contains no prefix.
     */
    public static String findQNamePrefix(String qName) {
	int i = qName.indexOf(':');
	if (i >= 0)
	    return qName.substring(0, i);
	else
	    return null;
    }

    /**
     * Returns the local part
     * of a <code>QName</code>.
     */
    public static String findLocalPart(String qName) {
	int i = qName.indexOf(':');
	if (i >= 0)
	    return qName.substring(i + 1, qName.length());
	else
	    return qName;
    }

}
