/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;


import xacute.common.QuiltException;
import xacute.common.Node;
import xacute.common.NodeList;

public interface Value
{
    


    // conversion methods
    /**
     * We add the environment in order to pass some info such as factories or special semantics.
     *
     **/

    public boolean isNodeValue();


    // Bool
    public boolean toBool(EvalContext con);   

    // Num
    public float toNum(EvalContext con)
	throws QuiltWrongTypeException;

    // String
    public String toString(EvalContext con);

    // Node
    public Node toNode(EvalContext con)
	throws QuiltWrongTypeException;

    // NodeList
    public NodeList toNodeList(EvalContext con);
    
    
    // output methods 
    // public String printResult();
    public void toSAX(org.xml.sax.DocumentHandler handler) 
	throws org.xml.sax.SAXException;


    // filtering/comparison methods
    public Value filter(QuiltExpression[] filters, EvalContext con) 
	throws QuiltException;

    /**
     * Compares two <code>Value</code> objects.
     * @param op the constant representing 
     *           the comparison to perform
     * @param v  the <code>Value</code> object 
     *           representing the right-hand 
     *           side of the comparison 
     */
    
    public boolean compare(int op, Value         v,  EvalContext con) throws QuiltException;
    public boolean compare(int op, ValueBool     b,  EvalContext con) throws QuiltException;
    public boolean compare(int op, ValueNum      f,  EvalContext con) throws QuiltException;
    public boolean compare(int op, ValueString   s,  EvalContext con) throws QuiltException;
    public boolean compare(int op, ValueNode     n,  EvalContext con) throws QuiltException;
    public boolean compare(int op, ValueNodeList nl, EvalContext con) throws QuiltException; 

}









