/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.QuiltException;


public class ValueBool
    extends ValueCompared
{

    private boolean value;
    private final static char[] TRUE = new String("true").toCharArray();
    private final static char[] FALSE = new String("false").toCharArray();
    
    /**
     * The unique constructor of this class.
     */
    public ValueBool(boolean b)
    {
	this.value = b;
    }

    public void toSAX(org.xml.sax.DocumentHandler handler) 
	throws org.xml.sax.SAXException
    {
	if (this.value)
	    handler.characters(TRUE, 0, TRUE.length);
	else
	    handler.characters(FALSE, 0, FALSE.length);
    }

    /**
     * Accessor to the boolean value.
     */
    
    public boolean toBool(EvalContext con)
    {
	return this.value;
    }
    
    public String toString(EvalContext con)
    {
	return (this.value ? "TRUE" : "FALSE");
    }

    public float toNum(EvalContext con)
    {
	return (this.value ? 1 : 0);
    }

    public String printResult()
    {
	return this.toString();
    }

    public Value filter(QuiltExpression[] filters, EvalContext con)
	throws QuiltException
    {
	for (int k=0; k < filters.length; k++)
	    if (!filters[k].eval(con).toBool(con))
		return new ValueNodeList(con.getNodeFactory().makeEmptyNodeList()); // choice of semantics
	return this;
    }

    // Bool -> Bool -> boolean
    public boolean compare(int op, ValueBool b, EvalContext con)
	throws QuiltException
    {
	return __compareBools__( this.value, op, b.toBool(con) );
    }

    private boolean __compareBools__(boolean left, int op, boolean right)
	throws QuiltException
    {
	switch (op)
	    {
	    case Constants.EQ:
		return (left == right);
	    case Constants.NEQ:
		return (left != right);
	    default:
		throw new QuiltException("Unable to perform comparison");
	    }
    }
	
    // Bool -> Num -> boolean
    public boolean compare(int op, ValueNum n, EvalContext con)
	throws QuiltException
    {
	return __compareBools__(this.value, op, n.toBool(con));
    }

    // Bool -> String -> boolean
    public boolean compare(int op, ValueString s, EvalContext con)
	throws QuiltException
    {
	return __compareBools__(this.value, op, s.toBool(con));
    }

    // Bool -> Node -> boolean
    public boolean compare(int op, ValueNode n, EvalContext con)
	throws QuiltException
    {
	return __compareBools__(this.value, op, n.toBool(con));
    }

    // Bool -> NodeList -> boolean
    public boolean compare(int op, ValueNodeList nl, EvalContext con)
	throws QuiltException
    {
	return __compareBools__(this.value, op, nl.toBool(con));
    }

/**
 * Insert the method's description here.
 * Creation date: (8/29/02 10:14:44 PM)
 * @return java.lang.Boolean
 */
public boolean getValue() {
	return this.value;
}
}











