/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.QuiltException;
import xacute.common.Node;
import xacute.common.NodeList;

public abstract class ValueCompared 
    extends QuiltExpression
    implements Value
{
    
    /**
     * This method implements
     * the corresponding method 
     * in the interface
     * <code>xacute.quilt.Value</code>.
     */
    public NodeList toNodeList(EvalContext con)
    {
	try
	    {
		return con.getNodeFactory().makeNodeList(this.toNode(con));
	    } catch (Exception e) {
		// never happens
		throw new RuntimeException
		    ("xacute.quilt.ValueCompared.toNodeList()");
	    }
    }
    
    public Node toNode(EvalContext con) 
	throws QuiltWrongTypeException
    {
	return con.getNodeFactory().makeTextNode(this.toString(con));
    }

    Value eval(EvalContext con) 
	throws QuiltException
    {
	return this;
    }

    public abstract String toString(EvalContext con);

    
    public boolean compare(int op, Value v, EvalContext con)
	throws QuiltException
    {
	if (this instanceof ValueNum)
	    return v.compare( Constants.COMPOPS_REVERSE[op], (ValueNum) this, con);

	if (this instanceof ValueString)
	    return v.compare( Constants.COMPOPS_REVERSE[op], (ValueString) this, con);

	
	if (this instanceof ValueBool)
	    return v.compare( Constants.COMPOPS_REVERSE[op], (ValueBool) this, con);

	if (this instanceof ValueNode)
	    return v.compare( Constants.COMPOPS_REVERSE[op], (ValueNode) this, con);

	if (this instanceof ValueNodeList)
	    return v.compare( Constants.COMPOPS_REVERSE[op], (ValueNodeList) this, con);
	throw new RuntimeException("Unexpected type");
    }
 
    public boolean isNodeValue()
    {
	return false;
    }
}
