/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;


public class ValueNode
    extends ValueCompared
{

    private Node node;

    ValueNode(Node node) { this.node = node; }
    public boolean isNodeValue() { return true; }
    
    /**
     *
     * For the following methods, you can use env.getSemantics to
     * identify which semantics you want to implement.
     *
     */
    
    public boolean toBool(EvalContext con)
    {
	return true;
    }
    
    public Node toNode(EvalContext con)
	throws QuiltWrongTypeException
    {
	return this.node;
    }

    public NodeList toNodeList(EvalContext con)
    {
	return con.getNodeFactory().makeNodeList(this.node);
    }
    
    public float toNum(EvalContext con)
	throws QuiltWrongTypeException
    {
	return new ValueString(this.node.stringValue()).toNum(con);
    }

    public String toString(EvalContext con)
    {
	return this.node.stringValue();
    }
	

    public void toSAX(org.xml.sax.DocumentHandler handler) 
	throws org.xml.sax.SAXException
    {
	this.node.toSAX(handler);
	return;
    }

    public Value filter(QuiltExpression[] filters, EvalContext con)
	throws QuiltException
    {
	/* we convert to node list because node lists handle context
	   (node, pos, size)
	*/
	return new ValueNodeList( this.toNodeList(con) ).filter( filters, con);
    }

    // Node -> Bool -> boolean 
    public boolean compare(int op, ValueBool b, EvalContext con)
	throws QuiltException
    {
	return b.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // Node -> Num -> boolean 
    public boolean compare(int op, ValueNum f, EvalContext con)
	throws QuiltException
    {
	return f.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // Node -> String -> boolean 
    public boolean compare(int op, ValueString s, EvalContext con)
	throws QuiltException
    {
	return s.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // Node -> Node -> boolean 
    public boolean compare(int op, ValueNode n, EvalContext con)
	throws QuiltException
    {
	throw new QuiltException("ValueNode.compare NYI");
    }

    public boolean compare(int op, ValueNodeList nl, EvalContext con)
	throws QuiltException
    {
	return new ValueNodeList(this.toNodeList(con)).compare(op, nl, con);
	//throw new QuiltException("ValueNode.compare NYI");
    }
}


