/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

import xacute.util.NodeHelper;

public class ValueNodeList
    extends ValueCompared
{

    private NodeList nodelist;

    ValueNodeList(NodeList nl) { this.nodelist = nl; }
    public boolean isNodeValue() { return true; }
    /**
     *
     * For the following methods, you can use env.getSemantics to
     * identify which semantics you want to implement.
     *
     */
    
    public boolean toBool(EvalContext con)
    {
	return (!nodelist.isEmpty());
    }
    
    public Node toNode(EvalContext con)
	throws QuiltWrongTypeException
    {
	if (this.nodelist.size()==1) return this.nodelist.get(0);
	throw new QuiltWrongTypeException("Trying convert a node list into a node");
    }

    public NodeList toNodeList(EvalContext con)
    {
	return this.nodelist;
    }
    
    public float toNum(EvalContext con)
	throws QuiltWrongTypeException
    {
	if (this.nodelist.isEmpty())
	    return (new Float(0)).floatValue();
	try
	    {
		return ( NodeHelper.toFloat( this.nodelist.get(0), con.getSemantics() ) );
	    }
	catch(QuiltException e)
	    {
		throw new QuiltWrongTypeException(e.toString() + this.nodelist.get(0).stringValue() );
	    }
    }

    public String toString(EvalContext con)
    {
	return this.nodelist.stringValue();
    }
	

    public void toSAX(org.xml.sax.DocumentHandler handler) 
	throws org.xml.sax.SAXException
    {
	for(int i=0; i<this.nodelist.size(); i++)
	    nodelist.get(i).toSAX(handler);
	return;
    }

    public Value filter(QuiltExpression[] filters, EvalContext con)
	throws QuiltException
    {
	NodeList cur = this.nodelist;
	NodeList nl = con.getNodeFactory().makeEmptyNodeList();
	for(int i=0; i < filters.length; i++)
	    {
		for(int j=0; j < cur.size(); j++)
		{
		    EvalContext newCon = new EvalContext( cur.get(j), j, cur.size(), con.getEnv() );
		    newCon.setNodeFactory( con.getNodeFactory() );
		    if (filters[i].isTrue(newCon))
			nl = nl.APPEND( cur.get(j) );
		}
		cur = nl;
	    }
	return new ValueNodeList(cur);
    }

    // NodeList -> Bool -> boolean  
    public boolean compare(int op, ValueBool b, EvalContext con)
    	throws QuiltException
    {
	return b.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // NodeList -> Num -> boolean 
    public boolean compare(int op, ValueNum f, EvalContext con)
	throws QuiltException
    {
	return f.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // NodeList -> String -> boolean 
    public boolean compare(int op, ValueString s, EvalContext con)
    	throws QuiltException
    {
	return compare( op, new ValueNodeList( con.getNodeFactory().makeNodeList( con.getNodeFactory().makeTextNode(s.toString()))), con );
 	// return s.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // NodeList -> Node -> boolean
    public boolean compare(int op, ValueNode n, EvalContext con)
    	throws QuiltException
    {
	return n.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // NodeList -> NodeList -> boolean 
    public boolean compare(int op, ValueNodeList n, EvalContext con)
    	throws QuiltException
    {
	switch(op)
	    {
	    case Constants.EQ:
		    return this.nodelist.isEqualTo(n.toNodeList(con));
	    case Constants.NEQ:
		return this.nodelist.isDifferentFrom(n.toNodeList(con));
	    default:
		try
		    {
			return new ValueNum( this.toNum(con) ).compare( op, new ValueNum(n.toNum(con)), con );
		    }
		catch (Exception e)
		    {
			return new ValueString( this.toString(con) ).compare( op, new ValueString(n.toString(con)), con );
		    }
		// throw new QuiltException("Comparison of 2 node lists is only defined for '=' and '!='");
	    }
    }

}

