/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.quilt;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeList;
import xacute.common.QuiltException;

/**
 * This class represents <i>Quilt</i>'s string
 * basic values.
 * <br> Whenever the string value can be converted to
 * a number, it <b>must</b> be done !
 */
public class ValueString
    extends ValueCompared
{

    private String value;

    public String returnString(){ 
	    return this.value;
	}
    
    /**
     * The unique constructor of this class.
     */
    public ValueString(String unquoted) {
	this.value = unquoted;
    }

    public void toSAX(org.xml.sax.DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	Tools.reportString(handler, this.toString(null));
    }

    public boolean toBool(EvalContext con)
    {
	return (this.value.length() > 0);
    }

    public String pprint()
    {
	return "\"" + this.value + "\"";
    }

    /**
     * Accessor to the string value.
     */
    public String toString(EvalContext con)
    {
	return this.value;
    }

    public float toNum(EvalContext con)
	throws QuiltWrongTypeException
    {
	try {
	    float f = (new Float(this.value)).floatValue();
	    return f;
	} catch (Exception e) {
	    throw new QuiltWrongTypeException("Impossible to convert "
					      + this.value
					      + " into a numeric value");
	}
    }

    public String printResult()
    {
	return this.toString();
    }

    public Value filter(QuiltExpression[] filters, EvalContext con)
	throws QuiltException
    {
	for (int k=0; k < filters.length; k++)
	    if (!filters[k].eval(con).toBool(con))
		return new ValueNodeList( con.getNodeFactory().makeEmptyNodeList()); // choice of semantics
	return this;
    }

    // String -> Bool -> boolean
    public boolean compare(int op, ValueBool b, EvalContext con)
	throws QuiltException
    {
	return b.compare( Constants.COMPOPS_REVERSE[op], this, con );
    }

    // String -> Num -> boolean
    public boolean compare(int op, ValueNum f, EvalContext con)
	throws QuiltException
    {
	try
	    {
		ValueNum left = new ValueNum( (new Float(this.value)).floatValue());
		return f.compare( Constants.COMPOPS_REVERSE[op], left, con);
	    } catch (Exception e) {
		return false;
	    }
    }

    // String -> String -> boolean
    public boolean compare(int op, ValueString s, EvalContext con)
	throws QuiltException
    {
	return __compareStrings__( this.value, op, s.toString(con) );
    }

    private boolean __compareStrings__( String s1, int op, String s2)
    {
	int k = s1.compareTo(s2);
	switch (op) {
	case Constants.LT:
	    return (k < 0);
	case Constants.GT:
	    return (k > 0);
	case Constants.GEQ:
	    return (k >= 0);
	case Constants.LEQ:
	    return (k <= 0);
	case Constants.EQ:
	    return (k == 0);
	case Constants.NEQ:
	    return (k != 0);
	default:
	    return false;
	}
    }

    // String -> Node -> boolean
    public boolean compare(int op, ValueNode n, EvalContext con)
	throws QuiltException
    {
	String s = n.toString(con);
	return __compareStrings__( this.value, op, s);
    }

    // String -> NodeList -> boolean
    public boolean compare(int op, ValueNodeList nl,  EvalContext con)
	throws QuiltException
    {
	if ((op == Constants.EQ) || (op == Constants.NEQ))
	return nl.compare( op, new ValueNodeList( con.getNodeFactory().makeNodeList( con.getNodeFactory().makeTextNode(this.toString(con)))), con );

	String s = nl.toString(con);
	return __compareStrings__( this.value, op, s);
    }

    public String getValue() {
      return value;
    }

}





