/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package xacute.quilt;

import xacute.common.QuiltException;
import xacute.common.Node;

/**
 * This class represents a
 * variable in <i>Quilt</i>.
 */
public class Var extends QuiltExpression implements TagNameable
{

    private String name;

    /**
     * The unique constructor of this class.
     * @param name the name of the variable, with the <code>$</code>
     *             character chopped off.
     */
    Var(String name)
    {
        this.name = name;
    }

    public String getName()
    {
        return this.name;
    }

    /**
     * When a variable is used to provide the value of a tag.
     * e.g. <$a/>
     **/
    public String getTagName(EvalContext con) throws QuiltException
    {
        return this.eval(con).toString(con);
    }

    public String getTagName()
    {
        return "$" + this.name;
    }

    Value eval(EvalContext con) throws QuiltException
    {
        if (!con.getEnv().isBoundVar(this.name))
            System.err.println("Problem with " + this.name);
        Value v = con.getEnv().getValue(this.name);
        if (v == null)
            throw new QuiltUnboundVarException("$" + this.name + ". Null value.");
        return v;
    }

    boolean typeCheck(int type)
    {
        return true;
    }

    public String pprint()
    {
        return "$" + name;
    }
}