/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



/**
 * NodeFactoryImpl.java
 *
 *
 * Created: Tue Jun 27 14:30:14 2000
 *
 * @author Arnaud Sahuguet
 * @version
 */

package xacute.util;

import xacute.common.Constants;
import xacute.common.Node;
import xacute.common.NodeFactory;
import xacute.common.NodeList;
import xacute.common.QuiltException;
import xacute.util.QuiltElementNode;
import xacute.util.QuiltShallowNode;
import xacute.util.QuiltTextNode;

import org.xml.sax.AttributeList;
import org.xml.sax.InputSource;
import org.apache.xerces.parsers.DOMParser;

import java.net.URL;
import java.io.FileReader;
import java.io.StringReader;
import java.util.Hashtable;
import java.util.Vector;

public abstract class NodeFactoryDefaultImpl
    implements NodeFactory
{

    /* methods that MUST be implemented by subclasses */

    /* provides the name of the factory */
    public abstract String getFactoryName();

    /** provides the way to create NodeList instances */
    public abstract NodeList makeEmptyNodeList();

    /** provides the parser to use
     * provides the way to create Node instances
     */
    protected abstract Node parse( InputSource is )
	throws Exception;
    
    
    public NodeList makeNodeList(Node n)
    {
	return makeEmptyNodeList().APPEND(n);
    }

    public NodeList makeNodeList(NodeList nl)
    {
	NodeList result = makeEmptyNodeList();
	for(int i=0; i<nl.size(); i++)
	    result.APPEND(nl.get(i));
	return result;
    }

    public Node getInLineXMLRoot(String xml)
    	throws QuiltException	       	       
    {
	InputSource is = new InputSource( new StringReader(xml) ); 
	try
	    {
		return parse( is );
	    }
	catch(Exception e)
	    {
		throw new QuiltException(e.toString());
	    }
    }
    
    public Node getDocumentRoot(String name)
    	throws QuiltException	       
    {
	Node root = cache.get(name);
	if (root != null)
	    {
		// System.err.println("Cache MISS for " + name);
		return root;
	    }
	try
	    {
		// System.err.println("Cache MISS for " + name);
		InputSource is;
		if (name.startsWith("http://"))
		    {
			URL url = new URL(name);
			is = new InputSource(url.openStream());
		    }
		else is = new InputSource( new FileReader(name) );
		
		root = parse(is);

		cache.add(name, root);
		return root;
	    }
	catch(Exception e)
	    {
		throw new QuiltException(e.toString());
	    }
    }

    public Node makeTextNode(String text)
    {
	return new QuiltTextNode(text);
    }
    
    public Node makeConstructedNode(String name, AttributeList atts, Vector children)
    {
	Node[] childNodes = new Node[children.size()];
	children.toArray(childNodes);
	return new QuiltElementNode(name, atts, childNodes);
    }
    
    public Node makeShallowNode(Node n)
    {
	return new QuiltShallowNode(n);
    }

    
    /* Caching */
    protected DocumentCache cache;
    
    public NodeFactoryDefaultImpl()
    {
	cache = new DocumentCache();
    }
    
    public void clearDocumentCache()
    {
	this.cache.clear();
    }

    
} // NodeFactoryImpl


