/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.util;

import xacute.common.Constants;
import xacute.common.ExpandedName;
import xacute.common.Hint;
import xacute.common.Node;
import xacute.common.NodeTest;
import xacute.common.NodeList;
import xacute.common.Predicate;
import xacute.common.QuiltException;

import org.xml.sax.DocumentHandler;
import org.xml.sax.AttributeList;
import org.xml.sax.helpers.AttributeListImpl;

/**
 * A <code>QuiltAttributeNode</code> is NOT the result an attribute creation in a <code>RETURN</code> clause.
 * It is used to return attributes from a <code>QuiltElementNode</code>
 *
 * Like for other Quilt nodes, we restrict navigation.
 *
 **/


public class QuiltAttributeNode
    implements Node
{
    private String name;
    private String value;
    private Node parent;
    
    public QuiltAttributeNode( String name, String value, Node parent )
    {
	this.name = name;
	this.value = value;
	this.parent = parent;
    }

    public boolean equals(Node n)
    {
	return (this == n);
    }

    public int compareTo(Node n)
    {
	return this.parent.compareTo(n);
    }
    
    public NodeList findAxis( int axis, NodeTest test)
    {
	NodeList nl = new NodeListDefaultImpl();
        switch (axis)
	    {
	    case Constants.ANCESTOR_AXIS:
		nl = nl.APPEND( this.parent.findAxis( axis, test ) );
		break;
	    case Constants.CHILD_AXIS:
	    case Constants.DESCENDANT_AXIS:
	    case Constants.DESCENDANT_SELF_AXIS:
	    case Constants.FOLLOWING_AXIS:
	    case Constants.PRECEDING_AXIS:
	    case Constants.FOLLOWING_SIBLING_AXIS:
	    case Constants.PRECEDING_SIBLING_AXIS:
		return nl;
	    case Constants.PARENT_AXIS:
		nl = nl.APPEND( this.parent);
		break;
	    case Constants.ATTRIBUTE_AXIS:
	    case Constants.NAMESPACE_AXIS:
		throw new RuntimeException("xacute.util.QuiltTextNode.findAxis(): invalid navigation");
	    case Constants.ANCESTOR_SELF_AXIS:
		nl.APPEND(this);
		nl.APPEND( this.parent.findAxis( axis, test ) );
		break;
	    case Constants.SELF_AXIS:
		nl = nl.APPEND(this);
		break;
	    }
	NodeList result = new NodeListDefaultImpl();
	for(int i=0; i < nl.size(); i++)
	    if (test.isTrue(nl.get(i), axis))
		result = result.APPEND(nl.get(i));
	return result;
    }

    public NodeList findAxis(int axis, NodeTest nt, Predicate[] predicates)
    	throws QuiltException
    {
	NodeList result = this.findAxis(axis, nt);
	for(int i=0; i < predicates.length; i++)
	    result = result.applyPredicate(predicates[i]);
	return result;
    }

    public Node getDocumentRoot()
	throws QuiltException
    {
	throw new QuiltException("A QuiltTextNode does not belong to any document");
    }
    
    public ExpandedName getExpandedName()
    	throws QuiltException
    {
	return new ExpandedName( this.name );
    }

    public ExpandedName expand(String qName)
	throws QuiltException
    {
	return new ExpandedName( this.name );
    }
    
    public String getQName()
    {
	return this.name;
    }
    
    public boolean isOfPrincipalType(int axis)
    {
	if (axis == Constants.ATTRIBUTE_AXIS) return true;
	return false;
    }
    
    public boolean isOfType(int type)
    {
	return (type == Constants.ATTRIBUTE_NODE);
    }
    
    public String stringValue()
    {
	return this.value;
    }

    public String stringValue_shallow()
    {
	throw new RuntimeException("QuiltAttribute.stringValue_shallow()");
    }

    
    public NodeList deref(String idref, Hint[] hints)
    {
	//	return new NodeListDefaultImpl();
	throw new RuntimeException("QuiltAttribute.deref()");
    }

    
    public void toSAX(DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	    // Looks like they didn't implement this feature. So, let's implement it then.
		//throw new RuntimeException("QuiltAttribute.toSAX()");
		// Start from here!
		// Remember, we only need the value of the attributes.
		if(this.value != null)
			handler.characters(this.value.toCharArray(), 0, this.value.length());
    }

    public void toSAX_shallow(DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	throw new RuntimeException("QuiltAttribute.toSAX_shallow()");
    }

}





