/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.util;

import xacute.common.Constants;
import xacute.common.ExpandedName;
import xacute.common.Hint;
import xacute.common.Node;
import xacute.common.NodeTest;
import xacute.common.NodeList;
import xacute.common.Predicate;
import xacute.common.QuiltException;

import org.xml.sax.DocumentHandler;
import org.xml.sax.AttributeList;
import org.xml.sax.helpers.AttributeListImpl;

/**
 * The <code>SHALLOW</code> keyword simply means that when the node is
 * displayed, the subtree has to be ignored. One way to implement this
 * is simply to make a copy of the node without the subtree. In Kweelt
 * we have chosen a different approach where a
 * <code>QuiltShallowNode</code> is a wrapper on top a a regular
 * node. The wrapper simply redefines some methods and make sure that
 * the subtree is always ignored.
 *
 * Like for the other Quilt_xxxx_Nodes, we prohibit navigation.
 *
 **/ 

public class QuiltShallowNode
    implements Node
{
    private Node node;

    public QuiltShallowNode(Node node)
    {
	this.node = node;
    }

    public boolean equals(Node n)
    {
	return (this.node == n);
    }

    public int compareTo(Node n)
    {
	System.err.println("Node comparison for SHALLOW nodes: to be fixed");
	// return Constants.BEFORE;
	return -1 * n.compareTo(this.node);
    }
    
    public NodeList findAxis( int axis, NodeTest test)
    {
	NodeList nl = new NodeListDefaultImpl();
        switch (axis)
	    {
	    case Constants.ANCESTOR_AXIS:
	    case Constants.CHILD_AXIS:
	    case Constants.DESCENDANT_AXIS:
	    case Constants.FOLLOWING_AXIS:
	    case Constants.PRECEDING_AXIS:
	    case Constants.FOLLOWING_SIBLING_AXIS:
	    case Constants.PRECEDING_SIBLING_AXIS:
	    case Constants.PARENT_AXIS:
		throw new RuntimeException("xacute.util.QuiltTextNode.findAxis(): invalid navigation");
	    case Constants.ATTRIBUTE_AXIS:
	    case Constants.NAMESPACE_AXIS:
		return this.node.findAxis(axis, test);
	    case Constants.ANCESTOR_SELF_AXIS:
	    case Constants.DESCENDANT_SELF_AXIS:
	    case Constants.SELF_AXIS:
		nl = nl.APPEND(this);
            break;
	    }
	NodeList result = new NodeListDefaultImpl();
	for(int i=0; i < nl.size(); i++)
	    if (test.isTrue(nl.get(i), axis))
		result = result.APPEND(nl.get(i));
	return result;
    }

    public NodeList findAxis(int axis, NodeTest nt, Predicate[] predicates)
    	throws QuiltException
    {
	NodeList result = this.findAxis(axis, nt);
	for(int i=0; i < predicates.length; i++)
	    result = result.applyPredicate(predicates[i]);
	return result;
    }

    public Node getDocumentRoot()
	throws QuiltException
    {
	throw new QuiltException("A ShallowNode does not belong to any document");
    }
    
    public ExpandedName getExpandedName()
    	throws QuiltException
    {
	return null;
    }

    public ExpandedName expand(String qName)
	throws QuiltException
    {
	return null;
    }
    
    public String getQName()
    {
	throw new RuntimeException("xacute.util.QuiltTextNode.getQName()");
    }
    
    public boolean isOfPrincipalType(int axis)
    {
	return this.node.isOfPrincipalType(axis);
    }
    
    public boolean isOfType(int type)
    {
	return this.node.isOfType(type);
    }
    
    public String stringValue()
    {
	return this.node.stringValue_shallow();
    }

    public String stringValue_shallow()
    {
	return this.node.stringValue_shallow();
    }
    

    public NodeList deref(String idref, Hint[] hints) { return new NodeListDefaultImpl(); }

    public void toSAX(DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	this.node.toSAX_shallow(handler);
    }
    
    public void toSAX_shallow(DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	this.node.toSAX_shallow(handler);
    }

}





