/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.util;

import xacute.common.Constants;
import xacute.common.ExpandedName;
import xacute.common.Hint;
import xacute.common.Node;
import xacute.common.NodeTest;
import xacute.common.NodeList;
import xacute.common.Predicate;
import xacute.common.QuiltException;

import org.xml.sax.DocumentHandler;


/**
 * A QuiltTextNode is used when a text value produced by Kweelt needs
 * to be inserted in the XML result tree.
 *
 * Since a QuiltNode is a regular node, it needs to implement the
 * <code>Node</code> interface and in particular the navigation
 * methods, which are not always clearly defined:
 * - do we allow navigation from a QuiltTextNode?
 * Right now, we do not, as should be clear from the class constructor that does not require a parent node.
 *
 **/

 
public class QuiltTextNode
    implements Node
{
    private String text;

    public QuiltTextNode(String text) { this.text = text; }

    public boolean equals(Node n)
    {
	return (this == n);
    }

    public int compareTo(Node n)
    {
	return Constants.NOT_COMPARABLE;
    }
    
    public NodeList findAxis( int axis, NodeTest test)
    {
	NodeList nl = new NodeListDefaultImpl();
        switch (axis)
	    {
	    case Constants.ANCESTOR_AXIS:
	    case Constants.CHILD_AXIS:
	    case Constants.DESCENDANT_AXIS:
	    case Constants.FOLLOWING_AXIS:
	    case Constants.PRECEDING_AXIS:
	    case Constants.FOLLOWING_SIBLING_AXIS:
	    case Constants.PRECEDING_SIBLING_AXIS:
	    case Constants.PARENT_AXIS:
		return nl;
	    case Constants.ATTRIBUTE_AXIS:
	    case Constants.NAMESPACE_AXIS:
		throw new RuntimeException("xacute.util.QuiltTextNode.findAxis(): invalid navigation");
	    case Constants.ANCESTOR_SELF_AXIS:
	    case Constants.DESCENDANT_SELF_AXIS:
	    case Constants.SELF_AXIS:
		nl = nl.APPEND(this);
            break;
	    }
	NodeList result = new NodeListDefaultImpl();
	for(int i=0; i < nl.size(); i++)
	    if (test.isTrue(nl.get(i), axis))
		result = result.APPEND(nl.get(i));
	return result;
    }

    public NodeList findAxis(int axis, NodeTest nt, Predicate[] predicates)
    	throws QuiltException
    {
	NodeList result = this.findAxis(axis, nt);
	for(int i=0; i < predicates.length; i++)
	    result = result.applyPredicate(predicates[i]);
	return result;
    }

    public Node getDocumentRoot()
	throws QuiltException
    {
	throw new QuiltException("A QuiltTextNode does not belong to any document");
    }
    
    public ExpandedName getExpandedName()
    	throws QuiltException
    {
	return null;
    }

    public ExpandedName expand(String qName)
	throws QuiltException
    {
	return null;
    }
    
    public String getQName()
    {
	throw new RuntimeException("xacute.util.QuiltTextNode.getQName()");
    }
    
    public boolean isOfPrincipalType(int axis)
    {
	return false;
    }
    
    public boolean isOfType(int type)
    {
	return (type == Constants.TEXT_NODE);
    }
    
    public String stringValue() { return this.text; }
    public String stringValue_shallow() { return this.text; }

    
    public NodeList deref(String idref, Hint[] hints) { return new NodeListDefaultImpl(); }

    public void toSAX(DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	handler.characters( text.toCharArray(), 0, text.length() );
    }
    
    public void toSAX_shallow(DocumentHandler handler)
	throws org.xml.sax.SAXException
    {
	handler.characters( text.toCharArray(), 0, text.length() );
    }

}





