/* Kweelt - a framework to query XML documents
 * Copyright (C) 2000,  Arnaud Sahuguet (Arnaud.Sahuguet@polytechnique.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


package xacute.util;

import java.io.PrintWriter;
import java.util.StringTokenizer;

import org.xml.sax.AttributeList;
import org.xml.sax.DocumentHandler;
import org.xml.sax.Locator;

public class ToStringHandler 
    implements DocumentHandler
{

    private StringBuffer buf;
    private int indent = 0;
    private final static int TAB = 2;
    
    private void untab() { indent -= TAB; }
    private void tab() { indent += TAB; }

    private void makeTab()
    {
	for(int i=0; i < indent; i++) buf.append(' ');
    }
    
    public ToStringHandler()
    {
        this.buf = new StringBuffer();
    }

    public String toString()
    {
	return buf.toString();
    }
    
    public void characters(char[] ch, int start, int length)
    {
	StringTokenizer tok = new StringTokenizer(new String(ch), "\n\r");
	while(tok.hasMoreTokens())
	      {
		  makeTab();
		  buf.append( tok.nextToken() );
	      }
	buf.append("\n");
	return;
    }
    
    public void endDocument()
    {
	buf.append("<!-- end of document -->\n");
    }
    
    public  void endElement(String name)
    {	
	untab();
	makeTab();
	buf.append("</" + name + ">\n");
    }
    
    public void ignorableWhitespace(char[] ch, int start, int length)
    {
	return;
    }

    public void processingInstruction(String target, String data)
    {
            return;
    }

    public void setDocumentLocator(Locator locator)
    {                                   
	return;
    }

    public void startDocument() {
	buf.append("<?xml version=\"1.0\"?>\n");
	return;
    }

    public void startElement(String name, AttributeList atts)
    {
	makeTab();
	buf.append("<" + name);
	for(int i = 0; i < atts.getLength(); ++i)
	    buf.append(" " + atts.getName(i) + "=\"" + atts.getValue(i) + "\"");
	buf.append(">\n");
	tab();
	return;
    }

}                                    


