package edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatemanagers;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.BitSet;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStamp;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStampException;
import edu.wpi.cs.dsrg.xmldb.xat.common.dag.XATTimeStampImp;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.JugglerTuple.JugglerTuple;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.JugglerTuple.JugglerTupleList;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.JugglerTuple.JugglerWindowTupleList;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.exploitationstructures.JoinExpTreeStructure;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.exploitationstructures.WindowExpStructureList;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicateBitRepresentation.ExtGeneralTypeBitSet;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicateBitRepresentation.ExtPredicateBitSet;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicateBitRepresentation.GeneralPredicateBitSet;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatelists.AttrPredOrderList;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatelists.AttrPredOrderListS;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatelists.BitPosition;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatelists.JWPredOrderList;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatelists.OrderList;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatelists.OrderListS;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatestructure.AttrPredicate;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.predicatestructure.Predicate;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.query.Query;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.query.QueryDependentEncoding;
import edu.wpi.cs.dsrg.xmldb.xat.component.operator.Juggler.query.QueryQED;
import edu.wpi.cs.dsrg.xmldb.xat.raindrop.utils.DeepCopyMaker;
/**
 * Updates the selectivities of the predicate types.
 * Creation date: (1/12/2003 8:58:49 PM)
 * @author: Administrator
 */
public class PredicateOrderingManager implements Serializable {
	protected JoinPredicateManager join;
	protected PredicateManager attribute;
	protected WindowPredicateManager window;
	protected ArrayList intermediateBuffer;
	protected Hashtable hash;
	protected java.lang.String[] mostSelectivePredicateTypesInOrder;
	protected QueryDependentEncoding outputQDE;
	/**
	 * PredicateOrderingManager constructor comment.
	 */
	public PredicateOrderingManager() {
		super();
		hash = new Hashtable();
		attribute = new PredicateManager("attribute");
		window = new WindowPredicateManager("window");
		join = new JoinPredicateManager("join");
		mostSelectivePredicateTypesInOrder = new String[3];
		//default order
		//test of dynamic change: attribute, join, window
		//
		mostSelectivePredicateTypesInOrder[0] = "join";
		mostSelectivePredicateTypesInOrder[1] = "attribute";
		mostSelectivePredicateTypesInOrder[2] = "window";
		//the owner of this queue is this predicate ordering manager
		intermediateBuffer = new ArrayList();

		outputQDE = new QueryDependentEncoding();
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 6:45:07 PM)
	 * @param numberOfPredicateTypes int
	 */
	public PredicateOrderingManager(int numberOfPredicateTypes) {

		super();
		attribute = new PredicateManager("attribute");
		window = new WindowPredicateManager("window");
		join = new JoinPredicateManager("join");
		mostSelectivePredicateTypesInOrder = new String[numberOfPredicateTypes];
		outputQDE = new QueryDependentEncoding();
		//the owner of this queue is this predicate ordering manager
		intermediateBuffer = new ArrayList();
		ArrayList registerOwner = new ArrayList();

	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 9:49:23 PM)
	 * @return java.util.ArrayList
	 * @param tuples java.util.Iterator
	 */
	public ArrayList applyNthSelectivePredicate(
		Iterator tuplesIt,
		int NthSelectivePredicate) {
		ArrayList intermediateTuples = new ArrayList();

		while (tuplesIt.hasNext()) {
			JugglerTupleList tuple = (JugglerTupleList) tuplesIt.next();
			//apply most selective predicate types in order	
			String currentPredType =
				mostSelectivePredicateTypesInOrder[NthSelectivePredicate];
			PredicateManager selectivePredManager = new PredicateManager();
			try {
				selectivePredManager = this.getPredManager(currentPredType);
			} catch (Exception e) {
				System.err.println();
			}
			selectivePredManager.applyPredicates(tuple, null);
			//add the tuples to our list
			intermediateTuples.add(tuple);
		}
		return intermediateTuples;
	}
	/**
	 * Enter tuples int the predicate exploitation structures.
	 * Also applies the first most selective predicate type.
	 * Returns the tuples with the predicate BitSet applied.
	 * Creation date: (1/12/2003 9:03:30 PM)
	 * @param tuples ArrayList
	 */
	public void enterTuples(ArrayList tuples) {

		for (int i = 0; i < tuples.size(); i++) {
			JugglerTupleList jlist = (JugglerTupleList) tuples.get(i);
			//create tuple's predicate BitSets
			ExtPredicateBitSet pbs = this.createExtPredicateBitSet();
			jlist.getTuple().setPredicates(pbs);
			//set all the attribute predicates as relevant
			setAllListsAsRelevant(jlist);
		}

		tuples = setupAndApplyAttrPredicates(tuples);
		attribute.enterTuples((ArrayList) tuples.clone());
		window.enterTuples((ArrayList) tuples.clone());
		join.enterTuples((ArrayList) tuples.clone());
	}

	/**
	 * Enter tuples int the predicate exploitation structures.
	 * Also applies the first most selective predicate type.
	 * Returns the tuples with the predicate BitSet applied.
	 * Creation date: (1/12/2003 9:03:30 PM)
	 * @param tuples ArrayList
	 */
	public void enterTuplesOptimized(ArrayList tuples) {

		for (int i = 0; i < tuples.size(); i++) {
			JugglerTupleList jlist = (JugglerTupleList) tuples.get(i);
			//create tuple's predicate BitSets
			ExtPredicateBitSet pbs = this.createExtPredicateBitSet();
			jlist.getTuple().setPredicates(pbs);
			//set all the attribute predicates as relevant
			setAllListsAsRelevant(jlist);
		}

		tuples = setupAndApplyAttrPredicates(tuples);
		//Iterator tuplesIt = tuples.iterator();
		//while (tuplesIt.hasNext()) {
		for (int i = 0; i < tuples.size(); i++) {
			//	JugglerTupleList element = (JugglerTupleList) tuplesIt.next();
			JugglerTupleList element = (JugglerTupleList) tuples.get(i);
			//enter into each predicate managers.
			window.enterTuple(element);
			join.enterTuple(element);
		}
		int j = 0;
		Iterator iter =
			WindowPredicateManager.getWindowRemoveBuckets().iterator();
		while (iter.hasNext()) {
			int winExpStrID = ((Integer) iter.next()).intValue();
			WindowExpStructureList winExpStr =
				(WindowExpStructureList) window.getExpStructures().get(
					winExpStrID);
			Iterator iter2 = winExpStr.getRemoveBucketList().iterator();
			while (iter2.hasNext()) {
				int bucketID = ((Integer) iter2.next()).intValue();
				WindowExpStructureList winList =
					winExpStr.getWindowExpStrList_With_BucketID(bucketID);
				JugglerWindowTupleList temp =
					winList.getThisWinExpStr().getListOfTuples();
				while (temp != null) {
					join.removeTupleList(temp.getJList());
					temp = temp.getNextWinTL();
				}
				winExpStr.removeBucket(bucketID);
			}
			winExpStr.getRemoveBucketList().clear();
		}
	}

	/**
	 * Returns the bitposition of the predicate.
	 * Creation date: (1/12/2003 9:06:24 PM)
	 * @return juggler.predicatestructure.BitPosition
	 * @param predicate juggler.predicatestructure.Predicate
	 */
	public BitPosition getAttrBitPosition(Predicate predicate) {
		return this.getAttributePredicateManager().getBitPosition(predicate);
	}
	/**
	 * Returns the attribute predicate manager.
	 * Creation date: (1/12/2003 9:01:51 PM)
	 * @return juggler.predicatemanagers.PredicateManager
	 */
	public PredicateManager getAttributePredicateManager() {
		return attribute;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 6:15:05 PM)
	 * @return juggler.buffer.IntermediateQueueImp
	 */
	public ArrayList getBuffer() {
		return intermediateBuffer;
	}
	/**
	 * Returns the bitposition of the predicate.
	 * Creation date: (1/12/2003 9:05:50 PM)
	 * @return juggler.predicatestructure.BitPosition
	 * @param predicate juggler.predicatestructure.Predicate
	 */
	public BitPosition getJoinBitPosition(Predicate predicate) {
		return this.getJoinPredicateManager().getBitPosition(predicate);
	}
	/**
	 * Returns the join predicate manager.
	 * Creation date: (1/12/2003 9:00:55 PM)
	 * @return juggler.predicatemanagers.JoinPredicateManager
	 */
	public JoinPredicateManager getJoinPredicateManager() {
		return join;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 7:32:09 PM)
	 * @return juggler.predicatemanagers.PredicateManager
	 * @param predicateTypeName java.lang.String
	 * @exception Exception There is no Predicate Type Manager registered.
	 */
	public PredicateManager getPredManager(String predicateTypeName)
		throws Exception {

		if (attribute.getTypeName() == predicateTypeName)
			return attribute;
		if (join.getTypeName() == predicateTypeName)
			return join;
		if (window.getTypeName() == predicateTypeName)
			return window;
		else {
			throw new Exception("There is no Predicate Type Manager registered");
		}
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (3/7/2003 10:20:53 PM)
	 * @return juggler.query.QueryDependentEncoding
	 */
	public QueryDependentEncoding getQDE() {
		return this.outputQDE;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 6:39:25 PM)
	 * @return java.lang.String
	 */
	public java.lang.String[] getSelectivePredicateType() {
		String[] selectivePredList =
			new String[this.mostSelectivePredicateTypesInOrder.length];

		//right now hard code to do attr, join and window
		selectivePredList[0] = attribute.getTypeName();
		selectivePredList[1] = join.getTypeName();
		selectivePredList[2] = window.getTypeName();

		return selectivePredList;
	}
	/**
	 * Returns the bitposition of the predicate.
	 * Creation date: (1/12/2003 9:05:17 PM)
	 * @return juggler.predicatestructure.BitPosition
	 * @param predicate juggler.predicatestructure.Predicate
	 */
	public BitPosition getWinBitPosition(Predicate predicate) {
		return this.getWindowPredicateManager().getBitPosition(predicate);
	}
	/**
	 * Returns the window predicate manager.
	 * Creation date: (1/12/2003 9:02:25 PM)
	 * @return juggler.predicatemanagers.WindowPredicateManager
	 */
	public WindowPredicateManager getWindowPredicateManager() {
		return window;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (4/15/2003 12:30:33 PM)
	 * @param joinedTuple java.util.Iterator
	 * @return Iterator
	 */
	public Iterator joinedTupleFormatCheck(Iterator unjoinedTuple) {
		ArrayList tuplesToTupleList = new ArrayList();
		while (unjoinedTuple.hasNext()) {
			JugglerTupleList aTupleList =
				(JugglerTupleList) unjoinedTuple.next();
			//JugglerTupleList aTupleList = new JugglerTupleList(aTuple);
			tuplesToTupleList.add(aTupleList);
		}

		unjoinedTuple = tuplesToTupleList.iterator();
		return unjoinedTuple;

	}

	/**
	 * joins the tuples using all the relevant predicates.
	 * This one applies all the predicates one tuple at a time.
	 * It is hard to find the selectivity for the first tuple.
	 * Creation date: (1/12/2003 9:03:56 PM)
	 * @param tuples java.util.Iterator
	 * @return ArrayList
	 */
	public ArrayList processTuples(java.util.Iterator tuples) {

		//create tuple's predicate BitSets
		ExtPredicateBitSet pbs = this.createExtPredicateBitSet();

		//make sure the joinedTuples are in the JugglerTupleList format, since this is the way the joined tuples are stored in the intermediate buffer
		tuples = joinedTupleFormatCheck(tuples);

		//this identifies if the attribute predicates need to be combined, this is needed if attr predicate is first or later
		boolean needToCombineAttributes = false;
		boolean combinedAttr = false;
		boolean anyResultingTuples = false;

		while (tuples.hasNext()) {
			JugglerTupleList tupleList = (JugglerTupleList) tuples.next();
			tupleList.getFirstTuple().getQuerySatisfied().clear();
			JugglerTupleList firstParttuple =
				(JugglerTupleList) tupleList.getFirstHalfJoinedTuple();
			firstParttuple.getFirstTuple().setPredicates(
				(ExtPredicateBitSet) pbs.getClone());
			for (int i = 0;
				i < mostSelectivePredicateTypesInOrder.length;
				i++) {

				//apply most selective predicate types in order	
				String currentPredType = mostSelectivePredicateTypesInOrder[i];
				PredicateManager selectivePredManager = new PredicateManager();
				int numInputTuplesToPredicateType = 0;
				try {
					selectivePredManager = this.getPredManager(currentPredType);
				} catch (Exception e) {
					System.err.println();
				}

				//apply predicates should take advantage of the irrelevant set bits
				Iterator joinedTuples = null;
				if (!hash.isEmpty()) {
					numInputTuplesToPredicateType = intermediateBuffer.size();

					joinedTuples =
						selectivePredManager.applyPredicates(
							firstParttuple,
							intermediateBuffer.iterator());
					intermediateBuffer.clear();
					hash.clear();

				} else {
					//if window or join predicate type is first, then set all the predicate lists as relevant.
					if (i == 0)
						setAllListsAsRelevant(
							selectivePredManager,
							firstParttuple);

					joinedTuples =
						selectivePredManager.applyPredicates(
							firstParttuple,
							null);

					numInputTuplesToPredicateType = 1;

					if (currentPredType == "attribute") {
						needToCombineAttributes = true;
					}
				}

				int numOFJoinedTuples_predicateOutput = 0;

				//if the selective PredManager was either join or window, therefore the tuples are joined and returned
				//as iterators
				if (joinedTuples != null
					&& !((i == 0) && (currentPredType.equals("attribute")))) {

					//identifies if teh attributes have been combined
					while (joinedTuples.hasNext()) {
						JugglerTupleList secondTuple =
							(JugglerTupleList) joinedTuples.next();

						boolean isRelevantTuple = true;

						//if attribute predicates need to be combined 
						if (needToCombineAttributes) {

							isRelevantTuple =
								processTupleIfRelevant(
									firstParttuple,
									secondTuple,
									"attribute",
									needToCombineAttributes);
							if (!isRelevantTuple)
								continue;
							combinedAttr = true;
						}

						//want to only set the relevan queries if the attributes have not been combined
						//only want to do this for the first predicate type
						if (i == 0) {

							firstParttuple.getFirstTuple().setPredicates(
								(ExtPredicateBitSet) pbs.getClone());
							isRelevantTuple =
								processTupleIfRelevant(
									firstParttuple,
									secondTuple,
									currentPredType,
									combinedAttr);
							if (!isRelevantTuple)
								continue;
						} else { //if its the last or next to last predicate type.. find what queries have been satisfied.
							//changed from firstTuple!!!!!!!!!

							ArrayList updatedQueryIDs =
								findSatisfiedQueries(
									firstParttuple,
									secondTuple,
									currentPredType);
							if (secondTuple != null)
								secondTuple.getFirstTuple().setQuerySatisfied(
									updatedQueryIDs);
							if ((i < 2) && (updatedQueryIDs.size() > 0))
								setTupleIrrelevantBits(
									secondTuple,
									currentPredType);

						}
						//add the tuples to our list
						//intermediateTuples.add(tempJoinedTuple);
						//we still have to set the irrelevant query bit sets for the last two predicate types
						//then we have to use the irrelevant bitsets
						//if the joined tuple has satisfied any queries.....
						if (secondTuple
							.getFirstTuple()
							.getQuerySatisfied()
							.size()
							> 0) {
							numOFJoinedTuples_predicateOutput++;
							intermediateBuffer.add(secondTuple);
							//add it into our hash??
							addIntermeidateTuplesToHash(secondTuple);
						}
					}
					//this join or window should result in some resulting tuples
					anyResultingTuples = true;
				}
				//if the selective PredMananger was only attribute, then it does not return an iterator
				else {
					//we still have to set the irrelevant query bit sets for the last two predicate types
					//then we have to use the irrelevant bitsets
					setTupleIrrelevantBits(firstParttuple, currentPredType);
					//add the tuples to our list
					//intermediateTuples.add(firstParttuple);

					numOFJoinedTuples_predicateOutput = 1;

				}
				//if combined the attribute predicates, no need to combine them again
				if (combinedAttr)
					needToCombineAttributes = false;
				//if there are no resulting tuples, then do not apply any more predicates
				if (anyResultingTuples && hash.isEmpty())
					return null;
				//what if no predicates passed, and it needs to be joined by all tuples..
				//this doesn't work for no streams relevant in a predicate

				//update selectivity for the predicate type
				selectivePredManager.calculateSelectivity(
					numInputTuplesToPredicateType,
					numOFJoinedTuples_predicateOutput);

			} //for each predicate type
			//dequeue old joined tuples?????...........
			//intermediateBuffer.remove(firstParttuple);
			try {
				firstParttuple.getTuple().setPredicates(null);
				firstParttuple.updatePredicateBitSetsInTupleList();
				return (joinOutputTuples_ToQueue(firstParttuple));
			} catch (Exception e) {
				e.printStackTrace();
			}

		} //for each tuple

		//return intermediateTuples;
		//dont' need to return do we?

		return null;

	}

	/**
	 * @param tuples
	 * @param pbs
	 * @return
	 */
	private Iterator setTuplePredicateFormat(
		Iterator tuples,
		ExtPredicateBitSet pbs) {
		ArrayList tuplesWithPredicateBS = new ArrayList();
		while (tuples.hasNext()) {
			JugglerTupleList firstParttuple = (JugglerTupleList) tuples.next();
			ExtPredicateBitSet tuplePredBS =
				(ExtPredicateBitSet) pbs.getClone();
			firstParttuple.getFirstTuple().setPredicates(tuplePredBS);
			tuplesWithPredicateBS.add(firstParttuple);
		}
		return tuplesWithPredicateBS.iterator();
	}
	/**
	 * 
	 * @param firstPart
	 * @return
	 * @throws Exception
	 */
	private ArrayList joinOutputTuples_ToQueue(JugglerTupleList firstPart)
		throws Exception {
		ArrayList outputQueue = new ArrayList();
		ArrayList queryNumbers = outputQDE.getQueryNums();
		Iterator queryNums = queryNumbers.iterator();
		boolean shouldCont = true;
		Hashtable secondaryHash;
		while (queryNums.hasNext()) {
			ArrayList tempOutputQueue = new ArrayList();
			//check to see if the hash contains anything of the query number
			String queryNum = (String) queryNums.next();
			if (hash.containsKey(queryNum)) {
				//retrieve the entry for the query number
				secondaryHash = (Hashtable) hash.get(queryNum);
				ArrayList tempQueryStreamNames =
					((QueryQED) outputQDE
						.getQueryQED((new Integer(queryNum)).intValue()))
						.getStreams();
				//now check to see if the entry has all the streams for the query
				//first remove the first part stream names
				ArrayList queryStreamNames =
					(ArrayList) DeepCopyMaker.makeDeepCopy(
						tempQueryStreamNames);
				queryStreamNames.removeAll(firstPart.getStreamNames());
				Enumeration secondaryHashKeySet = secondaryHash.keys();
				//now check to see if the see if the second part has all the remaining streams
				//for the query
				while (secondaryHashKeySet.hasMoreElements()) {
					ArrayList secondary_StreamNames =
						(ArrayList) secondaryHashKeySet.nextElement();
					queryStreamNames.removeAll(secondary_StreamNames);
				} // end while
				if (!queryStreamNames.isEmpty()) {
					//if the resulting tuples did not contain all the streams
					// then break... no use in combining
					//secondaryHash.clear();
					//hash.remove(queryNum);
					//break;
					ArrayList findAllTheseStreamNames = queryStreamNames;
					for (int j = 0; j < queryStreamNames.size(); j++) {
						String currStreamName =
							(String) queryStreamNames.get(j);
						JugglerWindowTupleList joinTheseTuples =
							new JugglerWindowTupleList();
						ArrayList currStreamNameList = new ArrayList();
						currStreamNameList.add(currStreamName);
						for (int i = 0;
							i < window.getExpStructures().size();
							i++) {
							WindowExpStructureList winSL =
								(WindowExpStructureList) window
									.getExpStructures()
									.get(
									i);
							ArrayList winStreamNameList =
								winSL.getStreamNames();
							if (winStreamNameList
								.containsAll(currStreamNameList)) {
								joinTheseTuples.appendTupleList(
									winSL.getListOfTuples());
								findAllTheseStreamNames.remove(currStreamName);
							} //end of if the win Exp Str contains the current stream name
						} //for each window exp structure
						secondaryHash.put(currStreamNameList, joinTheseTuples);
					}

					//if didn't find all the tuples for this query, then remove from hash
					if (!findAllTheseStreamNames.isEmpty()) {
						secondaryHash.clear();
						hash.remove(queryNum);
						break;
					} else {

					}
				}

				///////////////////////////////////////

				//now start joining for each stream entry for this query
				secondaryHashKeySet = secondaryHash.keys();

				// for each query stream Name
				while (secondaryHashKeySet.hasMoreElements()) {
					ArrayList secondary_StreamNames =
						(ArrayList) secondaryHashKeySet.nextElement();
					JugglerWindowTupleList jWTL =
						(JugglerWindowTupleList) secondaryHash.get(
							secondary_StreamNames);

					JugglerTupleList firstHalf;
					ArrayList tempTempOutputQueue = new ArrayList();
					//join each tuple list for a streamName
					while (jWTL != null) {
						if (tempOutputQueue.isEmpty())
							tempOutputQueue.add(
								firstPart.getJugglerTupleListClone());
						JugglerTupleList tempSecondPart =
							((JugglerTupleList) jWTL.getJList())
								.getJugglerTupleListClone();
						Iterator tempfirstPart_TupleLists =
							tempOutputQueue.iterator();

						ArrayList tempJoinedTuples =
							joinFinalTuples(
								tempfirstPart_TupleLists,
								tempSecondPart,
								queryNum);

						tempTempOutputQueue.addAll(tempJoinedTuples);

						jWTL = jWTL.getNextWinTL();
					} //end while jWTL.getNext
					tempOutputQueue.clear();
					//	add all partial joined tuples to tempOutputQueue for the next stream in the line
					tempOutputQueue.addAll(tempTempOutputQueue);
					//clear the tempTempOutput Queue for the next iteration for the next stream
					tempTempOutputQueue.clear();
				} //end while queryStreamNames

				outputQueue.add(tempOutputQueue);

			} // end if contains query Num
		} //end while query Numbers

		return outputQueue;

	}

	/**
	 * tempfirstPart_TupleList is only for the first tuple data values, the predicate bitsets and
	 * query satisfied are stored and used from the second part of the tuple.
	 * @param tempfirstPart_TupleLists
	 * @param tempSecondPart
	 * @param queryNum
	 * @return
	 */
	private ArrayList joinFinalTuples(
		Iterator tempfirstPart_TupleLists,
		JugglerTupleList tempSecondPart,
		String queryNum) {
		ArrayList tempTempOutputQueue = new ArrayList();
		while (tempfirstPart_TupleLists.hasNext()) {
			JugglerTupleList tempfirstPart =
				((JugglerTupleList) tempfirstPart_TupleLists.next())
					.getJugglerTupleListClone();

			//if its the first time joining, the second part's predicates are relevant
			//if its joing tuples that are not involved in any predicates, then the second part pre
			//dicates are relevant
			//if this is joining relevant other tuples, combine the predicates
			if (tempfirstPart.getFirstTuple().getPredicates() == null)
				tempfirstPart.getFirstTuple().setPredicates(
					tempSecondPart.getFirstTuple().getPredicates());
			else {
				if (tempSecondPart.getFirstTuple().getPredicates() == null)
					tempSecondPart.getFirstTuple().setPredicates(
						tempfirstPart.getFirstTuple().getPredicates());
				else
					tempfirstPart
						.getFirstTuple()
						.getPredicates()
						.combineBitSets(
						tempSecondPart.getFirstTuple().getPredicates());
			}

			ArrayList querySatisfied = new ArrayList();
			querySatisfied.add(queryNum);
			tempfirstPart.getFirstTuple().setQuerySatisfied(querySatisfied);
			//tempfirstPart.getFirstTuple().setQuerySatisfied(
			//	tempSecondPart
			//		.getFirstTuple()
			//		.getQuerySatisfied());
			tempfirstPart.appendTupleList(tempSecondPart);
			tempfirstPart.updatePredicateBitSetsInTupleList();
			tempfirstPart.setJoinTupleMarkerAfterJoining();
			XATTimeStamp ts = new XATTimeStampImp();

			try {
				XATTimeStamp firstTS =
					tempfirstPart.getFirstTuple().getTimeStamp();
				XATTimeStamp secondTS =
					tempSecondPart.getFirstTuple().getTimeStamp();
				ts.combine(firstTS, secondTS);
				//NOTE: sets the timestamp of tempfirstPart JugglerTupleLIst
				//before or one level above the tuple listing...
				tempfirstPart.setTimeStamp(ts);
			} catch (XATTimeStampException xse) {
				System.out.println(xse.getMessage());
			}

			//add partial joined tuples to tempTempOutputQueue
			tempTempOutputQueue.add(tempfirstPart);
		} //end while first part tupleLists
		return tempTempOutputQueue;
	}

	private void addIntermeidateTuplesToHash(JugglerTupleList secondPart) {
		ArrayList sec_streamNames = secondPart.getStreamNames();
		ArrayList satisfiedQueryIDs =
			secondPart.getFirstTuple().getQuerySatisfied();
		Iterator queryIDs = satisfiedQueryIDs.iterator();
		Hashtable secondaryHash = new Hashtable();
		while (queryIDs.hasNext()) {
			Integer queryID = (Integer) queryIDs.next();

			//if there is an entry for that query ID
			if (hash.containsKey(queryID.toString()))
				secondaryHash = (Hashtable) hash.get(queryID.toString());
			//if there is an entry for the streamNames
			if (secondaryHash.containsKey(sec_streamNames)) {
				JugglerWindowTupleList listOfJoinableTuples =
					(JugglerWindowTupleList) secondaryHash.get(sec_streamNames);
				listOfJoinableTuples.appendTupleList_IfNotContained(
					secondPart.getJugglerTupleListClone());
				secondaryHash.put(sec_streamNames, listOfJoinableTuples);
			}
			//if there isn't an entry for the streamNames
			else {
				secondaryHash.put(
					sec_streamNames,
					new JugglerWindowTupleList(secondPart, true));
			}

			hash.put(queryID.toString(), secondaryHash);

		}
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (3/16/2003 10:23:28 AM)
	 * @return java.lang.String
	 */
	public String queryToString() {
		String queryString = "";

		for (int i = 0; i < this.outputQDE.getQueryQEDs().size(); i++) {

			QueryQED queryQED = (QueryQED) this.outputQDE.getQueryQEDs().get(i);
			queryString += "Query: " + queryQED.getQueryNum() + "\n";
			ArrayList attrbitpos =
				queryQED
					.getPredicates()
					.getAppropriatePredicateTypeBitSet("attribute")
					.getSetBitIndexes();
			for (int j = 0; j < attrbitpos.size(); j++) {
				BitPosition bitP = (BitPosition) attrbitpos.get(j);
				//if the list or predicate is not relevant to this query
				BitSet attrRelBS =
					(BitSet) queryQED
						.getPredicates()
						.getIsEvaluated_Relevant("attribute")
						.getPredicateBitSets()[bitP
						.getPredListIndex()];
				if (attrRelBS.get(bitP.getPredIndex())) {
					AttrPredicate attrPred =
						(AttrPredicate) this.attribute.getPredicate(bitP);
					queryString += attrPred.toString() + "\n";
				}
			}
			ArrayList joinbitpos =
				queryQED
					.getPredicates()
					.getAppropriatePredicateTypeBitSet("join")
					.getSetBitIndexes();
			for (int j = 0; j < joinbitpos.size(); j++) {
				BitPosition bitP = (BitPosition) joinbitpos.get(j);
				//if the list or predicate is not relevant to this query
				BitSet joinRelBS =
					(BitSet) queryQED
						.getPredicates()
						.getIsEvaluated_Relevant("join")
						.getPredicateBitSets()[bitP
						.getPredListIndex()];
				if (joinRelBS.get(bitP.getPredIndex())) {
					Predicate joinPred =
						(Predicate) this.join.getPredicate(bitP);
					queryString += joinPred.toString() + "\n";
				}
			}
			ArrayList winbitpos =
				queryQED
					.getPredicates()
					.getAppropriatePredicateTypeBitSet("window")
					.getSetBitIndexes();
			for (int k = 0; k < winbitpos.size(); k++) {
				BitPosition bitP = (BitPosition) winbitpos.get(k);
				//if the list or predicate is not relevant to this query
				BitSet windowRelBS =
					(BitSet) queryQED
						.getPredicates()
						.getIsEvaluated_Relevant("window")
						.getPredicateBitSets()[bitP
						.getPredListIndex()];
				if (windowRelBS.get(bitP.getPredIndex())) {
					Predicate winPred =
						(Predicate) this.window.getPredicate(bitP);
					queryString += winPred.toString() + "\n";
				}
			}
		}

		return queryString;
	}
	/**
	 * * Registers the predicates of the queries and sets teh iput Query Dependent Encoding.
	 * Parameter is an arrayList of queries.
	 * Creation date: (1/12/2003 9:06:59 PM)
	 * @param query juggler.query.Query[]
	 */
	public void registerQueries(ArrayList queries) {

		//registers the queries and sets the queryDependentEncoding

		if (attribute != null)
			attribute.registerQueries(queries);
		if (join != null)
			join.registerQueries(queries);
		if (window != null)
			window.registerQueries(queries);

		//get the QED ready for the predicate bitsets to be set in the setupQDE function
		QueryQED queryQED = new QueryQED();
		setupBitSetInQDE(queryQED);
		//sets up all the predicate bit sets set to zero

		for (int i = 0; i < queries.size(); i++) {
			Query queryTemp = (Query) queries.get(i);
			//set query ID, it starts at its index
			QueryQED q = new QueryQED();
			queryQED.setQueryNum(queryTemp.getQueryNum());
			queryQED.setStreams(queryTemp.getStreams());
			q = (QueryQED) queryQED.getClone();
			this.getQDE().getQueryQEDs().add(i, q);
			//add the queryID to the list for this operator....
		}

		//set teh QDE as the size of the queries???
		this.setupQDE();
		//this.setIrrelvantBitsInQDE(); 
		this.setupExploitationStructures();
		// this.
	}
	/**
	 * Sets the attribute predicate manager.
	 * Creation date: (1/12/2003 9:01:51 PM)
	 * @param newAttributePredicateManager juggler.predicatemanagers.PredicateManager
	 */
	public void setAttributePredicateManager(PredicateManager newAttributePredicateManager) {
		attribute = newAttributePredicateManager;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 6:15:05 PM)
	 * @param newBuffer juggler.buffer.IntermediateQueueImp
	 */
	public void setBuffer(ArrayList newBuffer) {
		intermediateBuffer = newBuffer;
	}
	/**
	 * Sets the join predicate manager.
	 * Creation date: (1/12/2003 9:00:55 PM)
	 * @param newJoin juggler.predicatemanagers.JoinPredicateManager
	 */
	public void setJoinPredicateManager(JoinPredicateManager newJoin) {
		join = newJoin;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (3/7/2003 10:20:53 PM)
	 * @param newQDE juggler.exploitationstructures.QueryDependentEncoding
	 */
	public void setQDE(QueryDependentEncoding newQDE) {
		outputQDE = newQDE;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (2/26/2003 6:39:25 PM)
	 * @param newSelectivePredicateType java.lang.String
	 */
	public void setSelectivePredicateType(
		java.lang.String[] newSelectivePredicateType) {
		mostSelectivePredicateTypesInOrder = newSelectivePredicateType;
	}
	/**
	 * This sets teh bitsets by traversing all the predicate lists.
	 * Creation date: (3/10/2003 12:34:29 PM)
	 */
	public void setupBitSetInQDE(QueryQED q) {
		int attrListSize = attribute.getPredicateList().getOrderLists().size();
		int joinListSize = join.getPredicateOrderLists().size();
		int windListSize = window.getPredicateOrderLists().size();

		BitSet[] attributeRelList = new BitSet[attrListSize];
		BitSet[] attributePredList = new BitSet[attrListSize];
		BitSet[] joinRelList = new BitSet[joinListSize];
		BitSet[] joinPredList = new BitSet[joinListSize];
		BitSet[] windowRelList = new BitSet[windListSize];
		BitSet[] windowPredList = new BitSet[windListSize];

		//set queryQED's bitset sizes
		q
			.getPredicates()
			.getAppropriatePredicateTypeExt("attribute")
			.getIsEvaluated_Relevant()
			.setPredicateBitSets(new BitSet[attrListSize]);

		q
			.getPredicates()
			.getAppropriatePredicateTypeExt("attribute")
			.getPredBSList()
			.setPredicateBitSets(new BitSet[attrListSize]);

		q
			.getPredicates()
			.getAppropriatePredicateTypeExt("join")
			.getIsEvaluated_Relevant()
			.setPredicateBitSets(new BitSet[joinListSize]);
		q
			.getPredicates()
			.getAppropriatePredicateTypeExt("join")
			.getPredBSList()
			.setPredicateBitSets(new BitSet[joinListSize]);

		q
			.getPredicates()
			.getAppropriatePredicateTypeExt("window")
			.getIsEvaluated_Relevant()
			.setPredicateBitSets(new BitSet[windListSize]);
		q
			.getPredicates()
			.getAppropriatePredicateTypeExt("window")
			.getPredBSList()
			.setPredicateBitSets(new BitSet[windListSize]);

		//set up attribute bitsets
		for (int i = 0; i < attrListSize; i++) {
			//;make sure the attrList has the same list index as the i
			AttrPredOrderList attrList =
				(AttrPredOrderList) this
					.attribute
					.getPredicateList()
					.getOrderLists()
					.get(
					i);
			if (attrList.getListIndex() == i) {
				BitSet bs = new BitSet(attrList.size());
				BitSet relBS = new BitSet(attrList.size());
				q
					.getPredicates()
					.getAppropriatePredicateTypeExt("attribute")
					.getIsEvaluated_Relevant()
					.predicateBitSets[i] =
					bs;
				q
					.getPredicates()
					.getAppropriatePredicateTypeExt("attribute")
					.getPredBSList()
					.predicateBitSets[i] =
					(BitSet) bs.clone();

			}

		}

		//set up join bitsets
		for (int i = 0; i < joinListSize; i++) {
			//;make sure thejoinList has the same list index as the i
			OrderList joinList =
				(OrderList) this
					.join
					.getPredicateOrderLists()
					.getOrderLists()
					.get(
					i);
			if (joinList.getListIndex() == i) {
				BitSet bs = new BitSet(joinList.size());
				BitSet relBS = new BitSet(joinList.size());
				q
					.getPredicates()
					.getAppropriatePredicateTypeExt("join")
					.getIsEvaluated_Relevant()
					.predicateBitSets[i] =
					bs;
				q
					.getPredicates()
					.getAppropriatePredicateTypeExt("join")
					.getPredBSList()
					.predicateBitSets[i] =
					(BitSet) bs.clone();
			}

		}

		//set up window bitsets
		for (int i = 0; i < windListSize; i++) {
			//;make sure thejoinList has the same list index as the i
			OrderList winList =
				(OrderList) this
					.window
					.getPredicateOrderLists()
					.getOrderLists()
					.get(
					i);
			if (winList.getListIndex() == i) {
				BitSet bs = new BitSet(winList.size());
				BitSet relBS = new BitSet(winList.size());
				q
					.getPredicates()
					.getAppropriatePredicateTypeExt("window")
					.getIsEvaluated_Relevant()
					.getPredicateBitSets()[i] =
					bs;
				q
					.getPredicates()
					.getAppropriatePredicateTypeExt("window")
					.getPredBSList()
					.getPredicateBitSets()[i] =
					(BitSet) bs.clone();
			}

		}

	}
	/**
	 * Insert the method's description here.
	 * Creation date: (3/28/2003 12:33:04 AM)
	 */
	public void setupExploitationStructures() {
		setupJoinExploitationStructures();
		setupWindowExploitationStructures();
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (3/28/2003 12:34:37 AM)
	 */
	public void setupJoinExploitationStructures() {
		for (int i = 0;
			i < this.getJoinPredicateManager().getPredicateOrderLists().size();
			i++) {
			JWPredOrderList currOL =
				(JWPredOrderList) this
					.getJoinPredicateManager()
					.getPredicateOrderLists()
					.getOrderLists()
					.get(i);
			JoinExpTreeStructure joinExpStr1 = new JoinExpTreeStructure();
			joinExpStr1.setColumn(currOL.getColumn1());
			joinExpStr1.setColumnInt(currOL.getColumnInt1());
			ArrayList streamName1 = new ArrayList();
			streamName1 = currOL.getStreamName1();
			joinExpStr1.setStreamNames(streamName1);
			//joinExpStr.setStreamIDs(currOL.get);
			if (!this
				.getJoinPredicateManager()
				.containsJoinExpTreeStr(joinExpStr1))
				this.getJoinPredicateManager().getJoinTreeStructures().add(
					joinExpStr1);
			JoinExpTreeStructure joinExpStr2 = new JoinExpTreeStructure();
			ArrayList streamName2 = currOL.getStreamName2();
			joinExpStr2.setColumn(currOL.getColumn2());
			joinExpStr2.setStreamNames(streamName2);
			joinExpStr2.setColumnInt(currOL.getColumnInt2());
			//joinExpStr.setStreamIDs(currOL.get);
			if (!this
				.getJoinPredicateManager()
				.containsJoinExpTreeStr(joinExpStr2))
				this.getJoinPredicateManager().getJoinTreeStructures().add(
					joinExpStr2);

		}

	}
	/**
	 * This is called after the queries are registered and the its predicates are inserted into the linked lists.
	 * Creation date: (3/10/2003 9:42:32 PM)
	 */
	public void setupQDE() {

		//register attribute predicates
		AttrPredOrderListS attrLists = attribute.getPredicateList();

		for (int i = 0; i < attribute.getPredicateList().size(); i++) {
			AttrPredOrderList attrList =
				(AttrPredOrderList) attribute
					.getPredicateList()
					.getOrderLists()
					.get(
					i);
			//set list index
			attrList.setListIndex(i);

			//the qed's bitsets size must be set
			//BitSet bs  = new BitSet(attrList.size());
			//setupBitSetInQDE(type, i, bs);

			for (int j = 0; j < attrList.size(); j++) {
				BitPosition bp = new BitPosition(i, j);
				AttrPredicate pred = (AttrPredicate) attrList.getPredicate(j);
				//set the predicate bit positions for the predicate
				pred.setBitposition(bp);
				//set it in the ouput queryQDE's
				this.getQDE().registerPredicates(pred);
			}
		}

		//register join predicates
		OrderListS joinLists = join.getPredicateOrderLists();

		for (int k = 0; k < join.getPredicateOrderLists().size(); k++) {
			OrderList joinList =
				(OrderList) join.getPredicateOrderLists().getOrderLists().get(
					k);
			joinList.setListIndex(k);
			for (int q = 0; q < joinList.size(); q++) {
				BitPosition bpJoin = new BitPosition(k, q);
				Predicate predJoin = (Predicate) joinList.getPredicate(q);
				//set the predicate bit positions for the predicate
				predJoin.setBit(bpJoin);
				//set it in the ouput queryQDE's
				this.getQDE().registerPredicates(predJoin);
			}
		}

		//register window predicates
		OrderListS winLists = window.getPredicateOrderLists();

		for (int r = 0; r < winLists.getOrderLists().size(); r++) {
			OrderList winList =
				(OrderList) window
					.getPredicateOrderLists()
					.getOrderLists()
					.get(
					r);
			winList.setListIndex(r);
			for (int s = 0; s < winList.size(); s++) {
				BitPosition bpWin = new BitPosition(r, s);
				Predicate predWin = (Predicate) winList.getPredicate(s);
				//set the predicate bit positions for the predicate
				predWin.setBit(bpWin);
				//set it in the ouput queryQDE's
				this.getQDE().registerPredicates(predWin);
			}
		}

	}
	/**
	 * Insert the method's description here.
	 * Creation date: (3/28/2003 12:34:37 AM)
	 */
	public void setupWindowExploitationStructures() {

		for (int i = 0;
			i
				< this
					.getWindowPredicateManager()
					.getPredicateOrderLists()
					.size();
			i++) {
			JWPredOrderList currOL =
				(JWPredOrderList) this
					.getWindowPredicateManager()
					.getPredicateOrderLists()
					.getOrderLists()
					.get(i);
			//assume time increment is 30..and maxwindow of structure is 60,  for now.. need ot pass in as a parameter
			//time increment, arraylist of streamnames, arrayList of new streamIDs, maxwindow of this structure
			ArrayList streamNames1 = new ArrayList();
			streamNames1 = currOL.getStreamName1();

			WindowExpStructureList winExpStr1 =
				new WindowExpStructureList(30, streamNames1, null, 60);

			if (!this.getWindowPredicateManager().containsWinMan(winExpStr1))
				this.getWindowPredicateManager().getWindowManagers().add(
					winExpStr1);

			ArrayList streamNames2 = new ArrayList();
			streamNames2 = currOL.getStreamName2();

			WindowExpStructureList winExpStr2 =
				new WindowExpStructureList(30, streamNames2, null, 60);

			if (!this.getWindowPredicateManager().containsWinMan(winExpStr2))
				this.getWindowPredicateManager().getWindowManagers().add(
					winExpStr2);

		}
	}
	/**
	 * Sets the window predicate manager.
	 * Creation date: (1/12/2003 9:02:25 PM)
	 * @param newWindow juggler.predicatemanagers.WindowPredicateManager
	 */
	public void setWindowPredicateManager(WindowPredicateManager newWindow) {
		window = newWindow;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (3/5/2003 9:16:42 PM)
	 * @return java.lang.String
	 */
	public String toString() {

		String predicates = "";
		if (attribute != null) {
			predicates += "Attribute" + "\n";
			predicates += attribute.toString() + "\n" + "\n" + "\n";
		}
		if (join != null) {
			predicates += "Join" + "\n";
			predicates += join.toString() + "\n" + "\n" + "\n";
		}
		if (window != null) {
			predicates += "Window" + "\n";
			predicates += window.toString() + "\n" + "\n" + "\n";
		}

		return predicates;
	}
	/**
	 * Insert the method's description here.
	 * Creation date: (1/12/2003 9:04:14 PM)
	 */
	private void updateSelectivities() {

	}

	/**
	 * Creates all the Predicate bitset plus each has an extra last bitset to indicate irrelevant list to a tuple.
	 * Creation date: (4/9/2003 10:32:11 PM)
	 * @return juggler.stream.PredicateBitSet
	 */
	public ExtPredicateBitSet createExtPredicateBitSet() {

		BitSet[] attrBitSet =
			new BitSet[attribute.predicateList.getOrderLists().size()];
		BitSet[] joinBitSet =
			new BitSet[join.jwPredicateLists.getOrderLists().size()];
		BitSet[] windowBitSet =
			new BitSet[window.jwPredicateLists.getOrderLists().size()];
		BitSet[] attrRelBS =
			new BitSet[attribute.predicateList.getOrderLists().size()];
		BitSet[] joinRelBS =
			new BitSet[join.jwPredicateLists.getOrderLists().size()];
		BitSet[] winRelBS =
			new BitSet[window.jwPredicateLists.getOrderLists().size()];
		//create attibute BitSet
		for (int i = 0;
			i < this.attribute.predicateList.getOrderLists().size();
			i++) {
			AttrPredOrderList attrList =
				(AttrPredOrderList) this
					.attribute
					.predicateList
					.getOrderLists()
					.get(
					i);
			BitSet attrbs = new BitSet(attrList.size());
			BitSet relBS = new BitSet(attrList.size());
			attrBitSet[i] = attrbs;
			attrRelBS[i] = relBS;

		}

		//create join BitSet
		for (int j = 0;
			j < this.join.jwPredicateLists.getOrderLists().size();
			j++) {
			JWPredOrderList joinList =
				(JWPredOrderList) this
					.join
					.jwPredicateLists
					.getOrderLists()
					.get(
					j);
			BitSet joinbs = new BitSet(joinList.size());
			BitSet relBS = new BitSet(joinList.size());
			joinBitSet[j] = joinbs;
			joinRelBS[j] = relBS;
		}

		//create window BitSet
		for (int k = 0;
			k < this.window.jwPredicateLists.getOrderLists().size();
			k++) {
			JWPredOrderList windowList =
				(JWPredOrderList) this
					.window
					.jwPredicateLists
					.getOrderLists()
					.get(
					k);
			BitSet windowbs = new BitSet(windowList.size());
			BitSet relBS = new BitSet(windowList.size());
			windowBitSet[k] = windowbs;
			winRelBS[k] = relBS;
		}

		ArrayList predicateTypeNames = new ArrayList();
		predicateTypeNames.add("attribute");
		predicateTypeNames.add("join");
		predicateTypeNames.add("window");

		ExtPredicateBitSet predBS = new ExtPredicateBitSet(predicateTypeNames);

		ExtGeneralTypeBitSet predicateBSList =
			predBS.getAppropriatePredicateTypeExt("attribute");
		predicateBSList.getPredBSList().setPredicateBitSets(attrBitSet);
		predicateBSList.getIsEvaluated_Relevant().setPredicateBitSets(
			attrRelBS);

		predicateBSList = predBS.getAppropriatePredicateTypeExt("join");
		predicateBSList.getPredBSList().setPredicateBitSets(joinBitSet);
		predicateBSList.getIsEvaluated_Relevant().setPredicateBitSets(
			joinRelBS);

		predicateBSList = predBS.getAppropriatePredicateTypeExt("window");
		predicateBSList.getPredBSList().setPredicateBitSets(windowBitSet);
		predicateBSList.getIsEvaluated_Relevant().setPredicateBitSets(winRelBS);
		return predBS;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (3/11/2003 10:51:09 PM)
	 * @param tuple juggler.stream.Tuple
	 */
	public void findMostCoveringPredicates(JugglerTuple tuple) {

	}

	/**
	 * Insert the method's description here.
	 * Creation date: (4/9/2003 11:59:09 PM)
	 * @param t juggler.stream.Tuple
	 * @param currPredType String
	 */
	public void setTupleIrrelevantBits(JugglerTupleList t, String currPredType)
		throws IndexOutOfBoundsException {

		int indexCurrPredType;
		boolean foundCurrPredType = false;
		//how many next predicates are left?
		ArrayList nextPredTypes = new ArrayList();
		ArrayList prevPredTypes = new ArrayList();

		boolean attributeIsDone = false;
		boolean joinIsDone = false;
		boolean windowIsDone = false;
		String donePredType = "";
		String prevPredType = "";

		for (int i = 0; i < mostSelectivePredicateTypesInOrder.length; i++) {
			if (foundCurrPredType)
				nextPredTypes.add(
					(String) mostSelectivePredicateTypesInOrder[i]);
			else
				prevPredTypes.add(
					(String) mostSelectivePredicateTypesInOrder[i]);
			if ((String) mostSelectivePredicateTypesInOrder[i]
				== currPredType) {
				indexCurrPredType = i;
				foundCurrPredType = true;
			}
		}

		if (currPredType == "attribute")
			attributeIsDone = true;
		if (currPredType == "join")
			joinIsDone = true;
		if (currPredType == "window")
			windowIsDone = true;

		for (int j = 0; j < nextPredTypes.size(); j++) {
			donePredType = (String) nextPredTypes.get(j);
			if (donePredType == "attribute")
				attributeIsDone = false;
			if (donePredType == "join")
				joinIsDone = false;
			if (donePredType == "window")
				windowIsDone = false;
		}

		for (int k = 0; k < prevPredTypes.size(); k++) {
			prevPredType = (String) prevPredTypes.get(k);
			if (prevPredType == "attribute")
				attributeIsDone = true;
			if (prevPredType == "join")
				joinIsDone = true;
			if (prevPredType == "window")
				windowIsDone = true;
		}

		try {
			if (currPredType == "attribute") {
				for (int i = 0; i < nextPredTypes.size(); i++) {
					if (!joinIsDone
						&& ((String) nextPredTypes.get(i) == "join")) {
						//this.getQDE().setRelevantJoinBits(attributeIsDone, false, t);
						this.getQDE().setRelevantBits(
							attributeIsDone,
							false,
							false,
							t,
							"join");
						joinIsDone = true;
						return;
					}
					if (!windowIsDone
						&& ((String) nextPredTypes.get(i) == "window")) {
						//this.getQDE().setRelevantWindowBits(attributeIsDone, false, t);
						this.getQDE().setRelevantBits(
							attributeIsDone,
							false,
							false,
							t,
							"window");
						windowIsDone = true;
						return;
					}
				}
			}
			if (currPredType == "join") {
				for (int j = 0; j < nextPredTypes.size(); j++) {
					if (!windowIsDone
						&& ((String) nextPredTypes.get(j) == "window")) {
						//this.getQDE().setRelevantWindowBits(false, joinIsDone, t);
						this.getQDE().setRelevantBits(
							false,
							joinIsDone,
							false,
							t,
							"window");
						windowIsDone = true;
						return;
					}
					if (!attributeIsDone
						&& ((String) nextPredTypes.get(j) == "attribute")) {
						//this.getQDE().setRelevantAttrBits(joinIsDone, false, t);
						this.getQDE().setRelevantBits(
							false,
							joinIsDone,
							false,
							t,
							"attribute");
						attributeIsDone = true;
						return;
					}
				}
			}
			if (currPredType == "window") {
				for (int k = 0; k < nextPredTypes.size(); k++) {
					if (!joinIsDone
						&& ((String) nextPredTypes.get(k) == "join")) {
						//this.getQDE().setRelevantJoinBits(false, windowIsDone, t);
						this.getQDE().setRelevantBits(
							false,
							false,
							windowIsDone,
							t,
							"join");
						joinIsDone = true;
						return;
					}
					if (!attributeIsDone
						&& ((String) nextPredTypes.get(k) == "attribute")) {
						//this.getQDE().setRelevantAttrBits(false, windowIsDone, t);
						this.getQDE().setRelevantBits(
							false,
							false,
							windowIsDone,
							t,
							"attribute");
						attributeIsDone = true;
						return;
					}
				}
			}
		} catch (IndexOutOfBoundsException e) {
			System.out.println(e.toString());
			e.printStackTrace();
		}

	}

	/**
	 * Insert the method's description here.
	 * Creation date: (5/7/2003 9:15:10 PM)
	 * @param jwpredManager juggler.predicatemanagers.JWPredManager
	 * @param firstTupleList juggler.stream.TupleList
	 */
	public void setAllListsAsRelevant(
		PredicateManager predManager,
		JugglerTupleList firstTupleList) {
		String predicateType = predManager.getTypeName();
		GeneralPredicateBitSet gbs =
			(GeneralPredicateBitSet) firstTupleList
				.getTuple()
				.getPredicates()
				.getIsEvaluated_Relevant(predicateType);

		int listSize = 0;
		if (predManager.getTypeName().equals("attribute"))
			listSize = predManager.getPredicateList().size();
		else
			listSize =
				((JWPredManager) predManager).getPredicateOrderLists().size();

		for (int i = 0; i < listSize; i++) {
			OrderList list;
			if (predManager.getTypeName().equals("attribute"))
				list =
					(OrderList) predManager
						.getPredicateList()
						.getOrderLists()
						.get(
						i);
			else
				list =
					(OrderList) ((JWPredManager) predManager)
						.getPredicateOrderLists()
						.getOrderLists()
						.get(i);
			BitSet relJWList = new BitSet(list.size());
			for (int j = 0; j < list.size(); j++) {
				relJWList.set(j);
			}
			//must remove the initialized values to add the new values in
			//gbs.getPredicateBitSets().removeBitSetIndex(i);
			gbs.getPredicateBitSets()[i] = relJWList;
		}
		GeneralPredicateBitSet tupleGBS =
			(GeneralPredicateBitSet) firstTupleList
				.getTuple()
				.getPredicates()
				.getIsEvaluated_Relevant(predicateType);
		tupleGBS = gbs;

	}

	/**
	 * Insert the method's description here.
	 * Creation date: (4/9/2003 11:59:09 PM)
	 * @param t juggler.stream.Tuple
	 * @param currPredType String
	 */
	public ArrayList findSatisfiedQueries(
		JugglerTupleList first,
		JugglerTupleList second,
		String currPredType)
		throws IndexOutOfBoundsException {

		if (second.size() > 0) {
			if (!checkIfTupleIsRelevantToQueries(first, second, currPredType))
				return new ArrayList();
		}

		return this.getQDE().getSuperSetofQuery(
			second
				.getFirstTuple()
				.getPredicates()
				.getAppropriatePredicateTypeExt(currPredType)
				.getPredBSList()
				.getPredicateBitSets(),
			second
				.getFirstTuple()
				.getPredicates()
				.getAppropriatePredicateTypeExt(currPredType)
				.getIsEvaluated_Relevant()
				.getPredicateBitSets(),
			second.getFirstTuple().getQuerySatisfied(),
			currPredType);

	}

	/**
	 * Insert the method's description here.
	 * Creation date: (5/13/2003 10:34:18 PM)
	 * @param tuples java.util.ArrayList
	 */
	public ArrayList setupAndApplyAttrPredicates(ArrayList tuples) {

		Iterator it =
			this.attribute.applyAllRelevantPredicatesForInitialization(
				tuples.iterator());

		ArrayList attrPredTuples = new ArrayList();
		while (it.hasNext()) {
			JugglerTupleList tl = (JugglerTupleList) it.next();
			attrPredTuples.add(tl);
		}
		return attrPredTuples;

	}

	/**
	 *Sets all relevant lists for attribute predicates
	 * Creation date: (5/7/2003 9:15:10 PM)
	 * 
	 * @param firstTupleList juggler.stream.TupleList
	 */
	public void setAllListsAsRelevant(JugglerTupleList firstTupleList) {

		GeneralPredicateBitSet gbs =
			(GeneralPredicateBitSet) firstTupleList
				.getTuple()
				.getPredicates()
				.getIsEvaluated_Relevant("attribute");

		for (int i = 0; i < attribute.getPredicateList().size(); i++) {
			AttrPredOrderList attrList =
				(AttrPredOrderList) attribute
					.getPredicateList()
					.getOrderLists()
					.get(
					i);
			BitSet relAttrList = new BitSet(attrList.size());
			for (int j = 0; j < attrList.size(); j++) {
				relAttrList.set(j);
			}
			//must remove the initialized values to add the new values in
			//gbs.getPredicateBitSets().remove(i);
			gbs.getPredicateBitSets()[i] = relAttrList;
		}
		GeneralPredicateBitSet tupleGBS =
			(GeneralPredicateBitSet) firstTupleList
				.getTuple()
				.getPredicates()
				.getIsEvaluated_Relevant("attribute");
		tupleGBS = gbs;

	}

	/**
	 * Insert the method's description here.
	 * Creation date: (5/20/2003 1:47:47 PM)
	 * @return boolean
	 * @param firstTuple juggler.stream.JugglerTuple
	 */
	public boolean checkIfTupleIsRelevantToQueries(
		JugglerTupleList firstTuple,
		JugglerTupleList second,
		String currPredType) {

		JugglerTuple secondTuple = second.getFirstTuple();

		ArrayList queryIDToRemove = new ArrayList();

		if (secondTuple.getQuerySatisfied().size() > 0) {
			//check if the tuple is relevant to the query
			for (int i = 0; i < secondTuple.getQuerySatisfied().size(); i++) {
				int queryID =
					((Integer) secondTuple.getQuerySatisfied().get(i))
						.intValue();
				QueryQED qed = outputQDE.getQueryQED(queryID);
				ArrayList tupleStreamNames = firstTuple.getStreamNames();
				tupleStreamNames.addAll(second.getStreamNames());
				ArrayList queryStreamNames = qed.getStreams();

				if (!queryStreamNames.containsAll(tupleStreamNames)) {
					queryIDToRemove.add(new Integer(queryID));
				}
			}

			outputQDE.removeQueryPredicates(
				second,
				queryIDToRemove,
				currPredType);

			for (int i = 0; i < queryIDToRemove.size(); i++) {
				int removeQueryID =
					((Integer) queryIDToRemove.get(i)).intValue();
				secondTuple.getQuerySatisfied().remove(
					new Integer(removeQueryID));
			}

			if (secondTuple.getQuerySatisfied().size() == 0)
				return false;
		}
		return true;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (5/20/2003 1:47:47 PM)
	 * @return boolean
	 * @param firstTuple juggler.stream.JugglerTuple
	 */
	public boolean checkIfTupleIsRelevantToQueriesPredicates(
		JugglerTupleList jTL,
		String currPredType) {
		JugglerTuple firstTuple = jTL.getFirstHalfJoinedTuple().getFirstTuple();

		ArrayList queryIDToRemove = new ArrayList();

		if (firstTuple.getQuerySatisfied().size() > 0) {
			//check if the tuple is relevant to the query
			for (int i = 0; i < firstTuple.getQuerySatisfied().size(); i++) {
				int queryID =
					((Integer) firstTuple.getQuerySatisfied().get(i))
						.intValue();
				QueryQED qed = outputQDE.getQueryQED(queryID);
				ArrayList tupleStreamNames = jTL.getStreamNames();
				ArrayList queryStreamNames = qed.getStreams();

				if (!queryStreamNames.containsAll(tupleStreamNames)) {
					queryIDToRemove.add(new Integer(queryID));
				}
			}

			outputQDE.removeQueryPredicates(jTL, queryIDToRemove, currPredType);

			for (int i = 0; i < queryIDToRemove.size(); i++) {
				int removeQueryID =
					((Integer) queryIDToRemove.get(i)).intValue();
				firstTuple.getQuerySatisfied().remove(
					new Integer(removeQueryID));
			}

			if (firstTuple.getQuerySatisfied().size() == 0)
				return false;
		}
		return true;
	}

	/**
	 * Insert the method's description here.
	 * Creation date: (5/20/2003 1:00:32 PM)
	 * @return boolean
	 * @param jTL juggler.stream.JugglerTupleList
	 * @param currPredTypeName java.lang.String
	 */
	public boolean processTupleIfRelevant(
		JugglerTupleList first,
		JugglerTupleList second,
		String currPredTypeName,
		boolean onlyCombineAttr) {

		JugglerTuple firstTuple = second.getFirstTuple();

		if (firstTuple.getQuerySatisfied().size() > 0) {
			if (!checkIfTupleIsRelevantToQueries(first,
				second,
				currPredTypeName))
				return false;
		}
		//if (!onlyCombineAttr) //if we are only combining attributes, we do not need to set the bits for the next pred type
		setTupleIrrelevantBits(second, currPredTypeName);
		if (second.getFirstTuple().getQuerySatisfied().size() > 0)
			return true;
		else
			return false;
	}
	/**
	 * @return
	 */
	public ArrayList getIntermediateBuffer() {
		return intermediateBuffer;
	}

	/**
	 * @param list
	 */
	public void setIntermediateBuffer(ArrayList list) {
		intermediateBuffer = list;
	}
	/**
	 * 
	 */
	public void updateSelectivites() {
		join.selectivity = join.currSelectivity;
		attribute.selectivity = attribute.currSelectivity;
		window.selectivity = window.currSelectivity;
		double attrSelectivity = attribute.selectivity;
		double joinSelectivity = join.selectivity;
		double windowSelectivity = window.selectivity;
		PredicateManager temp = new PredicateManager();
		PredicateManager temp2 = new PredicateManager();

		double min = 0.0;
		double max = 0.0;
		double mid = 0.0;

		//find minimum
		for (int j = 0;
			j < mostSelectivePredicateTypesInOrder.length - 1;
			j++) {
			for (int i = 0;
				i < mostSelectivePredicateTypesInOrder.length - 1;
				i++) {
				String predicateName = mostSelectivePredicateTypesInOrder[i];
				String predicateName2 =
					mostSelectivePredicateTypesInOrder[i + 1];
				temp = getPredicateManager(predicateName);
				temp2 = getPredicateManager(predicateName2);
				if (temp.selectivity > temp2.selectivity) {
					mostSelectivePredicateTypesInOrder[i] = predicateName2;
					mostSelectivePredicateTypesInOrder[i + 1] = predicateName;
				}
			}
		}

	}

	/**
	 * 
	 */
	public PredicateManager getPredicateManager(String predicateName) {
		PredicateManager temp = new PredicateManager();

		if (predicateName.equals("attribute"))
			temp = attribute;
		if (predicateName.equals("window"))
			temp = window;
		if (predicateName.equals("join"))
			temp = join;
		return temp;

	}

	/**
		 * @param list
		 */
	public void setTimeIncrement(long time) {
		this.window.setTimeIncrement(time);
	}
	/**
	 * @return
	 */
	public boolean isExpStructuresEmpty() {
		if (window.isExpStrucutresEmpty() && join.isExpStrucutresEmpty())
			return true;
		else
			return false;
	}
	/**
	 * @param tuples_B
	 */
	public void enterTuplesOptimized_XATTuples(JugglerTuple[] tuplesXAT) {
		JugglerTuple[] tuples = new JugglerTuple[tuplesXAT.length];
		for (int i = 0; i < tuples.length; i++) {
			JugglerTuple jlist = (JugglerTuple) tuplesXAT[i];
			//create tuple's predicate BitSets
			ExtPredicateBitSet pbs = this.createExtPredicateBitSet();
			jlist.setPredicates(pbs);
			//set all the attribute predicates as relevant
			setAllListsAsRelevant_XATTuple(jlist);
			tuples[i] = jlist;
		}

		tuples = setupAndApplyAttrPredicates_XATTuple(tuples);
		//Iterator tuplesIt = tuples.iterator();
		//while (tuplesIt.hasNext()) {
		for (int i = 0; i < tuples.length; i++) {
			//	JugglerTupleList element = (JugglerTupleList) tuplesIt.next();
			JugglerTupleList element = new JugglerTupleList(tuples[i]);
			//enter into each predicate managers.
			window.enterTuple_XATTuple(element);
			join.enterTuple_XATTuple(element);
		}
		int j = 0;
		Iterator iter =
			WindowPredicateManager.getWindowRemoveBuckets().iterator();
		while (iter.hasNext()) {
			int winExpStrID = ((Integer) iter.next()).intValue();
			WindowExpStructureList winExpStr =
				(WindowExpStructureList) window.getExpStructures().get(
					winExpStrID);
			Iterator iter2 = winExpStr.getRemoveBucketList().iterator();
			while (iter2.hasNext()) {
				int bucketID = ((Integer) iter2.next()).intValue();
				WindowExpStructureList winList =
					winExpStr.getWindowExpStrList_With_BucketID(bucketID);
				JugglerWindowTupleList temp =
					winList.getThisWinExpStr().getListOfTuples();
				while (temp != null) {
					join.removeTupleList(temp.getJList());
					temp = temp.getNextWinTL();
				}
				winExpStr.removeBucket_XATTuple(bucketID);
			}
			winExpStr.getRemoveBucketList().clear();
		}
	}

	/**
	 * @param tuples_B
	 */
	public void enterTuplesOptimized_XATTuples(ArrayList tuples) {
		for (int i = 0; i < tuples.size(); i++) {
			JugglerTupleList jlist = (JugglerTupleList) tuples.get(i);
			//create tuple's predicate BitSets
			ExtPredicateBitSet pbs = this.createExtPredicateBitSet();
			jlist.getTuple().setPredicates(pbs);
			//set all the attribute predicates as relevant
			setAllListsAsRelevant(jlist);
		}

		tuples = setupAndApplyAttrPredicates(tuples);
		//Iterator tuplesIt = tuples.iterator();
		//while (tuplesIt.hasNext()) {
		for (int i = 0; i < tuples.size(); i++) {
			//	JugglerTupleList element = (JugglerTupleList) tuplesIt.next();
			JugglerTupleList element = (JugglerTupleList) tuples.get(i);
			//enter into each predicate managers.
			window.enterTuple(element);
			join.enterTuple(element);
		}
		int j = 0;
		Iterator iter =
			WindowPredicateManager.getWindowRemoveBuckets().iterator();
		while (iter.hasNext()) {
			int winExpStrID = ((Integer) iter.next()).intValue();
			WindowExpStructureList winExpStr =
				(WindowExpStructureList) window.getExpStructures().get(
					winExpStrID);
			Iterator iter2 = winExpStr.getRemoveBucketList().iterator();
			while (iter2.hasNext()) {
				int bucketID = ((Integer) iter2.next()).intValue();
				WindowExpStructureList winList =
					winExpStr.getWindowExpStrList_With_BucketID(bucketID);
				JugglerWindowTupleList temp =
					winList.getThisWinExpStr().getListOfTuples();
				while (temp != null) {
					join.removeTupleList(temp.getJList());
					temp = temp.getNextWinTL();
				}
				winExpStr.removeBucket(bucketID);
			}
			winExpStr.getRemoveBucketList().clear();
		}
	}
	/**
	 * @param tuples_B
	 * @return
	 */
	private JugglerTuple[] setupAndApplyAttrPredicates_XATTuple(JugglerTuple[] tuples) {
		tuples =
			this
				.attribute
				.applyAllRelevantPredicatesForInitialization_XATTuple(
				tuples);

		return tuples;
	}
	/**
	 * @param jlist
	 */
	private void setAllListsAsRelevant_XATTuple(JugglerTuple jlist) {
		GeneralPredicateBitSet gbs =
			(GeneralPredicateBitSet) jlist
				.getPredicates()
				.getIsEvaluated_Relevant(
				"attribute");

		for (int i = 0; i < attribute.getPredicateList().size(); i++) {
			AttrPredOrderList attrList =
				(AttrPredOrderList) attribute
					.getPredicateList()
					.getOrderLists()
					.get(
					i);
			BitSet relAttrList = new BitSet(attrList.size());
			for (int j = 0; j < attrList.size(); j++) {
				relAttrList.set(j);
			}
			//must remove the initialized values to add the new values in
			//gbs.getPredicateBitSets().remove(i);
			gbs.getPredicateBitSets()[i] = relAttrList;
		}
		GeneralPredicateBitSet tupleGBS =
			(GeneralPredicateBitSet) jlist
				.getPredicates()
				.getIsEvaluated_Relevant(
				"attribute");
		tupleGBS = gbs;

	}
	/**
	 * @return
	 */
	public boolean isExpStructuresEmpty_XATTuple() {
		if ((window.isExpStrucutresEmpty_XATTuple())
			&& (join.isExpStrucutresEmpty_XATTuple()))
			return true;
		else
			return false;
	}
	/**
	 * @param iterator
	 * @return
	 */
	public ArrayList processTuples_XATTuple(JugglerTuple[] tuples) {
		//create tuple's predicate BitSets
		ExtPredicateBitSet pbs = this.createExtPredicateBitSet();

		//make sure the joinedTuples are in the JugglerTupleList format, since this is the way the joined tuples are stored in the intermediate buffer
		tuples = joinedTupleFormatCheck_XATTuples(tuples);

		//this identifies if the attribute predicates need to be combined, this is needed if attr predicate is first or later
		boolean needToCombineAttributes = false;
		boolean combinedAttr = false;
		boolean anyResultingTuples = false;

		for (int k = 0; k < tuples.length; k++) {
			JugglerTuple tupleList = (JugglerTuple) tuples[k];
			if(tupleList == null){
				System.out.println("Tuples are null!");
				return null;
			}
			if(tupleList.getQuerySatisfied() != null)
			tupleList.getQuerySatisfied().clear();
			JugglerTupleList firstParttuple = new JugglerTupleList(tupleList);
			//(JugglerTuple) tupleList.getFirstHalfJoinedTuple();
			firstParttuple.getFirstTuple().setPredicates(
				(ExtPredicateBitSet) pbs.getClone());
			for (int i = 0;
				i < mostSelectivePredicateTypesInOrder.length;
				i++) {

				//apply most selective predicate types in order	
				String currentPredType = mostSelectivePredicateTypesInOrder[i];
				PredicateManager selectivePredManager = new PredicateManager();
				int numInputTuplesToPredicateType = 0;
				try {
					selectivePredManager = this.getPredManager(currentPredType);
				} catch (Exception e) {
					System.err.println();
				}

				//apply predicates should take advantage of the irrelevant set bits
				Iterator joinedTuples = null;
				if (!hash.isEmpty()) {
					numInputTuplesToPredicateType = intermediateBuffer.size();

					joinedTuples =
						selectivePredManager.applyPredicates_XATTuple(
							firstParttuple,
							intermediateBuffer.iterator());
					intermediateBuffer.clear();
					hash.clear();

				} else {
					//if window or join predicate type is first, then set all the predicate lists as relevant.
					if (i == 0)
						setAllListsAsRelevant(
							selectivePredManager,
							firstParttuple);

					joinedTuples =
						selectivePredManager.applyPredicates_XATTuple(
							firstParttuple,
							null);

					numInputTuplesToPredicateType = 1;

					if (currentPredType == "attribute") {
						needToCombineAttributes = true;
					}
				}

				int numOFJoinedTuples_predicateOutput = 0;

				//if the selective PredManager was either join or window, therefore the tuples are joined and returned
				//as iterators
				if (joinedTuples != null
					&& !((i == 0) && (currentPredType.equals("attribute")))) {

					//identifies if teh attributes have been combined
					while (joinedTuples.hasNext()) {
						JugglerTupleList secondTuple =
							(JugglerTupleList) joinedTuples.next();

						boolean isRelevantTuple = true;

						//if attribute predicates need to be combined 
						if (needToCombineAttributes) {

							isRelevantTuple =
								processTupleIfRelevant(
									firstParttuple,
									secondTuple,
									"attribute",
									needToCombineAttributes);
							if (!isRelevantTuple)
								continue;
							combinedAttr = true;
						}

						//want to only set the relevan queries if the attributes have not been combined
						//only want to do this for the first predicate type
						if (i == 0) {

							firstParttuple.getFirstTuple().setPredicates(
								(ExtPredicateBitSet) pbs.getClone());
							isRelevantTuple =
								processTupleIfRelevant(
									firstParttuple,
									secondTuple,
									currentPredType,
									combinedAttr);
							if (!isRelevantTuple)
								continue;
						} else { //if its the last or next to last predicate type.. find what queries have been satisfied.
							//changed from firstTuple!!!!!!!!!

							ArrayList updatedQueryIDs =
								findSatisfiedQueries(
									firstParttuple,
									secondTuple,
									currentPredType);
							if (secondTuple != null)
								secondTuple.getFirstTuple().setQuerySatisfied(
									updatedQueryIDs);
							if ((i < 2) && (updatedQueryIDs.size() > 0))
								setTupleIrrelevantBits(
									secondTuple,
									currentPredType);

						}
						//add the tuples to our list
						//intermediateTuples.add(tempJoinedTuple);
						//we still have to set the irrelevant query bit sets for the last two predicate types
						//then we have to use the irrelevant bitsets
						//if the joined tuple has satisfied any queries.....
						if (secondTuple
							.getFirstTuple()
							.getQuerySatisfied()
							.size()
							> 0) {
							numOFJoinedTuples_predicateOutput++;
							intermediateBuffer.add(secondTuple);
							//add it into our hash??
							addIntermeidateTuplesToHash(secondTuple);
						}
					}
					//this join or window should result in some resulting tuples
					anyResultingTuples = true;
				}
				//if the selective PredMananger was only attribute, then it does not return an iterator
				else {
					//we still have to set the irrelevant query bit sets for the last two predicate types
					//then we have to use the irrelevant bitsets
					setTupleIrrelevantBits(firstParttuple, currentPredType);
					//add the tuples to our list
					//intermediateTuples.add(firstParttuple);

					numOFJoinedTuples_predicateOutput = 1;

				}
				//if combined the attribute predicates, no need to combine them again
				if (combinedAttr)
					needToCombineAttributes = false;
				//if there are no resulting tuples, then do not apply any more predicates
				if (anyResultingTuples && hash.isEmpty())
					return null;
				//what if no predicates passed, and it needs to be joined by all tuples..
				//this doesn't work for no streams relevant in a predicate

				//update selectivity for the predicate type
				selectivePredManager.calculateSelectivity(
					numInputTuplesToPredicateType,
					numOFJoinedTuples_predicateOutput);

			} //for each predicate type
			//dequeue old joined tuples?????...........
			//intermediateBuffer.remove(firstParttuple);
			try {
				firstParttuple.getTuple().setPredicates(null);
				firstParttuple.updatePredicateBitSetsInTupleList();
				return (joinOutputTuples_ToQueue(firstParttuple));
			} catch (Exception e) {
				e.printStackTrace();
			}

		} //for each tuple

		//return intermediateTuples;
		//dont' need to return do we?

		return null;

	}
	/**
	 * @param tuples
	 * @return
	 */
	private JugglerTuple[] joinedTupleFormatCheck_XATTuples(JugglerTuple[] unjoinedTuple) {
		JugglerTuple[] tuplesToTuple =
			new JugglerTuple[unjoinedTuple.length];
		for (int i = 0; i < unjoinedTuple.length; i++) {
			JugglerTuple aTupleList = (JugglerTuple) unjoinedTuple[i];
			//JugglerTupleList aTupleList = new JugglerTupleList(aTuple);
			tuplesToTuple[i] = aTupleList;
		}

		return tuplesToTuple;
	}
}